      program makecomm
c ======================================================================
c
c     programmer  Guus Segal
c     version     1.0      date  11-11-1997
c
c   copyright (c) 1997-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Read the name of the file to be updated from the file
c     makecomment.name
c     Open de files name.f and name.dif
c     Create file name.ext from both files
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      character * 10 name
      character * 4 namecom
      character * 14 namef, nameext, namedif
      character * 100 line
      character * 1 action
      integer icount, ios, iline1_old, iline2_old, iline1_new,
     +        iline2_new, i, j, n, nchar
      logical ready
c **********************************************************************
c
c                       I/O
c
c    The program reads three files:
c
c     makecomment.name
c     name.f
c     name.dif
c
c    and creates a file
c
c    name.ext
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Open file makecomment.name
c    Read name from this file
c    Open files name.f and name.dif for reading
c    Open file name.ext for writing
c    Skip first 2 lines in name.dif
c    icount := 0
c    While there are still lines in name.dif do
c       Read next line (must contain numbers)
c       Interpret this line:
c          iline1_old := first line number old version
c          iline2_old := second line number old version
c          iline1_new := first line number new version
c          iline2_new := second line number new version
c          action := type of action (a, c, d)
c       Copy lines from name.f to name.ext from line icount+1 to line
c       iline1_new-1
c       Raise icount
c       if ( action.eq.'d' ) then
c          Put line with cold in name.ext
c          n := iline2_old - iline1_old+1
c          Copy all n lines from file name.dif starting with <
c          Replace first two columns by c
c          Put line with cold in name.ext
c       else if ( action.eq.'a' ) then
c          Put line with cnew in name.ext
c          n := iline2_new - iline1_new+1
c          Copy n lines from file name.f
c          Raise icount
c          Put line with cnew in name.ext
c       else if ( action.eq.'c' ) then
c          Put line with cold in name.ext
c          n := iline2_old - iline1_old+1
c          Copy all n lines from file name.dif starting with <
c          Replace first two columns by c
c          Put line with cold in name.ext
c          Put line with cnew in name.ext
c          n := iline2_new - iline1_new+1
c          Copy n lines from file name.f
c          Raise icount
c          Put line with cnew in name.ext
c       else
c          error
c    Copy rest of the file name.f to name.ext
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Open file makecomment.name

      open ( unit=10, file='makecomment.name' )

c     --- Read name from this file

      read ( 10, 100 ) namecom
      read ( 10, 100 ) name
100   format ( a )
      close ( unit=10 )

c     --- Open files name.f and name.dif for reading

      nchar = len_trim(name)
      namef = name(1:nchar)//'.f'
      namedif = name(1:nchar)//'.dif'
      open ( unit=11, file=namef )
      open ( unit=12, file=namedif )

c     --- Open file name.ext for writing

      nameext = name(1:nchar)//'.ext'
      open ( unit=13, file=nameext )

c     --- Skip first 2 lines in name.dif

      read ( unit=12, fmt=100 ) line
      read ( unit=12, fmt=100 ) line
      icount = 0
      ready  = .false.

c     --- While there are still lines in name.dif do

150   if ( .not. ready ) then

c     --- Read next line (must contain numbers)

         read ( unit=12, fmt=100, iostat=ios ) line
         nchar = len_trim(line)

         if ( ios.lt.0 ) then

c        --- end of file

            ready = .true.
            go to 150

         else if ( ios.lt.0 ) then

c        --- end of file

            print *, ' Error in reading dif file'
            ready = .true.
            go to 150

         end if

c        --- Interpret this line:
c            iline1_old := first line number old version
c            iline2_old := second line number old version
c            iline1_new := first line number new version
c            iline2_new := second line number new version
c            action := type of action (a, c, d)

         call interpret_line ( line, nchar, iline1_old, iline2_old,
     +   		       iline1_new, iline2_new, action )

c        --- Copy lines from name.f to name.ext from line icount+1 to line
c            iline1_new-1
c            Raise icount

         do i = icount+1, iline1_new-1
            read(11,100) line
            nchar = len_trim(line)
            write(13,100) line(1:nchar)
         end do
         icount = iline1_new-1
         if ( action.eq.'d' ) then

c        --- action = 'd'
c            Put line with cold in name.ext

            write(13,100) 'cold<'
            n = iline2_old - iline1_old+1

c           --- Copy all n lines from file name.dif starting with <
c        	Replace first two columns by c

            do j = 1, n
               read(12,100) line
               line(2:2) = 'c'
               nchar = len_trim(line)
               write(13,100) line(2:nchar)
            end do

c           --- Put line with cold in name.ext

            write(13,100) 'cold>'

         else if ( action.eq.'a' ) then

c        --- action = 'a'

c           Put line with cnew in name.ext

            write(13,100) namecom//'<'

            n = iline2_new - iline1_new+1

c           --- Copy n lines from file name.f
c               Raise icount

            do j = 1, n
               read(11,100) line
               nchar = len_trim(line)
               write(13,100) line(1:nchar)
            end do
            icount = icount+n

c           --- Put line with cnew in name.ext

            write(13,100) namecom//'>'

         else if ( action.eq.'c' ) then

c        --- action = 'c'
c            Put line with cold in name.ext

            write(13,100) 'cold<'
            n = iline2_old - iline1_old+1

c           --- Copy all n lines from file name.dif starting with <
c        	Replace first two columns by c

            do j = 1, n
               read(12,100) line
               line(2:2) = 'c'
               nchar = len_trim(line)
               write(13,100) line(2:nchar)
            end do


c           --- Put line with cold in name.ext

            write(13,100) 'cold>'

c           --- Put line with cnew in name.ext

            write(13,100) namecom//'<'
            n = iline2_new - iline1_new+1

c           --- Copy n lines from file name.f
c               Raise icount
c               Skip n lines from file name.dif

            do j = 1, n
               read(12,100) line
               read(11,100) line
               nchar = len_trim(line)
               write(13,100) line(1:nchar)
            end do
            icount = icount+n

c           --- Put line with cnew in name.ext

            write(13,100) namecom//'>'

         end if
         go to 150

      end if

c     --- Copy rest of the file name.f to name.ext

200   read(11,100, end = 1000) line
      nchar = len_trim(line)
      write(13,100) line(1:nchar)
      go to 200

1000  end

      subroutine interpret_line ( line, nchar, iline1_old, iline2_old,
     +   		          iline1_new, iline2_new, action )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   12-11-1997
c
c
c   copyright (c) 1997-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Interpret a line in a diff file of the following shape:
c   lineno [,lineno] action lineno [,lineno]
c   with action a, c or d
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer nchar, iline1_old, iline2_old, iline1_new, iline2_new
      character *(*) line, action

c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer icolum, ifirst
      character *10 format1
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      icolum = 1

c     --- Find first line number

100   if ( line(icolum:icolum).ge.'0' .and. line(icolum:icolum).le.'9' )
     +   then
         icolum = icolum+1
         go to 100
      end if
      write(format1,110) icolum-1
110   format('(i',i1,')')
      read ( line(1:icolum-1), fmt=format1 ) iline1_old
      if ( line(icolum:icolum).eq.',' ) then

c     --- , found, read next line number

         icolum = icolum + 1
         ifirst = icolum
200      if ( line(icolum:icolum).ge.'0' .and.
     +        line(icolum:icolum).le.'9' ) then
            icolum = icolum+1
            go to 200
         end if
         write(format1,110) icolum-ifirst
         read ( line(ifirst:icolum-1), fmt=format1 ) iline2_old

      else

c     --- no , found

         iline2_old = iline1_old

      end if
      action = line(icolum:icolum)
      if ( action.ne.'a' .and. action.ne.'d' .and. action.ne.'c' ) then
         print *, 'Wrong action found in dif file, action = ', action
         stop
      end if
      icolum = icolum + 1
      ifirst = icolum

c     --- Find first line number

300   if ( icolum.le.nchar .and. line(icolum:icolum).ge.'0' .and.
     +     line(icolum:icolum).le.'9' ) then
         icolum = icolum+1
         go to 300
      end if
      write(format1,110) icolum-ifirst
      read ( line(ifirst:icolum-1), fmt=format1 ) iline1_new
      if ( icolum.lt.nchar .and. line(icolum:icolum).eq.',' ) then

c     --- , found, read next line number

         icolum = icolum + 1
         ifirst = icolum
400      if ( icolum.le.nchar .and. line(icolum:icolum).ge.'0' .and.
     +        line(icolum:icolum).le.'9' ) then
            icolum = icolum+1
            go to 400
         end if
         write(format1,110) icolum-ifirst
         read ( line(ifirst:icolum-1), fmt=format1 ) iline2_new

      else

c     --- no , found

         iline2_new = iline1_new

      end if
      print *, iline1_old, iline2_old,
     +     iline1_new, iline2_new, action

      end
