      program buildres

c***********************************************************************
c
c  purpose: read and check       reserved.upd
c           update direct access reserved.unf
c           update               reserved.dat
c
c  usage:   buildres
c
c  files ( all are local ):
c
c           reserved.upd   i   unsorted free-format list of new explanations
c
c           reserved.unf       direct access sorted list of all explanations
c                          i   on input it need not exist
c                          o   it is (re)written only when no errors found
c
c           reserved.dat       free format sorted list of all explanations
c                          i   if it exists, it is ignored
c                          o   it is (re)written only when no errors found
c                              ( it can be edited and used as next *.upd )
c
c           reserved.cpr   o   compressed listing and  error messages
c
c  restrictions:
c
c           constants mxkeys, mxexpl define the upper bounds:
c
c             number_of_keywords(reserved.upd) +
c           + number_of_keywords(reserved.unf)            <= mxkeys
c
c             number_of_explanation_lines(reserved.upd) +
c           + number_of_explanation_lines(reserved.unf)   <= mxexpl
c
c  output messages:
c
c           error messages cannot be suppressed
c
c           logical constants ptr* control the amount of output
c
c           prthed    print headlines and final lines ( 2 + 3 lines )
c           prtupd    print list of keywords in     reserved.upd + statistics
c           prtold    print list of keywords in old reserved.unf + statistics
c           prtnew    print list of keywords in new reserved.unf + statistics
c           prtfil    print list of output files ( 6 lines )
c           prtnum    print also statistics like number of keywords
c
c***********************************************************************
c
c  programmer   Zdenek Vlasek
c  version      2.0  31-08-92  complete revision, new concept:
c                              update sorted reserved.unf, all in memory
c  version      1.2  26-08-92  filenames, filenumbers, safe opening
c  version      1.1  24-08-92  changes in layout, to be close to sepcheck
c  version      1.0  06-08-92  in Fortran
c  version      0.0  09-07-92  (in Turbo-Pascal)
c
c***********************************************************************
c
c  Syntax of input textfile reserved.upd is free:
c
c            column 1 is 'c' or 'C' or '*'
c            column 2 is ' ' or '|' or none
c            linesize <=80, rest is truncated
c            keyword should start anywhere in columns 3..mxkeypos=13
c            keyword consists of letters, digits, underscores and '@'
c            keyword length <= lenkey=8
c            i/o information (if present) is ignored: 'i' 'o' 'io' 'i/o'
c            one or more explanation lines for each keyword
c            first line of explanation should start at mxkeypos+1=14 or later
c                       (because we must recognize a next keyword)
c            next lines of the same explanation start where the first line does
c            continuation does not start sooner then the explanation
c            empty continuation is processed as any other continuation
c            length of explanation <= lenexpl=61  (e.g. columns 20..80)
c
c            an example ( lines starting with '*' ):
c
*     aaaaaa78 i/o This explanation starts at column 20 ( e.g. i/o  parameters )
*                      and it continues in columns 20..80
*     BBBBBB This explanation starts at column 14 and may not exceed col74
*                  and it continues in columns 14..74
c
c***********************************************************************
c
c  Syntax of output textfile reserved.dat is fix:
c            column 1 is 'c'
c            column 2 is ' ' or none
c            keyword     at column poskey=7, in lowercase letters
c            no i/o information
c            explanation in columns posexpl=20..80
c            1<=linesize<=80, no trailing blanks
c
c            an example (it corresponds to the above example):
c
c     aaaaaa78     This explanation starts at column 20 ( e.g. i/o  parameters )
c                       and it continues in columns 20..80
c     bbbbbb7      This explanation starts at column 14 and may not exceed col74
c                       and it continues in columns 14..74
c
c***********************************************************************
c
c  Syntax of direct access file reserved.unf :
c
c      recordsize = 1+5+lenkey+5+lenexpl=80
c
c      first record contains a warn string and the number of keywords stored
c      next records: sorted list of keywords and their positions in file
c      next records: keywords with their explanation
c
c***********************************************************************

cimp      implicit none

c     ---------------------- constants -----------

      logical prtfil, prthed, prtnew, prtnum, prtold, prtupd
      integer lenexpl, lenkey, mxexpl, mxkeypos, mxkeys,
     .        posexpl, poskey
      character * 50 warn
      parameter (
     .  lenexpl = 61,
     .  lenkey = 8,
     .  mxexpl = 20000,
     .  mxkeypos = 13,
     .  mxkeys = 2000,
     .  posexpl = 20,
     .  poskey = 7,
     .  prtfil = .false.,
     .  prthed = .true.,
     .  prtnew = .true.,
     .  prtnum = .false.,
     .  prtold = .false.,
     .  prtupd = .true.,
     .  warn = '!!!!!! do not edit this direct access file !!!!!! ')

c     ---------------------- local variables -----

      logical firstex, nextold, nextupd, oldexh, updexh
      integer iexpl, ikeys, ioldkeys, ios, ireor, iupdkeys,
     .        k1, k2, k3, k4, k5, k,
     .        nexpl, nln, nnewkeys, noldkeys, nupdexpl, nupdkeys
      character      w
      character *  3 io
      character *  8 oldkey, prevold, updkey
      character * 80 fixline
      character * 255 longline

c     ---------------------- commons to keep huge arrays on HP -------

      character *  8 keys(mxkeys)
      character * 80 keyexpl(mxexpl)
      common /ckeys/ keys, keyexpl

      integer bpos(mxkeys), epos(mxkeys), nkeys, kkeys, reord(mxkeys)
      common /cpos/ bpos, epos, nkeys, kkeys, reord

c     ---------------------- commons for subroutine errr -------------

      integer        fout, fupd, funf, fdat, iline, nerr
      character * 80 line
      common /crefs/ fout, fupd, funf, fdat, iline, nerr
     .       /cline/ line

c     ---------------------- local functions -----

      character c
      logical keychar

      keychar ( c ) = c.ge.'a' .and. c.le.'z' .or.
     .                c.ge.'A' .and. c.le.'Z' .or.
     .                c.ge.'0' .and. c.le.'9' .or.
     .                c.eq.'_'  .or.
     .                c.eq.'@'
c     *  Character '@' is used in FTN77-specific routines on PC

c***********************************************************************
c
c   bpos      where keys(*) begin in unsorted part of keyexpl(*)
c   epos      where keys(*) end   in unsorted part of keyexpl(*)
c   fout      reserved.cpr, output messages
c   fdat      reserved.dat, output file, fix-syntax
c   firstex   true = first line of an explanation
c   fixline   one fix-syntax line
c   funf      reserved.unf
c   fupd      reserved.upd
c   nextold   next oldkey is needed
c   nextupd   next updkey is needed
c   iexpl     position in keyexpl(*)
c   ikeys     position in keys(*)
c   iline     linenumber in a current file
c   io        string like 'i/o'
c   ioldkeys  position of oldkey in keys(*)
c   ios       iostat in open/read
c   ireor     position in reord(*)
c   iupdkeys  position of updkey in keys(*)
c   k1        column where the first word begins
c   k2        column where the first word ends
c   k3        column where the second word begins
c   k4        column where the second word ends
c   k5        column where the line ends
c   k         loop through a string
c   keyexpl   unsorted part and sorted part of fix-syntax lines
c   keys      sorted keywords from reserved.upd
c   kkeys     where is the current keyword in sorted keys(*)
c   lenexpl   maximal length of explanation
c   lenkey    maximal length of keyword
c   line      current line
c   longline  current input
c   mxexpl    max number of explanation lines in reserved.upd + reserved.unf
c   mxkeypos  max column where a keyword starts in reserved.upd
c   mxkeys    max number of keywords in reserved.upd + reserved.unf
c   nerr      counter of errors
c   nexpl     number of all explanation lines in keyexpl(*)
c   nkeys     number of all keywords in keys(*)
c   nln       number of explanation lines for current keyword
c   nnewkeys  number of keywords in new reserved.unf
c   noldkeys  number of keywords in old reserved.unf
c   nupdexpl  number of explanations in unsorted part of keyexpl(*)
c   nupdkeys  number of all keywords in reserved.upd, incl.duplicates
c   oldexh    oldkey list exhausted
c   oldkey    keyword from old reserved.unf
c   posexpl   column where the explanation starts in fix syntax
c   poskey    column where the keyword starts in fix syntax
c   prevold   previous oldkey
c   prtfil    true = print list of output files ( 8 lines )
c   prthed    true = print headlines and final lines ( 2 + 3 lines )
c   prtnew    true = print list of keywords in new reserved.unf
c   prtnum    true = print also statistics like number of keywords
c   prtold    true = print list of keywords in old reserved.unf
c   prtupd    true = print list of keywords in     reserved.upd
c   reord     mapping from new ordinal numbers 1..nnewkeys into keys(*)
c   upddexh   updkey list exhausted
c   updkey    key from reserved.upd
c   w         character to be inspected
c   warn      constant string, headline in reserved.unf
c
c***********************************************************************

c     *  Initialize

      fout = 21
      fupd = 22
      fdat = 23
      funf = 24

      nerr = 0

c     *****************************************************************
c     **      Open reserved.cpr ( listing and error messages )       **
c     *****************************************************************

      open ( fout, file = 'reserved.cpr' )

      if ( prthed ) write ( fout, 9999)
     . 'program "buildres" updates files reserved.unf and reserved.dat',
     . '=============================================================='

c     *****************************************************************
c     **   Open reserved.upd, read it, build sorted list of keywords **
c     *****************************************************************

      open ( fupd, file = 'reserved.upd', status='OLD', iostat=ios )
      if ( ios .ne. 0 ) then
        iline = -1
        call errr ( 1, 'unable to open input file reserved.upd')
      endif

      if ( prtupd ) write ( fout, 9999) ' ',
     .  'keywords in input file reserved.upd, unsorted',
     .  '---------------------------------------------'

      nupdkeys = 0
      nkeys = 0
      kkeys = 0
      nexpl = 0

c     *  This initialization enables to detect line 1 without keyword
      k3 = 0

c     *  Reading reserved.upd

      do 500 iline = 1, mxexpl

        longline = ' '
        read ( fupd, 9999, end=510, iostat=ios ) longline
        line = longline
        if ( ios .ne. 0 ) then
          call errr ( 1, 'unreadable line in input file reserved.upd')
        endif

c       *  Let k5 = length(longline)

        do k5 = 255, 1, -1
          if ( longline(k5:k5) .ne. ' ' ) goto 110
        enddo
        k5 = 0
  110   continue

c       *  Check length

        if ( k5 .gt. 80 ) then
          write ( fout, 9999) longline
          call errr ( 0, 'too long input line, truncated')
c         *  Truncate it
          k5 = 80
        endif

c       *  Check column 1

        w = line(1:1)
c       *  If k5=0 then w=' '
        if ( w .ne. 'c' .and. w .ne. 'C' .and. w .ne. '*' ) then
          call errr ( 0, 'column 1 must be c or C or *, replaced')
          line(1:1) = 'c'
        endif

c       *  Check column 2

        w = line(2:2)
        if ( w .ne. ' ' .and. w .ne. '|' ) then
          call errr ( 0, 'column 2 must be blank or none, replaced')
          line(2:2) = ' '
        endif

c       *  Analyse the line, keyword (if any)     :   line(k1:k2)
c       *                    second word (if any) :   line(k3:k4)

c       *  Fill k1 by position of first nonblank character, if any,
c       *          otherwise k1=81

        do k1 = 3, 80
          if ( line(k1:k1) .ne. ' ' ) goto 130
        enddo
        k1 = 81
  130   continue

c       *  Split between keywords      k1 = 3 .. maxkeypos
c       *  and continuing explanation  k1 =      maxkeypos+1 .. 81

        if ( k1 .le. mxkeypos ) then

c         *  A keyword begins at k1

          if ( .not. keychar(line(k1:k1)) ) then
            call errr ( 0, 'bad character in keyword field, replaced')
            line(k1:k1) = '?'
          endif

          firstex = .true.
          nupdkeys = nupdkeys + 1

c         *  Fill k2 by end position of keyword

          do k2 = k1, 79
            if ( .not. keychar (line(k2+1:k2+1)) ) goto 150
          enddo
          k2 = 80
  150     continue

c         *  Check keyword delimiter ( before the truncation )

          if ( k2 .lt. 80 .and. line(k2+1:k2+1) .ne. ' ' ) then
            call errr ( 0, 'bad character after keyword, replaced')
            line(k2+1:k2+1) = ' '
          endif

c         *  Check the length

          if ( k2-k1+1 .gt. lenkey ) then
            call errr ( 0, 'too long keyword, truncated')
            line(k1+lenkey:k2) = ' '
            k2 = k1+lenkey-1
          endif

c         *  Remember keyword and convert it to lowercase, ACSII

          updkey = line(k1:k2)
          do k = 1, lenkey
            if ( updkey(k:k) .ge. 'A' .and. updkey(k:k) .le. 'Z' )
     .           updkey(k:k) = char ( ichar(updkey(k:k)) + 32 )
          enddo

c         *  Fill k3 by position of the next nonblank character
c         *  at k2+1 is ' '

          do k3 = k2+2, 80
            if ( line(k3:k3) .ne. ' ' ) goto 170
          enddo
          k2 = 81
  170     continue

c         *  Skip i/o string, if any

c         *  Fill k4 by end position of 2nd word

          do k4 = k3, 79
            if ( line(k4+1:k4+1) .eq. ' ' ) goto 210
          enddo
          k4 = 80
  210     continue

c         *  Is 2nd word an i/o string ?

          if ( k4.le.78 .and. k4-k3 .ge. 0 .and. k4-k3 .le. 2 ) then
c           *  It may be an i/o string

            io = line(k3:k4)
            if ( io .eq. 'i  ' .or.
     .           io .eq. 'o  ' .or.
     .           io .eq. 'io ' .or.
     .           io .eq. 'i/o' ) then

c             *  It is an i/o string, ignore it
c             *  Let k3 is beginning of 3rd word, if any

              do k3 = k4+2, 80
                if ( line(k3:k3) .ne. ' ' ) goto 230
              enddo
              k3 = 81
  230         continue

            endif

          endif

        else

c         *  The explanation continues:        k1 > mxkeypos
c         *  Respect adjusting, store line(k3:k5)
c         *  When k3=0 then also nkey=0 and no keyword proceeds

          firstex = .false.

          if ( k3 .eq. 0 ) then
            call errr ( 0, 'no keyword on line 1.. , line ignored')
            goto  500
          endif

c         *  Keep  k3  from the line with keyword, check adjusting

          if ( k1 .lt. k3 ) then
            call errr ( 0, 'continuation starts too soon, truncated')
          endif

        endif

c       *  k1>=3, k5>=1, line(k3:k5) is explanation, if nonempty

        if ( k5-k3+1 .gt. lenexpl ) then
          call errr ( 0, 'too long explanation , truncated')
          k5 = k3 + lenexpl - 1
        endif

c       *  Compose fixline

        fixline = 'c'
        if ( firstex    ) fixline(poskey:)  = updkey
        if ( k3 .le. k5 ) fixline(posexpl:) = line(k3:k5)

        if ( firstex ) then
          if ( prtupd ) write ( fout, 9998) fixline
c         *  Update sorted keys(*)
          call buildsor ( updkey, nexpl+1,
     .                    mxkeys, keys, bpos, epos, nkeys, kkeys )
        endif

c       *  Put fixline in keyexpl

        if ( nexpl .ge. mxexpl ) then
          call errr ( 1, 'mxexpl overflow when reading reserved.upd')
        endif

        nexpl = nexpl + 1
        keyexpl(nexpl) = fixline

  500 continue

  510 continue
      nupdexpl = nexpl

      if ( prtupd .and. prtnum ) write ( fout, 9997)
     .  'number of keywords in reserved.upd =', nupdkeys,
     .  'number of explanation lines        =', iline-1,
     .  'number of errors                   =', nerr

      close ( fupd )

c     *  Close sorted list by a sentinel ( not stored )

      updkey = '~~~~~~~~'
      call buildsor ( updkey, nexpl+1,
     .                mxkeys, keys, bpos, epos, nkeys, kkeys )

c     ----------------------------------------------------------------
c     ----------------------------------------------------------------
c         Now, we have the sorted array keys(1:nkeys) of keywords
c                      without duplicates
c         and their pointers into array keyexpl(1:nupdexpl)
c     ----------------------------------------------------------------
c     ----------------------------------------------------------------

      if ( nerr .gt. 0 ) then
        iline = -1
        call errr ( 1, 'reserved.upd is not error-free, no changes')
      endif

c     ***********************************************************
c     **   Open the old file reserved.unf (if any) and read it **
c     ***********************************************************

      open ( funf, file = 'reserved.unf', status='OLD',
     .       access='DIRECT', recl=80, iostat=ios )

      if ( ios .eq. 0 ) then

c       *  Old reserved.dat exists

c       *  Read and check the first line

        iline = 1
        read ( funf, rec=iline, iostat=ios ) line
        if ( ios .ne. 0 ) then
          call errr ( 1, 'unreadable line 1 of old reserved.unf')
        endif
        if ( line(1:50) .ne. warn ) then
          call errr ( 1, 'line 1 of old reserved.unf is bad')
        endif
        read ( line, 9995, iostat=ios ) fixline, noldkeys
        if ( ios .ne. 0 ) then
          call errr ( 1, 'line 1 of reserve.unf, unreadable noldkeys')
        endif

*       Fill lower part of keys(*), bpos(*), epos(*)

        nkeys = nupdkeys + noldkeys
        if ( nkeys .gt. mxkeys ) then
          call errr ( 1, 'mxkeys overflow, reserved.unf, part 1' )
        endif

cxxx    if ( prtupd ) write ( fout, 9999) ' ',
cxxx .   'keywords and positions in old reserved.unf, sorted',
cxxx .   '--------------------------------------------------'

        prevold = ' '
        do ikeys = nupdkeys+1, nkeys
          iline = iline + 1
          read ( funf, rec=iline, iostat=ios ) line
          if ( ios .ne. 0 ) then
            call errr ( 1, 'unreadable line of old reserved.unf')
          endif

          read ( line, 9996, iostat=ios )
     .       ireor, keys(ikeys), bpos(ikeys), epos(ikeys)
          if ( ireor .ne. ikeys-nupdkeys ) then
            call errr ( 1, 'wrong keynumber in reserved.unf')
          endif
cxxx      if ( prtupd ) write ( fout, '(a30)' ) line
          if ( ios .ne. 0 ) then
            call errr ( 1, 'keyword or positions not readable in *.unf')
          endif

c         *  Shift bpos(*), epos(*) so they point into keyexpl(*)

          bpos(ikeys) = bpos(ikeys) - (1+noldkeys) + nupdexpl
          epos(ikeys) = epos(ikeys) - (1+noldkeys) + nupdexpl

          if ( prevold .ge. keys(ikeys) ) then
            call errr ( 1, 'wrong keyword order in reserved.unf')
          else
            prevold = keys(ikeys)
          endif

        enddo

        nexpl = epos(nkeys)
        if ( nexpl .gt. mxexpl ) then
          call errr ( 1, 'mxexpl overflow, reserved.unf, part 2')
        endif

*       Fill lower part of keyexpl(*), no check but iostat

        do iexpl = nupdexpl+1, nexpl
          iline = iline + 1
          read ( funf, rec=iline, iostat=ios ) keyexpl(iexpl)
          if ( ios .ne. 0 ) then
            call errr ( 1, 'unreadable line, reserved.unf, part 2' )
          endif
        enddo

c       *  Close funf

        if ( nerr .gt. 0 ) then
c         *  Preserve old reserved.unf
          close ( funf )
          iline = -1
          call errr ( 1, 'old reserved.unf is not error-free')
        else
c         *  Delete old reserved.unf
          close ( funf, status='DELETE' )
        endif

        if ( prtold ) then
          write ( fout, 9999) ' ',
     .      'keywords in old reserved.unf, sorted',
     .      '------------------------------------'
          write ( fout, 9999)
     .      ( keyexpl(bpos(ikeys)), ikeys=nupdkeys+1, nkeys )
        endif

      else

        write ( fout, 9999) ' ',
     .    'unable to open old file reserved.unf, but it need not exist',
     .    '-----------------------------------------------------------'

      endif

      if ( prtold .and. prtnum ) write ( fout, 9997)
     .    'number of keywords in old reserved.unf =', noldkeys,
     .    'number of explanation lines            =', nexpl-nupdexpl,
     .    'number of errors                       =', nerr

c     ***************************************************
c     **   Mapping from new ordinal numbers to keys(*) **
c     **     indirect merging the two sorted lists     **
c     ***************************************************

      iupdkeys = 0
      ioldkeys = nupdkeys
      nextupd = .true.
      nextold = .true.

      updexh = .false.
      oldexh = .false.

      do nnewkeys = 1, nkeys
c       *  leave by goto when both lists are exhausted

        if ( nextupd ) then
          if ( iupdkeys .lt. nupdkeys ) then
            iupdkeys = iupdkeys + 1
            updkey = keys(iupdkeys)
          else
            updkey = '~~~~~~~~'
            updexh = .true.
          endif
          nextupd = .false.
        endif

        if ( nextold ) then
          if ( ioldkeys .lt. nkeys ) then
            ioldkeys = ioldkeys + 1
            oldkey = keys(ioldkeys)
          else
            oldkey = '~~~~~~~~'
            oldexh = .true.
          endif
          nextold = .false.
        endif

        if ( updexh .and. oldexh ) goto 530

        if ( updkey .le. oldkey ) then
c         *  It is  updexh = false
          reord(nnewkeys) = iupdkeys
          nextupd = .true.
          if ( updkey .eq. oldkey ) then
c           *  It is  updexh = oldexh = false
            nextold = .true.
          endif
        else
c         *  It is  updkey > oldkey,  oldexh = false
          reord(nnewkeys) = ioldkeys
          nextold = .true.
        endif

      enddo
  530 continue

      nnewkeys = nnewkeys - 1

c     ***************************************************
c     **   Open file reserved.unf to write there       **
c     ***************************************************

      open ( funf, file = 'reserved.unf', status='NEW',
     .       access='DIRECT', recl=80, iostat=ios )
      if ( ios .ne. 0 ) then
        iline = -1
        call errr ( 1, 'unable to open file new reserved.unf')
      endif

c     *  The first line and sorted list of keywords + positions

      write ( fixline, 9995) warn, nnewkeys
      write ( funf, rec=1 ) fixline

      if ( prtnew ) write ( fout, 9999) ' ',
     .    'keywords in new reserved.unf and reserved.dat',
     .    '---------------------------------------------'

c     * begining of first keyexpl in new reserved.unf
      iline = 1 + 1 + nnewkeys

      do ireor = 1, nnewkeys
        ikeys = reord(ireor)
        nln = epos(ikeys) - bpos(ikeys) + 1
        write ( fixline, 9996) ireor, keys(ikeys), iline, iline+nln-1
        write ( funf, rec=1+ireor ) fixline
        iline = iline + nln
      enddo

c     ***************************************************
c     **   Open file reserved.dat to write there       **
c     ***************************************************

      open ( fdat, file='reserved.dat' )
c     *  Clean it, if no write happens then it must be empty
      close ( fdat, status='DELETE' )

      open ( fdat, file='reserved.dat' )

c     *  Initialize iline, last writing:  rec=1+nnewkeys
      iline = 1 + nnewkeys

      do ireor = 1, nnewkeys
        ikeys = reord(ireor)
        do iexpl = bpos(ikeys), epos(ikeys)

          iline = iline + 1
          line = keyexpl(iexpl)
          write ( funf, rec=iline ) line

c         *  Let k5 = length(line)
          do k5 = 80, 1, -1
            if ( line(k5:k5) .ne. ' ' ) goto 610
          enddo
          k5 = 0
  610     continue

          write ( fdat, 9999) line(1:k5)

          if ( prtnew .and. iexpl .eq. bpos(ikeys) ) then
            write ( fout, 9999) line(1:k5)
          endif

        enddo
      enddo

      close ( funf )
      close ( fdat )

      if ( prtnew .and. prtnum ) write ( fout, 9997)
     .    'number of keywords in new reserved.unf =', nnewkeys,
     .    'number of explanation lines            =', iline-nnewkeys-1
cxxx .   ,'number of all lines                    =', iline

      if ( prtfil ) write ( fout, 9999) ' ',
     .   'output files',
     .   '------------',
     .   '       reserved.unf  ... direct access explanations',
     .   '       reserved.dat  ... formated      explanations',
     .   '       reserved.cpr  ... report from program "buildres"'

      if ( prthed ) write ( fout, 9999 ) ' ',
     .  'normal end of program "buildres"',
     .  '================================'

      close ( fout )

 9995 format ( a50, i5 )
 9996 format ( i5, 2x, a8, 1x, 2i6, 52x )
 9997 format ( / (a,i6) )
 9998 format ( 5a )
 9999 format ( a )

      end
c ======================================================================
      subroutine buildsor ( updkey, iexpl,
     .                      mxkeys, keys, bpos, epos, nkeys, kkeys )

cimp      implicit none

c     ---------------------- i/o parameters ------

      integer iexpl, mxkeys, bpos(mxkeys), epos(mxkeys), nkeys, kkeys
      character * 8 updkey, keys(mxkeys)

c  updkey   i    ... see description in main
c  iexpl    i
c  mxkeys   i
c  keys    i/o
c  bpos    i/o
c  epos    i/o
c  nkeys   i/o
c  kkeys   i/o

c     ---------------------- constants -----------

      logical deb2
      parameter ( deb2 = .false. )

c     ---------------------- local variables -----

      integer ikeys

c     --------------------------------------------

c     *  A keyword is coming

c     *  Store end of previous explanation, if any

      if ( nkeys.gt.0 .and. kkeys.ne.0 ) then
        epos(kkeys) = iexpl - 1
c       *  Display previous, its pointers are complete
        if ( deb2 ) write (*,*) nkeys, ' ',
     .    keys(kkeys), ' : ', bpos(kkeys), epos(kkeys)
      endif

      if ( updkey .ne. '~~~~~~~~' ) then

c         *  Get a space for this keyword

          if ( nkeys.ge.mxkeys ) then
            call errr ( 1, 'mxkeys overflow, enlarge it')
          endif

          nkeys = nkeys + 1

c         *  1..nkeys-1 is sorted, nkeys should be inserted or ignored

c         *  Sequantial search from bottom, direct insert
c         *  It is effective even when huge sorted reserved.upd comes
c         *  ( as if reserved.dat is copied into reserved.unf )

c         *  We do not support huge random ordered reserved.upd !!!

          do 700 ikeys = nkeys-1, 1, -1
            if ( updkey .eq. keys(ikeys) ) then
c             *  Another occurence of the same keyword
              call errr ( 0, ' duplicate keyword, ignored')
              nkeys = nkeys - 1
              kkeys = 0
              goto 720
            elseif ( updkey .lt. keys(ikeys) ) then
c             * push  ....(ikeys)  down
              keys(ikeys+1) = keys(ikeys)
              bpos(ikeys+1) = bpos(ikeys)
              epos(ikeys+1) = epos(ikeys)
            else
c             *  updkey > keys(ikeys)
c             *  put  updkey at position ikeys+1
              kkeys = ikeys+1
              goto 710
            endif
  700     continue
          kkeys = 1
  710     continue
          keys(kkeys) = updkey
          bpos(kkeys) = iexpl
  720     continue

      else

c       *  The list is complete

        if ( deb2 ) then
          write (*,*)
          do 730 ikeys = 1, nkeys
            write (*,*) ikeys, ' ',
     .        keys(ikeys), bpos(ikeys), epos(ikeys)
  730     continue
        endif

      endif

      end
c ======================================================================
      subroutine errr ( stopit, message )

cimp      implicit none

c     ---------------------- i/o parameters ------

      integer stopit
      character * (*) message

c     message   i  an error message
c     stopit    i  0 = do not stop

c     --------------------- commons ------------------

      integer        fout, fupd, funf, fdat, iline, nerr
      character * 80 line
      common /crefs/ fout, fupd, funf, fdat, iline, nerr
     .       /cline/ line

c     --------------------------------------------

      nerr = nerr + 1

      if ( iline .ge. 0 ) then
        write ( fout, '( 80(''_'') / a / a, i5, 2a / 80(''^'') )' )
     .    line, '  !!! error, line ', iline, ': ', message
      else
        write ( fout, '( 80(''_'') / 2a / 80(''^'') )' )
     .    '  !!! error: ', message
      endif

      if ( stopit .ne. 0 ) then
        write ( fout, '(a)' ) '------- fatal error ------ stop --------'
        close ( fout )
        stop
      endif

      end
