      program contube
      implicit none

      integer nbuffr
      parameter( nbuffr = 4000000 )
      integer ibuffr
      common ibuffr(nbuffr)

      call ismain( nbuffr )

      end

      function usfunb ( ichoice, x, y, z, t )

c     User written function subroutine. It gives
c     the user the opportunity to define a
c     boundary condition as a function of space
c     and time.

      implicit none      

      double precision usfunb, x, y, z, t
      integer ichoice

c     x        i    x-coordinate 
c     y        i    y-coordinate 
c     t        i    actual time 
c     ichoice  i    choice parameter given by the user input
c     usfunb   o    computed boundary condition

      double precision u, u0, tke, eps, cmu, kappa,
     +                 lscale, it, h

c     cmu           an empirical constant
c     eps           dissipation rate of turbulent energy  
c     h             distance between wall and symmetry 
c     it            turbulence intensity 
c     kappa         von karman constant
c     lscale        length scale
c     tke           turbulent kinetic energy
c     u             tangential velocity
c     u0            centreline velocity

      cmu    = 9d-2
      kappa  = 41d-2

      u0     = 0.342d0
      It     = 3.13d-2
      H      = 0.0254d0

      u = u0*((y/H)**1.5625d-1)

      tke    = 15d-1*u*u*It*It
      if ( tke .eq. 0d0 ) tke = 1d-4

      lscale = min(kappa*y, 0.13*H)

      if ( lscale .eq. 0d0 ) then
        eps = 1d4
      else
        eps = (cmu**75d-2)*(tke**15d-1)/lscale
      end if

      if ( ichoice .eq. 1 ) then
        usfunb = -u
      else if ( ichoice .eq. 2 ) then
        usfunb =  tke
      else if ( ichoice .eq. 3 ) then
        usfunb =  eps 
      end if

      end



