      subroutine ischecktop ( nblocks, nfaces, maxnsubf, nsubfaces,
     +                        subfaces )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 07-09-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Check the block topology from the grid input file.
c
c **********************************************************************
c
c                       KEYWORDS
c
c   check
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxnsubf, nblocks, nfaces
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     maxnsubf       i    Maximum number of subfaces encountered on all subfaces
c     nblocks        i    Number of blocks
c     nfaces         i    Number of faces
c     nsubfaces      i    Array containing the number of subfaces of each face,
c                         see array itopol for an extended description
c     subfaces       i    This array contains a description of all subfaces
c                         subfaces(iblock,iface,isubface,.) refers to block iblo
c                         face iface and subface isubface
c                         Contents of the last index
c                         Pos. 1  i1
c                         Pos. 2  i2
c                         Pos. 3  j1
c                         Pos. 4  j2
c                         Pos. 5  block number neighbour block
c                         Pos. 6  face number neighbour face
c                         Pos. 7  subface number neighbour subface
c                         Pos. 8  iorient
c                         Pos. 9  Reserved for preprocessor only
c                         Pos. 10 Reserved for preprocessor only
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, iface, isubface, nbblock, nbsface, nsubf,
     +        nbface

c     iblock         Block sequence number
c     iface          Face number
c     isubface       Subface number
c     nbblock        Sequence number of a neighbouring block
c     nbface         Neighbouring face number
c     nbsface        Neighbouring subface number
c     nsubf          Number of subfaces
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     621   incorrect neighbour block number
c     622   neighbour face number # 0, whereas the neighbour block number = 0
c     623   neighbour subface number # 0, whereas the neighbour block number = 0
c     624   incorrect neighbour face number
c     625   incorrect neighbour subface number
c     626   incorrect topology
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For iblock := 1 (1) nblocks do
c       For iface := 1 (1) nfaces do
c          nsubf := nsubfaces(iblock,iface)
c          For isubfaces := 1 (1) nsubf do
c             Check position 5 6 and 7 of subfaces
c         End
c       End
c     End
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'ischecktop' )

c     --- Loop over blocks, faces, subfaces

      do iblock = 1, nblocks
         do iface = 1, nfaces
            nsubf = nsubfaces(iblock,iface)
            do isubface = 1, nsubf
               nbblock = subfaces(iblock,iface,isubface,5)
               nbface  = subfaces(iblock,iface,isubface,6)
               nbsface = subfaces(iblock,iface,isubface,7)
               if ( nbblock.lt.0 .or. nbblock.gt.nblocks ) then

c              --- nbblock out of range

                  call iserrint ( iblock, 1 )
                  call iserrint ( iface, 2 )
                  call iserrint ( isubface, 3 )
                  call iserrint ( nbblock, 4 )
                  call iserrint ( nblocks, 5 )
                  call iserrsub ( 621, 5, 0, 0 )

               end if
               if ( nbblock.gt.0 ) then

c              --- There is a neighbouring block, check topology

                  if ( nbface.lt.1 .or. nbface.gt.nfaces ) then

c                 --- nbface out of range

                     call iserrint ( iblock, 1 )
                     call iserrint ( iface, 2 )
                     call iserrint ( isubface, 3 )
                     call iserrint ( nbface, 4 )
                     call iserrint ( nfaces, 5 )
                     call iserrsub ( 624, 5, 0, 0 )
                     nbface = 1

                  end if
                  if ( nbsface.lt.1 .or.
     +                 nbsface.gt.nsubfaces(nbblock,nbface) ) then

c                 --- nbsface out of range

                     call iserrint ( iblock, 1 )
                     call iserrint ( iface, 2 )
                     call iserrint ( isubface, 3 )
                     call iserrint ( nbsface, 4 )
                     call iserrint ( nsubfaces(nbblock,nbface), 5 )
                     call iserrsub ( 625, 5, 0, 0 )
                     nbsface = 1

                  end if
                  if ( subfaces(nbblock,nbface,nbsface,5).ne.iblock .or.
     +                 subfaces(nbblock,nbface,nbsface,6).ne.iface .or.
     +                 subfaces(nbblock,nbface,nbsface,7).ne.isubface )
     +               then

c                 --- incorrect reference

                     call iserrint ( iblock, 1 )
                     call iserrint ( iface, 2 )
                     call iserrint ( isubface, 3 )
                     call iserrint ( nbblock, 4 )
                     call iserrint ( nbface, 5 )
                     call iserrint ( nbsface, 6 )
                     call iserrsub ( 626, 6, 0, 0 )

                  end if

               else

c              --- There is no neighbouring block, nbface and nbsface
c                  must be zero

                  if ( nbface.ne.0 ) then

c                 --- nbface # 0

                     call iserrint ( iblock, 1 )
                     call iserrint ( iface, 2 )
                     call iserrint ( isubface, 3 )
                     call iserrint ( nbface, 4 )
                     call iserrsub ( 622, 4, 0, 0 )

                  end if
                  if ( nbsface.ne.0 ) then

c                 --- nbsface # 0

                     call iserrint ( iblock, 1 )
                     call iserrint ( iface, 2 )
                     call iserrint ( isubface, 3 )
                     call iserrint ( nbsface, 4 )
                     call iserrsub ( 623, 4, 0, 0 )

                  end if

               end if

            end do
         end do
      end do

      call iserclos ( 'ischecktop' )

      end
