      subroutine iscoftu2 ( ichois, mturb  , ncoefs, ndegfd  ,
     +                      ntimlv, ni     , nj    , nvirtual,
     +                      coefs , rintur , solut , eddy    ,
     +                      work  , viscorr                   )
c ======================================================================
c
c        programmer     Marcel Zijlema
c
c        version 2.2    date   03-09-1996 Marcel: Nonlinear effects
c        version 2.1    date   11-07-1996 Marcel: Lam-Bremhorst model
c        version 2.0    date   23-05-1996 Marcel: Extended k-eps model
c        version 1.1    date   18-01-1995 Erik: added defensive programming
c                                         Virtual coefficients get ridiculous
c                                         values. They MUST(!!!!) be
c                                         overwritten by isfilvcf later.
c        version 1.0    date   29-11-1994
c
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fills time, place and solution dependent coefficients of
c     several turbulence equations (2D)
c **********************************************************************
c
c                       KEYWORDS
c
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ichois, mturb, ncoefs, ndegfd,
     +        ntimlv, ni, nj, nvirtual, viscorr
      double precision eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,1:ncoefs),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv),
     +                 rintur(*),
     +                 work(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*)

c     coefs    o    reserved
c     eddy     i    reserved
c     ichois   i    General choice parameter
c     mturb    i    Indicates type of turbulence modelling
c     ncoefs   i    Number of coefficients
c     ndegfd   i    Number of degrees of freedom
c     ni       i    Number of non-virtual cells in the i-direction (one block)
c     nj       i    Number of non-virtual cells in the j-direction (one block)
c     ntimlv   i    Number of time levels
c     nvirtual i    Number of rows of virtual cells
c     rintur   i    reserved
c     solut    i    reserved
c     viscorr  i    Indicate whether to use viscous corrections or not
c     work     i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp
      double precision cmu, sigmak, sigmaeps, c1eps, c2eps, c3eps,
     +                 eta0, betarng, alphakw, beta1kw, beta2kw,
     +                 sigma1kw, sigma2kw, eta, hdenom, fmu, f1, f2,
     +                 ret

c     alphakw   Parameter alpha for k-omega model
c     beta1kw   Parameter beta1 for k-omega model
c     beta2kw   Parameter beta2 for k-omega model
c     betarng   Parameter beta for RNG k-eps model
c     c1eps     Parameter c1_eps for standard or RNG k-eps model
c     c2eps     Parameter c2_eps for standard or RNG k-eps model
c     c3eps     Parameter c3_eps for extended k-eps model
c     cmu       Parameter c_mu for standard or RNG k-eps model
c     eta       Help variable for RNG k-eps model
c     eta0      Parameter eta_0 for RNG k-eps model
c     f1        Viscous damping function
c     f2        Viscous damping function
c     fmu       Viscous damping function
c     hdenom    Help variable for RNG k-eps model
c     icoef     Sequence number of coefficient
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     ret       Turbulent Reynolds number
c     sigma1kw  Parameter sigma1 for k-omega model
c     sigma2kw  Parameter sigma2 for k-omega model
c     sigmaeps  Parameter sigma_eps for standard or RNG k-eps model
c     sigmak    Parameter sigma_k for standard or RNG k-eps model
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      integer icoef

      cmu      = rintur(3)
      sigmak   = rintur(4)
      sigmaeps = rintur(5)
      c1eps    = rintur(6)
      c2eps    = rintur(7)
      eta0     = rintur(12)
      betarng  = rintur(13)
      alphakw  = rintur(15)
      beta1kw  = rintur(16)
      beta2kw  = rintur(17)
      sigma1kw = rintur(18)
      sigma2kw = rintur(19)
      c3eps    = rintur(21)

c     --- fill virtual coefficients with ridiculous values

      do icoef = 1, 6

         do jp = 1-nvirtual, nj+1+nvirtual

c        --- right side

            do ip = ni+1, ni+1+nvirtual

               coefs(ip,jp,icoef) = 1d20

            end do

c           --- left side

            do ip = 1-nvirtual, 0

               coefs(ip,jp,icoef) = 1d20

            end do

         end do

         do ip = 1-nvirtual, ni+1+nvirtual

c        --- lower side

            do jp = 1-nvirtual, 0

               coefs(ip,jp,icoef) = 1d20

            end do

c           --- upper side

            do jp = nj+1, nj+1+nvirtual

               coefs(ip,jp,icoef) = 1d20

            end do

         end do

      end do

      if ( mturb.eq.1 .or. mturb.eq.2 .or. mturb.eq.5 ) then

c     --- standard k-eps model or RNG k-eps model or
c         extended k-eps model of Chen and Kim

        if ( ichois.eq.-2 ) then

c       --- turbulent kinetic energy equation

          do 10 jp = 1, nj
            do 10 ip = 1, ni

              coefs(ip,jp,2) = 2d0*coefs(ip,jp,1)*solut(ip,jp,ndegfd,1)/
     +                         solut(ip,jp,ndegfd-1,1)
              coefs(ip,jp,3) = work(ip,jp,1) + coefs(ip,jp,1) *
     +                         solut(ip,jp,ndegfd,1)
              coefs(ip,jp,4) = coefs(ip,jp,4) + coefs(ip,jp,1) *
     +                         eddy(ip,jp) / sigmak
              coefs(ip,jp,6) = coefs(ip,jp,6) + coefs(ip,jp,1) *
     +                         eddy(ip,jp) / sigmak

              if ( work(ip,jp,2).ge.0 ) then
                coefs(ip,jp,3) = coefs(ip,jp,3) + work(ip,jp,2)
              else if ( work(ip,jp,2).lt.0 ) then
                coefs(ip,jp,2) = coefs(ip,jp,2) - work(ip,jp,2)/
     +                           solut(ip,jp,ndegfd-1,1)
              end if

10        continue

        else if ( ichois.eq.-1 ) then

c       --- dissipation rate equation

          if ( viscorr.eq.0 ) then

c         --- No low-Reynolds-number corrections

            do 20 jp = 1, nj
              do 20 ip = 1, ni

                coefs(ip,jp,2) = 2d0 * c2eps * coefs(ip,jp,1) *
     +                           solut(ip,jp,ndegfd,1) /
     +                           solut(ip,jp,ndegfd-1,1)
                coefs(ip,jp,3) = c1eps * solut(ip,jp,ndegfd,1) *
     +                           work(ip,jp,1)/solut(ip,jp,ndegfd-1,1) +
     +                           c2eps * coefs(ip,jp,1) *
     +                           solut(ip,jp,ndegfd,1) *
     +                           solut(ip,jp,ndegfd,1) /
     +                           solut(ip,jp,ndegfd-1,1)
                coefs(ip,jp,4) = coefs(ip,jp,4) + coefs(ip,jp,1) *
     +                           eddy(ip,jp) / sigmaeps
                coefs(ip,jp,6) = coefs(ip,jp,6) + coefs(ip,jp,1) *
     +                           eddy(ip,jp) / sigmaeps

                if ( work(ip,jp,2).ge.0 ) then
                  coefs(ip,jp,3) = coefs(ip,jp,3) + c1eps*
     +                             solut(ip,jp,ndegfd,1)*work(ip,jp,2)/
     +                             solut(ip,jp,ndegfd-1,1)
                else if ( work(ip,jp,2).lt.0 ) then
                  coefs(ip,jp,2) = coefs(ip,jp,2) - c1eps*work(ip,jp,2)/
     +                             solut(ip,jp,ndegfd-1,1)
                end if

20          continue

          else if ( viscorr.eq.1 ) then

c         --- Use Lam-Bremhorst near-wall modeling

            do 25 jp = 1, nj
              do 25 ip = 1, ni

                ret = coefs(ip,jp,1)*(solut(ip,jp,ndegfd-1,1)**2)/
     +                (coefs(ip,jp,4)*solut(ip,jp,ndegfd,1))
                fmu = solut(ip,jp,ndegfd,1) * eddy(ip,jp)/
     +                (cmu * (solut(ip,jp,ndegfd-1,1)**2))
                f1 = 1d0 + (5d-2/fmu)**3
                if ( ret.gt.8d0 ) then
                  f2 = 1d0
                else
                  f2 = 1d0 - exp(-(ret**2))
                end if
                if ( f2.lt.1d-2 ) f2 = 1d-2

                coefs(ip,jp,2) = 2d0 * f2 * c2eps * coefs(ip,jp,1) *
     +                           solut(ip,jp,ndegfd,1) /
     +                           solut(ip,jp,ndegfd-1,1)
                coefs(ip,jp,3) = f1 * c1eps * solut(ip,jp,ndegfd,1) *
     +                           work(ip,jp,1)/solut(ip,jp,ndegfd-1,1) +
     +                           f2 * c2eps * coefs(ip,jp,1) *
     +                           solut(ip,jp,ndegfd,1) *
     +                           solut(ip,jp,ndegfd,1) /
     +                           solut(ip,jp,ndegfd-1,1)
                coefs(ip,jp,4) = coefs(ip,jp,4) + coefs(ip,jp,1) *
     +                           eddy(ip,jp) / sigmaeps
                coefs(ip,jp,6) = coefs(ip,jp,6) + coefs(ip,jp,1) *
     +                           eddy(ip,jp) / sigmaeps

                if ( work(ip,jp,2).ge.0 ) then
                  coefs(ip,jp,3) = coefs(ip,jp,3) + f1 * c1eps *
     +                             solut(ip,jp,ndegfd,1)*work(ip,jp,2)/
     +                             solut(ip,jp,ndegfd-1,1)
                else if ( work(ip,jp,2).lt.0 ) then
                  coefs(ip,jp,2) = coefs(ip,jp,2) - f1 * c1eps *
     +                             work(ip,jp,2)/
     +                             solut(ip,jp,ndegfd-1,1)
                end if

25          continue

          end if

          if ( mturb.eq.2 ) then

c         --- Modification of the production term in eps-equation
c             to account for large strain rates

            do 30 jp = 1, nj
              do 30 ip = 1, ni

                eta = sqrt( work(ip,jp,1) / ( coefs(ip,jp,1) * cmu *
     +                      solut(ip,jp,ndegfd,1) ) )
                hdenom = 1d0/(1d0 + betarng*eta*eta*eta)

                coefs(ip,jp,2) = coefs(ip,jp,2) + eta * hdenom *
     +                           work(ip,jp,1)/solut(ip,jp,ndegfd-1,1)
                coefs(ip,jp,3) = coefs(ip,jp,3) +
     +                           (eta*eta*hdenom/eta0)*
     +                           solut(ip,jp,ndegfd,1)*work(ip,jp,1)/
     +                           solut(ip,jp,ndegfd-1,1)

                if ( work(ip,jp,2).ge.0 ) then
                  coefs(ip,jp,2) = coefs(ip,jp,2) + eta * hdenom *
     +                             work(ip,jp,2)/solut(ip,jp,ndegfd-1,1)
                  coefs(ip,jp,3) = coefs(ip,jp,3) +
     +                             (eta*eta*hdenom/eta0)*
     +                             solut(ip,jp,ndegfd,1)*work(ip,jp,2)/
     +                             solut(ip,jp,ndegfd-1,1)
                else if ( work(ip,jp,2).lt.0 ) then
                  coefs(ip,jp,2) = coefs(ip,jp,2) -
     +                             (eta*eta*hdenom/eta0)*work(ip,jp,2)/
     +                             solut(ip,jp,ndegfd-1,1)
                  coefs(ip,jp,3) = coefs(ip,jp,3) - eta * hdenom *
     +                             solut(ip,jp,ndegfd,1)*work(ip,jp,2)/
     +                             solut(ip,jp,ndegfd-1,1)
                end if

30          continue

          else if ( mturb.eq.5 ) then

c         --- Extended k-eps model of Chen and Kim

            do 35 jp = 1, nj
              do 35 ip = 1, ni

                coefs(ip,jp,3) = coefs(ip,jp,3) +
     +                           c3eps * work(ip,jp,1) * work(ip,jp,1)/
     +                           ( coefs(ip,jp,1) *
     +                             solut(ip,jp,ndegfd-1,1) )

                coefs(ip,jp,3) = coefs(ip,jp,3) +
     +                           c3eps * work(ip,jp,2) * work(ip,jp,2)/
     +                           ( coefs(ip,jp,1) *
     +                             solut(ip,jp,ndegfd-1,1) )
                if ( work(ip,jp,2).ge.0 ) then
                  coefs(ip,jp,3) = coefs(ip,jp,3) +
     +                             2d0*c3eps * work(ip,jp,1) *
     +                             work(ip,jp,2)/( coefs(ip,jp,1) *
     +                             solut(ip,jp,ndegfd-1,1) )
                else if ( work(ip,jp,2).lt.0 ) then
                  coefs(ip,jp,2) = coefs(ip,jp,2) -
     +                             2d0*c3eps * work(ip,jp,1) *
     +                             work(ip,jp,2)/( coefs(ip,jp,1) *
     +                             solut(ip,jp,ndegfd,1)*
     +                             solut(ip,jp,ndegfd-1,1) )
                end if

35          continue

          end if

        end if

      else if ( mturb.eq.3 ) then

c     --- Wilcox k-omega model

        if ( ichois.eq.-2 ) then

c       --- turbulent kinetic energy equation

          do 40 jp = 1, nj
            do 40 ip = 1, ni

              coefs(ip,jp,2) = 2d0*coefs(ip,jp,1)*beta1kw*
     +                         solut(ip,jp,ndegfd,1)
              coefs(ip,jp,3) = work(ip,jp,1) + coefs(ip,jp,1)*beta1kw*
     +                         solut(ip,jp,ndegfd-1,1)*
     +                         solut(ip,jp,ndegfd,1)
              coefs(ip,jp,4) = coefs(ip,jp,4) + coefs(ip,jp,1)*
     +                         eddy(ip,jp)*sigma1kw
              coefs(ip,jp,6) = coefs(ip,jp,6) + coefs(ip,jp,1)*
     +                         eddy(ip,jp)*sigma1kw

              if ( work(ip,jp,2).ge.0 ) then
                coefs(ip,jp,3) = coefs(ip,jp,3) + work(ip,jp,2)
              else if ( work(ip,jp,2).lt.0 ) then
                coefs(ip,jp,2) = coefs(ip,jp,2) - work(ip,jp,2)/
     +                           solut(ip,jp,ndegfd-1,1)
              end if

40        continue

        else if ( ichois.eq.-1 ) then

c       --- specific dissipation rate equation

          do 50 jp = 1, nj
            do 50 ip = 1, ni

              coefs(ip,jp,2) = 2d0*coefs(ip,jp,1)*beta2kw*
     +                         solut(ip,jp,ndegfd,1)
              coefs(ip,jp,3) = alphakw*solut(ip,jp,ndegfd,1)*
     +                         work(ip,jp,1)/solut(ip,jp,ndegfd-1,1) +
     +                         coefs(ip,jp,1)*beta2kw*
     +                         solut(ip,jp,ndegfd,1)*
     +                         solut(ip,jp,ndegfd,1)
              coefs(ip,jp,4) = coefs(ip,jp,4) + coefs(ip,jp,1)*
     +                         eddy(ip,jp)*sigma2kw
              coefs(ip,jp,6) = coefs(ip,jp,6) + coefs(ip,jp,1)*
     +                         eddy(ip,jp)*sigma2kw

              if ( work(ip,jp,2).ge.0 ) then
                coefs(ip,jp,3) = coefs(ip,jp,3) + alphakw*
     +                           solut(ip,jp,ndegfd,1)*work(ip,jp,2)/
     +                           solut(ip,jp,ndegfd-1,1)
              else if ( work(ip,jp,2).lt.0 ) then
                coefs(ip,jp,2) = coefs(ip,jp,2)-alphakw*work(ip,jp,2)/
     +                           solut(ip,jp,ndegfd-1,1)
              end if

50        continue

        end if

      end if

      end
