      subroutine iscorr ( iadmin, isiinput, isrinput, issolut,
     +                    isgmat, isitopol, isinfbnd, isgeom,
     +                    isigeom, isbndcon, iscoor, iscoefs,
     +                    nblocks )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.4  23-11-1998 Jason: removed calls to isini* and isterm*
c     version      2.3  31-08-1998 Jason: remove check nodeid
c     version      2.2  06-08-1998 Guus: Use boundary conditions if coupled
c     version      2.1  22-07-1998 Duncan:  changed eq cond double
c     version      2.0  04-06-1998 Charles: isigeom added
c     version      1.9  05-11-1997 Charles: adaptation of isfilvso if coupled
c     version      1.8  11-09-1997 Charles: extra parameters icoupled,
c                                           iseqpres
c     version      1.7  21-04-1997 Kees K.: minor change for 3D
c     version      1.6  07-11-1995 Kees V.: boundary conditions for theta = 0
c     version      1.5  26-10-1995 Kees K.: itrans for 3D
c     version      1.4  04-10-1995 Kees K.: warning for itrans
c     version      1.3  23-05-1995 Kees K.: extrapolation added
c     version      1.2  06-04-1995 Kees K.: iarray mout and moutrc
c                                  arrays of length 1
c     version      1.1  21-07-1994 Erik:  additional transport of
c                                  pressures + printing of the net
c                                  mass outflow over all blocks
c     version      1.0  24-05-1994
c
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Correct fluxes: compute  V(n+1) = V(*) + deltat*G*Dp
c     for all subdomains that are solved on this machine.
c     Some additional communication is performed because the virtual
c     fluxes must also be updated.
c **********************************************************************
c
c                       KEYWORDS
c
c     pressure_correction
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), isgmat(*), isiinput(*), nblocks,
     +        isrinput(*), issolut(*), isitopol, isinfbnd(*),
     +        isgeom(*), isigeom(*), isbndcon(nblocks,*),
     +        iscoor(*), iscoefs(*)

c     iadmin   i  reserved
c     isbndcon i  Array of sequence numbers (nblocks) of array bndcon
c     iscoefs  i  Array of sequence numbers (nblocks) of array coefs
c     iscoor   i  Array of sequence numbers (nblocks) of array coor
c     isgeom   i  Array of sequence numbers (nblocks) of array geom
c     isgmat   i  Array of sequence numbers (nblocks) of array gmat
c     isigeom  i  Array of sequence numbers (nblocks) of array igeom
c     isiinput i  Array of sequence numbers (nblocks) of array iinput
c     isinfbnd i  Array of sequence numbers (nblocks) of array infbnd
c     isitopol i  Sequence number of array itopol
c     isrinput i  Array of sequence numbers (nblocks) of array rinput
c     issolut  i  Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, idummy, ind, ilev,
     +        ipiinput, itrans, 
     +        ndim, iequa, itimlv, node,
     +        ni, nj, ntimlv, nvirtual,
     +        iprinput, iprintim, ndegfd, ipsolut, ipinfbnd,
     +        npointsv, ipinvjac, ipgeom, ipbndcon, ipcoor, lwork,
     +        iswork, ipwork, ipgeo, iseqpres, icoupled
      double precision mout(1), moutrc(1), tmout, theta
      logical usebndcn, ddummy

c     ddummy    dummy logical
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idummy    Integer dummy
c     iequa     General equation sequence number
c     ilev      Time level of the multi-block transport of information
c     ind       Sequence number of node
c     ipbndcon  Starting address of array bndcon in buffer
c     ipcoor    Starting address of array coor in buffer
c     ipgeo     Starting address of array GEO in array WORK
c     ipgeom    Starting address of array geom in buffer
c     ipiinput  Starting address of array iinput in ibuffr
c     ipinfbnd  Starting address of array infbnd in ibuffr
c     ipinvjac  Starting address of array invjac in geom
c     iprinput  Starting address of array rinput in buffer
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ipsolut   Starting address of array solut in buffer
c     ipwork    Starting address of array work in buffer
c     iseqpres  Sequence number of pressure equation
c     iswork    Sequence number of array work
c     itimlv    Loop variable with respect to a given time level
c     itrans    reserved
c     lwork     Length of array work
c     mout      Net mass out-flow of the local (node) or all (host) subdomains
c     moutrc    Net mass out-flow computed on a node received by the host
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     node      Node actually received from
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     theta     Parameter theta for theta method
c     tmout     Mout for one block
c     usebndcn  If true boundary conditions are used in the extrapolation
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer  isingetp, isdpgetp

c     ISACTIVE  Activate permanent array
c     ISBGEO2D  Fill geometrical quantities along boundary in R2
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISEXBO21  Fills virtual points at outer boundaries for theta is 0
c     ISEXBOU2  Fills the boundary conditions for theta = 0
c     ISFILVSO  Extrapolate solution at outer boundaries
c     ISGLSUM   ISNaS parallel global sum routine
c     ISINGETP  Get the pointer corresponding to an integer array
c     ISMBLK    Performs the local transport of multi-block information
c               within a node
c     ISNETFLO  Net outflow
c     ISPRC01   Pressure correction
c     ISRCDDHS  Receives the multi-block information from each node
c               and place it at the correct location in the arrays
c               of virtual unknowns.
c     ISRCDDND  Receives the virtual unknowns for all
c               local blocks at a node and puts them into the local
c               solut arrays
c     ISSNDDHS  Copy the arrays of virtual unknowns for all
c               blocks to the local copies at the nodes, using PVM
c               if necessary.
c     ISSNDDND  Copy unknowns near or on the boundary of the subdomain
c               solut arrays to the solut arrays of the neighbouring
c               blocks on the host
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Outline:
c       - correct fluxes and pressures
c       - perform an additional transport of virtual fluxes and pressures
c         to ensure a single global pressure and velocity field at the
c         next time step. This is essential for convergence of the
c         time integration.
c
c     More detailed explanation:
c
c       Correct fluxes and pressures for all local blocks and compute the
c       net mass outflow over all local blocks
c
c       if ( on the host)
c          put fluxes and pressures of the local blocks in the solut arrays
c          of the neighbouring blocks
c
c       if ( on the host)
c          receive fluxes and pressures and net mass outflow
c          from each node not on the host.
c          net_mass_outflow:= net_mass_outflow + received_mass_outflow
c       else
c          send fluxes and pressures and net mass outflow
c
c       if ( on the host)
c         send fluxes and pressures
c       else
c         receive fluxes and pressures
c
c       Update the solut arrays of the local blocks or (all blocks if
c       on the host) with the new flux and pressurevalues
c
c       Print the net mass outflow over all blocks.
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('iscorr')

c     --- compute some general parameters

      call isactive(isiinput(1))
      call isactive(isrinput(1))

      ipiinput     = isingetp(isiinput(1))

      ndim         = ibuffr(ipiinput + 57 - 1)
      ntimlv       = ibuffr(ipiinput + 64 - 1)
      nvirtual     = ibuffr(ipiinput + 54 - 1)
      icoupled     = ibuffr(ipiinput + 76 - 1)

      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim+1
      end if

c     --- correct the subdomain V(*) in solut and correct the pressure

      mout(1) = 0d0

      do iblock = 1, nblocks

         iblockrf = iblock

         if ( iadmin(iblock).eq.inodenr ) then

            call isactive(isiinput(iblock))
            call isactive(isrinput(iblock))
            call isactive(issolut(iblock))
            call isactive(isgmat(iblock))
            call isactive(iscoefs(iblock))
            call isactive(iscoor(iblock))
            call isactive(isgeom(iblock))
            call isactive(isinfbnd(iblock))
            if ( isigeom(iblock).gt.0 ) call isactive(isigeom(iblock))

c           --- correct fluxes and pressures

            call isprc01( isiinput(iblock), isrinput(iblock),
     +                    issolut(iblock), isgmat(iblock),
     +                    iscoefs(iblock), iscoor(iblock),
     +                    isgeom(iblock), isigeom(iblock),
     +                    isinfbnd(iblock) )

c           --- calculate net outflow

            call isnetflo( isiinput(iblock), issolut(iblock), tmout )
            mout(1) = mout(1) + tmout

         end if
      end do

c     --- put corrected fluxes and pressures of the local blocks
c         in the solut arrays of the neighbouring blocks if on the host

      if ( inodenr.eq.ISHOST ) then
         do iblock = 1, nblocks
            if ( iadmin(iblock).eq.inodenr) then

               itrans = 1
               ilev   = ntimlv

               call ismblk(iblock, itrans, ilev, isiinput, issolut,
     +                     iadmin, isitopol)

               itrans = ndim+1
               ilev   = ntimlv-1
               call ismblk(iblock, itrans, ilev, isiinput, issolut,
     +                     iadmin, isitopol)

            end if
         end do
      end if

c     --- extrapolate solution to virtual points

      do iblock = 1, nblocks

         if( iadmin(iblock).eq.inodenr) then

            do iequa = 1, iseqpres

               if ( iequa.lt.iseqpres) then
                  itrans = iequa
                  itimlv = 2
               else
                 itrans = iequa + ndim -1 - iseqpres + 2
                 itimlv = 1
               end if

c              --- The value of theta is determined

               call isactive(isiinput(iblock))
               call isactive(isrinput(iblock))

               ipiinput = isingetp( isiinput(iblock) )
               iprinput = isdpgetp( isrinput(iblock) )
               iprintim = ibuffr( ipiinput -1 + 29 )
               theta = buffer( iprinput - 1 + iprintim -1 + 5 )

               if ( abs(theta).lt.1d-12 .and. iequa.lt.iseqpres ) then

c              --- if theta is 0 and iequa < iseqpres ( the momentum equations)
c                  the boundary conditions are used to adapt the
c                  solution on the boundary and also in virtual points
c                  for the tangential velocities

                  ni       = ibuffr(ipiinput + 51 - 1)
                  nj       = ibuffr(ipiinput + 52 - 1)
                  ndegfd   = ibuffr(ipiinput + 55 - 1)
                  itimlv   = 2
                  npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
                  lwork    = 16*npointsv+32*(ni+nj+2+4*nvirtual)
                  iswork   = 0

                  call isdcreat(iswork, lwork, 'work' )

                  call isactive(issolut(iblock))
                  call isactive(isinfbnd(iblock))
                  call isactive(isgeom(iblock))

c                 --- call isactive(isbndcon(iblock))

                  call isactive(isbndcon(iblock,iequa))
                  call isactive(iscoor(iblock))

                  ipinfbnd = isingetp(isinfbnd(iblock))
                  ipsolut  = isdpgetp(issolut(iblock) )
                  ipgeom   = isdpgetp(isgeom(iblock)  )

c                 --- ipbndcon = isdpgetp(isbndcon(iblock))

                  ipbndcon = isdpgetp(isbndcon(iblock,iequa))
                  ipcoor   = isdpgetp(iscoor(iblock)  )
                  ipwork   = isdpgetp( iswork )
                  ipinvjac = npointsv*(2**ndim) + ipgeom
                  ipgeo    = ipwork+16*npointsv

c                 --- Fill covariant basis vectors in cva for all
c                     integration points

                  call iscova2d ( buffer(ipcoor), buffer(ipwork),
     +                            ni, nj, nvirtual )

c                 --- Fill array bcgeo and put into array buffer

                  call isbgeo2d ( buffer(ipgeo), ni, nj, nvirtual,
     +                            buffer(ipwork), ibuffr(ipinfbnd),
     +                            buffer(ipinvjac), buffer(ipgeom) )

c                 --- Finally the boundary conditions are used and the
c                     virtual points are filled

                  call isexbou2 ( ni, nj, nvirtual, ndegfd, itimlv,
     +                         buffer(ipsolut),
     +                         ibuffr(ipinfbnd+2*ndim*(1+2*(ndim-1))),
     +                         ibuffr(ipinfbnd), buffer(ipgeo),
     +                         buffer(ipbndcon) )

                  call isexbo21 ( iblock, itrans, itimlv, nblocks,
     +                         isiinput(iblock), issolut(iblock),
     +                         isitopol,
     +                         ibuffr(ipinfbnd+2*ndim*(1+2*(ndim-1))),
     +                         ibuffr(ipinfbnd), buffer(ipgeo),
     +                         buffer(ipbndcon) )

                  call isdelarr(iswork)
                  call isdcrpri

               else

                  if ( iequa.ne.iseqpres ) then
                     usebndcn = .true.
                  else
                     usebndcn = .false.
                  end if
                  call isfilvso ( iblock, itrans, itimlv, nblocks,
     +                            isiinput(iblock), issolut(iblock),
     +                            isitopol, isinfbnd(iblock),
     +                            isbndcon(iblock,iequa), icoupled,
     +                            usebndcn )

               end if

            end do

         end if

      end do

c     --- receive (host) or send (node) fluxes and pressures and net
c         mass outflow

      if ( inodenr.eq.ISHOST ) then

c     --- receive fluxes if on the host

         do ind = 1, iacnodes

c           --- receive fluxes

            call isrcddhs( idummy, isiinput, issolut, isitopol )

c           --- receive pressures

            call isrcddhs( idummy, isiinput, issolut, isitopol )

         end do

      else

c     --- send fluxes on the node

         ind = 0

c        --- send fluxes

         itrans = 1
         ilev   = ntimlv
         call issnddnd(itrans, ilev,   isiinput, issolut,
     +                 iadmin, isitopol)

c        --- send pressures

         itrans = ndim +1
         ilev   = ntimlv-1
         call issnddnd(itrans, ilev,   isiinput, issolut,
     +                 iadmin, isitopol)

      end if

      if ( iacnodes.gt.0 ) then

c        --- sum mass outflows of all nodes
	 call isglsum ( mout, moutrc, 1, ISDOUBLE)

	 mout(1) = moutrc(1)

      end if

c     --- send (host) or receive (node) fluxes

      if ( inodenr.eq.ISHOST ) then

c     --- send fluxes if on the host

         do ind = 1, iacnodes

c           --- send fluxes

            ddummy = .false.
            itrans = 1
            ilev   = ntimlv
            call issnddhs(nblocks, itrans, ilev, ind, isiinput,
     +                    issolut, isitopol, iadmin, ddummy)

c           --- send pressures

            itrans = ndim +1
            ilev   = ntimlv-1
            call issnddhs(nblocks, itrans, ilev, ind, isiinput,
     +                    issolut, isitopol, iadmin, ddummy)

         end do

      else

c     --- receive fluxes if on the node
c        --- receive fluxes

         call isrcddnd( nblocks, isiinput, issolut, isitopol,
     +                  iadmin, ddummy)

c        --- receive pressures

         call isrcddnd( nblocks, isiinput, issolut, isitopol,
     +                  iadmin, ddummy)

      end if

c     --- print the net mass outflow over all blocks

      if ( inodenr.eq.ISHOST ) write(*,1000) mout(1)

 1000 format(/,' THE NET MASS OUTFLOW OVER ALL BLOCKS IS ',
     +              SS,1P,E12.3E2)

      call iserclos('iscorr')

      end
