      subroutine iscptl ( isiinput, isrinput, issolut, iseddy )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 23-03-1999  Duncan: remove bug in istartup
c                                        add iindsc, remove include file
c        version 1.0    date 07-03-1996
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy timelevels for initial conditions
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer isiinput(*), isrinput(*), issolut(*), iseddy(*)

c     iseddy         i    Array of sequence numbers (nblocks) of array eddy
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, iprinput, ipsolut, ipwork,  iswork,
     +         nblocks, ndegfd, ndim,
     +         ni, nj, nk, ntimlv, nvirtual, ipeddy, ipiincom,
     +         iprincom, npointsv, mcom, ipiindsc,
     +         mscal, mcav, istartup, iprincav, igauss, ipiincav

c     iblock         Block sequence number
c     igauss         if igauss=1 non linear Gauss Seidel is applied
c                    else not
c     ipeddy         Starting address of array eddy in buffer
c     ipiincav       Starting address of array IINCAV in array IINPUT
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiindsc       Starting address of array IINDSC in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     iprincav       Starting address of array RINCAV in array RINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     iprinput       Starting address of array rinput in buffer
c     ipsolut        Starting address of array solut in buffer
c     ipwork         Starting address of array work in buffer
c     istartup       If 1 the cavity calculation is in the startup
c                    procedure, if 0 not
c     iswork         Sequence number of array work
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     nblocks        Number of blocks
c     ndegfd         Number of degrees of freedom
c     ndim           Dimension of space (2 or 3)
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     npointsv       Number of points in the mesh with virtual points
c     ntimlv         Number of time levels
c     nvirtual       Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE       Activate old permanent array
c     ISDCREAT       Create permanent space in array IBUFFR (Double precision ar
c     ISDELARR       Delete a permanent array
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISINCC2        Initialize array eddy
c     ISINCP2        Copy time level 1 of array solut to other time levels (2d)
c     ISINCP3        Copy time level 1 of array solut to other time levels (3d)
c     ISINGETP       get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Important remark:
c
c     At this moment the initial conditions are filled and copied to the other
c     time levels. After that the extrapolation to other blocks is carried
c     out.
c     In fact this is the wrong sequence:
c     - First everything must be filled
c     - Then the extrapolation and copying over block boundaries must be
c       performed
c     - Finally the copying to other time levels must be carried out
c
c     This must be improved in the future
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- determine some general parameters

      call iseropen( 'iscptl' )
      call isactive(isiinput(1))
      ipiinput = isingetp( isiinput(1) )

      nblocks  = ibuffr( ipiinput -1 + 73 )
      nvirtual = ibuffr( ipiinput -1 + 54 )
      ndegfd   = ibuffr( ipiinput -1 + 55 )
      ndim     = ibuffr( ipiinput -1 + 57 )
      ntimlv   = ibuffr( ipiinput -1 + 64 )

c     --- Copy time level 1 to time level 2

      do iblock = 1, nblocks

c     --- activate arrays

         call isactive(isiinput(iblock))
         call isactive(isrinput(iblock))
         call isactive(issolut(iblock))

         ipiinput = isingetp(isiinput(iblock))
         iprinput = isdpgetp(isrinput(iblock))
         ipiincom = ibuffr( ipiinput -1 + 9 )
         iprincom = ibuffr( ipiinput -1 + 34)
         ipiincav = ibuffr( ipiinput -1 + 11)
         iprincav = ibuffr( ipiinput -1 + 35)
         ipiindsc = ibuffr( ipiinput -1 + 6 )

         mcom = ibuffr(ipiinput-1+ipiincom)
         if ( mcom.eq.1 ) call isactive(iseddy(iblock))

c        --- determine some general parameters for initial conditions

         ni       = ibuffr( ipiinput -1 + 51 )
         nj       = ibuffr( ipiinput -1 + 52 )
         nk       = ibuffr( ipiinput -1 + 53 )
         ndegfd   = ibuffr( ipiinput + 55 - 1)
         ndim     = ibuffr( ipiinput + 57 - 1)

         if ( ndim.eq.2 ) then
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            iswork   = 0
            call isdcreat ( iswork, npointsv, 'work' )
         end if

c        --- compute pointers

         ipiinput = isingetp(isiinput(iblock))
         ipsolut  = isdpgetp(issolut(iblock))

         if(mcom.eq.1) then

            mscal  = ibuffr(ipiinput-1+4+ipiincom)
            mcav   = ibuffr(ipiinput-1+77)
            if(mcav.eq.1) then
              istartup = ibuffr( ipiinput -1 + ipiincav -1 + 2)
              igauss   = ibuffr( ipiinput -1 + ipiincav -1 + 1)
            else
              istartup = 0
              igauss   = 0
            end if

         end if

         if ( mcom.eq.1 ) then
            ipeddy = isdpgetp(iseddy(iblock))
         else
            ipeddy = 1
         end if

         if ( ndim.eq.2 ) then

c        --- ndim = 2

            call isincp2 ( buffer(ipsolut), ndegfd, nvirtual, ni, nj,
     +                     ntimlv, igauss )
            if ( mcom.eq.1 ) then

c           --- Compressible case, fill array eddy and perform scaling

               ipwork = isdpgetp(iswork  )
               call isincc2 ( buffer(ipsolut), ndegfd, nvirtual, ni,
     +                        nj, buffer(ipeddy), mscal, mcav,
     +                        buffer(iprinput-1+iprincom),
     +                        buffer(ipwork), istartup ,
     +                        buffer(iprinput-1+iprincav),
     +                        ibuffr(ipiinput-1+ipiindsc),
     +                        ibuffr(ipiinput-1+ipiincom) )
            end if

         else

c        --- ndim = 3

            call isincp3 ( buffer(ipsolut), ndegfd, nvirtual, ni, nj,
     +                     nk, ntimlv )

         end if

      end do

      if ( ndim.eq.2 ) call isdelarr(iswork)

      call iserclos( 'iscptl' )

      end
