      subroutine isdimsc(unscaled,scaled,rincom,ni,nj,itypesc)
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 1.0    date   05-03-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   ISnas Compressible subroutine isdimsc.f redimensionalizes the
c   pressure, density and velocity
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni,nj,itypesc
      double precision rincom(*), scaled(1:ni+1,1:nj+1),
     +              unscaled(1:ni+1,1:nj+1)

c     itypesc   i    itypesc=1 pressure ;itypesc=2 density ; itypesc=3 velocity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     rincom    i    reserved
c     scaled    i    scaled quantity
c     unscaled  o    unscaled quantity
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i,j
      double precision pref, rhoref, velref

c     i         General loop variable
c     j         General loop variable
c     pref      Reference value for pressure
c     rhoref    Reference value for density
c     velref    Reference value for velocity
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     for all points do
c     if (variable = pressure) pressure = pressure * rhoref*velref*.pout
c     if (variable = density)  density  = density * rhoref
c     if (variable = velocity) velocity = velocity * velref
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      pref   = rincom(7)
      rhoref = rincom(8)
      velref = rincom(9)

      if (itypesc.eq.1) then

          do j = 1, nj+1
            do i = 1, ni+1
              unscaled(i,j) = scaled(i,j)*(rhoref*velref**2d0)+pref
            end do
          end do

      else if (itypesc.eq.2) then

          do j = 1, nj+1
            do i = 1, ni+1
              unscaled(i,j) = scaled(i,j)*rhoref
            end do
          end do

      else if (itypesc.eq.3) then

          do j = 1, nj+1
            do i = 1, ni+1
              unscaled(i,j) = scaled(i,j)*velref
            end do
          end do

      end if

      end
