      subroutine isexpli ( iequa, nblocks, isiinput, isrinput, iscoor,
     +                     issolut, iscoefs, iseddy, isinfbnd, isbndcon,
     +                     ismatrix, isgmat, isrhside, isgeom, iadmin,
     +                     isitopol )
c ======================================================================
c
c        programmer     Kees Vuik
c
c        version 3.3    date 29-10-1998 Duncan: New call to isexmom
c        version 3.2    date 18-05-1998 Charles Bilinear discretization
c        version 3.1    date 05-11-1997 Guus: new call to isfilb
c        version 3.0    date 13-10-1997 Charles: uncoupled allowed
c        version 2.9    date 15-09-1997 Guus: extension because of compressible
c        version 2.8    date 10-08-1996 Guus: extension because of compressible
c                                       Navier-Stokes
c        version 1.5    date 13-02-1996 Zijlema: parameter list islsds01 changed
c        version 1.4    date 23-04-1996 Guus: New timing subroutines
c        version 1.3    date 10-04-1996 Kees K: parameter list isfilb changed
c        version 1.2    date 13-02-1996 Kees K: parameter list islsds01 changed
c        version 1.1    date 15-12-1995 Marcel: parameter list isfilb changed
c        version 1.0    date 12-09-1995
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine calculates the solution on the new time step
c     if theta is 0
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, nblocks, isiinput(*), isrinput(*), iscoor(*),
     +        issolut(*), iscoefs(*), iseddy(*), isinfbnd(*),
     +        isbndcon(*), ismatrix(nblocks, *), isgmat(*),
     +        isrhside(*), isgeom(*), iadmin(*), isitopol

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical firstcall
      integer iblock, ipcoefs, ipcoor, ipeddy, ipiincom, ipgeom,
     +        ipsolut, ipbndcon, ipinfbnd, ipgmat, ipiinput,
     +        ipinfmg, ipinfmm, ipinfmt, ipmatrix,
     +        iprhside, iprinput, ipwork, isinfmm, isinfmg ,
     +        isinfmt, iswork, linfmm, linfmg, linfmt, itypsten,
     +        lwork, ipiindsc, rspv(1:3,1:8,0:3,0:4,1:8),
     +        ni,  nj, nk, ndim, nvirtual, ndegfd  , ntimlv ,
     +        nturb, maxdim, maxdx, npf(0:3,0:4,1:8), itimlv, itrans,
     +        icoupled, iseqpres
      double precision cv(1:8,0:3,0:4,1:8)
      character*10 matname
      character*40 text
      save firstcall, npf, rspv, cv

c     cv        reserved
c     firstcall Indicator to find the first call of the routine
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ipbndcon  Starting address of array bndcon in buffer
c     ipcoefs   Starting address of array coefs in buffer
c     ipcoor    Starting address of array coor in buffer
c     ipeddy    Starting address of array eddy in buffer
c     ipgeom    Starting address of array geom in buffer
c     ipgmat    Starting address of array gmat in buffer
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipinfbnd  Starting address of array infbnd in ibuffr
c     ipinfmg   Starting address of array infmg in ibuffr
c     ipinfmm   Starting address of array infmm in ibuffr
c     ipinfmt   Starting address of array infmt in ibuffr
c     ipmatrix  Starting address of array matrix in buffer
c     iprhside  Starting address of array rhside in buffer
c     iprinput  Starting address of array rinput in buffer
c     ipsolut   Starting address of array solut in buffer
c     ipwork    Starting address of array work in buffer
c     iseqpres  Sequence number of pressure equation
c     isinfmg   Sequence number of array infmg
c     isinfmm   Sequence number of array infmm
c     isinfmt   Sequence number of array infmt
c     iswork    Sequence number of array work
c     itimlv    Loop variable with respect to a given time level
c     itrans    reserved
c     itypsten  Indicates the type of stencil
c               Possible values
c               R^2
c               1:  13-point stencil
c               2:  21-point stencil
c               3:  17-point stencil
c               R^3
c               1:  51-point stencil
c               2:  59-point stencil
c     linfmg    Length of array infmg
c     linfmm    Length of array infmm
c     linfmt    Length of array infmt
c     lwork     Length of array work
c     matname   Name of matrix to be build or solved
c     maxdim    Highest physical space dimension for which ISNaS can work
c     maxdx     Maximum relative shift between cell indices
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npf       reserved
c     ntimlv    Number of time levels
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c     rspv      reserved
c     text      String in which a text is stored
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISASKCLK  Print CPU time used
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c     ISEXMOM   Calculates the new solution for the momentum equations
c               for theta is 0
c     ISFILB    Fill boundary values
c     ISFILM    Fill integer information about the matrices
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISLSDS01  Compute lengths of temporary arrays
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISSETCLK  Set time in common cistime equal to actual CPU time
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      26   Explicit solution of transport has not yet been implemented
c **********************************************************************
c
c                       PSEUDO CODE
c
c         for iblock = 1 (1) nblocks do
c            if block(iblock) on this node  then
c              build equations for this block
c         end do
c      end if
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data firstcall / .true. /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isexpli')

      if (firstcall) then
      if ( inodenr.eq.0 ) write(irefwr,100)
 100   format('*****************************************',//,
     +           'Warning for the choice theta is 0,',
     +           'only part of the',/'Navier-Stokes equations',
     +           ' are implemented',//,
     +           '*****************************************')

c      --- Initialize tables containing interpolation and finite
c          difference formulas

         call isactive( isiinput(1) )
         ipiinput = isingetp(isiinput(1))
         ndim     = ibuffr(ipiinput-1+57)
         icoupled = ibuffr(ipiinput-1+76)
         if ( icoupled.eq.0 ) then
            iseqpres = 2
         else
            iseqpres = ndim + 1
         end if

         call ispndf(npf,rspv,cv,ndim)

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- compute solution on the new timestep

      do iblock = 1, nblocks

         if ( iadmin(iblock).eq.inodenr ) then

            if ( itime.eq.1 ) then

c           --- Start timer for the construction of the solution

               call issetclk(1)

            end if

c           --- compute lengths of local arrays

            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))

            call islsds01(ibuffr(ipiinput), linfmm, linfmg, linfmt,
     +                    lwork, ni, nj, nk, ndim , nvirtual, ndegfd,
     +                    ntimlv, nturb, maxdx, maxdim)
            if ( nerror.ne.0 ) go to 1000

c           --- create local arrays

            isinfmm   = 0
            isinfmg   = 0
            isinfmt   = 0
            iswork    = 0

            call isicreat(isinfmm, linfmm, 'infmm' )
            call isicreat(isinfmg, linfmg, 'infmg' )
            call isicreat(isinfmt, linfmt, 'infmt' )

            call isdcreat(iswork, lwork, 'work' )

c           --- get pointers

            call isactive(isiinput(iblock))
            call isactive(isrinput(iblock))
            call isactive(iscoor(iblock))
            call isactive(issolut(iblock))
            call isactive(iscoefs(iblock))
            if ( iseddy(iblock).gt.0 ) call isactive(iseddy(iblock))

            call isactive(isinfbnd(iblock))
            call isactive(ismatrix(iblock,iequa))
            call isactive(isbndcon(iblock))

            call isactive(isgmat(iblock))
            call isactive(isrhside(iblock))
            call isactive(isgeom(iblock))

            ipiinput = isingetp(isiinput(iblock))
            iprinput = isdpgetp(isrinput(iblock))
            ipcoor   = isdpgetp(iscoor(iblock)  )
            ipsolut  = isdpgetp(issolut(iblock) )
            ipcoefs  = isdpgetp(iscoefs(iblock) )
            if ( iseddy(iblock).gt.0 ) then
               ipeddy = isdpgetp(iseddy(iblock)  )
            else
               ipeddy = 1
            end if

            ipinfbnd = isingetp(isinfbnd(iblock))
            ipmatrix = isdpgetp(ismatrix(iblock,iequa))
            ipbndcon = isdpgetp(isbndcon(iblock))

            ipgmat   = isdpgetp(isgmat(iblock)  )
            iprhside = isdpgetp(isrhside(iblock))
            ipgeom   = isdpgetp(isgeom(iblock)  )

c           --- get pointers local arrays

            ipinfmm  = isingetp( isinfmm )
            ipinfmg  = isingetp( isinfmg )
            ipinfmt  = isingetp( isinfmt )

            ipwork   = isdpgetp( iswork )
            if ( nerror.ne.0 ) go to 1000

c           --- fill integer information about the matrices

            ipiindsc    = ibuffr(ipiinput-1+6)
            ipiincom = ibuffr(ipiinput-1+9)
            if ( ibuffr(ipiinput-1+ipiindsc+1).eq.2 .or.
     +           ibuffr(ipiinput-1+ipiindsc+1).eq.4 ) then
               itypsten = 2
            else if ( ibuffr(ipiinput-1+ipiincom).eq.1 .and.
     +                ibuffr(ipiinput-1+ipiindsc-1+3).eq.0 ) then
               itypsten = 3
            else
               itypsten = 1
            end if
            call isfilm ( ibuffr(ipinfmm), ibuffr(ipinfmg),
     +                    ibuffr(ipinfmt), ibuffr(ipiinput), itypsten )
            if ( nerror.ne.0 ) go to 1000

c           --- fill boundary values

            if ( iequa.ne.2 )
     +         call isfilb ( iequa, ibuffr(ipiinput), buffer(iprinput),
     +                       ibuffr(ipinfbnd),buffer(ipbndcon),
     +                       buffer(ipcoor), buffer(ipsolut),
     +                       buffer(ipcoefs), buffer(ipeddy),
     +                       buffer(ipgeom), ndim, 2, buffer(ipwork) )
            if ( nerror.ne.0 ) go to 1000

c           --- create system of equations
c               Step 1: discretize inner region and external boundary
c               conditions. Also discretize across internal boundaries

            if ( iequa.lt.iseqpres ) then

c           --- momentum equations

               itrans = 1
               itimlv = 2

               call isexmom ( ibuffr(ipiinput), buffer(iprinput),
     +                        buffer(ipmatrix), buffer(ipgmat),
     +                        buffer(iprhside), buffer(ipcoefs),
     +                        buffer(ipsolut), buffer(ipcoor),
     +                        ibuffr(ipinfmm), ibuffr(ipinfmg),
     +                        buffer(ipgeom), ibuffr(ipinfbnd),
     +                        buffer(ipbndcon), buffer(ipeddy),
     +                        buffer(ipwork),
     +                        iblock, itrans, itimlv, nblocks,
     +                        isiinput(iblock), issolut(iblock),
     +                        isitopol )

               matname = 'Momentum'

            else

               call iserrsub ( 26, 0, 0, 0 )
               go to 1000

            end if
            if ( nerror.ne.0 ) go to 1000

c           --- delete local arrays

            call isdelarr(isinfmm)
            call isdelarr(isinfmg)
            call isdelarr(isinfmt)
            call isdelarr(iswork)

            call isdcrpri
            if ( nerror.ne.0 ) go to 1000

         end if

         if ( itime.eq.1 ) then

c        --- report timing results for the construction of the solution

            text = 'for computing the '
            write(text(18:27),200) matname
200         format(a)
            text(28:40) = ' solution is '
            call isaskclk ( 1, text )

         end if

      end do

1000  call iserclos('isexpli')

      end
