      subroutine isfilb3(iinbc   , rinbc   , nside  , it1lo  , it1hi ,
     +                   it2lo   , it2hi   , ibinfor, bndcon , coor  ,
     +                   maxb    , ni      , nj     , nk     , ndim  ,
     +                   nturb   , ncoefs , ndegfd , iequa ,
     +                   nvirtual, time    , iprinbc, coefs  , eddy  ,
     +                   solut   , mturb   , viscorr, rintur , rincof,
     +                   itimeact, iseqpres )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.0    date   26-11-1997 Guus: remove ntrnsp
c                                         New definition of sequence
c        version 1.9    date   19-11-1997 Kees K.: btype=98:do nothing
c        version 1.8    date   10-09-1997 Charles extra parameter iseqpres
c        version 1.7    date   04-06-1996 Marcel extension parameter list
c        version 1.6    date   24-05-1996 Marcel extended k-eps model
c        version 1.5    date   10-04-1996 Kees K space for press.b.c.
c                                         and unreferenced removed
c        version 1.4    date   27-03-1996 Kees K itimeact added
c        version 1.3    date   21-02-1996 Marcel: call isomegb3
c        version 1.2    date   15-12-1995 Marcel: modified iswllfn3
c        version 1.1    date   15-12-1995 Kees K: main loop bounds changed
c        version 1.0    date   12-12-1995
c
c
c   copyright (c) 1995-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill arrays infbnd and bndcon from arrays iinbc and rinbc
c
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxb, ni, nj, nk, ndim, nvirtual, nturb,
     +        ncoefs, ndegfd, viscorr, mturb,
     +        iequa, iinbc(*), nside, it1lo, it1hi, it2lo, it2hi,
     +        ibinfor(it1lo:it1hi,it2lo:it2hi), iprinbc,
     +        itimeact, iseqpres
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:1+nk+nvirtual,1:ndim),
     +                 coefs(*), eddy(*), solut(*),
     +                 bndcon(1:maxb,it1lo:it1hi,it2lo:it2hi),
     +                 rinbc(*), time,  rintur(*), rincof(*)

c     bndcon    o    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     ibinfor   o    Third part of infbnd
c     iequa     i    General equation sequence number
c     iinbc     i    reserved
c     iprinbc   i    Starting address of array rinbc in array iinput
c     iseqpres  i    Sequence number of pressure equation
c     it1hi     i    Upper bound on first tangential index for destination
c     it1lo     i    Lower bound on first tangential index for destination
c     it2hi     i    Upper bound on second tangential index for destination
c     it2lo     i    Lower bound on second tangential index for destination
c     itimeact  i    Actual time step
c     maxb      i    Maximum number of real values for one boundary condition
c     mturb     i    Indicates type of turbulence modelling
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nside     i    Number of boundary for which we need the
c                    information
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rinbc     i    reserved
c     rincof    i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     time      i    Actual time of time step
c     viscorr   i    Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ntdir(1:3), ntsig(1:3), varea(1:2,1:3), carea(1:2,1:3),
     +        ndir, t1dir, t2dir, i, j, k, idim, ndegfdlc,
     +        nseg, ipstart, i1, i2, j1, j2, btype, bctyp(1:3),
     +        xp(1:3), mxp(1:3), nsig, ieq
      double precision coorx, coory, coorz, bcval(1:3), usfunb

c     bctyp     A given boundary type
c     bcval     A given boundary condition value
c     btype     Type of boundary condition
c     carea     Index bounds on set of grid cells adjacent to
c               this boundary
c     coorx     X-coordinaat
c     coory     Y-coordinaat
c     coorz     Z-coordinaat
c     i         General loop variable
c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     idim      Loop variable used for dimension
c     ieq       Equation number
c     ipstart   Starting address
c     j         General loop variable
c     j1        Lower index number along second direction of a face
c     j2        Upper index number along second direction of a face
c     k         General loop variable
c     mxp       Auxiliary array to calculate boundary point number
c     ndegfdlc  Local number of degrees of freedom
c     ndir      Direction of normal vector on this side
c     nseg      Number of segments the boundary is divided in
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     ntdir     Contains the ksi-alpha indices of the normal and
c               tangential directions
c               ntdir(1) refers to the normal direction
c               ntdir(2) refers to the first tangential direction
c               ntdir(3) refers to the second tangential direction
c     ntsig     Gives the direction of the normal or tangential vectors
c               with respect to the ksi co-ordinate axis
c               The normal is supposed to be the outward normal and
c               the tangential directions are taken into a counter
c               clockwise way
c               ntsig(1) refers to the normal direction
c               ntsig(2) refers to the first tangential direction
c               ntsig(3) refers to the second tangential direction
c     t1dir     Direction of first tangential vector on this side
c     t2dir     Direction of second tangential vector on this side
c     varea     Area of arrays
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBNBVI   Gives information on the relation between the ISNaS
c               basis vectors and the normal and tangential vectors on
c               a given boundary. Moreover it gives lower and upper bou
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c     ISOMEGB3  Fill boundary values for omega in wall adjacent
c               cells
c     ISWLLFN3  Fill boundary values for momentum  and coefs in
c               wall adjacent cells for turbulence equations
c               using wall functions in R3
c     USFUNB    user written subroutine
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     5    Three-dimensional version not yet available
c     1202 Only combination of k-eps type models and wall functions is
c          allowed
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen( 'isfilb3' )

      call isbnbvi(ndim,ni,nj,nk,nside,ntdir,ntsig,carea,varea)

      ndir  = ntdir(1)
      t1dir = ntdir(2)
      t2dir = ntdir(3)
      nsig  = ntsig(1)

c     --- Initialize the arrays

      if ( itimeact.eq.1 ) then
         do j = it2lo, it2hi
           do i = it1lo, it1hi
             ibinfor(i,j) = 0
             do idim = 1, ndim
               bndcon(idim,i,j) = 0d0
             end do
           end do
         end do
      end if

c     --- Loop over segments on this side

      nseg = iinbc( iinbc(nside) )
      if ( iseqpres.eq.2 ) then

c     --- Coupled approach

         ndegfdlc = ndegfd-2

      else

c     --- Decoupled approach

         ndegfdlc = ndegfd

      end if

      do k = 1, nseg

        ipstart = iinbc(nside) + (k-1) *(4 +ndegfdlc *4)

c       --- Get cell index bounds and change to vertex bounds

        i1 = iinbc(ipstart+1)
        j1 = iinbc(ipstart+2)
        i2 = iinbc(ipstart+1+2)+1
        j2 = iinbc(ipstart+2+2)+1

        btype = iinbc( ipstart +(iequa-1)*4 +5 )
        do idim = 1, ndim
          bctyp(idim) = iinbc( ipstart +(iequa-1)*4 +5 +idim)
          if (bctyp(idim).eq.-1) then
            bcval(idim) = rinbc( iprinbc +(iequa-1)*3 +idim)
          end if
        end do

        if ( btype.eq.34 ) ieq = iequa

        do j = 2*j1-1, 2*j2-1
          do i = 2*i1-1, 2*i2-1

            ibinfor(i,j) = btype

            if ( btype. eq. 3 .or. btype.eq.9 .or. btype.eq.34 ) then

c           --- If wall functions, then fill bndcon for momentum
c               and dissipation rate equations
c               Check if only k-eps type models are used

              if ( mturb.ne.1 .and. mturb.ne.2 .and. mturb.ne.5 ) then
                call iserrsub ( 1202, 0, 0, 0 )
              end if

              call iswllfn3(iequa, nside, i, j, bndcon, coefs, coor,
     +                      eddy, solut, rintur, ndim, ndir, t1dir,
     +                      t2dir, nsig, ni, nj, nk, ndegfd,
     +                      nvirtual, nturb, ncoefs, maxb,
     +                      rinbc(iprinbc+1),
     +                      it1lo, it2lo, it1hi, it2hi, ieq, rincof,
     +                      bcval(1) )

              if ( btype.eq.34 ) ibinfor(i,j) = 2

            else if ( btype.eq.12 ) then

c           --- At this moment, no parabolic inflow for 3D

              call iserrsub ( 5, 0, 0, 0 )

            else if ( btype.eq.27 ) then

c           --- analytical solution of omega in viscous sublayer

              call isomegb3(nside, i, j, bndcon, coefs, coor, solut,
     +                      rintur, ndir, t1dir, t2dir, ni, nj, nk,
     +                      nvirtual, ncoefs, maxb, eddy, viscorr,
     +                      nsig, it1lo, it2lo, it1hi, it2hi)

              ibinfor(i,j) = 3

           else if ( btype.eq.98 ) then

c          --- do nothing

            else

              do idim = 1, ndim

                if ( bctyp(idim).eq.-1 ) then

                  bndcon(idim,i,j) = bcval(idim)

                else if ( bctyp(idim).gt.0 ) then

                   if ( itimeact.eq.1 ) then

                      xp(ndir) = varea(1,ndir)
                      xp(t1dir) = (i+1)/2
                      xp(t2dir) = (j+1)/2
                      mxp(ndir) = varea(1,ndir)
                      mxp(t1dir) = (i+2)/2
                      mxp(t2dir) = (j+2)/2

                      coorx=( coor(xp(1),xp(2),xp(3),1) +
     +                    coor(mxp(1),mxp(2),mxp(3),1) )/2d0
                      coory=( coor(xp(1),xp(2),xp(3),2) +
     +                    coor(mxp(1),mxp(2),mxp(3),2) )/2d0
                      coorz=( coor(xp(1),xp(2),xp(3),3) +
     +                    coor(mxp(1),mxp(2),mxp(3),3) )/2d0

                      bndcon(idim,i,j) = usfunb(bctyp(idim),
     +                                      coorx,coory,coorz,time)

                   end if

                else if ( bctyp(idim).lt.-100 ) then

                   xp(ndir) = varea(1,ndir)
                   xp(t1dir) = (i+1)/2
                   xp(t2dir) = (j+1)/2
                   mxp(ndir) = varea(1,ndir)
                   mxp(t1dir) = (i+2)/2
                   mxp(t2dir) = (j+2)/2

                   coorx=( coor(xp(1),xp(2),xp(3),1) +
     +                 coor(mxp(1),mxp(2),mxp(3),1) )/2d0
                   coory=( coor(xp(1),xp(2),xp(3),2) +
     +                 coor(mxp(1),mxp(2),mxp(3),2) )/2d0
                   coorz=( coor(xp(1),xp(2),xp(3),3) +
     +                 coor(mxp(1),mxp(2),mxp(3),3) )/2d0

                   bndcon(idim,i,j) = usfunb(abs(100+bctyp(idim)),
     +                                      coorx,coory,coorz,time)

                else

                  bndcon(idim,i,j) = 0d0

                end if

              end do

            end if

          end do
        end do

        iprinbc = iprinbc +ndegfdlc*3

      end do

      call iserclos( 'isfilb3' )

      end
