      subroutine isfilc01 ( iequa, isiinput, isrinput, iscoefs,
     +                      issolut, iscoor, isgeom, iseddy, isitopol,
     +                      iseqpres, isigeom, isaverstr )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.8   date    08-12-1997 Guus: correction ichois
c     version      1.7   date    22-08-1997 Fred T: argument list changed
c     version      1.6   date    11-09-1997 Charles: extra parameter iseqpres
c     version      1.5   date    10-06-1997 Guus: changed lwork
c     version      1.4   date    21-08-1996 Marcel: changed lwork
c     version      1.3   date    22-05-1996 Marcel: changed lwork
c     version      1.2   date    05-03-1996 Marcel: changed lwork
c     version      1.1   date    29-11-1994 argument list changed
c     version      1.0   date    01-11-1994
c
c   copyright (c) 1994-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill coefficients
c
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, isiinput, isrinput, iscoefs, issolut, iscoor,
     +    isgeom, iseddy, isitopol, iseqpres, isigeom, isaverstr

c     iequa     i    General equation sequence number
c     isaverstr i    Array of sequence numbers (nblocks) of array averstr
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     iseqpres  i    Sequence number of pressure equation
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isigeom   i    Array of sequence numbers (nblocks) of array igeom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ichois, ipcoefs, ipcoor, ipeddy, ipgeom, ipiinput,
     +        iprinput, ipsolut, ipwork, iswork, lwork, ipitopol,
     +        maxvpnts, ndegfd, ndim, nturb, ipnsubf, ipsubf,
     +        ipigeom, ipaverstr

c     ichois    General choice parameter
c     ipaverstr Starting address of array averstr in buffer
c     ipcoefs   Starting address of array coefs in buffr
c     ipcoor    Starting address of array coor in buffr
c     ipeddy    Starting address of array eddy in buffr
c     ipgeom    Starting address of array geom in buffr
c     ipigeom   Starting address of array igeom in ibuffr
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     iprinput  Starting address of array rinput in buffr
c     ipsolut   Starting address of array solut in buffr
c     ipsubf    Starting address of the subfaces array
c     ipwork    Starting address of array work in buffr
c     iswork    Sequence number of array work
c     lwork     Length of array work
c     maxvpnts  Maximum number of points per block including virtual ones
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp,isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISDCREAT  Create real array
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILC02  fill geometrical quantities
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isfilc01')

      call isactive( isiinput )
      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      ipiinput = isingetp(isiinput)

      ndegfd     = ibuffr( ipiinput + 55 -1 )
      ndim       = ibuffr( ipiinput + 57 -1 )
      maxvpnts   = ibuffr( ipiinput + 69 -1 )
      nturb      = ibuffr( ipiinput + 70 -1 )
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

      if ( ndim.eq.2 ) then
        lwork    = 1 + maxvpnts * 43
      else
        lwork    = 1 + maxvpnts * 130
      end if
      iswork = 0
      call isdcreat( iswork, lwork, 'work')

      call isactive( isrinput )
      call isactive( iscoefs )
      call isactive( issolut )
      call isactive( iscoor )
      call isactive( isgeom )
      if ( isigeom.gt.0 ) call isactive( isigeom )
      if ( iseddy.gt.0 ) call isactive( iseddy )
      if ( isaverstr.gt.0 ) call isactive( isaverstr )

      ipiinput = isingetp( isiinput )

      iprinput  = isdpgetp( isrinput )
      ipcoefs   = isdpgetp( iscoefs )
      ipsolut   = isdpgetp( issolut )
      ipcoor    = isdpgetp( iscoor )
      ipgeom    = isdpgetp( isgeom )
      if ( isigeom.gt.0 ) then
         ipigeom = isingetp( isigeom )
      else
         ipigeom = 1
      end if
      if ( isaverstr.gt.0 ) then
         ipaverstr = isdpgetp( isaverstr )
      else
         ipaverstr = 1
      end if
      if ( iseddy.gt.0 ) then
         ipeddy = isdpgetp( iseddy )
      else
         ipeddy = 1
      end if
      ipwork   = isdpgetp( iswork )
      if ( nerror.ne.0 ) go to 1000

      if ( iequa.le.iseqpres ) then
         ichois = 1
      else if ( iequa.ge.ndegfd-ndim-nturb+2+iseqpres-2 ) then
         ichois = iequa - ndegfd + ndim - 2 - iseqpres + 2
      else
         ichois = iequa - 1
      end if

      call isfilc02 ( ichois, ibuffr(ipiinput), buffer(iprinput),
     +                buffer(ipcoefs), buffer(ipsolut), buffer(ipcoor),
     +                buffer(ipgeom), buffer(ipeddy),
     +                buffer(ipwork), lwork, ibuffr(ipitopol),
     +                ibuffr(ipnsubf), ibuffr(ipsubf), ibuffr(ipigeom),
     +                buffer(ipaverstr)  )

      call isdelarr( iswork)

1000  call iserclos('isfilc01')

      end
