      subroutine isfilg02 ( ni, nj, nvirtual, npointsv,
     +                      coor, geom, igeom, work, idiscret )
c ======================================================================
c
c     programmer     Ronald van Nooyen
c     version 2.1    date 04-06-1998 Charles: isinfa2d added
c     version 2.0    date 11-06-1996 Guus: complete new subroutine
c     version 1.1    date 01-11-1994 Kees K name changed
c     version 1.0    date 16-08-1994
c
c   copyright (c) 1994-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     computes the square root of the determinant of the metric
c     tensor upto nvirtual-1 cells beyond the boundary for all points
c     Alternative 1
c     First the covariant base-vectors are computed in the
c     points where the determinant must be computed, then
c     from these base-vectors the determinant is computed.
c     R^2 only
c **********************************************************************
c
c                       KEYWORDS
c
c     jacobian
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, npointsv
      integer idiscret, igeom( npointsv,* )
      double precision  coor(*), work(*), geom( npointsv,* )

c     coor       i   reserved
c     geom       o   reserved
c     idiscret   i   Type of discretization
c     igeom      o   reserved
c     ni         i   Number of non-virtual cells in the i-direction (one block)
c     nj         i   Number of non-virtual cells in the j-direction (one block)
c     npointsv   i   Number of points in the mesh with virtual points
c     nvirtual   i   Number of rows of virtual cells
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINFA2D  Give the bilinear metric coefficients for the pressure
c     ISSQRG2D  Compute Jacobian and its inverse in R2
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isfilg02' )

      call iscova2d ( coor, work, ni, nj, nvirtual )

      call issqrg2d ( geom, work, ni, nj, nvirtual )

      if ( idiscret.eq.4 ) then

         call isinfa2d ( geom (1, 9), igeom(1, 1),
     +                   coor, ni, nj, nvirtual )

      end if

      call iserclos ( 'isfilg02' )

      end
