      subroutine isfilm01(ndim,mnconct,gnconct,snconct,mmct,gmct,smct,
     +                    mmctinv, gmctinv, smctinv, mmst, gmst, smst,
     +                    maxdx, ni, nj, nk)
c ======================================================================
c
c     programmer   Ronald van Nooyen
c     version      1.2   date  22-04-1998 Guus: Layout
c     version      1.1   date  16-08-1994 3D adaptation
c     version      1.0   date  15-04-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initialize mct and mctinv
c     Part of Experimental Massively Parallel Solver Test Suite
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer maxdim
      parameter (maxdim = 3)
      integer ndim, mnconct,gnconct,snconct,maxdx
      integer ni, nj, nk
      integer mmct(1:ndim,1:ndim,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer gmct(1:ndim,1:1,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer smct(1:1,1:1,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer mmctinv(0:maxdim,1:ndim,1:mnconct)
      integer gmctinv(0:maxdim,1:ndim,1:gnconct)
      integer smctinv(0:maxdim,1:1,1:snconct)
      integer mmst( 1:mnconct, 1:ndim)
      integer gmst( 1:gnconct, 1:ndim)
      integer smst( 1:snconct, 1:1)

c     gmct      o    Matrix code table gradient matrix see array infmg
c     gmctinv   o    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     gmst      o    Matrix shift table gradient matrix see array infmg
c     gnconct   o    Number of connections in one row of the grad. matrix.
c     maxdim         Highest physical space dimension for which ISNaS can work
c     maxdx     i    Maximum relative shift between cell indices
c     mmct      o    Matrix code table transport equation see array infmt
c     mmctinv   o    Matrix code table inverse transport equation see
c                    array infmt
c     mmst      o    Matrix shift table transport equation see array infmt
c     mnconct   i    Number of connections in one row of the momentum matrix.
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     smct      o    Column index+row degree of freedom to connection
c     smctinv   o    Matrix code table inverse for the transport
c                    equation see array infmt
c     smst      o    Matrix shift table transport matrix see array infmt
c     snconct   o    Number of connections in one row of the transport matrix.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer di, dj, dk, xcode, alphat, betat

c     alphat    Variable or cell indices
c                     1...ndim: flux variable or flux matrix
c                     ndim+1:   pressure variable or
c                               divgrad/transport matrix
c     betat     Variable or cell indices
c                     1...ndim: flux variable or flux matrix
c                     ndim+1:   pressure variable or
c                               divgrad/transport matrix
c     di        Shift in stencil in i-direction
c     dj        Shift in stencil in j-direction
c     dk        Shift in stencil in k-direction
c     xcode     Diagonal number
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISFILM02  Makes the table of matrix column numbers in R2
c     ISFILM03  Makes the table of matrix column numbers in R3
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if (ndim.eq.2) call isfilm02(mmct,gmct,smct,mnconct,ndim,maxdx)
      if (ndim.eq.3) call isfilm03(mmct, gmct, smct, mnconct, maxdx)
      do xcode = 1,mnconct
         do alphat = 1,ndim
            mmctinv(0,alphat,xcode) = 0
            mmctinv(1,alphat,xcode) = 0
            mmctinv(2,alphat,xcode) = 0
            mmctinv(3,alphat,xcode) = 0
         end do
      end do
      do xcode = 1,gnconct
         do alphat = 1,ndim
            gmctinv(0,alphat,xcode) = 0
            gmctinv(1,alphat,xcode) = 0
            gmctinv(2,alphat,xcode) = 0
            gmctinv(3,alphat,xcode) = 0
         end do
      end do
      do xcode = 1,snconct
         do alphat = 1,1
            smctinv(0,alphat,xcode) = 0
            smctinv(1,alphat,xcode) = 0
            smctinv(2,alphat,xcode) = 0
            smctinv(3,alphat,xcode) = 0
         end do
      end do
      do dk = -maxdx,maxdx
         do dj = -maxdx,maxdx
            do di = -maxdx,maxdx
               do betat = 1,ndim
                  do alphat = 1,ndim
                     xcode = mmct(alphat,betat,di,dj,dk)
                     if (xcode.ge.1 .and. xcode.le.mnconct) then
                        mmctinv(0,alphat,xcode) = betat
                        mmctinv(1,alphat,xcode) = di
                        mmctinv(2,alphat,xcode) = dj
                        mmctinv(3,alphat,xcode) = dk

                        mmst(xcode, alphat) =
     +                       di+dj*(ni+1)+dk*(ni+1)*(nj+1)+
     +                       (betat-alphat)*(ni+1)*(nj+1)*(nk+1)
                     end if
                  end do
               end do
               do betat = 1,1
                  do alphat = 1,ndim
                     xcode = gmct(alphat,betat,di,dj,dk)
                     if (xcode.ge.1 .and. xcode.le.gnconct) then
                        gmctinv(0,alphat,xcode) = 1
                        gmctinv(1,alphat,xcode) = di
                        gmctinv(2,alphat,xcode) = dj
                        gmctinv(3,alphat,xcode) = dk

                        gmst(xcode, alphat) =
     +                       di+dj*(ni+1)+dk*(ni+1)*(nj+1)
                     end if
                  end do
               end do
               do betat = 1,1
                  do alphat = 1,1
                     xcode = smct(alphat,betat,di,dj,dk)
                     if (xcode.ge.1 .and. xcode.le.snconct) then
                        smctinv(0,1,xcode) = 1
                        smctinv(1,1,xcode) = di
                        smctinv(2,1,xcode) = dj
                        smctinv(3,1,xcode) = dk

                        smst(xcode, alphat) =
     +                       di+dj*(ni+1)+dk*(ni+1)*(nj+1)
                     end if
                  end do
               end do
            end do
         end do
      end do
      end
