      subroutine isincd3 ( iincnd, rincnd, solut, coor, ndegfd,
     +                     nvirtual, ni, nj, nk )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.0    date 19-02-1999 Guus: new parameter list
c        version 1.1    date 01-03-1995 Kees K: copy timelevels
c        version 1.0    date 01-08-1994
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill array solut from arrays iincnd and rincnd (3D)
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ndegfd, nvirtual,
     +        iincnd(1:ndegfd)
      double precision rincnd(1:ndegfd),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,*),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ndegfd)

c     coor       i    reserved
c     iincnd     i    reserved
c     ndegfd     i    Number of degrees of freedom
c     ni         i    Number of non-virtual cells in the i-direction (one block)
c     nj         i    Number of non-virtual cells in the j-direction (one block)
c     nk         i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual   i    Number of rows of virtual cells
c     rincnd     i    reserved
c     solut     i/o   reserved
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  idegfd, frsti, frstj, frstk, lasti, lastj, lastk,
     +         ip, jp, kp, j, choice, p, q
      double precision coord(1:3), acov(1:3,1:3), u(1:3)

c     acov         Covariant base vectors.
c                  Meaning: a(i,j) gives the value of the i-th
c                           component of the j-th base vector
c     choice       General choice parameter
c     coord        reserved
c     frsti        First index in i-direction
c     frstj        First index in j-direction
c     frstk        First index in k-direction
c     idegfd       Loop variable with respect to a given unknown
c     ip           Local counting variable i-direction
c     j            General loop variable
c     jp           Local counting variable j-direction
c     kp           Local counting variable k-direction
c     lasti        Last index in i-direction
c     lastj        Last index in j-direction
c     lastk        Last index in k-direction
c     p            Index in covariant base vectors
c     q            Index in covariant base vectors
c     u            Cartesian velocities
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision usfuni

c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRSUB  Error messages
c     USFUNI    user written subroutine to define the initial
c               condition as a function of space
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     8      Reading from backing storage not yet available
c     9      Reading from restart file not yet available
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For all solution vectors do
c
c       reset a given solution vector to zero
c
c       If solution = velocity then
c
c         compute the contravariant velocity components as follows:
c
c            p            p            (p)
c           V  = sqrt(g) U  = sqrt(g) a   . u
c                                     -     -
c
c         whereas
c
c                    (p)
c           sqrt(g) a     = a   x  a
c                   -       -(q)   -(r)
c
c         Hence, first compute the covariant base vectors, Cartesian
c         velocities from array rincnd or function USFUNI and then
c         the contravariant fluxes.
c
c       else if solution = scalar then
c
c         compute scalar from array rincnd or function USFUNI
c
c       end if
c
c       copy timelevel 1 to the other timelevels
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isincd3')

c     --- Determine all solution vectors

      do idegfd = 1, ndegfd

c     --- First reset a solution vector to zero

         frsti = 1-nvirtual
         frstj = 1-nvirtual
         frstk = 1-nvirtual
         lasti = ni + 1 + nvirtual
         lastj = nj + 1 + nvirtual
         lastk = nk + 1 + nvirtual

         do kp = frstk, lastk
            do jp = frstj, lastj
               do ip = frsti, lasti
                  solut( ip, jp, kp, idegfd ) = 0d0
               end do
            end do
         end do

         if ( idegfd.le.3 ) then

            if ( idegfd.eq.1 ) then
               lasti = ni + 1
               lastj = nj
               lastk = nk
            else if ( idegfd.eq.2 ) then
               lasti = ni
               lastj = nj + 1
               lastk = nk
            else if ( idegfd.eq.3 ) then
               lasti = ni
               lastj = nj
               lastk = nk + 1
            end if

c           --- Compute covariant base vectors, coordinates needed for
c               computation of Cartesian velocities with function USFUNI
c               and then finally compute contravariant fluxes.

            do kp = 1, lastk
               do jp = 1, lastj
                  do ip = 1, lasti

                     do j = 1, 3

                        if ( idegfd.eq.1 ) then

                           coord(j) = 25d-2*( coor(ip,jp,kp,j) +
     +                                        coor(ip,jp+1,kp,j) +
     +                                        coor(ip,jp,kp+1,j) +
     +                                        coor(ip,jp+1,kp+1,j) )

                           acov(j,2) = 5d-1*(  coor(ip,jp+1,kp,j)
     +                                    + coor(ip,jp+1,kp+1,j)
     +                                    - coor(ip,jp,kp,j)
     +                                    - coor(ip,jp,kp+1,j) )
                           acov(j,3) = 5d-1*(  coor(ip,jp,kp+1,j)
     +                                    + coor(ip,jp+1,kp+1,j)
     +                                    - coor(ip,jp,kp,j)
     +                                    - coor(ip,jp+1,kp,j) )

                        else if ( idegfd.eq.2 ) then

                           coord(j) = 25d-2*( coor(ip,jp,kp,j) +
     +                                        coor(ip+1,jp,kp,j) +
     +                                        coor(ip,jp,kp+1,j) +
     +                                        coor(ip+1,jp,kp+1,j) )

                           acov(j,1) = 5d-1*(  coor(ip+1,jp,kp,j)
     +                                    + coor(ip+1,jp,kp+1,j)
     +                                    - coor(ip,jp,kp,j)
     +                                    - coor(ip,jp,kp+1,j) )
                           acov(j,3) = 5d-1*(  coor(ip,jp,kp+1,j)
     +                                    + coor(ip+1,jp,kp+1,j)
     +                                    - coor(ip,jp,kp,j)
     +                                    - coor(ip+1,jp,kp,j) )

                        else if ( idegfd.eq.3 ) then

                           coord(j) = 25d-2*( coor(ip,jp,kp,j) +
     +                                        coor(ip+1,jp,kp,j) +
     +                                        coor(ip,jp+1,kp,j) +
     +                                        coor(ip+1,jp+1,kp,j) )

                           acov(j,1) = 5d-1*(  coor(ip+1,jp,kp,j)
     +                                    + coor(ip+1,jp+1,kp,j)
     +                                    - coor(ip,jp,kp,j)
     +                                    - coor(ip,jp+1,kp,j) )
                           acov(j,2) = 5d-1*(  coor(ip,jp+1,kp,j)
     +                                    + coor(ip+1,jp+1,kp,j)
     +                                    - coor(ip,jp,kp,j)
     +                                    - coor(ip+1,jp,kp,j) )

                        end if

                     end do

                     do j = 1, 3
                        u(j) = 0d0

                        if ( iincnd(j).eq.-1 ) then

                           u(j) = rincnd(j)

                        else if ( iincnd(j).lt.-100 ) then

                           choice = abs( iincnd(j)+100 )
                           u(j) = usfuni(choice,coord(1),coord(2),
     +                                   coord(3))

                        else if ( iincnd(j).gt.100 ) then

c                       --- Read solution vector idegfd from backing storage
c                               NOT YET AVAILABLE ***

                           call iserrchr('isincd3',1 )
                           call iserrsub ( 8, 0, 0, 1 )

                           go to 1000

                        else if ( iincnd(j).gt.0 .and.
     +                            iincnd(j).lt.100 ) then

c                       --- Read complete solution vector from the restart file
c                               NOT YET AVAILABLE ***

                           call iserrchr('isincd3',1 )
                           call iserrsub ( 9, 0, 0, 1 )

                           go to 1000

                        end if
                     end do

                     p = mod( idegfd+1,3 )
                     q = mod( idegfd+2,3 )
                     if ( p.eq.0 ) p = 3
                     if ( q.eq.0 ) q = 3

                     solut(ip,jp,kp,idegfd) = ( acov(2,p)*acov(3,q) -
     +                                   acov(3,p)*acov(2,q) )*u(1)
     +                                  + ( acov(3,p)*acov(1,q) -
     +                                   acov(1,p)*acov(3,q) )*u(2)
     +                                  + ( acov(1,p)*acov(2,q) -
     +                                   acov(2,p)*acov(1,q) )*u(3)

                  end do

               end do

            end do

         else

            lasti = ni
            lastj = nj
            lastk = nk

            if ( iincnd(idegfd).eq.-1 ) then

c           --- Fill solution vector idegfd with a constant

               do kp = 1, lastk
                  do jp = 1, lastj
                     do ip = 1, lasti
                        solut(ip,jp,kp,idegfd) = rincnd(idegfd)
                     end do
                  end do
               end do

            else if ( iincnd(idegfd).lt.-100 ) then

c           --- Fill solution vector idegfd with function USFUNI

               choice = abs( iincnd(idegfd) + 100 )

               do kp = 1, lastk
                  do jp = 1, lastj
                     do ip = 1, lasti

                        do j = 1, 3

                           coord(j) = 125d-3* ( coor(ip,jp,kp,j) +
     +                                          coor(ip,jp+1,kp,j) +
     +                                          coor(ip+1,jp,kp,j) +
     +                                          coor(ip+1,jp+1,kp,j) +
     +                                          coor(ip,jp,kp+1,j) +
     +                                          coor(ip,jp+1,kp+1,j) +
     +                                          coor(ip+1,jp,kp+1,j) +
     +                                          coor(ip+1,jp+1,kp+1,j) )

                        end do

                        solut(ip,jp,kp,idegfd) =
     +                     usfuni( choice,coord(1),coord(2),coord(3) )

                     end do

                  end do

               end do

            else if ( iincnd(j).gt.100 ) then

c           --- Read solution vector idegfd from backing storage
c                   NOT YET AVAILABLE ***

               call iserrchr('isincd3',1 )
               call iserrsub ( 8, 0, 0, 1 )

               go to 1000

            else if ( iincnd(j).gt.0 .and. iincnd(j).lt.100 ) then

c           --- Read complete solution vector from the restart file
c                   NOT YET AVAILABLE ***

               call iserrchr('isincd3',1 )
               call iserrsub ( 9, 0, 0, 1 )

               go to 1000

            end if

         end if

      end do

1000  call iserclos('isincd3')

      end
