      subroutine isinidbg ( nbuffr )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.1    date   18-09-95 (Layout <= 5 allowed)
c        version 1.0    date   17-02-95 (Subroutine extracted from the SEPRAN
c                                        subroutine inidbg)
c
c
c
c   copyright (c) 1995-1995  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   Check if the file isnas.dbg is available
c   If the file exists, read it and initialise debug facilities
c ********************************************************************
c
c                      KEYWORDS
c
c   debug
c   initializations
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer nbuffr

c     nbuffr   i/o   Length of buffer array ibuffr
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cdebug'
      include 'IScommon/cisnas'
      include 'IScommon/cseprd'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer nncode, nout, nstrng
      parameter (nncode=5, nout=1, nstrng=1)
      integer irefsv, ioutsv, ios, i, ncode(nncode),
     +        itype(nncode), ncodmx(nncode), nmain, iretur, nmcode(1)
      character *14 code(nncode)
      character *1 codemn(1), strout(nstrng)
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)
      logical check

c     check     Indication if a file has been opened (true) or not (false)
c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     codemn    Table of main keywords
c     i         General loop variable
c     ios       Iostatus variable
c     ioutsv    Help variable to store the input value of ioutpt
c     irefsv    Help variable to save the value of a reference number
c     iretur    Return code, indicates the sequence number of the new
c               main keyword found
c               If -1 an error has occurred
c               If -2 the end of file has been reached
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     nmain     Number of keywords in array codemn
c     nmcode    Array containing the number of significant characters
c               for each main keyword in codemn
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding to
c               the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c ********************************************************************
c
c                    I/O
c
c    This subroutine reads the file isnas.dbg if available
c    The following statements of this file are recognized:
c    debug_level = i
c    int_infin   = j
c    nbuffr      = k
c    print_layout = i
c    print_sequence = i
c
c    meaning of the parameters:
c
c    debug_level = i
c       Defines the type of debug level to be used with respect to the
c       memory management
c       Possible values:
c       0:  No memory management debugging
c       1:  All arrays declared in the buffer are surrounded by control
c           integers. These control integers are checked in each call to a
c           memory management subroutine
c       2:  See 1. In this case also the buffer array is initialized at
c           integer infinity. When an array is deleted the corresponding
c           part of the buffer is again set to integer infinity
c       Default value: 0
c    int_infin   = i
c       Defines the value of integer infinity
c       Default: 2**31-1
c    nbuffr      = i
c       Defines the value of nbuffr for debugging purposes.
c       This value may not exceed the actual value of nbuffr defined in the
c       program.
c       A smaller value of nbuffr may decrease the part that is set to infinity
c       and also allows the testing of memory management
c       Default: nbuffr from program
c    print_layout = i
c       See common cdebug: iprlayout
c       Default: 1
c    print_sequence = i
c       See common cdebug: iprseq
c       Default: 1
c ********************************************************************
c
c                    ERROR MESSAGES
c
c     706   error in opening file
c ********************************************************************
c
c                    PSEUDO CODE
c
c   Set all parameters to their default values
c   Check if the local file isnas.dbg exists
c   If ( exist .dbg ) then
c      read the file and set parameters
c   if ( iinfin = 0 and idebug > 1 ) then
c      iinfin = 2**31-1
c ********************************************************************
c
c                    DATA STATEMENTS
c
      data code / 'DEBUG_LEVEL', 'INT_INFIN', 'NBUFFR', 'PRINT_LAYOUT',
     +            'PRINT_SEQUENCE' /
      data ncode / 5, 3, 4, 9, 9 /,
     +     ncodmx / 11, 9, 6, 12, 14 /,
     +     itype  /  0, 0, 0,  0,  0 /,
     +     tabmin / 0d0,  0d0, 1d3, 1d0, 1d0 /,
     +     tabmax / 2d0, 1d10, 0d0, 2d0, 5d0 /

c     code    Table with keywords
c             Available keywords:
c             1:   DEBUG_LEVEL
c             2:   INT_INFIN
c             3:   NBUFFR
c             4:   PRINT_LAYOUT
c             5:   PRINT_SEQUENCE
c ======================================================================
c
      call iseropen ( 'isinidbg' )

c     --- Set parameters in common block

      idebug = 0
      iinfin = 0
      nbufls = 0
      nbufw1 = 0
      nbufw2 = 0
      nbufw3 = 0
      ifirws = 0
      narbuf = 0
      iprlayout = 1
      iprseq = 1
      ioutsv = ioutpt
      irefsv = irefre

c     --- check if the local file isnas.dbg exists

      inquire ( file='isnas.dbg', exist=check )

      if ( check ) then

c     --- File exists, read information until end of file has been found

         irefre = 55
         open ( unit=irefre, file='isnas.dbg', iostat=ios )
         if ( ios.ne.0 ) then
            call iserrchr ( 'isnas.dbg', 1 )
            call iserrint ( ios, 1)
            call iserrsub ( 706, 1, 0, 1 )
         end if

         ityprd = 1
         istop  = 2
         ioutpt = -1
         do 100 i = 1, nncode
            table(i) = -1d0
100      continue
         table(1) = 0d0
         table(4) = 1d0
         table(5) = 1d0
         tabmax(3) = nbuffr
         nmain = 0

c        --- Read isnas.dbg

         call isintx02 ( code, ncode, nncode, itype, table, codemn,
     +                   nmcode, nmain, iretur, tabmin, tabmax,
     +                   tabout, nout, strout, nstrng, ncodmx )
         idebug = nint(table(1))
         iprlayout = nint(table(4))
         iprseq = nint(table(5))
         if ( table(2).gt.0d0 ) iinfin = nint(table(2))
         if ( table(3).gt.0d0 ) nbuffr = nint(table(3))

      end if

      if ( idebug.le.1 .or. (idebug.gt.1 .and. iinfin.eq.0) ) then

c        ---set iinfin

         iinfin = 2 ** 15
         if ( machin.eq.7 ) then
            iinfin = 2 ** 23
         else
            iinfin = iinfin ** 2
         end if
         iinfin = 2*(iinfin-1)+1

      end if

      istop  = 1
      ioutpt = ioutsv
      irefre = irefsv
      ityprd = 1
      nbufmx = nbuffr

      call iserclos ( 'isinidbg' )
      end
