      subroutine islinsol ( iequa, iblock, nblocks, neq,
     +                      isiinput, isrinput, iscoor, issolut,
     +                      ismatrix, isgmat, isrhside, isitopol,
     +                      isgeom, isdensup)
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.44  date  10-12-1999 Duncan: new call to isgsckrs
c                                                 new call to isgsup
c                                                 new call to isgsdpsm
c     version      1.43  date  21-09-1999 Guus: extend lensmsg
c     version      1.42  date  11-05-1999 Duncan: new call to isgsckrs
c                                                 new call to isgsup
c                                                 new call to isgaus
c     version      1.41  date  17-11-1998 Duncan: new call to isgsckrs
c                                                 new call to isgsup
c     version      1.40  date  24-08-1998 Kees K: work array added in
c                                         calls of isuprhs and isrestore
c     version      1.39  date  14-05-1998 Duncan: extension for saving
c                                         coefficients dens upwind
c
c     version      1.38  date  07-07-1998 Kees V: nusol, nsolut cleaned
c     version      1.37  date  18-05-1998 Charles Bilinear discretization
c     version      1.36  date  12-04-1998 Duncan:fixed bug in matrix
c                                         update; added printing routines
c                                         for non-linear Gauss-Seidel
c     version      1.35  date  01-04-1998 Duncan:accelerated non-linear
c                                         Gauss-Seidel for pressure
c                                         equation added.
c     version      1.34  date  22-12-1997 Kees V. automatic
c                                         determination of lmatvir
c     version      1.33  date  09-12-1997 Kees V. efficient memory use
c                                         for the virtual matrix elements
c     version      1.32  date  25-11-1997 Kees V. matrix copy for ILUD
c                                         multiblock is removed
c     version      1.31  date  20-10-1997 Kees K. routine  isfilimv
c
c   copyright (c) 1993-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     single domain solver ( one block ). Solve the equations.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     single_domain_solver
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa   , iblock , nblocks, neq
      integer isiinput, isrinput, iscoor, issolut, isgmat, isrhside,
     +        isitopol, isgeom, isdensup
      integer ismatrix(1:nblocks, 1:neq, 1:2)

c     iblock    i    Block sequence number
c     iequa     i    General equation sequence number
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  i    Array of sequence numbers (nblocks) of array densup
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c     neq       i    Number of equations
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iareanv(1:2,1:3), iareav(1:2,1:3),
     +        iareac(1:2,1:3), ichoicem, idegfd, idegfd1, idegfd2,
     +        ipsolut, ipiinblk, ipiinput, itypsten,
     +        ipinfmg, ipinfmm, ipinfmt, ipmtvc, ipiincom,
     +        ipmatrix, ipprec, iprhside, iprinput,
     +        ipwork, linfmm, linfmg, linfmt  , lmtvc,
     +        lprec, lwork, lrhside , idiscret,
     +        lsolutnv, ipsolutnv, ipmatcop,
     +        iprhscop, ismatcop, isrhscop, ipiindsc, ismtvc,
     +        isinfmm , isinfmg , isinfmt, isprec, issolutnv, iswork,
     +        ni,  nj, nk, ndim, nvirtual, ndegfd, ntimlv ,
     +        maxdim, maxdx, npointsv, npointsnv, offset,
     +        method, iseqpres, mtimlv, itimlv,
     +        saveprec, leniinsol, ipiinsol,
     +        lensmsg, issmsg, ipsmsg, lmatvir, itergs, maxiter,
     +        igauss, lgausitr, isgausitr, ipgausitr, nusol,
     +        ichoiceg, ipgeom, isgeomnv, lgeomnv, ipgeomnv,
     +        ipiincav, mcav, mitergs, isigsinf, isrgsinf,
     +        ipigsinf, iprgsinf, ligsinf, lrgsinf, nints, nreal,
     +        ipdensup, ismatman, lmatman, ipmatman, isolv
      logical matonly, singular, coupled
      double precision residgs, tol, tolbsec

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     iareac    Bounds for copy operation
c     iareanv   Bounds for array without virtual points
c     iareav    Bounds for array with virtual points
c     ichoiceg  Choice parameter update matrix and rhside
c     ichoicem  Choice parameter multi block
c     idegfd    Loop variable with respect to a given unknown
c     idegfd1   Lower bound for a degree of freedom
c     idegfd2   Lower bound for a degree of freedom
c     idiscret  Type of discretization
c     igauss    if igauss=1 non linear Gauss Seidel is applied else not
c     ipdensup  Starting address of array densup in array buffer
c     ipgausitr Starting address of array gausitr in array buffer
c     ipgeom    Starting address of array geom in buffer
c     ipgeomnv  Starting address of array geomnv in array buffer
c     ipigsinf  Starting address of array igsinf in array ibuffr
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiincav  Starting address of array IINCAV in array IINPUT
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     ipinfmg   Starting address of array infmg in ibuffr
c     ipinfmm   Starting address of array infmm in ibuffr
c     ipinfmt   Starting address of array infmt in ibuffr
c     ipmatcop  Starting address of array matcop in buffer
c     ipmatman  Starting address of array matman in buffer
c     ipmatrix  Starting address of array matrix in buffer
c     ipmtvc    Starting address of array mtvc in buffer
c     ipprec    Starting address of array prec in buffer
c     iprgsinf  Starting address of array rgsinf in buffer
c     iprhscop  Starting address of array rhscop in buffer
c     iprhside  Starting address of array rhside in buffer
c     iprinput  Starting address of array rinput in buffer
c     ipsmsg    Starting address of array smsg in buffer
c     ipsolut   Starting address of array solut in buffer
c     ipsolutnv Starting address of array solut w/o virtual points in buffer
c     ipwork    Starting address of array work in buffer
c     iseqpres  Sequence number of pressure equation
c     isgausitr Sequence number of array gausitr
c     isgeomnv  Sequence number of array geomnv
c     isigsinf  Sequence number of array igsinf
c     isinfmg   Sequence number of array infmg
c     isinfmm   Sequence number of array infmm
c     isinfmt   Sequence number of array infmt
c     isitopol  Sequence number of array itopol
c     ismatcop  Sequence number of array matcop
c     ismatman  Sequence number of array matman
c     ismtvc    Sequence number of array mtvc
c     isolv     Called by solver for periodic b.c. isolv=1 else isolv=0
c     isprec    Sequence number of array prec
c     isrgsinf  Sequence number of array rgsinf
c     isrhscop  Sequence number of array rhscop
c     issmsg    Sequence number of array smsg
c     issolutnv Sequence number of array solutnv
c     iswork    Sequence number of array work
c     itergs    Number of the iteration in the mixed non linear Gauss-Seidel/
c               GMRES algorithm
c     itimlv    Loop variable with respect to a given time level
c     itypsten  Indicates the type of stencil
c               Possible values
c               R^2
c               1:  13-point stencil
c               2:  21-point stencil
c               3:  17-point stencil
c               R^3
c               1:  51-point stencil
c               2:  59-point stencil
c     leniinsol Length of first component of array iinsol
c     lensmsg   Length of array smsg
c     lgausitr  Length of array gausitr
c     lgeomnv   Length of array geomnv
c     ligsinf   Length of array igsinf
c     linfmg    Length of array infmg
c     linfmm    Length of array infmm
c     linfmt    Length of array infmt
c     lmatman   Length of the array for matrix manipulation
c     lmatvir   Length of the array of virtual matrix elements
c     lmtvc     Length of array mtvc
c     lprec     Length of array prec
c     lrgsinf   Length of array rgsinf
c     lrhside   Length of array rhside
c     lsolutnv  Length of temporary solution array
c     lwork     Length of array work
c     matonly   If true, only copy matrix elements and leave
c               right-hand side alone.
c     maxdim    Highest physical space dimension for which ISNaS can work
c     maxdx     Maximum relative shift between cell indices
c     maxiter   Maximum number of iterations
c     mcav      If 1 a cavity model is used, if 0 not
c     method    Control parameter indicating the iterative method.
c     mitergs   Maximum number of iterations in non-linear Gauss-Seidel
c     mtimlv    Starting value for copying time levels to work array solutnv
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nints     Number of integers in a message
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsnv Number of points in the mesh without virtual points
c     npointsv  Number of points in the mesh with virtual points
c     nreal     Number of reals in a message
c     ntimlv    Number of time levels
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     nvirtual  Number of rows of virtual cells
c     offset    General offset factor
c     residgs   Infinite norm of the residual in the pressure correction
c               when the non linear Gauss-Seidel algorithm is applied
c     saveprec  Save preconditioner across domain decomposition
c               iterations (=1) or not (=0)
c     singular  If true the pressure matrix is singular since at all
c               boundaries the normal velocity has been prescribed
c     tol       Tolerance for the non-linear Gauss-Seidel algorithm
c     tolbsec   Tolerance in the bisection step
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp, isdpleng

c     ISACTIVE  Activate permanent array
c     ISCORRH2  Correct pressure right-hand side (2D)
c     ISCORRH3  Correct pressure right-hand side (3D)
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISDPLENG  Return with the length of the array in double precisions
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILIMV  Fill array iinmtvc
c     ISFILM    Fill integer information about the matrices
c     ISGAUS    Performs one non-linear Gauss-Seidel step for
c               pressure correction
c     ISGSCKRS  Check the residual of the non-linear Gauss-Seidel algorithm
c     ISGSDPSM  Compute the total pressure correction, the sum of the Gauss-
c               Seidel step and the GMRES step
c     ISGSINT   Initialise the work arrays for non-linear Gauss-Seidel
c     ISGSUP    Update the matrix and the right hand side of pressure equation
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  Get the pointer corresponding to an integer array
c     ISLSDS02  Compute lengths of temporary arrays
c     ISLSOL    Solve system of equations
c     ISMBLK02  ?
c     ISPRINT   ISNaS general print subroutine
c     ISRESTORE Restore matrix and right-hand side
c     ISSCOPY   Copy operation
c     ISSTOP    Stop ISNAS
c     ISUPM2D   Add or remove compressible part of pressure matrix in case
c               of non-linear Gauss-Seidel
c     ISUPRHS   Update the right-hand side with linear combinations of
c               virtual unknowns
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c        create temporary space
c        get pointers
c
c        saveprec:= preconditioner saved (=1) or not (=0)
c        method  := 'subdomain_solution =ilu' (=6) or not (<> 6)
c        mkdin   := type of preconditioner
c
c        fill integer information about the matrices
c
c        if ( preconditioner not saved ) then
c
c           create space for preconditioner
c
c        else
c
c           reuse space for preconditioner
c
c        end if
c        create other arrays
c
c        multi_block_algorithm_active = nblocks > 1
c
c        if ( multi_block_algorithm_active ) then
c
c
c           update the right-hand side with linear combinations
c           of virtual unknowns. Save virtual coefficients of the
c           matrix and boundary elements of rhside. Set virtual
c           coefficients in the matrix to zero.
c
c        end if
c
c
c        copy solut with virtual points to solut without virtual points
c        (only the degrees of freedom that are solved for are copied)
c
c        solve system of equations
c
c        copy the results of solution back into the solut array with
c        virtual points
c
c        if ( multi_block_algorithm_active ) then
c
c           restore virtual coefficients of the matrix and boundary elements
c           of rhside.
c
c        end if
c
c        if ( preconditioner not saved ) delete space for preconditioner
c        delete local arrays
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('islinsol')

c     --- Find start address for array iinsol

      call isactive(isiinput)
      ipiinput = isingetp(isiinput)

      ipiinblk  = ibuffr(ipiinput-1+7)
      leniinsol = ibuffr(ipiinput-1+105)
      ipiinsol  = ibuffr(ipiinput-1+5)+(iequa-1)*leniinsol
      coupled   = ibuffr(ipiinput-1+76).eq.0
      mcav    = ibuffr(ipiinput - 1 + 77)

      if (mcav.eq.1) then
        ipiincav= ibuffr(ipiinput - 1 + 11)
        igauss  = ibuffr(ipiinput+ipiincav - 2 + 1)
      else
        igauss  =  0
      end if

c     --- Fill of saveprec, method, mkind and ichoisem

      saveprec = ibuffr( ipiinput + ipiinsol -2+15)
      method   = ibuffr( ipiinput + ipiinsol -2+7)
      if ( ibuffr( ipiinput + ipiinsol -2+21).eq.1 ) then
         singular = .true.
      else
         singular = .false.
      end if
      ichoicem = ibuffr( ipiinput + ipiinblk -2+1 )

c     --- Only in case of subdomain_solution = ilu with the parallel
c         algorithm, isuprhs and isrestore should do nothing more
c         than copy matrix-elements.

      matonly =  method.eq.6 .and. ichoicem.eq.1

c     --- matonly = .false.

c         --- compute lengths of temporary arrays

      if ( nerror.ne.0 ) go to 1000
      call islsds02(iequa, ibuffr(ipiinput), linfmm, linfmg, linfmt,
     +              lprec, lsolutnv,  lwork,  ni, nj, nk, ndim ,
     +              nvirtual, ndegfd , ntimlv, maxdx, maxdim )
      if ( coupled) then
         iseqpres = 2
      else
         iseqpres = ndim+1
      end if

      if ( nerror.ne.0 ) go to 1000

c     --- create local arrays

      lmtvc  = 20

      isinfmm   = 0
      isinfmg   = 0
      isinfmt   = 0
      ismtvc = 0
      issmsg = 0
      if ( saveprec.eq.0 ) then
         isprec = 0
      else
         isprec = ismatrix ( iblock, iequa, 2 )
      end if
      issolutnv = 0
      iswork    = 0
      isgausitr = 0
      ismatman  = 0
      isigsinf  = 0
      isrgsinf  = 0
      isgeomnv  = 0
      lgausitr  = (1+ni)*(1+nj)*(1+nk)
      lmatman   = 10*(1+ni+2*nvirtual)*(1+nj+2*nvirtual)
      lrgsinf   = 10
      ligsinf   = 10
      lgeomnv   = lgausitr*4

      if ( nk.eq.0 ) then
         lensmsg      = 4*(max(ni,nj)+1+2*nvirtual)
      else
         lensmsg      = 6*(max(ni,nj,nk)+1+2*nvirtual)*
     +                  (max(ni,nj,nk)+1+2*nvirtual)
      end if

c     --- activate and create arrays

      call isdcreat(issmsg, lensmsg, 'smsg' )
      call isicreat(isinfmm, linfmm, 'infmm' )
      call isicreat(isinfmg, linfmg, 'infmg' )
      call isicreat(isinfmt, linfmt, 'infmt' )
      call isicreat(ismtvc, lmtvc, 'iinmtvc' )

      if ( saveprec.eq.0 ) then
         call isdcreat(isprec, lprec, 'prec' )
      else
         call isactive(isprec)
      end if
      call isdcreat(issolutnv, lsolutnv, 'solutnv' )
      call isdcreat(isgausitr,lgausitr, 'gausitr')
      call isdcreat(ismatman,lmatman, 'matman')
      call isicreat(isigsinf,ligsinf, 'igsinf')
      call isdcreat(isrgsinf,lrgsinf, 'rgsinf')
      call isdcreat(isgeomnv, lgeomnv, 'geomnv')
      call isdcreat(iswork, lwork, 'work' )

c     --- get pointers

      call isactive(isiinput)
      call isactive(isrinput)
      call isactive(iscoor)
      call isactive(issolut)
      call isactive(ismatrix(iblock,iequa,1))
      call isactive(isgmat)
      call isactive(isrhside)
      call isactive(isgeom)
      call isactive(isdensup)

      ipiinput = isingetp(isiinput)
      iprinput = isdpgetp(isrinput)
      ipsolut  = isdpgetp(issolut )
      ipmatrix = isdpgetp(ismatrix(iblock,iequa,1))
      iprhside = isdpgetp(isrhside)
      ipgeom   = isdpgetp(isgeom)
      ipdensup = isdpgetp(isdensup)

c     --- get pointers local arrays

      ipinfmm    = isingetp( isinfmm )
      ipinfmg    = isingetp( isinfmg )
      ipinfmt    = isingetp( isinfmt )
      ipmtvc     = isingetp( ismtvc )

      ipsmsg     = isdpgetp( issmsg )
      ipprec     = isdpgetp( isprec )
      ipsolutnv  = isdpgetp( issolutnv )
      ipwork     = isdpgetp( iswork )
      ipgausitr  = isdpgetp( isgausitr )
      ipmatman   = isdpgetp( ismatman )
      ipigsinf   = isingetp( isigsinf )
      iprgsinf   = isdpgetp( isrgsinf )
      ipgeomnv   = isdpgetp( isgeomnv )

c     --- fill integer information about the matrices

      ipiindsc = ibuffr(ipiinput-1+6)
      ipiincom = ibuffr(ipiinput-1+9)
      idiscret = ibuffr(ipiinput-1+ipiindsc+1)
      if ( idiscret.eq.2.or.idiscret.eq.4 ) then
         itypsten = 2
      else if ( ibuffr(ipiinput-1+ipiincom).eq.1 .and.
     +          ibuffr(ipiinput-1+ipiindsc-1+3).eq.0 ) then
         itypsten = 3
      else
         itypsten = 1
      end if
      if ( nerror.ne.0 ) go to 1000
      call isfilm ( ibuffr(ipinfmm), ibuffr(ipinfmg),
     +              ibuffr(ipinfmt), ibuffr(ipiinput), itypsten )
      if ( nerror.ne.0 ) go to 1000

      lrhside = isdpleng(isrhside)
      isrhscop = 0
      call isdcreat(isrhscop, lrhside, 'rhscop' )
      iprhscop   = isdpgetp(isrhscop)

      if ( nblocks.gt.1 .or. method.eq.6 ) then

c     --- multi-block, copy of the virtual matrix elements needed

         lmatvir = ibuffr( ipiinput + ipiinsol -2+1)

         if ( lmatvir.eq.0 ) then

c        --- compute actual length

            ismatcop = ismatrix(iblock,iequa,1)
            ipmatcop = isdpgetp(ismatcop)

            call isuprhs( iequa, iblock,    matonly,
     +                    ibuffr(ipiinput), ibuffr(ipinfmm),
     +                    ibuffr(ipinfmt),  buffer(ipsolut),
     +                    buffer(ipmatrix), buffer(ipmatcop),
     +                    buffer(iprhside), buffer(iprhscop),
     +                    buffer(ipwork), lmatvir)

            ibuffr( ipiinput + ipiinsol -2+1) = lmatvir

         end if

         ismatcop = 0
         call isdcreat(ismatcop, lmatvir, 'matcop' )

      else

c     --- single-block, copy is not needed

         ismatcop = ismatrix(iblock,iequa,1)

      end if

      ipmatcop   = isdpgetp(ismatcop)

c     --- update the right-hand side with linear combinations
c         of virtual unknowns. Save the virtual coefficients of the
c         matrix in matcop and the affected right-hand side positions
c         in rhscop

      if ( nblocks.gt.1 .or. method.eq.6 ) then

         call isuprhs( iequa, iblock,    matonly,
     +                 ibuffr(ipiinput), ibuffr(ipinfmm),
     +                 ibuffr(ipinfmt),  buffer(ipsolut),
     +                 buffer(ipmatrix), buffer(ipmatcop),
     +                 buffer(iprhside), buffer(iprhscop),
     +                 buffer(ipwork), lmatvir)

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- copy array solut to the array solutnv which is identical
c         but without virtual points

      if ( coupled ) then

         if ( iequa.eq.1 ) then

            idegfd1 = 1
            idegfd2 = ndim

         else

            idegfd1 = iequa + ndim - 1
            idegfd2 = idegfd1

         end if

      else

         idegfd1 = iequa
         idegfd2 = idegfd1

      end if

      iareanv(1,1) = 1
      iareanv(2,1) = ni+1
      iareanv(1,2) = 1
      iareanv(2,2) = nj+1
      if (ndim.eq.2) then
         iareanv(1,3) = 1
         iareanv(2,3) = 1
      else
         iareanv(1,3) = 1
         iareanv(2,3) = nk+1
      end if

      iareav(1,1) = 1-nvirtual
      iareav(2,1) = ni+1+nvirtual
      iareav(1,2) = 1-nvirtual
      iareav(2,2) = nj+1+nvirtual

      if (ndim.eq.2) then
         iareav(1,3) = 1
         iareav(2,3) = 1
      else
         iareav(1,3) = 1-nvirtual
         iareav(2,3) = nk+1+nvirtual
      end if

      npointsnv = (iareanv(2,1)-iareanv(1,1)+1)*
     +              (iareanv(2,2)-iareanv(1,2)+1)*
     +              (iareanv(2,3)-iareanv(1,3)+1)
      npointsv = (iareav(2,1)-iareav(1,1)+1)*
     +              (iareav(2,2)-iareav(1,2)+1)*
     +              (iareav(2,3)-iareav(1,3)+1)

c     --- Should work for 2D and 3D

      do idegfd= idegfd1, idegfd2
         if ((igauss.eq.1).and.iequa.eq.iseqpres) then
            mtimlv=1
         else
            mtimlv=ntimlv-1
         end if

         do itimlv= mtimlv,ntimlv
            offset = (itimlv-1)*ndegfd+(idegfd-1)
            call isscopy(
     +                iareav(1,1),iareav(2,1),
     +                iareav(1,2),iareav(2,2),
     +                iareav(1,3),iareav(2,3),
     +                iareanv(1,1),iareanv(2,1),
     +                iareanv(1,2),iareanv(2,2),
     +                iareanv(1,3),iareanv(2,3),
     +                1,iareanv,buffer(ipsolut+offset*npointsv),
     +                buffer(ipsolutnv+offset*npointsnv),1d0)
         end do

         if ( nerror.ne.0 ) go to 1000

      end do

c     --- Correct for pressure right-hand side if neccesary

      if ( singular ) then

c     --- matrix is singular

         if ( ndim.eq.2 ) then

            call iscorrh2 ( buffer(iprhside), ni, nj )

         else

            call iscorrh3 ( buffer(iprhside), ni, nj, nk )

         end if

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- solve system of equations

      if ( coupled ) then
         if ( iequa.eq.1 ) then
            offset = (ntimlv-1)*ndegfd
         else
            offset = (ntimlv-1)*ndegfd + ndim + iequa - 2
         end if
      else
         offset = (ntimlv-1)*ndegfd + iequa - 1
      end if

      if ( ibuffr(ipinfmm) .gt.0 .or.
     +     ibuffr(ipinfmt) .gt.0 ) then

c     --- only in case of periodic b.c.

         call isfilimv(iequa, coupled, ni, nj, nk, nvirtual,
     +                 nblocks, isitopol, idiscret,
     +                 ipsmsg, ibuffr(ipmtvc) )
      end if

c     --- If pressure equation and the keyword Gauss_Seidel

      if (iequa.eq.iseqpres.and.igauss.eq.1) then

          maxiter = 100
          mitergs = 1
          tol     = 1d-6
          tolbsec = 1d-10
          residgs = 1d10
          nusol   = (ni+1)*(nj+1)*(nk+1)
          itergs  = 0

c         --- initialise the work arrays

         call isgsint(buffer(ipgausitr),lgausitr)
         call isgsint(buffer(ipsolutnv),nusol)

c        --- copy array geom to geomnv, identical but without virtual cells

         call isscopy(
     +          iareav(1,1),iareav(2,1),
     +          iareav(1,2),iareav(2,2),
     +          iareav(1,3),iareav(2,3),
     +          iareanv(1,1),iareanv(2,1),
     +          iareanv(1,2),iareanv(2,2),
     +          iareanv(1,3),iareanv(2,3),
     +          1,iareanv,buffer(ipgeom+3*npointsv),
     +          buffer(ipgeomnv+3*npointsnv),1d0)

c        --- remove compressible part of pressure matrix

         ichoiceg = -1
         call isupm2d(buffer(ipmatrix), ibuffr(ipiinput),
     +          buffer(iprinput), buffer(ipgeom+3*npointsv),
     +          buffer(ipsolut+(ndim + iequa - 2)*npointsv),
     +          buffer(ipsolut+(ndegfd+ndim + iequa - 2)*npointsv),
     +          ni, nj, nvirtual, ichoiceg, buffer(ipdensup),
     +          buffer(ipmatman), buffer(ipmatman+npointsv),
     +          buffer(ipmatman+npointsv*2),
     +          buffer(ipmatman+npointsv*3))

c        --- While residual > tolerance and itergs< maxiter do

  5       if (residgs.gt.tol.and.itergs.lt.maxiter) then

              itergs=itergs+1
              offset=ndim + iequa - 2

c             --- one non-linear Gauss-Seidel step

              call isgaus( buffer(ipmatrix),
     +                      buffer(ipgausitr), buffer(iprhside),
     +                      buffer(ipgeomnv+3*npointsnv),
     +                      buffer(ipdensup),
     +                      ibuffr(ipinfmt),
     +                      ibuffr(ipiinput), buffer(iprinput),
     +                      buffer(ipsolutnv+offset*nusol),
     +                      mitergs, tolbsec, nusol, ibuffr(ipmtvc),
     +                      ibuffr(ibuffr(ipmtvc + 14 - 1)),
     +                      ibuffr(ibuffr(ipmtvc + 15 - 1)))

c             --- update the RHS-vector and the main diagonal of
c                 the pressure equation

              ichoiceg = 1

              if ( coupled ) then
                if ( iequa.eq.1 ) then
                    offset = (ntimlv-1)*ndegfd
                else
                    offset = (ntimlv-1)*ndegfd + ndim + iequa - 2
                end if
              else
                offset = (ntimlv-1)*ndegfd + iequa - 1
              end if

              call isgsup(buffer(ipmatrix),buffer(iprhside),
     +            ibuffr(ipiinput),buffer(iprinput),
     +            buffer(ipgeomnv+3*npointsnv),
     +            buffer(ipsolutnv+(ndim + iequa - 2)*nusol),
     +            buffer(ipsolutnv),
     +            buffer(ipgausitr),
     +            ibuffr(ipinfmt),ni,nj,nvirtual, ichoiceg, itergs,
     +            buffer(ipdensup), buffer(ipmatman),
     +            buffer(ipmatman+nusol), buffer(ipmatman+nusol*2),
     +            buffer(ipmatman+nusol*3), buffer(ipmatman+nusol*4),
     +            buffer(ipmatman+nusol*5), buffer(ipmatman+nusol*6),
     +            buffer(ipmatman+nusol*7), ibuffr(ipmtvc),
     +            ibuffr(ibuffr(ipmtvc + 14 - 1)),
     +            ibuffr(ibuffr(ipmtvc + 15 - 1)))

c             --- one GMRES step

              call islsol(iequa, ibuffr(ipiinput),
     +                    buffer(iprinput),
     +                    ibuffr(ipinfmm), ibuffr(ipinfmt),
     +                    buffer(ipmatrix),buffer(iprhside),
     +                    buffer(ipsolutnv),
     +                    buffer(ipsolut+offset*npointsv),
     +                    buffer(ipprec), buffer(ipwork), lwork,
     +                    ibuffr(ipmtvc))

c             --- reupdate the pressure matrix and the rhs

              ichoiceg = -1

              call isgsup(buffer(ipmatrix),buffer(iprhside),
     +            ibuffr(ipiinput),buffer(iprinput),
     +            buffer(ipgeomnv+3*npointsnv),
     +            buffer(ipsolutnv+(ndim + iequa - 2)*nusol),
     +            buffer(ipsolutnv),
     +            buffer(ipgausitr),
     +            ibuffr(ipinfmt),ni,nj,nvirtual, ichoiceg, itergs,
     +            buffer(ipdensup), buffer(ipmatman),
     +            buffer(ipmatman+nusol), buffer(ipmatman+nusol*2),
     +            buffer(ipmatman+nusol*3), buffer(ipmatman+nusol*4),
     +            buffer(ipmatman+nusol*5), buffer(ipmatman+nusol*6),
     +            buffer(ipmatman+nusol*7), ibuffr(ipmtvc),
     +            ibuffr(ibuffr(ipmtvc + 14 - 1)),
     +            ibuffr(ibuffr(ipmtvc + 15 - 1)))

c             --- compute the total pressure correction and save
c                 the new pressure correction in work array

              call isgsdpsm(buffer(ipsolutnv+offset*nusol),
     +                      buffer(ipsolutnv),
     +                      buffer(ipgausitr), ibuffr(ipiinput),
     +                      itergs)

c             --- check the residual of the non linear equation

              call isgsckrs(buffer(ipsolutnv+
     +                           (ndim + iequa - 2)*nusol),
     +                      buffer(ipsolutnv+offset*nusol),
     +                      buffer(iprinput), ibuffr(ipinfmt),
     +                      buffer(ipmatrix),
     +                      buffer(ipgausitr),
     +                      ibuffr(ipiinput), buffer(iprhside),
     +                      buffer(ipgeomnv+3*npointsnv),
     +                      residgs, buffer(ipdensup), nusol, ni,
     +                      nj, buffer(ipmatman),
     +                      buffer(ipmatman+nusol),
     +                      buffer(ipmatman+nusol*2),
     +                      buffer(ipmatman+nusol*3),
     +                      buffer(ipmatman+nusol*4),
     +                      buffer(ipmatman+nusol*7),
     +                      ibuffr(ipmtvc),
     +                      ibuffr(ibuffr(ipmtvc + 14 - 1)),
     +                      ibuffr(ibuffr(ipmtvc + 15 - 1)))

c             --- if residual>tolerance and itergs < maxiter
c                                       go back for another iteration

              go to 5
          else

             if(itergs.eq.maxiter) then
               write(*,*) ' no convergence in gauss-seidel '
               call isstop
             end if

          end if

c         --- fill information arrays for printing

          if(residgs.gt.tol.and.itergs.ge.maxiter) then
            ibuffr(ipigsinf  )= 0
          else if(residgs.le.tol.and.itergs.le.maxiter) then
            ibuffr(ipigsinf  )= 0
          end if
          ibuffr(ipigsinf+1)  = mitergs
          ibuffr(ipigsinf+2)  = itergs
          ibuffr(ipigsinf+3)  = maxiter
          ibuffr(ipigsinf+4)  = 1
          ibuffr(ipigsinf+5)  = 1
          nints               = 5+1
          buffer(iprgsinf  )  = tolbsec
          buffer(iprgsinf+1)  = tol
          buffer(iprgsinf+2)  = residgs
          nreal               = 2+1

c         --- print information for convergence behaviour non-linear
c             --- Gauss-Seidel

          call isprint( 109, nints, ibuffr(ipigsinf), nreal,
     +                                  buffer(iprgsinf), 1, ' ')

      else

         call islsol(iequa, ibuffr(ipiinput),
     +               buffer(iprinput),
     +               ibuffr(ipinfmm), ibuffr(ipinfmt),
     +               buffer(ipmatrix),buffer(iprhside),
     +               buffer(ipsolutnv),
     +               buffer(ipsolut+offset*npointsv),
     +               buffer(ipprec), buffer(ipwork), lwork,
     +               ibuffr(ipmtvc))
      end if

      if ( nerror.ne.0 ) go to 1000

c     --- copy arrays solutnv to the array
c         solut which is identical but with
c         virtual points

      do idegfd= idegfd1, idegfd2

         iareac(1,1) = 1
         iareac(2,1) = ni
         iareac(1,2) = 1
         iareac(2,2) = nj
         if (ndim.eq.2) then
            iareac(1,3) = 1
            iareac(2,3) = 1
         else
            iareac(1,3) = 1
            iareac(2,3) = nk
         end if
         if (idegfd.le.ndim) iareac(2,idegfd) = iareac(2,idegfd)+1
         offset = (ntimlv-1)*ndegfd+(idegfd-1)
         call isscopy(
     +                iareanv(1,1),iareanv(2,1),
     +                iareanv(1,2),iareanv(2,2),
     +                iareanv(1,3),iareanv(2,3),
     +                iareav(1,1),iareav(2,1),
     +                iareav(1,2),iareav(2,2),
     +                iareav(1,3),iareav(2,3),
     +                1,iareac,buffer(ipsolutnv+offset*npointsnv),
     +                buffer(ipsolut+offset*npointsv),1d0)

         if ( (ibuffr(ipinfmm) .gt.0 .or.
     +         ibuffr(ipinfmt) .gt.0) .and.
     +            iequa.eq.iseqpres.and.igauss.eq.1) then

c        --- only in case of periodic b.c. explicitly exchange unknowns across
c            boundary for nonlinear Gauss-Seidel algorithm

            isolv = 1

            call ismblk02( ibuffr(ipmtvc+1-1), ibuffr(ipmtvc+2-1) ,
     +                     ibuffr(ipmtvc+3-1), ibuffr(ipmtvc+4-1) ,
     +                     ibuffr(ipmtvc+5-1), ibuffr(ipmtvc+6-1) ,
     +                     ibuffr(ipmtvc+7-1), ibuffr(ipmtvc+8-1) ,
     +                     ibuffr(ipmtvc+9-1), ibuffr(ipmtvc+10-1),
     +                     ibuffr(ipmtvc+11-1), ibuffr(ipmtvc+12-1),
     +                     ibuffr(ipmtvc+13-1), isolv ,
     +                     buffer(ipsolut+offset*npointsv),
     +                     isolv, isolv, isolv,
     +                     ibuffr(ibuffr(ipmtvc+14-1)),
     +                     ibuffr(ibuffr(ipmtvc+15-1)),
     +                     buffer(ibuffr(ipmtvc+16-1)) )
         end if
      end do

c     --- add new compressible part of pressure correction matrix

      if (iequa.eq.iseqpres.and.igauss.eq.1) then

          ichoiceg = 1

         call isupm2d(buffer(ipmatrix), ibuffr(ipiinput),
     +          buffer(iprinput), buffer(ipgeom+3*npointsv),
     +          buffer(ipsolut+(ndim + iequa - 2)*npointsv),
     +          buffer(ipsolut+offset*npointsv),
     +          ni, nj, nvirtual, ichoiceg, buffer(ipdensup),
     +          buffer(ipmatman), buffer(ipmatman+npointsv),
     +          buffer(ipmatman+npointsv*2),
     +          buffer(ipmatman+npointsv*3))

      end if

      if ( nerror.ne.0 ) go to 1000

c     --- restore the matrix and right-hand side
c         (Place virtual coefficients stored in matcop back into matrix
c         and restore the positions in the right-hand side that are
c         affected by linear combinations of virtual unknowns)

      if ( nblocks.gt.1 .or. method.eq.6 ) then

         call isrestore(iequa, matonly, ibuffr(ipiinput),
     +                  ibuffr(ipinfmm), ibuffr(ipinfmt),
     +                  buffer(ipmatrix), buffer(ipmatcop),
     +                  buffer(iprhside), buffer(iprhscop),
     +                  buffer(ipwork), lmatvir)

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- delete local arrays

      call isdelarr(ismtvc)
      call isdelarr(isinfmm)
      call isdelarr(isinfmg)
      call isdelarr(isinfmt)
      call isdelarr(issmsg)
      call isdelarr(isgausitr)
      call isdelarr(ismatman)
      call isdelarr(isigsinf)
      call isdelarr(isrgsinf)
      call isdelarr(isgeomnv)
      if ( saveprec.eq.0 ) call isdelarr(isprec)
      call isdelarr(issolutnv)
      call isdelarr(iswork)
      if ( nblocks.gt.1 .or. method.eq.6 ) call isdelarr(ismatcop)
      call isdelarr(isrhscop)

      call isdcrpri

1000  call iserclos('islinsol')

      end
