      subroutine islsds01 ( iinput, linfmm, linfmg , linfmt, lwork,
     +                      ni, nj, nk, ndim, nvirtual, ndegfd,
     +                      ntimlv, nturb, maxdx, maxdim )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.11  date    27-05-1998 Guus: length for 3D WesBeek
c     version      1.10  date    02-04-1997 Guus: set maxdim=3
c     version      1.9   date    01-10-1996 Kees K length of work changed
c                                           60+48, temporary for
c                                           Wesbeek discretization
c                                           newdiscr removed
c     version      1.8   date    07-05-1996 Marcel length of work changed
c                                           for transport
c     version      1.7   date    09-02-1996 Kees K length of work changed
c                                           for new discretization
c     version      1.6   date    31-11-1994 RvN virtual points in bcgeo
c     version      1.5   date    31-11-1994 Kees K lcoef,lgeom removed
c     version      1.4   date    26-10-1994 M. Zijlema  extension of
c                                           parameter list with nturb
c     version      1.3   date    26-10-1994 Kees K. new parameter list
c                                           was routine islsds
c     version      1.2   date    05-09-1994 R. van Nooyen now uses islinfmt
c     version      1.1   date    17-08-1994 R. van Nooyen 3D adaptation
c     version      1.0   date    17-01-1994
c
c   copyright (c) 1994-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Get some general information from array iinput and
c     compute lenghts of temporary arrays
c **********************************************************************
c
c                       KEYWORDS
c
c     lengths
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer linfmm, linfmg, linfmt
      integer iinput(*), lwork,
     +        ni, nj, nk, ndim, nturb,
     +        nvirtual, ndegfd, ntimlv, maxdx, maxdim

c     iinput    i    reserved
c     linfmg    o    Length of array infmg
c     linfmm    o    Length of array infmm
c     linfmt    o    Length of array infmt
c     lwork     o    Length of array work
c     maxdim    i    Highest physical space dimension for which ISNaS can work
c     maxdx     o    Maximum relative shift between cell indices
c     ndegfd    i    Number of degrees of freedom
c     ndim      o    Dimension of space (2 or 3)
c     ni        o    Number of non-virtual cells in the i-direction (one block)
c     nj        o    Number of non-virtual cells in the j-direction (one block)
c     nk        o    Number of non-virtual cells in the k-direction (one block)
c     ntimlv    i    Number of time levels
c     nturb     o    Number of extra transport equations for turbulence
c     nvirtual  o    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer lbcgeo, lworkm, lworkr, ipiindsc, idiscret,
     +        maxvpnts, nbc

c     idiscret    Type of discretization
c     ipiindsc    Starting address of array IINDSC in array IINPUT
c     lbcgeo      Length for auxiliary array bcgeo in matrix building
c                 routine
c     lworkm      Length work array for matrix
c     lworkr      Length of array workr
c     maxvpnts    Maximum number of points per block including virtual ones
c     nbc         Help variable
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISLINFMT  Get lengths of infmat arrays
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     get general information from array iinput
c     compute lengths of the arrays coef, cofron, solron
c     and cooron
c     compute length of array geom
c     compute length of work array for the matrix building routines
c     compute length of work array for linear solver routines
c     compute lengths of the arrays infmm, infmg, infmt
c     save maximum of work spaces
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('islsds')

c     --- get lengths for infmat arrays and values of basic variables

      call islinfmt ( iinput, linfmm, linfmg, linfmt,
     +                ni, nj, nk, ndim )

      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      ntimlv   = iinput(64)
      maxdx    = iinput(65)
      maxdim   = 3
      maxvpnts = iinput(69)
      nturb    = iinput(70)
      ipiindsc = iinput(6)
      idiscret = iinput(ipiindsc+1)

c     --- work space for matrix building routines

      nbc    = 2 * ( max(ni,nj,nk) + 1 + 2*nvirtual )
      lbcgeo = (2*ndim)*ndim*ndim*(nbc**(ndim-1))

      if ( ndim.eq.2) then
         lworkm = maxvpnts * (60+48)     +lbcgeo
      else if ( ndim.eq.3 .and. (idiscret.eq.2 .or.idiscret.eq.4) ) then
         lworkm = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +            (nk+1+2*nvirtual)*(114+9)
      else
         lworkm = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +            (nk+1+2*nvirtual)*130 +lbcgeo
      end if
      lworkr = maxvpnts * 10

c     --- save maximum of work spaces
c         make sure enough workspace is present for the
c         storage of covariant vectors during sqrtg calculation

      lwork = max(lworkm,lworkr,maxvpnts*ndim*ndim)

      call iserclos('islsds')

      end
