      subroutine islsds02 ( iequa, iinput, linfmm, linfmg, linfmt,
     +                      lprec, lsolutnv, lwork, ni, nj, nk, ndim,
     +                      nvirtual, ndegfd, ntimlv, maxdx, maxdim  )
c ======================================================================
c
c     programmer   Kees Kassels
c
c     version      2.5   date    31-03-1999 Kees V. bug fixed in length
c                                           of work array
c     version      2.4   date    24-08-1998 Kees K. maxcon added for
c                                 computing length of work array
c     version      2.3   date    19-09-1997 Kees V. changes for uncoupled
c     version      2.2   date    25-08-1997 Kees V. Multi-grid prec 3D added
c     version      2.1   date    06-05-1997 Kees V. correction of ipiinsol
c     version      2.0   date    17-04-1997 Kees V. Multi-grid prec added
c     version      1.9   date    02-04-1997 Guus: set maxdim=3
c     version      1.8   date    23-01-1997 Kees V. CGS added
c     version      1.7   date    21-01-1997 Kees V. space added for printing
c     version      1.6   date    27-06-1996 Kees V. allocation for
c                                           solver is made correct
c     version      1.5   date    18-03-1996 Kees V. adding ILU in 3D
c     version      1.4   date    15-09-1995 Kees V. memory for ILU_Fill
c     version      1.3   date    26-10-1994 Kees K. new parameter list
c                                         was routine islsds
c     version      1.2   date    05-09-1994 R. van Nooyen now uses islinfmt
c     version      1.1   date    17-08-1994 R. van Nooyen 3D adaptation
c     version      1.0   date    17-01-1994
c
c   copyright (c) 1993-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Get some general information from array iinput and
c     compute lenghts of temporary arrays
c **********************************************************************
c
c                       KEYWORDS
c
c     lengths
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer linfmm, linfmg, linfmt, iequa, iinput(*), lprec, lwork,
     +        lsolutnv, ni, nj, nk, ndim, nvirtual, ndegfd, ntimlv,
     +        maxdx, maxdim

c     iequa     General equation sequence number
c     iinput    i    reserved
c     linfmg    o    Length of array infmg
c     linfmm    o    Length of array infmm
c     linfmt    o    Length of array infmt
c     lprec     o    Length of array prec
c     lsolutnv  o    Length of temporary solution array
c     lwork     o    Length of array work
c     maxdim    o    Highest physical space dimension for which ISNaS can work
c     maxdx     o    Maximum relative shift between cell indices
c     ndegfd    o    Number of degrees of freedom
c     ndim      o    Dimension of space (2 or 3)
c     ni        o    Number of non-virtual cells in the i-direction (one block)
c     nj        o    Number of non-virtual cells in the j-direction (one block)
c     nk        o    Number of non-virtual cells in the k-direction (one block)
c     ntimlv    i    Number of time levels
c     nvirtual  o    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer leniinsol, ipiinsol, lworks, maxpoints, maxvpnts,
     +        nprec, nqmr, ndimso, ahilevel, nmaxf, maxcon
      logical coupled

c     ahilevel  Maximum level of coarse grids (level 0 is finest)
c               (formal parameter)
c     coupled   Logical which is true if the momentum equations
c               are coupled
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     leniinsol Length of first component of array iinsol
c     lworks    Length work array for solver
c     maxcon    Maximum number of non zero diagonals in a matrix
c     maxpoints Maximum number of points in a block
c     maxvpnts  Maximum number of points per block including virtual ones
c     ndimso    Integer indicating the dimension of the space in
c               which the problem must be solved (ndimso = 1 or ndim)
c     nmaxf     Number of grid points used to determine ahilevel
c     nprec     Maximum number of diagonals in preconditioning matrix
c     nqmr      An auxiliary variable to allocate memory space for
c               the linear solvers
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     AUTOSIZ3  Calculates required dimension of bufferspace (3D MG)
c     AUTOSIZE  Calculates required dimension of bufferspace (MG)
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISLINFMT  Get lengths of infmat arrays
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     get general information from array iinput
c     compute lengths of the arrays coef, cofron, solron
c     and cooron
c     compute length of array geom
c     compute length of work array for the matrix building routines
c     compute length of work array for linear solver routines
c     compute lengths of the arrays infmm, infmg, infmt
c     save maximum of work spaces
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('islsds02')

c     --- get lengths for infmat arrays and values of basic variables

      call islinfmt ( iinput, linfmm, linfmg, linfmt, ni, nj, nk, ndim )

      nvirtual   = iinput(54)
      ndegfd     = iinput(55)
      maxcon     = iinput(60)
      ntimlv     = iinput(64)
      maxdx      = iinput(65)
      maxdim     = 3
      maxvpnts   = iinput(69)
      maxpoints  = iinput(71)
      leniinsol  = iinput(105)
      ipiinsol   = iinput(5)+(iequa-1)*leniinsol
      coupled    = iinput(76).eq.0
      lsolutnv   = maxpoints * ndegfd * ntimlv

      iinput(ipiinsol-1+25) = iinput(76)

c     --- work space for the solver

      if( iequa.eq.1 .and. coupled ) then
         ndimso = ndim
      else
         ndimso = 1
      end if

      nprec = 0
      iinput(ipiinsol-1+22) = min ( iinput(ipiinsol-1+22), ni-2 )
      if ( abs(iinput(ipiinsol-1+2)).eq.1 ) nprec = 1
      if ( abs(iinput(ipiinsol-1+2)).eq.2 ) nprec = 2
      if ( abs(iinput(ipiinsol-1+2)).eq.3 ) then
         if ( ndim.eq.2 ) nprec = 9
         if ( ndim.eq.3 ) nprec = 19
      end if
      if ( abs(iinput(ipiinsol-1+2)).eq.4 )
     +    nprec = 2*iinput(ipiinsol-1+22)+9

      lprec = maxpoints * ndimso * nprec

      if ( abs(iinput(ipiinsol-1+2)).eq.5 ) then

c     --- Determination of workspace for the Multi-grid preconditioner

         if ( iinput(ipiinsol-1+9).le.0 ) then
            ahilevel = 0
            nmaxf = max ( ni, nj, nk )

            do while ( nmaxf.gt.2 )
               nmaxf = nmaxf/2 + 1
               ahilevel = ahilevel + 1
            end do
            iinput(ipiinsol-1+9) = ahilevel
         end if

         if ( ndim.eq.2 ) then
            call autosize ( ni, nj, 0, iinput(ipiinsol-1+9),
     +                      lprec, 0, 1, 0, 1 )
         else
            call autosiz3 ( ni, nj, nk, 0, iinput(ipiinsol-1+9),
     +                      lprec, 0, 1, 0, 1, 0, 1 )
         end if

         if ( iinput(ipiinsol-1+10).le.0 ) then
            iinput(ipiinsol-1+10) = 2
         end if
      end if
      lprec = max(lprec,1)

      if(iinput(ipiinsol-1+7).eq.0) nqmr = 7
      if(iinput(ipiinsol-1+7).eq.1) nqmr = 12
      if(iinput(ipiinsol-1+7).eq.2) then
         nqmr = iinput( ipiinsol -1 + 11)+5
      end if
      if(iinput(ipiinsol-1+7).eq.4) nqmr = 10
      if(iinput(ipiinsol-1+7).eq.6) nqmr = 1
      if(iinput(ipiinsol-1+13).gt.0) then
         if(iinput(ipiinsol-1+7).eq.0) nqmr = 7
         if(iinput(ipiinsol-1+7).eq.1) nqmr = 12
         if(iinput(ipiinsol-1+7).eq.2) then
            nqmr = iinput( ipiinsol -1 + 13)+5
         end if
         if(iinput(ipiinsol-1+7).eq.4) nqmr = 10
         if(iinput(ipiinsol-1+7).eq.6) nqmr = 1
         nqmr = nqmr+2*iinput(ipiinsol-1+12)+5
      end if
      nqmr   = max(nqmr,iinput( ipiinsol -1 + 14)+5)
      lworks = nqmr * maxpoints * ndimso

c     --- Space is added in order to store the norm of the residuals

      lworks = lworks + iinput(ipiinsol-1+6) + 2 +
     +         iinput( ipiinsol -1 + 13)

c     --- Space is added for printing in the iterative methods

      lworks = lworks + 100

c     --- save maximum of work spaces
c         make sure enough workspace is present for the
c         storage of covariant vectors during sqrtg calculation
c         and for the array mask when multi-block is used

      lwork = max(lworks,maxvpnts*maxcon*ndimso)

      call iserclos('islsds02')

      end
