      subroutine ismulmfr ( bndcon, massfrac, iside, i1, i2, ni, nj,
     +                      idim, nvirtual )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 04-12-1998 Include last point
c        version 1.0    date 04-11-1997
c
c   copyright (c) 1997-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Multiply array bndcon by the mass fraction for a specific side
c     and segment
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iside, i1, i2, ni, nj, idim, nvirtual
      double precision bndcon(2,1-nvirtual:*),
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual)

c     bndcon   i/o   reserved
c     i1        i    Lower index number along first direction of a face or edge
c     i2        i    Upper index number along first direction of a face or edge
c     idim      i    Loop variable used for dimension
c     iside     i    Sequence number of side
c     massfrac  i    Array containing the mass fraction
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inci, incj, ip, jp, i
      double precision massmean

c     i         General loop variable
c     inci      Offset between vertices in i-direction
c     incj      Offset between vertices in j-direction
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     massmean  Mean value of mass fraction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( iside.le.2 ) then
         jp = i1-1
         inci  = 0
         incj  = 1
         if ( iside.eq.1 ) then
            ip = 1
         else
            ip = ni+1
         end if
      else
         ip = i1-1
         inci  = 1
         incj  = 0
         if ( iside.eq.3 ) then
            jp = 1
         else
            jp = nj+1
         end if
      end if
      do i = i1, i2
         ip = ip+inci
         jp = jp+incj
         bndcon(idim,2*i-1) = bndcon(idim,2*i-1)*massfrac(ip,jp)
         massmean = 0.5d0*(massfrac(ip,jp)+massfrac(ip+inci,jp+incj))
         bndcon(idim,2*i) = bndcon(idim,2*i)*massmean
      end do
      end
