      subroutine isorient2 ( iside, tdir, ndir, tsign, nsign )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.1    date 15-02-1996 Kees V: Changed tsign, nsign
c                                       to tsign, nsign
c        version 1.0    date 28-09-1995
c
c   copyright (c) 1994-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION

c     Computes ndir, nsign, tdir, tsign for a 2-dimensional problem
c
c **********************************************************************
c
c                      KEYWORDS
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer iside, tdir, ndir, tsign, nsign
c     iside     i    Sequence number of side
c     ndir      o    Direction of normal vector on this side
c     nsign     o    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     tdir      o    Direction of tangent vector on this side
c     tsign     o    Inner product of tangential vector defined counter
c                    clockwise with respect to the outward normal and the
c                    base vector along the side
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS

c **********************************************************************
c
c             SUBROUTINES CALLED
c
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    PSEUDO CODE
c
c     Computes ndir, nsign, tdir, tsign for a 2-dimensional problem.
c     We have the following structure of the computational domain:
c
c                          4
c               -------------------------
c              |                         |
c              |                         |
c              |                         |
c              |                         |
c              |                         |
c           1  |                         |  2
c              |                         |
c              |                         |
c              |                         |
c              |                         |
c              |                         |
c              |                         |
c               -------------------------
c                          3
c
c     The normal component on side 1 and 2 is the x1-direction, whereas
c     the tangential component is in the x2-direction.
c     The normal component on side 3 and 4 is the x2-direction, whereas
c     the tangential component is in the x1-direction.
c
c     Furthermore the sign of the normal and tangential directions are
c     filled. The normal always point in outward direction, whereas
c     the tangential direction is such that if the normal vector is
c     rotated to the tangential direction the rotation is counter-clock
c     wise.
c
c **********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            tdir = 2
            ndir = 1
            if ( iside.eq.1 ) then

c           --- iside = 1:  left-hand side

               nsign = -1

            else

c           --- iside = 2:  right-hand side

               nsign = 1

            end if
            tsign = nsign

         else

c        --- iside = 3, 4:  lower- and upper side

            tdir = 1
            ndir = 2
            if ( iside.eq.3 ) then

c           --- iside = 3:  lower side

               nsign = -1

            else

c           --- iside = 4:  upper side

               nsign = 1

            end if
            tsign = -nsign

         end if
      end
