      subroutine isprc01 ( isiinput, isrinput, issolut, isgmat, iscoefs,
     +                     iscoor, isgeom, isigeom, isinfbnd )
c ======================================================================
c
c     programmer   Kees Kassels
c
c     version      3.2   date  02-02-1999 Kees K: axisymmetric case
c     version      3.1   date  27-07-1998 Duncan: extension to iterate timestep
c                                                 apply underrelaxation to
c                                                 pressure correction
c     version      3.0   date  04-06-1998 Charles: bilinear interpolation added
c     version      2.9   date  18-05-1998 Charles Bilinear discretization
c     version      2.8   date  15-12-1997 Charles: call of iscmcv2d changed
c     version      2.7   date  19-11-1997 Kees K.: call of ismtvc24 changed
c     version      2.6   date  05-11-1997 Charles: correction in dt
c     version      2.5   date  29-10-1997 Charles: change in collocated part
c     version      2.4   date  13-10-1997 Charles: collocated allowed
c     version      2.3   date  10-08-1996 Guus: correction call to isfilm
c     version      2.2   date  03-10-1996 Kees K.: newdiscr removed
c     version      2.1   date  22-07-1996 Guus: new subroutines for updating
c                                         velocity and pressure
c     version      2.0   date  07-06-1996 Kees K.: set d=1 for 3D
c     version      1.9   date  22-11-1995 Kees K.: call of isgmv changed
c     version      1.8   date  10-10-1995 Guus: set dt=1 for new discretizatio
c     version      1.7   date  30-05-1995 Hester: correct. print press.
c     version      1.6   date  04-05-1995 Guus: correction print
c     version      1.5   date  05-04-1995 Guus: new call to isfilm
c     version      1.4   date  28-03-1995 Erik: correct pressures also
c     version      1.3   date  21-03-1995 Guus: print pressure
c     version      1.2   date  20-12-1994 R. v. Nooyen wesbeek version
c     version      1.1   date  05-09-1994 R. v. Nooyen 3d version
c     version      1.0   date  27-05-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     single domain pressure correction ( one block )
c
c **********************************************************************
c
c                       KEYWORDS
c
c     pressure_correction
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isiinput, isrinput, issolut, isgmat, iscoefs, iscoor,
     +        isgeom, isigeom, isinfbnd

c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i   Array of sequence numbers (nblocks) of array gmat
c     isigeom   i   Array of sequence numbers (nblocks) of array igeom
c     isiinput  i   Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i   Array of sequence numbers (nblocks) of array infbnd
c     isrinput  i   Array of sequence numbers (nblocks) of array rinput
c     issolut   i   Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipgmat, ipiinput, ipinfmg, ipinfmm, ipinfmt, ipiintim,
     +        ipsolut, linfmg, auxdim, ntimlv, ipiinsol,
     +        linfmm, linfmt, ndim, ni, nj, nk, leniinsol, itimeact,
     +        isinfmm, isinfmg, isinfmt, ipiindsc, iprinsl, itime1,
     +        npointsv, area(1:2,1:3), varea(1:2,1:3), itypsten,
     +        ipnewflux, ipoldp, ipnewp, ndegfd, nvirtual, ipiincom,
     +        itime2, itimest, mshtyp, ipcoefs, iprinput,
     +        ipcoor, ipgeom, ipigeom, ipinfbnd,
     +        iswork1, lwork1, ipwork1,
     +        iswork2, lwork2, ipwork2,
     +        iswork3, lwork3, ipwork3,
     +        iswork4, lwork4, ipwork4,
     +        iswork5, lwork5, ipwork5,
     +        iconv, iprintim, axisym
      double precision dt, unfac

c     area      Summation index bounds
c     auxdim    Auxiliary loop index for space direction
c     axisym    In axisymmetric case axisym=1 else axisym=0
c     dt        Time step
c     iconv     If 1 the timestep is iterated till converged, otherwise (0)
c               apply only one iterative step
c     ipcoefs   Starting address of array coefs in buffer
c     ipcoor    Starting address of array coor in buffer
c     ipgeom    Starting address of array geom in buffer
c     ipgmat    Starting address of array gmat in buffer
c     ipigeom   Starting address of array igeom in ibuffr
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipinfbnd  Starting address of array infbnd in ibuffr
c     ipinfmg   Starting address of array infmg in ibuffr
c     ipinfmm   Starting address of array infmm in ibuffr
c     ipinfmt   Starting address of array infmt in ibuffr
c     ipnewflux Starting address of new flux in buffer
c     ipnewp    Starting address of new pressure in buffer
c     ipoldp    Starting address of old pressure in buffer
c     iprinput  Starting address of array rinput in buffer
c     iprinsl   If 1 the solution must be printed
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ipsolut   Starting address of array solut in buffer
c     ipwork1   Starting address of array work1 in buffer
c     ipwork2   Starting address of array work2 in buffer
c     ipwork3   Starting address of array work3 in buffer
c     ipwork4   Starting address of array work4 in buffer
c     ipwork5   Starting address of array work5 in buffer
c     isinfmg   Sequence number of array infmg
c     isinfmm   Sequence number of array infmm
c     isinfmt   Sequence number of array infmt
c     iswork1   Sequence number of array work1
c     iswork2   Sequence number of array work2
c     iswork3   Sequence number of array work3
c     iswork4   Sequence number of array work4
c     iswork5   Sequence number of array work5
c     itime1    First time at which the debug prints must be performed
c     itime2    Last time at which the debug prints must be performed
c     itimeact  Actual time step
c     itimest   Time step with respect to debug printing
c     itypsten  Indicates the type of stencil
c               Possible values
c               R^2
c               1:  13-point stencil
c               2:  21-point stencil
c               3:  17-point stencil
c               R^3
c               1:  51-point stencil
c               2:  59-point stencil
c     leniinsol Length of first component of array iinsol
c     linfmg    Length of array infmg
c     linfmm    Length of array infmm
c     linfmt    Length of array infmt
c     lwork1    Length of array work1
c     lwork2    Length of array work2
c     lwork3    Length of array work3
c     lwork4    Length of array work4
c     lwork5    Length of array work5
c     mshtyp    Type of grid arrangement
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     unfac     Under relaxation factor for the pressure correction
c     varea     Area of arrays
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     DAXPY     Computes a vector + a constant * other vector. This
c               is a BLAS routine.
c     ISACTIVE  Activate old permanent array
c     ISCMCV2D  Update velocity u* by -Gp in R2 for collocated
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILM    Fill integer information about the matrices
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISLINFMT  Compute lengths of temporary arrays
c     ISMTVC24  Update velocity u* by -Gp in R2
c     ISMTVC34  Update velocity u* by -Gp in R3
c     ISPRAR01  Print general field array defined in R2
c     ISPRAR02  Print general field array defined in R3
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c        create temporary space
c        get pointers
c
c        fill integer information about the matrices
c        compute pressure correction
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isprc01')

c     --- compute lengths of temporary arrays

      call isactive(isiinput)
      call isactive(isrinput)
      call isactive(issolut )
      call isactive(isgmat  )
      call isactive(iscoefs )
      call isactive(iscoor  )
      call isactive(isgeom  )
      if ( isigeom.gt.0 ) call isactive(isigeom )
      call isactive(isinfbnd)

      ipiinput = isingetp(isiinput)

      call islinfmt(ibuffr(ipiinput), linfmm, linfmg, linfmt,
     +              ni, nj, nk, ndim )

      nvirtual = ibuffr( ipiinput-1+54 )
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      lwork1   = 16*npointsv
      lwork2   = 4*npointsv
      lwork3   = 4*npointsv
      lwork4   = (ni+1)*(nj+1)
      lwork5   = 4*npointsv

c     --- create local arrays

      isinfmm  = 0
      isinfmg  = 0
      isinfmt  = 0

      call isicreat(isinfmm, linfmm, 'infmm,' )
      call isicreat(isinfmg, linfmg, 'infmg,' )
      call isicreat(isinfmt, linfmt, 'infmt,' )

      iswork1 = 0
      iswork2 = 0
      iswork3 = 0
      iswork4 = 0
      iswork5 = 0

      call isdcreat(iswork1, lwork1, 'work1' )
      call isdcreat(iswork2, lwork2, 'work2' )
      call isdcreat(iswork3, lwork3, 'work3' )
      call isdcreat(iswork4, lwork4, 'work4' )
      call isdcreat(iswork5, lwork5, 'work5' )

c     --- get pointers

      ipiinput = isingetp(isiinput)
      ipsolut  = isdpgetp(issolut )
      ipgmat   = isdpgetp(isgmat  )
      ipcoefs  = isdpgetp(iscoefs )
      ipcoor   = isdpgetp(iscoor  )
      iprinput = isdpgetp(isrinput)

      if ( isigeom.gt.0 ) then
         ipigeom  = isingetp(isigeom )
      else
         ipigeom = 1
      end if
      ipinfbnd = isingetp(isinfbnd)

      ipgeom   = isdpgetp(isgeom  )

c     --- get pointers local arrays

      ipinfmm  = isingetp( isinfmm )
      ipinfmg  = isingetp( isinfmg )
      ipinfmt  = isingetp( isinfmt )

      ipwork1  = isdpgetp( iswork1 )
      ipwork2  = isdpgetp( iswork2 )
      ipwork3  = isdpgetp( iswork3 )
      ipwork4  = isdpgetp( iswork4 )
      ipwork5  = isdpgetp( iswork5 )

      mshtyp   = ibuffr(ipiinput-1+75)
      ntimlv   = ibuffr(ipiinput-1+64)
      nvirtual = ibuffr(ipiinput-1+54)
      ndegfd   = ibuffr(ipiinput-1+55)

      ipiintim  = ibuffr(ipiinput-1+4)
      ipiinsol  = ibuffr(ipiinput-1+5)
      ipiindsc  = ibuffr(ipiinput-1+6)
      itimeact  = ibuffr(ipiinput-1+ipiintim+3)
      leniinsol = ibuffr(ipiinput-1+105)
      dt        = buffer(iprinput-1+ibuffr(ipiinput-1+29)-1+4)
      axisym    = ibuffr(ipiinput-1 + ipiindsc -1+10)
      iconv     = ibuffr(ipiinput- 1 + ipiintim -1 + 17)

      area(1,1) = 1
      area(1,2) = 1
      area(1,3) = 1
      area(2,1) = ni
      area(2,2) = nj
      area(2,3) = max(1,nk)

      varea(1,3) = 1
      varea(2,3) = 1
      do 100 auxdim=1,ndim
         varea(1,auxdim) = 1-nvirtual
         varea(2,auxdim) = area(2,auxdim)+1+nvirtual
 100  continue

      npointsv = (varea(2,1)-varea(1,1)+1)*(varea(2,2)-varea(1,2)+1)
      if (ndim.eq.3) npointsv = npointsv*(varea(2,3)-varea(1,3)+1)

      ipoldp    = ipsolut+(ntimlv-2)*ndegfd*npointsv+npointsv*ndim
      ipnewp    = ipsolut+(ntimlv-1)*ndegfd*npointsv+npointsv*ndim
      ipnewflux = ipsolut+(ntimlv-1)*ndegfd*npointsv

c     --- fill integer information about the matrices

      ipiincom = ibuffr(ipiinput-1+9)
      if ( ibuffr(ipiinput-1+ipiindsc+1).eq.2 ) then
         itypsten = 2
      else if ( ibuffr(ipiinput-1+ipiincom).eq.1 .and.
     +          ibuffr(ipiinput-1+ipiindsc-1+3).eq.0 ) then
         itypsten = 3
      else
         itypsten = 1
      end if
      call isfilm ( ibuffr(ipinfmm), ibuffr(ipinfmg),
     +              ibuffr(ipinfmt), ibuffr(ipiinput), itypsten )

c     --- compute pressure correction and update velocity
c         New discretization, dt has been stored in gmat
c         u = u(*) - (1-a)*G delta p

      unfac=0d0

      if (iconv.eq.1) then

         iprintim= ibuffr(ipiinput + 29 -1)
         unfac = buffer(iprinput+iprintim + 18 -2)

      end if

      if ( ndim.eq.2 ) then
         if ( mshtyp.eq.0 ) then
            call ismtvc24 ( buffer(ipgmat), buffer(ipnewp),
     +                      buffer(ipnewflux), ni, nj, nvirtual,
     +                      ibuffr(ipinfmg-1+ibuffr(ipinfmg+11)),
     +                      buffer(ipcoor), 0d0 , axisym)

         else if ( mshtyp.eq.1 ) then
            call iscmcv2d ( buffer(ipnewp), buffer(ipnewflux), ni, nj,
     +                      nvirtual, buffer(ipcoefs),
     +                      dt, buffer(ipcoor), buffer(ipwork1),
     +                      buffer(ipwork2), buffer(ipwork3),
     +                      buffer(ipgeom), buffer(ipwork4),
     +                      buffer(ipgmat),
     +                      ibuffr(ipinfmg-1+ibuffr(ipinfmg+11)),
     +                      buffer(ipgeom+4*npointsv-1),
     +                      ibuffr(ipinfbnd), ibuffr(ipinfbnd-1+13),
     +                      ibuffr(ipiinput-1+ipiindsc+1),
     +                      buffer(ipwork5),
     +                      buffer(ipgeom+8*npointsv),
     +                      ibuffr(ipigeom) )

         end if
      else
         call ismtvc34 ( buffer(ipgmat), buffer(ipnewp),
     +                   buffer(ipnewflux), ni, nj, nk, nvirtual,
     +                   ibuffr(ipinfmg-1+ibuffr(ipinfmg+11)),
     +                   ibuffr(ipinfmg) )
      end if

c     --- adjust previous pressure
c             p(n+1) = p(n) + (1-a)*delta p

      call daxpy ( npointsv, 1d0-unfac, buffer(ipnewp), 1,
     +                                          buffer(ipoldp), 1 )

c     --- First print velocity if necessary

      iprinsl  = ibuffr(ipiinput-1+ipiinsol+16)
      itime1   = ibuffr(ipiinput-1+ipiinsol+17)
      itime2   = ibuffr(ipiinput-1+ipiinsol+18)
      itimest  = ibuffr(ipiinput-1+ipiinsol+19)

      if ( iprinsl.eq.1 ) then

c     --- Print velocity

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            write(irefwr,200) 'U'
200         format( a )
            if ( ndim.eq.2 ) then
               call isprar01 ( buffer(ipnewflux), nvirtual, ni, nj, 2 )
            else
               call isprar02 ( buffer(ipnewflux), nvirtual, ni, nj,
     +                         nk, 3 )
            end if
         end if

      end if

c     --- Next print pressure if necessary

      ipiinsol = ipiinsol+leniinsol
      iprinsl  = ibuffr(ipiinput-1+ipiinsol+16)
      itime1   = ibuffr(ipiinput-1+ipiinsol+17)
      itime2   = ibuffr(ipiinput-1+ipiinsol+18)
      itimest  = ibuffr(ipiinput-1+ipiinsol+19)
      if ( iprinsl.eq.1 ) then

c     --- Print pressure

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            write(irefwr,200) 'pressure'
            if ( ndim.eq.2 ) then
               call isprar01 ( buffer(ipoldp), nvirtual, ni, nj, 1 )
            else
               call isprar02 ( buffer(ipoldp), nvirtual, ni, nj,
     +                         nk, 1 )
            end if
         end if

      end if

      call isdelarr(isinfmm)
      call isdelarr(isinfmg)
      call isdelarr(isinfmt)
      call isdelarr(iswork5)
      call isdelarr(iswork4)
      call isdelarr(iswork3)
      call isdelarr(iswork2)
      call isdelarr(iswork1)

      call isdcrpri

      call iserclos('isprc01')

      end
