      subroutine isquadtn( hightens,alphad, betad, pderiv, veloc, cva,
     +                     invjac, jctg, chsym, ni, nj, nvirtual,
     +                     idiscret, ctau1, ctau2, ctau3, iarea        )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date   16-08-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute quadratic products of strain and rotation tensors.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, idiscret, alphad, betad, iarea(2,2)
      double precision hightens(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 pderiv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1:2,1:2),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:2), ctau1, ctau2, ctau3
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,3),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     betad     i    Direction index, usually the direction perpendicular to the
c                    integration direction of the boundary integral.
c                    For example betad in local point (1,0) is 1 and in (0,1): 2
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     ctau1     i    Parameter ctau1 for nonlinear two-equation model
c     ctau2     i    Parameter ctau2 for nonlinear two-equation model
c     ctau3     i    Parameter ctau3 for nonlinear two-equation model
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     hightens  o    Higher order tensor constains quadratic products of
c                    strain and rotation tensors
c     iarea     i    Area of arrays
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pderiv    i    Partial derivative of flux in point pnttyp
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, deltal, epsl, gammal, i, ii, jj, istwo(1:2,1:2)
      double precision gac, gbc, contrst, covrst, cntdef1, cntdef2

c     cntdef1   Resulted double contraction of strain or rotation tensor
c     cntdef2   Resulted double contraction of strain or rotation tensor
c     contrst   Contravariant rate of strain tensor
c     covrst    Covariant rate of strain tensor
c     deltal    Loop index
c     epsl      Loop index
c     gac       Covariant metric tensor
c     gammal    Loop index
c     gbc       Covariant metric tensor
c     i         General loop variable
c     ii        Loop variable
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jj        Loop variable
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c ======================================================================
c
      if ( idiscret.eq.1 ) then

c     --- standard discretization

        do jp = iarea(1,2), iarea(2,2)
          do ip = iarea(1,1), iarea(2,1)

            hightens(ip,jp) = 0d0

            cntdef1 = 0

            do gammal = 1, 2

              cntdef2 = 0

              do deltal = 1, 2

                contrst = 0d0
                covrst  = 0d0

                do epsl = 1, 2

                  gac = 0d0
                  gbc = 0d0

                  do i = 1, 2
                    gac = gac + cva(ip,jp,i,deltal) *
     +                          cva(ip,jp,i,epsl)
                    gbc = gbc + cva(ip,jp,i,gammal)  *
     +                          cva(ip,jp,i,epsl)
                  end do

                  contrst = contrst + invjac(ip,jp) *
     +                      invjac(ip,jp) *
     +                      ( jctg(ip,jp,istwo(alphad,epsl)) *
     +                        pderiv(ip,jp,deltal,epsl) +
     +                        jctg(ip,jp,istwo(deltal,epsl)) *
     +                        pderiv(ip,jp,alphad,epsl) )

                  covrst = covrst + invjac(ip,jp) *
     +                     ( gac * pderiv(ip,jp,epsl,gammal) +
     +                       gbc * pderiv(ip,jp,epsl,deltal) )

                  do ii = 1, 2

                    contrst = contrst + invjac(ip,jp) *
     +                  invjac(ip,jp) *
     +                  ( jctg(ip,jp,istwo(alphad,epsl)) *
     +                    ( chsym(ip,jp,deltal,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,deltal) ) +
     +                    jctg(ip,jp,istwo(deltal,epsl)) *
     +                    ( chsym(ip,jp,alphad,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,alphad) ) )

                    covrst = covrst + invjac(ip,jp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,epsl,istwo(ii,gammal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(gammal,ii))*
     +                      veloc(ip,jp,epsl) ) +
     +                    gbc *
     +                    ( chsym(ip,jp,epsl,istwo(ii,deltal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(deltal,ii))*
     +                      veloc(ip,jp,epsl) ) )

                  end do

                end do

                cntdef2 = cntdef2 + contrst*covrst

              end do

              cntdef1 = cntdef1 + cntdef2*invjac(ip,jp)*
     +                  jctg(ip,jp,istwo(gammal,betad))

            end do

            hightens(ip,jp) = hightens(ip,jp) + cntdef1*ctau1

            cntdef1 = 0

            do deltal = 1, 2
              do epsl = 1, 2

                contrst = 0d0
                covrst  = 0d0

                do jj = 1, 2

                  gac = 0d0
                  gbc = 0d0

                  do i = 1, 2
                    gac = gac + cva(ip,jp,i,deltal) *
     +                          cva(ip,jp,i,jj)
                    gbc = gbc + cva(ip,jp,i,epsl)  *
     +                          cva(ip,jp,i,jj)
                  end do

                  contrst = contrst + invjac(ip,jp) *
     +                      invjac(ip,jp) *
     +                      ( jctg(ip,jp,istwo(epsl,jj)) *
     +                        pderiv(ip,jp,deltal,jj) +
     +                        jctg(ip,jp,istwo(deltal,jj)) *
     +                        pderiv(ip,jp,epsl,jj) )

                  covrst = covrst + invjac(ip,jp) *
     +                     ( gac * pderiv(ip,jp,jj,epsl) +
     +                       gbc * pderiv(ip,jp,jj,deltal) )

                  do ii = 1, 2

                    contrst = contrst + invjac(ip,jp) *
     +                  invjac(ip,jp) *
     +                  ( jctg(ip,jp,istwo(epsl,jj)) *
     +                    ( chsym(ip,jp,deltal,istwo(ii,jj))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(jj,ii))*
     +                      veloc(ip,jp,deltal) ) +
     +                    jctg(ip,jp,istwo(deltal,jj)) *
     +                    ( chsym(ip,jp,epsl,istwo(ii,jj))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(jj,ii))*
     +                      veloc(ip,jp,epsl) ) )

                    covrst = covrst + invjac(ip,jp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,jj,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,jj) ) +
     +                    gbc *
     +                    ( chsym(ip,jp,jj,istwo(ii,deltal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(deltal,ii))*
     +                      veloc(ip,jp,jj) ) )

                  end do

                end do

                cntdef1 = cntdef1 + contrst*covrst

              end do
            end do

            hightens(ip,jp) = hightens(ip,jp) - cntdef1*ctau1*
     +         invjac(ip,jp)*jctg(ip,jp,istwo(alphad,betad))/3

            cntdef1 = 0

            do gammal = 1, 2

              cntdef2 = 0

              do deltal = 1, 2

                contrst = 0d0
                covrst  = 0d0

                do epsl = 1, 2

                  gac = 0d0
                  gbc = 0d0

                  do i = 1, 2
                    gac = gac + cva(ip,jp,i,deltal) *
     +                          cva(ip,jp,i,epsl)
                    gbc = gbc + cva(ip,jp,i,gammal)  *
     +                          cva(ip,jp,i,epsl)
                  end do

                  contrst = contrst + invjac(ip,jp) *
     +                      invjac(ip,jp) *
     +                      ( -jctg(ip,jp,istwo(betad,epsl)) *
     +                        pderiv(ip,jp,deltal,epsl) +
     +                        jctg(ip,jp,istwo(deltal,epsl)) *
     +                        pderiv(ip,jp,betad,epsl) )

                  covrst = covrst + invjac(ip,jp) *
     +                     ( gac * pderiv(ip,jp,epsl,gammal) +
     +                       gbc * pderiv(ip,jp,epsl,deltal) )

                  do ii = 1, 2

                    contrst = contrst + invjac(ip,jp) *
     +                  invjac(ip,jp) *
     +                  ( -jctg(ip,jp,istwo(betad,epsl)) *
     +                    ( chsym(ip,jp,deltal,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,deltal) ) +
     +                    jctg(ip,jp,istwo(deltal,epsl)) *
     +                    ( chsym(ip,jp,betad,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,betad) ) )

                    covrst = covrst + invjac(ip,jp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,epsl,istwo(ii,gammal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(gammal,ii))*
     +                      veloc(ip,jp,epsl) ) +
     +                    gbc *
     +                    ( chsym(ip,jp,epsl,istwo(ii,deltal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(deltal,ii))*
     +                      veloc(ip,jp,epsl) ) )

                  end do

                end do

                cntdef2 = cntdef2 + contrst*covrst

              end do

              cntdef1 = cntdef1 + cntdef2*invjac(ip,jp)*
     +                  jctg(ip,jp,istwo(gammal,alphad))

            end do

            hightens(ip,jp) = hightens(ip,jp) + cntdef1*ctau2

            cntdef1 = 0

            do gammal = 1, 2

              cntdef2 = 0

              do deltal = 1, 2

                contrst = 0d0
                covrst  = 0d0

                do epsl = 1, 2

                  gac = 0d0
                  gbc = 0d0

                  do i = 1, 2
                    gac = gac + cva(ip,jp,i,deltal) *
     +                          cva(ip,jp,i,epsl)
                    gbc = gbc + cva(ip,jp,i,gammal)  *
     +                          cva(ip,jp,i,epsl)
                  end do

                  contrst = contrst + invjac(ip,jp) *
     +                      invjac(ip,jp) *
     +                      ( -jctg(ip,jp,istwo(alphad,epsl)) *
     +                        pderiv(ip,jp,deltal,epsl) +
     +                        jctg(ip,jp,istwo(deltal,epsl)) *
     +                        pderiv(ip,jp,alphad,epsl) )

                  covrst = covrst + invjac(ip,jp) *
     +                     ( gac * pderiv(ip,jp,epsl,gammal) +
     +                       gbc * pderiv(ip,jp,epsl,deltal) )

                  do ii = 1, 2

                    contrst = contrst + invjac(ip,jp) *
     +                  invjac(ip,jp) *
     +                  ( -jctg(ip,jp,istwo(alphad,epsl)) *
     +                    ( chsym(ip,jp,deltal,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,deltal) ) +
     +                    jctg(ip,jp,istwo(deltal,epsl)) *
     +                    ( chsym(ip,jp,alphad,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,alphad) ) )

                    covrst = covrst + invjac(ip,jp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,epsl,istwo(ii,gammal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(gammal,ii))*
     +                      veloc(ip,jp,epsl) ) +
     +                    gbc *
     +                    ( chsym(ip,jp,epsl,istwo(ii,deltal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(deltal,ii))*
     +                      veloc(ip,jp,epsl) ) )

                  end do

                end do

                cntdef2 = cntdef2 + contrst*covrst

              end do

              cntdef1 = cntdef1 + cntdef2*invjac(ip,jp)*
     +                  jctg(ip,jp,istwo(gammal,betad))

            end do

            hightens(ip,jp) = hightens(ip,jp) + cntdef1*ctau2

            cntdef1 = 0

            do gammal = 1, 2

              cntdef2 = 0

              do deltal = 1, 2

                contrst = 0d0
                covrst  = 0d0

                do epsl = 1, 2

                  gac = 0d0
                  gbc = 0d0

                  do i = 1, 2
                    gac = gac + cva(ip,jp,i,gammal) *
     +                          cva(ip,jp,i,epsl)
                    gbc = gbc + cva(ip,jp,i,deltal)  *
     +                          cva(ip,jp,i,epsl)
                  end do

                  contrst = contrst + invjac(ip,jp) *
     +                      invjac(ip,jp) *
     +                      ( -jctg(ip,jp,istwo(alphad,epsl)) *
     +                        pderiv(ip,jp,deltal,epsl) +
     +                        jctg(ip,jp,istwo(deltal,epsl)) *
     +                        pderiv(ip,jp,alphad,epsl) )

                  covrst = covrst + invjac(ip,jp) *
     +                     ( gac * pderiv(ip,jp,epsl,deltal) -
     +                       gbc * pderiv(ip,jp,epsl,gammal) )

                  do ii = 1, 2

                    contrst = contrst + invjac(ip,jp) *
     +                  invjac(ip,jp) *
     +                  ( -jctg(ip,jp,istwo(alphad,epsl)) *
     +                    ( chsym(ip,jp,deltal,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,deltal) ) +
     +                    jctg(ip,jp,istwo(deltal,epsl)) *
     +                    ( chsym(ip,jp,alphad,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,alphad) ) )

                    covrst = covrst + invjac(ip,jp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,epsl,istwo(ii,deltal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(deltal,ii))*
     +                      veloc(ip,jp,epsl) ) -
     +                    gbc *
     +                    ( chsym(ip,jp,epsl,istwo(ii,gammal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(gammal,ii))*
     +                      veloc(ip,jp,epsl) ) )

                  end do

                end do

                cntdef2 = cntdef2 + contrst*covrst

              end do

              cntdef1 = cntdef1 + cntdef2*invjac(ip,jp)*
     +                  jctg(ip,jp,istwo(gammal,betad))

            end do

            hightens(ip,jp) = hightens(ip,jp) + cntdef1*ctau3

            cntdef1 = 0

            do deltal = 1, 2
              do epsl = 1, 2

                contrst = 0d0
                covrst  = 0d0

                do jj = 1, 2

                  gac = 0d0
                  gbc = 0d0

                  do i = 1, 2
                    gac = gac + cva(ip,jp,i,deltal) *
     +                          cva(ip,jp,i,jj)
                    gbc = gbc + cva(ip,jp,i,epsl)  *
     +                          cva(ip,jp,i,jj)
                  end do

                  contrst = contrst + invjac(ip,jp) *
     +                      invjac(ip,jp) *
     +                      ( jctg(ip,jp,istwo(epsl,jj)) *
     +                        pderiv(ip,jp,deltal,jj) -
     +                        jctg(ip,jp,istwo(deltal,jj)) *
     +                        pderiv(ip,jp,epsl,jj) )

                  covrst = covrst + invjac(ip,jp) *
     +                     ( gac * pderiv(ip,jp,jj,epsl) -
     +                       gbc * pderiv(ip,jp,jj,deltal) )

                  do ii = 1, 2

                    contrst = contrst + invjac(ip,jp) *
     +                  invjac(ip,jp) *
     +                  ( jctg(ip,jp,istwo(epsl,jj)) *
     +                    ( chsym(ip,jp,deltal,istwo(ii,jj))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(jj,ii))*
     +                      veloc(ip,jp,deltal) ) -
     +                    jctg(ip,jp,istwo(deltal,jj)) *
     +                    ( chsym(ip,jp,epsl,istwo(ii,jj))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(jj,ii))*
     +                      veloc(ip,jp,epsl) ) )

                    covrst = covrst + invjac(ip,jp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,jj,istwo(ii,epsl))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(epsl,ii))*
     +                      veloc(ip,jp,jj) ) -
     +                    gbc *
     +                    ( chsym(ip,jp,jj,istwo(ii,deltal))*
     +                      veloc(ip,jp,ii) -
     +                      chsym(ip,jp,ii,istwo(deltal,ii))*
     +                      veloc(ip,jp,jj) ) )

                  end do

                end do

                cntdef1 = cntdef1 + contrst*covrst

              end do
            end do

            hightens(ip,jp) = hightens(ip,jp) - cntdef1*ctau3*
     +         invjac(ip,jp)*jctg(ip,jp,istwo(alphad,betad))/3

          end do
        end do

      else if ( idiscret.eq.2 ) then

c     --- Not yet implemented

      end if

      end
