      subroutine isstcr03(ni, nj, nk, ndim, ndegfd, ntimlv, nvirtual,
     +                    ntrnsp,  nturb, ntype, solut, norms)
c ======================================================================
c
c     programmer   Erik Brakkee
c
c
c     version      1.2  11-02-1997  Kees K. ntype added
c     version      1.1  16-09-1994  RvN Dummy version extended to
c                                equivalent of issctr02
c     version      1.0  13-7-1994  Erik:  initial revision
c
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute 2-norms and max-norms of the difference between old and new
c     solutions and compute the 2-norm of the new solution
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ndegfd, ndim, ni, nj, nk, ntimlv, nvirtual, ntrnsp, nturb
      integer ntype
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,
     +                 1-nvirtual:nk+1+nvirtual,
     +                 1:ndegfd, 1:ntimlv)
      double precision norms(2+ntrnsp+nturb, 2)

c     ndegfd     i  Number of degrees of freedom
c     ndim       i  Dimension of space (2 or 3)
c     ni         i  Number of non-virtual cells in the i-direction (one block)
c     nj         i  Number of non-virtual cells in the j-direction (one block)
c     nk         i  Number of non-virtual cells in the k-direction (one block)
c     norms      o  reserved
c     ntimlv     i  Number of time levels
c     ntrnsp     i  Number of transport equations
c     nturb      i  Number of extra transport equations for turbulence
c     ntype      i  Type of norm used for stopcriterium
c     nvirtual   i  Number of rows of virtual cells
c     solut      i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ieq, i, j, k

c     i      General loop variable
c     ieq    Equation number
c     j      General loop variable
c     k      General loop variable
c **********************************************************************
c
c                       I/O
c
c     None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     Compute norms for the velocity
c
c     Compute norms for the pressure
c
c     Do ieq = 3, 2+ntrnp+nturb
c
c        Compute norms for the transport/turbulence equation
c
c     Enddo
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      do ieq = 1, 2 + ntrnsp + nturb
         norms(ieq,1) = 0d0
         norms(ieq,2) = 0d0
      end do

      if ( ntype.eq.0 ) then

c     --- L2norm

c         --- Compute norms for the velocity

         ieq = 1

c        --- V1 component

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni
                  norms(ieq,1) = norms(ieq,1) +
     +                        5d-1*(solut(i,j,k,1,ntimlv)-
     +                        solut(i,j,k,1,ntimlv-1))**2 +
     +                        5d-1*(solut(i+1,j,k,1,ntimlv)-
     +                        solut(i+1,j,k,1,ntimlv-1))**2
                  norms(ieq,2) = norms(ieq,2) +
     +                        5d-1*solut(i,j,k,1,ntimlv)**2 +
     +                        5d-1*solut(i+1,j,k,1,ntimlv)**2
               end do
            end do
         end do

c        --- V2 component

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni
                  norms(ieq,1) = norms(ieq,1) +
     +                        5d-1*(solut(i,j,k,2,ntimlv)-
     +                        solut(i,j,k,2,ntimlv-1))**2 +
     +                        5d-1*(solut(i,j+1,k,2,ntimlv)-
     +                        solut(i,j+1,k,2,ntimlv-1))**2
                  norms(ieq,2) = norms(ieq,2) +
     +                        5d-1*solut(i,j,k,2,ntimlv)**2 +
     +                        5d-1*solut(i,j+1,k,2,ntimlv)**2
               end do
            end do
         end do

c        --- V3 component

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni
                  norms(ieq,1) = norms(ieq,1) +
     +                        5d-1*(solut(i,j,k,3,ntimlv)-
     +                        solut(i,j,k,3,ntimlv-1))**2 +
     +                        5d-1*(solut(i,j,k+1,3,ntimlv)-
     +                        solut(i,j,k+1,3,ntimlv-1))**2
                  norms(ieq,2) = norms(ieq,2) +
     +                        5d-1*solut(i,j,k,3,ntimlv)**2 +
     +                        5d-1*solut(i,j,k+1,3,ntimlv)**2
               end do
            end do
         end do

c        --- Compute norms for the pressure

         ieq = 2
         do k = 1, nk
            do j = 1, nj
               do i = 1, ni

c              --- Note: isstcr is called before isshift and therefore
c                  timelevel ntimlv-1 contains the pressure at the
c                  previous time level and not at the current time level.

                  norms(ieq,1) = norms(ieq,1) +
     +                        solut(i,j,k,ndim+1,ntimlv)**2
                  norms(ieq,2) = norms(ieq,2) +
     +                        (solut(i,j,k,ndim+1,ntimlv-1) +
     +                        solut(i,j,k,ndim+1,ntimlv))**2
               end do
            end do
         end do

c        --- Compute norms for the transport and turbulence equations

         do ieq = 3, 2 + ntrnsp + nturb

            do k = 1, nk
               do j = 1, nj
                  do i = 1, ni
                     norms(ieq,1) = norms(ieq,1) +
     +                           (solut(i,j,k,ieq+ndim-1,ntimlv) -
     +                            solut(i,j,k,ieq+ndim-1,ntimlv-1))**2
                     norms(ieq,2) = norms(ieq,2) +
     +                            solut(i,j,k,ieq+ndim-1,ntimlv)**2
                  end do
               end do
            end do

         end do

      else if ( ntype.eq.1 ) then

c     --- maxnorm

c         --- Compute norms for the velocity

      ieq = 1

c     --- V1 component

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni
                  norms(ieq,1) = max(norms(ieq,1),
     +                        5d-1*abs(solut(i,j,k,1,ntimlv) -
     +                        solut(i,j,k,1,ntimlv-1)) +
     +                        5d-1*abs(solut(i+1,j,k,1,ntimlv)-
     +                        solut(i+1,j,k,1,ntimlv-1)) )
                  norms(ieq,2) = max(norms(ieq,2),
     +                        5d-1*abs(solut(i,j,k,1,ntimlv)) +
     +                        5d-1*abs(solut(i+1,j,k,1,ntimlv)) )
               end do
            end do
         end do

c        --- V2 component

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni
                  norms(ieq,1) = max(norms(ieq,1),
     +                        5d-1*abs(solut(i,j,k,2,ntimlv)-
     +                        solut(i,j,k,2,ntimlv-1)) +
     +                        5d-1*abs(solut(i,j+1,k,2,ntimlv)-
     +                        solut(i,j+1,k,2,ntimlv-1)) )
                  norms(ieq,2) = max(norms(ieq,2),
     +                        5d-1*abs(solut(i,j,k,2,ntimlv)) +
     +                        5d-1*abs(solut(i,j+1,k,2,ntimlv)) )
               end do
            end do
         end do

c        --- V3 component

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni
                  norms(ieq,1) = max(norms(ieq,1),
     +                        5d-1*abs(solut(i,j,k,3,ntimlv)-
     +                        solut(i,j,k,3,ntimlv-1)) +
     +                        5d-1*abs(solut(i,j,k+1,3,ntimlv)-
     +                        solut(i,j,k+1,3,ntimlv-1)) )
                  norms(ieq,2) = max(norms(ieq,2),
     +                        5d-1*abs(solut(i,j,k,3,ntimlv)) +
     +                        5d-1*abs(solut(i,j,k+1,3,ntimlv)) )
               end do
            end do
         end do

c        --- Compute norms for the pressure

         ieq = 2
         do k = 1, nk
            do j = 1, nj
               do i = 1, ni

c              --- Note: isstcr is called before isshift and therefore
c                  timelevel ntimlv-1 contains the pressure at the
c                  previous time level and not at the current time level.

                  norms(ieq,1) = max(norms(ieq,1),
     +                        abs(solut(i,j,k,ndim+1,ntimlv)))
                  norms(ieq,2) = max( norms(ieq,2),
     +                        abs(solut(i,j,k,ndim+1,ntimlv-1) +
     +                        solut(i,j,k,ndim+1,ntimlv)) )

               end do
            end do
         end do

c        --- Compute norms for the transport and turbulence equations

         do ieq = 3, 2 + ntrnsp + nturb

            do k = 1, nk
               do j = 1, nj
                  do i = 1, ni
                     norms(ieq,1) = max(norms(ieq,1),
     +                           abs(solut(i,j,k,ieq+ndim-1,ntimlv) -
     +                           solut(i,j,k,ieq+ndim-1,ntimlv-1)))
                     norms(ieq,2) = max(norms(ieq,2),
     +                           abs(solut(i,j,k,ieq+ndim-1,ntimlv)) )
                  end do
               end do
            end do

         end do

      end if

      end
