      subroutine issyst ( iinput, rinput, matrix, gmat, rhside, coefs,
     +                    solut , coor  , infmt , infmg, geom, igeom,
     +                    infbnd, bndcon, work  ,
     +                    lwork, iequa, isiinput, issolut, isitopol )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      2.6   date    04-06-1998 Charles: bilinear interpolation
c     version      2.5   date    15-12-1997 Charles: extra arguments (gmat)
c     version      2.4   date    10-12-1997 Guus: remove ichois
c     version      2.3   date    05-11-1997 Charles: itimeact in itrans2d
c     version      2.2   date    13-10-1997 Charles: collocated allowed
c     version      2.1   date    01-05-1996 Marcel: update 3D
c     version      2.0   date    19-07-1995 Guus: new parameter list
c     version      1.4   date    30-05-1995 Hester: array eddy added
c     version      1.3   date    06-03-1995 Guus: correction prints
c     version      1.2   date    01-03-1995 Guus: incorporation prints
c     version      1.1   date    19-07-1994 Zijlema: extension of parameter
c                                                  list with variable iequa
c     version      1.0   date    16-12-1993
c
c
c   copyright (c) 1993-1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     create system of equations i.e. matrix and right-hand side of
c     transport equation
c **********************************************************************
c
c                       KEYWORDS
c
c     transport_matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), infmt(*), infmg(*), infbnd(*), lwork, iequa,
     +        isiinput(*), issolut(*), isitopol, igeom(*)
      double precision rinput(*), bndcon(*), matrix(*),
     +        rhside(*), coefs(*), solut(*), coor(*), geom(*) ,
     +        work(*), gmat(*)

c     bndcon    i    reserved
c     coefs    i/o   reserved
c     coor      i    reserved
c     geom      i    reserved
c     gmat      i    Gradient matrix
c     iequa     i    General equation sequence number
c     igeom     i    reserved
c     iinput    i    reserved
c     infbnd    i    reserved
c     infmg     i    Integer information for Pressure-gradient matrix
c     infmt     i    Integer information for Pressure and Transport matrix
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     lwork     i    Length of array work
c     matrix    o    reserved
c     rhside    o    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiindsc, ipiintim, iprinmt, iprinrhs, itime1,
     +        itime2, itimeact, itimest, ndim, ni, nj, nk, leniindsc,
     +        nvirtual, ndegfd, ipinvjac, ipbicof, ipibicof,
     +        iptsum, ipdtsum, ipwork, npointsv
      logical coupled

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     ipbicof   Starting address of array bicof in buffer
c     ipdtsum   Starting address of array dtsum in buffer
c     ipibicof  Starting address of array ibicof in ibuffr
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipinvjac  Starting address of array invjac in geom
c     iprinmt   Indicates whether the matrix must be printed or not
c               Possible values
c               0:  the matrix is not printed
c               1:  the standard matrix is printed
c               2:  the gradient matrix is printed
c               3:  the standard and the gradient matrix are printed
c               4:  the pressure matrix is printed
c               5:  the standard and the pressure matrix are printed
c               6:  the gradient and the pressure matrix are printed
c               7:  the standard, thegradient and the pressure matrix
c                   are printed
c     iprinrhs  Indicates whether the right-hand side must be printed or not
c               Possible values
c               0:  the right-hand side is not printed
c               1:  the standard right-hand side is printed
c               2:  the pressure right-hand side is printed
c               3:  the standard and the pressure  right-hand side are
c                   printed
c     iptsum    Starting address of array tsum in buffer
c     ipwork    Starting address of array work in buffer
c     itime1    First time at which the debug prints must be performed
c     itime2    Last time at which the debug prints must be performed
c     itimeact  Actual time step
c     itimest   Time step with respect to debug printing
c     leniindsc Length of first component of array iindsc
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISPRIMT1  Print 2D matrix
c     ISPRIMT2  Print 3D matrix
c     ISPRIRH1  Print 2D right-hand side
c     ISPRIRH2  Print 3D right-hand side
c     ISTRAN2D  Build matrix and right-hand side fro 2D transport equation
c     ISTRAN3D  Build matrix and right-hand side for 3D transport equation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Mark that tsum and dtsum use the same space since they are never used
c     together
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      ndim     = iinput(57)
      coupled  = iinput(76).eq.0
      npointsv  = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      if (ndim.eq.3) then
         npointsv = npointsv*(nk+1+2*nvirtual)
      end if
      ipiintim = iinput(4)
      ipiindsc = iinput(6)
      leniindsc = iinput(106)
      if ( coupled ) then
         ipiindsc = ipiindsc+leniindsc*(iequa-2)
      else
         ipiindsc = ipiindsc+leniindsc*(iequa-1)
      end if
      iprinmt  = iinput(ipiindsc+10)
      iprinrhs = iinput(ipiindsc+11)
      ipinvjac = npointsv*(2**ndim) + 1
      ipbicof  = ipinvjac + npointsv*(2**ndim)
      ipibicof = 1
      itime1   = iinput(ipiindsc+12)
      itime2   = iinput(ipiindsc+13)
      itimest  = iinput(ipiindsc+14)
      itimeact = iinput(ipiintim+3)

      iptsum  = 1
      ipdtsum = iptsum
      ipwork  = ipdtsum + ndim*npointsv
      lwork   = lwork-ipwork+1

      if ( ndim.eq.2 ) then

         call istran2d ( iinput, matrix, rhside, solut, coefs,
     +                   work(ipwork), geom(ipinvjac), rinput, npointsv,
     +                   infmt, work(iptsum), work(ipdtsum), coor,
     +                   infmt(infmt(11)), bndcon, infbnd, infbnd(13),
     +                   geom, iequa, ndegfd, lwork, gmat, infmg,
     +                   isiinput, issolut, isitopol, itimeact,
     +                   geom(ipbicof), igeom(ipibicof) )

      else

         call istran3d ( iinput, matrix, rhside, solut, coefs,
     +                   work(ipwork), geom(ipinvjac), rinput, npointsv,
     +                   infmt, work(iptsum), work(ipdtsum), coor,
     +                   infmt(infmt(11)), bndcon, infbnd(1), infbnd(7),
     +                   infbnd(31), geom, iequa, ndegfd, lwork )

      end if

      if ( iprinmt.ge.1 ) then

c     --- Print transport matrix

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then
               call isprimt1 ( 'transport matrix', matrix, ni, nj,
     +                         infmt(6), 2 )
            else
               call isprimt2 ( 'transport matrix', matrix, ni, nj, nk,
     +                         infmt(6), 2 )
            end if
         end if
      end if

      if ( iprinrhs.ge.1 ) then

c     --- Print transport right-hand side

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then
               call isprirh1 ( 'transport right-hand side', rhside,
     +                          ni, nj, 1 )
            else
               call isprirh2 ( 'transport right-hand side', rhside,
     +                          ni, nj, nk, 1 )
            end if
         end if

      end if

      end
