      subroutine iststep ( tact, istation, itimeact, tout, rout )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 5.1    date 20-01-2000 Duncan: put iterloc into cisnas
c        version 5.0    date 21-04-1999 Guus: complete revision
c        version 4.3    date 10-10-1999 KeesK: underscore removed from name
c        version 4.2    date 23-11-1998 Jason: removed isaction, isacth calls
c        version 4.1    date 03-11-1998 Guus: new call to issbstep
c        version 4.0    date 08-09-1998 Guus: complete revision
c        version 3.4    date 26-08-1998 Kees K: write nsubsteps removed
c        version 3.3    date 27-07-1998 Duncan: extension for iterative
c                                               solution within timestep
c                                               until convergence
c                                               indentation
c        version 3.2    date 14-05-1998 Duncan: extension for saving
c                                               coefficients dens upwind
c                                               new call to isacth
c                                               new call to isdd
c        version 3.1    date 27-03-1998 Kees K nsubsteps
c        version 3.0    date 13-11-1997 Fred.T new param.list,bug in output
c                                              time fixed, fluxes corrected
c                                              at first time step,before
c                                              the momentum eq.is solved
c        version 2.1    date 13-10-1997 Charles: small correction
c        version 2.0    date 10-09-1997 Guus: extra parameter icoupled, ndim
c        version 1.2    date 06-08-1997 Guus: Skip coefficients for pressure
c        version 1.1    date 11-06-1997 Guus: irefwr
c        version 1.0    date 04-06-1997
c
c   copyright (c) 1996-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Perform one time step
c     In this time-step various substeps are possible due to:
c     - generalized theta method (several values of theta)
c     - extrapolation per theta step
c     - iteration per substep in order to get a more implicit formulation
c     - substepping of the turbulence equations to prevent negative values
c       of k and eps
c **********************************************************************
c
c                       KEYWORDS
c
c     time_integration
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer istation, itimeact
      double precision tact, rout, tout

c     istation i/o   Choice whether a stationary solution is needed
c     itimeact i/o   Actual time step
c     rout     i/o   Indicates the next time output should be performed
c     tact     i/o   As t, the actual time, but not in common block
c     tout     i/o   Indicates the next time output should be performed
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisglob'
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ifrac, ipiinput, iprinput, eq, iclust, ieqn, igeotime,
     +        neq, nfrac, i,  methodtim, iseqequa(100), ipiclust,
     +        ieq, ipiseqeq, restartw, cartesian, nsubsteps,
     +        ipiintim, iterstep, ilocstr(20), ipiinfrs,
     +        ipiingls, ipiinlos, ipstart, istatloc, isubstep,
     +        maxitloc, minitloc, nclusters, numeqn, iprinlos,
     +        iprinfrs
      double precision t0, tend, dt, thetarr(2,5), theta, toutin,
     +                 touten, toutst, eps, dtheta,
     +                 rinit, rend, rstep, tsave, rlocstr(10)
     +
      logical globconv, turbonly, locconv, inititer, freesurf

c     cartesian Cartesian=1, write cartesian values in the vertices to
c               the restart file
c               cartesian=0, dump contravariant solution vector
c     dt        Time step
c     dtheta    dt * theta
c     eps       Accuracy with respect to the time parameter tout
c     eq        Counting variable for which equation
c     freesurf  If true information about the free surface has been read
c     globconv  If true global convergence has been reached
c     i         General loop variable
c     iclust    Cluster sequence number
c     ieq       Equation number
c     ieqn      Equation sequence number
c     ifrac     Actual fractional time step
c     igeotime  If 1 a free surface model is used, if 0 not
c     ilocstr   Contains information of the structure of the main program
c               for each cluster
c               See the programmers Guide for a description
c     inititer  If true the initial condition has been converged
c     ipiclust  Starting address of array ICLUSTER in array IINPUT
c     ipiinfrs  Starting address of array IINFRSUR in array IINPUT
c     ipiingls  Starting address of array IGLOBSTR in array IINPUT
c     ipiinlos  Starting address of array ILOCSTR in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipiseqeq  Starting address of array ISEQEQ in array IINPUT
c     iprinlos  Starting address of array RLOCSTR in array RINPUT
c     iprinput  Starting address of array rinput in buffer
c     ipstart   Starting address
c     iseqequa  Array of length 100 containing the inverse of array iseqeq
c     istatloc  Defines if a cluster must solved as a local stationary
c               problem (1) or not (0)
c     isubstep  Substep sequence number
c     iterstep  Iteration step sequence number
c     locconv   If true local convergence has been reached
c     maxblocki Maximum number of blocks
c     maxitloc  Maximum number of local iterations
c     maxneq    Maximum number of equations allowed
c     methodtim Type of time integration
c     minitloc  Minimum number of local iterations
c     nclusters Number of clusters
c     neq       Number of equations
c     nfrac     Number of fractional time steps
c     nsubsteps Number of sub-timesteps
c     numeqn    Number of equations in sub-step
c     rend      End time for writing to restart file
c     restartw  If restartw > 0, write restart file
c     rinit     Start time for writing solutions to the restart file
c     rlocstr   Contains information of the structure of the main program
c               for each cluster
c               See the programmers Guide for a description
c     rstep     Time step for writing to restart file
c     t0        Initial time
c     tend      End time
c     theta     Parameter theta for theta method
c     thetarr   Array of thetas for different theta methods
c     touten    End time for output of results
c     toutin    Initial time for output of results
c     toutst    Time step for output of results
c     tsave     Save variable for time
c     turbonly  Shift only for turbulence equations
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISCONGLB  Check global convergence
c     ISCONLOC  Check local convergence
c     ISCOPSTR  Copy part of structure arrays for one cluster
c     ISCORN    Correct normal fluxes
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISFILG    Fill geometrical quantities
c     ISINGETP  get the pointer corresponding to an integer array
c     ISNDT01   Put time-parameters in array rinput
c     ISNDT02   Get dt and test on stationary solution
c     ISOUT     Write solution arrays to output-file
c     ISREFINE  Write restart file with cartesian values in the vertices
c     ISREFLHS  Receive flow field on the host
c     ISRESTF   Write restart file
c     ISSBSTEP  Perform actual actions in one time step
c     ISSHIFT   Copy soloution time level 2 to time level 1
c     ISSNFLND  Send field from node to host
c     ISSTCR    Compute and print the stopping criterium
c     ISSYNC    Ensure value agrees on all nodes
c     ISTIMI01  Get number of equations and time integration values
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1013   No convergence in the iterative pressure correction algorithm
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Since the program is made for parallel as well as serial solution
c     Building of equations, solving and so on is not done directly in this
c     subroutine but in a complex way by calling subroutine isacth
c     This subroutine decides which action must be performed using array
c     iaction. Furthermore this subroutine sends information to nodes
c     in case of parallel computations
c
c     For each time step a number of equations will be solved separately
c     (i.e. decoupled)
c     The sequence of the equations may be given by the user
c     If the user did not give this sequence then the standard sequence is
c     as follows:
c     Incompressible case:
c        Solve velocity prediction
c        Solve pressure correction and adapt velocity and pressure
c        Solve transport equations (including turbulence equations)
c     Compressible case:
c        Solve energy equation
c        Solve velocity prediction
c        Solve pressure correction and adapt velocity and pressure
c        Solve transport equations (including turbulence equations)
c
c     Solve an equation consists of the following steps:
c        Compute all coefficients
c        Build system of equations
c        Solve linear system of equations
c
c     With respect to the velocity prediction we have to distinguish between
c     the coupled case and the decoupled case
c     In the coupled case we build one system of equations for the
c     momentum equations
c     The number of equations is in that case equal to 2 + number of transport
c     equations
c     In the decoupled case each velocity component is treated as a
c     separate unknown and therefore there is a separate transport equation
c     for each of the components.
c
c     The method applied for the solution velocity and pressure is
c     pressure-correction.
c
c     This method is based on the following steps:
c
c     -  Predict the velocity using the pressure at the old time level
c     -  Compute the pressure correction in order to satisfy the continuity
c        equation
c     -  Update the velocity and the pressure
c
c     With respect to the muliblock process it may be necessary to update
c     the normal components at the interfaces of the blocks in order to get
c     a uniquely defined velocity. This process will be called adpat fluxes.
c
c     read parmaters neq, t0, tend and dt from arrays iinput and rinput
c     for time := t0 (dt) tend
c        for eq  := 1  (1)  neq
c           perform one parallel domain decomposition step
c           if ( momentum equation ) then
c              adapt fluxes
c           if ( pressure equation ) then
c              update pressure and velocity
c           if ( eq=neq and compressible ) then
c              compute density
c        if (time = tout) then
c           perform output
c
c   The substepping algorithm is applied in the following way:
c
c   X--o--o--o--o--o--o--o--o--o--o--o--o--o--o--o--o--o--X--o--o-
c   t^n                                                   t^n+theta
c
c   u^n
c   p^n
c   Tr^n
c   Tu^n
c
c   At time t^n all quantities are known
c   First all quantities except the turbulence quantities are computed
c   at t^n+theta
c   Then all turbulent quantities are computed in the substeps using
c   the new computed velocities at t^n+theta
c   The substeps are performed with theta_sub = 1
c   Although this is not the most accurate method it is the simplest one for
c   implementation purposes
c   Once all quantities are known at t^n+theta everything is extrapolated
c   to t^n+1
c
c   The general algorithm can be written as:
c
c   nfrac := number of fractional steps
c   ncluster := number of clusters
c   For ifrac := 1 (1) nfrac do
c      While ( global iteration required ) do
c         if ( free surface and not initial iteration ) then
c            Compute new geometrical quantities
c         For all clusters of equations do
c            For all substeps of the cluster do
c               While ( local iteration required ) do
c                  Solve the set of equations
c                  Check convergence
c               If ( number of substeps > 1 ) then
c                  Copy v^n := v^(n+1)
c         Adapt coordinates if necessary
c         Check on global convergence
c      If ( 0 < theta < 1 ) then
c         Extrapolate solution
c      For all clusters of equations do
c         if ( number of substeps = 1 ) then
c            Copy v^n := v^(n+1)
c   If necessary check if stationary state has been reached
c   Adapt initial iteration if necessary
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('iststep')

      call isactive(isiinput(1))
      call isactive(isrinput(1))

      ipiinput = isingetp( isiinput(1) )
      iprinput = isdpgetp( isrinput(1) )

      ipiseqeq = ibuffr( ipiinput -1 + 10 )
      ipiintim = ibuffr( ipiinput -1 + 4 )
      ipiinfrs = ibuffr( ipiinput -1 + 12 )
      iprinfrs = ibuffr( ipiinput -1 + 36 )
      ipiclust = ibuffr( ipiinput -1 + 13 )
      ipiingls = ibuffr( ipiinput -1 + 14 )
      ipiinlos = ibuffr( ipiinput -1 + 15 )
      iprinlos = ibuffr( ipiinput -1 + 38 )
      igeotime = ibuffr( ipiinput -1 + 80 )
      nclusters = ibuffr( ipiinput -1 + 81 )
      inititer = ibuffr( ipiinput -1 + ipiingls-1 +3).gt.0
      freesurf = ibuffr( ipiinput -1 + ipiingls-1 +2).gt.0


c     --- indicate whether the momentum and thus pressure equation is
c         skipped or not
c         initialize thetarr
c         get number of equations and time parameters

      call istimi01( ibuffr(ipiinput), buffer(iprinput),
     +               neq, t0, tend,
     +               dt, thetarr, toutin, touten, toutst,
     +               rinit, rend, rstep, methodtim, nfrac, restartw,
     +               cartesian, nsubsteps )

      if ( nerror.ne.0 ) go to 1000

      eps   = 1d-2 * dt

      if ( itimeact.eq.0 ) then

c     --- Initialize tout and rout

         tout  = toutin
         rout  = rinit

      end if

c     --- get the values dt and istation

      call isndt02 ( ibuffr(ipiinput), buffer(iprinput), t, dt,
     +               toutst, rstep, istation )

c     --- Fill array iseqequa with the inverse sequence of the equations

      do i = 1, neq
         iseqequa(i) = 0
      end do
      do i = 1, neq
         ieq = ibuffr(ipiinput+ipiseqeq-1+i-1)
         if ( ieq.gt.0 ) iseqequa(ieq) = i
      end do

      if ( nerror.ne.0 ) go to 1000

      if ( itimeact.eq.0 ) then

c     --- adapt fluxes

         call iscorn ( iadmin, isiinput, issolut, isitopol )

         if ( nerror.ne.0 ) go to 1000

      end if

      if ( t + eps.gt.tout ) tout = tout + toutst

c     --- Do loop started here, here time step starts

      itimeact = itimeact + 1
      tact = t + dt

      if (inodenr.eq.ISHOST) write(irefwr,1001) itimeact, t + dt

c     --- For ifrac := 1 (1) nfrac do

      do ifrac = 1, nfrac

c     --- First the local theta, dtheta and t are computed

         if ( thetarr(2,ifrac).gt.0d0 ) then

c        --- normal (1), fractional (3), or generalized (4) theta method

            dtheta = thetarr(2,ifrac) * dt
            theta  = thetarr(2,ifrac) /
     +              (thetarr(2,ifrac)+thetarr(1,ifrac))

         else

            theta = 0
            dtheta = dt

         end if

c        --- Temporarily adapt t, dt, and theta for (fractional) theta
c            method

         tsave = t

c        --- While ( global iteration required ) do

         globconv = .false.
         iterstep = 0
         do while ( .not. globconv )

            iterstep = iterstep+1
            if ( freesurf .and. .not. inititer ) then

c           --- free surface and not initial iteration
c               Compute new geometrical quantities

               call isfilg ( nblocks, isiinput, isrinput, iscoor,
     +                       isgeom, isigeom, iadmin )

            end if

c           --- For all clusters of equations do

            do iclust = 1, nclusters

c           --- For all substeps for the cluster do
c               Get parameters for cluster and put in local arrays ilocstr,
c               rlocstr

               call isactive(isiinput(1))
               call isactive(isrinput(1))
               ipiinput = isingetp( isiinput(1) )
               iprinput = isdpgetp( isrinput(1) )
               call iscopstr ( iclust, ibuffr(ipiinput-1+ipiinlos),
     +                         buffer(iprinput-1+iprinlos), ilocstr,
     +                         rlocstr, nclusters )
               istatloc  = ilocstr(1)
               minitloc  = ilocstr(2)
               maxitloc  = ilocstr(3)

c              --- For all substeps of the cluster do

               nsubsteps = ilocstr(5)
               do isubstep = 1, nsubsteps

c              --- put time, dt (=dtheta), and theta into array rinput

                  if ( nsubsteps.gt.1 ) dtheta = dt/nsubsteps
                  t = tsave + isubstep * dtheta
                  call isndt01 ( nblocks, isiinput, isrinput, iadmin, t,
     +                           dtheta, theta, itimeact )
                  if ( nerror.ne.0 ) go to 1000

c                 --- While ( local iteration required ) do

                  locconv = .false.
                  iterloc = 0
                  do while ( .not. locconv )

                     iterloc = iterloc+1
                     call isactive(isiinput(1))
                     ipiinput = isingetp( isiinput(1) )
                     ipstart = ibuffr(ipiinput-1+ipiclust-1+iclust)
                     ipstart = ipiinput-1+ipiclust-1+ipstart

c                    --- Loop over the equations in the cluster

                     numeqn = ibuffr(ipstart)
                     do ieqn = 1, numeqn

c                    --- find equation number

                        ieq = ibuffr(ipstart+ieqn)

c                       --- Since the sequence of the equations may be adapted
c                           by the user the new sequence must be found from
c                           iseqeq. The sequence number is stored in eq.

                        eq = iseqequa(ieq)

                        if ( eq.ne.0 ) then

c                       --- eq # 0, solve the set of equations

                           call issbstep ( eq, theta, nblocks, isiinput,
     +                                     isrinput, iscoor, issolut,
     +                                     iscoefs, iseddy, isinfbnd,
     +                                     isbndcon, ismatrix, isgmat,
     +                                     isrhside, isgeom, isitopol,
     +                                     iadmin, isigeom, isaverstr,
     +                                     isdensup )

                        end if

                     end do

c                    --- Check local convergence

                     locconv = istatloc.eq.0
                     if ( minitloc.le.iterloc .and. .not. locconv ) then

c                    --- A local iteration is required

                        call isconloc ( nblocks, iadmin, issolut,
     +                                  isiinput, isrinput, locconv,
     +                                  ilocstr, rlocstr )

                     end if
                     if ( .not. locconv .and. iterloc.gt.maxitloc ) then

c                    --- The maximum number of local iterations has been
c                        reached, but no convergence

                        locconv = .true.
                        call iserrint ( iclust, 1 )
                        call iserrint ( iterloc, 2 )
                        call iserrsub ( 1013, 2, 0, 0 )
                        go to 1000

                     end if

c                    --- end of while loop local convergence

                  end do

                  if ( isubstep.lt.nsubsteps ) then

c                 --- number of substeps > 1
c                     Copy v^n := v^(n+1)

                     turbonly = .true.
                     call isshift ( iadmin, isiinput, isrinput,
     +                              issolut, iseddy, iscoefs, turbonly )

                     if ( nerror.ne.0 ) go to 1000

                  end if

c                 --- end loop over substeps

               end do

c              --- end loop over clusters

            end do

c           --- Adapt coordinates if necessary

            if ( freesurf .and. .not. inititer ) then

c           --- free surface and not initial iteration

               call isfrsf( nblocks, isiinput, issolut, iscoor,
     +                    iadmin, ibuffr(ipiinput -1 + ipiinfrs),
     +                    buffer(iprinput -1 + iprinfrs),
     +                    dtheta )


            end if

c           --- Check on global convergence

            call isconglb ( issolut, isiinput, isrinput, globconv )

c           --- end of while loop global convergence

         end do

c        --- If ( 0 < theta < 1 ) then
c               Extrapolate solution
c            For all clusters of equations do
c               if ( number of substeps = 1 ) then
c                  Copy v^n := v^(n+1)
c            This part must also be implemented

c              --- print stopping criterion for stationary solution

         call isstcr ( nblocks, iadmin, isiinput, isrinput, issolut )
         turbonly = .false.
         call isshift( iadmin, isiinput, isrinput, issolut, iseddy,
     +                 iscoefs, turbonly )

c        --- Correct t for (fractional/generalized) Theta method

         if ( thetarr(2,ifrac).gt.0d0 ) then

             t = t + thetarr(1,ifrac)*dt

         end if

c        --- Put dt back to original value in array rinput
c            if (generalized/fractional) theta method was applied

         call isndt01 ( nblocks, isiinput,
     +                  isrinput, iadmin, t, dt, theta, itimeact )
         if ( nerror.ne.0 ) go to 1000

c        --- end of loop over fractional steps

      end do

c     --- get the values dt, toutst, and istation

      call isndt02 ( ibuffr(ipiinput), buffer(iprinput), t, dt,
     +               toutst, rstep, istation             )
      if ( iacnodes.gt.0 ) call issync ( istation, 1, ISINT, ISHOST )

      if ( nerror.ne.0 ) go to 1000

      if ( t+eps.gt.tout  .and. t+eps.le.touten   .or.
     +     t-eps.gt.tout  .and. t+eps.le.touten   .or.
     +     istation.eq.-1 .or.  t+eps.gt.tend   ) then

c     --- perform output

c         --- get the entire flow field

         if ( inodenr.ne.ISHOST ) then

c        --- send field if not on the host

            call issnflnd( nblocks, isiinput, issolut, iadmin )

         else

c        --- receive field on the host

            call isreflhs(nblocks, isiinput, issolut, iadmin)

         end if

c        --- write output

         if (inodenr.eq.ISHOST)
     +       call isout( t, isiinput, isrinput, iscoor, issolut,
     +               isitopol, isgeom, iseddy, indorien )
         if ( nerror.ne.0 ) go to 1000

         tout = tout + toutst

      end if

      if ( restartw.ge.0 ) then

         if ( t+eps.gt.rout  .and. t+eps.le.rend   .or.
     +        t-eps.gt.rout  .and. t+eps.le.rend   .or.
     +        istation.eq.-1 .or.  t+eps.gt.tend   ) then

c        --- perform output
c            get the entire flow field

            if ( inodenr.ne.ISHOST ) then

c           --- send field if not on the host

               call issnflnd ( nblocks, isiinput, issolut, iadmin )

            else

c           --- receive field on the host

               call isreflhs ( nblocks, isiinput, issolut, iadmin )

            end if

c           --- write restart file

            if ( inodenr.eq.ISHOST ) then

               if ( cartesian.eq.1 ) then

                  call isrefine ( t, isiinput, isrinput, iscoor,
     +                            issolut, isitopol, isoutpg )
               else

                  call isrestf ( t, isiinput, isrinput, isitopol,
     +                           issolut )

               end if
               if ( nerror.ne.0 ) go to 1000

            end if

            rout = rout + rstep

         end if

      end if

c     --- If a stationary solution has been requested and found, the
c         time integration will be ended

      if ( istation.eq.-1 .and. inodenr.eq.ISHOST )
     +     write(irefwr,1002) t

c     --- End of Time step

1000  call iserclos('iststep')
      if ( nerror.ne.0 ) istation = -1
1001  format(//' TIME STEP ', I4, '    (t = ', d12.3, ')'// )
1002  format(//' A stationary solution was found at time (t = ',
     +          F6.2,')'//)

      end
