      subroutine istuvisc ( mturb , ndegfd  , ntimlv, rintur,
     +                      ndim  , ni      , nj    , nk    ,
     +                      npoint, nvirtual, solut , eddy  ,
     +                      rincof, viscorr , work, lwork, coor,
     +                      geom, ichois,idiscret , igeom,molvis,
     +                      rho,ntrnsp,nturb,iinbc,rinbc, itimeact,
     +                      averstr, lenaver,nwinsta,nwinend,timewind,
     +                      nwindowmx, deltat )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.2    date   19-08-1997 Fred T: parameter list changed
c        version 2.1    date   10-07-1996 Marcel: parameter list changed
c        version 2.0    date   29-11-1994 Zijlema: call istuvisc now
c                                                  depends on dimension
c        version 1.0    date   21-09-1994
c
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computes eddy viscosity, which depends on the choice
c     of a turbulence model
c **********************************************************************
c
c                       KEYWORDS
c
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer mturb, ndegfd, ni, nj, nk, ndim, ntimlv, nvirtual, npoint,
     +        viscorr,lwork, ichois,idiscret, igeom(*), ntrnsp,nturb,
     +        iinbc(*), itimeact, lenaver, nwinsta, nwindowmx, nwinend
      double precision rintur(*), rincof(*), eddy(1:npoint), solut(*),
     +                 work(*),coor(*),geom(*), molvis, rho,rinbc(*),
     +                 averstr(*), timewind, deltat

c     averstr   i    reserved
c     coor      i    reserved
c     deltat    i    Product of theta and dt
c     eddy      o    reserved
c     geom      i    reserved
c     ichois    i    General choice parameter
c     idiscret  i    Type of discretization
c     igeom     i    Integer array that returns the closest cell on a wall
c     iinbc     i    reserved
c     itimeact  i    Actual time step
c     lenaver   i    Help parameter for the length of array averstr
c     lwork     i    Length of array work
c     molvis    i    Molecular viscosity
c     mturb     i    Indicates type of turbulence modelling
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npoint    i    Number of points for one block
c     ntimlv    i    Number of time levels
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     nwindowmx i    Maximum size of the window to be used for time-averaging
c     nwinend   i    End position of time window
c     nwinsta   i    Starting position of time window
c     rho       i    Estimate of the asymptotic convergence rate
c     rinbc     i    reserved
c     rincof    i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     timewind  i    Time interval for time window
c     viscorr   i    Indicate whether to use viscous corrections or not
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer j

c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISTUVSC2      fill eddy-viscosity in 2D case
c     ISTUVSC3      fill eddy-viscosity in 3D case
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- First, initialize array eddy

      do j = 1, npoint
        eddy(j) = 0d0
      end do

      if ( ndim.eq.2 ) then

        call istuvsc2( mturb, ndegfd, ntimlv, rintur, ni, nj,
     +                 nvirtual, solut, eddy, rincof, viscorr,
     +                 geom(4*npoint*2+1) )

      else if ( ndim.eq.3 ) then

        call istuvsc3( mturb, ndegfd, ntimlv, rintur, ni, nj, nk,
     +                 nvirtual, solut, eddy, work, lwork, coor,
     +                 geom, ichois,idiscret,npoint, igeom, molvis,
     +                 rho, ntrnsp,nturb,iinbc,rinbc,itimeact,
     +                 averstr, lenaver, nwinsta,nwinend, timewind,
     +                 nwindowmx, deltat )

      end if

      end
