      subroutine istuvsc2( mturb   , ndegfd, ntimlv, rintur, ni, nj,
     +                     nvirtual, solut , eddy, rincof, viscorr,
     +                     wdis  )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.1    date   20-08-1998  Kees K. keep k positiv 
c        version 2.0    date   22-09-1997  Fred T. adaptation parameter list
c        version 1.2    date   10-07-1996  Marcel Lam-Bremhorst model
c        version 1.1    date   24-05-1996  Marcel extended k-eps model
c        version 1.0    date   29-11-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute eddy viscosity, which depends on the choice
c     of a turbulence model (2D)
c **********************************************************************
c
c                       KEYWORDS
c
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer mturb, ndegfd, ntimlv, nvirtual, ni, nj, viscorr
      double precision rintur(*), rincof(*),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv),
     +                 wdis(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual)

c     eddy      o    reserved
c     mturb     i    Indicates type of turbulence modelling
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     rincof    i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     viscorr   i    Indicate whether to use viscous corrections or not
c     wdis      i    Array containing the wall distance
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp
      double precision rhow, mu, ry, ret, fmu
      
c     fmu       Viscous damping function
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     mu        The laminar viscosity
c     ret       Turbulent Reynolds number
c     rhow      Specific weight
c     ry        Local Reynolds number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( mturb.eq.0 ) then

c     --- No turbulence model

        do jp = 1, nj
          do ip = 1, ni
            eddy(ip,jp) = 0d0
          end do
        end do

      else if ( mturb.eq.1 .or. mturb.eq.2 .or. mturb.eq.5 ) then

c     --- Standard k-eps model or RNG k-eps model or extended k-eps model

        if ( viscorr.eq.0 ) then

c       --- No low-Reynolds-number corrections

          do jp = 1, nj
            do ip = 1, ni
              solut(ip,jp,ndegfd-1,1) = max (solut(ip,jp,ndegfd-1,1),
     +                                       1d-6)
            end do
          end do

          do jp = 1, nj
            do ip = 1, ni
              eddy(ip,jp) =  rintur(3) * solut(ip,jp,ndegfd-1,1) *
     +                        solut(ip,jp,ndegfd-1,1) /
     +                        solut(ip,jp,ndegfd,1)
            end do
          end do

        else if ( viscorr.eq.1 ) then

c       --- Use Lam-Bremhorst near-wall modeling

          do jp = 1, nj
            do ip = 1, ni

              rhow = rincof(1)
              mu = rincof(2)
              ry = rhow*sqrt(solut(ip,jp,ndegfd-1,1))*wdis(ip,jp)/mu
              ret = rhow*(solut(ip,jp,ndegfd-1,1)**2)/
     +              ( solut(ip,jp,ndegfd,1)*mu )

              if ( ry.gt.1d0 ) then
                if ( ry.gt.3d3 ) then
                  fmu = 1d0+2.05d1/ret
                else
                  fmu = (1d0-exp(-1.65d-2*ry))**2*(1d0+2.05d1/ret)
                end if
              else
                fmu = (1.65d-2*ry)**2*(1d0+2.05d1/ret)
              end if
              if (ret.lt.1d0.and.fmu.gt.1d0) fmu = 1.116225d-2
              if (fmu.lt.1.116225d-2) fmu = 1.116225d-2

              eddy(ip,jp) = fmu*rintur(3) * solut(ip,jp,ndegfd-1,1) *
     +                        solut(ip,jp,ndegfd-1,1) /
     +                        solut(ip,jp,ndegfd,1)

            end do
          end do

        end if

      else if ( mturb.eq.3 ) then

c     --- Wilcox k-omega model

        do jp = 1, nj
          do ip = 1, ni
            eddy(ip,jp) = solut(ip,jp,ndegfd-1,1) /
     +                      solut(ip,jp,ndegfd,1)
            end do
        end do

      else if ( mturb.eq.4 ) then

c     --- LES Smargorinsky model. Not yet implemented !!!

      end if

      end
