      subroutine isupm2d(matrix, iinput, rinput, jac, pold, dpold,
     +           ni, nj, nvirtual, ichoiceg, densup, rhowork, drhodp,
     +                                          rhoint1, pinterm1 )
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      1.2    date 10-12-1999    Duncan: extend with virtual cells
c                                                    for multiblock
c     version      1.1    date 23-03-1999    Duncan: inclusion boundary points
c     version      1.0    date 21-11-1998
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c  subroutine ISUPM2d, removes and adds the compressible part of the
c  pressure matrix in case of accelerated multiblock.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iinput(*), ichoiceg
      double precision jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 matrix(ni+1,nj+1,*), rinput(*),
     +                 pold(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 dpold(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 densup(ni+1,nj+1,3,2),
     +                 rhowork(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 drhodp(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 rhoint1(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 pinterm1(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual)

c     densup    i    coefficients of density upwind in pressure matrix
c     dpold     i    pressure correction at previous time step
c     ichoiceg  i    Choice parameter update matrix and rhside
c     iinput    i    reserved
c     jac       i    sqrt(g) in pnttyp
c     matrix   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pinterm1  i    Work array for pressure
c     pold      i    Pressure at the old time level
c     rhoint1   i    Work array for density and density increment
c     rhowork   i    Work array for density
c     rinput    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iprincom, mcomupw, mscal, i, j, istartup, iprincav,
     +               npointsv
      double precision epssc, dt, deltatin, mnm, updat, resto

c     deltatin  Inverse of deltat, i.e. 1/deltat
c     drhodp    drho/dp
c     dt        Time step
c     epssc     Parameter in the secant type estimate for the drhodp derivative
c     i         General loop variable
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     istartup  If 1 the cavity calculation is in the startup procedure,
c               if 0 not
c     j         General loop variable
c     mcomupw   If zero the rho equation for compressible flow is solved by
c               a central scheme, otherwise by a upwind scheme
c     mnm       General help variable
c               a central scheme, otherwise by a upwind scheme
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     npointsv  Number of points in the mesh with virtual points
c     resto     General help paremeter
c     updat     General help parameter
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isstdls, isstdfl

c     DAXPY      Vector a*X + Y
c     DCOPY      Copy double precision array into other one (BLAS subroutine)
c     ISSTDFL    dimensionfull equation of state
c     ISSTDFL2   dimensionfull equation of state vector form
c     ISSTDLS    dimensionless equation of state
c     ISSTDLS2   dimensionless equation of state vector form
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c   In case of update do
c   for all non-virtual points do
c                                  __    drhodp
c   matrix(i,j,1)=matrix(i,j,1) + V g *  ------
c                                          dt
c   end do
c   in case of restore do
c   for all non-virtual points do
c                                  __    drhodp
c   matrix(i,j,1)=matrix(i,j,1) - V g *  ------
c                                           dt
c   end do
c   for density upwind do some more
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      updat    =  1d0
      resto    = -1d0

      if (ichoiceg.eq.1) then

          mnm  = updat

      else if (ichoiceg.eq.-1) then

          mnm  = resto

      end if

      dt       = rinput(iinput(29)-1 +4)
      deltatin = 1d0/dt
      iprincom = iinput(34)
      iprincav = iinput(35)
      mcomupw  = iinput(iinput(9)+1)
      mscal    = iinput(iinput(9)+4)
      epssc    = 1d-4
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      istartup = iinput(iinput(11)-1 +2)

c     --- calculate intermediate pressure pold + dpold  = pinterm1

      call dcopy(npointsv, pold, 1, pinterm1, 1)
      call daxpy(npointsv, 1d0, dpold, 1, pinterm1, 1)

c     --- calculate the intermediate density rhoint1=rho(pold + dpiter)

      if( mscal.eq.0) then
         call isstdls2(pold, rinput(iprincom), istartup,
     +                           rhowork, npointsv, rinput(iprincav))
         call isstdls2(pinterm1, rinput(iprincom), istartup,
     +                           rhoint1, npointsv, rinput(iprincav))
      else if( mscal.eq.1) then
         call isstdfl2(pold, istartup, rhowork, npointsv,
     +                                           rinput(iprincav))
         call isstdfl2(pinterm1, istartup, rhoint1, npointsv,
     +                                           rinput(iprincav))
      end if

c     --- calculate the increment in density, store in rhoint1

      call daxpy(npointsv, -1d0, rhowork, 1, rhoint1, 1)

c     --- nondimensional computation

      if( mscal.eq.0) then

         do j = 1-nvirtual, nj+1+nvirtual
           do i = 1-nvirtual, ni+1+nvirtual

               if(abs(dpold(i,j)).gt.1d-8) then

                 drhodp(i,j)= rhoint1(i,j)/dpold(i,j)
     +
               else

                 drhodp(i,j)=
     +                   (isstdls(pold(i,j)+epssc,
     +                       rinput(iprincom), istartup,
     +                         rinput(iprincav))-
     +                       rhowork(i,j))/
     +                     (epssc)

               end if
          end do
         end do

      else if( mscal.eq.1) then

c     --- dimensional computation                          --

         do j = 1-nvirtual, nj+1+nvirtual
           do i = 1-nvirtual, ni+1+nvirtual

               if(abs(dpold(i,j)).gt.epssc) then

                 drhodp(i,j)= rhoint1(i,j)/dpold(i,j)

               else

                 drhodp(i,j)=
     +                   (isstdfl(pold(i,j)+epssc,
     +                       istartup, rinput(iprincav))-
     +                       rhowork(i,j))/
     +                     (epssc)

               end if
          end do
         end do

      end if

c     --- Loop over all connections in the matrix

      do j=1,nj
         do i=1,ni
              matrix(i,j,1) = matrix(i,j,1)
     +                                +   mnm*
     +        drhodp(i,j)*jac(i,j)*deltatin
         end do
      end do

      if( mcomupw.eq.1) then

        do j=1,nj
           do i=1,ni
              matrix(i,j,1) =matrix(i,j,1)+mnm*
     +                  drhodp(i,j)*densup(i,j,2,1)
           end do
        end do

        do j=1,nj
           do i=1,ni
              matrix(i,j,1) =matrix(i,j,1)+mnm*
     +                  drhodp(i,j)*densup(i,j,2,2)
           end do
        end do

        do j=1,nj
           do i=1,ni
              matrix(i,j,3) =matrix(i,j,3)+ mnm*
     +                  drhodp(i,j-1)*densup(i,j,1,2)
           end do
        end do

        do j=1,nj
           do i=1,ni
              matrix(i,j,5) =matrix(i,j,5)+ mnm*
     +                  drhodp(i-1,j)*densup(i,j,1,1)
           end do
        end do

        do j=1,nj
           do i=1,ni
              matrix(i,j,6) =matrix(i,j,6)+ mnm*
     +                  drhodp(i+1,j)*densup(i,j,3,1)
           end do
        end do

        do j=1,nj
           do i=1,ni
              matrix(i,j,8) =matrix(i,j,8)+ mnm*
     +                  drhodp(i,j+1)*densup(i,j,3,2)
           end do
        end do

      end if

      end
