      subroutine isaxism ( ni, nj, nvirtual, matrix, coor, mu,
     +                      alphad, adir, cva )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 22-03-1999 upper boundary jp=nj+1
c        version 1.0    date 11-01-1999
c
c   copyright (c) 1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c                                       Ur
c    computes the extra term  2 * mu * ---- in the axisymmetric case
c                                       r
c **********************************************************************
c
c                       KEYWORDS
c
c    axisymmetric
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad
      double precision matrix(1:ni+1,1:nj+1,2,*),
     +                  coor(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                  mu(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                  adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                  cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     coor      i    reserved
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     matrix    i    reserved
c     mu        i    The laminar viscosity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp
      double precision factv1, factv2

c     factv1    Factor for v1
c     factv2    Factor for v2
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      if ( alphad.eq.1 ) then

         do ip = 2, ni
            do jp = 1, nj

               factv1 = adir(ip,jp,1) * 2d0 * mu(ip,jp) *
     +                  cva(ip,jp,1,1) /
     +                  ((5d-1*(coor(ip,jp,1)+coor(ip,jp+1,1)))**2)

               factv2 = 25d-2 * adir(ip,jp,1) * 2d0 * mu(ip,jp) *
     +                  cva(ip,jp,1,2) /
     +                  ((5d-1*(coor(ip,jp,1)+coor(ip,jp+1,1)))**2)

               matrix(ip,jp,1,1)  = matrix(ip,jp,1,1)  + factv1
               matrix(ip,jp,1,10) = matrix(ip,jp,1,10) + factv2
               matrix(ip,jp,1,11) = matrix(ip,jp,1,11) + factv2
               matrix(ip,jp,1,12) = matrix(ip,jp,1,12) + factv2
               matrix(ip,jp,1,13) = matrix(ip,jp,1,13) + factv2

            end do
         end do

      else

         do ip = 1, ni
            do jp = 2, nj+1

               factv1 = 25d-2 * adir(ip,jp,1) * 2d0 * mu(ip,jp) *
     +                  cva(ip,jp,1,1) /
     +                  ((5d-1*(coor(ip,jp,1)+coor(ip+1,jp,1)))**2)

               factv2 = adir(ip,jp,1) * 2d0 * mu(ip,jp) *
     +                  cva(ip,jp,1,2) /
     +                  ((5d-1*(coor(ip,jp,1)+coor(ip+1,jp,1)))**2)

               matrix(ip,jp,2,1)  = matrix(ip,jp,2,1)  + factv2
               matrix(ip,jp,2,10) = matrix(ip,jp,2,10) + factv1
               matrix(ip,jp,2,11) = matrix(ip,jp,2,11) + factv1
               matrix(ip,jp,2,12) = matrix(ip,jp,2,12) + factv1
               matrix(ip,jp,2,13) = matrix(ip,jp,2,13) + factv1

            end do
         end do

      end if

      end
