      subroutine isaxpy3 ( ni, nj, nvirtual, ishift, jshift, vsum,
     +                     iarea, ishift1, jshift1, matrix, facs,
     +                     vector, invjac, ishift2, jshift2 )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 27-08-1999 Duncan: move invjac to distribution
c                                       routines
c        version 1.0    date 02-03-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Perform a vector update for ISNaS over a 2-dimensional region
c    The vector to be added may be shifted with respect to the update vector
c    The update looks as follows:
c    rhside(i,j) = rhside(i,j)+veloc(i+ishift,j+jshift)* facs*jac(i,j)*
c                  invjac(i+ishift,j+jshift)*vsum(i+ishift,j+jshift)
c    Both arrays are defined over the complete region, however y is
c    restricted to the region without virtual points and x uses virtual
c    points.
c    In order to get an optimal code the shift has been applied in the
c    declaration instead of the code, so that the code can use the same
c    indices for the vector to be updated as the vector to be added.
c    Since this is dangerous trick the user must be aware of this application
c    This subroutine is adapted to the new implementation of the convective
c    terms
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ishift, jshift, iarea(1:2,1:2),
     +        ishift1, jshift1, ishift2, jshift2
      double precision matrix(ni+1,nj+1), facs,
     +                 vsum(1-nvirtual-ishift:ni+nvirtual+1-ishift,
     +                      1-nvirtual-jshift:nj+nvirtual+1-jshift),
     +                invjac(1-nvirtual-ishift-ishift2:
     +                               ni+nvirtual+1-ishift-ishift2,
     +                      1-nvirtual-jshift-jshift2:
     +                               nj+nvirtual+1-jshift-jshift2),
     +                 vector(1-nvirtual-ishift1:ni+nvirtual+1-ishift1,
     +                        1-nvirtual-jshift1:nj+nvirtual+1-jshift1)

c     facs      i    Multiplication factor
c     iarea     i    Area of arrays
c     invjac    i    1/sqrt(g) in pnttyp
c     ishift    i    Shift in i-direction
c     ishift1   i    Shift in i-direction
c     ishift2   i    Shift in i-direction
c     jshift    i    Shift in j-direction
c     jshift1   i    Shift in j-direction
c     jshift2   i    Shift in j-direction
c     matrix   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vector    i    Input flow field
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j

c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      do j = iarea(1,2), iarea(2,2)
         do i = iarea(1,1), iarea(2,1)
            matrix(i,j) = matrix(i,j)+vector(i,j)*facs*vsum(i,j)*
     +                                                    invjac(i,j)

c           --- write(*,*) i,j, vsum(i,j), invjac(i,j)

         end do
      end do
      end
