      subroutine isbgeo2d ( bcgeo, ni, nj, nvirtual, cva, ibipoint,
     +                      invjac, jac )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.1    date 12-08-95 (Enlarge region)
c        version 1.0    date 01-06-95
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Compute geometrical quantities at boundaries of 2D region and store
c    into array bcgeo. Two-dimensional case
c ********************************************************************
c
c                      KEYWORDS
c
c    base_vectors
c    interpolation
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 bcgeo(8,*),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,4),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,4)

c     bcgeo     o    reserved
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j, iside, ipibinf, icount, k, pnttyp, iparea(2,2)

c     i         General loop variable
c     icount    Counts the number of positions necessary for the number
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     j         General loop variable
c     k         General loop variable
c     pnttyp    reserved
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c     sqrt(g) x contravariant base vector in 2D is given by:
c
c              (1)     2                    (1)      2
c     sqrt(g) a     = a            sqrt(g) a     = -a
c              1       (2)                  2        (1)
c
c              (2)     1                    (2)      1
c     sqrt(g) a     =-a            sqrt(g) a     =  a
c              1       (2)                  2        (1)
c
c               alpha beta             (alpha)            (beta)
c     sqrt(g) g            = (sqrt(g) a        . sqrt(g) a      )/sqrt(g)
c
c
c     For iside := 1 (1) 4 do
c        compute loop bounds as function of iside
c        For k := 1 (1) 2 do
c           determine point type as function of iside and k
c           icount := starting address in array ibinfor depending on k and iside
c           Fill bcgeo in a loop over the side
c ======================================================================
c
      do iside = 1, 4

         ipibinf = ibipoint(iside)
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 0
            iparea(2,2) = nj+2
            if ( iside.eq.1 ) then

c           --- iside = 1:  left-hand side

               iparea(1,1) = 1
               iparea(2,1) = 1

            else

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 0
            iparea(2,1) = ni+2
            if ( iside.eq.3 ) then

c           --- iside = 3:  lower side

               iparea(1,2) = 1
               iparea(2,2) = 1

            else

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if
         do k = 1, 2
            if ( k.eq.1 ) then
               pnttyp = 1
            else if ( iside.eq.3 .or. iside.eq.4 ) then
               pnttyp = 2
            else
               pnttyp = 3
            end if
            icount = ipibinf+k-1
            do j = iparea(1,2), iparea(2,2)
               do i = iparea(1,1), iparea(2,1)
                  bcgeo(4,icount) = cva(i,j,2,2,pnttyp)
                  bcgeo(5,icount) = -cva(i,j,1,2,pnttyp)
                  bcgeo(6,icount) = -cva(i,j,2,1,pnttyp)
                  bcgeo(7,icount) = cva(i,j,1,1,pnttyp)
                  bcgeo(8,icount) = jac(i,j,pnttyp)
                  bcgeo(1,icount) = invjac(i,j,pnttyp) *
     +                             ( bcgeo(4,icount)*bcgeo(4,icount) +
     +                               bcgeo(5,icount)*bcgeo(5,icount) )
                  bcgeo(2,icount) = invjac(i,j,pnttyp) *
     +                             ( bcgeo(4,icount)*bcgeo(6,icount) +
     +                               bcgeo(5,icount)*bcgeo(7,icount) )
                  bcgeo(3,icount) = invjac(i,j,pnttyp) *
     +                             ( bcgeo(6,icount)*bcgeo(6,icount) +
     +                               bcgeo(7,icount)*bcgeo(7,icount) )
                  icount = icount + 2

               end do

            end do

         end do

      end do

      end
