      subroutine isbgeo3d ( iface, ni, nj, nk, nvirtual, ilow1,
     +                      ihigh1, ilow2, ihigh2, cva, bcgeo )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.1    date 01-02-96 iparea changed
c        version 1.0    date 31-08-95
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                      DESCRIPTION
c
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ilow1, ihigh1,ilow2, ihigh2,
     +        iface
      double precision bcgeo(21,ilow1:ihigh1,ilow2:ihigh2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,8)

c     bcgeo     o    reserved
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ict1, ict2, ictd1, ictd2, igeo, ip, iparea(2,3), jp,
     +        kp, ndir, nijk(3), alphal, betal, pnttyp, tdir(2),
     +        valpha(3), vbeta(3), xp(3)

      logical skipgeo

c     alphal    Loop index
c     betal     Loop index
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     ictd1     Loop variable for ict1
c     ictd2     Loop variable for ict2
c     igeo      Indicates type of geometry discretization
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     ndir      Direction of normal vector on this side
c     nijk      Vector of number of points in the i,j and k-directions
c     pnttyp    reserved
c     skipgeo   If true computation must be skipped
c     tdir      Direction of tangent vector on this side
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c     vbeta     Array for cyclic changing ordering
c               betal = 1        betal = 2    betal = 3
c               vbeta(1) =1 ---> vbeta(1)=2  vbeta(1)=3
c               vbeta(2) =2 ---> vbeta(2)=3  vbeta(2)=1
c               vbeta(3) =3 ---> vbeta(3)=1  vbeta(3)=2
c     xp        Auxiliary array to calculate boundary point number
c
c ********************************************************************
c
c                    SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c
c                    PSEUDO CODE
c
c     For iface := 1 (1) 6 do
c        compute loop bounds as function of iside
c        For k := 1 (1) 2 do
c           determine point type
c
c       --- first compute bcgeo( 13 ..... 21 ) in which the
c           contravariant base vectors x sqrt(g) are stored
c
c            (alpha)    1
c           a       = ----  .   a      ^ a          for alpha, beta, gamma
c                    sqrt(g)     (beta)    (gamma)       cyclic
c
c           so :
c
c                   (alpha)
c          sqrt(g) a        =    a      ^ a          for alpha, beta, gamma
c                                (beta)    (gamma)       cyclic
c
c
c       --- next bcgeo( 1 ..... 6 ) in which the
c           sqrt(contravariant metric tensor) x sqrt(g) is stored
c
c                 alpha,beta            alpha             beta
c            g x g           = sqrt(g) a      . sqrt(g) a
c
c           so :
c
c                      alpha,beta                   alpha             beta
c           sqrt( g x g           )= sqrt( sqrt(g) a      . sqrt(g) a      )
c
c           At this moment only bcgeo(1) bcgeo(4) and bcgeo(6) are
c           filled, in which the diagonal terms are stored.
c           The other positions are not yet used, it is not possible
c           to take the sqrt of the cross terms because they don't
c           need to be positive
c
c       --- next bcgeo( 1 ..... 6 ) in which the covariant metric
c           tensor is stored
c
c           g            = a       . a
c            alpha,beta      alpha    beta
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

c     --- Compute tdir and ndir

      if ( iface.le.2 ) then

c     --- iface = 1, 2:  back and front plane

         ndir    = 1
         tdir(1) = 2
         tdir(2) = 3

      else if ( iface.gt.2  .and. iface .lt. 5) then

c     --- iface = 3, 4:  left- and right plane

         ndir = 2
         tdir(1) = 3
         tdir(2) = 1

      else

c     --- iface = 5, 6:  bottom and top

         ndir = 3
         tdir(1) = 1
         tdir(2) = 2

      end if

      if ( mod(iface,2).eq.0 ) then
         iparea(1,ndir) = nijk(ndir)+1
         iparea(2,ndir) = nijk(ndir)+1
      else
         iparea(1,ndir) = 1
         iparea(2,ndir) = 1
      end if

      iparea(1,tdir(1)) = 0
      iparea(2,tdir(1)) = nijk(tdir(1))+nvirtual
      iparea(1,tdir(2)) = 0
      iparea(2,tdir(2)) = nijk(tdir(2))+nvirtual


c     --- loop over four points in  boundary plane

      do ictd1 = 0, 1
         do ictd2 = 0, 1

c        --- loop over cells

            do kp = iparea(1,3), iparea(2,3)
               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     xp(1) = ip
                     xp(2) = jp
                     xp(3) = kp
                     ict1 = 2*xp(tdir(1))-1 + ictd1
                     ict2 = 2*xp(tdir(2))-1 + ictd2

                     skipgeo = .false.
                     if ( ictd1.eq.1 .or. ictd2.eq.1 ) then
                       skipgeo = xp(tdir(1)) .eq. iparea(2,tdir(1)) .or.
     +                         xp(tdir(2)) .eq. iparea(2,tdir(2))
                     end if

                     if ( .not. skipgeo ) then

                     if ( ictd1.eq.0 .and. ictd2.eq.0 ) then
                        pnttyp = 1
                     else if ( ictd1.eq.0 .and. ictd2.eq.1 ) then
                        if ( iface .le. 2 ) then
                           pnttyp = 5
                        else if ( iface .gt. 2 .and. iface .lt. 5 ) then
                           pnttyp = 2
                        else
                           pnttyp = 3
                        end if
                     else if ( ictd1.eq.1 .and. ictd2.eq.0 ) then
                        if ( iface .le. 2 ) then
                           pnttyp = 3
                        else if ( iface .gt. 2 .and. iface .lt. 5 ) then
                           pnttyp = 5
                        else
                           pnttyp = 2
                        end if
                     else if ( ictd1.eq.1 .and. ictd2.eq.1 ) then
                        if ( iface .le. 2 ) then
                           pnttyp = 7
                        else if ( iface .gt. 2 .and. iface .lt. 5 ) then
                           pnttyp = 6
                        else
                           pnttyp = 4
                        end if
                     end if

                     do alphal = 1, 3
                        do betal = 1, 3
c
c                          --- cyclic
c
c                           alphal = 1       alphal = 2   alpha = 3
c
c                          valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c                          valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c                          valpha(3) =3 ---> valpha(3)=1  valpha(3)=2

                           valpha(1) = alphal
                           valpha(2) = mod(alphal  ,3) + 1
                           valpha(3) = mod(alphal+1,3) + 1

                           vbeta(1)  = betal
                           vbeta(2)  = mod(betal  ,3) + 1
                           vbeta(3)  = mod(betal+1,3) + 1

                           igeo = 12 + betal + (alphal -1)*3
                           bcgeo(igeo,ict1,ict2) =
     +                        cva(ip,jp,kp,vbeta(2),valpha(2),pnttyp)*
     +                        cva(ip,jp,kp,vbeta(3),valpha(3),pnttyp)-
     +                        cva(ip,jp,kp,vbeta(3),valpha(2),pnttyp)*
     +                        cva(ip,jp,kp,vbeta(2),valpha(3),pnttyp)

                        end do
                     end do


                     bcgeo(1,ict1,ict2) =
     +                  sqrt( bcgeo(13,ict1,ict2)*bcgeo(13,ict1,ict2)+
     +                        bcgeo(14,ict1,ict2)*bcgeo(14,ict1,ict2)+
     +                        bcgeo(15,ict1,ict2)*bcgeo(15,ict1,ict2))

                     bcgeo(2,ict1,ict2) = 0d0

                     bcgeo(3,ict1,ict2) = 0d0

                     bcgeo(4,ict1,ict2) =
     +                 sqrt(  bcgeo(16,ict1,ict2)*bcgeo(16,ict1,ict2)+
     +                        bcgeo(17,ict1,ict2)*bcgeo(17,ict1,ict2)+
     +                        bcgeo(18,ict1,ict2)*bcgeo(18,ict1,ict2))

                     bcgeo(5,ict1,ict2) = 0d0

                     bcgeo(6,ict1,ict2) =
     +                 sqrt(  bcgeo(19,ict1,ict2)*bcgeo(19,ict1,ict2)+
     +                        bcgeo(20,ict1,ict2)*bcgeo(20,ict1,ict2)+
     +                        bcgeo(21,ict1,ict2)*bcgeo(21,ict1,ict2))

                     bcgeo(7,ict1,ict2) =
     +                        cva(ip,jp,kp,1,1,pnttyp)*
     +                        cva(ip,jp,kp,1,1,pnttyp)+
     +                        cva(ip,jp,kp,2,1,pnttyp)*
     +                        cva(ip,jp,kp,2,1,pnttyp)+
     +                        cva(ip,jp,kp,3,1,pnttyp)*
     +                        cva(ip,jp,kp,3,1,pnttyp)

                     bcgeo(8,ict1,ict2) =
     +                        cva(ip,jp,kp,1,1,pnttyp)*
     +                        cva(ip,jp,kp,1,2,pnttyp)+
     +                        cva(ip,jp,kp,2,1,pnttyp)*
     +                        cva(ip,jp,kp,2,2,pnttyp)+
     +                        cva(ip,jp,kp,3,1,pnttyp)*
     +                        cva(ip,jp,kp,3,2,pnttyp)

                     bcgeo(9,ict1,ict2) =
     +                        cva(ip,jp,kp,1,1,pnttyp)*
     +                        cva(ip,jp,kp,1,3,pnttyp)+
     +                        cva(ip,jp,kp,2,1,pnttyp)*
     +                        cva(ip,jp,kp,2,3,pnttyp)+
     +                        cva(ip,jp,kp,3,1,pnttyp)*
     +                        cva(ip,jp,kp,3,3,pnttyp)

                     bcgeo(10,ict1,ict2) =
     +                        cva(ip,jp,kp,1,2,pnttyp)*
     +                        cva(ip,jp,kp,1,2,pnttyp)+
     +                        cva(ip,jp,kp,2,2,pnttyp)*
     +                        cva(ip,jp,kp,2,2,pnttyp)+
     +                        cva(ip,jp,kp,3,2,pnttyp)*
     +                        cva(ip,jp,kp,3,2,pnttyp)

                     bcgeo(11,ict1,ict2) =
     +                        cva(ip,jp,kp,1,2,pnttyp)*
     +                        cva(ip,jp,kp,1,3,pnttyp)+
     +                        cva(ip,jp,kp,2,2,pnttyp)*
     +                        cva(ip,jp,kp,2,3,pnttyp)+
     +                        cva(ip,jp,kp,3,2,pnttyp)*
     +                        cva(ip,jp,kp,3,3,pnttyp)

                     bcgeo(12,ict1,ict2) =
     +                        cva(ip,jp,kp,1,3,pnttyp)*
     +                        cva(ip,jp,kp,1,3,pnttyp)+
     +                        cva(ip,jp,kp,2,3,pnttyp)*
     +                        cva(ip,jp,kp,2,3,pnttyp)+
     +                        cva(ip,jp,kp,3,3,pnttyp)*
     +                        cva(ip,jp,kp,3,3,pnttyp)

                  end if

                  end do
               end do
            end do
         end do
      end do


      end
