      subroutine isbmcupw ( ni, nj, nvirtual, vsum, alphad,
     +                       locdx, ibinfor, ibipoint )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 08-12-1998 Use end do
c        version 1.0    date 12-10-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c    Adapt normal half cells in upwind case for WesBeek discretization
c
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    convection
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, locdx(2),
     +        iparea(2,2), alphad, ibipoint(*), ibinfor(*)
      double precision vsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut,  periodic, antiper, multiblk,
     +        uxuy, unsnt,  perjump, antijump,
     +        walllaw, parabolic, skipval, masspar
      parameter( unut = 4,  periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5,
     +           walllaw = 9, parabolic = 12)
      parameter (skipval = 300, masspar = 100)
      logical ungiven, noskip
      integer mip, mjp, xp(2), cbc, ip, jp, mxp(2), icbtype, btype,
     +         iside, tdir, ndir, ipibinf

c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     btype     Type of boundary condition
c     cbc       Boundary point number corresponding to the
c               integration cell centre
c     icbtype   Type of boundary condition with respect to centre of integration
c               cell
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     jp        Local counting variable j-direction
c     masspar   Boundary condition for massive parallel computations
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     multiblk  Multi-block boundary conditions
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     noskip    Indicate whether a given equation is skipped or not
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     skipval   This value indicates that effect of boundary condition
c               must be skipped
c     tdir      Direction of tangent vector on this side
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     walllaw   Law of the wall boundary condition
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c       Relevant formula for an internal cell
c
c
c    The convective terms in case of the classical discretization are given by:
c
c
c    rho      a  1 |i+1/2,j     rho      a  2 |i,j+1/2
c   ----- V  V  |         + -----    V  V  |          +
c   sqrt(g)        |i-1/2,j    sqrt(g)        |i,j-1/2
c
c
c    rho    |  a  |   c  b |
c   ----- |     |  V  V  |
c   sqrt(g) | b c |        |i,j
c
c
c   Application of Newton linearization:
c
c   a  b    _a  b      a _b     _a _b
c  V  V  =  V  V   +  V  V   -  V  V
c                                                 1
c   results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j     rho      2 |i,j+1/2      rho      a |i+1/2,j
c   ----- V  |         + -----    V  |          + -----    V  |         +
c   sqrt(g)     |i-1/2,j    sqrt(g)     |i,j-1/2     sqrt(g)     |i-1/2,j
c
c
c  2 rho    |  a  |   b |
c   ----- |     |  V  |
c   sqrt(g) | 1 b |     |i,j
c
c                                              2
c  and of course a similar expression for the V  component
c
c  Application of the Picard linearization (instead of the Newton lin.) gives
c  for the b-th momentum equation:
c
c     a  b     _a  b          b  b    _b  b
c    V  V   =  V  V   and    V  V  =  V  V
c
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              |-----+------|            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c   The cell centre is indicated by *, the integration points by +
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(2,2) = nj
            tdir = 2
            ndir = 1

            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(2,1) = ni
            tdir = 1
            ndir = 2

            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if

         noskip =  alphad.eq.ndir
         if ( noskip ) then

c        --- loop over cells

            do mjp = iparea(1,2), iparea(2,2)
               do mip = iparea(1,1), iparea(2,1)

c              --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp

c                 --- Get boundary type for integration cell centre

                  btype = ibinfor(ipibinf+2*mxp(tdir))

c                 --- Check if un is given in half cell

                  ungiven = btype.eq.uxuy .or. btype.eq.unut .or.
     +                      btype.eq.parabolic .or. btype.eq.unsnt .or.
     +                      btype.eq.walllaw

c                 --- Get reduced boundary condition type

                  if ( btype.eq.multiblk .or. btype.eq.periodic .or.
     +                 btype.eq.antiper .or.
     +                 btype.eq.perjump .or. btype.eq.antijump .or.
     +                 btype.eq.masspar ) btype = skipval
                  icbtype = btype

                  xp(1) = mxp(1)
                  xp(2) = mxp(2)

                  ip = xp(1)
                  jp = xp(2)

c                 --- Get integration point position for ibinfor

                  cbc = 2*mxp(tdir)+locdx(tdir)

c                 --- Get boundary information

                  btype = ibinfor(ipibinf+cbc)

c                 --- Get reduced boundary condition type

                  if ( btype.eq.parabolic ) then
                     btype = unut
                  else if ( btype.eq.walllaw ) then
                     btype = unsnt
                  else if ( btype.eq.multiblk .or. btype.eq.periodic
     +                      .or. btype.eq.antiper .or.
     +                      btype.eq.perjump .or. btype.eq.antijump .or.
     +                      btype.eq.masspar ) then
                     btype = skipval
                  end if

                  if ( ungiven ) then

c                 --- Normal velocity given, no contribution to stencil

                     vsum(ip,jp,1) = 0d0
                     vsum(ip,jp,2) = 0d0

                  else if ( icbtype.ne.skipval ) then

c                 --- Centre of cell is not adjacent to another block,
c                     construct basic stencil for half cell

                     if ( locdx(ndir).eq.0 .and.
     +                    locdx(tdir).ne.0) then

c                    --- Point (1,0) or (-1,0)
c                        Wesbeek discretization
c                        Picard linearization

                            vsum(ip,jp,1) = 5d-1 * vsum(ip,jp,1)
                            vsum(ip,jp,2) = 5d-1 * vsum(ip,jp,2)

                     end if

                  end if

               end do

            end do

         end if

      end do

      end
