      subroutine isbmoms2 ( ni, nj, nvirtual, vsum, dvsum, alphad,
     +                      mu, jctg, chsym, locdx, dcell, ibinfor,
     +                      ibipoint, bndcon, rhside, nconct, jac,
     +                      reduced, mequat )
c ======================================================================
c
c        programmer     Guus Segal
c        version 5.2    date 24-12-1998 Guus: adaptation for transition
c                                       multiblock fixed boundary
c        version 5.1    date 10-12-1997 Hester: comments added
c        version 5.0    date 08-10-1997 Hester: extra parameter mequat
c        version 4.0    date 12-03-1997 Guus: Extra param reduced
c        version 3.1    date 30-01-1997 Hester: extra parameter jac
c        version 3.0    date 03-10-1996 Guus: extra parameter nconct
c        version 2.0    date 30-10-1995 Guus: remove nzdvsum from parameter list
c        version 1.1    date 28-09-1995 Kees V. use of subroutine isorient2
c        version 1.0    date 29-05-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute stress contribution to matrix and right-hand side for "normal"
c   half cells. Boundary condition snnn given for Euler eqs. handled in
c   isbmomp2. If in this routine boundary condition handling is changed,
c   also change routine isbmomp2.
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c     matrix
c     stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, locdx(2), dcell(2), alphad, ibinfor(*),
     +        ibipoint(*), nconct, mequat
      double precision bndcon(2,*)
      double precision
     +   mu(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual),
     +   vsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +   dvsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,2),
     +   jctg(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,3),
     +   chsym(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,3),
     +   rhside(ni+1,nj+1,2),
     +   jac(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,4)
      logical reduced

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     dcell     i    reserved
c     dvsum     i    Flux derivative contributions to the matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     jac       i    sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     mu        i    The laminar viscosity
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     reduced   i    If true the reduced stress tensor corresponding
c                    to a constant viscosity and incompressible flow is used
c     rhside   i/o   reserved
c     vsum     i/o   Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, snnsnt, periodic, antiper, multiblk,
     +        uxuy, unsnt, snnut, perjump, antijump,
     +        walllaw, parabolic, skipval, masspar
      parameter ( skipval = 300, masspar = 100 )
      parameter( unut = 4, snnsnt = 7, periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5, snnut = 6,
     +           walllaw = 9, parabolic= 12)
      logical ungiven, noskip
      integer i, j, btype, mip, mjp, mxp(2), ipibinf, tsign,
     +        ict1, icbtype, ip, jp, xp(2), ndir,
     +        tdir, nsign, iptensor1, iside, istwo(2,2), iparea(2,2)
      double precision sigmann, sigmant, facs

c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     btype     Type of boundary condition
c     facs      Multiplication factor
c     i         General loop variable
c     icbtype   Type of boundary condition with respect to centre of integration
c               cell
c     ict1      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     iptensor1 Tensor index
c     iside     Sequence number of side
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     j         General loop variable
c     jp        Local counting variable j-direction
c     masspar   Boundary condition for massive parallel computations
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     multiblk  Multi-block boundary conditions
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     noskip    Indicate whether a given equation is skipped or not
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     sigmann   Value of sigma^nn
c     sigmant   Value of sigma^nt
c     skipval   This value indicates that effect of boundary condition
c               must be skipped
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     walllaw   Law of the wall boundary condition
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   First we consider the formulae at an internal cell:
c
c
c    The stress terms in case of the classical discretization are given by:
c
c
c             a1 |i+1/2,j              a2 |i,j+1/2            |  a  |   cb |
c   sqrt(g)  T   |         + sqrt(g)  T   |        +  sqrt(g) |     |  T   |
c                |i-1/2,j                 |i,j-1/2            | b c |      |i,j
c
c             ab
c   The term T   can be written as:
c
c
c    ab         ac b      bc a
c  -T   = mu ( g  U    + g  U   )
c                  ,c        ,c
c
c           aa         ac a
c   Hence -T   = 2 mu g  U
c                         ,c
c
c   If the incompressibility condition is substituted then:
c
c    12         11 2      22 1
c  -T   = mu ( g  U    + g  U   )
c                  ,1        ,2
c
c                    a                  a        b     |  a   |  c
c    The derivative U   is equal to  d U  / d Ksi    + |      | U
c                    ,b                                |b   c |
c
c
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c
c   The stress term for this cell has the following shape:
c
c  1          a1 |i+1/2,j              a2 |i,j+1/2   1        |  a  |   cb |
c  - sqrt(g) T   |         + sqrt(g)  T   |        + -sqrt(g) |     |  T   |
c  2             |i-1/2,j                 |i,j       2        | b c |      |i,j
c
c
c   The following boundary conditions have the following effects:
c
c
c   un given in boundary centroid:  matrix elements equal to 0
c
c   Sigma nn, Sigma nt given:       Replace all terms sigma nn and sigma nt
c                                   at the boundary by their given value
c
c      Formulae:
c
c          nn    nn  nn
c     sigma   = g   S
c
c
c          nt        (n)                sqrt(g^nn)  nt    g_nt      nn
c     sigma   = sgn(a   .n) sgn(a   .t) ---------- S    - ---- sigma
c                   -    -      -(t) -  sqrt(g_tt)        g_tt
c
c
c     sqrt(g^nn)/sqrt(g_tt) = 1/sqrt(g)
c     g_nt/g_tt sigma^nn = -g^nt/g^nn g^nn S^nn = -g^nt S^nn
c
c   Sigma nn, ut given:             Replace all terms sigma nn
c                                   at the boundary by the given value
c                                   Use the folding subroutine to eliminate
c                                   virtual tangential velocities
c
c    The comments refer to a cell at the lower boundary
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1

c        --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 1
            iparea(2,2) = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 1
            iparea(2,1) = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if
         noskip = locdx(ndir).ne.nsign .and. alphad.eq.ndir
         if ( noskip ) then

c        --- loop over cells

            do mjp = iparea(1,2), iparea(2,2)
               do mip = iparea(1,1), iparea(2,1)

c              --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp

c                 --- Get integration cell position for ibinfor

                  icbtype = ibinfor(ipibinf+2*mxp(tdir))

c                 --- Check for Dirichlet condition in half cell

                  ungiven = icbtype.eq.uxuy .or. icbtype.eq.unut .or.
     +                      icbtype.eq.parabolic .or. icbtype.eq.unsnt
     +                      .or. icbtype.eq.walllaw

c                 --- Get reduced boundary condition type

                  btype = icbtype
                  if ( btype.eq.parabolic .or. btype.eq.88 ) then
                     btype = unut
                  end if
                  if (btype.eq.walllaw) then
                     btype = unsnt
                  end if
                  if (btype.eq.multiblk .or. btype.eq.periodic .or.
     +               btype.eq.antiper .or.
     +               btype.eq.perjump .or. btype.eq.antijump .or.
     +               btype.eq.masspar) then
                     btype = skipval
                  end if
                  icbtype = btype

c                 --- Get integration point index

                  xp(tdir) = mxp(tdir) + dcell(tdir)
                  xp(ndir) = mxp(ndir) + dcell(ndir)

                  ip = xp(1)
                  jp = xp(2)

c                 --- Get boundary information for integration point

                  ict1 = ipibinf+2*mxp(tdir)+locdx(tdir)
                  btype = ibinfor(ict1)

c                 --- Get reduced boundary condition type

                  if ( btype.eq.parabolic .or. btype.eq.88 )
     +               btype = unut
                  if ( btype.eq.walllaw ) btype = unsnt
                  if ( btype.eq.multiblk .or. btype.eq.periodic .or.
     +                 btype.eq.antiper .or. btype.eq.perjump .or.
     +                 btype.eq.antijump .or. btype.eq.masspar )
     +                 btype = skipval

c                 --- Local contribution of covariant derivatives
c                     to christoffel symbols

                  if ( ungiven ) then

c                 --- Dirichlet half cell

                     vsum(ip,jp,1) = 0d0
                     vsum(ip,jp,2) = 0d0
                     dvsum(ip,jp,1,1) = 0d0
                     dvsum(ip,jp,2,1) = 0d0
                     dvsum(ip,jp,1,2) = 0d0
                     dvsum(ip,jp,2,2) = 0d0

                  else if ( icbtype.ne.skipval ) then

c                 --- velocity integration cell (half cell for V^ndir)
c                     Construct the basic stencil in so far necessary
c                     In this situation we have always sigma^nn given

                     vsum(ip,jp,1) = 0d0
                     vsum(ip,jp,2) = 0d0
                     dvsum(ip,jp,1,1) = 0d0
                     dvsum(ip,jp,2,1) = 0d0
                     dvsum(ip,jp,1,2) = 0d0
                     dvsum(ip,jp,2,2) = 0d0

                     if ( locdx(tdir).eq.0 .and.locdx(ndir).eq.0 ) then

c                    --- Point (0,0)
c                        Only the term involving sigma^tt is common for both
c                        cases

                        if ( .not. reduced ) then

c                       --- standard case

                           iptensor1 = istwo(tdir,tdir)
                           facs = -mu(ip,jp)*chsym(ip,jp,ndir,iptensor1)
                           do i = 1, 2
                              dvsum(ip,jp,tdir,i) = facs*
     +                             jctg(ip,jp,istwo(tdir,i))
                           end do

                        end if

                        if ( nconct.eq.17 .and. mequat.eq.0 ) then

                           dvsum(ip,jp,1,1)=dvsum(ip,jp,1,1)+
     +                           mu(ip,jp)*jac(ip,jp,4)*
     +                           ( chsym(ip,jp,alphad,1) +
     +                             chsym(ip,jp,alphad,3) )/3d0
                           dvsum(ip,jp,2,2)=dvsum(ip,jp,2,2)+
     +                           mu(ip,jp)*jac(ip,jp,4)*
     +                           ( chsym(ip,jp,alphad,1) +
     +                             chsym(ip,jp,alphad,3) )/3d0

                        end if

                     else if ( locdx(tdir).eq.0 .and.
     +                         locdx(ndir).eq.-nsign ) then

c                    --- Point (0,1)
c                        Term sqrt(g)*sigma^22

                        if ( reduced ) then
                           i = ndir
                           dvsum(ip,jp,ndir,i) = -locdx(ndir)*
     +                        mu(ip,jp)*jctg(ip,jp,istwo(i,ndir))
                        else
                           do i = 1, 2
                              dvsum(ip,jp,ndir,i) = -locdx(ndir)*
     +                           2d0*mu(ip,jp)*jctg(ip,jp,istwo(i,ndir))

                           end do
                        end if

                        if ( nconct.eq.17 .and. mequat.eq.0 ) then

                           dvsum(ip,jp,1,1)=dvsum(ip,jp,1,1)+2d0*
     +                       locdx(ndir)*mu(ip,jp)*jac(ip,jp,2)/3d0
                           dvsum(ip,jp,2,2)=dvsum(ip,jp,2,2)+2d0*
     +                       locdx(ndir)*mu(ip,jp)*jac(ip,jp,2)/3d0
                        end if

                     else if ( locdx(tdir).ne.0 .and.
     +                         locdx(ndir).eq.0 ) then

c                    --- Points (-1,0) and (1,0)
c                        Term 0.5*sqrt(g)*sigma^21
c                        The incompressibility condition is used to eliminate
c                        the derivatives U1,1 and U2,2

                        if ( .not. reduced ) then

c                       --- standard case

                           dvsum(ip,jp,tdir,ndir) = -locdx(tdir)*
     +                      0.5d0*mu(ip,jp)*jctg(ip,jp,istwo(ndir,ndir))

                        end if

                        dvsum(ip,jp,ndir,tdir) = -locdx(tdir)*
     +                      0.5d0*mu(ip,jp)*jctg(ip,jp,istwo(tdir,tdir))

                        if ( nconct.eq.17 ) then
                           dvsum(ip,jp,1,1) = -locdx(tdir)*
     +                         0.5d0*mu(ip,jp)*jctg(ip,jp,2)
                           dvsum(ip,jp,2,2) = -locdx(tdir)*
     +                         0.5d0*mu(ip,jp)*jctg(ip,jp,2)
                        end if

                     end if

c                    --- Modify basic stencil for half cell to take
c                        into account the boundary conditions

                     if ( btype.eq.snnsnt ) then

c                    --- Both sigma^nn and sigma^nt given in integration point
c                        Get boundary condition information

                        sigmant = tsign*nsign*bndcon(2,ict1)+
     +                            jctg(ip,jp,2)*bndcon(1,ict1)
                        sigmann = jctg(ip,jp,istwo(ndir,ndir))*
     +                            bndcon(1,ict1)

                        if ( locdx(tdir).eq.0 .and. locdx(ndir).eq.0 )
     +                     then

c                       --- Point (0,0), substitute terms in right-hand side

                           rhside(ip,jp,ndir) = rhside(ip,jp,ndir) +
     +                        nsign*sigmann + 0.5d0*
     +                        chsym(ip,jp,ndir,istwo(ndir,ndir))*
     +                        sigmann+
     +                        chsym(ip,jp,ndir,istwo(ndir,tdir))*sigmant

                        else if ( locdx(tdir).ne.0 .and.
     +                            locdx(ndir).eq.0 ) then

c                       --- Points (1,0) and (-1,0)
c                           substitute terms in right-hand side

                           dvsum(ip,jp,tdir,ndir) = 0d0
                           dvsum(ip,jp,ndir,tdir) = 0d0
                           rhside(mip,mjp,ndir) = rhside(mip,mjp,ndir) +
     +                        0.5d0*locdx(tdir)*sigmant

                        end if
                     else if ( btype.eq.snnut ) then

c                    --- sigma^nn provided and ut provided

                        if (locdx(tdir).eq.0 .and. locdx(ndir).eq.0)
     +                     then

c                       --- Point (0,0)

                           sigmann = jctg(ip,jp,istwo(ndir,ndir))*
     +                               bndcon(1,ict1)
                           rhside(ip,jp,alphad) = rhside(ip,jp,alphad)+
     +                        nsign*sigmann
                           rhside(ip,jp,alphad) = rhside(ip,jp,alphad)+
     +                        0.5d0*
     +                        chsym(ip,jp,ndir,istwo(ndir,ndir))*sigmann

c                          --- Matrix contribution: 1/2 sqrt(g) |2; 1 2|
c                              sigma^12

                           facs = 0.5d0*mu(ip,jp)*
     +                     chsym(ip,jp,ndir,istwo(1,2))
                           dvsum(ip,jp,1,2) = dvsum(ip,jp,1,2) -
     +                           facs*jctg(ip,jp,istwo(2,2))
                           dvsum(ip,jp,2,1) = dvsum(ip,jp,2,1) -
     +                           facs*jctg(ip,jp,istwo(1,1))
                        end if

                     end if

c                    --- Add Christoffel symbols for dvsum terms

                     do j = 1, 2
                        do i = 1, 2
                           vsum(ip,jp,1) = vsum(ip,jp,1)+
     +                                     dvsum(ip,jp,j,i)*
     +                                     chsym(ip,jp,j,istwo(i,1))
                           vsum(ip,jp,2) = vsum(ip,jp,2)+
     +                                     dvsum(ip,jp,j,i)*
     +                                     chsym(ip,jp,j,istwo(i,2))
                        end do
                     end do
                  end if
               end do
            end do

         else if ( locdx(ndir).eq.nsign .and. alphad.eq.ndir ) then

c        --- Point outside region
c            loop over cells

            do mjp = iparea(1,2), iparea(2,2)
               do mip = iparea(1,1), iparea(2,1)

c              --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp

c                 --- Get integration cell position for ibinfor

                  btype = ibinfor(ipibinf+2*mxp(tdir))
                  if (btype.eq.multiblk .or. btype.eq.periodic .or.
     +               btype.eq.antiper .or.
     +               btype.eq.perjump .or. btype.eq.antijump .or.
     +               btype.eq.masspar) then
                     btype = skipval
                  end if

c                 --- Get integration point index

                  xp(tdir) = mxp(tdir) + dcell(tdir)
                  xp(ndir) = mxp(ndir) + dcell(ndir)

                  ip = xp(1)
                  jp = xp(2)
                  if ( btype.ne.skipval ) then
                     vsum(ip,jp,1) = 0d0
                     vsum(ip,jp,2) = 0d0
                     dvsum(ip,jp,1,1) = 0d0
                     dvsum(ip,jp,2,1) = 0d0
                     dvsum(ip,jp,1,2) = 0d0
                     dvsum(ip,jp,2,2) = 0d0
                  end if

               end do
            end do

         end if

      end do

      end
