      subroutine isbmomt3 ( ni, nj, nk, matrix, rhside, alphad, ibinfor,
     +                      ilow1, ihigh1, ilow2, ihigh2, iface)
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 22-06-95 ( as isbmomt2 of Guus Segal)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     correct the time derivative part of the matrix diagonal with respect to
c     the boundary cells where sigma^nn is given
c     R^3 only
c
c     Remark: the present version of this subroutines assumes that boundary
c             conditions  may differ in each point
c             The subroutine may be more efficient if information is stored
c             as sections or per side
c ********************************************************************
c
c                      KEYWORDS
c
c     3d
c     matrix
c     right_hand_side
c     time_derivative
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer alphad, ni, nj, nk, iface,
     +        ilow1, ihigh1, ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2)

      double precision matrix(ni+1,nj+1,nk+1), rhside(ni+1,nj+1,nk+1)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     matrix   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     rhside   i/o   reserved
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ip, jp, kp, snnsnt, snnut, ndir, iparea(2,3), tdir(2),
     +        xp(3), btype, ict1, ict2
      parameter( snnsnt = 7, snnut = 6 )

c     btype     Type of boundary condition
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     ndir      Direction of normal vector on this side
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     xp        Auxiliary array to calculate boundary point number
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c      This subroutine only treats those half cells where the normal stress
c      is given.
c      In those cells it is necessary to divide the value of the
c      matrix element as well as the right-hand side element by 2
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         iparea(1,3) = 1
         iparea(2,3) = 1

         if ( iface.le.2 ) then

c        --- iface = 1, 2:  back and front plane

            iparea(2,2) = nj
            iparea(2,3) = nk
            ndir    = 1
            tdir(1) = 2
            tdir(2) = 3

            if ( iface.eq.2 ) then

c           --- iface = 2:  front

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else if ( iface.gt.2  .and. iface .lt. 5) then

c        --- iface = 3, 4:  left- and right plane

            iparea(2,1) = ni
            iparea(2,3) = nk
            ndir = 2
            tdir(1) = 3
            tdir(2) = 1

            if ( iface.eq.4 ) then

c           --- iface = 4:  right side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         else if ( iface.gt.4  ) then

c        --- iface = 5, 6:  bottom and top

            iparea(2,1) = ni
            iparea(2,2) = nj
            ndir = 3
            tdir(1) = 1
            tdir(2) = 2

            if ( iface.eq.6 ) then

c           --- iface = 6:  upper side

               iparea(1,3) = nk+1
               iparea(2,3) = nk+1

            end if

         end if

         if ( ndir.eq.alphad ) then

c        --- alphad = normal direction, hence half cell found

            do 120 kp = iparea(1,3), iparea(2,3)
               do 110 jp = iparea(1,2), iparea(2,2)
                  do 100 ip =  iparea(1,1), iparea(2,1)
                     xp(1) = ip
                     xp(2) = jp
                     xp(3) = kp
                     ict1 = 2*xp(tdir(1))
                     ict2 = 2*xp(tdir(2))
                     btype = ibinfor(ict1,ict2)
                     if ( btype.eq.snnut .or. btype.eq.snnsnt ) then
                        matrix(ip,jp,kp) = matrix(ip,jp,kp) * 0.5d0
                        rhside(ip,jp,kp) = rhside(ip,jp,kp) * 0.5d0
                     end if
100               continue
110            continue
120         continue

         end if

      end
