      subroutine iscexst2d ( ni, nj, nvirtual, kdif, alphad, cva, jcta,
     +                       cvc, ctc, solut, rhside, drhside )
c ======================================================================
c
c     programmer  Charles Moulinec
c     version     1.2   24-08-1998    Kees K: drhside from local to input
c     version     1.1   05-11-1997    Charles: correction in the loops
c     version     1.0   15-07-1997
c
c   copyright (c) 1997-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute the contribution of the explicit part of the stress term
c     for the collocated momentum equation
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2)
      double precision rhside(ni+1,nj+1)
      double precision kdif(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,3)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 cvc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 ctc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2)
      double precision jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2)
      double precision drhside(ni+1,nj+1,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ctc       i    Work array containing the contravariant WESBEEK vectors
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     cvc       i    Work array containing the covariant WESBEEK vectors
c     drhside  i/o   Contribution of the explicit part of the stress term on
c                    every face
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     kdif      i    The diffusion coefficient in a given integration point
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, betad, dx(2,2), gammad
      double precision det

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     det       Determinant
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     gammad    Direction index
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      The extra diffusion terms are evaluated by the WesBeek discretization
c
c      After integration on a control volume, the term
c
c                   (a)        (b)             |+e(a)/2
c        - sqrt(g) a   . (mu dU   / dx      )  |
c                  - b   -            alphad   |-e(a)/2
c
c      is isolated and discretized.
c
c      The derivative of the velocity component is calculated by
c      the Wesbeek approach.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Wesbeek discretization

         dx(1,1)      =  0
         dx(1,2)      =  0
         dx(1,alphad) = -1

         do betad = 1, 2
            do gammad = 1, 2
               dx(2,1)      =  0
               dx(2,2)      =  0
               dx(2,gammad) = -1
               if ( gammad.eq.alphad ) then

                  do jp = 2, nj
                     do ip = 2, ni
                        cvc(ip,jp,betad,gammad) = 0.5d0 *
     +                      ( cva(ip+dx(1,1),jp+dx(1,2),betad,gammad) +
     +                        cva(ip,jp,betad,gammad) )
                     end do
                  end do

               else

                  do jp = 2, nj
                     do ip = 2, ni

                        cvc(ip,jp,betad,gammad) = 125d-3 *
     +                     ( cva(ip+dx(1,1)-dx(2,1),jp+dx(1,2)-dx(2,2),
     +                       betad,gammad) + 2d0 *
     +                       cva(ip+dx(1,1),jp+dx(1,2),betad,gammad) +
     +                       cva(ip+dx(1,1)+dx(2,1),jp+dx(1,2)+dx(2,2),
     +                           betad,gammad)+
     +                       cva(ip-dx(2,1),jp-dx(2,2),betad,gammad) +
     +                       2d0 * cva(ip,jp,betad,gammad) +
     +                       cva(ip+dx(2,1),jp+dx(2,2),betad,gammad) )

                     end do
                  end do

               end if

            end do
         end do

c        --- Determine contravariant WESBEEK vectors

         do jp = 2, nj
            do ip = 2, ni

c           --- First, calculate determinant ...

               det = cvc(ip,jp,1,1)*cvc(ip,jp,2,2) -
     +               cvc(ip,jp,2,1)*cvc(ip,jp,1,2)

c              --- then contravariant WESBEEK vectors

               ctc(ip,jp,1,1) =  cvc(ip,jp,2,2)/det
               ctc(ip,jp,2,1) = -cvc(ip,jp,1,2)/det
               ctc(ip,jp,1,2) = -cvc(ip,jp,2,1)/det
               ctc(ip,jp,2,2) =  cvc(ip,jp,1,1)/det

            end do
         end do

         do betad = 1, 2
            do gammad = 1, 2

               do jp = 2, nj
                  do ip = 2, ni

                     drhside(ip,jp,gammad) =
     +      jcta(ip,jp,betad,gammad)*kdif(ip,jp,gammad+betad/2)*
     +      (ctc(ip,jp,alphad,1)*
     +      (solut(ip,jp,betad)-
     +       solut(ip+dx(1,3-alphad),jp+dx(1,alphad),betad))+
     +       ctc(ip,jp,alphad,2)*25d-2*
     +      (solut(ip-dx(1,3-alphad),jp-dx(1,alphad),betad)-
     +       solut(ip+dx(1,3-alphad),jp+dx(1,alphad),betad)+
     +       solut(ip+dx(1,alphad)-dx(1,3-alphad)
     +            ,jp-dx(1,alphad)+dx(1,3-alphad),betad)-
     +       solut(ip-1,jp-1,betad)))
                  end do
               end do
            end do
         end do

         do jp = 2, nj-1
            do ip = 2, ni-1

               rhside(ip,jp) = rhside(ip,jp) - drhside(ip,jp,1)
     +                                       + drhside(ip+1,jp,1)
     +                                       - drhside(ip,jp,2)
     +                                       + drhside(ip,jp+1,2)
     +
            end do
         end do

      end
