      subroutine iscmfl2 ( ni, nj, nvirtual, ndegfd, solut, eddy,
     +                   intrho, workfl )
c ======================================================================
c
c     programmer   Duncan van der Heul
c     version      1.1    date 9-04-1999 Duncan: removed bug in call to
c                                                isintcf2, extended for
c                                                  j direction
c     version      1.0    date 1-04-1999
c
c
c   copyright (c) 1995-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    ISnas for CoMpressible flow compute extended velocity in case of
c    Newton linearisation
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  ni, nj, nvirtual, ndegfd
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,ndegfd,*),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,3),
     +                 intrho(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 workfl(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,*)

c     eddy      i    reserved
c     intrho    i    Interpolated values of the density, used for compressible
c                    comp.
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     workfl   i/o   Work array containing the pseudo velocity of Newton
c                    linearisation
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer acttyp, vartyp, i, j, pnttyp
      double precision dummy(1)

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     dummy     Dummy
c     i         General loop variable
c     j         General loop variable
c     pnttyp    reserved
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINTCF2  Compute interpolation or finite differences of a
c               coefficient in an integration point
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c   In case of "Picard" type linearisation of convective terms,
c   the convective terms are linearised as:
c
c              n   n+1
c           ( U  m     )
c                       x
c
c  In case of Newton linearisation the canvective terms are linearised
c  as;
c            n+1 n    n  n+1   n n
c          (U   m  + U  m  -  U m )
c                                  x
c    or:
c            n+1 n    n  n+1   n n
c       (   m   m  + U  m  -  U m )
c         ----- x
c             n+1
c          rho
c
c    or:
c               n    n   n+1   n n
c         ((   m  + U  )m  -  U m )
c            ----- x
c                n+1
c             rho
c
c   the term
c               n    n
c          (   m  + U  )
c            -----
c                n+1
c             rho
c  is calculated by iscmfl2
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('iscmfl2')

      do j = 1-nvirtual, nj+1+nvirtual
         do i = 1-nvirtual, ni+1+nvirtual
            intrho(i,j)=0d0
         end do
      end do

      do j = 1-nvirtual, nj+1+nvirtual
         do i = 1-nvirtual, ni+1+nvirtual
            workfl(i,j,1)=0d0
         end do
      end do

      do j = 1-nvirtual, nj+1+nvirtual
         do i = 1-nvirtual, ni+1+nvirtual
            workfl(i,j,2)=0d0
         end do
      end do

      vartyp = 3

c     --- interpolate compressible density to V-1 integration points

      pnttyp = 3
      acttyp = 0

      call isintcf2 ( solut(1-nvirtual,1-nvirtual,4,2),
     +                intrho, ni, nj, nvirtual,
     +                vartyp, acttyp, pnttyp, 0, dummy )

c     --- compute component in the i direction

      do j = 2-nvirtual, nj+nvirtual

         i=1-nvirtual

         workfl(i,j,1) = solut(i,j,1,1)+eddy(i,j,2)/intrho(i+1,j)

         do i = 2-nvirtual, ni+nvirtual
            workfl(i,j,1) = solut(i,j,1,1)+eddy(i,j,2)/intrho(i,j)
         end do

         i=1-nvirtual

         workfl(i,j,1) = solut(i,j,1,1)+eddy(i,j,2)/intrho(i+1,j)

      end do

c     --- interpolate compressible density to V-2 integration points

      do i = 1-nvirtual, ni+1+nvirtual
         do j = 1-nvirtual, nj+1+nvirtual
            intrho(i,j)=0d0
         end do
      end do

      pnttyp = 2
      acttyp = 0

      call isintcf2 ( solut(1-nvirtual,1-nvirtual,4,2),
     +                intrho, ni, nj, nvirtual,
     +                vartyp, acttyp, pnttyp, 0, dummy )

c     --- compute component in the i direction

      j = 1-nvirtual
      do i = 2-nvirtual, ni+nvirtual

         workfl(i,j,2) = solut(i,j,2,1)+eddy(i,j,3)/intrho(i,j+1)

      end do

      do j = 2-nvirtual, nj+nvirtual
         do i = 2-nvirtual, ni+nvirtual

            workfl(i,j,2) = solut(i,j,2,1)+ eddy(i,j,3)/intrho(i,j)

         end do
      end do

      j = nj+1+nvirtual
      do i = 2-nvirtual, ni+nvirtual
         workfl(i,j,2) = solut(i,j,2,1)+eddy(i,j,3)/intrho(i,j-1)
      end do

      call iserclos('iscmfl2')

      end
