      subroutine iscmrho2 ( ni, nj, ndegfd, ndim, nvirtual,
     +                      solut, rincom, eddy, istrt, iend,
     +                      jstrt, jend, mscal, mcav, itimelv,
     +                      istartup, rincav, iincom )
c ======================================================================
c
c     programmer   Hester Bijl
c     version      1.6    date 04-07-1999 Guus: extend with user written
c                                         subroutine
c     version      1.5    date 04-03-1999 Duncan: Cavity input added
c     version      1.4    date 22-07-1998 Duncan: Extension to handle
c                                         different time levels
c     version      1.3    date 05-03-1998 Duncan: Extension for cavity
c     version      1.2    date 12-11-1995 Hester:parameter mscal added
c     version      1.1    date 05-07-1995 Hester: cleaned up
c     version      1.0    date 30-05-1995
c
c
c   copyright (c) 1995-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   ISnas CoMpressible computation of density, RHO, from the equation
c   of state, 2D.
c **********************************************************************
c
c                       KEYWORDS
c
c    compressible
c    density
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, ndegfd, ndim, nvirtual, istrt, iend,
     +        jstrt, jend, mscal, mcav, itimelv, istartup, iincom(*)
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,1:ndegfd,2),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,2), rincom(6),
     +                 rincav(*)

c     eddy          i/o   array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     iend           i    End value for a loop variable
c     iincom         i    Array IINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     istartup       i    If 1 the cavity calculation is in the startup
c                         procedure, if 0 not
c     istrt          i    Starting value for a loop variable
c     itimelv        i    Timelevel in eddy under consideration
c     jend           i    End value for a loop variable
c     jstrt          i    Starting value for a loop variable
c     mcav           i    If 1 a cavity model is used, if 0 not
c     mscal          i    Indicates use of non-dimensionalisation for
c                         compr. flow:
c                         0:   Equations are non-dimensionalised
c                         1:   Equations are in dimensional formulation
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rincav         i    Array RINCAV is part of array RINPUT and hence is
c                         coupled to one block only.
c                         For a description see the programmers guide
c     rincom         i    Array RINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, meqstate, nk
      double precision gamcm, mach2, pv, cons, cons1, cons2, dummy

c     cons           Constant used in equation of state for compressible
c                    computations
c     cons1          Constant used in equation of state for compressible
c                    computations
c     cons2          Constant used in equation of state for compressible
c                    computations
c     dummy          Dummy
c     gamcm          Gas constant gamma (for compressible comput.)
c     i              General loop variable
c     j              General loop variable
c     mach2          Square of the mach number (for compressible comput.)
c     meqstate       Defines the type of equation of state
c                    See the programmers Guide array IINCOM for the
c                    possible values
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     pv             Ratio of outlet and inlet pressure used for
c                    compressible flow in the equation of state
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
        double precision isstdls, isstdfl

c     ISSTDFL        dimensionfull equation of state
c     ISSTDLS        dimensionless equation of state
c     USCALCRHO      User subroutine to compute the density from the pressure
c                    and the energy
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c          for all points do
c
c          if ( mcav.eq.1) then
c              if (mscal.eq.1) then
c                 rho = rho(dimensional pressure)
c              else
c                 rho = rho(dimensionless pressure)
c              end if
c          else
c              if (mscal.eq.1) then
c                  rho = cons*p/h
c              else
c                  rho = cons1(mach,gamcm)*p/h + cons2(p_v,mach,gamcm)*1/h
c              end if
c          end if
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      gamcm = rincom(1)
      mach2 = rincom(2)**2d0
      pv    = rincom(5)
      meqstate  = iincom(9)
      nk = 0

      if ( meqstate.eq.1 ) then

c     --- User provided equation of state

         call uscalcrho ( eddy, solut, ni, nj, nk, nvirtual, ndim,
     +                    ndegfd, istrt, iend, jstrt, jend, 0, 0 )

      else if ( mscal.eq.0 ) then

c     --- nondimensional, scaled computation

        cons  = 1d0 + ((gamcm-1d0)*mach2)/2d0
        cons1 =  (gamcm*mach2)/cons
        cons2 =  pv*((cons**(-gamcm/(gamcm-1d0))) - 1d0) + 1d0

         do i = istrt, iend
           do j = jstrt, jend

               if (mcav.eq.1) then

                  eddy(i,j,itimelv)  =
     +                 isstdls(solut(i,j,ndim+1,1), rincom,
     +                                          istartup, rincav)

               else

                  eddy(i,j,itimelv)  = cons1*solut(i,j,ndim+1,1)/
     +            solut(i,j,ndim+2,2) + cons2/solut(i,j,ndim+2,2)

               end if

           end do
         end do

      else if (mscal.eq.1) then

c     --- dimensional, unscaled computation

        cons=gamcm/(gamcm-1d0)

        do i = istrt, iend
           do j = jstrt, jend

               if (mcav.eq.1) then

                  dummy = isstdfl(solut(i,j,ndim+1,1),
     +                                              istartup, rincav)
                  eddy(i,j,itimelv)  = isstdfl(solut(i,j,ndim+1,1),
     +                                              istartup, rincav)

               else

                  eddy(i,j,itimelv)  = cons*solut(i,j,ndim+1,1)/
     +                       solut(i,j,ndim+2,2)

               end if

           end do
        end do

      end if

      end
