      subroutine iscmvel2 ( ni, nj, nvirtual, solut, comrho, intrho )
c ======================================================================
c
c     programmer   Hester Bijl
c     version      2.4    date 27-01-1999 Guus: extend to virtual cells
c     version      2.3    date 21-10-1997 Hester: stored rho*u in eddy
c     version      2.2    date 03-07-1997 Guus: new call to isintcf2
c     version      2.1    date 07-12-1995 Marcel: local parameter acttyp
c     version      2.0    date 10-10-1995 Guus: new parameter list
c     version      1.1    date 05-07-1995 Hester: cleaned up
c     version      1.0    date 30-05-1995
c
c
c   copyright (c) 1995-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    ISnas for CoMpressible flow compute VeLocity components, 2 D.
c    Since in compressible case on the new time level fluxes, density*V,
c    are computed, these fluxes are divided here by the density.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  ni, nj, nvirtual
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,*),
     +                 comrho(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,3),
     +                 intrho(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)

c     comrho    i/o    The density for compressible computations.
c     intrho    i    Interpolated values of the density, used for compressible
c                    comp.
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer acttyp, vartyp, i, j, pnttyp
      double precision dummy(1)

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     dummy     Dummy
c     i         General loop variable
c     j         General loop variable
c     pnttyp    reserved
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINTCF2  Compute interpolation or finite differences of a
c               coefficient in an integration point
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('iscmvel')

      do i = 1-nvirtual, ni+1+nvirtual
         do j = 1-nvirtual, nj+1+nvirtual
            intrho(i,j)=0d0
         end do
      end do

c     --- store rho*u at old time level in eddy(*,*,3) and eddy(*,*,4)
c         ,equal to comrho(*,*,2) and (*,*,3)

      do j = 1-nvirtual, nj+1+nvirtual
         do i = 1-nvirtual, ni+1+nvirtual
            comrho(i,j,2) = solut(i,j,1)
            comrho(i,j,3) = solut(i,j,2)
         end do
      end do

      vartyp = 3

c     --- interpolate compressible density to V-1 integration points

      pnttyp = 3
      acttyp = 0
      call isintcf2 ( comrho, intrho, ni, nj, nvirtual, vartyp,
     +                acttyp, pnttyp, 0, dummy )

c     --- compute first velocity component from the previously computed
c         fluxes: u = V-1/rho

      do i = 2-nvirtual, ni+nvirtual
         do j = 2-nvirtual, nj+nvirtual
            solut(i,j,1) = solut(i,j,1)/intrho(i,j)
         end do
      end do

c     --- interpolate compressible density to V-2 integration points

      pnttyp = 2
      acttyp = 0
      call isintcf2 ( comrho, intrho, ni, nj, nvirtual, vartyp,
     +                acttyp, pnttyp, 0, dummy )

c     --- compute second velocity component from the previously computed
c         fluxes: v = V-2/rho

      do i = 2-nvirtual, ni+nvirtual
         do j = 2-nvirtual, nj+nvirtual
            solut(i,j,2) = solut(i,j,2)/intrho(i,j)
         end do
      end do

      call iserclos('iscmvel')

      end
