      subroutine isconc2d ( output, vector1, vector2,
     +                      jcta, ni, nj, nvirtual, alphad )
c ======================================================================
c
c     programmer  Charles Moulinec
c     version     1.2   04-06-1998    Charles: change in the loops
c     version     1.1   05-11-1997    Charles: correction
c     version     1.0   15-07-1997
c
c   copyright (c) 1997-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the calculation of the contravariant
c     components at the faces from the Cartesian components.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad
      double precision jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2),
     +                 output(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)
      double precision vector1(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual),
     +                 vector2(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     output    o    Input flow field (cartesian) converted in output
c                    flow field (contravariant)
c     vector1   i    Storage vector for the first component
c     vector2   i    Storage vector for the second component
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp

c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- transform to contravariant components

      do jp = 2-nvirtual, nj+nvirtual
         do ip = 2-nvirtual, ni+nvirtual
            output(ip,jp) = jcta(ip,jp,1,alphad) * vector1(ip,jp)
     +                    + jcta(ip,jp,2,alphad) * vector2(ip,jp)
         end do
      end do

      end
