      subroutine iscont2d ( adir, jac, cva, ni, nj, nvirtual, alphad )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date 11-02-1998 Charles: invjac removed
c        version 2.0    date 02-12-1997 Extra parameter jac;
c                                       Correction formulas
c        version 1.0    date 12-06-1996
c
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute contravariant base vectors in centroids of alphad cells
c    two-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad
      double precision adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,4)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, pnttyp
      double precision const

c     const     General const
c     i         General loop variable
c     j         General loop variable
c     pnttyp    reserved
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The contravariant base vectors may be computed from the covariant ones
c     in the following way:
c
c      (1)                  2     1
c     a    =  1/sqrt(g) ( a    ,-a   )
c                          (2)    (2)
c
c
c      (2)                  2     1
c     a    =  1/sqrt(g) (-a    , a   )
c                          (1)    (1)
c
c
c     In this subroutine only the alphad vector is filled.
c     Since the contravariant vector is only required in the centres of
c     the integration cells, it is sufficient to loop over all unknowns
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( alphad.eq.1 ) then

c     --- alphad = 1 cell

         pnttyp = 3

         do j = 1, nj
            do i = 1, ni+1
               const = 1d0 / (0.5d0*jac(i,j,pnttyp) +
     +                  0.25d0*(jac(i-1,j,4)+jac(i,j,4)) )
               adir(i,j,1) = cva(i,j,2,2,pnttyp)*const
               adir(i,j,2) =-cva(i,j,1,2,pnttyp)*const
            end do
         end do

      else

c     --- alphad = 2 cell

         pnttyp = 2

         do j = 1, nj+1
            do i = 1, ni
               const = 1d0 / (0.5d0*jac(i,j,pnttyp) +
     +                  0.25d0*(jac(i,j-1,4)+jac(i,j,4)) )
               adir(i,j,1) =-cva(i,j,2,1,pnttyp)*const
               adir(i,j,2) = cva(i,j,1,1,pnttyp)*const

            end do
         end do

      end if

      end
