      subroutine isconti3 ( adiri, cva, ni, nj, nk, nvirtual, locdx,
     +                      alphad )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 11-05-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute contravariant base vectors times jacobian
c    three-dimensional case in the integration points
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad, locdx(3)
      double precision adiri(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,*)

c     adiri     i    Contravariant base vector in integration point
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, pnttyp, valpha(3), vbeta(3), vgamma(3), betal,
     +        betad

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     betal     Loop index
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     pnttyp    reserved
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c     vbeta     Array for cyclic changing ordering
c               betal = 1        betal = 2    betal = 3
c               vbeta(1) =1 ---> vbeta(1)=2  vbeta(1)=3
c               vbeta(2) =2 ---> vbeta(2)=3  vbeta(2)=1
c               vbeta(3) =3 ---> vbeta(3)=1  vbeta(3)=2
c     vgamma    Array for cyclic changing ordering
c               gammal = 1        gammal = 2    gammal = 3
c               vgamma(1) =1 ---> vgamma(1)=2  vgamma(1)=3
c               vgamma(2) =2 ---> vgamma(2)=3  vgamma(2)=1
c               vgamma(3) =3 ---> vgamma(3)=1  vgamma(3)=2
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The contravariant base vectors may be computed from the covariant ones
c     in the following way:
c
c      (1)                  2     1
c     a    =  1/sqrt(g) ( a    ,-a   )
c                          (2)    (2)
c
c
c      (2)                  2     1
c     a    =  1/sqrt(g) (-a    , a   )
c                          (1)    (1)
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      valpha(1) = alphad
      valpha(2) = mod(alphad  ,3) + 1
      valpha(3) = mod(alphad+1,3) + 1

c     --- initialize adiri

      do k = 1-nvirtual, nk+1+nvirtual
         do j = 1-nvirtual, nj+1+nvirtual
            do i = 1-nvirtual, ni+1+nvirtual
               adiri(i,j,k,1) = 0d0
               adiri(i,j,k,2) = 0d0
               adiri(i,j,k,3) = 0d0
            end do
         end do
      end do

c     --- determine which a^beta must be computed and in which point

      if ( locdx(valpha(1)).ne.0 ) then
         betad = valpha(1)
         pnttyp = 8
      else if ( locdx(valpha(2)).ne.0 ) then
         betad = valpha(2)
         if ( valpha(1).eq.1) then
            pnttyp = 5
         else if ( valpha(1).eq.2) then
            pnttyp = 2
         else
            pnttyp = 3
         end if
      else
         betad = valpha(3)
         if ( valpha(1).eq.1) then
            pnttyp = 3
         else if ( valpha(1).eq.2) then
            pnttyp = 5
         else
            pnttyp = 2
         end if

      end if

c     --- Rest is copy from icont3d, with alpha replaced by beta

      vgamma(1) = betad
      vgamma(2) = mod(betad  ,3) + 1
      vgamma(3) = mod(betad+1,3) + 1

      do betal = 1, 3
         vbeta(1) = betal
         vbeta(2) = mod(betal  ,3) + 1
         vbeta(3) = mod(betal+1,3) + 1
         do k = 1, nk+1
            do j = 1, nj+1
               do i = 1, ni+1
                 adiri(i,j,k,betal) =
     +               (cva(i,j,k,vbeta(2),vgamma(2),pnttyp)*
     +                cva(i,j,k,vbeta(3),vgamma(3),pnttyp)-
     +                cva(i,j,k,vbeta(3),vgamma(2),pnttyp)*
     +                cva(i,j,k,vbeta(2),vgamma(3),pnttyp))
               end do
            end do
         end do
      end do

      end
