      subroutine iscrmo2d ( iinput, npointsv, ndegfd, alphad, bndcon,
     +                      rhside, work, coor, solut, coefs, jac,
     +                      isiinput, issolut, isitopol,
     +                      ibinfor, ibipoint, iseqpres, gmat, gmctinv,
     +                      invjac, idiscret,
     +                      bicof, ibicof )
c ======================================================================
c
c     programmer   Charles Moulinec
c     version      1.3   date    24-08-1998 Kees K.:ipdrhside added
c     version      1.2   date    04-06-1998 call to iscpgmo2d changed
c     version      1.1   date    15-12-1997 extra arguments gmat
c     version      1.0   date    15-07-1997
c
c
c   copyright (c) 1997-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     create the right-hand side of the momentum equation for collocated
c     arrangement (pressure gradient and tranposed part of the stress
c     term)
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), npointsv, ndegfd, alphad,
     +        isiinput(*), isitopol, issolut(*),
     +        ibipoint(*), ibinfor(*), iseqpres, gmctinv(0:3,2,6),
     +        idiscret
      integer ibicof(*)
      double precision bicof(*)
      double precision bndcon(2,*), rhside(*), work(*), coor(*),
     +                 solut(npointsv,ndegfd,*), coefs(npointsv,*),
     +                 jac(npointsv,*), invjac(npointsv,*), gmat(*)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bicof     i    Array containing bilinear coefficients
c     bndcon    i    reserved
c     coefs    i/o   reserved
c     coor      i    reserved
c     gmat      i    Gradient matrix
c     gmctinv   i    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     ibicof    i    Integer array containing bilinear coefficients
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     idiscret  i    Type of discretization
c     iinput    i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     iseqpres  i    Sequence number of pressure equation
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     jac       i    sqrt(g) in pnttyp
c     ndegfd    i    Number of degrees of freedom
c     npointsv  i    Number of points in the mesh with virtual points
c     rhside    o    reserved
c     solut     i    reserved
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ni, nj, nvirtual, mshtyp, ipcva,
     +        ipjcta, ipkdif1, ipkdif2, ipkdif3,
     +        ipcvc, ipctc, pnttyp, vartyp, acttyp,
     +        nblocks, iblock, itimlv, itrans,
     +        iside, icoupled, ipadir, ipdrhside
      logical usebndcn

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ipadir    Starting address of array ADIR in array WORK
c     ipctc     Starting address of array CTC in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipcvc     Starting address of array CVC in array WORK
c     ipdrhside Starting address of array DRHSIDE in array WORK
c     ipjcta    Starting address of array JCTA in array WORK
c     ipkdif1   Starting address of array KDIF1 in array WORK
c     ipkdif2   Starting address of array KDIF2 in array WORK
c     ipkdif3   Starting address of array KDIF3 in array WORK
c     iside     Sequence number of side
c     itimlv    Loop variable with respect to a given time level
c     itrans    reserved
c     mshtyp    Type of grid arrangement
c     nblocks   Number of blocks
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     usebndcn  If true boundary conditions are used in the extrapolation
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCEXST2D Compute the explicit part of the stress term in the
c               momentum equation for collocated arrangement
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISCPGMO2D Give contribution of the pressure for collocated in R2
c     ISCRSB2D  Correct right-hand side of the collocated momentum equation
c     ISFILVSO  Extrapolate solution at outer boundaries
c     ISINTCF2  Compute interpolation or finite differences of a
c               coefficient in an integration point
c     ISJCTA2D  Fill Jacobian times contravariant base vectors
c               in one integration point in R2
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute constants

      ni         = iinput(51)
      nj         = iinput(52)
      nvirtual   = iinput(54)
      mshtyp     = iinput(75)
      nblocks    = iinput(73)
      npointsv   = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      ipcva      = 1
      ipjcta     = ipcva+16*npointsv
      ipkdif1    = ipjcta+7*npointsv
      ipkdif2    = ipkdif1+npointsv
      ipkdif3    = ipkdif2+npointsv
      ipcvc      = ipkdif3+npointsv
      ipctc      = ipcvc+4*npointsv
      ipadir     = ipctc+4*npointsv
      ipdrhside  = ipadir+2*npointsv

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- Fill contravariant basis vectors in cell centre

      pnttyp = 4

      call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                work(ipjcta), ni, nj, nvirtual )

c     --- Compute the pressure gradient in the right-hand side of the
c         momentum equation

      call iscpgmo2d ( ni, nj, nvirtual, rhside, jac,
     +                 alphad, work(ipcva), work(ipcvc), work(ipctc),
     +                 solut(1,iseqpres,1), gmat, gmctinv, invjac,
     +                 ibipoint, ibinfor, idiscret, work(ipadir),
     +                 bicof, ibicof )

c     --- Compute extrapolation of the pressure on the boundaries

      iblock = 1
      itimlv = 1
      itrans = 3
      icoupled = 1
      usebndcn = .false.
      call isfilvso ( iblock , itrans, itimlv, nblocks, isiinput,
     +                issolut, isitopol, isitopol, isitopol,
     +                icoupled, usebndcn )

c     --- Compute the diffusive coefficient in the stress-term
c         at the faces

      call isintcf2 ( coefs(1,2), work(ipkdif1), ni, nj,
     +                nvirtual, vartyp, acttyp, pnttyp,
     +                mshtyp, coor )
      call isintcf2 ( coefs(1,6), work(ipkdif2), ni, nj,
     +                nvirtual, vartyp, acttyp, pnttyp,
     +                mshtyp, coor )
      call isintcf2 ( coefs(1,2), work(ipkdif3), ni, nj,
     +                nvirtual, vartyp, acttyp, pnttyp,
     +                mshtyp, coor )

c     --- Compute the transposed part of the stress-term

      call iscexst2d ( ni, nj, nvirtual, work(ipkdif1),
     +                 alphad, work(ipcva), work(ipjcta),
     +                 work(ipcvc), work(ipctc), solut,
     +                 rhside, work(ipdrhside) )

c     --- Compute effect of boundary conditions

      do iside = 1 , 4

         call iscrsb2d ( iside, ni, nj, nvirtual, solut(1,alphad,1),
     +                   alphad, ibinfor, ibipoint, bndcon, rhside,
     +                   work(ipjcta) )

      end do

      end
