      subroutine iscrsb2d ( iside, ni, nj, nvirtual, solut, alphad,
     +                      ibinfor, ibipoint, bndcon, rhside, jcta )
c ======================================================================
c
c        programmer  Charles Moulinec
c        version 1.0    date 15-07-1997
c
c   copyright (c) 1997-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Correct right-hand side of explicit part of stress-term
c   for Robbins type boundary conditions for the collocated
c   momentum equation in R2
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibinfor(*), ibipoint(*), iside, alphad
      double precision bndcon(2,*),
     +                 rhside(ni+1,nj+1),
     +                 jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2)
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iside     i    Sequence number of side
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer robb
      parameter ( robb = 2 )
      integer iarea(2,2), ip, jp, mip, mjp, mxp(2), ishift,
     +        jshift, tdir, ndir, nsign, tsign, ipibinf, ict1
      double precision ljcta

c     iarea     Area of arrays
c     ict1      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     ishift    Shift in i-direction
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     ljcta     Length of a contravariant base vector times Jacobian
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     robb      Robbins boundary condition, i.e. aT + (k grad T).n given
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   Consider a boundary cell at the lower boundary:
c
c
c
c                -------|-------
c                |             |
c                |             |
c                |             |
c                -      +      -
c                |             |
c                |             |
c                |             |
c           ------------|------------------
c
c
c   For the internal elements we have a contribution for all four
c   integration points.
c
c   For the classical discretization the contribution to the lower
c   boundary is:
c
c              ab        |
c   - sqrt(g) K     T,   |              (a=1)
c                     b  | (0,-1)
c
c   Because of the Robbins boundary condition this term is replaced by:
c
c       / brhs - sigma T,
c   boundary
c
c   where brhs is stored in bndcon(2,.) and sigma in bndcon(1,.)
c
c   The integral is equal to
c
c   g_tt (brhs - sigma T) = sqrt(g) sqrt(g^nn) (brhs - sigma T)
c
c   Hence the dtsum contribution for the boundary point should be made zero,
c   and the contribution to matrix and right-hand side are given above
c
c   The term T should be distributed over the points (0,0) and (0,-1),
c   with a factor 1/2, since (0,-1/2) does not contain a transport unknown
c
c   However, since (0,-1) is a virtual point, this point is folded
c   inwardly in the following way:
c
c   T(0,-1) = 2T(0,0) - T(0,1)
c
c   Hence the final distribution becomes:
c
c   T(0,-1/2) = 3/2 T(0,0) - 1/2T(0,1)
c
c   In the Wesbeek we follow the same approach
c **********************************************************************
c
c
c
c
c
c
c
c
c
c
c                      PSEUDO CODE
c
c   Compute constants depending on the side sequence number
c   For all mid side points at the boundary iside do
c     if ( boundary condition is Robbns condition) then
c        set dtsum equal to zero
c        update matrix and right-hand side
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      iarea(1,1) = 1
      iarea(2,1) = ni
      iarea(1,2) = 1
      iarea(2,2) = nj
      ishift = 0
      jshift = 0
      if ( iside.eq.1 ) then
         iarea(2,1) = 1
      else if ( iside.eq.2 ) then
         iarea(1,1) = ni
         ishift = 1
      else if ( iside.eq.3 ) then
         iarea(2,2) = 1
      else
         iarea(1,2) = nj
         jshift = 1
      end if
      ipibinf = ibipoint(iside)-1+nvirtual

c     --- Compute tdir and ndir

      call isorient2 ( iside, tdir, ndir, tsign, nsign )

c     --- WesBeek discretization
c         loop over boundary cells along side iside

         do mjp = iarea(1,2), iarea(2,2)
            do mip = iarea(1,1), iarea(2,1)

c           --- Get integration cell index

               mxp(1) = mip
               mxp(2) = mjp
               ict1   = ipibinf+2*mxp(tdir)

c              --- Get integration cell position for ibinfor

               if ( ibinfor(ict1).eq.robb ) then

c              --- Robbins boundary conditions found

                  ip = mip+ishift
                  jp = mjp+jshift
                  ljcta = sqrt(jcta(ip,jp,1,ndir)**2 +
     +                         jcta(ip,jp,2,ndir)**2) * nsign

                  rhside(mip,mjp) = rhside(mip,mjp) + ljcta *
     +                            ( bndcon(2,ict1) -
     +                              bndcon(1,ict1) *
     +                            ( 1.5d0 * solut(mip,mjp,alphad) -
     +                              0.5d0 * solut(mip+nsign*ishift,
     +                                            mjp+nsign*jshift,
     +                                            alphad) ) )

               end if

            end do
         end do

      end
