      subroutine isdijup ( alphad, intn, dijup )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 30-093-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Wesbeek discretization first order upwind
c     Compute some quantities with respect to interpolation, position and so
c     on for one integration point and one direction. (2D)
c **********************************************************************
c
c                       KEYWORDS
c
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer alphad, intn, dijup(2,6,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dijup     o    Array containing the distances of the interpolation
c                    points to the cell centra in global co-ordinates for
c                    first order upwind.
c                    dijup(alpha,i,j) denotes the distance of the i-th
c                    interpolation point, component j and with respect to
c                    the V-alpha unknown (distribution)
c     intn      i    Integration point number
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer betad, locdx(2), i, j, k

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c        Create first 2D list of local coordinates of the
c        integration points w.r.t. the integration cell centre,
c        for the momentum matrix integration cells. These local
c        coordinates are stored in array locdx, which is filled
c        for one celltype.
c        In 2D the number of integration points for the internal
c                             1
c        cells is 5, for the V  cell point 1 is integration cell centre,
c        points 2 and 3 are the midpoints of the left and rigth side,
c        midpoints of the left and rigth side, points 4 and 5 are the
c        midpoints of the lower and upper side of the integration cell
c
c        In this routine for the wesbeek discretization per
c        integration point ( except for the integration cell centre)
c        the position of the integration point relative to the
c        cell centre is given in locdx.
c        For first order upwind per integration point for the
c        alphad unknown 2 points are involved, and for the
c        betad direction 6 points. For these 2 and 6 points
c        the position relative to the cell centre is given in 
c        dijup.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- initialize dijup

      do k = 1,2
         do j = 1, 6
            do i = 1, 2
               dijup(i,j,k) = 0
            end do
         end do
      end do

      betad  = 3-alphad

      go to ( 100, 200, 300, 400, 500 ), intn

c     --- Integration cell centre          integration point 1:(0,0)

100      continue
         go to 1000

c        --- integration point 2: V1:(-1,0),   V2:(0,-1)

200      locdx(alphad) = -1
         locdx(betad)  =  0
         dijup(alphad,1,1) = locdx(1)
         dijup(alphad,1,2) = locdx(2)
         dijup(alphad,2,1) = 0
         dijup(alphad,2,2) = 0
         dijup(betad,1,alphad) = -2
         dijup(betad,1,betad)  = 0
         dijup(betad,2,alphad) = -2
         dijup(betad,2,betad)  = 1
         dijup(betad,3,alphad) = -1
         dijup(betad,3,betad) = 0
         dijup(betad,4,alphad)  = -1
         dijup(betad,4,betad) = 1
         dijup(betad,5,alphad)  = 0
         dijup(betad,5,betad) = 0
         dijup(betad,6,alphad)  = 0
         dijup(betad,6,betad)  = 1
         go to 1000

c        --- integration point 3: V1:(1,0),   V2:(0,1)

300      locdx(alphad) =  1
         locdx(betad)  =  0
         dijup(alphad,1,1) = 0
         dijup(alphad,1,2) = 0
         dijup(alphad,2,1) = locdx(1)
         dijup(alphad,2,2) = locdx(2)
         dijup(betad,1,alphad) = -1
         dijup(betad,1,betad)  = 0
         dijup(betad,2,alphad) = -1
         dijup(betad,2,betad)  = 1
         dijup(betad,3,alphad) = 0
         dijup(betad,3,betad)  = 0
         dijup(betad,4,alphad) = 0
         dijup(betad,4,betad)  = 1
         dijup(betad,5,alphad) = 1
         dijup(betad,5,betad)  = 0
         dijup(betad,6,alphad) = 1
         dijup(betad,6,betad)  = 1
         go to 1000

c        --- integration point 4: V1:(0,-1),   V2:(-1,0)

400      locdx(alphad) =  0
         locdx(betad)  = -1
         dijup(alphad,1,1) = locdx(1)
         dijup(alphad,1,2) = locdx(2)
         dijup(alphad,2,1) = 0
         dijup(alphad,2,2) = 0
         dijup(betad,1,alphad) = -1
         dijup(betad,1,betad)  = -1
         dijup(betad,2,alphad) = 0
         dijup(betad,2,betad)  = -1
         dijup(betad,3,alphad) = -1
         dijup(betad,3,betad)  = 0
         dijup(betad,4,alphad) = 0
         dijup(betad,4,betad)  = 0
         dijup(betad,5,alphad) = -1
         dijup(betad,5,betad)  = 1
         dijup(betad,6,alphad) = 0
         dijup(betad,6,betad)  = 1
         go to 1000

c        --- integration point 5: V1:(0,1),   V2:(1,0)

500      locdx(alphad) =  0
         locdx(betad)  =  1
         dijup(alphad,1,1) = 0
         dijup(alphad,1,2) = 0
         dijup(alphad,2,1) = locdx(1)
         dijup(alphad,2,2) = locdx(2)
         dijup(betad,1,alphad) = -1
         dijup(betad,1,betad)  = 0
         dijup(betad,2,alphad) = 0
         dijup(betad,2,betad)  = 0
         dijup(betad,3,alphad) = -1
         dijup(betad,3,betad)  = 1
         dijup(betad,4,alphad) = 0
         dijup(betad,4,betad)  = 1
         dijup(betad,5,alphad) = -1
         dijup(betad,5,betad)  = 2
         dijup(betad,6,alphad) = 0
         dijup(betad,6,betad)  = 2

1000  continue

      end
