      subroutine isdisc2d ( ni, nj, nvirtual, matrix, vsum,
     +                      dcell, alphad, iarea, iindsc,
     +                      nintpol, dij, mct, intn )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.2    date 04-11-1996 Kees K contribution to rhside
c                                       removed here
c        version 2.1    date 22-03-1996 Marcel parameter list changed
c        version 2.0    date 19-10-1995 New parameter list
c        version 1.0    date 23-05-1995 Extracted from subroutine isdstndf
c                                       of Ronald van Nooyen
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Distribution of interpolation or finite difference formulas
c    over integration point with respect to the convection term
c **********************************************************************
c
c                       KEYWORDS
c
c     distribution
c     discretization
c     matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, dcell(2), iindsc(*), alphad, intn,
     +        iarea(2,2), nintpol(2), dij(2,4,2),
     +        mct(2,2,-2:2,-2:2,-2:2)
      double precision matrix((ni+1)*(nj+1),2,*),
     +                 vsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     i    reserved
c     dij       i    Array containing the distances of the interpolation
c                    points to the cell centra in global co-ordinates
c                    dij(alpha,i,j) denotes the distance of the i-th
c                    interpolation point, component j and with respect to
c                    the V-alpha unknown (distribution)
c     iarea     i    Area of arrays
c     iindsc    i    reserved
c     intn      i    Integration point number
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nintpol   i    Number of interpolation points in the distribution
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, kdir, iconct
      double precision fact

c     fact      Multiplication factor
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     kdir      General loop variable for 1...ndim loop
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXPY2D  Perform a vector update for ISNaS over a 2-dimensional region
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    This is the main distribution subroutine for the convection
c    This subroutine is called in a loop over the integration points and
c    in the case of a momentum equation also in a loop over the unknowns
c    The contribution of the integration point to the molecule has been stored
c    in the array vsum which has the length of all points in the mesh
c    including the virtual points
c    This array is built as if the integration point is an entry of
c    the matrix.
c    However, in general the integration point does not contain the
c    unknown to which the matrix corresponds.
c
c    For example consider the V1 component in the V1 row of the convection
c    matrix in R2 (Newton linearization)
c
c          *----------------*-----------------*
c          |                |                 |
c          |                |                 |
c          |                |                 |
c          *----------------*-----------------*
c          |        |       |        |        |
c          |                -i,j              |
c          |        |       |        |        |
c          *----------------*-----------------*
c
c    One of the contributions to the molecule is the term
c
c    rho/sqrt(g) V2 in the integration point i,j+1/2
c
c    This contribution has been computed before and stored in array vsum
c
c    However in point i,j+1/2 there is no V1 unknown and this contribution
c    must be distributed over the two surrounding points i,j and i,j+1
c
c    In this subroutine the distribution takes place and the corresponding
c    matrix elements (rho/sqrt(g) V2 )/2 are placed in the matrix in the
c    correct positions
c
c    The following tables and parameters define how the distrution must be
c    carried out:
c
c    dcell contains the global coordinate distance from integration point
c          to the centre of the integration cell (in the example (0,1))
c    dij   contains the global coordinate distance from the distribution points
c          to the centre of the integration cell (in the example (0,1))
c    nintpol contains the number of distribution points
c    mct   contains the column number in the matrix where the value
c          must be stored
c
c                    PSEUDO CODE
c
c
c    For each velocity component in the integration point do
c       fact := 1/(number of distribution points)
c       for i := 1 (1) number of distribution points do
c          Compute diagonal in matrix in which the coefficient must be put
c          Add vsum to matrix
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      do kdir = 1, 2

c     --- Loop over points in interpolation formula

         fact = 1d0/nintpol(kdir)
         do i = 1, nintpol(kdir)

c        --- determine corresponding diagonal in the matrix

            iconct = mct(alphad,kdir,dij(kdir,i,1),
     +                   dij(kdir,i,2),0)

c           --- distribute vsum

            call isaxpy2d ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                      matrix(1,alphad,iconct), intn, i,
     +                      fact, vsum(1,kdir), iarea, iindsc(5) )
         end do

      end do

      end
