      subroutine isdisc3d ( ni, nj, nk, nvirtual, matrix, rhside, vsum,
     +                      veloc, mct, dcell, npf, rspv, cv,
     +                      alphad, pnttyp, iarea, iindsc, intn )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.0    date 22-03-1996 Marcel parameter list changed
c        version 1.0    date 02-06-1995 (copy of isdisc2d
c                                      of Guus Segal)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Distribution of interpolation or finite difference formulas
c    over integration point with respect to the convection term
c **********************************************************************
c
c                       KEYWORDS
c
c     distribution
c     discretization
c     matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, dcell(3), iindsc(*), intn,
     +        alphad, acttyp, pnttyp, iarea(1:2,3),
     +        npf(0:3,0:4,1:8), rspv(1:3,1:8,0:3,0:4,1:8),
     +        mct(3,3,-2:2,-2:2,-2:2)
      double precision cv(1:8,0:3,0:4,1:8),
     +                 matrix((ni+1)*(nj+1)*(nk+1),3,*), rhside(*),
     +                 vsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                      (nk+1+2*nvirtual),*),
     +                 veloc((ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                       (nk+1+2*nvirtual),*)

c     acttyp    i    Type of action to be taken
c                    0      : interpolation for point value
c                    1..ndim: finite difference for partial derivative
c                             in the direction given by value of acttyp
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cv        i    reserved
c     dcell     i    reserved
c     iarea     i    Area of arrays
c     iindsc    i    reserved
c     intn      i    Integration point number
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npf       i    reserved
c     nvirtual  i    Number of rows of virtual cells
c     pnttyp    i    reserved
c     rhside   i/o   reserved
c     rspv      i    reserved
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      o    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, kdir, iconct, dx(3)

c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     kdir      General loop variable for 1...ndim loop
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXPY3D  Perform a vector update for ISNaS over a 3-dimensional region
c     ISAXYZ3D  Perform a vector*vector update for ISNaS over a
c               3-dimensional region
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    This is the main distribution subroutine
c    This subroutine is called in a loop over the integration points and
c    in the case of a momentum equation also in a loop over the unknowns
c    The contribution of the integration point to the molecule has been stored
c    in the array update which has the length of all points in the mesh
c    including the virtual points
c    This array is build as if the integration point is an entry of
c    the matrix.
c    However, in general the integration point does not contain the
c    unknown to which the matrix corresponds.
c
c    For example consider the V1 component in the V1 row of the convection
c    matrix in R2 (Newton linearization)
c
c          *----------------*-----------------*
c          |                |                 |
c          |                |                 |
c          |                |                 |
c          *----------------*-----------------*
c          |        |       |        |        |
c          |                -i,j              |
c          |        |       |        |        |
c          *----------------*-----------------*
c
c    One of the contributions to the molecule is the term
c
c    rho/sqrt(g) V2 in the integration point i,j+1/2
c
c    This contribution has been computed before and stored in array update
c
c    However in point i,j+1/2 there is no V1 unknown and this contribution
c    must be distributed over the two surrounding points i,j and i,j+1
c
c    In this subroutine the distribution takes place and the corresponding
c    matrix elements (rho/sqrt(g) V2 )/2 are placed in the matrix in the
c    correct positions
c
c    The following tables and parameters define how the distrution must be
c    carried out:
c
c    dcell contains the global coordinate distance from integration point
c          to the centre of the integration cell (in the example (0,1))
c    rpsv  contains the relative shift in global co-ordinates of the
c          distributed points to the integration point
c          In the example (i,j+1) has rpsv=(0,0)
c                 and     (i,j)   has rpsv=(0,-1)
c    dx    contains the sum of dcell and rpsv, which means the distance
c          in global co-ordinates from distributed point to cell centre
c          In the example (i,j+1) has dx=(0,1)
c                 and     (i,j)   has dx=(0,0)
c    mct   contains the column number in the matrix where the value
c          must be stored
c    cv    contains the multiplication factor for the distribution
c          in the example 2
c    npf   contains the number of points over which the distribution
c          takes place
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      acttyp = 0
      do 150 kdir = 1, 3

c     --- Loop over points in interpolation formula

         do 100 i = 1, npf(acttyp,kdir,pnttyp)

c        --- shift to point in formula

            dx(1) = dcell(1)+rspv(1,i,acttyp,kdir,pnttyp)
            dx(2) = dcell(2)+rspv(2,i,acttyp,kdir,pnttyp)
            dx(3) = dcell(3)+rspv(3,i,acttyp,kdir,pnttyp)

c           --- determine corresponding diagonal in the matrix

            iconct = mct(alphad,kdir,dx(1),dx(2),dx(3))

            call isaxpy3d ( ni, nj, nk, nvirtual, dcell(1), dcell(2),
     +                      dcell(3), matrix(1,alphad,iconct), intn,
     +                      i, cv(i,acttyp,kdir,pnttyp), vsum(1,kdir),
     +                      iarea, iindsc(5) )

100      continue
         if ( iindsc(8).eq.1 ) then

c        --- Newton linearization
c            The right-hand side is updated by a factor
c            update_matrix * velocity / 2
c            The right-hand side vector is situated in the
c            centres of the integration cells
c            The update matrix is positioned in the
c            integration point. So the multiplication must
c            be caried out in the integration points
c            As a consequence both the velocity vactor and
c            the matrix must be shifted with dcell

            call isaxyz3d ( ni, nj, nk, nvirtual, dcell(1), dcell(2),
     +                      dcell(3), dcell(1), dcell(2), dcell(3),
     +                      rhside, 0.5d0, vsum(1,kdir),
     +                      veloc(1,kdir), iarea )

         end if
150   continue

      end
