      subroutine isexmat2 ( intn, ni, nj, nvirtual, vsum, dvsum,
     +                      invjac, mct, dcell, alphad, iarea, nintpol,
     +                      dij, rhside, solut, ndegfd, nconct )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.1    date 09-10-1996 Guus: extra parameter nconct
c        version 1.0    date 23-05-1995 Extracted from subroutine isdiss2d
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Distribution of interpolation or finite difference formulas
c    over integration point with respect to the stress term
c **********************************************************************
c
c                       KEYWORDS
c
c     distribution
c     discretization
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, dcell(2), intn,
     +        alphad, iarea(1:2,2), dij(2,4,2), nintpol(2),
     +        mct(2,2,-2:2,-2:2,-2:2), ndegfd, nconct
      double precision vsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 dvsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),2,2),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 rhside(ni+1,nj+1,2), solut(*)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     i    reserved
c     dij       i    Array containing the distances of the interpolation
c                    points to the cell centra in global co-ordinates
c                    dij(alpha,i,j) denotes the distance of the i-th
c                    interpolation point, component j and with respect to
c                    the V-alpha unknown (distribution)
c     dvsum     i    Flux derivative contributions to the matrix
c     iarea     i    Area of arrays
c     intn      i    Integration point number
c     invjac    i    1/sqrt(g) in pnttyp
c     mct       i    Integer matrix information
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nintpol   i    Number of interpolation points in the distribution
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, jdir, iconct, kdir, nintder(2,2),
     +        didj(2,2,4,2)
      double precision cv(2,2,4), fact

c     cv        reserved
c     didj      Array containing the distances of the fitinte difference
c               points to the cell centra in global co-ordinates
c               didj(alpha,beta,i,j) denotes the distance of the i-th
c               finite difference point, component j and with respect to
c               the d V-alpha/ d ksi beta derivative
c     fact      Multiplication factor
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     jdir      General loop variable for 1...ndim loop
c     kdir      General loop variable for 1...ndim loop
c     nintder   Number of points in the distribution of the finite
c               difference
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXYZ2D  Perform a vector*vector update for ISNaS over a
c               2-dimensional region
c     ISMOMDS2  Make a list in R2 of the number of points for the
c               differentiation and the corresponding connections for
c               each direction in one integration point
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    This is the main distribution subroutine with respect to the stress
c    This subroutine is called in a loop over the integration points and
c    also in a loop over the unknowns
c    The contribution of the integration point to the molecule has been stored
c    in the arrays vsum and dvsum which have the length of all points
c    in the mesh including the virtual points
c    vsum contains the coefficient for the unknown velocities in the
c    integration points
c    dvsum contains the coefficients for the partial derivatives of the
c    velocities in the computational domain
c    This array is build as if the integration point is an entry of
c    the matrix.
c    However, in general the integration point does not contain the
c    unknown to which the matrix corresponds.
c
c    For example consider the V1 component in the V1 row of the convection
c    matrix in R2 (Newton linearization)
c
c          *----------------*-----------------*
c          |                |                 |
c          |                |                 |
c          |                |                 |
c          *----------------*-----------------*
c          |        |       |        |        |
c          |                -i,j              |
c          |        |       |        |        |
c          *----------------*-----------------*
c
c    One of the contributions to the molecule is the term
c
c    rho/sqrt(g) V2 in the integration point i,j+1/2
c
c    This contribution has been computed before and stored in array update
c
c    However in point i,j+1/2 there is no V1 unknown and this contribution
c    must be distributed over the two surrounding points i,j and i,j+1
c
c    In this subroutine the distribution takes place and the corresponding
c    matrix elements (rho/sqrt(g) V2 )/2 are placed in the matrix in the
c    correct positions
c
c    The following tables and parameters define how the distrution must be
c    carried out:
c
c
c    dcell contains the global coordinate distance from integration point
c          to the centre of the integration cell (in the example (0,1))
c    dij   contains the global coordinate distance from the distribution points
c          to the centre of the integration cell (in the example (0,1))
c    nintpol contains the number of distribution points
c    mct   contains the column number in the matrix where the value
c          must be stored
c    didj  contains the global coordinate distance from the distribution points
c          to the centre of the integration cell in the case of a finite
c          difference
c    nintder contains the number of distribution points for a finite difference
c    cv    contains the multiplication factors for a finite difference
c
c                    PSEUDO CODE
c
c    Compute finite difference formula for partial derivatives of velocities
c    For each velocity component in the integration point do
c       fact := 1/(number of distribution points)
c       for i := 1 (1) number of distribution points do
c          Compute diagonal in matrix in which the coefficient must be put
c          Add vsum to matrix
c       For each derivative in the integration point do
c       for i := 1 (1) number of finite differnece points do
c          Compute diagonal in matrix in which the coefficient must be put
c          Add dvsum to matrix
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Compute finite difference formula for partial derivatives of
c         velocities

      call ismomds2 ( intn, alphad, nintder, cv, didj, nconct )

      do kdir = 1, 2

c     --- Loop over points in interpolation formula

         fact = 1d0/nintpol(kdir)
         do i = 1, nintpol(kdir)

c        --- determine corresponding diagonal in the matrix

            iconct = mct(alphad,kdir,dij(kdir,i,1),
     +                   dij(kdir,i,2),0)

c           --- distribute vsum

            call isexup2d ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                      dij(kdir,i,1), dij(kdir,i,2),
     +                      rhside(1,1,alphad), fact,
     +                      invjac(1,4-kdir), vsum(1,kdir), iarea,
     +                      alphad, iconct, solut, ndegfd )

         end do
         do jdir = 1, 2

            do i = 1, nintder(kdir,jdir)

c           --- determine corresponding diagonal in the matrix

               iconct = mct(alphad,kdir,didj(kdir,jdir,i,1),
     +                      didj(kdir,jdir,i,2),0)

c              --- distribute dvsum

               call isexup2d ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                         didj(kdir,jdir,i,1), didj(kdir,jdir,i,2),
     +                         rhside(1,1,alphad), cv(kdir,jdir,i),
     +                         invjac(1,4-kdir), dvsum(1,kdir,jdir),
     +                         iarea, alphad, iconct, solut, ndegfd )
            end do

         end do

      end do

      end
