      subroutine isglobsh3 ( pnttypin, ishiftlc, ishiftgl, pnttyput )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 20-05-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute the shift in global numbering given the local shift and the
c    point type. Also the new point type number is detected
c    With respect to the local shift, each cell has a length of 2 in each
c    direction
c    3d version
c **********************************************************************
c
c                       KEYWORDS
c
c    point_type
c    shift
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer pnttypin, ishiftlc(3), ishiftgl(3), pnttyput

c     ishiftgl  o    Array containing the shift in global coordinates
c     ishiftlc  i    Array containing the shift in local coordinates
c     pnttypin  i    Point type of point from which shift must be taken
c     pnttyput  o    Point type of point where the shift goes to
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ishiftpn(3,8), locshift(3), k

c     ishiftpn  Array of size ndim x 2^ndim containing the shifts of
c               points in the cell with respect to the left under point
c               ishiftpn(i,pnttyp) contains the shift in i-direction for
c               a point with point type pnttyp
c     k         General loop variable
c     locshift  Array containing the local shifts
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    From the point type we can detect a local shift with respect to
c    the "left under" point of a cell (point type 1)
c    This local shift is stored in array ishiftpn
c    The actual local shift is defined by shift + ishift(.,pnttypin)
c    This is stored in locshift
c    The global shift satisfies glob_shift = locshift/2, where the
c    FORTRAN integer division is applied
c    To compute the new point type we proceed as follows:
c       locshift := locshift-2*glob_shift
c       pnttype = 1+locshift(1)*1+locshift(2)*2+locshift(3)*4
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data ishiftpn / 0, 0, 0,
     +                1, 0, 0,
     +                0, 1, 0,
     +                1, 1, 0,
     +                0, 0, 1,
     +                1, 0, 1,
     +                0, 1, 1,
     +                1, 1, 1 /
c ======================================================================
c
c     --- Compute the actual local shift

      locshift(1) = ishiftlc(1) + ishiftpn(1,pnttypin)
      locshift(2) = ishiftlc(2) + ishiftpn(2,pnttypin)
      locshift(3) = ishiftlc(3) + ishiftpn(3,pnttypin)

c     --- Compute the global shift

      do k = 1, 3
         if ( locshift(k).gt.0 ) then
            ishiftgl(k) = locshift(k)/2
         else if ( locshift(k).eq.0 ) then
            ishiftgl(k) = 0
         else
            ishiftgl(k) = (locshift(k)-1)/2
         end if
      end do

c     --- Update the actual local shift

      locshift(1) = locshift(1) - ishiftgl(1)*2
      locshift(2) = locshift(2) - ishiftgl(2)*2
      locshift(3) = locshift(3) - ishiftgl(3)*2

c     --- Compute new point type

      pnttyput = 1+locshift(1)+locshift(2)*2+locshift(3)*4

      end
