      subroutine isgmat12 ( ni, nj, nvirtual, gmat, cva, invjac,
     +                      ibipoint, ibinfor )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.3    date 24-12-1998 Guus: adaptation for transition
c                                       multiblock fixed boundary
c        version 1.2    date 20-03-1996 Allow btype=0
c        version 1.1    date 16-11-1995 Remove some bc parameters
c        version 1.0    date 20-09-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Correct the gradient matrix for boundary cells in 2D
c    At this moment the discretization is restricted to standard ISNaS
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*)
      double precision gmat(ni+1,nj+1,2,6),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,4)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     gmat     i/o   Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     invjac    i    1/sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, snnut
      parameter ( snnsnt = 7, snnut = 6 )
      integer ip, jp, iparea(2,2), k, mxp(2), tdir, ndir, pnttyp,
     +        iside, ipibinf, di, dj, inci, incj, nsign,
     +        ip1, jp1, btype, inside
      logical skip
      double precision gtt, gnt, gnn, chsym

c     btype     Type of boundary condition
c     chsym     Christoffel symbols of the second kind in the
c               point given by pnttyp
c               (symmetric contravariant part stored as vector
c               with index encoding given by istwo)
c     di        Shift in stencil in i-direction
c     dj        Shift in stencil in j-direction
c     gnn       Contravariant metric tensor
c     gnt       Contravariant metric tensor in nt direction
c     gtt       Contravariant metric tensor in tt direction
c     inci      Offset between vertices in i-direction
c     incj      Offset between vertices in j-direction
c     inside    Position of connection corresponding to point inside the inner
c               region opposite the boundary point
c     ip        Local counting variable i-direction
c     ip1       Shifted position in i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     jp        Local counting variable j-direction
c     jp1       Shifted position in j-direction
c     k         General loop variable
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     pnttyp    reserved
c     skip      If true boundary condition must be skipped
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   The discretization for the unknowns has been carried out before
c   as if all unknowns are internal
c   In this subroutine only the boundary cells are corrected.
c   Virtual pressures are folded insidely in subroutine isgmatf2
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c
c   The stress term for this cell has the following shape:
c
c  1          a1 |i+1/2,j              a2 |i,j+1/2   1        |  a  |   cb |
c  - sqrt(g) T   |         + sqrt(g)  T   |        + -sqrt(g) |     |  T   |
c  2             |i-1/2,j                 |i,j       2        | b c |      |i,j
c
c
c                                         ab    ab
c   With respect to the pressure we have T   = g  p
c
c   The following boundary conditions have the following effects for the
c   pressure
c
c
c   un given in boundary centroid:  matrix elements equal to 0
c
c   Sigma nn, Sigma nt given:
c
c   The terms involving the pressure that remain are:
c
c            22                         | 2 |  11
c   sqrt(g) g   p |       + 0.5 sqrt(g) |   | g  p |
c                 |(0,1)                |1 1|      |(0,0)
c
c   However, in order to make things simple (0,0) is replaced by (0,1)
c
c   Sigma nn, ut given:
c
c   The terms involving the pressure that remain are:
c
c                21  |(1,0)            22
c   0.5 sqrt(g) g  p |      + sqrt(g) g   p |      +
c                    |(-1,0)                |(0,1)
c
c
c                 | 2 |  11     | 2 |  12
c    0.5 sqrt(g) (|   | g  p  +2|   | g  p |
c                 |1 1|         |1 2|      |(0,0)
c
c   However, in order to make things simple (0,0) is replaced by (0,1),
c   (1,0) by (1,1) and (-1,0) by (-1,1)
c
c   Remark: this approach does not seem logical since the pressure
c           discretization in fact differs from the standard approach
c           In the future this term should be considered carefully
c
c
c
c   Consider a tangential cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c
c
c         ---------------+--------------------------
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         |      +       *     +      |            |
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         ---------------+--------------------------
c
c    The only position influenced by the boundary conditions is the point (0,1),
c    where we have the term:
c
c    -sqrt(g) sigma^21 |
c                      | (0,-1)
c
c    Hence only in the case sigma^nt given some action should be carried out
c
c    In the case sigma^nn and sigma^nt given we have sigma^21 given.
c    It is sufficient to remove the pressure term from the point (0,-1)
c    Since boundary conditions and discretization do not match it is hard to
c    choose which term should be removed.
c
c    In the case sigma^nt and u^n given we have:
c
c    sigma^12 = fac sigma^nt - g_21/g_11 sigma^22
c    Hence first we have to remove the pressure term from the point (0,-1)
c    and then we have to add the contribution:
c
c    -g_21/g_11 g^22 sqrt(g) = g^12 sqrt(g)
c
c    This term is exactly equal to the term just subtracted, which
c    implies that in this case no action at all is taken.
c
c
c    In order to compute the geometrical quantities, we use the following
c    formulae:
c
c    g^cc = a^c . a^c
c    a^(tdir)_tdir = 1/sqrt(g)  a^ndir_(ndir)
c    a^(tdir)_ndir =-1/sqrt(g)  a^ndir_(tdir)
c    a^(ndir)_tdir =-1/sqrt(g)  a^tdir_(ndir)
c    a^(ndir)_ndir = 1/sqrt(g)  a^tdir_(tdir)
c
c    chrys^ndir_tdir,tdir = a^(ndir) . da_(tdir)/dxsi^tdir
c    chrys^ndir_ndir,tdir = a^(ndir) . da_(ndir)/dxsi^tdir
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         di = 0
         dj = 0
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 1
            iparea(2,2) = nj
            tdir = 2
            ndir = 1
            inci = 0
            incj = 1
            pnttyp = 2
            if ( iside.eq.1 ) then

c           --- iside = 1:  left-hand side

               nsign = -1

            else

c           --- iside = 2:  right-hand side

               nsign = 1
               iparea(1,1) = ni+1
               iparea(2,1) = ni+1
               di = -1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 1
            iparea(2,1) = ni
            tdir = 1
            ndir = 2
            pnttyp = 3
            inci = 1
            incj = 0
            if ( iside.eq.3 ) then

c           --- iside = 3:  lower side

               nsign = -1

            else

c           --- iside = 4:  upper side

               nsign = 1
               iparea(1,2) = nj+1
               iparea(2,2) = nj+1
               dj = -1

            end if

         end if
         if ( nsign.eq.-1 ) then
            inside = 4
         else
            inside = 3
         end if

c        --- First the normal cells are treated

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

c           --- Get integration cell index

               mxp(1) = ip
               mxp(2) = jp

c              --- Get integration cell position for ibinfor

               btype = ibinfor(ipibinf+2*mxp(tdir))

c              --- clear row in matrix if not periodical or multiblock

               skip = btype.ge.88 .and. btype.le.100
               if ( btype.gt.0 .and. .not. skip ) then
                  do k = 1, 6
                     gmat(ip,jp,ndir,k) = 0d0
                  end do
               end if

               if ( btype.eq.0 ) then

c              --- No boundary condition given, fold inwards

                  if ( mod(iside,2).eq.1 ) then

c                 --- sides 1 and 3

                     do k = 1, 5, 2
                        gmat(ip,jp,ndir,k+1) = gmat(ip,jp,ndir,k+1) +
     +                     gmat(ip,jp,ndir,k)
                        gmat(ip,jp,ndir,k) = 0d0
                     end do

                  else

c                 --- sides 2 and 4

                     do k = 2, 6, 2
                        gmat(ip,jp,ndir,k-1) = gmat(ip,jp,ndir,k-1) +
     +                     gmat(ip,jp,ndir,k)
                        gmat(ip,jp,ndir,k) = 0d0
                     end do

                  end if

               else if ( btype.eq.snnut ) then

c              --- Normal stress and tangential velocity given
c                  All contributions in gmat become zero except
c                  (0,1): sqrt(g) g^22 + 0.5 sqrt(g) Chris^2_11  g^11
c                                            sqrt(g) Chris^2_12  g^12
c                  (-1,1): -0.5 sqrt(g) g^21
c                  (1,1):   0.5 sqrt(g) g^21
c                  First point (0,1)
c                  The ip,jp indices are ip+di, jp+dj

                  ip1 = ip+di
                  jp1 = jp+dj
                  gtt = invjac(ip1,jp1,4)*
     +                  (cva(ip1,jp1,ndir,ndir,4)**2+
     +                   cva(ip1,jp1,tdir,ndir,4)**2)
                  gnn = invjac(ip1,jp1,4)*
     +                  (cva(ip1,jp1,tdir,tdir,4)**2+
     +                   cva(ip1,jp1,ndir,tdir,4)**2)
                  chsym = invjac(ip1,jp1,4)*
     +                  (-cva(ip1,jp1,ndir,tdir,4)*
     +                   (cva(ip1+inci,jp1+incj,tdir,tdir,pnttyp)-
     +                    cva(ip1,jp1,tdir,tdir,pnttyp))+
     +                    cva(ip1,jp1,tdir,tdir,4)*
     +                   (cva(ip1+inci,jp1+incj,ndir,tdir,pnttyp)-
     +                    cva(ip1,jp1,ndir,tdir,pnttyp)))
                  gmat(ip,jp,ndir,inside) = -nsign*gnn+0.5d0*gtt*chsym
                  gnt = invjac(ip1,jp1,4)*
     +                  (-cva(ip1,jp1,ndir,tdir,4)*
     +                    cva(ip1,jp1,ndir,ndir,4)
     +                   -cva(ip1,jp1,tdir,tdir,4)*
     +                    cva(ip1,jp1,tdir,ndir,4))
                  chsym = invjac(ip1,jp1,4)*
     +                  (-cva(ip1,jp1,ndir,tdir,4)*
     +                   (cva(ip1+inci,jp1+incj,tdir,ndir,pnttyp)-
     +                    cva(ip1,jp1,tdir,ndir,pnttyp))+
     +                    cva(ip1,jp1,tdir,tdir,4)*
     +                   (cva(ip1+inci,jp1+incj,ndir,ndir,pnttyp)-
     +                    cva(ip1,jp1,ndir,ndir,pnttyp)))
                  gmat(ip,jp,ndir,inside) = gmat(ip,jp,ndir,inside) +
     +                                      gnt*chsym

c                 --- Next point (-1,1)
c                     The ip,jp indices are ip+di, jp+dj

                  gnt = invjac(ip1,jp1,pnttyp)*
     +                  (-cva(ip1,jp1,ndir,tdir,pnttyp)*
     +                    cva(ip1,jp1,ndir,ndir,pnttyp)
     +                   -cva(ip1,jp1,tdir,tdir,pnttyp)*
     +                    cva(ip1,jp1,tdir,ndir,pnttyp))

c                 --- Distribute contribution

                  gmat(ip,jp,ndir,inside-2) = -0.25d0*gnt
                  gmat(ip,jp,ndir,inside) = gmat(ip,jp,ndir,inside) -
     +                                      0.25d0*gnt

c                 --- Next point (1,1)
c                     The ip,jp indices are ip+di+inci, jp+dj+incj

                  ip1 = ip1+inci
                  jp1 = jp1+incj
                  gnt = invjac(ip1,jp1,pnttyp)*
     +                  (-cva(ip1,jp1,tdir,tdir,pnttyp)*
     +                    cva(ip1,jp1,tdir,ndir,pnttyp)
     +                   -cva(ip1,jp1,ndir,ndir,pnttyp)*
     +                    cva(ip1,jp1,ndir,tdir,pnttyp))

c                 --- Distribute contribution

                  gmat(ip,jp,ndir,inside+2) = 0.25d0*gnt
                  gmat(ip,jp,ndir,inside) = gmat(ip,jp,ndir,inside) +
     +                                      0.25d0*gnt

               else if ( btype.eq.snnsnt ) then

c              --- Stress given
c                  All contributions in gmat become zero except the one
c                  inside the normal velocity component
c                  The ip,jp indices of the inside point are ip+di, jp+dj
c                  This point gets the value sqrt(g) g^22 +
c                                        0.5 sqrt(g) Chris^2_11  g^11

                  ip1 = ip+di
                  jp1 = jp+dj
                  gtt = invjac(ip1,jp1,4)*
     +                  (cva(ip1,jp1,ndir,ndir,4)**2+
     +                   cva(ip1,jp1,tdir,ndir,4)**2)
                  gnn = invjac(ip1,jp1,4)*
     +                  (cva(ip1,jp1,tdir,tdir,4)**2+
     +                   cva(ip1,jp1,ndir,tdir,4)**2)
                  chsym = invjac(ip1,jp1,4)*
     +                  (-cva(ip1,jp1,ndir,tdir,4)*
     +                   (cva(ip1+inci,jp1+incj,tdir,tdir,pnttyp)-
     +                    cva(ip1,jp1,tdir,tdir,pnttyp))+
     +                    cva(ip1,jp1,tdir,tdir,4)*
     +                   (cva(ip1+inci,jp1+incj,ndir,tdir,pnttyp)-
     +                    cva(ip1,jp1,ndir,tdir,pnttyp)))
                  gmat(ip,jp,ndir,inside) = -nsign*gnn+0.5d0*gtt*chsym

               end if

            end do

         end do

c        --- Next the tangential cells are treated

         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         di = 0
         dj = 0
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 2
            iparea(2,2) = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni
               iparea(2,1) = ni
               di = 1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 2
            iparea(2,1) = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj
               iparea(2,2) = nj
               dj = 1

            end if

         end if
         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

c           --- Get integration cell index

               mxp(1) = ip
               mxp(2) = jp

c              --- Get integration cell position for ibinfor

               btype = ibinfor(ipibinf+2*mxp(tdir)-1)

               if ( btype.eq.snnsnt ) then

c              --- Tangential stress given
c                  Remove pressure from point (0,-1), i.e. subtract g^nt sqrt(g)

                  gnt = gmat(ip,jp,tdir,3+2*nsign)
                  gmat(ip,jp,tdir,3) = gmat(ip,jp,tdir,3) - gnt
                  gmat(ip,jp,tdir,4) = gmat(ip,jp,tdir,4) - gnt
                  gmat(ip,jp,tdir,3+2*nsign) = 0d0
                  gmat(ip,jp,tdir,4+2*nsign) = 0d0

               end if

            end do

         end do

      end do

      end
