      subroutine isgmatw01 ( ni, nj, nk, nvirtual, gmat, xcentr, jac,
     +                       alphad, adir )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 01-04-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute internal part of gradient matrix in R^3 for one direction
c    WesBeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad
      double precision gmat(ni+1,nj+1,nk+1,3,10),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual),
     +                 xcentr(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     gmat      o    Gradient matrix
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     xcentr    i    Coordinates of centroids of cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, l
      double precision fact, facs, cmat(3,3), det

c     cmat      c_matrix
c     det       Determinant
c     facs      Multiplication factor
c     fact      Multiplication factor
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   The following discretization is used in the WesBeek case
c
c   V^1-cell:
c
c      a^(1) . sqrt(g) grad p (0,0,0), with
c
c      grad p (0,0,0) = c^1 (p(1,0,0)-p(-1,0,0)) +
c                       c^2 (p(-1,2,0)-p(-1,-2,0) + p(1,2,0)-p(1,-2,0))+
c                       c^3 (p(-1,0,2)-p(-1,0,-2) + p(1,0,2)-p(1,0,-2))
c
c      c_1 = x(1,0,0)-x(-1,0,0)
c      c_2 = x(-1,2,0)-x(-1,-2,0) + x(1,2,0)-x(1,-2,0)
c      c_3 = x(-1,0,2)-x(-1,0,-2) + x(1,0,2)-x(1,0,-2)
c
c      Cmat = ( c_1, c_2, c_3 ) ^T
c
c      c^1 = Cmat^(-1) (1,0,0)^T
c      c^2 = Cmat^(-1) (0,1,0)^T
c      c^3 = Cmat^(-1) (0,0,1)^T
c
c
c      The other cells arise by cyclic permutation
c
c                    PSEUDO CODE
c
c
c    First clear part corresponding to virtual unknowns
c    Loop over Va unknown  (a=alpha)
c       Compute factor before (1,0) and (-1,0) (a=1) or (0,1) and (0,-1)
c       Store factor in gmat(a,4) and -gmat(a,3)
c       Compute factor before (0,1) and (0,-1) (a=1) or (1,0) and (-1,0)
c       and multiply by .25
c       Store factor in gmat(a,5), gmat(a,6) and -gmat(a,1), -gmat(a,2)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- First clear part corresponding to virtual unknowns

      if ( alphad.eq.1 ) then

c     --- V1-cell
c         Actually filling of gmat

         do i = 1, ni+1
            do j = 1, nj
               do k = 1, nk

c              --- Fill array C with the coordinate differences

                  do l = 1, 3
                     cmat(1,l) = xcentr(i,j,k,l)-xcentr(i-1,j,k,l)
                     cmat(2,l) = xcentr(i,j+1,k,l)-xcentr(i,j-1,k,l)+
     +                           xcentr(i-1,j+1,k,l)-xcentr(i-1,j-1,k,l)
                     cmat(3,l) = xcentr(i,j,k+1,l)-xcentr(i,j,k-1,l)+
     +                           xcentr(i-1,j,k+1,l)-xcentr(i-1,j,k-1,l)
                  end do
                  det =
     +               cmat(1,1)*(cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))
     +              +cmat(1,2)*(cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))
     +              +cmat(1,3)*(cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1))
                  fact = jac(i,j,k)/det

c                 --- Compute a^1 . c^k

                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1)))
                  gmat(i,j,k,alphad,3) = -facs
                  gmat(i,j,k,alphad,4) =  facs
                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(3,2)*cmat(1,3)-cmat(3,3)*cmat(1,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(3,3)*cmat(1,1)-cmat(3,1)*cmat(1,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(3,1)*cmat(1,2)-cmat(3,2)*cmat(1,1)))
                  gmat(i,j,k,alphad,1) = -facs
                  gmat(i,j,k,alphad,2) = -facs
                  gmat(i,j,k,alphad,5) =  facs
                  gmat(i,j,k,alphad,6) =  facs
                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(1,2)*cmat(2,3)-cmat(1,3)*cmat(2,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(1,3)*cmat(2,1)-cmat(1,1)*cmat(2,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(1,1)*cmat(2,2)-cmat(1,2)*cmat(2,1)))
                  gmat(i,j,k,alphad,7) = -facs
                  gmat(i,j,k,alphad,8) = -facs
                  gmat(i,j,k,alphad,9) =  facs
                  gmat(i,j,k,alphad,10)=  facs
               end do
            end do
         end do
      else if ( alphad.eq.2 ) then

c     --- V2-cell
c         Actually filling of gmat

         do i = 1, ni
            do j = 1, nj+1
               do k = 1, nk
                  fact = jac(i,j,k)

c                 --- Fill array C with the coordinate differences

                  do l = 1, 3
                     cmat(2,l) = xcentr(i,j,k,l)-xcentr(i,j-1,k,l)
                     cmat(3,l) = xcentr(i,j,k+1,l)-xcentr(i,j,k-1,l)+
     +                           xcentr(i,j-1,k+1,l)-xcentr(i,j-1,k-1,l)
                     cmat(1,l) = xcentr(i+1,j,k,l)-xcentr(i-1,j,k,l)+
     +                           xcentr(i+1,j-1,k,l)-xcentr(i-1,j-1,k,l)
                  end do
                  det =
     +               cmat(1,1)*(cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))
     +              +cmat(1,2)*(cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))
     +              +cmat(1,3)*(cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1))
                  fact = jac(i,j,k)/det

c                 --- Compute a^1 . c^k

                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1)))
                  gmat(i,j,k,alphad,7) = -facs
                  gmat(i,j,k,alphad,8) = -facs
                  gmat(i,j,k,alphad,9) =  facs
                  gmat(i,j,k,alphad,10)=  facs
                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(3,2)*cmat(1,3)-cmat(3,3)*cmat(1,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(3,3)*cmat(1,1)-cmat(3,1)*cmat(1,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(3,1)*cmat(1,2)-cmat(3,2)*cmat(1,1)))
                  gmat(i,j,k,alphad,3) = -facs
                  gmat(i,j,k,alphad,4) =  facs
                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(1,2)*cmat(2,3)-cmat(1,3)*cmat(2,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(1,3)*cmat(2,1)-cmat(1,1)*cmat(2,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(1,1)*cmat(2,2)-cmat(1,2)*cmat(2,1)))
                  gmat(i,j,k,alphad,1) = -facs
                  gmat(i,j,k,alphad,2) = -facs
                  gmat(i,j,k,alphad,5) =  facs
                  gmat(i,j,k,alphad,6) =  facs
               end do
            end do
         end do
      else

c     --- V3-cell
c         Actually filling of gmat

         do i = 1, ni
            do j = 1, nj
               do k = 1, nk+1
                  fact = jac(i,j,k)

c                 --- Fill array C with the coordinate differences

                  do l = 1, 3
                     cmat(3,l) = xcentr(i,j,k,l)-xcentr(i,j,k-1,l)
                     cmat(1,l) = xcentr(i+1,j,k,l)-xcentr(i-1,j,k,l)+
     +                           xcentr(i+1,j,k-1,l)-xcentr(i-1,j,k-1,l)
                     cmat(2,l) = xcentr(i,j+1,k,l)-xcentr(i,j-1,k,l)+
     +                           xcentr(i,j+1,k-1,l)-xcentr(i,j-1,k-1,l)
                  end do
                  det =
     +               cmat(1,1)*(cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))
     +              +cmat(1,2)*(cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))
     +              +cmat(1,3)*(cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1))
                  fact = jac(i,j,k)/det

c                 --- Compute a^1 . c^k

                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1)))
                  gmat(i,j,k,alphad,1) = -facs
                  gmat(i,j,k,alphad,2) = -facs
                  gmat(i,j,k,alphad,5) =  facs
                  gmat(i,j,k,alphad,6) =  facs
                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(3,2)*cmat(1,3)-cmat(3,3)*cmat(1,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(3,3)*cmat(1,1)-cmat(3,1)*cmat(1,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(3,1)*cmat(1,2)-cmat(3,2)*cmat(1,1)))
                  gmat(i,j,k,alphad,7) = -facs
                  gmat(i,j,k,alphad,8) = -facs
                  gmat(i,j,k,alphad,9) =  facs
                  gmat(i,j,k,alphad,10)=  facs
                  facs = fact*
     +                   (adir(i,j,k,1)*
     +                   (cmat(1,2)*cmat(2,3)-cmat(1,3)*cmat(2,2))+
     +                   adir(i,j,k,2)*
     +                   (cmat(1,3)*cmat(2,1)-cmat(1,1)*cmat(2,3))+
     +                   adir(i,j,k,3)*
     +                   (cmat(1,1)*cmat(2,2)-cmat(1,2)*cmat(2,1)))
                  gmat(i,j,k,alphad,3) = -facs
                  gmat(i,j,k,alphad,4) =  facs
               end do
            end do
         end do
      end if
      end
