      subroutine isgmbf ( ndim, ni, nj, nk,
     +                    nconct, maxdx, it1lo, it1hi, it2lo, it2hi,
     +                    mct, mctinv, matrix, ibinfor,
     +                    iface, iconct, idir, matarea, iphase )
c=======================================================================
c
c
c        programmer     Ronald van Nooyen
c        version 1.0    date 11-11-94 ismcbcf+isgmbf replace isgmbf2
c
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c*********************************************************************
c
c                      DESCRIPTION
c
c   Gradient matrix boundary condition folder
c
c*********************************************************************
c
c                      KEYWORDS
c
c*********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
c
      integer maxdim
      parameter ( maxdim = 3 )
c
c
      integer iconct, idir, it1hi, it1lo, it2hi, it2lo, nconct,
     +        ndim, ni, nj, nk, maxdx

      integer mct(1:ndim,1:1,-maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx),
     +        mctinv(0:maxdim,1:ndim,1:nconct),
     +        matarea(1:2,1:maxdim), iphase

      integer ibinfor(it1lo:it1hi,it2lo:it2hi)

      double precision matrix(1:ni+1, 1:nj+1, 1:nk+1,1:ndim, 1:nconct)

c     ibinfor   i    Third part of infbnd
c     iconct    i    Loop variable for loop over all connections in the matrx
c     idir      i    General loop variable for 1...ndim loop
c     iface     i    Face number
c     iphase    i    Phase in folding process
c     it1hi     i    Upper bound on first tangential index for destination
c     it1lo     i    Lower bound on first tangential index for destination
c     it2hi     i    Upper bound on second tangential index for destination
c     it2lo     i    Lower bound on second tangential index for destination
c     matarea   i    Index bounds for the active area of the matrix
c     matrix    i    reserved
c     maxdx     i    Maximum relative shift between cell indices
c     mct       i    Integer matrix information
c     mctinv    i    Matrix information
c     nconct    i    Number of connections in a row of the matrix
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS
c
c
      integer unut, periodic, antiper, multiblk,
     +        uxuy, unsnt, perjump, antijump,
     +        oldturb, parabolic
      parameter( unut = 4, periodic = 98,
     +   antiper = 97, perjump = 96, antijump =95,
     +                   multiblk = 99, uxuy = 10, unsnt = 5,
     +                   oldturb = 9, parabolic = 12)

      integer outside, leavealone, masspar
      parameter (outside = 101, leavealone = 300, masspar = 100)

      logical dbhc

      integer icdp(1:maxdim), icp(1:maxdim), ivp(1:maxdim),
     +        ifcarea(1:2,1:maxdim), ifvarea(1:2,1:maxdim),
     +        ievp(1:maxdim,-1:1), ieconct(-1:1),
     +        nsig, ntdir(1:maxdim), ntsig(1:maxdim),
     +        ictp(1:maxdim-1), ivtp(1:maxdim-1), iface,
     +        ip, jp, kp, icbtype, ivbtype, ibtype, ndist, i, jdim,
     +        ndir, lcell(1:maxdim), lvar(1:maxdim)

      double precision exratio
c
c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     dbhc    ?
c     exratio   Ratio of distance between base points for extrapolation
c               and distance between the point to which we extrapolate
c               and the base point closest to it
c
c                        Examples:
c
c                             |     *
c                             -     -
c                             |     *
c                        --|--+--|--*--|--+--|--+
c                          B1    B2          E
c
c                        Linear extrapolation:
c                        f(E) = (exratio+1d0) f(B2) - exratio f(B1)
c                        with exratio = 2
c
c
c                             |     *
c                             -     -
c                             |     *
c                        --|--+--|--*--|--+--|--+
c                                B1 B2       E
c
c                        Linear extrapolation:
c                        f(E) = (exratio+1d0) f(B2) - exratio f(B1)
c                        with exratio = 3
c
c     i         General loop variable
c     ibtype  ?
c     icbtype  ?
c     icdp      Index offset of cell containing the current variable
c               relative to the cell containing the integration cell centre
c     icp       Vector of integration cell indices
c     ictp      Vector of ibinfor indices for integation cell centre
c     ieconct  ?
c     ievp    ?
c     ifcarea   Index bounds for cells on this face
c     ifvarea   Index bounds for vertices on this face
c     ip        Local counting variable i-direction
c     ivbtype  ?
c     ivp       Vector of variable location cell indices
c     ivtp      Vector of ibinfor indices for current variable
c     jdim      Loop variable used for dimension
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     lcell     Location of the integration cell centre within a grid cell
c               in Cell Local Coordinates
c               (i.e. (0,0,0): lower left vertex, (1,1,1): cell centre, etc.)
c     leavealone  ?
c     lvar      Location of the current variable within a grid cell
c               in Cell Local Coordinates
c               (i.e. (0,0,0): lower left vertex, (1,1,1): cell centre, etc.)
c     masspar  ?
c     maxdim    Highest physical space dimension for which ISNaS can work
c     multiblk  Multi-block boundary conditions
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     ntdir     Basis vector a_ntdir(1) corresponding to the
c               normal direction
c               Basis vector a_ntdir(2) corresponding to the
c               (in 3D: first) tangential direction
c               in 3D: a_ntdir(3) basis vector corresponding to
c               the second tangential direction
c     ntsig     Sign of inner product of true normal or
c               tangential vectors and the corresponding basis
c               vector
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     outside  ?
c     parabolic  ?
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c
c
c
c*********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISBNBVI   Boundary information
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c*********************************************************************
c
c                      I/O
c
c*********************************************************************
c
c                      ERROR MESSAGES
c
c*********************************************************************
c
c                      PSEUDO CODE
c
c     Loop over connections
c        Loop over integration cell types
c
c=======================================================================
c
      call iseropen ( 'isgmbf' )

c     --- Get the normal and tangential directions (ntdir,ntsig), the
c         grid cells adjacent to the boundary (ifcarea) and the
c         vertices on the boundary (ifvarea)

      call isbnbvi(ndim, ni, nj, nk, iface, ntdir, ntsig,
     +             ifcarea, ifvarea)

c     --- Get direction and sign of outward normal

      ndir = ntdir(1)
      nsig = ntsig(1)

c     --- Get local integration cell centre position

      lcell(1) = 1
      lcell(2) = 1

      if (ndim.eq.2) then
c     --- zero for k direction
         lcell(3) = 0
      else
         lcell(3) = 1
      end if

      lcell(idir) = 0

c     --- Get local variable position

      lvar(1)  = 1
      lvar(2)  = 1

      if (ndim.eq.2) then
c     --- zero for k direction
         lvar(3)  = 0
      else
         lvar(3)  = 1
      end if

c     --- Get cell index offset relative to integration cell for
c         current variable

      icdp(1) = mctinv(1,idir,iconct)
      icdp(2) = mctinv(2,idir,iconct)
      icdp(3) = mctinv(3,idir,iconct)

c     --- Loop over integration cell centres

      do 10000 kp=matarea(1,3),matarea(2,3)
         do 11000 jp=matarea(1,2),matarea(2,2)
            do 11100 ip=matarea(1,1),matarea(2,1)

               icp(1) = ip
               icp(2) = jp
               icp(3) = kp
               ictp(1) = 2*icp(ntdir(2))-1+lcell(ntdir(2))

               if (ndim.eq.2) then
                  ictp(2) = 1
               else
                  ictp(2) = 2*icp(ntdir(3))-1+lcell(ntdir(3))
               end if

               icbtype = ibinfor(ictp(1),ictp(2))

c              --- Check for Dirichlet half cell

               dbhc = ( idir.eq.ndir .and. (icbtype.eq.uxuy .or.
     +                 icbtype.eq.unut .or. icbtype.eq.parabolic .or.
     +                 icbtype.eq.unsnt .or. icbtype.eq.oldturb ) )

c              --- Check for btypes that need no further action

               ivp(1) = icp(1)+icdp(1)
               ivp(2) = icp(2)+icdp(2)
               ivp(3) = icp(3)+icdp(3)

               ivtp(1) = 2*ivp(ntdir(2))-1+lvar(ntdir(2))

               if (ndim.eq.2) then
                  ivtp(2) = 1
               else
                  ivtp(2) = 2*ivp(ntdir(3))-1+lvar(ntdir(3))
               end if

c              --- Get boundary information

               if ( ivtp(1).lt.it1lo .or. ivtp(1).gt.it1hi .or.
     +            ivtp(2).lt.it2lo .or. ivtp(2).gt.it2hi ) then
c                    --- No boundary information available
                  ivbtype = outside
               else
                  ivbtype = ibinfor(ivtp(1),ivtp(2))
               end if

c              --- Get reduced boundary condition type

               ibtype = ivbtype
               if (ibtype.eq.parabolic) then
                  ibtype = unut
               end if
               if (ibtype.eq.oldturb) then
                  ibtype = unsnt
               end if
               if (ibtype.eq.multiblk .or. ibtype.eq.periodic .or.
     +            ibtype.eq.antiper .or.
     +            ibtype.eq.perjump .or. ibtype.eq.antijump .or.
     +            ibtype.eq.masspar) then
                  ibtype = leavealone
               end if
               ivbtype = ibtype

c              --- Determine distance to domain edge

               if (nsig.eq.-1) then
c              --- Lower edge
                  ndist = 2*(ivp(ndir)-ifvarea(1,ndir))+1
               else
c              --- Upper edge
                  ndist = 2*(ivp(ndir)-ifvarea(1,ndir))+1
               end if

c              --- Point outside: ndist positive
               ndist = nsig*ndist

               if (dbhc .and. iphase.eq.ndim+1) then
c              --- Dirichlet boundary half cell

                  matrix(icp(1),icp(2),icp(3),ndir,iconct) = 0d0

               else if (ivbtype.eq.leavealone) then
c              --- Leave this matrix element as it is
               else if (ndist.lt.0) then
c              --- Matrix element inside the domain in normal direction
c                 --- No action needed in this direction
c                      If folding is needed in tdir(iface), then
c                      this is done in the first or
c                      second call of isgmbf
c                      for the edge jface
c                      where ndir(jface).eq.tdir(iface)
               else if (ndist.gt.0 .and. iphase.le.ndim) then
c              --- Matrix element outside domain in normal direction
c                 --- Eliminate virtual matrix elements where necessary
c                     First determine the cells where the variables
c                     needed for elmination are located

                  do 11110 i=-1,1
                     do 11111 jdim=1,maxdim
                        ievp(jdim,i) = ivp(jdim)
11111                continue
11110             continue

                  if (nsig.eq.-1) then
c                 --- Closest value within domain shares ndir
c                     coordinate with edge vertices
                     ievp(ndir, 0) = ifvarea(1,ndir)
                  else
c                 --- Closest value within domain shares ndir
c                     coordinate with edge cells
                        ievp(ndir, 0) = ifcarea(1,ndir)
                  end if

c                 --- ndir location of next closest element

                  ievp(ndir,-1) = ievp(ndir,0)-nsig

c                 --- Get the matrix connection indices corresponding to
c                     the elements involved in the elimination

                  do 11120 i=-1,1
                     ieconct(i) = mct(idir,1,
     +                  ievp(1,i)-icp(1),ievp(2,i)-icp(2),
     +                  ievp(3,i)-icp(3))
11120             continue

c                 --- Perform the elimination where possible

                  if (ieconct(-1).lt.1 .or. ieconct(-1).gt.nconct)
     +                  then
c                 --- Unknown pressure and
c                     insufficient points in stencil for extrapolation
c                     Assume this is taken care of during an earlier
c                     phase of the matrix construction and leave it

                  else
c                 --- Unknown pressure, perform extrapolation

                     exratio = 1d0*((ndist+1)/2)

                     matrix(icp(1),icp(2),icp(3),idir,ieconct(-1)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,ieconct(-1)) -
     +                  exratio*matrix(icp(1),icp(2),icp(3),idir,iconct)
                     matrix(icp(1),icp(2),icp(3),idir,ieconct(0)) =
     +                  matrix(icp(1),icp(2),icp(3),idir,ieconct(0)) +
     +                  (exratio+1d0)*
     +                  matrix(icp(1),icp(2),icp(3),idir,iconct)

                     matrix(icp(1),icp(2),icp(3),idir,iconct) = 0d0

                  end if
               end if

11100       continue
11000    continue
10000 continue

      call iserclos ( 'isgmbf' )
      end

