      subroutine isgmtr23 ( iface, ni, nj, nk, nvirtual, ilow1,
     +                       ihigh1, ilow2, ihigh2, ibinfor, rhoinv )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 13-03-1996 name changed 8 letters
c        version 1.0    date 02-02-1996
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c **********************************************************************
c
c                      KEYWORDS
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ilow1, ihigh1,ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), iface

      double precision rhoinv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3)

c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhoinv    i    Array containing the inverse of rho in the integration
c                    points, multiplied by theta dt
c
c
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer snnsnt, snnut
      parameter( snnut = 6, snnsnt = 7 )
      integer  ndir, tdir(2), iparea(2,3), nijk(3),
     +         xp(3), ict1, ict2, btype, ip, jp, kp

c     btype     Type of boundary condition
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     ndir      Direction of normal vector on this side
c     nijk      Vector of number of points in the i,j and k-directions
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     xp        Auxiliary array to calculate boundary point number
c
c
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    PSEUDO CODE
c
c
c **********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- loop over faces

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

c     --- Compute tdir and ndir

      if ( iface.le.2 ) then

c     --- iface = 1, 2:  back and front plane

         ndir    = 1
         tdir(1) = 2
         tdir(2) = 3

      else if ( iface.gt.2  .and. iface.lt.5) then

c     --- iface = 3, 4:  left- and right plane

         ndir = 2
         tdir(1) = 3
         tdir(2) = 1

      else

c     --- iface = 5, 6:  bottom and top

         ndir = 3
         tdir(1) = 1
         tdir(2) = 2

c        --- loop over normal half cells
c            Compute loop bounds

      end if

      if ( mod(iface,2).eq.0 ) then
         iparea(1,ndir) = nijk(ndir)+1
         iparea(2,ndir) = nijk(ndir)+1
      else
         iparea(1,ndir) = 1
         iparea(2,ndir) = 1
      end if

      iparea(1,tdir(1)) = 1
      iparea(2,tdir(1)) = nijk(tdir(1))
      iparea(1,tdir(2)) = 1
      iparea(2,tdir(2)) = nijk(tdir(2))

      do 400 kp = iparea(1,3), iparea(2,3)
         do 300 jp = iparea(1,2), iparea(2,2)
            do 200 ip = iparea(1,1), iparea(2,1)

c           --- Store integration cell indices in vector

               xp(1) = ip
               xp(2) = jp
               xp(3) = kp

c              --- Get corresponding index into ibinfor
c                  Get boundary information

               ict1 = 2*xp(tdir(1))
               ict2 = 2*xp(tdir(2))
               btype = ibinfor(ict1,ict2)

               if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c              --- stress given, multiply rhoinv by 2

                  rhoinv(ip,jp,kp,ndir) = 2d0*rhoinv(ip,jp,kp,ndir)

               end if

200         continue
300      continue
400   continue

      end
