      subroutine isintcf3 ( coef, coefint, ni, nj, nk, nvirtual, npf,
     +                       rspv, cv, acttyp, vartyp, pnttyp )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 23-05-95 (as isintcf2)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Compute interpolation of a coefficient in one integration point in R^3
c
c    This subroutine may be made more efficient by using the fact that a
c    coefficient is constant
c    Another possibility to increase efficiency is to interpolate to all
c    integration points in one step.
c ********************************************************************
c
c                      KEYWORDS
c
c    coefficients
c    interpolation
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision coef(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual)
      double precision coefint(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual,
     +                         1-nvirtual:nk+1+nvirtual)
      integer acttyp, vartyp, pnttyp
      integer npf(0:3,0:4,1:8), rspv(1:3,1:8,0:3,0:4,1:8)
      double precision cv(1:8,0:3,0:4,1:8)

c     acttyp    i    Type of action to be taken
c                    0      : interpolation for point value
c                    1..ndim: finite difference for partial derivative
c                             in the direction given by value of acttyp
c     coef      i    A specific coefficient
c     coefint   o    Interpolated coefficient
c     cv        i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npf       i    reserved
c     nvirtual  i    Number of rows of virtual cells
c     pnttyp    i    reserved
c     rspv      i    reserved
c     vartyp    i    Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or :
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j, k
      integer lnpf
      integer lrs(3,8)
      double precision lcv(8)

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     lcv       Cv for the given acttyp, vartyp and pnttyp
c     lnpf      Npf for the given acttyp, vartyp and pnttyp
c     lrs       Rspv for the given acttyp, vartyp and pnttyp
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c ======================================================================
c

c     --- Introduce abbreviations for interpolation formula components

      lnpf = npf(acttyp,vartyp,pnttyp)
      do 110 j = 1, lnpf
         do 100 i = 1, 3
            lrs(i,j) = rspv(i,j,acttyp,vartyp,pnttyp)
100      continue
         lcv(j) = cv(j,acttyp,vartyp,pnttyp)
110   continue

c     --- for higher efficiency we deal with each possible lnpf
c         separately (maximize calculation vs. memory access of coefint)

      if ( lnpf.eq.1 ) then
         do 220 k = 2-nvirtual, nk+nvirtual-1
            do 210 j = 2-nvirtual, nj+nvirtual-1
               do 200 i = 2-nvirtual, ni+nvirtual-1
                  coefint(i,j,k) = coef(i,j,k)
200            continue
210         continue
220      continue

      else if ( lnpf.eq.2 ) then

         do 320 k = 2-nvirtual, nk+nvirtual-1
            do 310 j = 2-nvirtual, nj+nvirtual-1
               do 300 i = 2-nvirtual, ni+nvirtual-1
                  coefint(i,j,k) =
     +               (lcv(1)*coef(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1))+
     +                lcv(2)*coef(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)))
300            continue
310         continue
320      continue

      else if (lnpf.eq.4) then

         do 420 k = 2-nvirtual, nk+nvirtual-1
            do 410 j = 2-nvirtual, nj+nvirtual-1
               do 400 i = 2-nvirtual, ni+nvirtual-1
                  coefint(i,j,k) =
     +               (lcv(1)*coef(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1))+
     +                lcv(2)*coef(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2))+
     +                lcv(3)*coef(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3))+
     +                lcv(4)*coef(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)))
400           continue
410         continue
420      continue

      else if (lnpf.eq.8) then

         do 520 k = 2-nvirtual, nk+nvirtual-1
            do 510 j = 2-nvirtual, nj+nvirtual-1
               do 500 i = 2-nvirtual, ni+nvirtual-1
                  coefint(i,j,k) =
     +               (lcv(1)*coef(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1))+
     +                lcv(2)*coef(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2))+
     +                lcv(3)*coef(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3))+
     +                lcv(4)*coef(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4))+
     +                lcv(5)*coef(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5))+
     +                lcv(6)*coef(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6))+
     +                lcv(7)*coef(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7))+
     +                lcv(8)*coef(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)))
500           continue
510         continue
520      continue

      end if

      end
