      subroutine isjcta2d ( cva, jcta, ni, nj, nvirtual )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date 20-04-95 (Subroutine extracted from isjcta)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Compute sqrt(g) x contra variant base vectors in integration point
c    two-dimensional case
c ********************************************************************
c
c                      KEYWORDS
c
c    base_vectors
c    interpolation
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2)
      double precision jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     jcta      o    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j

c     i         General loop variable
c     j         General loop variable
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c     sqrt(g) x contravariant base vector in 2D is given by:
c
c              (1)     2                    (1)      2
c     sqrt(g) a     = a            sqrt(g) a     = -a
c              1       (2)                  2        (1)
c
c              (2)     1                    (2)      1
c     sqrt(g) a     =-a            sqrt(g) a     =  a
c              1       (2)                  2        (1)
c ======================================================================
c
      do j = 1-nvirtual, nj+1+nvirtual
         do i = 1-nvirtual, ni+1+nvirtual
            jcta(i,j,1,1) = cva(i,j,2,2)
            jcta(i,j,2,1) = -cva(i,j,1,2)
            jcta(i,j,1,2) = -cva(i,j,2,1)
            jcta(i,j,2,2) = cva(i,j,1,1)
         end do
      end do

      end
