      subroutine isjcta3d ( cva, jcta, ni, nj, nk, nvirtual )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 23-05-1995 (as isjcta2d from isjcta)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute sqrt(g) x contra variant base vectors in integration point
c    three-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3)
      double precision jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3,3)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     jcta      o    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp, alpha, beta, valpha(3), vbeta(3)

c     alpha     Relaxation parameter alpha
c     beta      Parameter beta for turbulence modelling
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c     vbeta     Array for cyclic changing ordering
c               betal = 1        betal = 2    betal = 3
c               vbeta(1) =1 ---> vbeta(1)=2  vbeta(1)=3
c               vbeta(2) =2 ---> vbeta(2)=3  vbeta(2)=1
c               vbeta(3) =3 ---> vbeta(3)=1  vbeta(3)=2
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     sqrt(g) x contravariant base vector in 2D is given by:
c
c              (alpha)
c     sqrt(g) a        = a        ^     a         for alpha, beta, gamma
c                         (betta)        (gamma)      cyclic
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- In three dimensions

         do alpha=1,3

c        --- cyclic
c            valpha(1) =1 ---> valpha(2)=2  valpha(3)=3
c            valpha(1) =2 ---> valpha(2)=3  valpha(3)=1
c            valpha(1) =3 ---> valpha(2)=1  valpha(3)=2

            valpha(1) = alpha
            valpha(2) = mod(alpha  ,3) + 1
            valpha(3) = mod(alpha+1,3) + 1
            do beta=1,3
         vbeta(1) = beta
               vbeta(2) = mod(beta  ,3) + 1
               vbeta(3) = mod(beta+1,3) + 1
               do kp=1-nvirtual, nk+1+nvirtual
                  do jp=1-nvirtual, nj+1+nvirtual
                     do ip=1-nvirtual, ni+1+nvirtual

c                    --- contravariant vectors before division by sqrt(g)

                        jcta(ip,jp,kp,beta,alpha) =
     +                       cva(ip,jp,kp,vbeta(2),valpha(2))*
     +                       cva(ip,jp,kp,vbeta(3),valpha(3))-
     +                       cva(ip,jp,kp,vbeta(3),valpha(2))*
     +                       cva(ip,jp,kp,vbeta(2),valpha(3))
                  end do
               end do
            end do
         end do
      end do
      end
