      subroutine ismf3dwb ( iface, ni, nj, nk, nconct, ilow1, ihigh1,
     +                       ilow2, ihigh2, mct, matrix, rhside,
     +                       ibinfor, bcgeo, bndcon )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 02-07-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    All parts of the matrix outside the region are folded to internal
c    domain for the momentum matrix in R3,
c    Wesbeek version.
c    The folding process depends on the types of boundary conditions.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ilow1, ihigh1,ilow2, ihigh2, nconct,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), iface,
     +        mct(3,3,-2:2,-2:2,-2:2)
      double precision bcgeo(21,ilow1:ihigh1,ilow2:ihigh2),
     +                 rhside(ni+1,nj+1,nk+1,3),
     +                 matrix(ni+1,nj+1,nk+1,3,*),
     +                 bndcon(3,ilow1:ihigh1,ilow2:ihigh2)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     matrix    i    reserved
c     mct       i    Integer matrix information
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     rhside    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  ndir, tdir(2), nsign, iparea(2,3), nijk(3)

c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ndir      Direction of normal vector on this side
c     nijk      Vector of number of points in the i,j and k-directions
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISMFWB34  Folding part for 3d case, Wesbeek version.
c               Normal unknowns of normal cells are folded
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The folding process will be carried out per face.
c    Some rows corresponding to unknowns in the neighbourhood of the edges
c    and vertices require special attention
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- loop over faces

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

c     --- Compute tdir and ndir

      if ( iface.le.2 ) then

c     --- iface = 1, 2:  back and front plane

         ndir    = 1
         tdir(1) = 2
         tdir(2) = 3
         if ( iface.eq.1 ) then

c        --- iface = 1:  back

            nsign = -1

         else

c        --- iface = 2:  front

            nsign = 1

         end if

      else if ( iface.gt.2  .and. iface.lt.5) then

c     --- iface = 3, 4:  left- and right plane

         ndir = 2
         tdir(1) = 3
         tdir(2) = 1
         if ( iface.eq.3 ) then

c        --- iface = 3:  left

            nsign = -1

         else

c        --- iface = 4:  right

            nsign = 1

         end if

      else

c     --- iface = 5, 6:  bottom and top

         ndir = 3
         tdir(1) = 1
         tdir(2) = 2

         if ( iface.eq.5 ) then

c        --- iface = 5:  bottom

            nsign = -1

         else

c        --- iface = 6:  top

            nsign = 1

         end if

      end if

c     --- loop over normal half cells
c         Compute loop bounds

      if ( mod(iface,2).eq.0 ) then
         iparea(1,ndir) = nijk(ndir)+1
         iparea(2,ndir) = nijk(ndir)+1
      else
         iparea(1,ndir) = 1
         iparea(2,ndir) = 1
      end if

      iparea(1,tdir(1)) = 1
      iparea(1,tdir(2)) = 1
      iparea(2,tdir(1)) = nijk(tdir(1))
      iparea(2,tdir(2)) = nijk(tdir(2))

      call ismfwb34 ( ndir, ni, nj, nk, nconct, iparea, ilow1, ihigh1,
     +                ilow2, ihigh2, mct, matrix, rhside,
     +                ibinfor, bcgeo, bndcon, nsign, tdir )

      end
