      subroutine ismom2d ( iinput, matrix, rhside, solut, coefs, work,
     +                     rinput, npointsv, infmm, vsum, dvsum, invjac,
     +                     coor, mct, bndcon, ibipoint, ibinfor, jac,
     +                     eddy, gmat, infmg )
c ======================================================================
c
c        programmer     Guus Segal
c        version 5.24   date 29-09-1999 Guus: new call to ismomrh2
c        version 5.23   date 23-04-1999 Duncan: changes for conservative upwind
c        version 5.22   date 04-04-1999 Duncan: new call to ismnw2d
c        version 5.21   date 31-03-1999 Duncan: extended for compressible
c                                               Newton linearisation
c        version 5.20   date 23-03-1999 Guus: correction conservative upwind
c                                             for multiblock
c        version 5.19   date 22-03-1999 KeesK: call ismomtd2 changed
c        version 5.18   date 10-02-1999 KeesK: in compressible case make rho=1
c                                       independent of input for convection
c                                       terms
c        version 5.17   date 27-07-1998 Duncan: extension to iterate timestep
c                                               get the whole eddy 1-4 not 2-4
c                                               apply underrelaxation
c        version 5.16   date 22-06-1998 Charles: change in comments
c        version 5.15   date 16-02-1998 Guus: remove WesBeek
c        version 5.14   date 05-01-1998 Guus: ad-hoc extension with the zeroth
c                                       order term
c        version 5.13   date 10-12-1997 Hester: new call to ispmom2d
c        version 5.12   date 02-12-1997 Guus: new call to iscont2d
c        version 5.11   date 26-11-1997 Guus: Multiplication eps mu in ismulmsfr
c        version 5.10   date 28-10-1997 Guus: extension for multiphase gas flow
c        version 5.9    date 21-10-1997 Hester: compr. higher order upw.
c        version 5.8    date 07-07-1997 Guus: new call to islin2d
c        version 5.7    date 12-03-1997 Guus: possibility reduced stress
c        version 5.6    date 17-02-1997 Marcel: parameter list of
c                                               isurhs2d changed
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build momentum matrix and right-hand side in R2
c
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    momentum_equation
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), npointsv, infmm(*), infmg(*),
     +        mct(2,2,-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*)
      double precision rhside(*), work(*), matrix(*), coor(*),
     +                 solut(npointsv,*), coefs(npointsv,*), rinput(*),
     +                 vsum(npointsv,*),  invjac(npointsv,*),
     +                 dvsum(npointsv,2,2), bndcon(2,*),
     +                 jac(npointsv,*), eddy(6*npointsv), gmat(*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    Array containing the coordinates for one block
c                    In 2d coor(i,j,.) contains the x resp y-coordinate
c                    of point i,j
c                    in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     dvsum     i    Flux derivative contributions to the matrix
c     eddy      i    reserved
c     gmat      i    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iinput    i    reserved
c     infmg     i    Integer information for Pressure-gradient matrix
c     infmm     i    Integer information for Momentum matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     matrix    o    reserved
c     mct       i    Integer matrix information
c     npointsv  i    Number of points in the mesh with virtual points
c     rhside    o    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     vsum      i    Flux contributions to the matrix
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iprho, nconct, alphad, iprintim, pnttyp, vartyp, ni,
     +        nj, nvirtual, ipmatrix, iprhside, ipmu, iarea(2,2),
     +        locdx(2,5), dcell(2,5), iparea(2,2), ipcva, ipjcta,
     +        ipjctg, ipchsym, ipveloc, iptemp, ipgeo, i, idir, iside,
     +        ipiindsc, mcom, ipiincom, nintpol(2,5), dij(2,4,2,5),
     +        acttyp, iprindsc, ipiintur, ichoisdis, iprintur, ndegfd,
     +        maniso, iprhoinv, mequat, ipgradu, ipwork, mturb,
     +        ipadir, idiscret, ipmassfr, ipprems, pnttyps(5), iconserv,
     +        iupwind, iconv, ipiintim, ipflux, idummy, ipinrho,
     +        newtcom, iprhsinc
      double precision deltat
      logical reuse, tangcell, reduced

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     reserved
c     deltat    Product of theta and dt
c     dij       Array containing the distances of the interpolation
c               points to the cell centra in global co-ordinates
c               dij(alpha,i,j) denotes the distance of the i-th
c               interpolation point, component j and with respect to
c               the V-alpha unknown (distribution)
c     i         General loop variable
c     iarea     Area of arrays
c     ichoisdis Choice parameter with respect to the convection scheme
c               for discretization of convective term of transport
c               equation(s) including turbulence.
c               Meaning of ichoisdis:
c                 ichoisdis = 0: 2nd order central scheme
c                 ichoisdis = 1: 1st order standard upwind scheme
c                 ichoisdis = 2: Hybrid central/upwind scheme
c                 ichoisdis = 3: Linear kappa-scheme
c                 ichoisdis = 4: Sweby Phi-limiter
c                 ichoisdis = 5: R-kappa limiter
c                 ichoisdis = 6: USR-kappa limiter
c                 ichoisdis = 7: PL-kappa limiter
c                 ichoisdis = 8: USPL-kappa limiter
c                 ichoisdis = 9: Symmetric PL-kappa limiter
c                 ichoisdis = 10: MPL2-kappa limiter
c                 ichoisdis = 11: Symmetric PL-kappa limiter
c     iconserv  If 1 the conservative scheme is used, otherwise (0) the
c               non-conservative one
c     iconv     If 1 the timestep is iterated till converged, otherwise (0)
c               apply only one iterative step
c     idir      General loop variable for 1...ndim loop
c     idiscret  Type of discretization
c     idummy    Integer dummy
c     ipadir    Starting address of array ADIR in array WORK
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipflux    Starting address of array FLUX in array WORK
c     ipgeo     Starting address of array GEO in array WORK
c     ipgradu   Starting address of array GRADU in array WORK
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     ipinrho   Starting address of array INRHO in array WORK
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ipmassfr  Starting address of array MASSFRAC in array WORK
c     ipmatrix  Starting address of array matrix in buffer
c     ipmu      Starting address of array MU in array WORK
c     ipprems   Starting address of array PREMS in array WORK
c     iprho     Starting address of array rho either in BUFFER or WORK
c     iprhoinv  Starting address of array rhoinv in work
c     iprhside  Starting address of array rhside in buffer
c     iprhsinc  Starting address of array rhsinc if work
c     iprindsc  Starting address of array RINDSC in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iprintur  Starting address of array rintur in array RINPUT
c     iptemp    Starting address of temporary array in IBUFFR
c     ipveloc   Starting address of array VELOC in array WORK
c     ipwork    Starting address of array work in buffer
c     iside     Sequence number of side
c     iupwind   Type of upwinding
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c     maniso    Indicates type of nonlinear turbulence (anisotropic)
c               modelling
c     mcom      If zero the flow is incompressible else (1) compressible
c     mequat    Indicates the type of momentum equation to be solved:
c               0:  standard equation
c               1:  multi-phase gas flow
c     mturb     Indicates type of turbulence modelling
c     nconct    Number of connections in a row of the matrix
c     ndegfd    Number of degrees of freedom
c     newtcom   If one Newton linearisation is used for convective terms
c               if compressible flow.
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nintpol   Number of interpolation points in the distribution
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     pnttyps   Contains the value of pnttype for all 5 integration points
c     reduced   If true the reduced stress tensor corresponding
c               to a constant viscosity and incompressible flow is used
c     reuse     If true the computed coefficients may be reused again
c     tangcell  If true a tangential cell has been found with integration point
c               at the boundary
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBGEO2D  Fill geometrical quantities along boundary in R2
c     ISBMOMCC2 Fill normal half cells for convection in R2
c     ISBMOMP2  Fill normal half cells for pressure bc for Euler eqs. in R2
c     ISBMOMS1  Fill tangential cells for stress in R2
c     ISBMOMS2  Fill normal half cells for stress in R2
c     ISBMOMT2  Correct the time derivative part of the matrix diagonal
c               with respect to the boundary cells where sigma^nn is given
c     ISBUPWVEL Update the multiplication vector which indicates from which
c               side upwind must be taken for half cells where we have an
c               outflow boundary condition
c     ISCCRHSD  Right-hand-side part of newton linearized convection term
c     ISCHRYS2  Fill chrystoffel symbols in one integration point in R2
c     ISCMFL2   Compute pseudo velocity of Newton linearisation
c     ISCNRHSD  Right-hand-side part of newton linearized convection term
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISDISCC2D Distribution of convective terms
c     ISDISCN2D Distribution of convective terms
c     ISDISCU2D Distribution of convective terms
c     ISDISS2D  Distribution of stress terms
c     ISGMAT2D  Compute gradient matrix in R^2
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISJCTA2D  Fill Jacobian times contravariant base vectors
c               in one integration point in R2
c     ISJCTG2D  Fill Jacobian times contravariant metric tensors
c               in one integration point in R2
c     ISMAKMPC2 Make a list of integration points for the momentum equations
c               in R2
c     ISMNW2D   Add contribution newton linearisation compressible flow
c               to rhside in R2
c     ISMOMCCV2 Fill convective terms for one integration point in R2
c     ISMOMF22  All parts of the matrix outside the region are folded to
c               internal domain for the momentum matrix in R2
c               This subroutine is meant for a 23-point stencil,
c               compressible case.
c     ISMOMF2D  Momentum folding subroutine in R2
c     ISMOMRH2  Build the right-hand side source term in R2
c     ISMOMST2  Fill stress terms for one integration point in R2
c     ISMOMTD2  Adds the time derivative to the idagonal of the matrix and
c               the right-hand side
c     ISMOMUR   Apply underrelaxation with regard to the previous step in
c               the momentum equation
c     ISMTVC23  Gradient matrix times pressure in 2D
c     ISMULMSFR Multiply viscosity mu by massfraction eps
c     ISNLIN2D  Update right-hand side vector with nonlinear effects of
c               stress-strain relationship in case of nonlinear modeling
c     ISSETCON  Make array equal to a constant
c     ISUPWVEL2 Compute sign vectors corresponding to velocity for upwind
c     ISURHSC2D Update right-hand side vector with higher order upwind
c               scheme (2D)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Make list of integration points
c    Interpolate coefficients to integration points
c    For idim := 1 (1) 2 do
c       Build time-derivative part of the matrix and rhside
c       Make list of integration points
c       For all integration points do
c          Fill coefficients for convective terms
c          Fill multiplication factors for convective terms
c          distribute convective terms
c          Fill coefficients for stress terms
c          Fill multiplication factors for stress terms
c          Correct boundary cells for stress terms
c          distribute stress terms
c    Compute gradient matrix
c    Correct right-hand-side term with minus gradient matrix times p^n
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute constants

      ipiindsc = iinput(6)
      ipiintim = iinput(4)
      ipiintur = iinput(8)
      iprintim = iinput(29)
      iprintur = iinput(31)
      iprindsc = iinput(33)
      ni       = iinput(51)
      nj       = iinput(52)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      ipiincom = iinput(9)
      mcom     = iinput(ipiincom)
      mequat   = iinput(ipiincom+2)
      mturb    = iinput(ipiintur-1+2)
      maniso   = iinput(ipiintur-1+4)
      nconct   = infmm(6)
      deltat   = rinput(iprintim -1 +4)

c     --- compute pointers in array work

      ipcva    = 1
      ipjcta   = ipcva     + 16*npointsv
      ipjctg   = ipjcta    +  4*npointsv
      ipchsym  = ipjctg    +  3*npointsv
      iprho    = ipchsym   +  6*npointsv
      ipmu     = iprho     +  1*npointsv
      ipveloc  = ipmu      +  1*npointsv
      iprhoinv = ipveloc   +  2*npointsv
      ipgradu  = iprhoinv  +  1*npointsv
      ipmassfr = ipgradu   +  1*npointsv
      ipflux   = ipmassfr  +  8*npointsv
      iprhsinc = ipflux    +  2*npointsv
      ipinrho  = iprhsinc  +  2*npointsv
      ipprems  = ipinrho   +  2*npointsv
      ipgeo    = ipprems   + 16*npointsv

      ipadir   = ipjctg

      idiscret = iinput(ipiindsc+1)
      iupwind  = iinput(ipiindsc+4)
      iconserv = iinput(ipiindsc+8)
      iconv    = iinput(ipiintim -1 + 17)

      if ( iinput(ipiindsc-1+8).eq.1.and.mcom.eq.1) then
         newtcom  = 1
      else
         newtcom  = 0
      end if

      if ( idiscret.eq.3 ) then
         idiscret = 1
         reduced = .true.
      else
         reduced = .false.
      end if

c     --- Clear array matrix
c         Clear array rhside

      call issetcon ( matrix, (ni+1)*(nj+1)*nconct*2, 0d0 )
      call issetcon ( rhside, (ni+1)*(nj+1)*2, 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- Fill array bcgeo and put into array work

      call isbgeo2d ( work(ipgeo), ni, nj, nvirtual, work(ipcva),
     +                ibipoint, invjac, jac )

      do alphad = 1, 2

c     --- Build time-derivative part of the matrix and rhside
c         Interpolate rho to integration point (Cell centre)

         iarea(1,1) = 1
         iarea(1,2) = 1
         iarea(2,1) = ni
         iarea(2,2) = nj
         if ( alphad.eq.1 ) then
            iarea(2,1) = ni+1
         else
            iarea(2,2) = nj+1
         end if
         vartyp = 3
         ipmatrix = 1+(alphad-1)*(ni+1)*(nj+1)
         iprhside = ipmatrix

c        --- Compute local co-ordinates, distances to cell center in
c            global co-ordinates, point type, number of interpolation
c            points and interpolatoin formula

         call ismakmpc2 ( alphad, locdx, pnttyps, dcell,
     +                    nintpol, dij )
         pnttyp = pnttyps(1)
         if ( mcom.eq.0 ) then

c        --- incompressible case, interpolate rho from coefs

            acttyp = 0
            call isintcf2 ( coefs(1,1), work(iprho), ni, nj, nvirtual,
     +                      vartyp, acttyp, pnttyp, 0, coor )

         else

c        --- compressible case, interpolate rho from eddy

            acttyp = 0
            call isintcf2 ( eddy, work(iprho), ni, nj, nvirtual,
     +                      vartyp, acttyp, pnttyp, 0, coor )

         end if
         if ( mequat.eq.1 ) then

c        --- two-phase gas flow, interpolate eps from coefs
c            Store in work from position ipmassfr

            acttyp = 0
            call isintcf2 ( coefs(1,6), work(ipmassfr), ni, nj,
     +                      nvirtual, vartyp, acttyp, pnttyp, 0, coor )

c           --- The coeffcient of the zeroth order term is interpolated
c               At this moment this coefficient is stored in the position
c               of the velocity, since that vector is only used in the
c               convective part

            call isintcf2 ( coefs(1,7+alphad), work(ipveloc), ni, nj,
     +                      nvirtual, vartyp, acttyp, pnttyp, 0, coor )

         end if

         idummy = 0
         call ismomtd2 ( alphad, ni, nj, nvirtual, work(iprho),
     +                   solut(1,alphad), matrix(ipmatrix),
     +                   rhside(iprhside), coor, deltat, iarea, mcom,
     +                   work(ipmassfr), mequat, work(ipveloc), idummy )
         if ( mcom.eq.1 ) call issetcon ( work(iprho), npointsv, 1d0 )

c        --- Correct for half cells

         call isbmomt2 ( ni, nj, nvirtual, matrix(ipmatrix),
     +                   rhside(iprhside), alphad, ibinfor, ibipoint )

c        --- Compute inverse of Jacobian, contravariant basis vectors
c            and christoffel symbols

         call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                   work(ipjcta), ni, nj, nvirtual )

c        --- The metric tensor times sqrt(g)

         call isjctg2d ( invjac(1,pnttyp), work(ipjcta),
     +                   work(ipjctg), ni, nj, nvirtual )

c        --- The christoffel symbols of the second kind

         call ischrys2 ( work(ipcva), work(ipjcta),
     +                   work(ipchsym), invjac(1,pnttyp), ni,
     +                   nj, nvirtual, pnttyp )

c        --- Next the convective terms

         iconserv = iinput(ipiindsc+8)

         if ( iinput(ipiindsc).eq.1 ) then

c        --- iindsc(1)=1, use convection term
c            Loop over integration points

            do i = 0, 2

c           --- First clear array vsum
c               The conservative scheme is only applied in the direction
c               corresponding to the unknown in the cell centre
c               In the other direction a non-conservative scheme is applied

               if ( i.eq.2 ) iconserv = 0
               reuse = i.eq.0 .or. iconserv.eq.1 .and. i.eq.1 .or.
     +                 iupwind.gt.0 .and. iconserv.eq.1
               call issetcon ( vsum, 2*npointsv, 0d0 )

               if ( i.gt.0 ) pnttyp = pnttyps(2*i+1)

c              --- Fill some coefficients

               if ( .not. reuse ) then

c              --- Fill rho except for first integration point
c                  In the case of a central conservative scheme we use also
c                  Centroid points for the second integration point

                  if ( mcom.eq.0 ) then
                     vartyp = 3
                     acttyp = 0
                     call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )
                  end if
               end if

c              --- Fill velocity componentwise

               if ( iconserv.eq.0 .or. .not. reuse .or. i.eq.0 ) then

c              --- The velocity is required in all integration points,
c                  but in a central conservative scheme we use also
c                  Centroid points for the second integration point
c                  use second timelevel of velocity for iterative pressure
c                  correction

                  if( newtcom.eq.1 ) then

c                 --- calculate pseudo velocity for newton linearisation
c                     compressible flow

                     call iscmfl2 ( ni, nj, nvirtual, ndegfd, solut,
     +                              eddy(1+npointsv), work(ipinrho),
     +                              work(ipflux) )
                  end if

                  acttyp = 0

c                 --- if standard pressure correction or iterative pressure
c                     correction use second timelevel of velocity
c                     if newton linearisation of compressible flow use
c                     pseudo velocity

                  if ( iconv.eq.0 .and. newtcom.ne.1 ) then

                    do idir = 1, 2
                       vartyp = idir
                       iptemp = ipveloc+(idir-1)*npointsv

                       call isintcf2 ( solut(1,idir),
     +                                 work(iptemp), ni,
     +                                 nj, nvirtual, vartyp, acttyp,
     +                                 pnttyp, 0, coor )
                    end do

                  else if ( iconv.eq.1 ) then

                    do idir = 1, 2
                       vartyp = idir
                       iptemp = ipveloc+(idir-1)*npointsv
                       call isintcf2 ( solut(1,idir+ndegfd),
     +                                 work(iptemp), ni,
     +                                 nj, nvirtual, vartyp, acttyp,
     +                                 pnttyp, 0, coor )

                    end do

                  else if ( newtcom.eq.1 ) then

                    do idir = 1, 2
                      vartyp = idir
                      iptemp = ipveloc+(idir-1)*npointsv
                      call isintcf2 ( work(ipflux+(idir-1)*npointsv),
     +                                work(iptemp), ni,
     +                                nj, nvirtual, vartyp, acttyp,
     +                                pnttyp, 0, coor )

                    end do

                  end if

               end if

c              --- Next fill vsum with contribution of convective terms to
c                  integration points

               call ismomccv2 ( ni, nj, nvirtual, work(iprho),
     +                          work(ipchsym), work(ipveloc), vsum,
     +                          alphad, i, iinput(ipiindsc), mcom )

c              --- Correct contribution of convective terms for half cells

               call isbmomcc2 ( ni, nj, nvirtual, work(ipveloc), vsum,
     +                          alphad, work(iprho),
     +                          i, ibinfor, ibipoint,
     +                          iinput(ipiindsc), mcom )

               if ( iconserv.eq.0 ) then

c              --- Non-conservative scheme

                  if ( iinput(ipiindsc-1+8).eq.1 .and. mcom.eq.0 ) then

c                 --- Newton linearization of convective terms,
c                     right-hand-side part, incompressible flow

                     call iscnrhsd ( ni, nj, nvirtual, rhside(iprhside),
     +                               vsum, work(ipveloc), alphad, i,
     +                               iarea, invjac(1,pnttyp) )

                  end if

c                 --- Distribute computed values over the positions
c                     corresponding to the unknowns

                  call isdiscn2d ( ni, nj, nvirtual, matrix, vsum,
     +                             alphad, iarea, iinput(ipiindsc),
     +                             nintpol, dij, mct, i,
     +                             invjac(1,pnttyp) )

               else

c              --- Conservative scheme

                  if ( iinput(ipiindsc-1+8).eq.1 .and. mcom.eq.0 ) then

c                 --- Newton linearization of convective terms,
c                     right-hand-side part, incompressible flow

                     call isccrhsd ( ni, nj, nvirtual, rhside(iprhside),
     +                               vsum, work(ipveloc), alphad, i,
     +                               iarea, invjac(1,pnttyp))

                  end if

c                 --- Distribute computed values over the positions
c                     corresponding to the unknowns

                  if ( iupwind.eq.0 ) then

c                 --- central scheme

                     call isdiscc2d ( ni, nj, nvirtual, matrix, vsum,
     +                                alphad, iarea, iinput(ipiindsc),
     +                                nintpol, dij, mct, i,
     +                                invjac(1,pnttyp) )
                  else

c                 --- upwind scheme
c                     First store terms like (1-sign(v^alpha)) in array
c                     work from position ipprems

                     if ( i.ne.0 ) then

c                    --- Not the central point

                        call isupwvel2 ( ni, nj, nvirtual, solut,
     +                                   work(ipprems), alphad, i )

c                       --- Correct for half cells

                        call isbupwvel ( ni, nj, nvirtual, alphad, i,
     +                                   ibinfor, ibipoint,
     +                                   work(ipprems) )
                     end if
                     call isdiscu2d ( ni, nj, nvirtual, matrix, vsum,
     +                                alphad, iarea, iinput(ipiindsc),
     +                                nintpol, dij, mct, i,
     +                                work(ipprems), invjac(1,pnttyp))

                  end if

               end if

               ichoisdis = iinput(ipiindsc-1+5)
               if ( ichoisdis.gt.2 .and. i.gt.0 ) then

c              --- Higher order upwind scheme
c                  Since defect correction is used this implies updating the
c                  right-hand side

                  if ( mcom.eq.0 ) then

                     call isurhsc2d ( alphad, ni, nj, nvirtual,
     +                                rhside(iprhside), vsum,
     +                                work(ipveloc), solut(1,alphad),
     +                                iarea, i,
     +                                ichoisdis, rinput(iprindsc-1+1),
     +                                rinput(iprindsc-1+2),
     +                                rinput(iprindsc-1+3),
     +                                rinput(iprindsc-1+4),
     +                                invjac(1,pnttyp) )

                  else

                     call isurhsc2d ( alphad, ni, nj, nvirtual,
     +                                rhside(iprhside), vsum,
     +                                work(ipveloc),
     +                                eddy((1+alphad)*npointsv+1),
     +                                iarea, i,
     +                                ichoisdis, rinput(iprindsc-1+1),
     +                                rinput(iprindsc-1+2),
     +                                rinput(iprindsc-1+3),
     +                                rinput(iprindsc-1+4),
     +                                invjac(1,pnttyp) )
                  end if

               end if

            end do

c           --- add contribution of newton linearisation for compressible flow
c               to the rhside

            if ( newtcom.eq.1 ) then

               call ismnw2d ( rhside, solut, coefs, work(ipchsym),
     +                        vsum, invjac, coor, mct, ibipoint,
     +                        ibinfor, work(ipveloc), work(iprho),
     +                        eddy, work(iprhsinc),
     +                        iinput(ipiindsc), rinput(iprindsc),
     +                        pnttyps, work(ipprems), iarea,
     +                        ni, nj, iconserv, iupwind, npointsv,
     +                        iconv, iprhside, nvirtual, pnttyp, alphad,
     +                        nintpol, dij, mcom, ndegfd)
            end if

         end if

c        --- Stress terms

         if ( iinput(ipiindsc-1+3).eq.0 ) then

c        --- Loop over integration points

            do i = 1, 5

c           --- Clear arrays vsum and dvsum

               call issetcon ( vsum, 2*npointsv, 0d0 )
               call issetcon ( dvsum, 4*npointsv, 0d0 )
               pnttyp = pnttyps(i)

               reuse = locdx(1,i).eq.1 .or. locdx(2,i).eq.1

c              --- List of loop bounds for all integration points
c                  iparea gives the bounds for the cell indices of the
c                  integration points
c                  These are the indices of the centres of the integration cells
c                  but shifted over a distance dcell, since the actual
c                  integration points may be in another cell

               iparea(1,1) = iarea(1,1)+dcell(1,i)
               iparea(2,1) = iarea(2,1)+dcell(1,i)
               iparea(1,2) = iarea(1,2)+dcell(2,i)
               iparea(2,2) = iarea(2,2)+dcell(2,i)

               if ( i.ne.1 .and. .not. reuse ) then

c              --- Compute inverse of Jacobian, contravariant basis vectors
c                  and christoffel symbols

                  call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                            work(ipjcta), ni, nj, nvirtual )

c                 --- The metric tensor times sqrt(g)

                  call isjctg2d ( invjac(1,pnttyp), work(ipjcta),
     +                            work(ipjctg), ni, nj, nvirtual )

c                 --- The christoffel symbols of the second kind

                  call ischrys2 ( work(ipcva), work(ipjcta),
     +                            work(ipchsym), invjac(1,pnttyp), ni,
     +                            nj, nvirtual, pnttyp )

               end if
               vartyp = 3
               acttyp = 0
               if ( .not. reuse ) then
                  call isintcf2 ( coefs(1,2), work(ipmu), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )
                  if ( mequat.eq.1 .and. i.ne.1 ) then
                     call isintcf2 ( coefs(1,6), work(ipmassfr), ni, nj,
     +                               nvirtual, vartyp, acttyp, pnttyp,
     +                               0, coor )

c                    --- Store mu eps in mu

                     call ismulmsfr ( work(ipmu), work(ipmassfr), ni,
     +                                nj, nvirtual )

                  end if
               end if

               call ismomst2 ( ni, nj, nvirtual, work(ipmu),
     +                         work(ipjctg), work(ipchsym), vsum,
     +                         dvsum, alphad, locdx(1,i), iparea,
     +                         nconct, jac, reduced, mequat )

c              --- Correction for tangential boundary cells

               if ( i.gt.1 ) then

c              --- Only the integration point at the boundary is important

                  tangcell = .false.
                  if ( alphad.eq.1 ) then
                     if ( locdx(2,i).eq.-1 ) then
                        iside = 3
                        tangcell = .true.
                     else if ( locdx(2,i).eq.1 ) then
                        iside = 4
                        tangcell = .true.
                     end if
                  else
                     if ( locdx(1,i).eq.-1 ) then
                        iside = 1
                        tangcell = .true.
                     else if ( locdx(1,i).eq.1 ) then
                        iside = 2
                        tangcell = .true.
                     end if
                  end if

c                 --- Get information on boundary

                  if ( tangcell ) then
                     call isbmoms1 ( ni, nj, nvirtual, iside, vsum,
     +                               dvsum, alphad, work(ipmu),
     +                               work(ipjctg), work(ipchsym),
     +                               dcell(1,i), ibinfor, ibipoint,
     +                               bndcon, rhside, matrix,
     +                               invjac, mct, nconct, reduced,
     +                               mequat )
                  end if

               end if

c              --- Correct contribution of stress terms for half cells

               call isbmoms2 ( ni, nj, nvirtual, vsum, dvsum, alphad,
     +                         work(ipmu), work(ipjctg), work(ipchsym),
     +                         locdx(1,i), dcell(1,i), ibinfor,
     +                         ibipoint, bndcon, rhside, nconct, jac,
     +                         reduced, mequat )

c              --- Distribution of matrix

               call isdiss2d ( i, ni, nj, nvirtual, matrix, vsum,
     +                         dvsum, invjac, mct, dcell(1,i), alphad,
     +                         iarea, nintpol(1,i), dij(1,1,1,i),
     +                         nconct, work(iprhoinv), mcom,
     +                         eddy(npointsv+1),
     +                         mequat, coefs(1,6) )

               if ( maniso.gt.0 ) then

c              --- Adding nonlinear effects of stress-strain relationship
c                  Since defect correction is used this implies updating the
c                  right-hand side

                  ichoisdis = iinput(ipiindsc-1+1)
                  ipwork  = ipveloc+2*npointsv
                  if ( iinput(ipiindsc).ne.1 .and. i.ne.1 .and.
     +                 .not. reuse ) then

c                 --- In the case there is no convection,
c                     rho must computed in integration points

                     vartyp = 3
                     acttyp = 0
                     call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                               nvirtual, vartyp, acttyp, pnttyp,
     +                               0, coor )
                  end if
                  call isnlin2d ( rhside(iprhside), work(ipwork), solut,
     +                            work(ipcva+4*npointsv*(pnttyp-1)),
     +                            work(ipjctg), work(ipchsym),
     +                            invjac(1,pnttyp), rinput(iprintur),
     +                            work(iprho), jac, locdx(1,i), alphad,
     +                            ichoisdis, ndegfd, ni, nj, npointsv,
     +                            mturb, nvirtual, pnttyp, iarea,
     +                            dcell(1,i) )

               end if

            end do

         else

c        --- For Euler boundary condition for pressure
c            Compute contribution rhside of pressure boundary
c            condition for Euler eqs., sigmann given.

            call isbmomp2 ( ni, nj, nvirtual, alphad,
     +                      work(ipjctg), work(ipchsym),
     +                      ibinfor, ibipoint, bndcon,
     +                      rhside )

         end if

c        --- Compute the g-matrix

         call isgmat2d ( alphad, ni, nj, nvirtual, gmat, work(ipcva),
     +                   invjac, ibipoint, ibinfor, idiscret, jac,
     +                   work(ipadir) )

      end do

c     --- Finally the folding subroutine
c         All unknowns outside the region are folded insidewards

      if ( idiscret.eq.1 .and. nconct.eq.17 ) then

c     --- Standard discretization for compressible N-S

         call ismomf22 ( ni, nj, nvirtual, mct, matrix, rhside,
     +                   ibinfor, ibipoint, work(ipgeo), bndcon )

      else

c     --- Standard discretization

         call ismomf2d ( ni, nj, nvirtual, mct, matrix, rhside,
     +                   ibinfor, ibipoint, work(ipgeo), bndcon )

      end if

c     --- Update the right-hand side with -Gp^n
c         In two-phase flow: -eps Gp^n**

      call ismtvc23 ( gmat, solut(1,3), rhside, ni, nj, nvirtual,
     +                infmg(infmg(12)), coefs(1,6), mequat )

      if ( mcom.eq.0 ) then

c     --- incompressible case, use rho from coefs

         call ismomrh2 ( coefs(1,3), coefs(1,1), coor, work, gmat,
     +                   infmg(infmg(12)), rhside, work(1+2*npointsv),
     +                   ni, nj, nvirtual, ibipoint, ibinfor,
     +                   work(ipcva) )

      else

c     --- compressible case, use rho from eddy

         call ismomrh2 ( coefs(1,3), eddy(npointsv+1), coor, work,
     +                   gmat, infmg(infmg(12)), rhside,
     +                   work(1+2*npointsv), ni, nj, nvirtual,
     +                   ibipoint, ibinfor, work(ipcva) )

      end if

c     --- apply underrelaxation to the momentum matrix

      if( iconv.eq.1) then

         call ismomur(matrix, rhside, eddy(2*npointsv+1),
     +                                     ni, nj, nconct, nvirtual)

      end if
      end
