      subroutine istrad3d ( ni, nj, nk, nvirtual, kdif, jac, dtsum,
     +                      iparea, locdx, alphad, coef, cva, jcta,
     +                      cvc, ctc, idiscret )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.1    date   02-09-1996 small error found
c        version 1.0    date   01-05-1996
c
c
c   copyright (c) 1995-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute contribution of diffusion term in R3 for the transport
c     equations
c     One integration point only
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c     discretization
c     transport_matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni ,nj, nk, nvirtual, alphad, iparea(2,3), locdx, idiscret
      double precision kdif(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,6),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual),
     +                 dtsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3)
      double precision coef(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3),
     +                 cvc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3),
     +                 ctc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3)
      double precision jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3,3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     coef      i    A specific coefficient
c     ctc       i    Work array containing the contravariant WESBEEK vectors
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     cvc       i    Work array containing the covariant WESBEEK vectors
c     dtsum     o    Flux derivative contributions to the matrix
c     idiscret  i    Type of discretization
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     jac       i    sqrt(g) in pnttyp
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     kdif      i    The diffusion coefficient in a given integration point
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp, betad, istwo(3,3), dx(2,3), gammad, i,
     +        iptensor1, cyclb(1:3), cyclg(1:3)
      double precision det

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     cyclb     Cyclic index
c     cyclg     Cyclic index
c     det       Determinant
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     gammad    Direction index
c     i         General loop variable
c     ip        Local counting variable i-direction
c     iptensor1 Tensor index
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      This subroutine may only be called for centres of integration cell
c      sides.
c      The diffusion terms in the classical discretization are given by
c
c                   ab         |+e(alphad)/2
c        - sqrt(g) K   T,      |
c                        betad |-e(alphad)/2
c
c      The diffusion terms in the WesBeek discretization are given by
c
c
c                   (a)  (b)     |+e(a)/2
c        - sqrt(g) a   .c    T,  |
c                  -    -      b |-e(a)/2
c
c             (b)
c      where c    are the so-called WesBeek vectors defined in the following
c            -
c      way:
c
c      In principle, we want to approximate, for example, the following
c      diffusion term
c
c                (1)    _
c      -sqrt(g) a   . k VT  in point (i+1/2,j,k)
c               -
c            _          1       2       3                         (1)
c      where VT = (dT/dx , dT/dx , dT/dx ) and note that sqrt(g) a
c                                                                -
c                          1
c      is continue along xi  = constant.
c                                           _
c      Suppose that k is discontinue,  then VT is discontinue, whereas
c       _                                             _
c      kVT is smooth. Use this property to determine kVT at cell faces.
c      This can be done as follows:
c
c        |(i+1,j,k)      1  _        _                 1
c      T |           = S - kVT dx = kVT|           . S - dx
c        |(i,j,k)        k      -       (i+1/2,j,k)    k  -
c
c      (note: S is denoted as a integral!!)
c
c      The last integral can be approximated as follows:
c
c                          a                a
c               1      1   -(1)             -(1)
c      c    = S -dx  = - (----- |        + ----- |         )
c      -(1)     k -    2    k    (i,j,k)     k    (i+1,j,k)
c
c      In order to find two more equations, other integration paths have
c      to be taken. With this, we can calculate the other covariant WesBeek
c      vectors. These are:
c
c                a                  a               a
c             1  -(2)               -(2)            -(2)
c      c    = -(-----|          + 2-----|        + -----|          +
c      -(2)   8   k   (i,j-1,k)      k   (i,j,k)     k   (i,j+1,k)
c
c
c            a                    a                 a
c            -(2)                 -(2)              -(2)
c           -----|            + 2-----|          + -----|           )
c             k   (i+1,j-1,k)      k   (i+1,j,k)     k   (i+1,j+1,k)
c
c
c      and
c
c                a                  a               a
c             1  -(3)               -(3)            -(3)
c      c    = -(-----|          + 2-----|        + -----|          +
c      -(3)   8   k   (i,j,k-1)      k   (i,j,k)     k   (i,j,k+1)
c
c
c            a                    a                 a
c            -(3)                 -(3)              -(3)
c           -----|            + 2-----|          + -----|           )
c             k   (i+1,j,k-1)      k   (i+1,j,k)     k   (i+1,j,k+1)
c
c
c                                   _
c      Solving three equations for kVT results in
c
c       _                 (a)
c      kVT|            = c    T   |
c          (i+1/2,j,k)   -     ,a  (i+1/2,j,k)
c                                                  (a)
c      Finally, the contravariant WesBeek vectors c    can be computed
c                                                 -
c      with the following formulae
c
c              c    x c
c       (a)    -(b)   -(c)
c      c     = -----------          a,b,c cyclic
c      -            C
c
c      with C = c   .(c    x c   )
c               -(1)  -(2)   -(3)
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2, 4,
     +             2, 3, 5,
     +             4, 5, 6 /
c ======================================================================
c
      if ( idiscret.eq.1 ) then

c     --- Standard ISNaS discretization
c         Calculate the coefficient in point e(alphad)/2

        do betad = 1, 3

          iptensor1 = istwo ( alphad, betad )
          do kp = iparea(1,3), iparea(2,3)
            do jp = iparea(1,2), iparea(2,2)
              do ip = iparea(1,1), iparea(2,1)
                dtsum(ip,jp,kp,betad) = dtsum(ip,jp,kp,betad) - locdx*
     +                                  kdif(ip,jp,kp,iptensor1)*
     +                                  jac(ip,jp,kp)
              end do
            end do
          end do

        end do

      else

c     --- Wesbeek discretization

        dx(1,1)      =  0
        dx(1,2)      =  0
        dx(1,3)      =  0
        dx(1,alphad) = -1

        do kp = iparea(1,3), iparea(2,3)
          do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

              do betad = 1, 3
                do gammad = 1, 3

                  dx(2,1)      =  0
                  dx(2,2)      =  0
                  dx(2,3)      =  0
                  dx(2,gammad) = -1

                  if ( gammad.eq.alphad ) then

                    cvc(ip,jp,kp,betad,gammad) = 5d-1 *
     +                                   ( cva(ip+dx(1,1),jp+dx(1,2),
     +                                         kp+dx(1,3),betad,gammad)/
     +                                     coef(ip+dx(1,1),jp+dx(1,2),
     +                                          kp+dx(1,3)) +
     +                                     cva(ip,jp,kp,betad,gammad)/
     +                                     coef(ip,jp,kp) )

                  else

                    cvc(ip,jp,kp,betad,gammad) = 125d-3 *
     +               ( cva(ip+dx(1,1)-dx(2,1),jp+dx(1,2)-dx(2,2),
     +                   kp+dx(1,3)-dx(2,3),betad,gammad)/
     +               coef(ip+dx(1,1)-dx(2,1),jp+dx(1,2)-dx(2,2),
     +                    kp+dx(1,3)-dx(2,3)) + 2d0 *
     +               cva(ip+dx(1,1),jp+dx(1,2),kp+dx(1,3),betad,gammad)/
     +               coef(ip+dx(1,1),jp+dx(1,2),kp+dx(1,3)) +
     +               cva(ip+dx(1,1)+dx(2,1),jp+dx(1,2)+dx(2,2),
     +                   kp+dx(1,3)+dx(2,3),betad,gammad)/
     +               coef(ip+dx(1,1)+dx(2,1),jp+dx(1,2)+dx(2,2),
     +                    kp+dx(1,3)+dx(2,3)) +
     +               cva(ip-dx(2,1),jp-dx(2,2),kp-dx(2,3),betad,gammad)/
     +               coef(ip-dx(2,1),jp-dx(2,2),kp-dx(2,3)) + 2d0 *
     +               cva(ip,jp,kp,betad,gammad)/coef(ip,jp,kp) +
     +               cva(ip+dx(2,1),jp+dx(2,2),kp+dx(2,3),betad,gammad)/
     +               coef(ip+dx(2,1),jp+dx(2,2),kp+dx(2,3)) )

                  end if

                end do
              end do

            end do
          end do
        end do

c       --- Determine contravariant WESBEEK vectors

        do kp = iparea(1,3), iparea(2,3)
          do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

c           --- First, calculate determinant ...

               det = cvc(ip,jp,kp,1,1) *
     +               ( cvc(ip,jp,kp,2,2)*cvc(ip,jp,kp,3,3) -
     +                 cvc(ip,jp,kp,3,2)*cvc(ip,jp,kp,2,3) ) +
     +               cvc(ip,jp,kp,2,1) *
     +               ( cvc(ip,jp,kp,3,2)*cvc(ip,jp,kp,1,3) -
     +                 cvc(ip,jp,kp,1,2)*cvc(ip,jp,kp,3,3) ) +
     +               cvc(ip,jp,kp,3,1) *
     +               ( cvc(ip,jp,kp,1,2)*cvc(ip,jp,kp,2,3) -
     +                 cvc(ip,jp,kp,2,2)*cvc(ip,jp,kp,1,3) )

               do betad = 1, 3

                 cyclb(1) = betad
                 cyclb(2) = mod(betad  ,3) + 1
                 cyclb(3) = mod(betad+1,3) + 1

                 do gammad = 1, 3

                   cyclg(1) = gammad
                   cyclg(2) = mod(gammad  ,3) + 1
                   cyclg(3) = mod(gammad+1,3) + 1

c                  --- contravariant WESBEEK vectors

                   ctc(ip,jp,kp,gammad,betad) =
     +                  ( cvc(ip,jp,kp,cyclg(2),cyclb(2)) *
     +                    cvc(ip,jp,kp,cyclg(3),cyclb(3)) -
     +                    cvc(ip,jp,kp,cyclg(3),cyclb(2)) *
     +                    cvc(ip,jp,kp,cyclg(2),cyclb(3))  )/det

                 end do

               end do

            end do
          end do
        end do

        do betad = 1, 3

          do kp = iparea(1,3), iparea(2,3)
            do jp = iparea(1,2), iparea(2,2)
              do ip = iparea(1,1), iparea(2,1)

                do i = 1, 3
                  dtsum(ip,jp,kp,betad) = dtsum(ip,jp,kp,betad) -
     +                                    locdx * ctc(ip,jp,kp,i,betad)*
     +                                    jcta(ip,jp,kp,i,alphad)
                end do

              end do
            end do
          end do

        end do

      end if

      end
