      subroutine istraf3d ( ni, nj, nk, nvirtual, mct, matrix, rhside,
     +                      ibinfor, ibipoint, bndcon )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date 03-05-1996
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    All parts of the matrix outside the region are folded to internal
c    domain for the transport matrix in R3
c    The folding process depends on the types of boundary conditions
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ibipoint(*), ibinfor(*),
     +        mct(-2:2,-2:2,-2:2)
      double precision rhside(ni+1,nj+1,nk+1),
     +                 matrix(ni+1,nj+1,nk+1,*), bndcon(3,*)

c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer diri, robb, dtur
      parameter( diri = 1, robb=2, dtur=3 )
      logical multbn1, multbn2
      integer btype1, btype2, ipibinf, ipoint, i, btype,
     +        iconcto, iconctc, iconcti, ip, jp, kp, xp(3), ndir, k,
     +        l, m, tdir(2), nsign, iside, iparea(2,3), nij
      double precision fact

c     btype     Type of boundary condition
c     btype1    Type of boundary condition at one of the boundaries in vertex
c     btype2    Type of boundary condition at one of the boundaries in vertex
c     diri      Dirichlet boundary condition, i.e. T is given
c     dtur      Dirichlet condition for turbulence, i.e. T
c               given at the centroids of boundary cells
c     fact      Multiplication factor
c     i         General loop variable
c     iconctc   Connection in matrix for central point
c     iconcti   Connection in matrix for internal point
c     iconcto   Connection in matrix for outside point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     iside     Sequence number of side
c     jp        Local counting variable j-direction
c     k         General loop variable
c     kp        Local counting variable k-direction
c     l         General loop variable
c     m         General loop variable
c     multbn1   If true the boundary is periodical or multiblock
c     multbn2   If true the boundary is periodical or multiblock
c     ndir      Direction of normal vector on this side
c     nij       ni or nj
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     robb      Robbins boundary condition, i.e. aT + (k grad T).n given
c                                                       T
c     tdir      Direction of tangent vector on this side
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The folding process will be carried out per side.
c    Some rows corresponding to unknowns in the neighbourhood of the rectangle
c    require special attention
c
c    For illustration purposes we limit ourselves to the lower side and
c    in case of a corner to the combination of lower side and left-hand side
c    in two dimensions
c
c            -------------------------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     -     +     -     +     |
c            |           |           |           |
c            ------------------|-------------------------- lower boundary
c
c                  +           +           +
c
c
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. T given at the boundary (.,.,-1):
c         T(.,.,-2) = 2T(.,.,-1) - T(.,.,0)                         (3)
c    c. T not given:
c         T(.,.,-2) = 2T(.,.,0) - T(.,.,2)                          (4)
c
c
c    Finally we consider a cell at a corner
c
c                        -------------------------
c                        |           |           |
c                  +     |     +     |     +     |
c                        |           |           |
c                        ------|------------------
c                        |           |           |
c                  +     -     +     -     +     |
c                        |           |           |
c                        ------|-------------------------- lower boundary
c
c                  +           +           +
c
c    The folding process is as follows:
c
c    a. If point (-2,-2,0) does not correspond to periodical or multiblock bc's
c       - T(-2,-2,0) = ( 2T(0,-2,0)-T(2,-2,0) + 2T(-2,0,0)-T(-2,2,0) )/2   (8)
c    b. All other points are eliminated at their respective boundaries
c
c
c                    PSEUDO CODE
c
c      For corner normal cells do
c            eliminate the point "(-2,-2,.)" if necessary
c      For iside := 1 (1) 6 do
c         For all boundary cells do
c            Perform the folding process over all outside connections
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- First the 12 sets of virtual vertex unknowns are eliminated if
c         necessary

c         --- Vertex (i,1,1)

      do ip = 1, ni

        btype1 = ibinfor(ibipoint(3)-1+1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(2*ip-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(5)-1+2*ip-1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ip,1,1,10)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ip,1,1,12) = matrix(ip,1,1,12)+fact
            matrix(ip,1,1,14) = matrix(ip,1,1,14)-0.5d0*fact
            matrix(ip,1,1,3) = matrix(ip,1,1,3)+fact
            matrix(ip,1,1,15) = matrix(ip,1,1,15)-0.5d0*fact
            matrix(ip,1,1,10) = 0d0

          else

c         --- Boundary 5 periodic, eliminate -1,-1,0 in direction 5

            matrix(ip,1,1,12) = matrix(ip,1,1,12)+2d0*fact
            matrix(ip,1,1,14) = matrix(ip,1,1,14)-fact
            matrix(ip,1,1,10) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 3 periodic, eliminate -1,-1,0 in direction 3

          matrix(ip,1,1,3) = matrix(ip,1,1,3)+2d0*fact
          matrix(ip,1,1,15) = matrix(ip,1,1,15)-fact
          matrix(ip,1,1,10) = 0d0

        end if

      end do

c     --- Vertex (i,nj,1)

      do ip = 1, ni

        btype1 = ibinfor(ibipoint(4)-1+1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(2*ip-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(5)-1+2*ip-1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(2*nj+1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ip,nj,1,14)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ip,nj,1,12) = matrix(ip,nj,1,12)+fact
            matrix(ip,nj,1,10) = matrix(ip,nj,1,10)-0.5d0*fact
            matrix(ip,nj,1,8) = matrix(ip,nj,1,8)+fact
            matrix(ip,nj,1,19) = matrix(ip,nj,1,19)-0.5d0*fact
            matrix(ip,nj,1,14) = 0d0

          else

c         --- Boundary 5 periodic, eliminate -1,-1,0 in direction 5

            matrix(ip,nj,1,12) = matrix(ip,nj,1,12)+2d0*fact
            matrix(ip,nj,1,10) = matrix(ip,nj,1,10)-fact
            matrix(ip,nj,1,14) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 4 periodic, eliminate -1,-1,0 in direction 4

          matrix(ip,nj,1,8) = matrix(ip,nj,1,8)+2d0*fact
          matrix(ip,nj,1,19) = matrix(ip,nj,1,19)-fact
          matrix(ip,nj,1,14) = 0d0

        end if

      end do

c     --- Vertex (i,1,nk)

      do ip = 1, ni

        btype1 = ibinfor(ibipoint(3)-1+2*nk+1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(2*ip-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(6)-1+2*ip-1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ip,1,nk,15)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ip,1,nk,17) = matrix(ip,1,nk,17)+fact
            matrix(ip,1,nk,19) = matrix(ip,1,nk,19)-0.5d0*fact
            matrix(ip,1,nk,3) = matrix(ip,1,nk,3)+fact
            matrix(ip,1,nk,10) = matrix(ip,1,nk,10)-0.5d0*fact
            matrix(ip,1,nk,15) = 0d0

          else

c         --- Boundary 6 periodic, eliminate -1,-1,0 in direction 6

            matrix(ip,1,nk,17) = matrix(ip,1,nk,17)+2d0*fact
            matrix(ip,1,nk,19) = matrix(ip,1,nk,19)-fact
            matrix(ip,1,nk,15) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 3 periodic, eliminate -1,-1,0 in direction 3

          matrix(ip,1,nk,3) = matrix(ip,1,nk,3)+2d0*fact
          matrix(ip,1,nk,10) = matrix(ip,1,nk,10)-fact
          matrix(ip,1,nk,15) = 0d0

        end if

      end do

c     --- Vertex (i,nj,nk)

      do ip = 1, ni

        btype1 = ibinfor(ibipoint(4)-1+2*nk+1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(2*ip-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(6)-1+2*ip-1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(2*nj+1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ip,nj,nk,19)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ip,nj,nk,17) = matrix(ip,nj,nk,17)+fact
            matrix(ip,nj,nk,15) = matrix(ip,nj,nk,15)-0.5d0*fact
            matrix(ip,nj,nk,8) = matrix(ip,nj,nk,8)+fact
            matrix(ip,nj,nk,14) = matrix(ip,nj,nk,14)-0.5d0*fact
            matrix(ip,nj,nk,19) = 0d0

          else

c         --- Boundary 6 periodic, eliminate -1,-1,0 in direction 6

            matrix(ip,nj,nk,17) = matrix(ip,nj,nk,17)+2d0*fact
            matrix(ip,nj,nk,15) = matrix(ip,nj,nk,15)-fact
            matrix(ip,nj,nk,19) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 4 periodic, eliminate -1,-1,0 in direction 4

          matrix(ip,nj,nk,8) = matrix(ip,nj,nk,8)+2d0*fact
          matrix(ip,nj,nk,14) = matrix(ip,nj,nk,14)-fact
          matrix(ip,nj,nk,19) = 0d0

        end if

      end do

c     --- Vertex (1,i,1)

      do jp = 1, nj

        btype1 = ibinfor(ibipoint(1)-1+2*jp-1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(1+nvirtual-1))
        btype2 = ibinfor(ibipoint(5)-1+1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(2*jp-1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(1,jp,1,11)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(1,jp,1,12) = matrix(1,jp,1,12)+fact
            matrix(1,jp,1,13) = matrix(1,jp,1,13)-0.5d0*fact
            matrix(1,jp,1,5) = matrix(1,jp,1,5)+fact
            matrix(1,jp,1,16) = matrix(1,jp,1,16)-0.5d0*fact
            matrix(1,jp,1,11) = 0d0

          else

c         --- Boundary 5 periodic, eliminate -1,-1,0 in direction 5

            matrix(1,jp,1,12) = matrix(1,jp,1,12)+2d0*fact
            matrix(1,jp,1,13) = matrix(1,jp,1,13)-fact
            matrix(1,jp,1,11) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 1 periodic, eliminate -1,-1,0 in direction 1

          matrix(1,jp,1,5) = matrix(1,jp,1,5)+2d0*fact
          matrix(1,jp,1,16) = matrix(1,jp,1,16)-fact
          matrix(1,jp,1,11) = 0d0

        end if

      end do

c     --- Vertex (ni,i,1)

      do jp = 1, nj

        btype1 = ibinfor(ibipoint(2)-1+2*jp-1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(1+nvirtual-1))
        btype2 = ibinfor(ibipoint(5)-1+2*ni+1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(2*jp-1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ni,jp,1,13)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ni,jp,1,12) = matrix(ni,jp,1,12)+fact
            matrix(ni,jp,1,11) = matrix(ni,jp,1,11)-0.5d0*fact
            matrix(ni,jp,1,6) = matrix(ni,jp,1,6)+fact
            matrix(ni,jp,1,18) = matrix(ni,jp,1,18)-0.5d0*fact
            matrix(ni,jp,1,13) = 0d0

          else

c         --- Boundary 5 periodic, eliminate -1,-1,0 in direction 5

            matrix(ni,jp,1,12) = matrix(ni,jp,1,12)+2d0*fact
            matrix(ni,jp,1,11) = matrix(ni,jp,1,11)-fact
            matrix(ni,jp,1,13) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 2 periodic, eliminate -1,-1,0 in direction 2

          matrix(ni,jp,1,6) = matrix(ni,jp,1,6)+2d0*fact
          matrix(ni,jp,1,18) = matrix(ni,jp,1,18)-fact
          matrix(ni,jp,1,13) = 0d0

        end if

      end do

c     --- Vertex (1,i,nk)

      do jp = 1, nj

        btype1 = ibinfor(ibipoint(1)-1+2*jp-1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(2*nk+1+nvirtual-1))
        btype2 = ibinfor(ibipoint(6)-1+1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(2*jp-1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(1,jp,nk,16)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(1,jp,nk,17) = matrix(1,jp,nk,17)+fact
            matrix(1,jp,nk,18) = matrix(1,jp,nk,18)-0.5d0*fact
            matrix(1,jp,nk,5) = matrix(1,jp,nk,5)+fact
            matrix(1,jp,nk,11) = matrix(1,jp,nk,11)-0.5d0*fact
            matrix(1,jp,nk,16) = 0d0

          else

c         --- Boundary 6 periodic, eliminate -1,-1,0 in direction 6

            matrix(1,jp,nk,17) = matrix(1,jp,nk,17)+2d0*fact
            matrix(1,jp,nk,18) = matrix(1,jp,nk,18)-fact
            matrix(1,jp,nk,16) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 1 periodic, eliminate -1,-1,0 in direction 1

          matrix(1,jp,nk,5) = matrix(1,jp,nk,5)+2d0*fact
          matrix(1,jp,nk,11) = matrix(1,jp,nk,11)-fact
          matrix(1,jp,nk,16) = 0d0

        end if

      end do

c     --- Vertex (ni,i,nk)

      do jp = 1, nj

        btype1 = ibinfor(ibipoint(2)-1+2*jp-1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(2*nk+1+nvirtual-1))
        btype2 = ibinfor(ibipoint(6)-1+2*ni+1+nvirtual+
     +                   (2*(nvirtual+ni)+1)*(2*jp-1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ni,jp,nk,18)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ni,jp,nk,17) = matrix(ni,jp,nk,17)+fact
            matrix(ni,jp,nk,16) = matrix(ni,jp,nk,16)-0.5d0*fact
            matrix(ni,jp,nk,6) = matrix(ni,jp,nk,6)+fact
            matrix(ni,jp,nk,13) = matrix(ni,jp,nk,13)-0.5d0*fact
            matrix(ni,jp,nk,18) = 0d0

          else

c         --- Boundary 6 periodic, eliminate -1,-1,0 in direction 6

            matrix(ni,jp,nk,17) = matrix(ni,jp,nk,17)+2d0*fact
            matrix(ni,jp,nk,16) = matrix(ni,jp,nk,16)-fact
            matrix(ni,jp,nk,18) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 2 periodic, eliminate -1,-1,0 in direction 2

          matrix(ni,jp,nk,6) = matrix(ni,jp,nk,6)+2d0*fact
          matrix(ni,jp,nk,13) = matrix(ni,jp,nk,13)-fact
          matrix(ni,jp,nk,18) = 0d0

        end if

      end do

c     --- Vertex (1,1,i)

      do kp = 1, nk

        btype1 = ibinfor(ibipoint(1)-1+1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(2*kp-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(3)-1+2*kp-1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(1,1,kp,2)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(1,1,kp,3) = matrix(1,1,kp,3)+fact
            matrix(1,1,kp,4) = matrix(1,1,kp,4)-0.5d0*fact
            matrix(1,1,kp,5) = matrix(1,1,kp,5)+fact
            matrix(1,1,kp,7) = matrix(1,1,kp,7)-0.5d0*fact
            matrix(1,1,kp,2) = 0d0

          else

c         --- Boundary 3 periodic, eliminate -1,-1,0 in direction 3

            matrix(1,1,kp,3) = matrix(1,1,kp,3)+2d0*fact
            matrix(1,1,kp,4) = matrix(1,1,kp,4)-fact
            matrix(1,1,kp,2) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 1 periodic, eliminate -1,-1,0 in direction 1

          matrix(1,1,kp,5) = matrix(1,1,kp,5)+2d0*fact
          matrix(1,1,kp,7) = matrix(1,1,kp,7)-fact
          matrix(1,1,kp,2) = 0d0

        end if

      end do

c     --- Vertex (ni,1,i)

      do kp = 1, nk

        btype1 = ibinfor(ibipoint(2)-1+1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(2*kp-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(3)-1+2*kp-1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(2*ni+1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ni,1,kp,4)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ni,1,kp,3) = matrix(ni,1,kp,3)+fact
            matrix(ni,1,kp,2) = matrix(ni,1,kp,2)-0.5d0*fact
            matrix(ni,1,kp,6) = matrix(ni,1,kp,6)+fact
            matrix(ni,1,kp,9) = matrix(ni,1,kp,9)-0.5d0*fact
            matrix(ni,1,kp,4) = 0d0

          else

c         --- Boundary 3 periodic, eliminate -1,-1,0 in direction 3

            matrix(ni,1,kp,3) = matrix(ni,1,kp,3)+2d0*fact
            matrix(ni,1,kp,2) = matrix(ni,1,kp,2)-fact
            matrix(ni,1,kp,4) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 2 periodic, eliminate -1,-1,0 in direction 2

          matrix(ni,1,kp,6) = matrix(ni,1,kp,6)+2d0*fact
          matrix(ni,1,kp,9) = matrix(ni,1,kp,9)-fact
          matrix(ni,1,kp,4) = 0d0

        end if

      end do

c     --- Vertex (1,nj,i)

      do kp = 1, nk

        btype1 = ibinfor(ibipoint(1)-1+2*nj+1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(2*kp-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(4)-1+2*kp-1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(1,nj,kp,7)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(1,nj,kp,8) = matrix(1,nj,kp,8)+fact
            matrix(1,nj,kp,9) = matrix(1,nj,kp,9)-0.5d0*fact
            matrix(1,nj,kp,5) = matrix(1,nj,kp,5)+fact
            matrix(1,nj,kp,2) = matrix(1,nj,kp,2)-0.5d0*fact
            matrix(1,nj,kp,7) = 0d0

          else

c         --- Boundary 4 periodic, eliminate -1,-1,0 in direction 4

            matrix(1,nj,kp,8) = matrix(1,nj,kp,8)+2d0*fact
            matrix(1,nj,kp,9) = matrix(1,nj,kp,9)-fact
            matrix(1,nj,kp,7) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 1 periodic, eliminate -1,-1,0 in direction 1

          matrix(1,nj,kp,5) = matrix(1,nj,kp,5)+2d0*fact
          matrix(1,nj,kp,2) = matrix(1,nj,kp,2)-fact
          matrix(1,nj,kp,7) = 0d0

        end if

      end do

c     --- Vertex (ni,nj,i)

      do kp = 1, nk

        btype1 = ibinfor(ibipoint(2)-1+2*nj+1+nvirtual+
     +                   (2*(nvirtual+nj)+1)*(2*kp-1+nvirtual-1))
        btype2 = ibinfor(ibipoint(4)-1+2*kp-1+nvirtual+
     +                   (2*(nvirtual+nk)+1)*(2*ni+1+nvirtual-1))
        multbn1 = btype1.ge.95 .and. btype1.le.99
        multbn2 = btype2.ge.95 .and. btype2.le.99
        if ( btype1.eq.99 .and. btype2.eq.99 ) then

c       --- Both sides are multiblock, treat as eliminate all

          multbn1 = .false.
          multbn2 = .false.

        end if
        fact = matrix(ni,nj,kp,9)
        if ( .not. multbn1 ) then
          if ( .not. multbn2 ) then

c         --- Standard situation, eliminate -1,-1,0 in two directions

            matrix(ni,nj,kp,8) = matrix(ni,nj,kp,8)+fact
            matrix(ni,nj,kp,7) = matrix(ni,nj,kp,7)-0.5d0*fact
            matrix(ni,nj,kp,6) = matrix(ni,nj,kp,6)+fact
            matrix(ni,nj,kp,4) = matrix(ni,nj,kp,4)-0.5d0*fact
            matrix(ni,nj,kp,9) = 0d0

          else

c         --- Boundary 4 periodic, eliminate -1,-1,0 in direction 4

            matrix(ni,nj,kp,8) = matrix(ni,nj,kp,8)+2d0*fact
            matrix(ni,nj,kp,7) = matrix(ni,nj,kp,7)-fact
            matrix(ni,nj,kp,9) = 0d0

          end if

        else if ( .not. multbn2 ) then

c       --- Boundary 2 periodic, eliminate -1,-1,0 in direction 2

          matrix(ni,nj,kp,6) = matrix(ni,nj,kp,6)+2d0*fact
          matrix(ni,nj,kp,4) = matrix(ni,nj,kp,4)-fact
          matrix(ni,nj,kp,9) = 0d0

        end if

      end do

c     --- loop over faces

      do iside = 1, 6

c     --- Compute nij, nsig, tdir and ndir

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  back and front plane

           nij     = nj
           ndir    = 1
           tdir(1) = 2
           tdir(2) = 3
           if ( iside.eq.1 ) then

c          --- iside = 1:  back

              nsign = -1

           else

c          --- iside = 2:  front

              nsign = 1

           end if

         else if ( iside.gt.2  .and. iside.lt.5) then

c        --- iside = 3, 4:  left and right plane

           nij     = nk
           ndir    = 2
           tdir(1) = 3
           tdir(2) = 1
           if ( iside.eq.3 ) then

c          --- iside = 3:  left

              nsign = -1

           else

c          --- iside = 4:  right

              nsign = 1

           end if

         else

c        --- iside = 5, 6:  bottom and top

           nij     = ni
           ndir    = 3
           tdir(1) = 1
           tdir(2) = 2

           if ( iside.eq.5 ) then

c          --- iside = 5:  bottom

              nsign = -1

           else

c          --- iside = 6:  top

              nsign = 1

           end if

         end if

         ipibinf = ibipoint(iside)-1+
     +             nvirtual+(2*(nvirtual+nij)+1)*(nvirtual-1)

c        --- loop over cells
c            Compute loop bounds

         iparea(1,1) = 1
         iparea(1,2) = 1
         iparea(1,3) = 1
         iparea(2,1) = 1
         iparea(2,2) = 1
         iparea(2,3) = 1

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  back and front plane

            iparea(2,2) = nj
            iparea(2,3) = nk
            if ( iside.eq.2 ) then

c           --- iside = 2:  front plane

               iparea(1,1) = ni
               iparea(2,1) = ni

            end if

         else if (iside.gt.2  .and. iside.lt.5) then

c        --- iside = 3, 4:  left and right plane

            iparea(2,1) = ni
            iparea(2,3) = nk
            if ( iside.eq.4 ) then

c           --- iside = 4:  right plane

               iparea(1,2) = nj
               iparea(2,2) = nj

            end if

         else

c        --- iside = 5, 6:  lower and upper plane

            iparea(2,1) = ni
            iparea(2,2) = nj
            if ( iside.eq.6 ) then

c           --- iside = 6:  upper plane

               iparea(1,3) = nk
               iparea(2,3) = nk

            end if

         end if

         do k = -2, 2

           l = k - min(1,abs(k))*sign(1,k)
           m = k - 2*l

           if ( ndir.eq.1 ) then
             iconcto = mct ( nsign, m, l )
             iconctc = mct (     0, m, l )
             iconcti = mct (-nsign, m, l )
           else if ( ndir.eq.2 ) then
             iconcto = mct ( l, nsign, m )
             iconctc = mct ( l,     0, m )
             iconcti = mct ( l,-nsign, m )
           else
             iconcto = mct ( m, l, nsign )
             iconctc = mct ( m, l,     0 )
             iconcti = mct ( m, l,-nsign )
           end if

c          --- loop over cells

           do kp = iparea(1,3), iparea(2,3)
             do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                 xp(1) = ip
                 xp(2) = jp
                 xp(3) = kp
                 ipoint = ipibinf+2*(xp(tdir(1))+m)+
     +                    2*(xp(tdir(2))+l)*(2*nvirtual+2*nij+1)
                 btype = ibinfor(ipoint)
                 fact = matrix(ip,jp,kp,iconcto)
                 if ( btype.eq.diri ) then

c                --- Dirichlet boundary condition

                   matrix(ip,jp,kp,iconcto) = 0d0
                   matrix(ip,jp,kp,iconctc) =
     +                             matrix(ip,jp,kp,iconctc) - fact
                   rhside(ip,jp,kp) = rhside(ip,jp,kp) -
     +                                2d0*fact*bndcon(1,ipoint)
                 else if ( btype.eq.robb .or. btype.eq.0 ) then

c                --- Robbins boundary condition or no one at all

                   matrix(ip,jp,kp,iconcto) = 0d0
                   matrix(ip,jp,kp,iconctc) =
     +                   matrix(ip,jp,kp,iconctc)+2d0*fact
                   matrix(ip,jp,kp,iconcti) =
     +                   matrix(ip,jp,kp,iconcti)-fact
                 else if ( btype.eq.dtur .and. k.eq.0 ) then

c                --- Fill matrix elements with zero's, except main diagonal
c                    element which is filled with one, solut and rhside with
c                    bndcon when Dirichlet condition for turbulence is given

                   do i = 2, 19
                     matrix(ip,jp,kp,i) = 0d0
                   end do
                   matrix(ip,jp,kp,1) = 1d0
                   rhside(ip,jp,kp) = bndcon(1,ipoint)

                 end if

               end do
             end do
           end do

         end do

      end do

      end
