      subroutine ischtop ( lensfread, nblocks, nfaces, maxnsubf,
     +                     nsubfaces, subfaces, indorien )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 26-06-1996
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Interchange the block topology if necessary
c
c **********************************************************************
c
c                       KEYWORDS
c
c   read_mesh
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer lensfread, maxnsubf, nblocks, nfaces
      integer nsubfaces(1:nblocks, 1:nfaces), indorien(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     lensfread i    Number of integers to be read for each subface
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, iface, isubface, j, nbblock, ih
      logical change

c     change    If true the orientation of the block must be changed
c     i         General loop variable
c     iblock    Block sequence number
c     iface     Face number
c     ih        Help variable to store a constant temporarily
c     isubface  Subface number
c     j         General loop variable
c     nbblock   Sequence number of a neighbouring block
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Initialize nsubfaces and subfaces array to zeroes
c
c     For iblock = 1, nblocks
c       For iface = 1, nfaces
c         Read nsubfaces
c         For isubfaces = 1, nsubfaces
c           Read lensfread integers from itopol
c         End
c       End
c     End
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Check if sequence must be interchanged

      change = .false.
      do iblock = 1, nblocks

         if ( indorien(iblock).eq.-1 ) then
            change = .true.
            ih = nsubfaces(iblock,1)
            nsubfaces(iblock,1) = nsubfaces(iblock,3)
            nsubfaces(iblock,3) = ih
            ih = nsubfaces(iblock,2)
            nsubfaces(iblock,2) = nsubfaces(iblock,4)
            nsubfaces(iblock,4) = ih
            do i = 1, lensfread
               do j = 1, maxnsubf
                  ih = subfaces(iblock,1,j,i)
                  subfaces(iblock,1,j,i) = subfaces(iblock,3,j,i)
                  subfaces(iblock,3,j,i) = ih
                  ih = subfaces(iblock,2,j,i)
                  subfaces(iblock,2,j,i) = subfaces(iblock,4,j,i)
                  subfaces(iblock,4,j,i) = ih
               end do
            end do
         end if
      end do

      if ( change .and. nblocks.gt.1 ) then

c     --- Sequence has been interchanged
c         Correct reference to neighbour blocks

         do iblock = 1, nblocks
            do iface = 1, nfaces
               do isubface = 1, nsubfaces(iblock,iface)
                  nbblock = subfaces(iblock,iface,isubface,5)
                  if ( indorien(nbblock).eq.-1 ) then
                     if ( subfaces(iblock,iface,isubface,6).le.2 ) then
                        subfaces(iblock,iface,isubface,6) =
     +                     subfaces(iblock,iface,isubface,6)+2
                     else
                        subfaces(iblock,iface,isubface,6) =
     +                     subfaces(iblock,iface,isubface,6)-2
                     end if
                  end if
               end do
            end do
         end do

      end if

      end
