      subroutine isfili(nblocks, isiinput, iscoloc, iscoor)
c=====================================================================
c
c        programmer   Erik Brakkee
c        version 1.0    date 23-05-94
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Fill the inner regions of the coordinate arrays (extended with row of
c     virtual unknowns) with values from a coordinate array
c     without virtual points.
c***********************************************************************
c
c                  KEYWORDS
c
c   mapping_array
c   topology
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks
      integer iscoloc(1:nblocks)
      integer iscoor(1:nblocks)
      integer isiinput(1:nblocks)

c     iscoloc   i    array of pointers to coor arrays without virtual points
c     iscoor    i    array of pointers to local coor arrays
c                    (with virtual points)
c     isiinput  i    array of pointers to subdomain iinput arrays
c     nblocks   i    number of blocks
c***********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer ipcoloc, ipcoor, ipiinput, ndim, ni, nj, nk,
     +        nvirtual

c     ipcoloc   address of a local coordinate array in buffer
c     ipcoor    address of a coordinate array in buffer
c     ipiinput  address of an iinput array in ibuffr
c     ndim      dimension of the problem
c     ni        i-dimension of a block
c     nj        j-dimension of a block
c     nk        k-dimension of a block
c     nvirtual  number of virtual rows of coordinates
c**********************************************************************
c
c                   I/O
c
c      none
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDPGETP  Get the pointer corresponding to a double precision array
c     ISFILI02  Fill coordinates in the inner region for a certain block (2-D)
c     ISFILI03  Fill coordinates in the inner region for a certain block (3-D)
c     ISINGETP  Get the pointer corresponding to an integer array
c**********************************************************************
c
c                   ERROR MESSAGES
c
c        -
c***********************************************************************
c
c                   PSEUDO CODE
c
c     For iblockrf = 1, nblocks
c        Fill inner region of the local coordinate array of block iblockrf
c        using routine isfili02
c     End
c=======================================================================
c
c  ---  first compute nvirtual and ndim

      call isactive(isiinput(1))

      ipiinput     = isingetp(isiinput(1))
      nvirtual     = ibuffr(ipiinput + 54 - 1)
      ndim         = ibuffr(ipiinput + 57 - 1)

      do iblockrf = 1, nblocks

         call isactive(isiinput(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))
         ni = ibuffr(ipiinput + 51 - 1)
         nj = ibuffr(ipiinput + 52 - 1)
         nk = ibuffr(ipiinput + 53 - 1)

         call isactive(iscoloc(iblockrf))
         call isactive(iscoor(iblockrf))

         ipcoloc  = isdpgetp(iscoloc(iblockrf))
         ipcoor   = isdpgetp(iscoor(iblockrf))

         if ( ndim .eq. 2 ) then

            call isfili02(ni, nj, ndim, nvirtual,
     +                 buffer(ipcoloc), buffer(ipcoor))

         else

            call isfili03(ni, nj, nk, ndim, nvirtual,
     +                 buffer(ipcoloc), buffer(ipcoor))

         endif

      enddo

      end
