      subroutine isreadbc ( iinbc, rinbc, maxlenin, maxlenrl, leniinbc,
     +                      lenrinbc, ndim, ntrans, ni, nj, nk, dnumbr,
     +                      iinput, singular, iindsc )
c ======================================================================
c
c        programmer     Guus Segal
c        version 4.8    date 26-11-1997 Guus: New numbering of unknowns with
c                                       respect to boundary conditions
c        version 4.7    date 24-11-1997 Guus: Compute singular also in the
c                                       decoupled case
c        version 4.6    date 20-11-1997 Kees K.: periodic changed
c        version 4.5    date 18-09-1997 Guus: extension because of decoupled
c                                       velocity
c        version 4.4    date 09-04-1996 Kees K.: space in iinbc and rinbc
c                                       for pressure bound. cond.
c        version 4.3    date 20-03-1996 Guus: no message in case of btype = 0
c        version 4.2    date 12-01-1996 Marcel: correction periodicity for
c                                               transport if not momentum
c        version 4.1    date 06-12-1995 Guus: Reorder sequence faces 3 and 4
c        version 4.0    date 02-10-1995 Guus: extra error message btype,
c                                             extra parameter iindsc
c        version 3.0    date 04-09-1995 Guus: extra parameter singular
c        version 2.1    date 05-04-1995 Guus: correction iinput
c        version 2.0    date 03-04-1995 Guus: extra parameter iinput with
c                                       respect to periodical boundary cond.
c        version 1.2    date 31-01-1995 Marcel small correction
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the boundary conditions from the intermediate
c    file and store in the arrays iinbc and rinbc for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxlenin, maxlenrl, leniinbc, iinbc(maxlenin), lenrinbc,
     +        ndim, ntrans, ni, nj, nk, iinput(*), iindsc(20,ntrans+1)
      double precision rinbc(maxlenrl), dnumbr(*)
      logical singular

c     dnumbr         Work array to store a series numbers that have been read
c     iinbc     o    reserved
c     iindsc    i    reserved
c     iinput   i/o   reserved
c     leniinbc  o    Computed length of array iinbc
c     lenrinbc  o    Computed length of array rinbc
c     maxlenin  i    Maximum length of integer array
c     maxlenrl  i    Maximum length of real array
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntrans    i    Number of transport equations plus number of turbulence
c                    equations
c     rinbc     o    reserved
c     singular  o    If true the pressure matrix is singular since at all
c                    boundaries the normal velocity has been prescribed
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, i1, i2, inum, isegment, j1, j2, k, max1, max2, nseg,
     +        iperiod, btype, ndegfd
      logical coupled

c     btype     Type of boundary condition
c     coupled   Logical which is true if the momentum equations
c               are coupled
c     i         General loop variable
c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     inum      Indicates the number of numbers that have been read
c     iperiod   Parameter indicating if periodical boundary conditions
c               are present per boundary
c               iperiod is a 3 digit number with the sequence 321
c               Each digit refers to 1 direction
c               The digits have the following meaning
c               0:  no periodical boundary conditions
c               1:  symmetrical periodical boundary conditions
c               2:  anti-symmetrical periodical boundary conditions
c     isegment  Sequence number of segment
c     j1        Lower index number along second direction of a face
c     j2        Upper index number along second direction of a face
c     k         General loop variable
c     max1      Maximum number of cells in "1"-direction
c     max2      Maximum number of cells in "2"-direction
c     ndegfd    Number of degrees of freedom
c     nseg      Number of segments the boundary is divided in
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     Array iinbc (length complex, see Basis design ISNaS incompressible flow
c                  solver)
c     Array rinbc (length complex, see Basis design ISNaS incompressible flow
c                  solver)
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     513   numseg < 1 or numseg > ni
c     514   i1 < 1 or i1 > ni
c     515   i2 < i1 or i2 > ni
c     516   Array IINBC too small
c     517   Array RINBC too small
c     518   j1 < 1 or j1 > ni
c     519   j2 < j1 or j2 > ni
c     601   btype out of range
c **********************************************************************
c
c                       PSEUDO CODE
c
c     leniinbc := 2*ndim+1
c     lenrinbc := 0
c     iperiod  := 0
c     If ( ndim=2 ) then
c        for i := 1 (1) 2*ndim do
c            iinbc(i) := leniinbc
c            Read and check number of segments (nseg)
c            leniinbc := leniinbc+1
c            iinbc(leniinbc) := nseg
c            for isegment := 1 (1) nseg do
c               Read and check begin and end segment (i1,i2)
c               Put in iinbc
c               leniinbc := leniinbc+2
c               Read and check itype and put in iinbc
c               leniinbc := leniinbc+3*(ntrans+2)
c               lenrinbc := lenrinbc+2*(ntrans+2)
c               if ( i=1 and isegment=1 and itype=96 or 98 ) then
c                  iperiod := iperiod + 1
c               else if ( i=1 and isegment=1 and itype=95 or 97 ) then
c                  iperiod := iperiod + 2
c               if ( i=3 and isegment=1 and itype=96 or 98 ) then
c                  iperiod := iperiod + 10
c               else if ( i=3 and isegment=1 and itype=95 or 97 ) then
c                  iperiod := iperiod + 20
c     else ( ndim=3 )
c        for i := 1 (1) 2*ndim do
c            iinbc(i) := leniinbc
c            Read and check number of segments (nseg)
c            leniinbc := leniinbc+1
c            iinbc(leniinbc) := nseg
c            for isegment := 1 (1) nseg do
c               Read and check begin and end segment (i1,j1,i2,j2)
c               Put in iinbc
c               leniinbc := leniinbc+4
c               Read and check itype and put in iinbc
c               leniinbc := leniinbc+4*(ntrans+2)
c               lenrinbc := lenrinbc+3*(ntrans+2)
c               if ( i=1 and isegment=1 and itype=96 or 98 ) then
c                  iperiod := iperiod + 1
c               else if ( i=1 and isegment=1 and itype=95 or 97 ) then
c                  iperiod := iperiod + 2
c               if ( i=3 and isegment=1 and itype=96 or 98 ) then
c                  iperiod := iperiod + 10
c               else if ( i=3 and isegment=1 and itype=95 or 97 ) then
c                  iperiod := iperiod + 20
c               if ( i=5 and isegment=1 and itype=96 or 98 ) then
c                  iperiod := iperiod + 100
c               else if ( i=5 and isegment=1 and itype=95 or 97 ) then
c                  iperiod := iperiod + 200
c     Read input for rinbc
c     Check lenrinbc and leniinbc
c     iinput(56) := iperiod
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isreadbc' )

      leniinbc = 2*ndim
      lenrinbc = 0
      iperiod  = 0
      singular = .true.
      coupled = iinput(76).eq.0
      if ( ndim.eq.2 ) then

c     --- ndim = 2

         if ( coupled ) then
            ndegfd = ntrans+2
         else
            ndegfd = ntrans+3
         end if
         do i = 1, 2*ndim

            iinbc(i) = leniinbc + 1
            if ( i.lt.3 ) then

c           --- Boundaries 1 or 2

               max1 = nj

            else

c           --- Boundaries 3 or 4

               max1 = ni

            end if

c           --- Read and check number of segments (nseg)

            call isreadrl ( dnumbr, 1, inum )
            nseg = nint(dnumbr(1))
            iinbc(leniinbc+1) = nseg
            leniinbc = leniinbc+1
            if ( nseg.lt.1 .or. nseg.gt.max1 ) then
               call iserrint ( leniinbc, 1 )
               call iserrint ( i, 2 )
               call iserrint ( nseg, 3 )
               call iserrint ( max1, 4 )
               call iserrsub ( 513, 4, 0, 0 )
            end if
            do isegment = 1, nseg

c           --- Read and check begin and end segment (i1,i2)

               call isreadrl ( dnumbr, 2, inum )
               i1 = nint(dnumbr(1))
               i2 = nint(dnumbr(2))
               if ( i1.lt.1 .or. i1.gt.max1 ) then
                  call iserrint ( i, 1 )
                  call iserrint ( leniinbc+1, 2 )
                  call iserrint ( i1, 3 )
                  call iserrint ( max1, 4 )
                  call iserrint ( isegment, 5 )
                  call iserrsub ( 514, 5, 0, 0 )
               end if
               if ( i2.lt.i1 .or. i2.gt.max1 ) then
                  call iserrint ( i, 1 )
                  call iserrint ( leniinbc+2, 2 )
                  call iserrint ( i2, 3 )
                  call iserrint ( max1, 4 )
                  call iserrint ( i1, 5 )
                  call iserrint ( isegment, 6 )
                  call iserrsub ( 515, 6, 0, 0 )
               end if

c              --- Put in iinbc

               iinbc(leniinbc+1) = i1
               iinbc(leniinbc+2) = i2
               leniinbc = leniinbc+2

c              --- Read itype and put in iinbc

               call isreadrl ( dnumbr, 3*ndegfd, inum )
               do k = 1, 3*ndegfd
                  btype = nint(dnumbr(k))
                  iinbc(leniinbc+k) = nint(dnumbr(k))
               end do

               do k = 1, 3*ndegfd, 3
                  btype = iinbc(leniinbc+k)
                  if ( coupled .and. k.eq.1 .and. iindsc(1,k).ne.0 )
     +               then

c                 --- k = 1, momentum

                     if ( btype.ne.0 .and. (btype.lt.4 .or.
     +                    btype.eq.8 .or.
     +                    btype.eq.11 .or. btype.gt.99 .or.
     +                    btype.gt.13 .and. btype.lt.95) ) then

c                    --- btype is incorrect

                        call iserrint ( btype, 1 )
                        call iserrint ( k, 2 )
                        call iserrint ( i, 3 )
                        call iserrint ( isegment, 4 )
                        call iserrsub ( 601, 4, 0, 0 )

                     end if

                  else if ( .not. coupled .and. k.eq.1 .and.
     +                      iindsc(1,k).ne.0 .or.
     +                      k.gt.2 .and. iindsc(1,k).ne.0 ) then

c                 --- k > 2, transport
c                     or decoupled approach

                     if ( btype.lt.0 .or. btype.eq.7 ) then

c                    --- btype is incorrect

                        call iserrint ( btype, 1 )
                        call iserrint ( (k-1)/3+1, 2 )
                        call iserrint ( i, 3 )
                        call iserrint ( isegment, 4 )
                        call iserrsub ( 601, 4, 0, 0 )

                     end if

                  end if

               end do
               if ( iindsc(1,1).ne.0 ) then
                 btype = iinbc(leniinbc+1)
               else
                 btype = iinbc(leniinbc+1+2*3)
               end if
               if ( coupled ) then

c              --- Coupled approach

                  singular = singular .and.
     +                       ( btype.eq.10 .or. btype.eq.5 .or.
     +                         btype.eq. 9 .or. btype.eq.4 .or.
     +                         btype.eq.12 )
               else

c              --- Decoupled approach

                  if ( k.eq.1 .and. i.le.2 ) then
                     singular = singular .and. btype.eq.1
                  else if ( k.eq.4 .and. i.gt.2 ) then
                     singular = singular .and. btype.eq.1
                  end if

               end if

               if ( btype.eq.96 .or. btype.eq.98 ) then
                  iperiod = 1
               end if

               leniinbc = leniinbc+3*ndegfd
               lenrinbc = lenrinbc+2*ndegfd
            end do
            if ( leniinbc.gt.maxlenin ) then
               call iserrint ( leniinbc, 1 )
               call iserrint ( maxlenin, 2 )
               call iserrsub ( 516, 2, 0, 0 )
            end if
            if ( lenrinbc.gt.maxlenrl ) then
               call iserrint ( lenrinbc, 1 )
               call iserrint ( maxlenrl, 2 )
               call iserrsub ( 517, 2, 0, 0 )
            end if
         end do

      else

c     --- ndim = 3

         if ( coupled ) then
            ndegfd = ntrans+2
         else
            ndegfd = ntrans+4
         end if
         do i = 1, 2*ndim

            iinbc(i) = leniinbc + 1
            if ( i.ge.5 ) then

c           --- Boundaries 5 or 6

               max1 = ni
               max2 = nj

            else if ( i.ge.3 ) then

c           --- Boundaries 3 or 4

               max1 = nk
               max2 = ni

            else

c           --- Boundaries 1 or 2

               max1 = nj
               max2 = nk

            end if

c           --- Read and check number of segments (nseg)

            call isreadrl ( dnumbr, 1, inum )
            nseg = nint(dnumbr(1))
            leniinbc = leniinbc+1
            iinbc(leniinbc) = nseg
            if ( nseg.gt.min(max1,max2) ) then
               call iserrint ( i, 1 )
               call iserrint ( leniinbc, 2 )
               call iserrint ( nseg, 3 )
               call iserrint ( min(max1,max2), 4 )
               call iserrsub ( 513, 4, 0, 0 )
            end if
            do isegment = 1, nseg

c           --- Read and check begin and end segment (i1,j1,i2,j2)

               call isreadrl ( dnumbr, 4, inum )
               i1 = nint(dnumbr(1))
               j1 = nint(dnumbr(2))
               i2 = nint(dnumbr(3))
               j2 = nint(dnumbr(4))
               if ( i1.lt.1 .or. i1.gt.max1 ) then
                  call iserrint ( i, 1 )
                  call iserrint ( leniinbc+1, 2 )
                  call iserrint ( i1, 3 )
                  call iserrint ( max1, 4 )
                  call iserrint ( isegment, 5 )
                  call iserrsub ( 514, 5, 0, 0 )
               end if
               if ( j1.lt.1 .or. j1.gt.max2 ) then
                  call iserrint ( i, 1 )
                  call iserrint ( leniinbc+1, 2 )
                  call iserrint ( j1, 3 )
                  call iserrint ( max2, 4 )
                  call iserrint ( isegment, 5 )
                  call iserrsub ( 518, 5, 0, 0 )
               end if
               if ( i2.lt.i1 .or. i2.gt.max1 ) then
                  call iserrint ( i, 1 )
                  call iserrint ( leniinbc+2, 2 )
                  call iserrint ( i2, 3 )
                  call iserrint ( max1, 4 )
                  call iserrint ( i1, 5 )
                  call iserrint ( isegment, 6 )
                  call iserrsub ( 515, 6, 0, 0 )
               end if
               if ( j2.lt.j1 .or. j2.gt.max2 ) then
                  call iserrint ( i, 1 )
                  call iserrint ( leniinbc+2, 2 )
                  call iserrint ( j2, 3 )
                  call iserrint ( max2, 4 )
                  call iserrint ( j1, 5 )
                  call iserrint ( isegment, 6 )
                  call iserrsub ( 519, 6, 0, 0 )
               end if

c              --- Put in iinbc

               iinbc(leniinbc+1) = i1
               iinbc(leniinbc+2) = j1
               iinbc(leniinbc+3) = i2
               iinbc(leniinbc+4) = j2
               leniinbc = leniinbc+4

c              --- Read itype and put in iinbc

               call isreadrl ( dnumbr, 4*ndegfd, inum )
               do k = 1, 4*ndegfd
                  iinbc(leniinbc+k) = nint(dnumbr(k))
               end do
               do k = 1, 4*ndegfd, 4
                  btype = iinbc(leniinbc+k)
                  if ( coupled .and. k.eq.1 .and. iindsc(1,k).ne.0 )
     +               then

c                 --- k = 1, momentum

                     if ( btype.ne.0 .and. (btype.lt.4 .or.
     +                    btype.eq.8 .or.
     +                    btype.eq.11 .or. btype.gt.99 .or.
     +                    btype.gt.13 .and. btype.lt.95) ) then

c                    --- btype is incorrect

                        call iserrint ( btype, 1 )
                        call iserrint ( k, 2 )
                        call iserrint ( i, 3 )
                        call iserrint ( isegment, 4 )
                        call iserrsub ( 601, 4, 0, 0 )

                     end if

                  else if ( .not. coupled .and. k.eq.1 .and.
     +                      iindsc(1,k).ne.0 .or.
     +                      k.gt.2 .and. iindsc(1,k).ne.0 ) then

c                 --- k > 2, transport
c                     or decoupled approach

                     if ( btype.lt.0 .or. btype.eq.7 ) then

c                    --- btype is incorrect

                        call iserrint ( btype, 1 )
                        call iserrint ( (k-4)/4+1, 2 )
                        call iserrint ( i, 3 )
                        call iserrint ( isegment, 4 )
                        call iserrsub ( 601, 4, 0, 0 )

                     end if

                  end if
               end do
               if ( iindsc(1,1).ne.0 ) then
                 btype = iinbc(leniinbc+1)
               else
                 btype = iinbc(leniinbc+1+2*4)
               end if
               singular = singular .and.
     +                    ( btype.eq.10 .or. btype.eq.5 .or.
     +                      btype.eq. 9 .or. btype.eq.4 .or.
     +                      btype.eq.12 )
               if ( i.eq.1 .and. isegment.eq.1 .and.
     +              (btype.eq.96 .or. btype.eq.98) ) then
                  iperiod = 1
               else if ( i.eq.1 .and. isegment.eq.1 .and.
     +              (btype.eq.95 .or. btype.eq.97) ) then
                  iperiod = 2
               end if
               if ( i.eq.3 .and. isegment.eq.1 .and.
     +              (btype.eq.96 .or. btype.eq.98) ) then
                  iperiod = iperiod + 10
               else if ( i.eq.3 .and. isegment.eq.1 .and.
     +              (btype.eq.95 .or. btype.eq.97) ) then
                  iperiod = iperiod + 20
               end if
               if ( i.eq.5 .and. isegment.eq.1 .and.
     +              (btype.eq.96 .or. btype.eq.98) ) then
                  iperiod = iperiod + 100
               else if ( i.eq.5 .and. isegment.eq.1 .and.
     +              (btype.eq.95 .or. btype.eq.97) ) then
                  iperiod = iperiod + 200
               end if
               leniinbc = leniinbc+4*ndegfd
               lenrinbc = lenrinbc+3*ndegfd
            end do
            if ( leniinbc.gt.maxlenin ) then
               call iserrint ( leniinbc, 1 )
               call iserrint ( maxlenin, 2 )
               call iserrsub ( 516, 2, 0, 0 )
            end if
            if ( lenrinbc.gt.maxlenrl ) then
               call iserrint ( lenrinbc, 1 )
               call iserrint ( maxlenrl, 2 )
               call iserrsub ( 517, 2, 0, 0 )
            end if
         end do

      end if

c     --- Read rinbc

      call isreadrl ( rinbc, lenrinbc, inum )
      iinput(56) = iperiod

      call iserclos ( 'isreadbc' )
      end
