      subroutine isreadcv ( iincav, rincav, dnumbr, mcav )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date 03-03-1999 Duncan: added new inputs
c        version 1.3    date 29-07-1998 Extension iincav(2)
c        version 1.2    date 31-04-1998 Extension iincav(1)
c        version 1.1    date 05-03-1998 Duncan: corrected min/max values
c        version 1.0    date 07-09-1997
c
c
c   copyright (c) 1997-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the cavity model from the
c    intermediate file and store in arrays iincav and rincav for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    cavity
c    compressible
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iincav(*), mcav
      double precision dnumbr(*), rincav(*)

c     dnumbr         Work array to store a series numbers that have been read
c     iincav    o    reserved
c     mcav      i    If 1 a cavity model is used, if 0 not
c     rincav    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inum
      double precision param

c     inum      Indicates the number of numbers that have been read
c     param     Arbitrary parameter read
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     100 reals with respect to iincav
c     100 reals with respect to rincav
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     512   rincav(i) in input file incorrrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isreadcv' )

c     --- Read iincav

      call isreadrl ( dnumbr, 100, inum )
      iincav(1) = nint(dnumbr(1))
      iincav(2) = nint(dnumbr(2))

c     --- Read rincav

      call isreadrl ( rincav, 10, inum )
      if ( mcav.gt.0 ) then

c     --- Compressible flow, test parameters

         param = rincav(1)
         if ( param.lt.1d-6 .or. param.gt.1d6 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'c_liquid', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincav(2)
         if ( param.lt.1d-6 .or. param.gt.1d6 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'c_vapour', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincav(3)
         if ( param.lt.1d-6 .or. param.gt.1d6 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'lowertransitpressure', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincav(4)
         if ( param.lt.1d-6 .or. param.gt.1d6 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'uppertransitpressure', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincav(5)
         if ( param.lt.1d-1 .or. param.gt.1d3 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'deltarho', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if

      end if

      call isreadrl ( dnumbr, 90, inum )

      call iserclos ( 'isreadcv' )
      end
