      subroutine isreadsl ( iinsol, rinsol, ntrans, dnumbr, ndim,
     +                      icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.4    date 26-05-1997 Extra possiblities iinsol/rinsol
c        version 1.3    date 27-01-1997 Extra possiblities iinsol/rinsol
c        version 1.2    date 12-07-1995 Allow default for alpha
c        version 1.1    date 31-01-1995 Erik: check for correct
c                                       subdomain solution extended with
c                                       subdomain_solution = ilu
c        version 1.0    date 19-05-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the linear solver from the intermediate
c    file and store in the arrays iinsol and rinsol for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    linear_solver
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ntrans, iinsol(25,*), icoupled, ndim
      double precision dnumbr(*), rinsol(25,*)

c     dnumbr         Work array to store a series numbers that have been read
c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     iinsol    o    reserved
c     ndim      i    Dimension of space (2 or 3)
c     ntrans    i    Number of transport equations plus number of turbulence
c                    equations
c     rinsol    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, idegpol, ifile, inum, iortho, ioutp, ipreco, irestart,
     +      istart, istop, j, maxiter, methodsol, numritz
      double precision absaccur, alpha, divaccur, conaccur

c     absaccur  Absolute accuracy with respect to residual
c     alpha     Relaxation parameter alpha
c     conaccur  Relative accuracy in common sense for GMRES only
c     divaccur  Truncation criterion for pressure equation
c     i         General loop variable
c     idegpol   Degree of polynomial in polynomial preconditioning
c     ifile     File reference number
c     inum      Indicates the number of numbers that have been read
c     iortho    Maximal number of vectors used in the orthogonalization process
c     ioutp     Defines amount of output
c     ipreco    Type of preconditioner
c     irestart  Indicates after how many steps the method restarts
c     istart    Indicates how the iteration process in started
c     istop     Indicates which actions should be done in case of termination
c     j         General loop variable
c     maxiter   Maximum number of iterations
c     methodsol Type of linear solver
c     numritz   Number of Ritz values to be computed
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     For i := 1(1) ntrans+2
c        iinsol(j), j = 1, 25
c        rinsol(j), j = 1, 25
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     504   iinsol(i) in input file incorrrect
c     505   rinsol(i) in input file incorrrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For i := 1 (1) ntrans+1
c        Read iinsol
c        Check iinsol
c        Read rinsol
c        Check rinsol
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isreadsl' )

      do i = 1, ntrans+2+icoupled*(ndim-1)

c     --- Read iinsol

         call isreadrl ( dnumbr, 25, inum )
         do j = 1, 25
            iinsol(j,i) = nint(dnumbr(j))
         end do
         ipreco = iinsol(2,i)
         if ( ipreco.lt.-6 .or. ipreco.gt.6 ) then
            call iserrint ( 2, 1 )
            call iserrint ( i, 2 )
            call iserrint ( ipreco, 3 )
            call iserrchr ( 'type of preconditioner', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         ioutp = iinsol(3,i)
         if ( ioutp.lt.-1 .or. ioutp.gt.2 ) then
            call iserrint ( 3, 1 )
            call iserrint ( i, 2 )
            call iserrint ( ioutp, 3 )
            call iserrchr ( 'amount of output', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         istart = iinsol(4,i)
         if ( istart.lt.0 .or. istart.gt.3 ) then
            call iserrint ( 4, 1 )
            call iserrint ( i, 2 )
            call iserrint ( istart, 3 )
            call iserrchr ( 'start of iteration process', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         istop = iinsol(5,i)
         if ( istop.lt.0 .or. istop.gt.1 ) then
            call iserrint ( 5, 1 )
            call iserrint ( i, 2 )
            call iserrint ( istop, 3 )
            call iserrchr ( 'stop if error', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         maxiter = iinsol(6,i)
         if ( maxiter.lt.1 .or. maxiter.gt.100000 ) then
            call iserrint ( 6, 1 )
            call iserrint ( i, 2 )
            call iserrint ( maxiter, 3 )
            call iserrchr ( 'maxiter', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         methodsol = iinsol(7,i)
         if ( methodsol.lt.0 .or. methodsol.gt.6 ) then
            call iserrint ( 7, 1 )
            call iserrint ( i, 2 )
            call iserrint ( methodsol, 3 )
            call iserrchr ( 'methodsol', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         ifile = iinsol(8,i)
         if ( ifile.lt.0 .or. ifile.gt.99 ) then
            call iserrint ( 8, 1 )
            call iserrint ( i, 2 )
            call iserrint ( ifile, 3 )
            call iserrchr ( 'ifile', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         ifile = iinsol(9,i)
         if ( ifile.lt.0 .or. ifile.gt.30 ) then
            call iserrint ( 9, 1 )
            call iserrint ( i, 2 )
            call iserrint ( ifile, 3 )
            call iserrchr ( 'max_level', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         ifile = iinsol(10,i)
         if ( ifile.lt.0 .or. ifile.gt.10 ) then
            call iserrint ( 10, 1 )
            call iserrint ( i, 2 )
            call iserrint ( ifile, 3 )
            call iserrchr ( 'smooth_steps', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         irestart = iinsol(11,i)
         if ( irestart.lt.1 .or. irestart.gt.10000 ) then
            call iserrint ( 11, 1 )
            call iserrint ( i, 2 )
            call iserrint ( irestart, 3 )
            call iserrchr ( 'irestart', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         iortho = iinsol(12,i)
         if ( iortho.lt.1 .or. iortho.gt.10000 ) then
            call iserrint ( 12, 1 )
            call iserrint ( i, 2 )
            call iserrint ( iortho, 3 )
            call iserrchr ( 'iortho', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         idegpol = iinsol(13,i)
         if ( idegpol.lt.0 .or. idegpol.gt.100 ) then
            call iserrint ( 13, 1 )
            call iserrint ( i, 2 )
            call iserrint ( idegpol, 3 )
            call iserrchr ( 'idegpol', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         numritz = iinsol(14,i)
         if ( numritz.lt.0 .or. numritz.gt.1000 ) then
            call iserrint ( 14, 1 )
            call iserrint ( i, 2 )
            call iserrint ( numritz, 3 )
            call iserrchr ( 'numritz', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         numritz = iinsol(23,i)
         if ( numritz.lt.0 .or. numritz.gt.1 ) then
            call iserrint ( 23, 1 )
            call iserrint ( i, 2 )
            call iserrint ( numritz, 3 )
            call iserrchr ( 'neumayer', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if
         numritz = iinsol(24,i)
         if ( numritz.lt.1 .or. numritz.gt.16 ) then
            call iserrint ( 24, 1 )
            call iserrint ( i, 2 )
            call iserrint ( numritz, 3 )
            call iserrchr ( 'cg_inner', 1 )
            call iserrsub ( 504, 3, 0, 1 )
         end if

c        --- Read rinsol

         call isreadrl ( rinsol(1,i), 25, inum )
         absaccur = rinsol(1,i)
         if ( absaccur.lt.0d0 .or. absaccur.gt.1d2 ) then
            call iserrint ( 2, 1 )
            call iserrint ( i, 2 )
            call iserreal ( absaccur, 1 )
            call iserrchr ( 'abs accuracy', 1 )
            call iserrsub ( 505, 2, 1, 1 )
         end if
         conaccur = rinsol(5,i)
         if ( conaccur.lt.0d0 .or. conaccur.gt.1d-1 ) then
            call iserrint ( 5, 1 )
            call iserrint ( i, 2 )
            call iserreal ( conaccur, 1 )
            call iserrchr ( 'conaccuracy', 1 )
            call iserrsub ( 505, 2, 1, 1 )
         end if
         divaccur = rinsol(6,i)
         if ( divaccur.lt.0d0 .or. divaccur.gt.1d-1 ) then
            call iserrint ( 6, 1 )
            call iserrint ( i, 2 )
            call iserreal ( divaccur, 1 )
            call iserrchr ( 'divaccuracy', 1 )
            call iserrsub ( 505, 2, 1, 1 )
         end if
         alpha = rinsol(7,i)
         if ( abs(alpha+1d0).gt.1d-10 .and.
     +        (alpha.lt.0d0 .or. alpha.gt.1d0) ) then
            call iserrint ( 7, 1 )
            call iserrint ( i, 2 )
            call iserreal ( alpha, 1 )
            call iserrchr ( 'alpha', 1 )
            call iserrsub ( 505, 2, 1, 1 )
         end if
         alpha = rinsol(8,i)
         if ( alpha.lt.0d0 .or. alpha.gt.1d0 ) then
            call iserrint ( 8, 1 )
            call iserrint ( i, 2 )
            call iserreal ( alpha, 1 )
            call iserrchr ( 'omega_bicgstab', 1 )
            call iserrsub ( 505, 2, 1, 1 )
         end if

      end do

      call iserclos ( 'isreadsl' )
      end
