      subroutine isreadtu ( iintur, rintur, dnumbr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.6    date 15-09-1997 Guus: extension rintur and iintur
c        version 1.5    date 10-09-1996 Marcel extension rintur
c        version 1.4    date 23-08-1996 Marcel extension iintur
c        version 1.3    date 11-07-1996 Marcel update error message
c        version 1.2    date 23-05-1996 Marcel Extension rintur
c        version 1.1    date 22-09-1994 Marcel Zijlema: some corrections
c                                                     and extension with
c                                                     Wilcox k-omega model
c        version 1.0    date 16-08-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the turbulence modelling from the intermediate
c    file and store in arrays iintur and rintur for one block
c    Warning: if arrays iintur and rintur are extended, update subroutine
c             isreadif
c **********************************************************************
c
c                       KEYWORDS
c
c    read_cards
c    turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iintur(10)
      double precision dnumbr(*), rintur(30)

c     dnumbr         Work array to store a series numbers that have been read
c     iintur    o    reserved
c     rintur    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inum, nturb, mturb, maniso, nturbcom, viscorr, lesmodel,
     +        nwindmx, i
      double precision a, e, erough, rkappa, sigmaeps, sigmak,
     +      sigmat, c1eps, c2eps, c3eps, cd, cmu, eta0, betarng, cs,
     +      alphakw, beta1kw, beta2kw, sigma1kw, sigma2kw, modprod,
     +      ctau1, ctau2, ctau3

c     a         Parameter A for turbulence modelling
c     alphakw   Parameter alpha for k-omega model
c     beta1kw   Parameter beta1 for k-omega model
c     beta2kw   Parameter beta2 for k-omega model
c     betarng   Parameter beta for RNG k-eps model
c     c1eps     Parameter c1_eps for standard or RNG k-eps model
c     c2eps     Parameter c2_eps for standard or RNG k-eps model
c     c3eps     Parameter c3_eps for extended k-eps model
c     cd        Parameter c_D for turbulence modelling
c     cmu       Parameter c_mu for standard or RNG k-eps model
c     cs        Parameter c_s for LES Smagorinsky model
c     ctau1     Parameter ctau1 for nonlinear two-equation model
c     ctau2     Parameter ctau2 for nonlinear two-equation model
c     ctau3     Parameter ctau3 for nonlinear two-equation model
c     e         Parameter E for turbulence modelling
c     erough    Parameter E_rough for turbulence modelling
c     eta0      Parameter eta_0 for RNG k-eps model
c     i         General loop variable
c     inum      Indicates the number of numbers that have been read
c     lesmodel  Type of LES model
c     maniso    Indicates type of nonlinear turbulence (anisotropic)
c               modelling
c     modprod   Parameter modprod for production of k in
c               k-eps/k-omega models
c     mturb     Indicates type of turbulence modelling
c     nturb     Number of extra transport equations for turbulence
c     nturbcom  Computed value of nturb due to mturb
c     nwindmx   Maximum number of steps in a time window
c     rkappa    Parameter kappa for turbulence modelling
c     sigma1kw  Parameter sigma1 for k-omega model
c     sigma2kw  Parameter sigma2 for k-omega model
c     sigmaeps  Parameter sigma_eps for standard or RNG k-eps model
c     sigmak    Parameter sigma_k for standard or RNG k-eps model
c     sigmat    Parameter sigma_T for turbulence modelling
c     viscorr   Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     100 reals with respect to iintur
c     100 reals with respect to rintur
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     502   Parameter in input file incorrrect
c     512   rintur(i) in input file incorrrect
c     523   nturb does not correspond to mturb
c     607   Viscous corrections should not be used for all turbulence models
c           except standard k-eps and k-omega models
c     608   In case of Rubinstein-Barton model RNG k-eps model should
c           be used
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isreadtu' )

c     --- Read iintur

      call isreadrl ( dnumbr, 100, inum )
      nturb    = nint(dnumbr(1))
      mturb    = nint(dnumbr(2))
      viscorr  = nint(dnumbr(3))
      maniso   = nint(dnumbr(4))
      lesmodel = nint(dnumbr(5))
      nwindmx  = nint(dnumbr(6))
      if ( mturb.lt.0 .or. mturb.gt.5 ) then
         call iserrint ( mturb, 1 )
         call iserrchr ( 'mturb', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( mturb.eq.0 .or. mturb.eq.4 ) then
         nturbcom = 0
      else
         nturbcom = 2
      end if
      if ( nturb.ne.nturbcom ) then
         call iserrint ( mturb, 1 )
         call iserrint ( nturb, 2 )
         call iserrint ( nturbcom, 3 )
         call iserrsub ( 523, 3, 0, 0 )
      end if
      if ( viscorr.lt.0 .or. viscorr.gt.1 ) then
         call iserrint ( viscorr, 1 )
         call iserrchr ( 'viscorr', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( lesmodel.lt.0 .or. lesmodel.gt.1 ) then
         call iserrint ( lesmodel, 1 )
         call iserrchr ( 'lesmodel', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( nwindmx.lt.0 .or. nwindmx.gt.500 ) then
         call iserrint ( nwindmx, 1 )
         call iserrchr ( 'nwindmx', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( mturb.ne.1 .and. mturb.ne.3 .and. mturb.ne.4 ) then
        if ( viscorr.ne.0 ) then
          call iserrint ( mturb, 1 )
          call iserrint ( viscorr, 2 )
          call iserrsub ( 607, 2, 0, 0 )
        end if
      end if
      if ( maniso.lt.0 .or. maniso.gt.4 ) then
         call iserrint ( maniso, 1 )
         call iserrchr ( 'maniso', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( maniso.eq.2 .and. mturb.ne.2 ) then
         call iserrint ( maniso, 1 )
         call iserrint ( mturb, 2 )
         call iserrsub ( 608, 2, 0, 0 )
      end if
      iintur(1) = nturb
      iintur(2) = mturb
      iintur(3) = viscorr
      iintur(4) = maniso
      iintur(5) = lesmodel
      iintur(6) = nwindmx
      do i = 7, 10
         iintur(i) = 0
      end do

c     --- Read rintur

      call isreadrl ( rintur, 30, inum )
      rkappa = rintur(1)
      if ( rkappa.lt.0.4d0 .or. rkappa.gt.0.42d0 ) then
         call iserrint ( 1, 1 )
         call iserreal ( rkappa, 1 )
         call iserrchr ( 'kappa', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      e = rintur(2)
      if ( e.lt.7.45d0 .or. e.gt.1d1 ) then
         call iserrint ( 2, 1 )
         call iserreal ( e, 1 )
         call iserrchr ( 'E', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      cmu = rintur(3)
      if ( cmu.lt.0.01d0 .or. cmu.gt.0.36d0 ) then
         call iserrint ( 3, 1 )
         call iserreal ( cmu, 1 )
         call iserrchr ( 'c_mu', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      sigmak = rintur(4)
      if ( sigmak.lt.0.7d0 .or. sigmak.gt.1.4d0 ) then
         call iserrint ( 4, 1 )
         call iserreal ( sigmak, 1 )
         call iserrchr ( 'sigma_k', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      sigmaeps = rintur(5)
      if ( sigmaeps.lt.0.7d0 .or. sigmaeps.gt.1.4d0 ) then
         call iserrint ( 5, 1 )
         call iserreal ( sigmaeps, 1 )
         call iserrchr ( 'sigma_eps', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      c1eps = rintur(6)
      if ( c1eps.lt.1.0d0 .or. c1eps.gt.1.55d0 ) then
         call iserrint ( 6, 1 )
         call iserreal ( c1eps, 1 )
         call iserrchr ( 'c1_eps', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      c2eps = rintur(7)
      if ( c2eps.lt.1.5d0 .or. c2eps.gt.2.0d0 ) then
         call iserrint ( 7, 1 )
         call iserreal ( c2eps, 1 )
         call iserrchr ( 'c2_eps', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      cd = rintur(8)
      if ( cd.lt.0.04d0 .or. cd.gt.0.15d0 ) then
         call iserrint ( 8, 1 )
         call iserreal ( cd, 1 )
         call iserrchr ( 'c_D', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      sigmat = rintur(9)
      if ( sigmat.lt.0.5d0 .or. sigmat.gt.4.0d0 ) then
         call iserrint ( 9, 1 )
         call iserreal ( sigmat, 1 )
         call iserrchr ( 'sigma_T', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      a = rintur(10)
      if ( a.lt.20d0 .or. a.gt.30d0 ) then
         call iserrint ( 10, 1 )
         call iserreal ( a, 1 )
         call iserrchr ( 'A', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      erough = rintur(11)
      if ( erough.lt.26d0 .or. erough.gt.37d0 ) then
         call iserrint ( 11, 1 )
         call iserreal ( erough, 1 )
         call iserrchr ( 'E_rough', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      eta0 = rintur(12)
      if ( eta0.lt.2d0 .or. eta0.gt.16d0 ) then
         call iserrint ( 12, 1 )
         call iserreal ( eta0, 1 )
         call iserrchr ( 'eta_0', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      betarng = rintur(13)
      if ( betarng.lt.0.01d0 .or. betarng.gt.0.015d0 ) then
         call iserrint ( 13, 1 )
         call iserreal ( betarng, 1 )
         call iserrchr ( 'Beta', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      cs = rintur(14)
      if ( cs.lt.0.1d0 .or. cs.gt.0.25d0 ) then
         call iserrint ( 14, 1 )
         call iserreal ( cs, 1 )
         call iserrchr ( 'c_s', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      alphakw = rintur(15)
      if ( alphakw.lt.0.1d0 .or. alphakw.gt.1.0d0 ) then
         call iserrint ( 15, 1 )
         call iserreal ( alphakw, 1 )
         call iserrchr ( 'alpha', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      beta1kw = rintur(16)
      if ( beta1kw.lt.0.04d0 .or. beta1kw.gt.0.15d0 ) then
         call iserrint ( 16, 1 )
         call iserreal ( beta1kw, 1 )
         call iserrchr ( 'beta1', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      beta2kw = rintur(17)
      if ( beta2kw.lt.0.01d0 .or. beta2kw.gt.0.1d0 ) then
         call iserrint ( 17, 1 )
         call iserreal ( beta2kw, 1 )
         call iserrchr ( 'beta2', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      sigma1kw = rintur(18)
      if ( sigma1kw.lt.0.5d0 .or. sigma1kw.gt.2.0d0 ) then
         call iserrint ( 18, 1 )
         call iserreal ( sigma1kw, 1 )
         call iserrchr ( 'sigma1', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      sigma2kw = rintur(19)
      if ( sigma2kw.lt.0.5d0 .or. sigma2kw.gt.2.0d0 ) then
         call iserrint ( 19, 1 )
         call iserreal ( sigma2kw, 1 )
         call iserrchr ( 'sigma2', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      modprod = rintur(20)
      if ( modprod.lt.0d0 .or. modprod.gt.1.0d0 ) then
         call iserrint ( 20, 1 )
         call iserreal ( modprod, 1 )
         call iserrchr ( 'mod_prod', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      c3eps = rintur(21)
      if ( c3eps.lt.0.0d0 .or. c3eps.gt.0.4d0 ) then
         call iserrint ( 21, 1 )
         call iserreal ( c3eps, 1 )
         call iserrchr ( 'c3_eps', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      ctau1 = rintur(22)
      if ( ctau1.lt.-5.0d0 .or. ctau1.gt.5.0d0 ) then
         call iserrint ( 22, 1 )
         call iserreal ( ctau1, 1 )
         call iserrchr ( 'ctau_1', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      ctau2 = rintur(23)
      if ( ctau2.lt.-5.0d0 .or. ctau2.gt.5.0d0 ) then
         call iserrint ( 23, 1 )
         call iserreal ( ctau2, 1 )
         call iserrchr ( 'ctau_2', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if
      ctau3 = rintur(24)
      if ( ctau3.lt.-5.0d0 .or. ctau3.gt.5.0d0 ) then
         call iserrint ( 24, 1 )
         call iserreal ( ctau3, 1 )
         call iserrchr ( 'ctau_3', 1 )
         call iserrsub ( 512, 1, 1, 1 )
      end if

      call isreadrl ( dnumbr, 70, inum )

      call iserclos ( 'isreadtu' )
      end
