      subroutine isblkmat ( nmbsol, rhsd, x, work, iequa, isol,
     +                      isiinput, isrinput, iscoor  , issolut ,
     +                      iscoefs , iseddy  , isinfbnd, isbndcon,
     +                      ismatrix, isgmat  , isrhside, isgeom  ,
     +                      isitopol, isdensup, iadmin, nblocks    )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.0    date 17-03-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Performs a matrix vector product for the multiblock algorithm with
c     accurate or inaccurate solution of the subdomains.
c **********************************************************************
c
c                       KEYWORDS
c
c     domain_decompostion
c     linear_solver
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nmbsol, iequa, iadmin(*), isbndcon(*), iscoor(*),
     +                 iseddy(*), isgmat(*), isiinput(*), isinfbnd(*),
     +                 isitopol, ismatrix(*), isrhside(*), isrinput(*),
     +                 issolut(*), iscoefs(*), isgeom(*), isdensup(*),
     +                 isol, nblocks
      double precision x(nmbsol), work(nmbsol), rhsd(nmbsol)

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  i    Array of sequence numbers (nblocks) of array densup
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isol      i    Accuracy of subdomain solution in relation with
c                    domain decomposition.
c                      0 = accurate enough
c                      1 = inaccurate with respect to multi-block accuracy
c                      2 = inaccurate using ilu approximation
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c     nmbsol    i    Length of the multi-block solution vector
c     rhsd      i    Vector containing the right-hand side vector of the
c                    system of equations.
c     work      i    Work array
c     x         i    The original vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISACCMA   Matrix vector product with accurate subdomain solution
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISGLOMA   Global matrix-vector product over all subdomains
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Depending on isol a matrix vector product is done with accurate
c     or inaccurate solution of the subdomain problems.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isblkmat')

      if ( isol.eq.0 ) then

         call isaccma ( nmbsol, rhsd, x, work, iequa, isiinput,
     +                  isrinput, iscoor, issolut, iscoefs, iseddy,
     +                  isinfbnd, isbndcon, ismatrix, isgmat, isrhside,
     +                  isgeom, isitopol, isdensup, iadmin )

      else

         call isgloma ( nblocks, iequa, isiinput, issolut, ismatrix,
     +                  isitopol, iadmin, x, work )

         end if

      call iserclos('isblkmat')
      end
