      function isddge02 ( iblock, iface, isubface, itrans, ilev, ni,
     +                    nj, ntraq1, ntraq2, nvirtual, nblocks, nfaces,
     +                    maxnsubf, idiscret, isolv, traq, subfaces,
     +                    smsg )
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 1.8    date 21-09-1999 Set nrow = nvirtual if idiscret=10
c        version 1.7    date 18-05-1998 Charles Bilinear discretization
c        version 1.6    date 14-10-1997 Kees K: isolv  added
c        version 1.5    date 20-01-1997 Kees K: idiscret added
c        version 1.4    date 15-02-1996 Guus: correction negative sign
c                                           coefficients
c        version 1.3    date 28-03-1995 Erik:  optimized copying
c        version 1.2    date 22-11-1994 Erik:  adapted for transport of
c                                       coefficients (changed solut to traq)
c        version 1.1    date 21-07-1994 Erik:  different block
c                                       orientations are taken into account
c        version 1.0    date 06-04-1994
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copies the unknowns of a subface of a block that are virtual to another
c     block to an intermediate representation of these unknowns.
c     The intermediate representation is a one dimensional
c     array which is equal to the double precision data in the multi-block
c     messages from node to host.
c     The intermediate representation smsg is filled using the subdomain
c     traq array.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, ilev, isddge02, isubface, itrans,
     +      maxnsubf, nblocks, ntraq1, nfaces, ni, nj, ntraq2,
     +      nvirtual, idiscret, isolv
      double precision smsg(*)
      double precision traq(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1:ntraq1, 1:ntraq2)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock    i    Block sequence number
c     idiscret  i    Type of discretization
c     iface     i    Face number
c     ilev      i    Time level of the multi-block transport of information
c     isddge02  o    Actual length of the submessage
c     isolv     i    Called by solver for periodic b.c. isolv=1 else isolv=0
c     isubface  i    Subface number
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntraq1    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  i    Number of rows of virtual cells
c     smsg      i    reserved
c     subfaces  i    reserved
c     traq      i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ibto, ifto, ihigh, ii, ilow, iorient,
     +      isfto, ismsg, iunk(2), iunktraq,
     +      jj, nsignmb, nunks, relblckor, tsignmb,
     +      irow, nrow
      double precision usign
      logical reverse

c     i         General loop variable
c     ibto      Target block
c     ifto      Target face
c     ihigh     Upper index of subface
c     ii        Loop variable
c     ilow      Lower index of subface
c     iorient   Relative orientation of subfaces, see itopol
c     irow      Loop variable for rows
c     isfto     Target subface
c     ismsg     Counter for smsg
c     iunk      Array iunk(1:nunks) containing the number of the unknown
c               to be put into smsg.
c               iunk = 0 -> normal flux at block interfaces
c               iunk = 1 -> tangential flux
c               iunk = 2 -> normal flux
c               iunk > 3 -> iunk = ndegfd
c               iunk < 0 -> degree of freedom -iunk
c     iunktraq  Number of the unknown iunk(i) in traq
c     jj        Loop variable
c     nrow      Number of rows
c     nsignmb   Nsignmb = 0 if the normal direction of the subface
c               coincides with that of its neighbouring block, otherwise
c               nsignmb = 1
c     nunks     See iunk
c     relblckor Relative block orientation. If the neighbouring
c               block has sqrt(g) with opposite sign to this block
c               then relblckor = 1 otherwise it is 0
c     reverse   Reverse the order of copying unknowns (true) or not (false)
c     tsignmb   If the tangential direction of the subface coincides with
c               that of its neighbouring block tsignmb = 0, otherwise
c               tsignmb = 1
c     usign     Sign to multiply the unknowns with when copying from traq
c               to the intermediate representation
c **********************************************************************
c
c                       I/O
c
c     None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     None
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     if ( itrans.eq.2 ) then
c        --- assume a solut array
c        iunk(1) = normal fluxes on the block interfaces
c        nunks = 1
c     else if ( itrans.eq.1 ) then
c        --- assume a solut array
c        iunk(1) = tangential velocity unknowns
c        iunk(2) = normal velocity unknowns
c        nunks = 2
c     else
c        --- assume solut for itrans > 0 and coefs for itrans < 0
c        iunk(1) = itrans
c        nunks = 1
c     end if
c
c     ismsg = 1
c     FOR unknowns to be transported
c     DO
c        determine index range [i1,i2] X [j1,j2]
c
c        determine order and sign (usign) of copying
c
c        FOR I = I1, I2, ISTEP
c        DO
c           FOR J = J1, J2, JSTEP
c           DO
c              smsg(ismsg) = usign*traq(I,J)
c              ismsg = ismsg + 1
c           DONE
c        DONE
c     DONE
c     RETURN ismsg - 1
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- initialize iunk

      if ( itrans.eq.2 ) then

c     --- assume solut, normal flux components on the boundary

         nunks = 1
         iunk(1) = 0

      else if ( itrans.eq.1 ) then

c     --- assume solut, tangential fluxes across the boundary,
c         followed by normal fluxes across the boundary

         nunks = 2
         iunk(1) = 1
         iunk(2) = 2

      else

c     --- assume solut for itrans > 0 and coefs for itrans < 0
c         degree of freedom = abs(itrans)

         nunks = 1
         iunk(1) = itrans

      end if

      ismsg = 1

      do i = 1, nunks

         nrow = 1

         iunktraq = abs(iunk(i))
         if ( iunk(i).le.2 .and. iunk(i).ge.0 ) then

            if ( iunk(i).eq.0 ) then
               iunktraq = 2
            else if ( iunk(i).eq.1 ) then
               iunktraq = 1
            else
               iunktraq = 2
            end if

            if ( iunktraq.eq.1 ) then

               if ( idiscret.eq.2 .or. idiscret.eq.4 ) nrow = 2

            end if

            if ( iface.eq.ileft .or. iface.eq.iright ) then
               iunktraq = 3 - iunktraq

               if ( iunktraq.eq.2 ) then

                  if ( idiscret.eq.2 .or. idiscret.eq.4 ) nrow = 2

               end if

            end if

         end if

c        --- In the special case that idiscret=10, we copy nvirtual cells

         if ( idiscret.eq.10 ) nrow = nvirtual

c        --- in case of periodic b.c. this routine will be called in
c            the linear solver, we have then a restricted solution vector

         if ( iunktraq.gt.2 .and. isolv.eq.1) then
            iunktraq = 1
         end if

c        --- put the unknown iunk(i) into smsg
c            determine the index range

         ilow = subfaces(iblock, iface, isubface, 1)
         ihigh = subfaces(iblock, iface, isubface, 2)

         if ( iunk(i).eq.1 ) then
            ihigh = ihigh + 1
         end if

c        --- relative orientation

         iorient    = subfaces(iblock,iface,isubface,8)
         tsignmb      = mod(iorient,2)
         nsignmb      = mod(iorient/2,2)
         relblckor = iorient/4
         ibto       = subfaces(iblock,iface,isubface,5)
         ifto       = subfaces(iblock,iface,isubface,6)
         isfto      = subfaces(iblock,iface,isubface,7)

         reverse = .false.
         usign   = 1d0

         if ( (iblock*nfaces + iface)*maxnsubf + isubface .ge.
     +        (ibto*nfaces + ifto)*maxnsubf + isfto ) then

c        --- the representation is determined by the neighbouring block

            if ( itrans.le.2 .and. relblckor .eq.1 ) usign = -usign

            if ( tsignmb.eq.1 ) then

               if ( iunk(i).eq.1 ) usign = -usign
               reverse = .true.

            end if

            if ( nsignmb.eq.1 ) then

               if (iunk(i).eq.2 .or. iunk(i).eq.0 ) usign = -usign

            end if

         end if

         do irow = 1, nrow

            if ( iface.eq.ileft ) then

               ii = 1
               if ( iunk(i).eq.2 ) ii = ii + 1

               ii = ii -1 + irow

               if ( reverse ) then

                  do jj = ilow, ihigh
                     smsg(ismsg+ihigh-jj) =
     +                  traq(ii,jj,iunktraq,ilev)
                  end do

               else

                  do jj = ilow, ihigh
                    smsg(ismsg+jj-ilow) =
     +                 traq(ii,jj,iunktraq,ilev)
                  end do

               end if

               if ( itrans.ge.0 .and. usign.lt.0d0 ) then

                  do ii = ismsg, ismsg + ihigh - ilow
                     smsg(ii) = -smsg(ii)
                  end do

               end if

               ismsg = ismsg + ihigh - ilow + 1

            else if ( iface.eq.iright ) then

               ii = ni
               if ( iunk(i).eq.0 ) ii = ii + 1

               ii = ii + 1 - irow

               if ( reverse ) then

                  do jj = ilow, ihigh
                     smsg(ismsg+ihigh-jj) =
     +                  traq(ii,jj,iunktraq,ilev)
                  end do

               else

                  do jj = ilow, ihigh
                    smsg(ismsg+jj-ilow) =
     +                 traq(ii,jj,iunktraq,ilev)
                  end do

               end if

               if ( itrans.ge.0 .and. usign.lt.0d0 ) then

                  do ii = ismsg, ismsg + ihigh - ilow
                     smsg(ii) = -smsg(ii)
                  end do

               end if

               ismsg = ismsg + ihigh - ilow + 1

            else if ( iface.eq.idown ) then

              jj = 1
               if ( iunk(i).eq.2 ) jj = jj + 1

               jj = jj - 1 + irow

               if ( reverse ) then

                  do ii = ilow, ihigh
                     smsg(ismsg + ihigh - ii) =
     +                  traq(ii,jj,iunktraq,ilev)
                  end do

               else

                  do ii = ilow, ihigh
                     smsg(ismsg + ii - ilow) =
     +                  traq(ii,jj,iunktraq,ilev)
                  end do

               end if

               if ( itrans.ge.0 .and. usign.lt.0d0 ) then

                  do ii = ismsg, ismsg + ihigh - ilow
                     smsg(ii) = -smsg(ii)
                  end do

               end if

               ismsg = ismsg + ihigh - ilow + 1

            else

               jj = nj
               if ( iunk(i).eq.0 ) jj = jj + 1

               jj = jj + 1 - irow

               if ( reverse ) then

                  do ii = ilow, ihigh
                     smsg(ismsg + ihigh - ii) =
     +                  traq(ii,jj,iunktraq,ilev)
                  end do

               else

                  do ii = ilow, ihigh
                     smsg(ismsg + ii - ilow) =
     +                  traq(ii,jj,iunktraq,ilev)
                  end do

               end if

               if ( itrans.ge.0 .and. usign.lt.0d0 ) then

                  do ii = ismsg, ismsg + ihigh - ilow
                     smsg(ii) = -smsg(ii)
                  end do

               end if

               ismsg = ismsg + ihigh - ilow + 1

            end if

         end do
      end do

      isddge02 = ismsg - 1

      end
