      function isddpu03 ( iblock, iface, isubface, itrans, ilev, ni, nj,
     +                    nk, ntraq1, ntraq2, nvirtual, nblocks, nfaces,
     +                    maxnsubf, traq, subfaces, smsg )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 27-02-1997
c
c   copyright (c) 1997-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copies the unknowns of a subface of a block that are virtual to another
c     block to an intermediate representation of these unknowns.
c     The intermediate representation is a one dimensional
c     array which is equal to the double precision data in the multi-block
c     messages from node to host.
c     The intermediate representation smsg is filled using the subdomain
c     traq array.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, ilev, isddpu03, isubface, itrans,
     +      maxnsubf, nblocks, ntraq1, nfaces, ni, nj, nk, ntraq2,
     +      nvirtual
      double precision smsg(*)
      double precision traq(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,
     +                      1:ntraq1, 1:ntraq2)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock    i    Block sequence number
c     iface     i    Face number
c     ilev      i    Time level of the multi-block transport of information
c     isddpu03  o    Number of virtual unknowns copied
c     isubface  i    Subface number
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntraq1    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  i    Number of rows of virtual cells
c     smsg      i    reserved
c     subfaces  i    reserved
c     traq      i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ibto, ifto, ihigh, ihighs, ii, ilow, ilows, ilast,
     +      iorient, isfto, ismsg, iunk(3), iunktraq,
     +      jj, jlow, jhigh, jlast, nsignmb, nunks, iunks, istartp,
     +      xp(3), nijk(3), ipos , istep, jstep, jlows, kk, kndface
      integer fiunk(3,6)
      integer kface(6)
      integer facenrs(6,2)
      double precision usign
      logical ireverse, jreverse

c     facenrs   array with two times six facenumbers for kind_of_face=1
c               and kind_of_face=2, used for kndface
c     fiunk     array with the numbers of the three velocity components
c               in the right sequence per face
c     i         General loop variable
c     ibto      Target block
c     ifto      Target face
c     ihigh     Upper index of subface
c     ihighs    help variable to save ihigh
c     ii        Loop variable
c     ilast     Last position in ibuffr that may be used
c     ilow      Lower index of subface
c     ilows     help variable to save ilow
c     iorient   Relative orientation of subfaces, see itopol
c     ipos      Position in array
c     ireverse  indicating if in i-direction should be counted in reverse
c               order
c     isfto     Target subface
c     ismsg     Counter for smsg
c     istartp   indicating if two neighbouring faces are rotated
c               w.r.t. each other( counter clockwise).
c               istartp = 1  : no rotation
c               istartp = 2  : rotation of pi * 1/2
c               istartp = 3  : rotation of pi
c               istartp = 4  : rotation of pi * 3/2
c     istep     Step in the i-direction
c     iunk      Array iunk(1:nunks) containing the number of the unknown
c               to be put into smsg.
c               iunk = 0 -> normal flux at block interfaces
c               iunk = 1 -> tangential flux
c               iunk = 2 -> normal flux
c               iunk > 3 -> iunk = ndegfd
c               iunk < 0 -> degree of freedom -iunk
c     iunks     help variable to save iunk
c     iunktraq  Number of the unknown iunk(i) in traq
c     jhigh     Upper j-index for a subface
c     jj        Loop variable
c     jlast     changed upper j-index for a subface
c     jlow      Lower j-index for a subface
c     jlows     help variable to save jlow
c     jreverse  indicating if in j-direction should be counted in reverse
c               order
c     jstep     Step in the j-direction
c     kface     array with kind of face numbers
c     kk        Loop variable
c     kndface   we have to kinds of faces w.r.t. the direction of
c               the tangential velocities; if the orientation is the same
c               it means that the tangential directions can be made the same
c               by rotation of one of the faces.
c               a face has the same orientation
c               with the faces 1,4,5 or with the faces 2,3,6
c               kind of face = 1 :  1,4,5,  2,3,6  ; same with  1,4,5,
c               kind of face = 2 :  2,3,6,  1,4,5  ; same with  2,3,6,
c     nijk      Vector of number of points in the i,j and k-directions
c     nsignmb   Nsignmb = 0 if the normal direction of the subface
c               coincides with that of its neighbouring block, otherwise
c               nsignmb = 1
c     nunks     See iunk
c     usign     Sign to multiply the unknowns with when copying from traq
c               to the intermediate representation
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     if ( itrans.eq.2 ) then
c        --- assume a solut array
c        iunk(1) = normal fluxes on the block interfaces
c        nunks = 1
c     else if ( itrans.eq.1 ) then
c        --- assume a solut array
c        iunk(1) = tangential velocity unknowns
c        iunk(2) = tangential velocity unknowns
c        iunk(3) = normal velocity unknowns
c        nunks = 3
c     else
c        --- assume solut for itrans > 0 and coefs for itrans < 0
c        iunk(1) = itrans
c        nunks = 1
c     end if
c
c     ismsg = 1
c     FOR unknowns to be transported
c     DO
c        determine index range [i1,i2] X [j1,j2]
c
c        determine order and sign (usign) of copying
c
c        FOR I = I1, I2, ISTEP
c        DO
c           FOR J = J1, J2, JSTEP
c           DO
c              smsg(ismsg) = usign*traq(I,J)
c              ismsg = ismsg + 1
c           DONE
c        DONE
c     DONE
c     RETURN ismsg - 1
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data fiunk / 2,3,1, 2,3,1, 1,3,2, 1,3,2, 1,2,3, 1,2,3 /
      data kface / 2,1,1,2,2,1 /
      data facenrs / 1,4,5, 2,3,6,
     +               2,3,6, 1,4,5 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- initialize iunk

      iunk(1) = fiunk(1,iface)
      iunk(2) = fiunk(2,iface)
      iunk(3) = fiunk(3,iface)

      if ( itrans.eq.2 ) then

c     --- assume solut, normal flux components on the boundary

         nunks   = 1

      else if ( itrans.eq.1 ) then

c     --- assume solut, tangential fluxes across the boundary,
c         followed by normal fluxes across the boundary

         nunks = 3

      else

c     --- assume solut for itrans > 0 and coefs for itrans < 0
c         degree of freedom = abs(itrans)

         nunks = 1

      end if

      ireverse = .false.
      jreverse = .false.
      nsignmb = 1

      if ( iface.eq.3 .or. iface.eq.4 ) then
         ilow    = subfaces(iblock, iface, isubface, 3)
         ihigh   = subfaces(iblock, iface, isubface, 4)
         jlow    = subfaces(iblock, iface, isubface, 1)
         jhigh   = subfaces(iblock, iface, isubface, 2)
      else
         ilow    = subfaces(iblock, iface, isubface, 1)
         ihigh   = subfaces(iblock, iface, isubface, 2)
         jlow    = subfaces(iblock, iface, isubface, 3)
         jhigh   = subfaces(iblock, iface, isubface, 4)
      end if

      ibto    = subfaces(iblock, iface, isubface, 5)
      ifto    = subfaces(iblock, iface, isubface, 6)
      isfto   = subfaces(iblock, iface, isubface, 7)
      istartp = subfaces(iblock, iface, isubface, 8)

      if ( (iblock*nfaces + iface)*maxnsubf + isubface
     +     .ge.
     +     (ibto*nfaces + ifto)*maxnsubf + isfto ) then

c     --- the representation is determined by the neighbouring block
c         sign of normal velocity component
c         face number odd : same sign with neighbour faces 2,4,6
c                           not the same with              1,3,5
c         face number even: same sign with neighbour faces 1,3,5
c                           not the same with              2,4,6

         if ( mod(iface,2).ne.0) then

c        --- facenr is odd

            if ( mod(ifto,2).ne.0 ) nsignmb = -1

         end if

         if ( mod(iface,2).eq.0) then

c        --- facenr is even

            if ( mod(ifto,2).eq.0 ) nsignmb = -1

         end if

c        --- we have to kinds of faces w.r.t. the direction of
c            the tangential velocities; if the orientation is the same
c            it means that the tangential directions can be made the same
c            by rotation of one of the faces.
c            a face has the same orientation
c            with the faces 1,4,5 or with the faces 2,3,6
c            kind of face = 1 :  1,4,5,  2,3,6  ; same with  1,4,5,
c            kind of face = 2 :  2,3,6,  1,4,5  ; same with  2,3,6,

          kndface = kface(ifto)

          do i =1, 6
             if ( iface.eq.facenrs(i,kndface) ) ipos=i
          end do
          if ( ipos.le.3) then

c         --- orientation is the same

             iorient = 1

          else

c         --- orientation is not the same

             iorient = 0

          end if

          if (  iorient.eq.1 ) then

c         --- orientation is the same

c             --- if startpoint=1 no order change

             if ( istartp.eq.2 ) then

c            --- exchange i and j  and take then j in reverse order

                ilows  = ilow
                ihighs = ihigh

                ilow  = jlow
                ihigh = jhigh
                jlow  = ilows
                jhigh = ihighs

                iunks   = iunk(1)
                iunk(1) = iunk(2)
                iunk(2) = - iunks

                ireverse = .false.
                jreverse = .true.

             else if ( istartp.eq.3 ) then

c            --- take i and j in reverse order

                iunk(1) = - iunk(1)
                iunk(2) = - iunk(2)

                ireverse = .true.
                jreverse = .true.

             else if ( istartp.eq.4 ) then

c            --- exchange i and j  and take then i in reverse order

                ilows  = ilow
                ihighs = ihigh

                ilow  = jlow
                ihigh = jhigh
                jlow  = ilows
                jhigh = ihighs

                iunks   = iunk(1)
                iunk(1) = - iunk(2)
                iunk(2) = iunks

                ireverse = .true.
                jreverse = .false.

             end if

         else

c        --- orientation is not the same

             if ( istartp.eq.1 ) then

c            --- exchange i and j

                ilows  = ilow
                ihighs = ihigh

                ilow  = jlow
                ihigh = jhigh
                jlow  = ilows
                jhigh = ihighs

                iunks   = iunk(1)
                iunk(1) = iunk(2)
                iunk(2) = iunks

                ireverse = .false.
                jreverse = .false.

             else if ( istartp.eq.2 ) then

c            --- take i in reverse order

                iunk(1) = - iunk(1)

                ireverse = .true.
                jreverse = .false.

             else if ( istartp.eq.3 ) then

c            --- exchange i and j  and take then i and j in reverse order

                ilows  = ilow
                ihighs = ihigh

                ilow  = jlow
                ihigh = jhigh
                jlow  = ilows
                jhigh = ihighs

                iunks   = iunk(1)
                iunk(1) = - iunk(2)
                iunk(2) = - iunks

                ireverse = .true.
                jreverse = .true.

             else if ( istartp.eq.4 ) then

c            --- take  j in reverse order

                iunk(2) = - iunk(2)

                ireverse = .false.
                jreverse = .true.

             end if

         end if

      end if

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

      kk = 0
      if( mod(iface,2).eq.0 ) kk = nijk(iface/2) + 1
      if (  itrans.eq.2 ) then
         if( mod(iface,2).ne.0 ) kk = 1
      end if

      ismsg =1

      do i =1, nunks

         usign = 1d0

         if ( itrans.eq.1 ) then

            if ( i.le.2) then
              if( iunk(i).lt.0) usign = -1d0
            else
               usign = nsignmb
               if( kk.gt.1 ) kk = kk + 1
            end if
            iunktraq = abs(iunk(i))
            ilast = ihigh
            jlast = jhigh
            if( i.eq.1) ilast = ilast + 1
            if( i.eq.2) jlast = jlast + 1

         else if ( itrans.eq.2) then

            iunktraq = abs(iunk(3))
            usign = nsignmb
            ilast = ihigh
            jlast = jhigh

         else

            iunktraq = abs(itrans)
            ilast = ihigh
            jlast = jhigh

         end if

         istep = 1
         jstep = 1
         if ( jreverse ) then
            jlows = jlow
            jlow  = jlast
            jlast = jlows
            jstep = -1
         end if
         if ( ireverse ) then
            ilows = ilow
            ilow  = ilast
            ilast = ilows
            istep = -1
         end if

         do jj = jlow, jlast, jstep
            do ii = ilow, ilast, istep

                xp(abs(iunk(1))) = ii
                xp(abs(iunk(2))) = jj
                xp(abs(iunk(3))) = kk

                traq(xp(1),xp(2),xp(3),iunktraq,ilev) =
     +              usign * smsg(ismsg)
                ismsg = ismsg + 1

            end do
         end do

      end do

      isddpu03 = ismsg - 1

      end
