      function isddputmn ( iblock, iface, isubface, ilev, ni,
     +                     nj, nk, ntraq1, ntraq2, nvirtual, nblocks,
     +                     nfaces, maxnsubf, idiscret, isolv, traq,
     +                     nsubfaces, subfaces, smsg )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 13-01-1999 Copied from isddput
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine copies an intermediate representation of virtual unknowns
c     for a certain subface into the traq array of a certain block.
c     The intermediate representation is a one dimensional array which is equal
c     to the double precision data in the multi-block messages from node to
c     host.
c     The intermediate representation is stored in smsg and is copied to
c     the local traq array.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, ilev, isddputmn, isubface,
     +        maxnsubf, nblocks, ntraq1, nfaces, ni, nj, nk, ntraq2,
     +        nvirtual, idiscret, isolv
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision smsg(*)
      double precision traq(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock     i  Block sequence number
c     idiscret   i  Type of discretization
c     iface      i  Face number
c     ilev       i  Time level of the multi-block transport of information
c     isddputmn  o  Number of virtual unknowns copied
c     isolv      i  Called by solver for periodic b.c. isolv=1 else isolv=0
c     isubface   i  Subface number
c     maxnsubf   i  Maximum number of subfaces encountered on all subfaces
c     nblocks    i  Number of blocks
c     nfaces     i  Number of faces
c     ni         i  Number of non-virtual cells in the i-direction (one block)
c     nj         i  Number of non-virtual cells in the j-direction (one block)
c     nk         i  Number of non-virtual cells in the k-direction (one block)
c     nsubfaces  i  Array containing the number of subfaces of each face,
c                   see array itopol for an extended description
c     ntraq1     i  Dimensioning parameter of the TRAQ array.
c                   (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2     i  Dimensioning parameter of the TRAQ array.
c                   (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual   i  Number of rows of virtual cells
c     smsg       o  reserved
c     subfaces   i  reserved
c     traq      i/o  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddpu04, isddpu05

c     ISDDPU04  Put unknowns at common face into a block (2D case) and compute
c               average value
c     ISDDPU05  Put unknowns at common face into a block (3D case) and compute
c               average value
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     1600    iblock out of range   (iblock,nblocks)
c     1601    iface out of range    (iface,nfaces)
c     1602    isubface out of range (isubface,nsubfaces)
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     If (2-D) call 2-D multi-block transport routine
c     Else     call 3-D multi-block transport routine
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- first perform some checks

      if ( iblock.lt.1 .or. iblock.gt.nblocks ) then

         call iseropen('isddputmn')
         call iserrint(iblock, 1)
         call iserrint(nblocks, 2)
         call iserrsub ( 1600, 2, 0, 0)
         call iserclos('isddputmn')

      end if

      if ( iface.lt.1 .or. iface.gt.nfaces ) then

         call iseropen('isddputmn')
         call iserrint(iface, 1)
         call iserrint(nfaces, 2)
         call iserrsub ( 1601, 2, 0, 0)
         call iserclos('isddputmn')

      end if

      if ( isubface.lt.1 .or.
     +     isubface.gt.nsubfaces(iblock,iface) ) then

         call iseropen('isddputmn')
         call iserrint(isubface, 1)
         call iserrint(nsubfaces(iblock, iface), 2)
         call iserrsub ( 1602, 2, 0, 0)
         call iserclos('isddputmn')

      end if

      if ( nk.eq.0 ) then

c     --- nk = 0, i.e. R^2

         isddputmn = isddpu04 ( iblock, iface, isubface, ilev,
     +                          ni, nj, ntraq1, ntraq2, nvirtual,
     +                          nblocks, nfaces, maxnsubf, idiscret,
     +                          isolv, traq, subfaces, smsg )

      else

c     --- nk # 0, i.e. R^3

         isddputmn = isddpu05 ( iblock, iface, isubface, ilev,
     +                          ni, nj, nk, ntraq1, ntraq2, nvirtual,
     +                          nblocks, nfaces, maxnsubf, traq,
     +                          subfaces, smsg )

      end if

      end
