      subroutine isddsol ( nblocks, iequa, isiinput, issolut,
     +                     isitopol, iadmin, mbsol )
c ======================================================================
c
c     programmer    Erik Brakkee
c     version 1.6   date  09-12-1999 Jason: call to ismbexch
c     version 1.5   date  21-09-1999 Guus: new call to ismblk
c     version 1.4   date  18-01-1999 Jason: call to isexchvc
c     version 1.3   date  27-07-1998 Guus update comment
c     version 1.2   date  13-10-1997 Charles: uncoupled allowed
c     version 1.1   date  24-04-1997 Kees K: minor change for 3D
c     version 1.0   date  04-01-1995 Erik: initial revision
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copying of the inner region of the solution of a certain equation
c     from a single one-dimensional array to all blocks
c     IMPORTANT!!!
c        The vector sol in the subroutine parameterlist may not be governed
c        by memory management routines since this routine also uses
c        memory management.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*)
      integer iequa, nblocks
      integer isitopol
      integer isiinput(1:nblocks), issolut(1:nblocks)
      double precision mbsol(*)

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     mbsol     i    Multi-block solution vector
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, idegfd, ipiinput, ipsolut, istartv, itrans
      integer ndegfd, ndim, ni, nj, nk, npointsv, ntimlv,
     +        nvirtual, offset, icoupled, iseqpres, isol, ipiinblk
      integer maxdim
      parameter (maxdim = 3)
      integer icparea(1:2,1:maxdim),
     +        ilosrc, ihisrc, jlosrc, jhisrc, klosrc, khisrc,
     +        ilodest, ihidest, jlodest, jhidest, klodest, khidest
      logical mean

c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     icparea   Area to be copied
c     idegfd    Loop variable with respect to a given unknown
c     ihidest   Upper bound on i-index for destination
c     ihisrc    Upper bound on i-index for source
c     ilodest   Lower bound on i-index for destination
c     ilosrc    Lower bound on i-index for source
c     ipiinblk  Starting address of array iinblk in ibuffr
c     ipiinput  Starting address of array iinput in ibuffr
c     ipsolut   Starting address of array solut in buffer
c     iseqpres  Sequence number of pressure equation
c     isol      Specifies accurate or inaccurate solution method
c     istartv   First free position in a vector
c     itrans    reserved
c     jhidest   Upper bound on j-index for destination
c     jhisrc    Upper bound on j-index for source
c     jlodest   Lower bound on j-index for destination
c     jlosrc    Lower bound on j-index for source
c     khidest   Upper bound on k-index for destination
c     khisrc    Upper bound on k-index for source
c     klodest   Lower bound on k-index for destination
c     klosrc    Lower bound on k-index for source
c     maxdim    Highest physical space dimension for which ISNaS can work
c     mean      indicates mean normal fluxes should be exchanged
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     offset    General offset factor
c **********************************************************************
c
c                       I/O
c
c      None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE       Activate old permanent array
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISINGETP       get the pointer corresponding to an integer array
c     ISMBEXCH       Transport certain variables to the neighboring blocks
c     ISSCOPY        Copy one array to another
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     istart = 1
c
c     For iblock:= 1(1)nblocks
c
c        copy array sol starting from position istart to
c        inner region of solution of equation iequa
c
c        istart = istart + (ni+1)*(nj+1)*(nk+1)
c
c **********************************************************************
c
c
c
c
c
c
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isddsol' )
      istartv = 1

      do iblock = 1, nblocks

         if ( iadmin(iblock).eq.inodenr ) then

         call isactive(isiinput(iblock))
         call isactive(issolut(iblock))

         ipiinput = isingetp(isiinput(iblock))
         ipsolut  = isdpgetp(issolut(iblock))

         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         ndim     = ibuffr(ipiinput + 57 - 1)
         ndegfd   = ibuffr(ipiinput + 55 - 1)
         ntimlv   = ibuffr(ipiinput + 64 - 1)
         nvirtual = ibuffr(ipiinput + 54 - 1)
         icoupled = ibuffr(ipiinput + 76 - 1)
         if ( icoupled.eq.0 ) then
            iseqpres = 2
         else
            iseqpres = ndim + 1
         end if

         idegfd = iequa
         if ( iequa.gt.1 ) idegfd = idegfd +  (ndim-1)*(1-icoupled )

c        --- initialize copy parameters
c            dimensions of source array

         ilosrc = 1
         ihisrc = ni+1
         jlosrc = 1
         jhisrc = nj+1
         klosrc = 1
         khisrc = nk+1

c        --- dimensions of target array

         ilodest = 1-nvirtual
         ihidest = ni+1+nvirtual
         jlodest = 1-nvirtual
         jhidest = nj+1+nvirtual
         if ( ndim.eq.2 ) then
            klodest = 1
            khidest = 1
         else
            klodest = 1-nvirtual
            khidest = nk+1+nvirtual
         end if

         npointsv = ( ihidest - ilodest + 1) *
     +              ( jhidest - jlodest + 1) *
     +              ( khidest - klodest + 1)

         if ( iequa.lt.iseqpres ) then

c        --- area to be copied for V1

            icparea(1,1) = 1
            icparea(2,1) = ni+1
            icparea(1,2) = 1
            icparea(2,2) = nj
            if ( ndim.eq.2 ) then
               icparea(1,3) = 1
               icparea(2,3) = 1
            else
               icparea(1,3) = 1
               icparea(2,3) = nk
            end if

            offset = (ntimlv-1)*ndegfd + idegfd-1

            call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                     klosrc, khisrc,
     +                     ilodest, ihidest, jlodest, jhidest,
     +                     klodest, khidest,
     +                     1, icparea, mbsol(istartv),
     +                     buffer(ipsolut+offset*npointsv), 1d0)

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)
            idegfd  = idegfd + 1

c           --- area to be copied for V2

            icparea(1,1) = 1
            icparea(2,1) = ni
            icparea(1,2) = 1
            icparea(2,2) = nj+1
            if ( ndim.eq.2 ) then
               icparea(1,3) = 1
               icparea(2,3) = 1
            else
               icparea(1,3) = 1
               icparea(2,3) = nk
            end if

            offset = (ntimlv-1)*ndegfd + idegfd-1

            call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                     klosrc, khisrc,
     +                     ilodest, ihidest, jlodest, jhidest,
     +                     klodest, khidest,
     +                     1, icparea, mbsol(istartv),
     +                     buffer(ipsolut + offset*npointsv), 1d0)

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)
            idegfd  = idegfd + 1

c           --- area to be copied for V3

            if ( ndim.eq.3 ) then

               icparea(1,1) = 1
               icparea(2,1) = ni
               icparea(1,2) = 1
               icparea(2,2) = nj
               icparea(1,3) = 1
               icparea(2,3) = nk+1

               offset = (ntimlv-1)*ndegfd + idegfd-1

               call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                        klosrc, khisrc,
     +                        ilodest, ihidest, jlodest, jhidest,
     +                        klodest, khidest,
     +                        1, icparea, mbsol(istartv),
     +                        buffer(ipsolut + offset*npointsv), 1d0)

               istartv = istartv + (ni+1)*(nj+1)*(nk+1)
               idegfd  = idegfd + 1

            end if

         else

c        --- area to be copied for pressure or transport unknown

            icparea(1,1) = 1
            icparea(2,1) = ni
            icparea(1,2) = 1
            icparea(2,2) = nj
            if ( ndim.eq.2 ) then
               icparea(1,3) = 1
               icparea(2,3) = 1
            else
               icparea(1,3) = 1
               icparea(2,3) = nk
            end if

            offset = (ntimlv-1)*ndegfd + idegfd-1

            call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                     klosrc, khisrc,
     +                     ilodest, ihidest, jlodest, jhidest,
     +                     klodest, khidest,
     +                     1, icparea, mbsol(istartv),
     +                     buffer(ipsolut + offset*npointsv), 1d0)

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)
            idegfd  = idegfd + 1

         end if

      end if

      end do

c     --- fill local virtual unknowns by transport of information

      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      isol     = ibuffr(ipiinblk + 2 - 1)

      if ( iequa.lt.iseqpres ) then
         itrans = 1
      else
         itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
      end if

      mean = .false.
      call ismbexch (itrans, -ntimlv, isiinput, issolut, iadmin,
     +               isitopol, isol.eq.0, mean)

      call iserclos ( 'isddsol' )

      end
