      function isddvget ( iblock, iface, isubface, itrans, ilev, ni, nj,
     +                    ntraq1, ntraq2, nvirtual, nblocks, nfaces,
     +                    maxnsubf, idiscret, traq, subfaces, smsg )
c ======================================================================
c
c        programmer   Erik Brakkee
c        version 2.2    date 18-05-1998 Charles Bilinear discretization
c        version 2.1    date 20-01-1997 Kees K: isdiscret added
c        version 2.0    date 13-03-1996 Guus: remove nsubfaces, nk
c        version 1.2    date 22-11-1994 Erik: extended for coefs.
c                                       changed solut into traq
c        version 1.1    date 21-07-1994 Erik: different block
c                                       orientations are taken into
c                                       account
c        version 1.0    date 07-04-1994
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine copies an intermediate representation of virtual unknowns
c     for a certain subface into the traq array of a certain block.
c     The intermediate representation is a one dimensional array which is
c     equal to the double precision data in the multi-block messages from
c     node to host.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, ilev, isddvget, isubface, itrans,
     +      maxnsubf, nblocks, ntraq1, nfaces, ni, nj, ntraq2,
     +      nvirtual, idiscret
      double precision smsg(*)
      double precision traq(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ntraq1, 1:ntraq2)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock     i  Block sequence number
c     idiscret   i  Type of discretization
c     iface      i  Face number
c     ilev       i  Time level of the multi-block transport of information
c     isddvget   o  Number of virtual unknowns copied.
c     isubface   i  Subface number
c     itrans     i  reserved
c     maxnsubf   i  Maximum number of subfaces encountered on all subfaces
c     nblocks    i  Number of blocks
c     nfaces     i  Number of faces
c     ni         i  Number of non-virtual cells in the i-direction (one block)
c     nj         i  Number of non-virtual cells in the j-direction (one block)
c     ntraq1     i  Dimensioning parameter of the TRAQ array.
c                   (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2     i  Dimensioning parameter of the TRAQ array.
c                   (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual   i  Number of rows of virtual cells
c     smsg       o  reserved
c     subfaces   i  reserved
c     traq       i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, i1, i2, ibto, idummy, ifto, ihigh, ii, ilow, iorient,
     +      isfto, ismsg,
     +      istep, iunk(2), iunktraq, j1,
     +      j2, jj, jstep, nsignmb, nunks, relblckor, tsignmb,
     +      irow, nrow
      double precision usign
      logical reverse

c     i           General loop variable
c     i1          Lower index number along first direction of a face or edge
c     i2          Upper index number along first direction of a face or edge
c     ibto        Target block
c     idummy      Integer dummy
c     ifto        Target face
c     ihigh       Upper index of subface
c     ii          Loop variable
c     ilow        Lower index of subface
c     iorient     Relative orientation of subfaces, see itopol
c     irow        Loop variable for rows
c     isfto       Target subface
c     ismsg       Counter for smsg
c     istep       Step in the i-direction
c     iunk        Array iunk(1:nunks) containing the number of the unknown
c                 to be put into smsg.
c                 iunk = 0 -> normal flux at block interfaces
c                 iunk = 1 -> tangential flux
c                 iunk = 2 -> normal flux
c                 iunk > 3 -> iunk = ndegfd
c                 iunk < 0 -> degree of freedom -iunk
c     iunktraq    Number of the unknown iunk(i) in traq
c     j1          Lower index number along second direction of a face
c     j2          Upper index number along second direction of a face
c     jj          Loop variable
c     jstep       Step in the j-direction
c     nrow        Number of rows
c     nsignmb     Nsignmb = 0 if the normal direction of the subface
c                 coincides with that of its neighbouring block, otherwise
c                 nsignmb = 1
c     nunks       See iunk
c     relblckor   Relative block orientation. If the neighbouring
c                 block has sqrt(g) with opposite sign to this block
c                 then relblckor = 1 otherwise it is 0
c     reverse     Reverse the order of copying unknowns (true) or not (false)
c     tsignmb     If the tangential direction of the subface coincides with
c                 that of its neighbouring block tsignmb = 0, otherwise
c                 tsignmb = 1
c     usign       Sign to multiply the unknowns with when copying from traq
c                 to the intermediate representation
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     if ( itrans.eq.2 ) then
c        --- assume a solut array
c        iunk(1) = normal fluxes on the block interfaces
c        nunks = 1
c     else if ( itrans.eq.1 ) then
c        --- assume a solut array
c        iunk(1) = tangential velocity unknowns
c        iunk(2) = normal velocity unknowns
c        nunks = 2
c     else
c        --- assume solut for itrans > 0 and coefs for itrans < 0
c        iunk(1) = itrans
c        nunks = 1
c     end if
c
c     ismsg = 1
c     FOR unknowns to be transported
c     DO
c        determine index range [i1,i2] X [j1,j2]
c
c        determine order and sign (usign) of copying
c
c        FOR I = I1, I2, ISTEP
c        DO
c           FOR J = J1, J2, JSTEP
c           DO
c              smsg(ismsg)= usign * traq(I,J)
c              ismsg = ismsg + 1
c           DONE
c        DONE
c     DONE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- initialize iunk

      if ( itrans.eq.2 ) then

c     --- assume solut, normal fluxes on the block interface

         nunks = 1
         iunk(1) = 0

      else if ( itrans.eq.1 ) then

c     --- assume solut, tangential fluxes across the block interface
c         followed by normal fluxes across the block interface

         nunks = 2
         iunk(1) = 1
         iunk(2) = 2

      else

c     --- assume solut for itrans > 0 and coefs for itrans < 0

         nunks = 1
         iunk(1) = itrans

      end if

      ismsg = 1

      do i = 1, nunks

         nrow = 1

         iunktraq = abs(iunk(i))

         if ( iunk(i).le.2 .and. iunk(i).ge.0 ) then

            if ( iunk(i).eq.0 ) then
               iunktraq = 2
            else if ( iunk(i).eq.1 ) then
               iunktraq = 1
            else if ( iunk(i).eq.2 ) then
               iunktraq = 2
            end if

            if ( iunktraq.eq.1 ) then
               if ( idiscret.eq.2 .or. idiscret.eq.4 ) nrow = 2
            end if

            if ( iface.eq.ileft .or. iface.eq.iright ) then
               iunktraq = 3 - iunktraq

               if ( iunktraq.eq.2 ) then
                  if ( idiscret.eq.2 .or. idiscret.eq.4 ) nrow = 2
               end if

            end if

         end if

c        --- put the unknown iunk(i) into smsg
c            determine the index range

         ilow = subfaces(iblock, iface, isubface, 1)
         ihigh = subfaces(iblock, iface, isubface, 2)

         if ( iunk(i).eq.1 ) then
            ihigh = ihigh + 1
         end if

c        --- relative orientation

         iorient    = subfaces(iblock,iface,isubface,8)
         tsignmb      = mod(iorient,2)
         nsignmb      = mod(iorient/2,2)
         relblckor = iorient/4
         ibto       = subfaces(iblock,iface,isubface,5)
         ifto       = subfaces(iblock,iface,isubface,6)
         isfto      = subfaces(iblock,iface,isubface,7)

         reverse = .false.
         usign   = 1d0

         if ( (iblock*nfaces + iface)*maxnsubf + isubface .ge.
     +        (ibto*nfaces + ifto)*maxnsubf + isfto ) then

c        --- the representation is determined by the neighbouring block

            if ( itrans.le.2 .and. relblckor.eq.1 ) usign = -usign

            if ( tsignmb.eq.1 ) then

               if ( iunk(i).eq.1 ) usign = -usign
               reverse = .true.

            end if

            if ( nsignmb.eq.1 ) then

               if (iunk(i).eq.2 .or. iunk(i).eq.0 ) usign = -usign

            end if

         end if

         do irow = 1, nrow

            if ( iface.eq.ileft ) then

               i1 = 1 - irow
               j1 = ilow
               j2 = ihigh
               if ( iunk(i).eq.0 ) i1 = i1 + 1
               i2 = i1

            else if ( iface.eq.iright ) then

               i1 = ni + irow
               j1 = ilow
               j2 = ihigh
               if ( iunk(i).eq.2 ) i1 = i1 + 1
               i2 = i1

            else if ( iface.eq.idown ) then

               j1 = 1 - irow
               i1 = ilow
               i2 = ihigh
               if ( iunk(i).eq.0 ) j1 = j1 + 1
               j2 = j1

            else

               j1 = nj + irow
               i1 = ilow
               i2 = ihigh
               if ( iunk(i).eq.2 ) j1 = j1 + 1
               j2 = j1

            end if

            istep = 1
            jstep = 1

            if ( reverse ) then

c           --- reverse the order of copying by swapping both i1,i2 and
c               j1 and j2. This is allowed because either i1=i2 or j1=j2

               idummy = i1
               i1     = i2
               i2     = idummy
               istep  = -istep

               idummy = j1
               j1     = j2
               j2     = idummy
               jstep  = -jstep

            end if

            do ii = i1, i2, istep
               do jj = j1, j2, jstep
                 smsg(ismsg) = usign*traq(ii,jj,iunktraq,ilev)
                 ismsg = ismsg + 1
               end do
            end do

         end do

      end do

      isddvget = ismsg - 1

      end
