      subroutine isglom02( ni, nj, nk, ndegfd, ntimlv, nvirtual, iequa,
     +                     matrix, solut, outvec )
c=====================================================================
c
c     programmer   Erik Brakkee
c
c
c     version      1.0  4-1-95    Erik:  initial revision
c
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Compute pressure/transport matrix-vector product, using a matrix
c     with virtual coefficients and a vector with virtual unknowns.
c
c
c***********************************************************************
c
c                  KEYWORDS
c
c
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
      implicit none
c
      integer iequa, ndegfd, ni, nj, nk, ntimlv, nvirtual

      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)

      double precision matrix(1:ni+1, 1:nj+1, 1:9)

      double precision outvec(1:ni+1, 1:nj+1)

c
c     iequa      i  General equation sequence number
c     matrix     i  reserved
c     ndegfd     i  Number of degrees of freedom
c     ni         i  Number of non-virtual cells in the i-direction (one block)
c     nj         i  Number of non-virtual cells in the j-direction (one block)
c     nk         i  Number of non-virtual cells in the k-direction (one block)
c     ntimlv     i  Number of time levels
c     nvirtual   i  Number of rows of virtual cells
c     outvec     o  Output vector
c     solut      i  reserved
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisnas'
c
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer i, idegfd, j
c
c     i         General loop variable
c     idegfd    Loop variable with respect to a given unknown
c     j         General loop variable
c
c**********************************************************************
c
c                   I/O
c
c     None
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c
c=======================================================================
c

      idegfd = iequa + 1

      i = ni+1
      do j = 1, nj+1
         outvec(i,j) = 0d0
      end do

      j = nj+1
      do i = 1, ni+1
         outvec(i,j) = 0d0
      end do

      do j = 1, nj
         do i = 1, ni
            outvec(i,j) = matrix(i,j,1)*solut(i,   j,   idegfd,ntimlv) +
     +               matrix(i,j,2)*solut(i-1, j-1, idegfd,ntimlv) +
     +               matrix(i,j,3)*solut(i,   j-1, idegfd,ntimlv) +
     +               matrix(i,j,4)*solut(i+1, j-1, idegfd,ntimlv) +
     +               matrix(i,j,5)*solut(i-1, j,   idegfd,ntimlv) +
     +               matrix(i,j,6)*solut(i+1, j,   idegfd,ntimlv) +
     +               matrix(i,j,7)*solut(i-1, j+1, idegfd,ntimlv) +
     +               matrix(i,j,8)*solut(i,   j+1, idegfd,ntimlv) +
     +               matrix(i,j,9)*solut(i+1, j+1, idegfd,ntimlv)

         end do
      end do


      end














