      subroutine isglom31( ni, nj, nk, ndegfd, ndim, ntimlv, nvirtual,
     +                     matrix, solut, outvec )
c ======================================================================
c
c     programmer   Kees Kassels
c
c     version      1.0  date 25-04-1997
c
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute momentum matrix-vector product, using a matrix with
c     virtual coefficients and a vector with virtual unknowns.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndegfd, ndim, ni, nj, nk, ntimlv, nvirtual
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)
      double precision matrix(1:ni+1, 1:nj+1, 1:nk+1, 1:ndim, *)
      double precision outvec(1:ni+1, 1:nj+1, 1:nk+1, 1:ndim)

c     matrix     i  reserved
c     ndegfd     i  Number of degrees of freedom
c     ndim       i  Dimension of space (2 or 3)
c     ni         i  Number of non-virtual cells in the i-direction (one block)
c     nj         i  Number of non-virtual cells in the j-direction (one block)
c     nk         i  Number of non-virtual cells in the k-direction (one block)
c     ntimlv     i  Number of time levels
c     nvirtual   i  Number of rows of virtual cells
c     outvec     o  Output vector
c     solut      i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, idim , idim1, idim2, idim3

c     i         General loop variable
c     idim      Loop variable used for dimension
c     idim1     Loop variable used for dimension
c     idim2     Loop variable used for dimension
c     idim3     Loop variable used for dimension
c     j         General loop variable
c     k         General loop variable
c **********************************************************************
c
c                       I/O
c
c     None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      idim = 1
      j    = nj+1
      do k = 1, nk+1
         do i = 1, ni+1
            outvec(i,j,k,idim) = 0d0
         end do
      end do
      k    = nk+1
      do j = 1, nj+1
         do i = 1, ni+1
            outvec(i,j,k,idim) = 0d0
         end do
      end do

      idim = 2
      i = ni+1
      do k = 1, nk+1
         do j = 1, nj+1
            outvec(i,j,k,idim) = 0d0
         end do
      end do
      k = nk+1
      do j = 1, nj+1
         do i = 1, ni+1
            outvec(i,j,k,idim) = 0d0
         end do
      end do

      idim = 3
      i = ni+1
      do k = 1, nk+1
         do j = 1, nj+1
            outvec(i,j,k,idim) = 0d0
         end do
      end do
      j = nj+1
      do k = 1, nk+1
         do i = 1, ni+1
            outvec(i,j,k,idim) = 0d0
         end do
      end do

c     --- handle V1 component

      idim1 = 1
      idim2 = 2
      idim3 = 3
      do k = 1, nk
         do j = 1, nj
            do i = 1, ni+1

            outvec(i,j,k,idim1) =
     +         matrix(i,j,k,idim1,1)  * solut(i  ,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,2)  * solut(i-1,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,3)  * solut(i  ,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,4)  * solut(i+1,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,5)  * solut(i-1,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,6)  * solut(i+1,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,7)  * solut(i-1,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,8)  * solut(i  ,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,9)  * solut(i+1,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,10) * solut(i  ,j-1,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,11) * solut(i-1,j  ,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,12) * solut(i  ,j  ,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,13) * solut(i+1,j  ,k-1,idim1,ntimlv)
            outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,14) * solut(i  ,j+1,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,15) * solut(i  ,j-1,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,16) * solut(i-1,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,17) * solut(i  ,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,18) * solut(i+1,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,19) * solut(i  ,j+1,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,20) * solut(i-1,j  ,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,21) * solut(i  ,j  ,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,22) * solut(i-1,j+1,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,23) * solut(i  ,j+1,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,24) * solut(i-1,j-1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,25) * solut(i  ,j-1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,26) * solut(i-1,j  ,k  ,idim2,ntimlv)
           outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,27) * solut(i  ,j  ,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,28) * solut(i-1,j+1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,29) * solut(i  ,j+1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,30) * solut(i-1,j+2,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,31) * solut(i  ,j+2,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,32) * solut(i-1,j  ,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,33) * solut(i  ,j  ,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,34) * solut(i-1,j+1,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,35) * solut(i  ,j+1,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,36) * solut(i-1,j  ,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,37) * solut(i  ,j  ,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,38) * solut(i-1,j-1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,39) * solut(i  ,j-1,k  ,idim3,ntimlv)
           outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,40) * solut(i-1,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,41) * solut(i  ,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,42) * solut(i-1,j+1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,43) * solut(i  ,j+1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,44) * solut(i-1,j-1,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,45) * solut(i  ,j-1,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,46) * solut(i-1,j  ,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,47) * solut(i  ,j  ,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,48) * solut(i-1,j+1,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,49) * solut(i  ,j+1,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,50) * solut(i-1,j  ,k+2,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,51) * solut(i  ,j  ,k+2,idim3,ntimlv)

            end do
         end do
      end do

c     --- handle V2 component

      idim1 = 2
      idim2 = 3
      idim3 = 1
      do k = 1, nk
         do j = 1, nj+1
            do i = 1, ni
            outvec(i,j,k,idim1) =
     +         matrix(i,j,k,idim1,1)  * solut(i  ,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,2)  * solut(i-1,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,3)  * solut(i  ,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,4)  * solut(i+1,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,5)  * solut(i-1,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,6)  * solut(i+1,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,7)  * solut(i-1,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,8)  * solut(i  ,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,9)  * solut(i+1,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,10) * solut(i  ,j-1,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,11) * solut(i-1,j  ,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,12) * solut(i  ,j  ,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,13) * solut(i+1,j  ,k-1,idim1,ntimlv)
            outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,14) * solut(i  ,j+1,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,15) * solut(i  ,j-1,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,16) * solut(i-1,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,17) * solut(i  ,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,18) * solut(i+1,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,19) * solut(i  ,j+1,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,20) * solut(i  ,j-1,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,21) * solut(i  ,j  ,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,22) * solut(i-1,j-1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,23) * solut(i  ,j-1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,24) * solut(i+1,j-1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,25) * solut(i-1,j  ,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,26) * solut(i  ,j  ,k  ,idim2,ntimlv)
           outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,27) * solut(i+1,j  ,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,28) * solut(i-1,j-1,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,29) * solut(i  ,j-1,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,30) * solut(i+1,j-1,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,31) * solut(i-1,j  ,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,32) * solut(i  ,j  ,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,33) * solut(i+1,j  ,k+1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,34) * solut(i  ,j-1,k+2,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,35) * solut(i  ,j  ,k+2,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,36) * solut(i  ,j-1,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,37) * solut(i+1,j-1,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,38) * solut(i  ,j  ,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,39) * solut(i+1,j  ,k-1,idim3,ntimlv)
           outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,40) * solut(i-1,j-1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,41) * solut(i  ,j-1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,42) * solut(i+1,j-1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,43) * solut(i+2,j-1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,44) * solut(i-1,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,45) * solut(i  ,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,46) * solut(i+1,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,47) * solut(i+2,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,48) * solut(i  ,j-1,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,49) * solut(i+1,j-1,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,50) * solut(i  ,j  ,k+1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,51) * solut(i+1,j  ,k+1,idim3,ntimlv)

            end do
         end do
      end do

c     --- handle V3 component

      idim1 = 3
      idim2 = 1
      idim3 = 2
      do k = 1, nk+1
         do j = 1, nj
            do i = 1, ni
            outvec(i,j,k,idim1) =
     +         matrix(i,j,k,idim1,1)  * solut(i  ,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,2)  * solut(i-1,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,3)  * solut(i  ,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,4)  * solut(i+1,j-1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,5)  * solut(i-1,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,6)  * solut(i+1,j  ,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,7)  * solut(i-1,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,8)  * solut(i  ,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,9)  * solut(i+1,j+1,k  ,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,10) * solut(i  ,j-1,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,11) * solut(i-1,j  ,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,12) * solut(i  ,j  ,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,13) * solut(i+1,j  ,k-1,idim1,ntimlv)
            outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,14) * solut(i  ,j+1,k-1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,15) * solut(i  ,j-1,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,16) * solut(i-1,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,17) * solut(i  ,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,18) * solut(i+1,j  ,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,19) * solut(i  ,j+1,k+1,idim1,ntimlv)+
     +         matrix(i,j,k,idim1,20) * solut(i  ,j-1,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,21) * solut(i+1,j-1,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,22) * solut(i-1,j  ,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,23) * solut(i  ,j  ,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,24) * solut(i+1,j  ,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,25) * solut(i+2,j  ,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,26) * solut(i  ,j+1,k-1,idim2,ntimlv)
           outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,27) * solut(i+1,j+1,k-1,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,28) * solut(i  ,j-1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,29) * solut(i+1,j-1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,30) * solut(i-1,j  ,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,31) * solut(i  ,j  ,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,32) * solut(i+1,j  ,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,33) * solut(i+2,j  ,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,34) * solut(i  ,j+1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,35) * solut(i+1,j+1,k  ,idim2,ntimlv)+
     +         matrix(i,j,k,idim1,36) * solut(i  ,j-1,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,37) * solut(i-1,j  ,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,38) * solut(i  ,j  ,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,39) * solut(i+1,j  ,k-1,idim3,ntimlv)
           outvec(i,j,k,idim1) = outvec(i,j,k,idim1) +
     +         matrix(i,j,k,idim1,40) * solut(i-1,j+1,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,41) * solut(i  ,j+1,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,42) * solut(i+1,j+1,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,43) * solut(i  ,j+2,k-1,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,44) * solut(i  ,j-1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,45) * solut(i-1,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,46) * solut(i  ,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,47) * solut(i+1,j  ,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,48) * solut(i-1,j+1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,49) * solut(i  ,j+1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,50) * solut(i+1,j+1,k  ,idim3,ntimlv)+
     +         matrix(i,j,k,idim1,51) * solut(i  ,j+2,k  ,idim3,ntimlv)

            end do
         end do
      end do

      end
