      subroutine ismbexch ( itrans, ilev, isiinput, istraq,
     +                      iadmin, isitopol, accurate, mean )
c ======================================================================
c
c        programmer     Jason Frank
c        version 1.0    date 28-10-1999  Jason: revision of ismblk
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Transport certain variables (itrans,ilev) to the neighboring blocks
c     (even if those are on another process.)
c **********************************************************************
c
c                       KEYWORDS
c
c     multiblock
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), ilev, isiinput(*), isitopol,
     +        istraq(*), itrans
      logical accurate, mean

c     accurate i  if true, all blocks are treated as local to the host
c     iadmin   i  reserved
c     ilev     i  Time level of the multi-block transport of information
c     isiinput i  Array of sequence numbers (nblocks) of array iinput
c     isitopol i  Sequence number of array itopol
c     istraq   i  Array of sequence numbers (nblocks) of array traq
c     itrans   i  reserved
c     mean     i  if true, mean normal fluxes will be computed on the interface
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, ipitopol, ipnsubf, ipsmsg,
     +        ipsubf,  issmsg, lensmsg, maxnsubf,
     +        nblocks, nfaces, ni, nj, nk, ntraq1, ntraq2, nvirtual,
     +        ipiindsc, idiscret, isolv, ind, dummy

c     dummy     dummy integer
c     iblock    block number
c     idiscret  Type of discretization
c     ind       Loop index
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsmsg    Starting address of array smsg in buffer
c     ipsubf    Starting address of the subfaces array
c     isolv     Called by solver for periodic b.c. isolv=1 else isolv=0
c     issmsg    Sequence number of array smsg
c     lensmsg   Length of array smsg
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nblocks   Number of blocks
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntraq1    Dimensioning parameter of the TRAQ array.
c               (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    Dimensioning parameter of the TRAQ array.
c               (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISMBEX02  does all the work for this routine
c     ISRCDDHS  receive virtual cell data on the host
c     ISSNDDND  send virtual cell data from the node
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     1. compute some general parameters
c     2. compute an upper bound for a multi-block submessage (smsg)
c     3. activate and create arrays
c     4. compute pointers
c     5. actual multi-block transport (ismblk02)
c     6. delete array smsg
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('ismbexch')

c     --- compute nblocks, nfaces and maxnsubfaces

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol + 1 - 1)
      nfaces   = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)
      ipnsubf  = ibuffr(ipitopol + 5 - 1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol + 6 - 1) + ipitopol - 1

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ipiindsc = ibuffr(ipiinput-1+6)
      idiscret = ibuffr(ipiinput-1+ipiindsc+1)
      nvirtual = ibuffr(ipiinput + 54 - 1)

c     --- if ilev<0, then isdiscret is set equal to 10 indicating that
c         nvirtual rows of virtual cells must be copied

      if ( ilev.lt.0 ) idiscret = 10

      if ( mean ) itrans = 2

      if ( itrans.lt.0 ) then

c     --- coefs array: ntraq1 = ncoefs, ntraq2 = 0

         ntraq1 = ibuffr(ipiinput + 66 - 1)
         ntraq2 = 1

      else

c     --- solut array, ntraq1 = ndegfd, ntraq2 = ntimlv

         ntraq1 = ibuffr(ipiinput + 55 - 1)
         ntraq2 = ibuffr(ipiinput + 64 - 1)

      end if

c     --- compute an upper bound for the length of a multi-block submessage

      lensmsg = 0
      do iblock = 1, nblocks

c     --- activate istraq(iblock) and isiinput(iblock) while we're at it

         if (isiinput(iblock).ne.0) then

            call isactive(istraq(iblock))
            call isactive(isiinput(iblock))
            ipiinput = isingetp( isiinput(iblock))

            ni  = ibuffr(ipiinput + 51 - 1)
            nj  = ibuffr(ipiinput + 52 - 1)
            nk  = ibuffr(ipiinput + 53 - 1)
            if ( nk.eq.0 ) then
               lensmsg = max(lensmsg, 4*(max(ni,nj)+1+2*nvirtual))
            else
               lensmsg = max(lensmsg,
     +                        6*(max(ni,nj,nk)+1+2*nvirtual)**2)
            end if

         end if

      end do

c     --- create array

      issmsg = 0
      call isdcreat(issmsg, lensmsg, 'smsg' )
      call isactive (issmsg)
      ipsmsg   = isdpgetp(issmsg)

c     --- indicating that ismbex02 is not called in the linear solver

      isolv = 0

      call ismbex02 ( itrans, abs(ilev), ntraq1, ntraq2, nvirtual,
     +                nblocks,  nfaces, maxnsubf, idiscret,
     +                isolv, isiinput, istraq, iadmin,
     +                ibuffr(ipnsubf), ibuffr(ipsubf),
     +                buffer(ipsmsg), lensmsg, mean )

      if ( accurate ) then

         if ( inodenr.eq.ISHOST ) then

c        --- receive normal fluxes if on the host

            do ind = 1, iacnodes

c           --- receive normal fluxes

               call isrcddhs( dummy, isiinput, istraq, isitopol )

            end do

         else

c        --- send normal fluxes on the node

           call issnddnd ( itrans, abs(ilev), isiinput, istraq,
     +                     iadmin, isitopol )

         end if

      end if

      call isdelarr(issmsg)
      call isdcrpri

      call iserclos('ismbexch')

      end
