      subroutine isrcddnd ( nblocks, isiinput, istraq, isitopol,
     +                      iadmin, done )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.8  23-11-1998 Jason: isrecmpi version
c     version      1.7  18-05-1998 Charles Bilinear discretization
c     version      1.6  04-03-1997 Kees K.: extension 3D
c     version      1.5  14-01-1997 Kees K.: arraylength changed for
c                                           wesbeek discretization
c     version      1.3  12-12-1996 Kees K: idiscret added.
c     version      1.2  22-11-1994 Erik: extended for coefs.
c                                   changed solut into traq
c     version      1.1  24-08-1994 Erik: internal representation of
c                                        virt no longer used.
c     version      1.0  02-05-1994
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The purpose of this routine is to receive virtual unknowns on the
c     node from the host using routine isreceiv.
c     On entry, the message from the host is assumed to be already open.
c     At return, the message is still open.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), isiinput(*), isitopol, istraq(*), nblocks
      logical done

c     done      o    indicates the node domain decomposition should stop
c     iadmin    i    reserved
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     istraq    i    Array of sequence numbers (nblocks) of array traq
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  iarray(3), iblock, ilev, ind, issmsg,
     +         ipiinput, ipitopol, ipnsubf, ipsmsg, ipsubf, iptraq,
     +         itrans, lengmes, lensmsg, maxnsubf, ntraq1,
     +         nfaces, ni, nj, nk, ntraq2, nvirtual, vlen, ndim,
     +         ipiindsc, idiscret, ihost

c     iarray    Integer array for issend and isreceiv
c     iblock    Block sequence number
c     idiscret  Type of discretization
c     ihost     rank of the host process
c     ilev      Time level of the multi-block transport of information
c     ind       Sequence number of node
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsmsg    Starting address of array smsg in buffer
c     ipsubf    Starting address of the subfaces array
c     iptraq    Starting address of array traq in buffer
c     issmsg    Sequence number of array smsg
c     itrans    reserved
c     lengmes   Length of the message
c     lensmsg   Length of array smsg
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntraq1    Dimensioning parameter of the TRAQ array.
c               (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    Dimensioning parameter of the TRAQ array.
c               (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  Number of rows of virtual cells
c     vlen      Number of virtual tangential velocities needed from
c               neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp, isredd02

c     ISACTIVE  Activate permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all permanent arrays
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISRECMPI  Receive data from node or host
c     ISREDD02  put virtual unknowns of a block in traq
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     Receive(iblock, itrans, ilev)
c
c     do while ( iblock <> 0 )
c
c       receive data part using isrcnd02(...)
c
c       Receive(iblock, itrans, ilev)
c
c     end do
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isrcddnd')

      done = .false.
      ihost = ISHOST

      iarray(1) = 1
      iarray(2) = 0
      iarray(3) = 0

c     --- create an array for storing a submessage

      lensmsg = 0
      do iblock = 1, nblocks
         if ( iadmin(iblock).eq.inodenr ) then
            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))
            lensmsg  = max(lensmsg, 3*ibuffr(ipiinput + 58 - 1))
         end if
      end do
      lensmsg = lensmsg + 1
      issmsg = 0
      call isdcreat(issmsg, lensmsg, 'smsg' )

      ipiindsc     = ibuffr(ipiinput-1+6)
      idiscret     = ibuffr(ipiinput-1+ipiindsc+1)

      lengmes = 3
      call isrecmpi (iarray, lengmes, ISINT, ihost, ICVCINFO)
      iblock = iarray(1)
      itrans = iarray(2)
      ilev   = iarray(3)

      do while ( iblock.ne.0 )

c     --- activate the needed arrays

         call isactive(issmsg)
         call isactive(isitopol)
         call isactive(istraq(iblock))
         call isactive(isiinput(iblock))

c        --- compute addresses

         ipsmsg       = isdpgetp(issmsg)
         iptraq       = isdpgetp(istraq(iblock))
         ipiinput     = isingetp(isiinput(iblock))
         ipitopol     = isingetp(isitopol)
         ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
         ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c        --- compute some parameters

         nfaces       = ibuffr(ipitopol + 2 - 1)
         maxnsubf = ibuffr(ipitopol + 3 - 1)
         ni           = ibuffr(ipiinput + 51 - 1)
         nj           = ibuffr(ipiinput + 52 - 1)
         nk           = ibuffr(Ipiinput + 53 - 1)
         ndim         = ibuffr(Ipiinput + 57 - 1)
         if ( itrans.lt.0 ) then

c        --- coefs array: ntraq1 = ncoefs, ntraq2 = 0

            ntraq1    = ibuffr(ipiinput + 66 - 1)
            ntraq2    = 1

         else

c        --- solut array, ntraq1 = ndegfd, ntraq2 = ntimlv

            ntraq1    = ibuffr(ipiinput + 55 - 1)
            ntraq2    = ibuffr(ipiinput + 64 - 1)

         end if
         nvirtual     = ibuffr(ipiinput + 54 - 1)
         vlen         = ibuffr(ipiinput + 58 - 1)

c        --- receive virtual unknowns

         lengmes = vlen

         if ( ndim.eq.2 ) then
            if ( idiscret.eq.2 .or. idiscret.eq.4 ) then
               if ( itrans.eq.1 ) lengmes = lengmes*3
            else
               if ( itrans.eq.1 ) lengmes = lengmes*2
            end if
         else
            if ( itrans.eq.1 ) lengmes = lengmes*3
         end if

         call isrecmpi (buffer(ipsmsg), lengmes, ISDOUBLE, ihost,
     +      ICVCDATA)

         lengmes = isredd02(iblock, itrans,  ilev,
     +                     ni, nj, nk,
     +                     ntraq1, ntraq2, nvirtual,
     +                     nblocks, nfaces, maxnsubf,
     +                     idiscret, buffer(iptraq),
     +                     ibuffr(ipnsubf), ibuffr(ipsubf),
     +                     buffer(ipsmsg))

c        --- receive information about next submessage

         lengmes = 3
         call isrecmpi (iarray, lengmes, ISINT, ihost, ICVCINFO)
         iblock = iarray(1)
         itrans = iarray(2)
         ilev   = iarray(3)

      end do

      if ( ilev.eq.-1 ) done = .true.

      call isdelarr(issmsg)
      call isdcrpri

      call iserclos('isrcddnd')

      end
