      subroutine isrchs02 ( nblocks, nfaces, maxnsubf,
     +                      iblock, iface, isubface, itrans, ilev,
     +                      ind, istraq, isiinput,
     +                      nsubfaces, subfaces, smsg )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.6  date 23-11-1998 Jason: isrecmpi version
c     version      1.5  date 18-05-1998 Charles Bilinear discretization
c     version      1.4  date 21-10-1997 Kees K.: call of isddput with isolv
c     version      1.3  date 28-02-1997 Kees K.: extension 3D
c     version      1.2  date 12-12-1996 Kees K.: idiscret added
c     version      1.1  date 22-11-1994 Erik: extended for coefs.
c                                       changed solut into traq
c     version      1.0  date 03-05-1994
c
c   copyright (c) 1994-1998 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Receives virtual unknowns from a certain subface of a certain block
c     from node ind and place them in the traq arrays.
c     On entry, the message from node ind has already been opened.
c     At return, the message from node ind is not terminated and
c     can still be read from.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, ilev, ind, isiinput(*), isubface, itrans,
     +      maxnsubf, nblocks, nfaces, istraq(*)
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision smsg(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock    i    Block sequence number
c     iface     i    Face number
c     ilev      i    Time level of the multi-block transport of information
c     ind       i    Sequence number of node
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     istraq    i    Array of sequence numbers (nblocks) of array traq
c     isubface  i    Subface number
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     smsg      i    reserved
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer dummy, ibto, ifto, ipiinput, iptraq,
     +      isfto, lensmsg, ntraq1, ni, nj, nk, ntraq2, nvirtual
      integer ipiindsc, idiscret, isolv

c     dummy     Dummy
c     ibto      Target block
c     idiscret  Type of discretization
c     ifto      Target face
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     iptraq    Starting address of array traq in buffer
c     isfto     Target subface
c     isolv     Called by solver for periodic b.c. isolv=1 else isolv=0
c     lensmsg   Length of array smsg
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntraq1    Dimensioning parameter of the TRAQ array.
c               (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    Dimensioning parameter of the TRAQ array.
c               (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp, isddput

c     ISDDPUT   copy virtual unknowns to a certain subface of a certain block
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISINGETP  get the pointer corresponding to an integer array
c     ISRECMPI  receive data using MPI
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Receive data from subface (iblock,iface,isubface)
c
c     Determine the neighbouring subface of that subface
c
c     Place the data in the traq array at the neighbouring subface.
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- set some isreceive parameters

      ibto   = subfaces(iblock, iface, isubface, 5)
      ifto   = subfaces(iblock, iface, isubface, 6)
      isfto  = subfaces(iblock, iface, isubface, 7)

c     --- determine pointers

      ipiinput =  isingetp(isiinput(ibto))
      iptraq   =  isdpgetp(istraq(ibto))
      ipiindsc =  ibuffr(ipiinput-1+6)
      idiscret =  ibuffr(ipiinput-1+ipiindsc+1)

      ni       =  ibuffr(ipiinput + 51 - 1)
      nj       =  ibuffr(ipiinput + 52 - 1)
      nk       =  ibuffr(ipiinput + 53 - 1)

c     --- determine length of the sub message

      if ( nk.eq.0 ) then

c     --- ndim = 2

         lensmsg = subfaces(iblock,iface,isubface,2) -
     +          subfaces(iblock,iface,isubface,1) + 1

         if ( idiscret.eq.2 .or. idiscret.eq.4 ) then
            if ( itrans.eq.1 ) lensmsg = 3*lensmsg + 2
         else
            if ( itrans.eq.1 ) lensmsg = 2*lensmsg + 1
         end if

      else

c     --- ndim = 3

         lensmsg = ( subfaces(iblock,iface,isubface,2) -
     +               subfaces(iblock,iface,isubface,1) + 1 ) *
     +             ( subfaces(iblock,iface,isubface,4) -
     +               subfaces(iblock,iface,isubface,3) + 1 )

         if ( itrans.eq.1 )
     +      lensmsg = ( subfaces(iblock,iface,isubface,2) -
     +                  subfaces(iblock,iface,isubface,1) + 2 ) *
     +                ( subfaces(iblock,iface,isubface,4) -
     +                  subfaces(iblock,iface,isubface,3) + 1 ) +
     +                ( subfaces(iblock,iface,isubface,2) -
     +                  subfaces(iblock,iface,isubface,1) + 1 ) *
     +                ( subfaces(iblock,iface,isubface,4) -
     +                  subfaces(iblock,iface,isubface,3) + 2 ) +
     +                ( subfaces(iblock,iface,isubface,2) -
     +                  subfaces(iblock,iface,isubface,1) + 1 ) *
     +                ( subfaces(iblock,iface,isubface,4) -
     +                  subfaces(iblock,iface,isubface,3) + 1 )

      end if

c     --- receive data

      call isrecmpi (smsg, lensmsg, ISDOUBLE, ind, ICVCDATA)

c     --- determine some parameters

      if ( itrans.lt.0 ) then

c     --- coefs array: ntraq1 = ncoefs, ntraq2 = 0

         ntraq1    = ibuffr(ipiinput + 66 - 1)
         ntraq2    = 1

      else

c     --- solut array, ntraq1 = ndegfd, ntraq2 = ntimlv

         ntraq1    = ibuffr(ipiinput + 55 - 1)
         ntraq2    = ibuffr(ipiinput + 64 - 1)

      end if
      nvirtual =  ibuffr(ipiinput + 54 - 1)

      isolv = 0
      dummy=  isddput(ibto, ifto, isfto, itrans, ilev,
     +                ni, nj, nk,
     +                ntraq1, ntraq2, nvirtual,
     +                nblocks, nfaces, maxnsubf, idiscret, isolv,
     +                buffer(iptraq), nsubfaces, subfaces, smsg)

      end
