      subroutine isreldd(itrans, ilev, isiinput, istraq, isitopol,
     +                   relvar)
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.3  date 11-12-1996  Kees K: idiscret
c     version      1.2  date 22-11-1994  Erik:  extended for coefs
c                                        changed solut into traq
c     version      1.1  date 24-08-1994  Erik:  internal representation of
c                                        virt is not used anymore
c     version      1.0  date 18-04-1994  Erik:  initial revision
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy all virtual unknowns of a certain type (itrans) at a certain
c     time level (ilev) to a single one-dimensional array of relevant
c     variables (relvar)
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ilev, isiinput(*), istraq(*), isitopol, itrans
      double precision relvar(*)

c     ilev      i    Time level of the multi-block transport of information
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     istraq    i    Array of sequence numbers (nblocks) of array traq
c     itrans    i    reserved
c     relvar    o    Array containing copied virtual unknowns
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, ipitopol, ipnsubf,
     +        ipsubf, iptraq, istrtrel,
     +        maxnsubf, nblocks, ndegfd, nfaces, ni,
     +        nj, nk, ntimlv,  nvirtual,
     +        ipiindsc, idiscret

c     iblock    Block sequence number
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsubf    Starting address of the subfaces array
c     iptraq    Starting address of array traq in buffer
c     istrtrel  First free position in relvar
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isredd02, isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISREDD02  Put all virtual unknowns of a certain type for a certain
c               block back
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c     do iblock = 1, nblocks
c
c        get virtual unknowns of block iblock from relvar
c        and place into traq using isredd02
c
c     end do
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('isreldd')

c     --- activate arrays

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol + 1 - 1)
      do iblock = 1, nblocks
         call isactive(isiinput(iblock))
         call isactive(istraq(iblock))
      end do

c     --- compute pointers

      ipitopol     = isingetp(isitopol)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c     --- compute some general parameters

      nblocks      = ibuffr(ipitopol + 1 - 1)
      nfaces       = ibuffr(ipitopol + 2 - 1)
      maxnsubf     = ibuffr(ipitopol + 3 - 1)
      ipiinput     = isingetp(isiinput(1))
      ndegfd       = ibuffr(ipiinput + 55 - 1)
      ntimlv       = ibuffr(ipiinput + 64 - 1)
      nvirtual     = ibuffr(ipiinput + 54 - 1)
      ipiindsc     = ibuffr(ipiinput-1+6)
      idiscret     = ibuffr(ipiinput-1+ipiindsc+1)

c     --- main loop

      istrtrel = 1
      do iblock = 1, nblocks

         iblockrf = iblock
         ipiinput = isingetp(isiinput(iblock))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         iptraq   = isdpgetp(istraq(iblock))

         istrtrel = istrtrel +
     +      isredd02(iblock, itrans, ilev, ni, nj, nk,
     +               ndegfd, ntimlv, nvirtual,
     +               nblocks, nfaces, maxnsubf, idiscret,
     +               buffer(iptraq),
     +               ibuffr(ipnsubf), ibuffr(ipsubf),
     +               relvar(istrtrel))

      end do

      call iserclos('isreldd')

      end
