      subroutine issdd (iequa, nblocks, isiinput, isrinput,   iscoor,
     +                  issolut, ismatrix, isgmat , isrhside, isitopol,
     +                  iadmin, isinfbnd, isbndcon, isgeom,
     +                  isdensup)
c ======================================================================
c
c     programmer   Kees Kassels
c     version      2.6   date    21-09-1999 Guus: new call to ismblk
c     version      2.5   date    23-11-1998 Jason: converted from isndd
c     version      2.4   date    14-05-1998 Duncan: extension for saving
c                                           coefficients dens upwind
c                                           new call to islinsol
c     version      2.3   date    01-04-1998 Duncan: new call to islinsol
c     version      2.2   date    13-11-1997 Kees K: call islinsol with isitopol
c     version      2.1   date    05-11-1997 Charles: change of usebndcn
c     version      2.0   date    13-10-1997 Charles: uncoupled allowed
c     version      1.9   date    06-08-1997 Guus: return in case of error
c     version      1.8   date    30-01-1996 Guus: set iblockrf
c     version      1.7   date    26-10-1995 Kees K.: itrans for 3D
c     version      1.6   date    04-10-1995 Kees K.: warning for itrans
c     version      1.5   date    01-02-1995 Kees K: skip removed
c     version      1.4   date    31-01-1995 Erik: name change issds01 toc
c                                           islinsol. New dimensions for
c                                           ismatrix
c     version      1.3   date     5- 1-1995 Erik: isndd does not build
c                                           matrices anymore
c     version      1.2   date    25-11-1994 extrapolation unknowns added
c     version      1.1   date    31-10-1994 coefs, geom added
c     version      1.0   date    19-05-1994
c
c   copyright (c) 1993-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     sequential domain decomposition over the local blocks
c     In case of a single block this is merely the linear solver
c     The following steps are performed
c     For all local blocks
c        Solve linear system
c        In case of multiblock perform transport actions
c        Extrapolate the solution vector to virtual points
c **********************************************************************
c
c                       KEYWORDS
c
c     node_domain_decomposition
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, nblocks
      integer isiinput(*), isrinput(*), iscoor(*)  , issolut(*) ,
     +        ismatrix(*), isgmat(*), isrhside(*), isitopol,
     +        iadmin(*), isinfbnd(*), isbndcon(nblocks,*), isgeom(*),
     +        isdensup(*)

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  I    Array of sequence numbers (nblocks) of array densup
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, ntimlv, itrans, ipiinput, ipiinblk,
     +        ichoicem, itimlv, ipsolut,
     +        ndegfd, ndim, neq, icoupled, iseqpres
      logical usebndcn

c     i         General loop variable
c     iblock    Block sequence number
c     ichoicem  Choice parameter multi block
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipsolut   Starting address of array solut in buffer
c     iseqpres  Sequence number of pressure equation
c     itimlv    Loop variable with respect to a given time level
c     itrans    reserved
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     neq       Number of equations
c     ntimlv    Number of time levels
c     usebndcn  If true boundary conditions are used in the extrapolation
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISFILVSO  Extrapolate solution at outer boundaries
c     ISINGETP  get the pointer corresponding to an integer array
c     ISLINSOL  Solve subdomain equations
c     ISLSOLPR  print the solution just after the computation by the linear
c               solver
c     ISMBLK    Performs the local transport of multi-block information
c               within a node
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c      for iblock = 1 (1) nblocks do
c
c         if block(iblock) on this node  then
c
c           solve equations for this block
c
c           extrapolate solution to virtual points
c
c           if sequential algorithm then
c
c              perform multiblock transport
c
c      end do
c
c      if parallel algorithm then
c
c         perform multiblock transport
c
c      end if
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('issdd')

c     --- get information whether a given equation (momentum or
c         transport) is completely skipped

      call isactive( isiinput(1) )
      ipiinput = isingetp( isiinput(1) )
      ndegfd   = ibuffr( ipiinput -1 + 55 )
      ndim     = ibuffr( ipiinput -1 + 57 )
      icoupled = ibuffr( ipiinput -1 + 76 )
      if ( icoupled.eq.0 ) then
         neq = ndegfd - ndim + 1
         iseqpres = 2
      else
         neq = ndegfd
         iseqpres = ndim + 1
      end if

c     --- get ntimlv and choice parameter for multi block

      ntimlv   = ibuffr( ipiinput + 64 -1 )

      ipiinblk = ibuffr( ipiinput + 7  -1 )
      ichoicem = ibuffr( ipiinput + ipiinblk -1 )

c     --- solve equations

      do iblock = 1, nblocks

         iblockrf = iblock
         if( iadmin(iblock).eq.inodenr) then

            call islinsol ( iequa, iblock, nblocks, neq,
     +                      isiinput(iblock), isrinput(iblock),
     +                      iscoor(iblock)  , issolut(iblock) ,
     +                      ismatrix,         isgmat(iblock)  ,
     +                      isrhside(iblock), isitopol,
     +                      isgeom(iblock), isdensup(iblock))

            if ( nerror.ne.0 ) go to 1000

            if ( ichoicem.eq.0 ) then

c           --- sequential algorithm

               if ( iequa.lt.iseqpres ) then
                  itrans = 1
               else
                  itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
               end if

               call isactive(isiinput(iblock))
               call isactive(issolut(iblock))
               call isactive(isitopol)
               do i = 1, nblocks
                  if ( issolut(i).ne.0 )
     +               call isactive(issolut(i))
               end do

               if ( nerror.ne.0 ) go to 1000

c              --- ismblk is called with -ntimlv to indicate that nvirtual
c                  cells must be copied

               if ( nblocks.gt.1 )
     +            call ismblk ( iblock, itrans, -ntimlv, isiinput,
     +                          issolut, iadmin, isitopol )
               if ( nerror.ne.0 ) go to 1000

            end if

         end if

      end do

      if ( ichoicem.eq.1 ) then

         do iblock = 1, nblocks

            iblockrf = iblock
            if( iadmin(iblock).eq.inodenr) then

c           --- parallel algorithm

               if ( iequa.lt.iseqpres ) then
                  itrans = 1
               else
                  itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
               end if

               call isactive(isiinput(iblock))
               call isactive(issolut(iblock))
               call isactive(isitopol)
               do i = 1, nblocks
                  if ( issolut(i).ne.0 )
     +               call isactive(issolut(i))
               end do

               if ( nerror.ne.0 ) go to 1000
               call ismblk ( iblock, itrans, ntimlv, isiinput,
     +                       issolut, iadmin, isitopol )
               if ( nerror.ne.0 ) go to 1000
            end if

         end do

      end if

c     --- extrapolate solution to virtual points

      do iblock = 1, nblocks

         iblockrf = iblock
         if( iadmin(iblock).eq.inodenr) then

            if ( iequa.lt.iseqpres ) then
               itrans = iequa
            else
               itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
            end if

            itimlv = 2
            if ( nerror.ne.0 ) go to 1000

            usebndcn = iequa.ne.iseqpres
            call isfilvso ( iblock, itrans, itimlv, nblocks,
     +                      isiinput(iblock), issolut(iblock),
     +                      isitopol, isinfbnd(iblock),
     +                      isbndcon(iblock,iequa), icoupled,
     +                      usebndcn )
            if ( nblocks.le.1 ) then

c           --- Printing of the solution is only performed for single block

               call isactive(isiinput(iblock))
               call isactive(issolut(iblock))
               ipiinput = isingetp(isiinput(iblock))
               ipsolut  = isdpgetp(issolut(iblock) )
               call islsolpr ( iequa, ibuffr(ipiinput),
     +                         buffer(ipsolut) )
            end if

            if ( nerror.ne.0 ) go to 1000

         end if

      end do

1000  call iserclos('issdd')

      end
