      subroutine issnddnd(itrans, ilev,   isiinput, istraq,
     +                    iadmin, isitopol)
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.4  03-03-1997  Kees K.: extension 3D
c     version      1.3  14-01-1997  Kees K.: arraylength changed for
c                                            wesbeek discretization
c     version      1.2  13-12-1996  Kees K.: idiscret added
c     version      1.1  22-11-04  Erik: extended for coefs
c                                       changed solut into traq
c     version      1.0  03-05-1994
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Send certain virtual unknowns (itrans,ilev) of the local blocks
c     to the host using subroutine issend.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ilev, isitopol, istraq(*), itrans, iadmin(*),
     +      isiinput(*)

c     iadmin    i    reserved
c     ilev      i    Time level of the multi-block transport of information
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     istraq    i    Array of sequence numbers (nblocks) of array traq
c     itrans    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  iblock, issmsg, ipiinput, ipitopol,
     +      ipnsubf, ipsmsg, ipsubf, lensmsg,
     +      maxnsubf, nblocks, nfaces, ni, nj, nk,
     +      ipiindsc, idiscret

c     iblock    Block sequence number
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsmsg    Starting address of a submessage array
c     ipsubf    Starting address of the subfaces array
c     issmsg    Sequence number of  a submessage array
c     lensmsg   Upper bound for the length of a submessage array
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nblocks   Number of blocks
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all permanent arrays
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISSNND02  Does all the work for this subroutine
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     DO for all internal subfaces of the local blocks
c        Get virtual unknowns from this segment
c        Send virtual unknowns to the host
c     ENDDO
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('issnddnd')

c     --- compute nblocks, nfaces and maxnsubf

      call isactive(isitopol)
      ipitopol     = isingetp(isitopol)
      nblocks      = ibuffr(ipitopol + 1 - 1)
      nfaces       = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)

c     --- create an array for storing a submessage and activate some arrays

      lensmsg = 0

      do iblock = 1, nblocks
         if ( iadmin(iblock).eq.inodenr ) then

            call isactive(isiinput(iblock))
            call isactive(istraq(iblock))
            ipiinput = isingetp(isiinput(iblock))
            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            if ( nk.eq.0 ) then
               lensmsg  = max(lensmsg, 3*(max(ni,nj)+1))
            else
               lensmsg  = max(lensmsg, 3*(max(ni,nj,nk)+1)*
     +                                   (max(ni,nj,nk)+1) )
            end if
         end if
      end do

      lensmsg = lensmsg + 1
      issmsg = 0
      call isdcreat(issmsg, lensmsg, 'smsg' )

c     --- compute pointers

      ipitopol     = isingetp(isitopol)
      maxnsubf     = ibuffr(ipitopol + 3 - 1)
      ipnsubf      = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf       = ibuffr(ipitopol+6-1) + ipitopol - 1
      ipsmsg       = isdpgetp(issmsg)

      ipiindsc     = ibuffr(ipiinput-1+6)
      idiscret     = ibuffr(ipiinput-1+ipiindsc+1)

      call issnnd02(itrans, ilev, nblocks, nfaces, maxnsubf,
     +              idiscret, isiinput, istraq, iadmin,
     +              ibuffr(ipnsubf), ibuffr(ipsubf), buffer(ipsmsg))

      call isdelarr(issmsg)
      call isdcrpri

      call iserclos('issnddnd')

      end
