      subroutine luy3 ( mat3, luymat, nxa, nya, nza,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    10-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c     LUy3 calculates the LU decomposition for the tridiagonal blocks in
c     y direction
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nxa, nya, nza
      double precision
     +mat3(1-sl1:nxa+sl2,1-sl3:nya+sl4,
     +1-sl5:nza+sl6, 9),
     +luymat(1-sl1:nxa+sl2,1-sl3:nya+sl4,
     +1-sl5:nza+sl6, 3)

c     luymat    o    LU dec of "diagonal" blocks vert numbering
c     mat3      i    The nx x ny x nz x 27  coefficient matrix of the
c                    discrete Laplacian
c     nxa       i    Number of grid x points on this level
c     nya       i    Number of grid y points on this level
c     nza       i    Number of grid z points on this level
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixa, iya, iza

c     diag      Diagonal index in tridiag matrix (module mgpreset)
c     ixa       Counter to number of grid x points on this level
c     iya       Counter to number of grid y points on this level
c     iza       Counter to number of grid z points on this level
c     p101      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     sub       Subdiagonal index in tridiagonal matrix (module
c               mgpreset)
c     sup       Superdiagonal index in tridiagonal matrix (module
c               mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c  trivial
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      do iza = 1, nza
        do ixa = 1, nxa
          luymat (ixa, 1, iza, DIAG) = mat3 (ixa, 1, iza, p111)
          luymat (ixa, 1, iza, SUP) = mat3 (ixa, 1, iza, p121)
          do iya = 2, nya
            luyMat (ixa, iya, iza, SUB) = mat3 (ixa, iya, iza, p101)/
     +             luyMat (ixa, iya - 1, iza, DIAG)
            luymat (ixa, iya, iza, DIAG) = mat3 (ixa, iya, iza, p111) -
     +         luymat (ixa, iya-1, iza, SUP)*luymat (ixa, iya, iza, SUB)
            luymat (ixa, iya, iza, SUP) = mat3 (ixa, iya, iza, p121)
          end do
        end do
      end do
      end
