      subroutine RAPy (finemat, coarsmat, nxf, nyf, sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.2    15-05-97
c         New cell centered R & P
c         nxc = nxf/2 + 1 instead of (nxf + 1)/2
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculate RAP matrix with semicoarsening in y-direction
c   centering shall be decided by the parity of nyf.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4
      integer nxf, nyf
      double precision
     +finemat(1-sl1:nxf+sl2,1-sl3:nyf+sl4,9),
     + coarsmat (nxf, nyf/2+1, 9)

c     coarsmat  i    Matrix on coarse level
c     finemat   i    Matrix on fine level
c     nxf       Number of grid x points on fine grid
c     nyf       Number of grid y points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, iyc, nyc, centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     centring  0 = cellcentering, 1 = vertex centering
c     ixf       Counter to number of grid x points on fine grid
c     iyc       Counter to number of grid y points on coarse grid
c     iyf       Counter to number of grid y points on fine grid
c     nyc       Number of grid y points on coarse grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nyc = nyf/2 + 1
      centring = mod (nyf, 2)
      if (centring.eq.CELLC) then

c     --- BEGIN Case cell centered

      aa0 = 0.25d0
      aa1 = 0.75d0
      do ixf = 1, nxf

c     --- stencil lower row

        coarsmat (ixf, 1, p00) = 0d0
        coarsmat (ixf, 1, p10) = 0d0
        coarsmat (ixf, 1, p20) = 0d0

        do iyc = 2, nyc-1
          iyf = 2 * iyc - 2
          coarsmat (ixf , iyc, p00) = aa1 * finemat (ixf , iyf, p00) +
     +       aa0 * (finemat (ixf, iyf, p01) + finemat (ixf, iyf+1, p00))
          coarsmat (ixf , iyc, p10) = aa1 * finemat (ixf , iyf, p10) +
     +       aa0 * (finemat (ixf, iyf, p11) + finemat (ixf, iyf+1, p10))
          coarsmat (ixf , iyc, p20) = aa1 * finemat (ixf , iyf, p20) +
     +       aa0 * (finemat (ixf, iyf, p21) + finemat (ixf, iyf+1, p20))
        end do

          coarsmat (ixf , nyc, p00) = aa1 * finemat (ixf , nyf, p00) +
     +       aa0 * finemat (ixf, nyf, p01) 
          coarsmat (ixf , nyc, p10) = aa1 * finemat (ixf , nyf, p10) +
     +       aa0 * finemat (ixf, nyf, p11) 
          coarsmat (ixf , nyc, p20) = aa1 * finemat (ixf , nyf, p20) +
     +       aa0 * finemat (ixf, nyf, p21) 

c       --- end stencil low row

c           --- stencil middle row

          coarsmat (ixf , 1, p01) = 
     +      aa0 * finemat (ixf, 1, p02) + aa1 * finemat (ixf, 1, p01)
          coarsmat (ixf , 1, p11) = 
     +      aa0 * finemat (ixf, 1, p12) + aa1 * finemat (ixf, 1, p11)
          coarsmat (ixf , 1, p21) = 
     +      aa0 * finemat (ixf, 1, p22) + aa1 * finemat (ixf, 1, p21)

        do iyc = 2, nyc-1
          iyf = 2 * iyc - 2
          coarsmat (ixf , iyc, p01) = aa0 * finemat (ixf , iyf, p00) +
     +       aa1 * (finemat (ixf, iyf, p01) + finemat (ixf, iyf+1, p00))
     +      +aa0 * finemat (ixf , iyf+1, p02) +
     +       aa1 * (finemat (ixf, iyf, p02) + finemat (ixf, iyf+1, p01))
          coarsmat (ixf , iyc, p11) = aa0 * finemat (ixf , iyf, p10) +
     +       aa1 * (finemat (ixf, iyf, p11) + finemat (ixf, iyf+1, p10))
     +      +aa0 * finemat (ixf , iyf+1, p12) +
     +       aa1 * (finemat (ixf, iyf, p12) + finemat (ixf, iyf+1, p11))
          coarsmat (ixf , iyc, p21) = aa0 * finemat (ixf , iyf, p20) +
     +       aa1 * (finemat (ixf, iyf, p21) + finemat (ixf, iyf+1, p20))
     +      +aa0 * finemat (ixf , iyf+1, p22) +
     +       aa1 * (finemat (ixf, iyf, p22) + finemat (ixf, iyf+1, p21))
        end do

          coarsmat (ixf , nyc, p01) = aa0 * finemat (ixf , nyf, p00) +
     +       aa1 * finemat (ixf, nyf, p01)
          coarsmat (ixf , nyc, p11) = aa0 * finemat (ixf , nyf, p10) +
     +       aa1 * finemat (ixf, nyf, p11)
          coarsmat (ixf , nyc, p21) = aa0 * finemat (ixf , nyf, p20) +
     +       aa1 * finemat (ixf, nyf, p21)

c       --- end stencil middle row

c           --- stencil upper row

          coarsmat (ixf , 1, p02) = aa1 * finemat (ixf , 1, p02) +
     +       aa0 *  finemat (ixf, 1, p01)
          coarsmat (ixf , 1, p12) = aa1 * finemat (ixf , 1, p12) +
     +       aa0 *  finemat (ixf, 1, p11)
          coarsmat (ixf , 1, p22) = aa1 * finemat (ixf , 1, p22) +
     +       aa0 *  finemat (ixf, 1, p21)

        do iyc = 2, nyc - 1
          iyf = 2 * iyc - 2
          coarsmat (ixf , iyc, p02) = aa1 * finemat (ixf , iyf+1, p02) +
     +       aa0 * (finemat (ixf, iyf, p02) + finemat (ixf, iyf+1, p01))
          coarsmat (ixf , iyc, p12) = aa1 * finemat (ixf , iyf+1, p12) +
     +       aa0 * (finemat (ixf, iyf, p12) + finemat (ixf, iyf+1, p11))
          coarsmat (ixf , iyc, p22) = aa1 * finemat (ixf , iyf+1, p22) +
     +       aa0 * (finemat (ixf, iyf, p22) + finemat (ixf, iyf+1, p21))
        end do
        coarsmat (ixf, nyc, p02) = 0d0
        coarsmat (ixf, nyc, p12) = 0d0
        coarsmat (ixf, nyc, p22) = 0d0

c       --- end stencil upper row

      end do

c--- END Case cell centered

      else

c--- BEGIN Case vertex centered
      aa0 = 0.25d0
      aa1 = 0.5d0

c     --- first we do all *internal* lines

c         --- BEGIN internal lines

      do iyc = 2, nyc - 1
        iyf = 2*iyc - 1

c--- stencil center column
        do ixf = 1, nxf
          coarsmat (ixf, iyc, p10) = aa1 * (finemat (ixf, iyf-1, p10)
     +     + finemat (ixf, iyf, p10)) +
     +     aa0*finemat (ixf, iyf-1, p11)
          coarsmat (ixf, iyc, p12) = aa1 * (finemat (ixf, iyf+1, p12)
     +     + finemat (ixf, iyf, p12)) +
     +     aa0*finemat (ixf, iyf+1, p11)
          coarsmat (ixf, iyc, p11) = finemat (ixf, iyf, p11) +
     +     aa1 * (finemat (ixf, iyf-1, p12) + finemat (ixf, iyf, p10))+
     +     aa1 * (finemat (ixf, iyf+1, p10) + finemat (ixf, iyf, p12))+
     +     aa0*finemat (ixf, iyf-1, p11) +
     +     aa0*finemat (ixf, iyf+1, p11)
        end do

c--- stencil left column
        do ixf = 2, nxf
          coarsmat (ixf, iyc, p00) = aa1 * (finemat (ixf, iyf-1, p00)
     +     + finemat (ixf, iyf, p00)) +
     +     aa0*finemat (ixf, iyf-1, p01)
          coarsmat (ixf, iyc, p02) = aa1 * (finemat (ixf, iyf+1, p02)
     +     + finemat (ixf, iyf, p02)) +
     +     aa0*finemat (ixf, iyf+1, p01)
          coarsmat (ixf, iyc, p01) = finemat (ixf, iyf, p01) +
     +     aa1 * (finemat (ixf, iyf-1, p02) + finemat (ixf, iyf, p00))+
     +     aa1 * (finemat (ixf, iyf+1, p00) + finemat (ixf, iyf, p02))+
     +     aa0*finemat (ixf, iyf-1, p01) +
     +     aa0*finemat (ixf, iyf+1, p01)
        end do

c--  region left boundary
        coarsmat (1, iyc, p00) = 0d0
        coarsmat (1, iyc, p02) = 0d0
        coarsmat (1, iyc, p01) = 0d0

c-- stencil right column

        do ixf = 1, nxf-1
          coarsmat (ixf, iyc, p20) = aa1 * (finemat (ixf, iyf-1, p20)
     +     + finemat (ixf, iyf, p20)) +
     +     aa0*finemat (ixf, iyf-1, p21)
          coarsmat (ixf, iyc, p22) = aa1 * (finemat (ixf, iyf+1, p22)
     +     + finemat (ixf, iyf, p22)) +
     +     aa0*finemat (ixf, iyf+1, p21)
          coarsmat (ixf, iyc, p21) = finemat (ixf, iyf, p21) +
     +     aa1 * (finemat (ixf, iyf-1, p22) + finemat (ixf, iyf, p20))+
     +     aa1 * (finemat (ixf, iyf+1, p20) + finemat (ixf, iyf, p22))+
     +     aa0*finemat (ixf, iyf-1, p21) +
     +     aa0*finemat (ixf, iyf+1, p21)
        end do

c       --- region right boundary

        coarsmat (nxf, iyc, p20) =0d0
        coarsmat (nxf, iyc, p21) =0d0
        coarsmat (nxf, iyc, p22) =0d0
      end do

c     --- END internal lines

c         --- High and low region boundary still have to be done
c         --- low region boundary : cm (ixf, 1, pi0) = 0
c         rest normal
c         BEGIN low region boundary
c         --- stencil left column
c         --- region left boundary

      coarsmat (1, 1, p00) = 0d0
      coarsmat (1, 1, p01) = 0d0
      coarsmat (1, 1, p02) = 0d0

      do ixf = 2, nxf
        coarsmat (ixf, 1, p00) =0d0
        coarsmat (ixf, 1, p01) = finemat (ixf, 1, p01) +
     +     aa1 * (finemat (ixf, 1+1, p00) + finemat (ixf, 1, p02))+
     +     aa0*finemat (ixf, 1+1, p01)
        coarsmat (ixf, 1, p02) = aa1 * (finemat (ixf, 1+1, p02)
     +     + finemat (ixf, 1, p02)) +
     +     aa0*finemat (ixf, 1+1, p01)
      end do

c--- stencil center column
      do ixf = 1, nxf
        coarsmat (ixf, 1, p10) =0d0
        coarsmat (ixf, 1, p11) = finemat (ixf, 1, p11) +
     +     aa1 * (finemat (ixf, 1+1, p10) + finemat (ixf, 1, p12))+
     +     aa0*finemat (ixf, 1+1, p11)
        coarsmat (ixf, 1, p12) = aa1 * (finemat (ixf, 1+1, p12)
     +     + finemat (ixf, 1, p12)) +
     +     aa0*finemat (ixf, 1+1, p11)
      end do

c--- stencil right column
      do ixf = 1, nxf-1
        coarsmat (ixf, 1, p20) =0d0
        coarsmat (ixf, 1, p21) = finemat (ixf, 1, p21) +
     +     aa1 * (finemat (ixf, 1+1, p20) + finemat (ixf, 1, p22))+
     +     aa0*finemat (ixf, 1+1, p21)
        coarsmat (ixf, 1, p22) = aa1 * (finemat (ixf, 1+1, p22)
     +     + finemat (ixf, 1, p22)) +
     +     aa0*finemat (ixf, 1+1, p21)
      end do

c--- region right boundary
      coarsmat (nxf, 1, p20) = 0d0
      coarsmat (nxf, 1, p21) = 0d0
      coarsmat (nxf, 1, p22) = 0d0

c     --- END low region boundary

c         --- high region boundary
c         cm (ixf, nyc, pi2) = 0d0
c         BEGIN high region boundary
c         --- stencil left column
c         --- region left boundary

      coarsmat (1, nyc, p00) = 0d0
      coarsmat (1, nyc, p01) = 0d0
      coarsmat (1, nyc, p02) = 0d0
      do ixf = 2, nxf
          coarsmat (ixf, nyc, p00) = aa1 * (finemat (ixf, nyf-1, p00)
     +     + finemat (ixf, nyf, p00)) +
     +     aa0*finemat (ixf, nyf-1, p01)
          coarsmat (ixf, nyc, p02) = 0d0
          coarsmat (ixf, nyc, p01) = finemat (ixf, nyf, p01) +
     +     aa1 * (finemat (ixf, nyf-1, p02) + finemat (ixf, nyf, p00))+
     +     aa0*finemat (ixf, nyf-1, p01)
      end do

c--- stencil center column
      do ixf = 1, nxf
          coarsmat (ixf, nyc, p10) = aa1 * (finemat (ixf, nyf-1, p10)
     +     + finemat (ixf, nyf, p10)) +
     +     aa0*finemat (ixf, nyf-1, p11)
          coarsmat (ixf, nyc, p12) = 0d0
          coarsmat (ixf, nyc, p11) = finemat (ixf, nyf, p11) +
     +     aa1 * (finemat (ixf, nyf-1, p12) + finemat (ixf, nyf, p10))+
     +     aa0*finemat (ixf, nyf-1, p11)
      end do

c     --- stencil right column

      do ixf = 1, nxf - 1
          coarsmat (ixf, nyc, p20) = aa1 * (finemat (ixf, nyf-1, p20)
     +     + finemat (ixf, nyf, p20)) +
     +     aa0*finemat (ixf, nyf-1, p21)
          coarsmat (ixf, nyc, p22) = 0d0
          coarsmat (ixf, nyc, p21) = finemat (ixf, nyf, p21) +
     +     aa1 * (finemat (ixf, nyf-1, p22) + finemat (ixf, nyf, p20))+
     +     aa0*finemat (ixf, nyf-1, p21)
      end do

c     --- region right boundary

      coarsmat (nxf, nyc, p20) = 0d0
      coarsmat (nxf, nyc, p21) = 0d0
      coarsmat (nxf, nyc, p22) = 0d0

c     --- END high region boundary
c         --  END Case vertex centered

      end if
      end
