      subroutine RAPz3 ( finemat, coarsmat, nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.2    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   12-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculate RAP matrix with semicoarsening in z-direction
c   centring shall be decided by the parity of nzf.
c **********************************************************************
c
c                       KEYWORDS
c
c    multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nxf, nyf, nzf
      double precision
     +finemat(1-sl1:nxf+sl2,1-sl3:nyf+sl4,
     + 1-sl5:nzf+sl6, 27),
     + coarsmat (nxf, nyf, nzf/2+1, 27)

c     coarsmat  o    Matrix on coarse level
c     finemat   i    Matrix on fine level
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     nzf       i    Number of grid z points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, izf, izc, nzc, centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     cellc     Cell centered (module mgpreset)
c     centring  0 = cellcentering, 1 = vertex centering
c     ixf       Counter to number of grid x points on fine grid
c     iyf       Counter to number of grid y points on fine grid
c     izc       Counter to number of grid z points on coarse grid
c     izf       Counter to number of grid z points on fine grid
c     nzc       Number of grid z points on coarse grid
c     p000      Index of point in stencil 3D (module mgpreset)
c     p001      Index of point in stencil 3D (module mgpreset)
c     p002      Index of point in stencil 3D (module mgpreset)
c     p010      Index of point in stencil 3D (module mgpreset)
c     p011      Index of point in stencil 3D (module mgpreset)
c     p012      Index of point in stencil 3D (module mgpreset)
c     p020      Index of point in stencil 3D (module mgpreset)
c     p021      Index of point in stencil 3D (module mgpreset)
c     p022      Index of point in stencil 3D (module mgpreset)
c     p100      Index of point in stencil 3D (module mgpreset)
c     p101      Index of point in stencil 3D (module mgpreset)
c     p102      Index of point in stencil 3D (module mgpreset)
c     p110      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p112      Index of point in stencil 3D (module mgpreset)
c     p120      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     p122      Index of point in stencil 3D (module mgpreset)
c     p200      Index of point in stencil 3D (module mgpreset)
c     p201      Index of point in stencil 3D (module mgpreset)
c     p202      Index of point in stencil 3D (module mgpreset)
c     p210      Index of point in stencil 3D (module mgpreset)
c     p211      Index of point in stencil 3D (module mgpreset)
c     p212      Index of point in stencil 3D (module mgpreset)
c     p220      Index of point in stencil 3D (module mgpreset)
c     p221      Index of point in stencil 3D (module mgpreset)
c     p222      Index of point in stencil 3D (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nzc = nzf/2 + 1
      centring = mod (nzf, 2)
      if (centring.eq.CELLC) then

c     --- BEGIN Case cell centered

      aa0 = 0.25d0
      aa1 = 0.75d0
      do iyf = 1, nyf
      do ixf = 1, nxf

c     --- stencil lower plane

        coarsmat (ixf,iyf,1,p000)=0d0
        coarsmat (ixf,iyf,1,p100)=0d0
        coarsmat (ixf,iyf,1,p200)=0d0
        coarsmat (ixf,iyf,1,p010)=0d0
        coarsmat (ixf,iyf,1,p110)=0d0
        coarsmat (ixf,iyf,1,p210)=0d0
        coarsmat (ixf,iyf,1,p020)=0d0
        coarsmat (ixf,iyf,1,p120)=0d0
        coarsmat (ixf,iyf,1,p220)=0d0
        do izc = 2, nzc-1
          izf = 2 * izc - 2
      coarsmat (ixf,iyf,izc,p000)=aa1*finemat (ixf,iyf,izf,p000)+
     +  aa0*(finemat (ixf,iyf,izf,p001)+finemat (ixf,iyf,izf+1,p000))
      coarsmat (ixf,iyf,izc,p100)=aa1*finemat (ixf,iyf,izf,p100)+
     +  aa0*(finemat (ixf,iyf,izf,p101)+finemat (ixf,iyf,izf+1,p100))
      coarsmat (ixf,iyf,izc,p200)=aa1*finemat (ixf,iyf,izf,p200)+
     +  aa0*(finemat (ixf,iyf,izf,p201)+finemat (ixf,iyf,izf+1,p200))
      coarsmat (ixf,iyf,izc,p010)=aa1*finemat (ixf,iyf,izf,p010)+
     +  aa0*(finemat (ixf,iyf,izf,p011)+finemat (ixf,iyf,izf+1,p010))
      coarsmat (ixf,iyf,izc,p110)=aa1*finemat (ixf,iyf,izf,p110)+
     +  aa0*(finemat (ixf,iyf,izf,p111)+finemat (ixf,iyf,izf+1,p110))
      coarsmat (ixf,iyf,izc,p210)=aa1*finemat (ixf,iyf,izf,p210)+
     +  aa0*(finemat (ixf,iyf,izf,p211)+finemat (ixf,iyf,izf+1,p210))
      coarsmat (ixf,iyf,izc,p020)=aa1*finemat (ixf,iyf,izf,p020)+
     +  aa0*(finemat (ixf,iyf,izf,p021)+finemat (ixf,iyf,izf+1,p020))
      coarsmat (ixf,iyf,izc,p120)=aa1*finemat (ixf,iyf,izf,p120)+
     +  aa0*(finemat (ixf,iyf,izf,p121)+finemat (ixf,iyf,izf+1,p120))
      coarsmat (ixf,iyf,izc,p220)=aa1*finemat (ixf,iyf,izf,p220)+
     +  aa0*(finemat (ixf,iyf,izf,p221)+finemat (ixf,iyf,izf+1,p220))
        end do

      coarsmat (ixf,iyf,nzc,p000)=aa1*finemat (ixf,iyf,nzf,p000)+
     +  aa0*(finemat (ixf,iyf,nzf,p001))
      coarsmat (ixf,iyf,nzc,p100)=aa1*finemat (ixf,iyf,nzf,p100)+
     +  aa0*(finemat (ixf,iyf,nzf,p101))
      coarsmat (ixf,iyf,nzc,p200)=aa1*finemat (ixf,iyf,nzf,p200)+
     +  aa0*(finemat (ixf,iyf,nzf,p201))
      coarsmat (ixf,iyf,nzc,p010)=aa1*finemat (ixf,iyf,nzf,p010)+
     +  aa0*(finemat (ixf,iyf,nzf,p011))
      coarsmat (ixf,iyf,nzc,p110)=aa1*finemat (ixf,iyf,nzf,p110)+
     +  aa0*(finemat (ixf,iyf,nzf,p111))
      coarsmat (ixf,iyf,nzc,p210)=aa1*finemat (ixf,iyf,nzf,p210)+
     +  aa0*(finemat (ixf,iyf,nzf,p211))
      coarsmat (ixf,iyf,nzc,p020)=aa1*finemat (ixf,iyf,nzf,p020)+
     +  aa0*(finemat (ixf,iyf,nzf,p021))
      coarsmat (ixf,iyf,nzc,p120)=aa1*finemat (ixf,iyf,nzf,p120)+
     +  aa0*(finemat (ixf,iyf,nzf,p121))
      coarsmat (ixf,iyf,nzc,p220)=aa1*finemat (ixf,iyf,nzf,p220)+
     +  aa0*(finemat (ixf,iyf,nzf,p221))

c     --- end stencil low plane

c         ---stencil middle plane

          coarsmat (ixf,iyf,1,p001)=
     +     aa1*(finemat (ixf,iyf,1,p000))
     +     +aa0*finemat (ixf,iyf,1,p002)+
     +     aa1*(finemat (ixf,iyf,1,p001))
          coarsmat (ixf,iyf,1,p101)=
     +     aa1*(finemat (ixf,iyf,1,p100))
     +     +aa0*finemat (ixf,iyf,1,p102)+
     +     aa1*(finemat (ixf,iyf,1,p101))
          coarsmat (ixf,iyf,1,p201)=
     +     aa1*(finemat (ixf,iyf,1,p200))
     +     +aa0*finemat (ixf,iyf,1,p202)+
     +     aa1*(finemat (ixf,iyf,1,p201))
          coarsmat (ixf,iyf,1,p011)=
     +     aa1*(finemat (ixf,iyf,1,p010))
     +     +aa0*finemat (ixf,iyf,1,p012)+
     +     aa1*(finemat (ixf,iyf,1,p011))
          coarsmat (ixf,iyf,1,p111)=
     +     aa1*(finemat (ixf,iyf,1,p110))
     +     +aa0*finemat (ixf,iyf,1,p112)+
     +     aa1*(finemat (ixf,iyf,1,p111))
          coarsmat (ixf,iyf,1,p211)=
     +     aa1*(finemat (ixf,iyf,1,p210))
     +     +aa0*finemat (ixf,iyf,1,p212)+
     +     aa1*(finemat (ixf,iyf,1,p211))
          coarsmat (ixf,iyf,1,p021)=
     +     aa1*(finemat (ixf,iyf,1,p020))
     +     +aa0*finemat (ixf,iyf,1,p022)+
     +     aa1*(finemat (ixf,iyf,1,p021))
          coarsmat (ixf,iyf,1,p121)=
     +     aa1*(finemat (ixf,iyf,1,p120))
     +     +aa0*finemat (ixf,iyf,1,p122)+
     +     aa1*(finemat (ixf,iyf,1,p121))
          coarsmat (ixf,iyf,1,p221)=
     +     aa1*(finemat (ixf,iyf,1,p220))
     +     +aa0*finemat (ixf,iyf,1,p222)+
     +     aa1*(finemat (ixf,iyf,1,p221))
        do izc =2,nzc-1
          izf=2*izc -2
          coarsmat (ixf,iyf,izc,p001)=aa0*finemat (ixf,iyf,izf,p000)+
     +     aa1*(finemat (ixf,iyf,izf,p001)+finemat (ixf,iyf,izf+1,p000))
     ++aa0*finemat (ixf,iyf,izf+1,p002)+
     +     aa1*(finemat (ixf,iyf,izf,p002)+finemat (ixf,iyf,izf+1,p001))
          coarsmat (ixf,iyf,izc,p101)=aa0*finemat (ixf,iyf,izf,p100)+
     +     aa1*(finemat (ixf,iyf,izf,p101)+finemat (ixf,iyf,izf+1,p100))
     ++aa0*finemat (ixf,iyf,izf+1,p102)+
     +     aa1*(finemat (ixf,iyf,izf,p102)+finemat (ixf,iyf,izf+1,p101))
          coarsmat (ixf,iyf,izc,p201)=aa0*finemat (ixf,iyf,izf,p200)+
     +     aa1*(finemat (ixf,iyf,izf,p201)+finemat (ixf,iyf,izf+1,p200))
     ++aa0*finemat (ixf,iyf,izf+1,p202)+
     +     aa1*(finemat (ixf,iyf,izf,p202)+finemat (ixf,iyf,izf+1,p201))
          coarsmat (ixf,iyf,izc,p011)=aa0*finemat (ixf,iyf,izf,p010)+
     +     aa1*(finemat (ixf,iyf,izf,p011)+finemat (ixf,iyf,izf+1,p010))
     ++aa0*finemat (ixf,iyf,izf+1,p012)+
     +     aa1*(finemat (ixf,iyf,izf,p012)+finemat (ixf,iyf,izf+1,p011))
          coarsmat (ixf,iyf,izc,p111)=aa0*finemat (ixf,iyf,izf,p110)+
     +     aa1*(finemat (ixf,iyf,izf,p111)+finemat (ixf,iyf,izf+1,p110))
     ++aa0*finemat (ixf,iyf,izf+1,p112)+
     +     aa1*(finemat (ixf,iyf,izf,p112)+finemat (ixf,iyf,izf+1,p111))
          coarsmat (ixf,iyf,izc,p211)=aa0*finemat (ixf,iyf,izf,p210)+
     +     aa1*(finemat (ixf,iyf,izf,p211)+finemat (ixf,iyf,izf+1,p210))
     ++aa0*finemat (ixf,iyf,izf+1,p212)+
     +     aa1*(finemat (ixf,iyf,izf,p212)+finemat (ixf,iyf,izf+1,p211))
          coarsmat (ixf,iyf,izc,p021)=aa0*finemat (ixf,iyf,izf,p020)+
     +     aa1*(finemat (ixf,iyf,izf,p021)+finemat (ixf,iyf,izf+1,p020))
     ++aa0*finemat (ixf,iyf,izf+1,p022)+
     +     aa1*(finemat (ixf,iyf,izf,p022)+finemat (ixf,iyf,izf+1,p021))
          coarsmat (ixf,iyf,izc,p121)=aa0*finemat (ixf,iyf,izf,p120)+
     +     aa1*(finemat (ixf,iyf,izf,p121)+finemat (ixf,iyf,izf+1,p120))
     ++aa0*finemat (ixf,iyf,izf+1,p122)+
     +     aa1*(finemat (ixf,iyf,izf,p122)+finemat (ixf,iyf,izf+1,p121))
          coarsmat (ixf,iyf,izc,p221)=aa0*finemat (ixf,iyf,izf,p220)+
     +     aa1*(finemat (ixf,iyf,izf,p221)+finemat (ixf,iyf,izf+1,p220))
     ++aa0*finemat (ixf,iyf,izf+1,p222)+
     +     aa1*(finemat (ixf,iyf,izf,p222)+finemat (ixf,iyf,izf+1,p221))
        end do

          coarsmat (ixf,iyf,nzc,p001)=aa0*finemat (ixf,iyf,nzf,p000)+
     +     aa1*(finemat (ixf,iyf,nzf,p001))+
     +     aa1*(finemat (ixf,iyf,nzf,p002))
          coarsmat (ixf,iyf,nzc,p101)=aa0*finemat (ixf,iyf,nzf,p100)+
     +     aa1*(finemat (ixf,iyf,nzf,p101))+
     +     aa1*(finemat (ixf,iyf,nzf,p102))
          coarsmat (ixf,iyf,nzc,p201)=aa0*finemat (ixf,iyf,nzf,p200)+
     +     aa1*(finemat (ixf,iyf,nzf,p201))+
     +     aa1*(finemat (ixf,iyf,nzf,p202))
          coarsmat (ixf,iyf,nzc,p011)=aa0*finemat (ixf,iyf,nzf,p010)+
     +     aa1*(finemat (ixf,iyf,nzf,p011))+
     +     aa1*(finemat (ixf,iyf,nzf,p012))
          coarsmat (ixf,iyf,nzc,p111)=aa0*finemat (ixf,iyf,nzf,p110)+
     +     aa1*(finemat (ixf,iyf,nzf,p111))+
     +     aa1*(finemat (ixf,iyf,nzf,p112))
          coarsmat (ixf,iyf,nzc,p211)=aa0*finemat (ixf,iyf,nzf,p210)+
     +     aa1*(finemat (ixf,iyf,nzf,p211))+
     +     aa1*(finemat (ixf,iyf,nzf,p212))
          coarsmat (ixf,iyf,nzc,p021)=aa0*finemat (ixf,iyf,nzf,p020)+
     +     aa1*(finemat (ixf,iyf,nzf,p021))+
     +     aa1*(finemat (ixf,iyf,nzf,p022))
          coarsmat (ixf,iyf,nzc,p121)=aa0*finemat (ixf,iyf,nzf,p120)+
     +     aa1*(finemat (ixf,iyf,nzf,p121))+
     +     aa1*(finemat (ixf,iyf,nzf,p122))
          coarsmat (ixf,iyf,nzc,p221)=aa0*finemat (ixf,iyf,nzf,p220)+
     +     aa1*(finemat (ixf,iyf,nzf,p221))+
     +     aa1*(finemat (ixf,iyf,nzf,p222))

c         --- end stencil middle plane

c             ---stencil upper plane

          coarsmat (ixf,iyf,1,p002)=aa1*finemat (ixf,iyf,1,p002)+
     +     aa0*(finemat (ixf,iyf,1,p001))
          coarsmat (ixf,iyf,1,p102)=aa1*finemat (ixf,iyf,1,p102)+
     +     aa0*(finemat (ixf,iyf,1,p101))
          coarsmat (ixf,iyf,1,p202)=aa1*finemat (ixf,iyf,1,p202)+
     +     aa0*(finemat (ixf,iyf,1,p201))
          coarsmat (ixf,iyf,1,p012)=aa1*finemat (ixf,iyf,1,p012)+
     +     aa0*(finemat (ixf,iyf,1,p011))
          coarsmat (ixf,iyf,1,p112)=aa1*finemat (ixf,iyf,1,p112)+
     +     aa0*(finemat (ixf,iyf,1,p111))
          coarsmat (ixf,iyf,1,p212)=aa1*finemat (ixf,iyf,1,p212)+
     +     aa0*(finemat (ixf,iyf,1,p211))
          coarsmat (ixf,iyf,1,p022)=aa1*finemat (ixf,iyf,1,p022)+
     +     aa0*(finemat (ixf,iyf,1,p021))
          coarsmat (ixf,iyf,1,p122)=aa1*finemat (ixf,iyf,1,p122)+
     +     aa0*(finemat (ixf,iyf,1,p121))
          coarsmat (ixf,iyf,1,p222)=aa1*finemat (ixf,iyf,1,p222)+
     +     aa0*(finemat (ixf,iyf,1,p221))

        do izc =2,nzc -1
          izf=2*izc -2
          coarsmat (ixf,iyf,izc,p002)=aa1*finemat (ixf,iyf,izf+1,p002)+
     +     aa0*(finemat (ixf,iyf,izf,p002)+finemat (ixf,iyf,izf+1,p001))
          coarsmat (ixf,iyf,izc,p102)=aa1*finemat (ixf,iyf,izf+1,p102)+
     +     aa0*(finemat (ixf,iyf,izf,p102)+finemat (ixf,iyf,izf+1,p101))
          coarsmat (ixf,iyf,izc,p202)=aa1*finemat (ixf,iyf,izf+1,p202)+
     +     aa0*(finemat (ixf,iyf,izf,p202)+finemat (ixf,iyf,izf+1,p201))
          coarsmat (ixf,iyf,izc,p012)=aa1*finemat (ixf,iyf,izf+1,p012)+
     +     aa0*(finemat (ixf,iyf,izf,p012)+finemat (ixf,iyf,izf+1,p011))
          coarsmat (ixf,iyf,izc,p112)=aa1*finemat (ixf,iyf,izf+1,p112)+
     +     aa0*(finemat (ixf,iyf,izf,p112)+finemat (ixf,iyf,izf+1,p111))
          coarsmat (ixf,iyf,izc,p212)=aa1*finemat (ixf,iyf,izf+1,p212)+
     +     aa0*(finemat (ixf,iyf,izf,p212)+finemat (ixf,iyf,izf+1,p211))
          coarsmat (ixf,iyf,izc,p022)=aa1*finemat (ixf,iyf,izf+1,p022)+
     +     aa0*(finemat (ixf,iyf,izf,p022)+finemat (ixf,iyf,izf+1,p021))
          coarsmat (ixf,iyf,izc,p122)=aa1*finemat (ixf,iyf,izf+1,p122)+
     +     aa0*(finemat (ixf,iyf,izf,p122)+finemat (ixf,iyf,izf+1,p121))
          coarsmat (ixf,iyf,izc,p222)=aa1*finemat (ixf,iyf,izf+1,p222)+
     +     aa0*(finemat (ixf,iyf,izf,p222)+finemat (ixf,iyf,izf+1,p221))
        end do
        coarsmat (ixf,iyf,nzc,p002)=0d0
        coarsmat (ixf,iyf,nzc,p102)=0d0
        coarsmat (ixf,iyf,nzc,p202)=0d0
        coarsmat (ixf,iyf,nzc,p012)=0d0
        coarsmat (ixf,iyf,nzc,p112)=0d0
        coarsmat (ixf,iyf,nzc,p212)=0d0
        coarsmat (ixf,iyf,nzc,p022)=0d0
        coarsmat (ixf,iyf,nzc,p122)=0d0
        coarsmat (ixf,iyf,nzc,p222)=0d0

c       --- end stencil upper row

      end do
      end do

c---END Case cell centered

      else

c--- BEGINCase vertex centered
      aa0=0.25d0
      aa1=0.5d0

c     --- first we do all *internal*lines

c         ---BEGINinternal lines

      do izc =2,nzc -1
        izf=2*izc -1

c---stencil center plane
      do iyf = 1, nyf
        do ixf=1,nxf
      coarsmat (ixf,iyf,izc,p100)=aa1*(finemat (ixf,iyf,izf-1,p100)
     ++finemat (ixf,iyf,izf,p100))+
     +aa0*finemat (ixf,iyf,izf-1,p101)
      coarsmat (ixf,iyf,izc,p102)=aa1*(finemat (ixf,iyf,izf+1,p102)
     ++finemat (ixf,iyf,izf,p102))+
     +aa0*finemat (ixf,iyf,izf+1,p101)
      coarsmat (ixf,iyf,izc,p101)=finemat (ixf,iyf,izf,p101)+
     +aa1*(finemat (ixf,iyf,izf-1,p102)+finemat (ixf,iyf,izf,p100))+
     +aa1*(finemat (ixf,iyf,izf+1,p100)+finemat (ixf,iyf,izf,p102))+
     +aa0*finemat (ixf,iyf,izf-1,p101)+
     +aa0*finemat (ixf,iyf,izf+1,p101)
      coarsmat (ixf,iyf,izc,p110)=aa1*(finemat (ixf,iyf,izf-1,p110)
     ++finemat (ixf,iyf,izf,p110))+
     +aa0*finemat (ixf,iyf,izf-1,p111)
      coarsmat (ixf,iyf,izc,p112)=aa1*(finemat (ixf,iyf,izf+1,p112)
     ++finemat (ixf,iyf,izf,p112))+
     +aa0*finemat (ixf,iyf,izf+1,p111)
      coarsmat (ixf,iyf,izc,p111)=finemat (ixf,iyf,izf,p111)+
     +aa1*(finemat (ixf,iyf,izf-1,p112)+finemat (ixf,iyf,izf,p110))+
     +aa1*(finemat (ixf,iyf,izf+1,p110)+finemat (ixf,iyf,izf,p112))+
     +aa0*finemat (ixf,iyf,izf-1,p111)+
     +aa0*finemat (ixf,iyf,izf+1,p111)
      coarsmat (ixf,iyf,izc,p120)=aa1*(finemat (ixf,iyf,izf-1,p120)
     ++finemat (ixf,iyf,izf,p120))+
     +aa0*finemat (ixf,iyf,izf-1,p121)
      coarsmat (ixf,iyf,izc,p122)=aa1*(finemat (ixf,iyf,izf+1,p122)
     ++finemat (ixf,iyf,izf,p122))+
     +aa0*finemat (ixf,iyf,izf+1,p121)
      coarsmat (ixf,iyf,izc,p121)=finemat (ixf,iyf,izf,p121)+
     +aa1*(finemat (ixf,iyf,izf-1,p122)+finemat (ixf,iyf,izf,p120))+
     +aa1*(finemat (ixf,iyf,izf+1,p120)+finemat (ixf,iyf,izf,p122))+
     +aa0*finemat (ixf,iyf,izf-1,p121)+
     +aa0*finemat (ixf,iyf,izf+1,p121)
        end do

c---stencil left plane
        do ixf=1,nxf
          coarsmat (ixf,iyf,izc,p000)=aa1*(finemat (ixf,iyf,izf-1,p000)
     +    +finemat (ixf,iyf,izf,p000))+
     +aa0*finemat (ixf,iyf,izf-1,p001)
          coarsmat (ixf,iyf,izc,p002)=aa1*(finemat (ixf,iyf,izf+1,p002)
     +    +finemat (ixf,iyf,izf,p002))+
     +aa0*finemat (ixf,iyf,izf+1,p001)
          coarsmat (ixf,iyf,izc,p001)=finemat (ixf,iyf,izf,p001)+
     +    aa1*(finemat (ixf,iyf,izf-1,p002)+finemat (ixf,iyf,izf,p000))+
     +    aa1*(finemat (ixf,iyf,izf+1,p000)+finemat (ixf,iyf,izf,p002))+
     +    aa0*finemat (ixf,iyf,izf-1,p001)+
     +    aa0*finemat (ixf,iyf,izf+1,p001)
          coarsmat (ixf,iyf,izc,p010)=aa1*(finemat (ixf,iyf,izf-1,p010)
     +    +finemat (ixf,iyf,izf,p010))+
     +aa0*finemat (ixf,iyf,izf-1,p011)
          coarsmat (ixf,iyf,izc,p012)=aa1*(finemat (ixf,iyf,izf+1,p012)
     +    +finemat (ixf,iyf,izf,p012))+
     +aa0*finemat (ixf,iyf,izf+1,p011)
          coarsmat (ixf,iyf,izc,p011)=finemat (ixf,iyf,izf,p011)+
     +    aa1*(finemat (ixf,iyf,izf-1,p012)+finemat (ixf,iyf,izf,p010))+
     +    aa1*(finemat (ixf,iyf,izf+1,p010)+finemat (ixf,iyf,izf,p012))+
     +    aa0*finemat (ixf,iyf,izf-1,p011)+
     +    aa0*finemat (ixf,iyf,izf+1,p011)
          coarsmat (ixf,iyf,izc,p020)=aa1*(finemat (ixf,iyf,izf-1,p020)
     +    +finemat (ixf,iyf,izf,p020))+
     +aa0*finemat (ixf,iyf,izf-1,p021)
          coarsmat (ixf,iyf,izc,p022)=aa1*(finemat (ixf,iyf,izf+1,p022)
     +    +finemat (ixf,iyf,izf,p022))+
     +aa0*finemat (ixf,iyf,izf+1,p021)
          coarsmat (ixf,iyf,izc,p021)=finemat (ixf,iyf,izf,p021)+
     +    aa1*(finemat (ixf,iyf,izf-1,p022)+finemat (ixf,iyf,izf,p020))+
     +    aa1*(finemat (ixf,iyf,izf+1,p020)+finemat (ixf,iyf,izf,p022))+
     +    aa0*finemat (ixf,iyf,izf-1,p021)+
     +    aa0*finemat (ixf,iyf,izf+1,p021)
        end do

c--- end stencil left plane

c--stencil right plane

        do ixf=1,nxf
          coarsmat (ixf,iyf,izc,p200)=aa1*(finemat (ixf,iyf,izf-1,p200)
     +    +finemat (ixf,iyf,izf,p200))+
     +aa0*finemat (ixf,iyf,izf-1,p201)
          coarsmat (ixf,iyf,izc,p202)=aa1*(finemat (ixf,iyf,izf+1,p202)
     +    +finemat (ixf,iyf,izf,p202))+
     +aa0*finemat (ixf,iyf,izf+1,p201)
          coarsmat (ixf,iyf,izc,p201)=finemat (ixf,iyf,izf,p201)+
     +    aa1*(finemat (ixf,iyf,izf-1,p202)+finemat (ixf,iyf,izf,p200))+
     +    aa1*(finemat (ixf,iyf,izf+1,p200)+finemat (ixf,iyf,izf,p202))+
     +    aa0*finemat (ixf,iyf,izf-1,p201)+
     +    aa0*finemat (ixf,iyf,izf+1,p201)
          coarsmat (ixf,iyf,izc,p210)=aa1*(finemat (ixf,iyf,izf-1,p210)
     +    +finemat (ixf,iyf,izf,p210))+
     +aa0*finemat (ixf,iyf,izf-1,p211)
          coarsmat (ixf,iyf,izc,p212)=aa1*(finemat (ixf,iyf,izf+1,p212)
     +    +finemat (ixf,iyf,izf,p212))+
     +aa0*finemat (ixf,iyf,izf+1,p211)
          coarsmat (ixf,iyf,izc,p211)=finemat (ixf,iyf,izf,p211)+
     +    aa1*(finemat (ixf,iyf,izf-1,p212)+finemat (ixf,iyf,izf,p210))+
     +    aa1*(finemat (ixf,iyf,izf+1,p210)+finemat (ixf,iyf,izf,p212))+
     +    aa0*finemat (ixf,iyf,izf-1,p211)+
     +    aa0*finemat (ixf,iyf,izf+1,p211)
          coarsmat (ixf,iyf,izc,p220)=aa1*(finemat (ixf,iyf,izf-1,p220)
     +    +finemat (ixf,iyf,izf,p220))+
     +aa0*finemat (ixf,iyf,izf-1,p221)
          coarsmat (ixf,iyf,izc,p222)=aa1*(finemat (ixf,iyf,izf+1,p222)
     +    +finemat (ixf,iyf,izf,p222))+
     +aa0*finemat (ixf,iyf,izf+1,p221)
          coarsmat (ixf,iyf,izc,p221)=finemat (ixf,iyf,izf,p221)+
     +    aa1*(finemat (ixf,iyf,izf-1,p222)+finemat (ixf,iyf,izf,p220))+
     +    aa1*(finemat (ixf,iyf,izf+1,p220)+finemat (ixf,iyf,izf,p222))+
     +    aa0*finemat (ixf,iyf,izf-1,p221)+
     +    aa0*finemat (ixf,iyf,izf+1,p221)
        end do
        end do
      end do

c     --- END internal planes

c         ---High and low region planes still have to be done

c         --- stencil left plane

      do iyf=1,nyf
      do ixf=1,nxf
        coarsmat (ixf,iyf,1,p000)=0d0
        coarsmat (ixf,iyf,1,p001)=finemat (ixf,iyf,1,p001)+
     +    aa1*(finemat (ixf,iyf,2,p000)+finemat (ixf,iyf,1,p002))+
     +    aa0*finemat (ixf,iyf,2,p001)
        coarsmat (ixf,iyf,1,p002)=aa1*(finemat (ixf,iyf,2,p002)
     +    +finemat (ixf,iyf,1,p002))+
     +aa0*finemat (ixf,iyf,2,p001)
        coarsmat (ixf,iyf,1,p010)=0d0
        coarsmat (ixf,iyf,1,p011)=finemat (ixf,iyf,1,p011)+
     +    aa1*(finemat (ixf,iyf,2,p010)+finemat (ixf,iyf,1,p012))+
     +    aa0*finemat (ixf,iyf,2,p011)
        coarsmat (ixf,iyf,1,p012)=aa1*(finemat (ixf,iyf,2,p012)
     +    +finemat (ixf,iyf,1,p012))+
     +aa0*finemat (ixf,iyf,2,p011)
        coarsmat (ixf,iyf,1,p020)=0d0
        coarsmat (ixf,iyf,1,p021)=finemat (ixf,iyf,1,p021)+
     +    aa1*(finemat (ixf,iyf,2,p020)+finemat (ixf,iyf,1,p022))+
     +    aa0*finemat (ixf,iyf,2,p021)
        coarsmat (ixf,iyf,1,p022)=aa1*(finemat (ixf,iyf,2,p022)
     +    +finemat (ixf,iyf,1,p022))+
     +aa0*finemat (ixf,iyf,2,p021)
      end do

c---stencil center plane
      do ixf=1,nxf
        coarsmat (ixf,iyf,1,p100)=0d0
        coarsmat (ixf,iyf,1,p101)=finemat (ixf,iyf,1,p101)+
     +    aa1*(finemat (ixf,iyf,1+1,p100)+finemat (ixf,iyf,1,p102))+
     +    aa0*finemat (ixf,iyf,1+1,p101)
        coarsmat (ixf,iyf,1,p102)=aa1*(finemat (ixf,iyf,1+1,p102)
     +    +finemat (ixf,iyf,1,p102))+
     +aa0*finemat (ixf,iyf,1+1,p101)
        coarsmat (ixf,iyf,1,p110)=0d0
        coarsmat (ixf,iyf,1,p111)=finemat (ixf,iyf,1,p111)+
     +    aa1*(finemat (ixf,iyf,1+1,p110)+finemat (ixf,iyf,1,p112))+
     +    aa0*finemat (ixf,iyf,1+1,p111)
        coarsmat (ixf,iyf,1,p112)=aa1*(finemat (ixf,iyf,1+1,p112)
     +    +finemat (ixf,iyf,1,p112))+
     +aa0*finemat (ixf,iyf,1+1,p111)
        coarsmat (ixf,iyf,1,p120)=0d0
        coarsmat (ixf,iyf,1,p121)=finemat (ixf,iyf,1,p121)+
     +    aa1*(finemat (ixf,iyf,1+1,p120)+finemat (ixf,iyf,1,p122))+
     +    aa0*finemat (ixf,iyf,1+1,p121)
        coarsmat (ixf,iyf,1,p122)=aa1*(finemat (ixf,iyf,1+1,p122)
     +    +finemat (ixf,iyf,1,p122))+
     +aa0*finemat (ixf,iyf,1+1,p121)
      end do

c---stencil right plane
      do ixf=1,nxf
        coarsmat (ixf,iyf,1,p200)=0d0
        coarsmat (ixf,iyf,1,p201)=finemat (ixf,iyf,1,p201)+
     +    aa1*(finemat (ixf,iyf,2,p200)+finemat (ixf,iyf,1,p202))+
     +    aa0*finemat (ixf,iyf,2,p201)
        coarsmat (ixf,iyf,1,p202)=aa1*(finemat (ixf,iyf,2,p202)
     +    +finemat (ixf,iyf,1,p202))+
     +aa0*finemat (ixf,iyf,2,p201)
        coarsmat (ixf,iyf,1,p210)=0d0
        coarsmat (ixf,iyf,1,p211)=finemat (ixf,iyf,1,p211)+
     +    aa1*(finemat (ixf,iyf,2,p210)+finemat (ixf,iyf,1,p212))+
     +    aa0*finemat (ixf,iyf,2,p211)
        coarsmat (ixf,iyf,1,p212)=aa1*(finemat (ixf,iyf,2,p212)
     +    +finemat (ixf,iyf,1,p212))+
     +aa0*finemat (ixf,iyf,2,p211)
        coarsmat (ixf,iyf,1,p220)=0d0
        coarsmat (ixf,iyf,1,p221)=finemat (ixf,iyf,1,p221)+
     +    aa1*(finemat (ixf,iyf,2,p220)+finemat (ixf,iyf,1,p222))+
     +    aa0*finemat (ixf,iyf,2,p221)
        coarsmat (ixf,iyf,1,p222)=aa1*(finemat (ixf,iyf,2,p222)
     +    +finemat (ixf,iyf,1,p222))+
     +aa0*finemat (ixf,iyf,2,p221)
      end do
      end do

c     --- END low region plane

c         ---high region plane

      do iyf = 1, nyf

c     --- stencil left plane

      do ixf=1,nxf
          coarsmat (ixf,iyf,nzc,p000)=aa1*(finemat (ixf,iyf,nzf-1,p000)
     +    +finemat (ixf,iyf,nzf,p000))+
     +aa0*finemat (ixf,iyf,nzf-1,p001)
          coarsmat (ixf,iyf,nzc,p002)=0d0
          coarsmat (ixf,iyf,nzc,p001)=finemat (ixf,iyf,nzf,p001)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p002)+finemat (ixf,iyf,nzf,p000))+
     +    aa0*finemat (ixf,iyf,nzf-1,p001)
          coarsmat (ixf,iyf,nzc,p010)=aa1*(finemat (ixf,iyf,nzf-1,p010)
     +    +finemat (ixf,iyf,nzf,p010))+
     +aa0*finemat (ixf,iyf,nzf-1,p011)
          coarsmat (ixf,iyf,nzc,p012)=0d0
          coarsmat (ixf,iyf,nzc,p011)=finemat (ixf,iyf,nzf,p011)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p012)+finemat (ixf,iyf,nzf,p010))+
     +    aa0*finemat (ixf,iyf,nzf-1,p011)
          coarsmat (ixf,iyf,nzc,p020)=aa1*(finemat (ixf,iyf,nzf-1,p020)
     +    +finemat (ixf,iyf,nzf,p020))+
     +aa0*finemat (ixf,iyf,nzf-1,p021)
          coarsmat (ixf,iyf,nzc,p022)=0d0
          coarsmat (ixf,iyf,nzc,p021)=finemat (ixf,iyf,nzf,p021)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p022)+finemat (ixf,iyf,nzf,p020))+
     +    aa0*finemat (ixf,iyf,nzf-1,p021)
      end do

c---stencil center plane
      do ixf=1,nxf
          coarsmat (ixf,iyf,nzc,p100)=aa1*(finemat (ixf,iyf,nzf-1,p100)
     +    +finemat (ixf,iyf,nzf,p100))+
     +aa0*finemat (ixf,iyf,nzf-1,p101)
          coarsmat (ixf,iyf,nzc,p102)=0d0
          coarsmat (ixf,iyf,nzc,p101)=finemat (ixf,iyf,nzf,p101)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p102)+finemat (ixf,iyf,nzf,p100))+
     +    aa0*finemat (ixf,iyf,nzf-1,p101)
          coarsmat (ixf,iyf,nzc,p110)=aa1*(finemat (ixf,iyf,nzf-1,p110)
     +    +finemat (ixf,iyf,nzf,p110))+
     +aa0*finemat (ixf,iyf,nzf-1,p111)
          coarsmat (ixf,iyf,nzc,p112)=0d0
          coarsmat (ixf,iyf,nzc,p111)=finemat (ixf,iyf,nzf,p111)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p112)+finemat (ixf,iyf,nzf,p110))+
     +    aa0*finemat (ixf,iyf,nzf-1,p111)
          coarsmat (ixf,iyf,nzc,p120)=aa1*(finemat (ixf,iyf,nzf-1,p120)
     +    +finemat (ixf,iyf,nzf,p120))+
     +aa0*finemat (ixf,iyf,nzf-1,p121)
          coarsmat (ixf,iyf,nzc,p122)=0d0
          coarsmat (ixf,iyf,nzc,p121)=finemat (ixf,iyf,nzf,p121)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p122)+finemat (ixf,iyf,nzf,p120))+
     +    aa0*finemat (ixf,iyf,nzf-1,p121)
      end do

c     --- stencil right plane

      do ixf=1,nxf
          coarsmat (ixf,iyf,nzc,p200)=aa1*(finemat (ixf,iyf,nzf-1,p200)
     +    +finemat (ixf,iyf,nzf,p200))+
     +aa0*finemat (ixf,iyf,nzf-1,p201)
          coarsmat (ixf,iyf,nzc,p202)=0d0
          coarsmat (ixf,iyf,nzc,p201)=finemat (ixf,iyf,nzf,p201)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p202)+finemat (ixf,iyf,nzf,p200))+
     +    aa0*finemat (ixf,iyf,nzf-1,p201)
          coarsmat (ixf,iyf,nzc,p210)=aa1*(finemat (ixf,iyf,nzf-1,p210)
     +    +finemat (ixf,iyf,nzf,p210))+
     +aa0*finemat (ixf,iyf,nzf-1,p211)
          coarsmat (ixf,iyf,nzc,p212)=0d0
          coarsmat (ixf,iyf,nzc,p211)=finemat (ixf,iyf,nzf,p211)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p212)+finemat (ixf,iyf,nzf,p210))+
     +    aa0*finemat (ixf,iyf,nzf-1,p211)
          coarsmat (ixf,iyf,nzc,p220)=aa1*(finemat (ixf,iyf,nzf-1,p220)
     +    +finemat (ixf,iyf,nzf,p220))+
     +aa0*finemat (ixf,iyf,nzf-1,p221)
          coarsmat (ixf,iyf,nzc,p222)=0d0
          coarsmat (ixf,iyf,nzc,p221)=finemat (ixf,iyf,nzf,p221)+
     +    aa1*(finemat (ixf,iyf,nzf-1,p222)+finemat (ixf,iyf,nzf,p220))+
     +    aa0*finemat (ixf,iyf,nzf-1,p221)
      end do
      end do

c     --- END high region plane

c         -- ENDCase vertex centered

      end if
      end
