      subroutine autosiz3 ( nx, ny, nz, acorsnin, ahilevel,
     +        bufsize, sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    10-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c  autosize calculates the required dimension of rbuf to accomodate all
c  coarse grid info and scratch space.
c  Output in bufsize. This should (at least) be the declared
c  dimension of rbuf in the calling program to the mg - package.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer bufsize
      integer nx, ny, nz, acorsnin, ahilevel,
     + sl1, sl2, sl3, sl4, sl5, sl6

c     acorsnin  i    0=full coarsening 1=semicoarsening  (formal parameter)
c     ahilevel  i    Maximum level of coarse grids (level 0 is finest)
c                    (formal parameter)
c     bufsize   o    Size of buffer needed to accommodate all coarse grid
c                    quantities
c     nx        i    Number of grid x points on finest grid
c     ny        i    Number of grid y points on finest grid
c     nz        i    Number of points in the z-direction
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibuf (300)

c     corsnini  Ibuf (corsnini) contains the method of coarsening
c               for this block. (fullcors or semicors) (module
c               mgpreset)
c     fullcors  Fullcoarsening (type coarsening) (module mgpreset)
c     hileveli  Ibuf (hileveli) contains the highest level of
c               coarsening for this block. (<= MAXLEVEL) (module
c               mgpreset)
c     ibuf      Integer array of dimension 300 that contains the
c               administration of this particular multi grid block.
c     maxlevel  Maximum level allowed for (semi)coarsening (module
c               mgpreset)
c     maxrbufi  Ibuf (maxrbufi) contains maximum size of rbuf (module
c               mgpreset)
c     offrbufi  Ibuf (offrbufi) contains pointer to first unused place
c               in rbuf (module mgpreset)
c     offscri   Ibuf (offscri) contains index where scratch space begins
c               in rbuf (module mgpreset)
c     sizexi    Ibuf(sizexi + ilevel)) contains number of points
c               in x direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizeyi    Ibuf(sizeyi + ilevel)) contains number of points
c               in y direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizezi    Ibuf(sizezi + ilevel)) contains number of points
c               in z direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     slacki    Ibuf (slacki) is the first element of the 6 element
c               slack array xslacklo, xslackhi, yslacklo, yslackhi,
c               zslacklo, zslackhi, (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ERRMSG3   Reports and counts errors (MG3)
c     MKLVINF3  Reserves space for all matrices and arrays (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- sanity check

      ibuf = 0
      if (ahilevel.gt.MAXLEVEL) then
         call errmsg3 ( "autosize", "hilevel exceeds MAXLEVEL", .FALSE.,
     +    ibuf)
      end if
      ibuf (MAXRBUFi) = 64000000
      ibuf (offrbufi) = 1
      ibuf (offscri) = ibuf (MAXRBUFi) + 1
      ibuf (hileveli) = ahilevel
      ibuf (corsnini) = acorsnin
      ibuf (sizexi) = nx
      ibuf (sizeyi) = ny
      ibuf (sizezi) = nz
      ibuf (slacki) = sl1
      ibuf (slacki + 1) = sl2
      ibuf (slacki + 2) = sl3
      ibuf (slacki + 3) = sl4
      ibuf (slacki + 4) = sl5
      ibuf (slacki + 5) = sl6
      call mklvinf3 (ibuf)
      bufsize = ibuf (offrbufi)
      if (ibuf (corsnini).eq.FULLCORS) then
         bufsize = bufsize + ibuf (sizexi + 1) * ny * nz * 9
     +             + ibuf (sizexi + 1) * ibuf (sizeyi + 1) * 9
      else
         bufsize = bufsize + (nx + sl1 + sl2)*
     +   (ny + sl3 + sl4) * (nz + sl5 + sl6)
      end if
      end
