      subroutine linejacx (LUxmat, res, cor, nxa, nya,
     +           sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculates a line jacobi *correction* in x direction.
c   LUx contains the LU decomposition of the diagonal blocks
c   D_x c = b - A x_n = r_n
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4
      integer nxa, nya
      double precision
     +cor(1-sl1:nxa+sl2,1-sl3:nya+sl4),
     +res(1-sl1:nxa+sl2,1-sl3:nya+sl4),
     +LUxmat(1-sl1:nxa+sl2,1-sl3:nya+sl4,3)

c     cor       i    correction vector
c     luxmat    i    LU dec of "diagonal" blocks hor numbering
c     nxa       i    Number of grid x points on this level
c     nya       i    Number of grid y points on this level
c     res       i    Residual (any level)
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixa, iya

c     ixa       Counter to number of grid x points on this level
c     iya       Counter to number of grid y points on this level
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      trivial
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      do iya = 1, nya

c     --- first forward substitution : L s = b

        cor (1, iya) = res (1, iya)
        do ixa = 2, nxa
          cor (ixa, iya) = res (ixa, iya) - cor (ixa-1, iya) *
     +            LUxmat (ixa, iya, SUB)
        end do

c       --- then backward substitution Ux = s

        cor (nxa, iya) = cor (nxa, iya) / LUxmat (nxa, iya, DIAG)
        do ixa = nxa - 1, 1, -1
          cor (ixa, iya) = (cor (ixa, iya) - cor (ixa + 1, iya) *
     +       LUxmat (ixa, iya, SUP))/LUxmat (ixa, iya, DIAG)
        end do
      end do
      end
