      subroutine m7mv3 ( mat3, vec, bresul, nxa, nya, nza,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   10-03-1997
c
c   copyright (c) 1997
c   all rights bresulerved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c  calculates Ax on any level, using the 7 point molecule
c  i.e only p1[012]1, p11[02] and p[02]11 are assumed nonzero.
c **********************************************************************
c
c                       KEYWORDS
c
c  multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nxa, nya, nza
      integer sl1, sl2, sl3, sl4, sl5, sl6
      double precision
     +  mat3 (1-sl1:nxa+sl2, 1-sl3:nya+sl4,
     +  1-sl5:nza+sl6, 27),
     +  vec (1-sl1:nxa+sl2, 1-sl3:nya+sl4,
     +  1-sl5:nza+sl6),
     +  bresul (1-sl1:nxa+sl2, 1-sl3:nya+sl4,
     +  1-sl5:nza+sl6)

c     bresul    i    The result vector
c     mat3      i    The nx x ny x nz x 27  coefficient matrix of the
c                    discrete Laplacian
c     nxa       i    Number of grid x points on this level
c     nya       i    Number of grid y points on this level
c     nza       i    Number of grid z points on this level
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c     vec       i    Generic name for quantity with the dimension of
c                    sol or rhs
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixa, iya, iza

c     ixa       Counter to number of grid x points on this level
c     iya       Counter to number of grid y points on this level
c     iza       Counter to number of grid z points on this level
c     p011      Index of point in stencil 3D (module mgpreset)
c     p101      Index of point in stencil 3D (module mgpreset)
c     p110      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p112      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     p211      Index of point in stencil 3D (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c   trivial
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- You could do this in a sloppy way: multiply everything.
c         this would be all right, since all mat3rix coefficients that do not
c         contribute are 0.
c         We prefer to do it the right way. Somewhat more lines though. (27)
c         --- The corners (8)

      bresul (1, 1, 1) =
     +       mat3 (1, 1, 1, p111) * vec (1, 1, 1)+
     +       mat3 (1, 1, 1, p112) * vec (1, 1, 2)+
     +       mat3 (1, 1, 1, p121) * vec (1, 2, 1)+
     +       mat3 (1, 1, 1, p211) * vec (2, 1, 1)
      bresul (nxa, 1, 1) =
     +       mat3 (nxa, 1, 1, p011) * vec (nxa-1, 1, 1)+
     +       mat3 (nxa, 1, 1, p111) * vec (nxa, 1, 1)+
     +       mat3 (nxa, 1, 1, p112) * vec (nxa, 1, 2)+
     +       mat3 (nxa, 1, 1, p121) * vec (nxa, 2, 1)
      bresul (1, nya, 1) =
     +       mat3 (1, nya, 1, p101) * vec (1, nya-1, 1)+
     +       mat3 (1, nya, 1, p111) * vec (1, nya, 1)+
     +       mat3 (1, nya, 1, p112) * vec (1, nya, 2)+
     +       mat3 (1, nya, 1, p211) * vec (2, nya, 1)
      bresul (nxa, nya, 1) =
     +       mat3 (nxa, nya, 1, p011) * vec (nxa-1, nya, 1)+
     +       mat3 (nxa, nya, 1, p101) * vec (nxa, nya-1, 1)+
     +       mat3 (nxa, nya, 1, p111) * vec (nxa, nya, 1)+
     +       mat3 (nxa, nya, 1, p112) * vec (nxa, nya, 2)
      bresul (1, 1, nza) =
     +       mat3 (1, 1, nza, p110) * vec (1, 1, nza-1)+
     +       mat3 (1, 1, nza, p111) * vec (1, 1, nza)+
     +       mat3 (1, 1, nza, p121) * vec (1, 2, nza)+
     +       mat3 (1, 1, nza, p211) * vec (2, 1, nza)
      bresul (nxa, 1, nza) =
     +       mat3 (nxa, 1, nza, p011) * vec (nxa-1, 1, nza)+
     +       mat3 (nxa, 1, nza, p110) * vec (nxa, 1, nza-1)+
     +       mat3 (nxa, 1, nza, p111) * vec (nxa, 1, nza)+
     +       mat3 (nxa, 1, nza, p121) * vec (nxa, 2, nza)
      bresul (1, nya, nza) =
     +       mat3 (1, nya, nza, p101) * vec (1, nya-1, nza)+
     +       mat3 (1, nya, nza, p110) * vec (1, nya, nza-1)+
     +       mat3 (1, nya, nza, p111) * vec (1, nya, nza)+
     +       mat3 (1, nya, nza, p211) * vec (2, nya, nza)
      bresul (nxa, nya, nza) =
     +       mat3 (nxa, nya, nza, p011) * vec (nxa-1, nya, nza)+
     +       mat3 (nxa, nya, nza, p101) * vec (nxa, nya-1, nza)+
     +       mat3 (nxa, nya, nza, p110) * vec (nxa, nya, nza-1)+
     +       mat3 (nxa, nya, nza, p111) * vec (nxa, nya, nza)

c     --- The edges (12)

      do ixa = 2, nxa - 1
      bresul (ixa, 1, 1) =
     +       mat3 (ixa, 1, 1, p011) * vec (ixa-1, 1, 1)+
     +       mat3 (ixa, 1, 1, p111) * vec (ixa, 1, 1)+
     +       mat3 (ixa, 1, 1, p112) * vec (ixa, 1, 2)+
     +       mat3 (ixa, 1, 1, p121) * vec (ixa, 2, 1)+
     +       mat3 (ixa, 1, 1, p211) * vec (ixa+1, 1, 1)
      bresul (ixa, nya, 1) =
     +       mat3 (ixa, nya, 1, p011) * vec (ixa-1, nya, 1)+
     +       mat3 (ixa, nya, 1, p101) * vec (ixa, nya-1, 1)+
     +       mat3 (ixa, nya, 1, p111) * vec (ixa, nya, 1)+
     +       mat3 (ixa, nya, 1, p112) * vec (ixa, nya, 2)+
     +       mat3 (ixa, nya, 1, p211) * vec (ixa+1, nya, 1)
      bresul (ixa, 1, nza) =
     +       mat3 (ixa, 1, nza, p011) * vec (ixa-1, 1, nza)+
     +       mat3 (ixa, 1, nza, p110) * vec (ixa, 1, nza-1)+
     +       mat3 (ixa, 1, nza, p111) * vec (ixa, 1, nza)+
     +       mat3 (ixa, 1, nza, p121) * vec (ixa, 2, nza)+
     +       mat3 (ixa, 1, nza, p211) * vec (ixa+1, 1, nza)
      bresul (ixa, nya, nza) =
     +       mat3 (ixa, nya, nza, p011) * vec (ixa-1, nya, nza)+
     +       mat3 (ixa, nya, nza, p101) * vec (ixa, nya-1, nza)+
     +       mat3 (ixa, nya, nza, p110) * vec (ixa, nya, nza-1)+
     +       mat3 (ixa, nya, nza, p111) * vec (ixa, nya, nza)+
     +       mat3 (ixa, nya, nza, p211) * vec (ixa+1, nya, nza)
      end do

      do iya = 2, nya - 1
      bresul (1, iya, 1) =
     +       mat3 (1, iya, 1, p101) * vec (1, iya-1, 1)+
     +       mat3 (1, iya, 1, p111) * vec (1, iya, 1)+
     +       mat3 (1, iya, 1, p112) * vec (1, iya, 2)+
     +       mat3 (1, iya, 1, p121) * vec (1, iya+1, 1)+
     +       mat3 (1, iya, 1, p211) * vec (2, iya, 1)
      bresul (nxa, iya, 1) =
     +       mat3 (nxa, iya, 1, p011) * vec (nxa-1, iya, 1)+
     +       mat3 (nxa, iya, 1, p101) * vec (nxa, iya-1, 1)+
     +       mat3 (nxa, iya, 1, p111) * vec (nxa, iya, 1)+
     +       mat3 (nxa, iya, 1, p112) * vec (nxa, iya, 2)+
     +       mat3 (nxa, iya, 1, p121) * vec (nxa, iya+1, 1)
      bresul (1, iya, nza) =
     +       mat3 (1, iya, nza, p101) * vec (1, iya-1, nza)+
     +       mat3 (1, iya, nza, p110) * vec (1, iya, nza-1)+
     +       mat3 (1, iya, nza, p111) * vec (1, iya, nza)+
     +       mat3 (1, iya, nza, p121) * vec (1, iya+1, nza)+
     +       mat3 (1, iya, nza, p211) * vec (1+1, iya, nza)
      bresul (nxa, iya, nza) =
     +       mat3 (nxa, iya, nza, p011) * vec (nxa-1, iya, nza)+
     +       mat3 (nxa, iya, nza, p101) * vec (nxa, iya-1, nza)+
     +       mat3 (nxa, iya, nza, p110) * vec (nxa, iya, nza-1)+
     +       mat3 (nxa, iya, nza, p111) * vec (nxa, iya, nza)+
     +       mat3 (nxa, iya, nza, p121) * vec (nxa, iya+1, nza)
      end do

      do iza = 2, nza - 1
      bresul (1, 1, iza) =
     +       mat3 (1, 1, iza, p110) * vec (1, 1, iza-1)+
     +       mat3 (1, 1, iza, p111) * vec (1, 1, iza)+
     +       mat3 (1, 1, iza, p112) * vec (1, 1, iza+1)+
     +       mat3 (1, 1, iza, p121) * vec (1, 2, iza)+
     +       mat3 (1, 1, iza, p211) * vec (2, 1, iza)
      bresul (nxa, 1, iza) =
     +       mat3 (nxa, 1, iza, p011) * vec (nxa-1, 1, iza)+
     +       mat3 (nxa, 1, iza, p110) * vec (nxa, 1, iza-1)+
     +       mat3 (nxa, 1, iza, p111) * vec (nxa, 1, iza)+
     +       mat3 (nxa, 1, iza, p112) * vec (nxa, 1, iza+1)+
     +       mat3 (nxa, 1, iza, p121) * vec (nxa, 2, iza)
      bresul (1, nya, iza) =
     +       mat3 (1, nya, iza, p101) * vec (1, nya-1, iza)+
     +       mat3 (1, nya, iza, p110) * vec (1, nya, iza-1)+
     +       mat3 (1, nya, iza, p111) * vec (1, nya, iza)+
     +       mat3 (1, nya, iza, p112) * vec (1, nya, iza+1)+
     +       mat3 (1, nya, iza, p211) * vec (2, nya, iza)
      bresul (nxa, nya, iza) =
     +       mat3 (nxa, nya, iza, p011) * vec (nxa-1, nya, iza)+
     +       mat3 (nxa, nya, iza, p101) * vec (nxa, nya-1, iza)+
     +       mat3 (nxa, nya, iza, p110) * vec (nxa, nya, iza-1)+
     +       mat3 (nxa, nya, iza, p111) * vec (nxa, nya, iza)+
     +       mat3 (nxa, nya, iza, p112) * vec (nxa, nya, iza+1)
      end do

c     --- the faces (6)

      do iya = 2, nya - 1
      do ixa = 2, nxa - 1
      bresul (ixa, iya, 1) =
     +       mat3 (ixa, iya, 1, p011) * vec (ixa-1, iya, 1)+
     +       mat3 (ixa, iya, 1, p101) * vec (ixa, iya-1, 1)+
     +       mat3 (ixa, iya, 1, p111) * vec (ixa, iya, 1)+
     +       mat3 (ixa, iya, 1, p112) * vec (ixa, iya, 2)+
     +       mat3 (ixa, iya, 1, p121) * vec (ixa, iya+1, 1)+
     +       mat3 (ixa, iya, 1, p211) * vec (ixa+1, iya, 1)
      bresul (ixa, iya, nza) =
     +       mat3 (ixa, iya, nza, p011) * vec (ixa-1, iya, nza)+
     +       mat3 (ixa, iya, nza, p101) * vec (ixa, iya-1, nza)+
     +       mat3 (ixa, iya, nza, p110) * vec (ixa, iya, nza-1)+
     +       mat3 (ixa, iya, nza, p111) * vec (ixa, iya, nza)+
     +       mat3 (ixa, iya, nza, p121) * vec (ixa, iya+1, nza)+
     +       mat3 (ixa, iya, nza, p211) * vec (ixa+1, iya, nza)
      end do
      end do

      do iza = 2, nza - 1
      do ixa = 2, nxa - 1
      bresul (ixa, 1, iza) =
     +       mat3 (ixa, 1, iza, p011) * vec (ixa-1, 1, iza)+
     +       mat3 (ixa, 1, iza, p110) * vec (ixa, 1, iza-1)+
     +       mat3 (ixa, 1, iza, p111) * vec (ixa, 1, iza)+
     +       mat3 (ixa, 1, iza, p112) * vec (ixa, 1, iza+1)+
     +       mat3 (ixa, 1, iza, p121) * vec (ixa, 2, iza)+
     +       mat3 (ixa, 1, iza, p211) * vec (ixa+1, 1, iza)
      bresul (ixa, nya, iza) =
     +       mat3 (ixa, nya, iza, p011) * vec (ixa-1, nya, iza)+
     +       mat3 (ixa, nya, iza, p101) * vec (ixa, nya-1, iza)+
     +       mat3 (ixa, nya, iza, p110) * vec (ixa, nya, iza-1)+
     +       mat3 (ixa, nya, iza, p111) * vec (ixa, nya, iza)+
     +       mat3 (ixa, nya, iza, p112) * vec (ixa, nya, iza+1)+
     +       mat3 (ixa, nya, iza, p211) * vec (ixa+1, nya, iza)
      end do
      end do

      do iza = 2, nza - 1
      do iya = 2, nya - 1
      bresul (1, iya, iza) =
     +       mat3 (1, iya, iza, p101) * vec (1, iya-1, iza)+
     +       mat3 (1, iya, iza, p110) * vec (1, iya, iza-1)+
     +       mat3 (1, iya, iza, p111) * vec (1, iya, iza)+
     +       mat3 (1, iya, iza, p112) * vec (1, iya, iza+1)+
     +       mat3 (1, iya, iza, p121) * vec (1, iya+1, iza)+
     +       mat3 (1, iya, iza, p211) * vec (2, iya, iza)
      bresul (nxa, iya, iza) =
     +       mat3 (nxa, iya, iza, p011) * vec (nxa-1, iya, iza)+
     +       mat3 (nxa, iya, iza, p101) * vec (nxa, iya-1, iza)+
     +       mat3 (nxa, iya, iza, p110) * vec (nxa, iya, iza-1)+
     +       mat3 (nxa, iya, iza, p111) * vec (nxa, iya, iza)+
     +       mat3 (nxa, iya, iza, p112) * vec (nxa, iya, iza+1)+
     +       mat3 (nxa, iya, iza, p121) * vec (nxa, iya+1, iza)
      end do
      end do

c     --- The interior (generic case)

      do iza = 2, nza - 1
      do iya = 2, nya - 1
      do ixa = 2, nxa - 1
      bresul (ixa, iya, iza) =
     +       mat3 (ixa, iya, iza, p011) * vec (ixa-1, iya, iza)+
     +       mat3 (ixa, iya, iza, p101) * vec (ixa, iya-1, iza)+
     +       mat3 (ixa, iya, iza, p110) * vec (ixa, iya, iza-1)+
     +       mat3 (ixa, iya, iza, p111) * vec (ixa, iya, iza)+
     +       mat3 (ixa, iya, iza, p112) * vec (ixa, iya, iza+1)+
     +       mat3 (ixa, iya, iza, p121) * vec (ixa, iya+1, iza)+
     +       mat3 (ixa, iya, iza, p211) * vec (ixa+1, iya, iza)
      end do
      end do
      end do
      end
