      subroutine pmap ( maparr )
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Serves to change the default mapping of the 9-point laplacian
c   molecule. The user can change the order by giving a permutation
c   in the 9-entry array pmap of the vector 1..9
c   p00 -> maparr (1), p10 -> maparr  (2) ... p22 -> maparr  (9)
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maparr (9)

c     maparr    i    Contains permutation for p00 ... p22 (1..9 by
c                          default)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, perm (9)

c     i         General loop variable
c     perm      Permutation vector
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     MGEXIT    Wrapper to exit MG in case of fatal error. (MG)
c               APPLICATION HAS TO PROVIDE THIS. Defaults to stop. (MG)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- sanity check
c         we check whether pmap is a permutation
c         if not an error is signalled

      do i = 1, 9
      perm (i) = 0
      end do

      do i = 1, 9
      perm (maparr (i)) = 1
      end do

      do i = 1, 9
      if (perm (i).eq.0) then
      write (*, *) "Fatal MG error in pmap: map array is not a ",
     +             "permutation. Program halted."
      call mgexit
      end if
      end do

      p00 = maparr (1)
      p10 = maparr (2)
      p20 = maparr (3)
      p01 = maparr (4)
      p11 = maparr (5)
      p21 = maparr (6)
      p02 = maparr (7)
      p12 = maparr (8)
      p22 = maparr (9)
      end
