      subroutine v1step3  ( mat3, res, nsmooth, sol, rbuf, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c       version 1.2    date  8- 7-1997
c       version 1.1    10-04-1997
c        version 1.0    date Tue Apr  8 1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c    3D Preconditioner for mat3 based on a multigrid V-cycle
c    with 3 way alternating Line Jacobi as smoother.
c   Result in sol.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid preconditioner V-cycle line-jacobi smoother
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ibuf (*)
      double precision mat3 (*), res (*), sol(*), rbuf (*)
      integer nsmooth

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat3      i    The nx x ny x nz x 27  coefficient matrix of the
c                    discrete Laplacian
c     nsmooth   i    Number of times smoother is applied (all levels)
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c     res       i    Residual (any level)
c     sol       o    Solution vector (any level)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilevel, ismooth, hilevel
      double precision pi, lowb, aczero
      double precision w (100)
      double precision ChebLB (10)
      data ChebLB/ 0.82287565553230,  0.75375214376509,
     +             0.71927884886952,  0.70042075768052,
     +             0.68897086241745,  0.68140606981492,
     +             0.67607178474481,  0.67211862680551,
     +             0.66907476690353,  0.66665981350962 /

c     aczero    actual zero of the shifted Chebyshev polynomial
c     cheblb    Lower bound of shifted Chebyshev interval
c               (high bound is always 2), depending on nsmooth
c               for nsmooth > 10 ChebLb = 2/3.
c     corri     Rbuf (ibuf (corri + ilevel)) is the first element
c               of the correction vector at level ilevel. (ilevel >=0)
c               (module mgpreset)
c     flevel0   Flag to denote whether we are on level 0
c               (module mgpreset)
c     hilevel   Maximum level of coarse grids (level 0 is finest)
c     hileveli  Ibuf (hileveli) contains the highest level of
c               coarsening for this block. (<= MAXLEVEL) (module
c               mgpreset)
c     ilevel    Levelcounter (multi grid levels)
c     ismooth   Counter to number of times smoother has been applied
c     lowb      calculated lowerbound of the intervals of the shifted
c               Chebyshev polynomials (upper bound is always 2).
c     luxri     Rbuf (ibuf (LUxri + ilevel)) is the first element
c               of the LUx decomposition at level ilevel (ilevel >= 0)
c               (module mgpreset)
c     luyri     Rbuf (ibuf (LUyri + ilevel)) is the first element
c               of the LUy decomposition at level ilevel (ilevel >= 0)
c               (module mgpreset)
c     luzri     Rbuf (ibuf (LUzri + ilevel)) is the first element
c               of the LUz decomposition at level ilevel (ilevel >= 0)
c               (module mgpreset)
c     matri     Rbuf (ibuf (matri + ilevel)) is the first element
c               of the RAP operator at level ilevel (ilevel >= 1)
c               (module mgpreset)
c     offscri   Ibuf (offscri) contains index where scratch space begins
c               in rbuf (module mgpreset)
c     pi        the famous constant
c     prelud_i  Ibuf (prelud_i) = 100 if prelude has been called
c               for this block. (module mgpreset)
c     resri     Rbuf (ibuf (resri + ilevel)) is the first element
c               of the res vector at level ilevel (ilevel >= 0) (module
c               mgpreset)
c     rhsri     Rbuf (ibuf (rhsri + ilevel)) is the first element
c               of the right hand side vector at level ilevel
c               (ilevel >= 1) (module mgpreset)
c     sizexi    Ibuf(sizexi + ilevel)) contains number of points
c               in x direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizeyi    Ibuf(sizeyi + ilevel)) contains number of points
c               in y direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizezi    Ibuf(sizezi + ilevel)) contains number of points
c               in z direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     slacki    Ibuf (slacki) is the first element of the 6 element
c               slack array xslacklo, xslackhi, yslacklo, yslackhi,
c               zslacklo, zslackhi, (module mgpreset)
c     solri     Rbuf (ibuf (solri + ilevel)) is the first element
c               of the sol vector at level ilevel (ilevel >= 1) (module
c               mgpreset)
c     w         Array of Chebyshev coefficients
c     yes       Named constant (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     CALCRES3  Calculates residal on any level (MG3)
c     ERRMSG3   Reports and counts errors (MG3)
c     FREESCS3  Gives reserved scratch space back to buffer (MG3)
c     GETSCS3   Reserves a block of scratch space from buffer (MG3)
c     LINEJCX3  Calculates 1 step line jacobi preconditioner x-dir (MG3)
c     LINEJCY3  Calculates 1 step line jacobi preconditioner y-dir (MG3)
c     LINEJCZ3  Calculates 1 step line jacobi preconditioner z-dir (MG3)
c     PROLONG3  Prolongation (all coarsening types) (MG3)
c     RESTRI3   Restriction (all coarsening types) (MG3)
c     TRIADV3   Adds x + b y with x and y vectors, b scalar (MG3)
c     ZEROIZE3  Initialize vector to all zeros (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if (ibuf (prelud_i).ne.YES) then
      call errmsg3 ('v1step3', 'prelude has not been called previously',
     +              .true., ibuf)

c     --- fatal error

      end if
      if (nsmooth.gt.100) then
      call errmsg3 ('v1step3', 'nsmooth too large', .false., ibuf)
      nsmooth = 100
      end if

c     --- calculate chebyshev coefficients

      pi = 4d0 * atan (1d0)
      if (nsmooth.le.10) then
      lowb = ChebLB (nsmooth)
      else

c     --- asymptotic value

      lowb = 2d0/3d0
      end if

      do ismooth = 1, nsmooth

c     --- calculate zero of Chebyshev polynomial

      aczero = cos (pi * (ismooth - 0.5d0)/nsmooth)
      aczero = (1d0 - 0.5 * lowb) * aczero + (1d0 + 0.5 * lowb)
      w (ismooth) = 1d0/aczero
      end do

c     --- we take the given res as rhs and restrict to all levels

      call restri3 (res, rbuf (ibuf (rhsri + 1)),
     +       ibuf (sizexi), ibuf (sizeyi), ibuf (sizezi),
     +       ibuf (sizexi+1), ibuf (sizeyi+1), ibuf (sizeyi+1),
     +       rbuf, ibuf,
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))
      hilevel = ibuf (hileveli)

      flevel0 = .false.
      do ilevel = 1, hilevel-1
        call restri3 (rbuf (ibuf (rhsri + ilevel)),
     +                 rbuf (ibuf (rhsri + ilevel + 1)),
     +        ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +        ibuf (sizezi + ilevel),
     +        ibuf (sizexi + ilevel + 1), ibuf (sizeyi + ilevel + 1),
     +        ibuf (sizezi + ilevel + 1),
     +        rbuf, ibuf, 0, 0, 0, 0, 0, 0)
      end do

c     --- initialize solution to 0 on all levels

      call zeroize3 (sol, ibuf (sizexi), ibuf (sizeyi), ibuf (sizezi),
     +
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))
      do ilevel = 1, hilevel
        call zeroize3 (rbuf (ibuf (solri + ilevel)),
     +    ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +    ibuf (sizezi + ilevel), 0, 0, 0, 0, 0, 0)
      end do

c--- get some space to store A x
      call getscs3 ( ibuf,
     +         (ibuf (sizexi) + ibuf (slacki) + ibuf (slacki + 1))*
     +         (ibuf (sizeyi) + ibuf (slacki + 2)+ ibuf (slacki + 3))*
     +         (ibuf (sizezi) + ibuf (slacki + 4)+ ibuf (slacki + 5)))

c     --- solution on coarsest grid

      do ismooth = 1, nsmooth
       call calcres3 (rbuf (ibuf (matri + hilevel)),
     +               rbuf (ibuf (solri + hilevel)),
     +               rbuf (ibuf (rhsri + hilevel)),
     +               rbuf (ibuf (resri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      rbuf (ibuf (offscri)), 0, 0, 0, 0, 0, 0)

       call linejcx3 (rbuf (ibuf (LUxri + hilevel)),
     +                rbuf (ibuf (resri + hilevel)),
     +                rbuf (ibuf (corri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      0, 0, 0, 0, 0, 0)

       call triadv3 (rbuf (ibuf (solri + hilevel)), w(ismooth),
     +                rbuf (ibuf (corri + hilevel)),
     +                rbuf (ibuf (solri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      0, 0, 0, 0, 0, 0)

       call calcres3 (rbuf (ibuf (matri + hilevel)),
     +               rbuf (ibuf (solri + hilevel)),
     +               rbuf (ibuf (rhsri + hilevel)),
     +               rbuf (ibuf (resri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      rbuf (ibuf (offscri)), 0, 0, 0, 0, 0, 0)

       call linejcy3 (rbuf (ibuf (LUyri + hilevel)),
     +                rbuf (ibuf (resri + hilevel)),
     +                rbuf (ibuf (corri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      0, 0, 0, 0, 0, 0)

       call triadv3 (rbuf (ibuf (solri + hilevel)), w(ismooth),
     +                rbuf (ibuf (corri + hilevel)),
     +                rbuf (ibuf (solri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      0, 0, 0, 0, 0, 0)

       call calcres3 (rbuf (ibuf (matri + hilevel)),
     +               rbuf (ibuf (solri + hilevel)),
     +               rbuf (ibuf (rhsri + hilevel)),
     +               rbuf (ibuf (resri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      rbuf (ibuf (offscri)), 0, 0, 0, 0, 0, 0)

       call linejcz3 (rbuf (ibuf (LUzri + hilevel)),
     +                rbuf (ibuf (resri + hilevel)),
     +                rbuf (ibuf (corri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      0, 0, 0, 0, 0, 0)

       call triadv3 (rbuf (ibuf (solri + hilevel)), w(ismooth),
     +                rbuf (ibuf (corri + hilevel)),
     +                rbuf (ibuf (solri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      ibuf (sizezi + hilevel),
     +      0, 0, 0, 0, 0, 0)
       end do

c      --- for remaining levels

      do ilevel = hilevel - 1, 1, -1

c     --- prolongate to next finer level

        call prolong3 (rbuf (ibuf (solri + ilevel)),
     +                   rbuf (ibuf (solri + ilevel + 1)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      ibuf (sizexi + ilevel+1), ibuf (sizeyi + ilevel+1),
     +      ibuf (sizezi + ilevel + 1),
     +      rbuf, ibuf,
     +      0, 0, 0, 0, 0, 0)

c       --- smoothe solution on this level

        do ismooth = 1, nsmooth
       call calcres3 (rbuf ( ibuf (matri + ilevel)),
     +               rbuf ( ibuf (solri + ilevel)),
     +               rbuf ( ibuf (rhsri + ilevel)),
     +               rbuf ( ibuf (resri + ilevel)),
     +      ibuf (sizexi + ilevel),  ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      rbuf (ibuf (offscri)),
     +      0, 0, 0, 0, 0, 0)

        call linejcx3 (rbuf (ibuf (LUxri + ilevel)),
     +                 rbuf (ibuf (resri + ilevel)),
     +                 rbuf (ibuf (corri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      0, 0, 0, 0, 0, 0)

        call triadv3 (rbuf (ibuf (solri + ilevel)), w(ismooth),
     +                 rbuf (ibuf (corri + ilevel)),
     +                 rbuf (ibuf (solri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      0, 0, 0, 0, 0, 0)

        call calcres3 (rbuf (ibuf (matri + ilevel)),
     +                rbuf (ibuf (solri + ilevel)),
     +                rbuf (ibuf (rhsri + ilevel)),
     +                rbuf (ibuf (resri + ilevel)),
     +      ibuf (sizexi + ilevel),  ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      rbuf (ibuf (offscri)),
     +      0, 0, 0, 0, 0, 0)

        call linejcy3 (rbuf (ibuf (LUyri + ilevel)),
     +                 rbuf (ibuf (resri + ilevel)),
     +                 rbuf (ibuf (corri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      0, 0, 0, 0, 0, 0)

        call triadv3 (rbuf (ibuf (solri + ilevel)), w(ismooth),
     +                 rbuf (ibuf (corri + ilevel)),
     +                 rbuf (ibuf (solri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      0, 0, 0, 0, 0, 0)

        call calcres3 (rbuf (ibuf (matri + ilevel)),
     +                rbuf (ibuf (solri + ilevel)),
     +                rbuf (ibuf (rhsri + ilevel)),
     +                rbuf (ibuf (resri + ilevel)),
     +      ibuf (sizexi + ilevel),  ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      rbuf (ibuf (offscri)),
     +      0, 0, 0, 0, 0, 0)

        call linejcz3 (rbuf (ibuf (LUzri + ilevel)),
     +                 rbuf (ibuf (resri + ilevel)),
     +                 rbuf (ibuf (corri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      0, 0, 0, 0, 0, 0)

        call triadv3 (rbuf (ibuf (solri + ilevel)), w(ismooth),
     +                 rbuf (ibuf (corri + ilevel)),
     +                 rbuf (ibuf (solri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizezi + ilevel),
     +      0, 0, 0, 0, 0, 0)
        end do

      end do

c     --- prolongate to highest level

        call prolong3 (sol,
     +       rbuf (ibuf (solri + 1)),
     +       ibuf (sizexi), ibuf (sizeyi),
     +       ibuf (sizezi),
     +       ibuf (sizexi + 1), ibuf (sizeyi + 1),
     +       ibuf (sizezi + 1),
     +       rbuf, ibuf,
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

c       --- still smoothing required on highest level

      flevel0 = .true.
      do ismooth = 1, nsmooth
      call calcres3 (mat3, sol, res,
     +       rbuf(ibuf (resri)),
     +       ibuf (sizexi), ibuf (sizeyi),
     +       ibuf (sizezi),
     +       rbuf (ibuf (offscri)),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call linejcx3 (rbuf (ibuf (LUxri)), rbuf (ibuf (resri)),
     +       rbuf (ibuf (corri)), ibuf (sizexi), ibuf (sizeyi),
     +       ibuf (sizezi),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call triadv3 ( sol, w(ismooth), rbuf (ibuf (corri)), sol,
     +       ibuf (sizexi),  ibuf (sizeyi),
     +       ibuf (sizezi),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call calcres3 (mat3, sol, res, rbuf(ibuf (resri)),
     +       ibuf (sizexi), ibuf (sizeyi), ibuf (sizezi),
     +       rbuf (ibuf (offscri)),
     +
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call linejcy3 (rbuf (ibuf (LUyri)), rbuf (ibuf (resri)),
     +       rbuf (ibuf (corri)), ibuf (sizexi), ibuf (sizeyi),
     +       ibuf (sizezi),
     +
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call triadv3 ( sol, w(ismooth), rbuf (ibuf (corri)), sol,
     +       ibuf (sizexi),  ibuf (sizeyi),
     +       ibuf (sizezi),
     +
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call calcres3 (mat3, sol, res, rbuf(ibuf (resri)),
     +       ibuf (sizexi), ibuf (sizeyi), ibuf (sizezi),
     +       rbuf (ibuf (offscri)),
     +
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call linejcz3 (rbuf (ibuf (LUzri)), rbuf (ibuf (resri)),
     +       rbuf (ibuf (corri)), ibuf (sizexi), ibuf (sizeyi),
     +       ibuf (sizezi),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))

      call triadv3 ( sol, w(ismooth), rbuf (ibuf (corri)), sol,
     +       ibuf (sizexi),  ibuf (sizeyi),
     +       ibuf (sizezi),
     +
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))
      end do

c--- return scratch space
      call freescs3 (ibuf)
      end
