      subroutine isinirdf ( iinput, rinput, itopol, lentopol, work,
     +                      lwork, refine )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.4    date 15-12-1999 remove stops
c        version 1.3    date 15-10-1998 error 719 added
c        version 1.2    date 08-04-1998 refine=2 means lam2turb added
c        version 1.1    date 08-05-1995 refine added
c        version 1.0    date 01-05-1995
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Startup of reading the restart file
c
c **********************************************************************
c
c                       KEYWORDS
c
c     backing_storage
c     restart
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer refine
      integer iinput(*), lentopol, itopol(lentopol), lwork, work(lwork)
      double precision rinput(*)

c     iinput     i    reserved
c     itopol     i    reserved
c     lentopol   i    Length of the itopol array
c     lwork      i    Length of array work
c     refine     i    refine=1 : refine refine=0 : not
c     rinput     i    reserved
c     work       i/o  Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisnasch'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer maxnsess
      parameter ( maxnsess = 100 )
      logical exstat,  notfound
      integer i, ios, iprintim,  lsess,
     +        nsessions, sesname(maxnsess), sesrecord(maxnsess),
     +        itimenext, ndim, ntrnsp, nturb,
     +        sessionn, ipiincnd, iprincnd,
     +        irecses, irecsol, irectime
      double precision  rinit, time, dt, eps

c     dt          Time step
c     eps         Accuracy with respect to the time parameter tout
c     exstat      Existstatus variable
c     i           General loop variable
c     ios         Iostatus variable
c     ipiincnd    Starting address of array iincnd in array iinput
c     iprincnd    Starting address of array rincnd in array rinput
c     iprintim    Starting address of array RINTIM in array RINPUT
c     irecses     Record number of session record
c     irecsol
c     irectime    Record number of time record
c     itimenext   Record number of next time record
c     lsess       Help variable for the length of arrays sesname and sesrecord
c     maxnsess    Maximum number of sessions
c     ndim        Dimension of space (2 or 3)
c     notfound    Logical help variable
c     nsessions   Number of sessions
c     ntrnsp      Number of transport equations
c     nturb       Number of extra transport equations for turbulence
c     rinit       Start time for writing solutions to the restart file
c     sesname     Array with session numbers of all sessions
c     sesrecord   Array with first record numbers of all sessions
c     sessionn    Session number
c     time        Actual time of time step
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINIFIL  Open files
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     20      Error in reading record
c     21      Difference found between current session and session
c             to which will be appended
c     708     Recordlength of restartfile to short
c     709
c     710
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The restart file is a direct access file to which the solution
c     will be written for restarting. The restart file contains a
c     list of sessions. Each session consists of a list of time steps.
c     The first record of the restart file contains information
c     about the list of sessions and about the division of the restart
c     file into records
c
c
c     purpose of this code:
c     To initialize the reading of the restart file by computing
c     the record number where reading must be started
c
c     Contents of
c
c        RECORD 1
c                   reclen                Length of a record
c                   ifree                 First free record
c                   maxnsess              Maximum number of sessions
c                   nsessions             Number of sessions stored
c                   sesname(1:maxnsess)   Sessionnames
c                   sesrecord(1:maxnsess) First record numbers of all sessions
c
c        RECORD ISESSION
c                   isession              Record number of next session
c                                         isession = ifree if there are no
c                                         more sessions
c                   ndim                  Dimension of space
c                   ntrnsp                Number of transport equations
c                   nturb                 Number of turbulence equations
c                   itime                 Record number of first time step
c                   itopol                Topology array
c
c        RECORD ITIME
c                   time                  Time at which the solution was compute
c                   itimenext             record number where the next time
c                                         step is stored. itimenext = -1 if
c                                         it is the last time step.
c                   isol                  Record number where the first part of
c                                         the solution is stored
c
c     check with inquire if restartfile exist
c        if ( not exist) then
c           error
c        else
c           open restart file
c           read record 1
c     end if
c
c
c     irecses = first record number of OUTSESSION = <sessionname>
c
c     irecses = 0
c     do i=1, nsessions
c        if( sessionname.eq.sesname(i) ) then
c           irecses = sesrecord(i)
c           isession = i
c        end if
c     end do
c
c     if ( irecses = 0 )
c
c        then
c
c           session not found
c
c        else
c
c           session found
c           check if the session can be used
c           read from record(irecses) ndim, nturb, ntrnsp, array itopol
c
c          find rigth time step
c
c        end if
c
c
c
c    --- end of initialization, ifree2 is number of record from which
c        the solution must be read ( common cisnas )
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- check length of work array, which is equal to nrec2, which is
c         the recordlength of the restartfile

      if ( lwork.lt.(4+2*maxnsess) .or.
     +     lwork.lt.(5 + lentopol)      ) then
         call iseropen ( 'isinirdf' )
         call iserrsub ( 708, 0, 0, 0 )
         call iserclos ( 'isinirdf' )
         stop
      end if

      inquire( file=namef2, exist=exstat)

      if ( .not. exstat ) then

c     --- error restart file does not exist

         call iseropen ( 'isinirdf' )
         call iserrsub ( 719, 0, 0, 0 )
         call iserclos ( 'isinirdf' )
         go to 1000

      else

c     --- open file 2

         call isinifil(2)

c        --- read record 1

         lsess = maxnsess
         read(iref2, rec=1, iostat=ios ) nrec2, ifree2,
     +      lsess, nsessions, (sesname(i),i=1,lsess),
     +      (sesrecord(i),i=1,lsess)

         if ( ios.ne.0 ) then

c        --- error in reading record 1

            call iseropen ( 'isinirdf' )
            call iserrint ( iref2, 1 )
            call iserrint ( ios, 2 )
            call iserrsub ( 20, 2, 0, 0 )
            call iserclos ( 'isinirdf' )
            go to 1000
         end if

      end if

c     --- irecses = first record number of insession
c         find the session with the same name ( number)

      ipiincnd = iinput(3)
      sessionn = iinput(ipiincnd-1+2)
      irecses = 0
      do i =1, maxnsess
         if( sessionn.eq.sesname(i) ) then
            irecses = sesrecord(i)
            goto 10
         end if
      end do
10    if ( irecses.eq.0 ) then

c     --- SESSION NOT FOUND

         call iseropen ( 'isinirdf' )
         call iserrsub ( 709, 0, 0, 0 )
         call iserclos ( 'isinirdf' )
         go to 1000

      else

c     --- THIS IS AN EXISTING SESSION
c         read record isession

         read(iref2, rec=irecses, iostat=ios )
     +         (work(i),i=1,lwork)
         if ( ios.ne.0 ) then

c        --- error in reading record

            call iseropen ( 'isinirdf' )
            call iserrint ( iref2, 1 )
            call iserrint ( ios, 2 )
            call iserrsub ( 20, 2, 0, 0 )
            call iserclos ( 'isinirdf' )
            go to 1000
         end if

         if ( refine.lt.2 ) then

c        --- if refine=2, which means restarting from laminar to
c            turbulent there is no checking
c            check for differences in
c            ndim, ntrnsp, nturb and array itopol

            ndim   = iinput(57)
            ntrnsp = iinput(63)
            nturb  = iinput(70)

            if ( ndim.ne.work(2) ) then

c           --- error

               call iseropen ( 'isinirdf' )
               call iserrchr ( 'ndim', 1 )
               call iserrsub ( 21, 0, 0, 1 )
               call iserclos ( 'isinirdf' )
               go to 1000
            else if ( ntrnsp.ne.work(3) ) then

c           --- error

               call iseropen ( 'isinirdf' )
               call iserrchr ( 'ntrnsp', 1 )
               call iserrsub ( 21, 0, 0, 1 )
               call iserclos ( 'isinirdf' )
               go to 1000

            else if ( nturb.ne.work(4) ) then

c              --- error

               call iseropen ( 'isinirdf' )
               call iserrchr ( 'nturb', 1 )
               call iserrsub ( 21, 0, 0, 1 )
               call iserclos ( 'isinirdf' )
               go to 1000
            end if

         end if

         if ( refine.eq.0 ) then
            do i = 6, lentopol+5
               if( itopol(i-5).ne.work(i) ) then

c              --- error

                  call iseropen ( 'isinirdf' )
                  call iserrchr ( 'itopol', 1 )
                  call iserrsub ( 21, 0, 0, 1 )
                  call iserclos ( 'isinirdf' )
                  go to 1000
               end if
            end do
         end if

c        --- get pointer to  time step to be read
c            by investigating rinit

         iprintim = iinput(29)
         dt = rinput(iprintim -1 + 4)
         eps = 1d-2 * dt

         iprincnd = iinput(28)
         rinit = rinput(iprincnd-1+1)

         irectime = work(5)
         notfound = .true.
100      if( notfound ) then
            read(iref2, rec=irectime, iostat=ios ) time, itimenext,
     +                                             irecsol
            if ( rinit.gt.(time-eps) .and.
     +           rinit.lt.(time+eps)) then
               notfound = .false.
            else
               irectime = itimenext
               if( itimenext.eq.-1 ) then

c              --- error timestep cannot be found

                  call iseropen ( 'isinirdf' )
                  call iserrsub ( 710, 0, 0, 0 )
                  call iserclos ( 'isinirdf' )
                  go to 1000

               end if

            end if
            goto 100
         end if

         ifree2 = irecsol

      end if

1000  end
