      subroutine isinirst ( iinput, rinput, itopol, lentopol, work,
     +                      lwork, irecses, irectime, irectimef )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.3    date 02-04-1998 ( change maxnsess)
c        version 1.2    date 27-02-1998 ( change for existing session)
c        version 1.1    date 22-04-1996 ( places in iintim changed)
c        version 1.0    date 12-04-1995
c
c   copyright (c) 1994-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Startup of writing the restart file
c
c **********************************************************************
c
c                       KEYWORDS
c
c     backing_storage
c     restart
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), lentopol, itopol(lentopol), lwork, work(lwork),
     +        irecses, irectime, irectimef
      double precision rinput(*)

c     iinput     i    reserved
c     irecses    o    Record number of session record
c     irectime   o    Record number of time record
c     irectimef  o    Record number of first time record of the session
c     itopol     i    reserved
c     lentopol   i    Length of the itopol array
c     lwork      i    Length of array work
c     rinput     i    reserved
c     work       i/o  Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisnasch'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer maxnsess
      parameter ( maxnsess = 100 )
      logical exstat, opnstat, notfound
      integer forcedel, i, ios, ipiintim, iprintim, ireclbo, lsess,
     +        nsessions, sesname(maxnsess), sesrecord(maxnsess),
     +        isession, itimenext, ndim,  noverwrt, ntrnsp,
     +        nturb, restart, sessionn
      double precision  rinit, time

c     exstat      Existstatus variable
c     forcedel    Overwrite allowed forcedel=1 else forcedel=0
c     i           General loop variable
c     ios         Iostatus variable
c     ipiintim    Starting address of array IINTIM in array IINPUT
c     iprintim    Starting address of array RINTIM in array RINPUT
c     ireclbo     Record number of session last but one record
c     isession    Session number
c     itimenext   Record number of next time record
c     lsess       Help variable for the length of arrays sesname and sesrecord
c     maxnsess    Maximum number of sessions
c     ndim        Dimension of space (2 or 3)
c     notfound    Logical help variable
c     noverwrt    Number of sessions to be overwritten
c     nsessions   Number of sessions
c     ntrnsp      Number of transport equations
c     nturb       Number of extra transport equations for turbulence
c     opnstat     Openstatus variable
c     restart     Restart = -1 : no restart file will be written
c                 Restart = 0 : overwrite   restart = 1 : append
c     rinit       Start time for writing solutions to the restart file
c     sesname     Array with session numbers of all sessions
c     sesrecord   Array with first record numbers of all sessions
c     sessionn    Session number
c     time        Actual time of time step
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINIFIL  Open files
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     19      Error in writing record
c     20      Error in reading record
c     21      Difference found between current session and session
c             to which will be appended
c     707     No force deletion given for overwrite actions
c     708     Recordlength of restartfile to short
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The restart file is a direct access file to which the solution
c     will be written for restarting. The restart file contains a
c     list of sessions. Each session consists of a list of time steps.
c     The first record of the restart file contains information
c     about the list of sessions and about the division of the restart
c     file into records
c     purpose of this code:
c     To initialize the writing of the restart file by computing
c     the first record number of the indicated SESSION record and
c     the first record number of the first TIME record where
c     the next time step will be written.
c     Contents of
c        RECORD 1
c                   reclen                Length of a record
c                   ifree                 First free record
c                   maxnsess              Maximum number of sessions
c                   nsessions             Number of sessions stored
c                   sesname(1:maxnsess)   Sessionnames
c                   sesrecord(1:maxnsess) First record numbers of all sessions
c        RECORD ISESSION
c                   isession              Record number of next session
c                                         isession = ifree if there are no
c                                         more sessions
c                   ndim                  Dimension of space
c                   ntrnsp                Number of transport equations
c                   nturb                 Number of turbulence equations
c                   itime                 Record number of first time step
c                   itopol                Topology array
c        RECORD ITIME
c                   time                  Time at which the solution was compute
c                   itimenext             record number where the next time
c                                         step is stored. itimenext = -1 if
c                                         it is the last time step.
c                   isol                  Record number where the first part of
c                                         the solution is stored
c     check with inquire if restartfile exist already
c     check with inquire if restartfile opened already ( for initial conditions
c        if ( not opened ) then
c           open restartfile with call of isinifil(2)
c     end if
c     if (not exist ) then
c         initialize values for record 1
c     else
c         read record 1 from restart file
c     irecses = first record number of OUTSESSION = <sessionname>
c     irecses = 0
c     do i=1, nsessions
c        if( sessionname.eq.sesname(i) ) then
c           irecses = sesrecord(i)
c           isession = i
c        end if
c     end do
c     if ( irecses = 0 )
c     then
c        --- a new session
c        if ( restart = overwrite ) then
c           if ( nsessions.gt.0 ) then
c              --- irecses = first record number of the last session
c              irecses = sesrecord(nsesions)
c              sesname(nsessions) = sessionname
c              --- last session will be overwritten, this must be allowed
c              if ( NOT force_deletion ) then
c                 error('The last session in the restart file will
c                        be overwritten')
c                 error('This is not allowed, use FORCE_DELETION in
c                        the problem file')
c                 stop
c              end if
c           else
c             --- this session is the first session in the restartfile
c             --- irecses = first free record number in the restart file
c             irecses = ifree
c             nsessions = nsessions + 1
c             sesname(nsessions) = sessionname
c             sesrecord(nsessions) = irecses
c           end if
c        else
c        --- restart = append
c        --- irecses = first free record number in the restart file
c           irecses = ifree
c           nsessions = nsessions + 1
c           sesname(nsessions) = sessionname
c           sesrecord(nsessions) = irecses
c           --- now a new session wil be added ,
c           --- so read RECORD ISESSION_LAST_BUT_ONE and
c           --- WRITE it back with isession = irecses
c        end if
c        --- irectime = record number of the first time step of this session
c            irectime = irecses + 1
c            write session record
c     else
c     --- this is an existing record
c     --- irecses = number of this record
c         isession = number of this session
c         nsessions = total number of sessions written to the restart file
c     --- irectime    = record number of first time step of this record
c         irectime = irecses + 1
c         noverwritten = nsessions-isession  # no. of sessions to be overwritten
c        if ( restart = overwrite ) then
c        --- the current record will also be overwritten
c           noverwritten = noverwritten + 1
c           write session record
c        end if
c        if ( noverwritten > 0 AND NOT FORCE_DELETION ) then
c            error('The following sessions in the restart file will be overwritt
c            error('.......')
c            error('This is not allowed, use FORCE_DELETION in the problem file'
c            stop
c        end if
c        if ( restart = append ) then
c           --- check if we can append the current session to this session
c           --- read from record(irecses) ndim, nturb, ntrnsp, array itopol
c           if ( nturb, ntrnsp, ndim, itopol not equal to those
c                from the restarted session ) then
c             error('restart = append can only be used for exactly the same')
c             error('problem type, laminar <> turbulent, etc. ')
c          end if
c          --- irectime = get pointer to the first time step to be overwritten
c                      by investigating rinit, which is the first time
c                      at which the solution must be written to the
c                      restart file
c          read from record(irecses) irectime
c          while ( notfound ) do
c             read from record(irectime) time, itimenext
c             if ( time = rinit ) then
c                notfound = 'false'
c                irectime = itimenext
c             end if
c          end do
c       end if
c    end if
c    write record 1
c    --- end of initialization, irecses contains the first record number of
c    --- the session to be written, irectime contains the first record number
c    --- of the first time step to be written
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- check length of work array, which is equal to nrec2, which is
c         the recordlength of the restartfile


      if ( lwork.lt.(4+2*maxnsess) .or.
     +     lwork.lt.(5 + lentopol)      ) then
         call iseropen ( 'isinirst' )
         call iserrsub ( 708, 0, 0, 0 )
         call iserclos ( 'isinirst' )
         stop
      end if

      inquire( file=namef2, exist=exstat)

      inquire( unit=iref2, opened=opnstat)
      if ( .not. opnstat ) then
         call isinifil(2)
      end if

      if ( .not. exstat ) then

c     --- initialize values for record 1

         nsessions = 0
         do i=1, maxnsess
            sesname(i)   = 0
            sesrecord(i) = 0
         end do
         ifree2 =  2

      else

c     --- read record 1

         lsess = maxnsess
         read(iref2, rec=1, iostat=ios ) nrec2, ifree2,
     +      lsess, nsessions, (sesname(i),i=1,lsess),
     +      (sesrecord(i),i=1,lsess)

         if ( ios.ne.0 ) then

c        --- error in reading record 1

            call iseropen ( 'isinirst' )
            call iserrint ( iref2, 1 )
            call iserrint ( ios, 2 )
            call iserrsub ( 20, 2, 0, 0 )
            call iserclos ( 'isinirst' )
            stop
         end if

      end if

c     --- irecses = first record number of outsession

c         --- check if there is already a session with the same name ( number)

      ipiintim = iinput(4)
      sessionn = iinput(ipiintim-1+14)
      irecses = 0
      do i =1, maxnsess
         if( sessionn.eq.sesname(i) ) then
            irecses = sesrecord(i)
            isession = i
            goto 10
         end if
      end do
10    continue

      restart  = iinput(ipiintim-1+11)
      forcedel = iinput(ipiintim-1+13)

      if ( irecses.eq.0 ) then

c     --- A NEW SESSION

         if ( restart.eq.0 ) then

c        --- restart = overwrite

            if ( nsessions.gt.0 ) then

c           --- irecses = first record of the last session

               irecses = sesrecord(nsessions)
               sesname(nsessions) = sessionn

c              --- last record will be overwritten change value for ifree2

               ifree2 = sesrecord(nsessions)

c              --- last record will be overwritten this must be allowed

               if ( forcedel.eq.0 ) then

c              --- error

               call iseropen ( 'isinirst' )
               call iserrsub ( 707, 0, 0, 0 )
               call iserclos ( 'isinirst' )
               stop

               end if

            else

c           --- this session is the first session in the restart file
c               irecses = first free record number in the restart file

               irecses = ifree2
               nsessions = nsessions + 1
               sesname(nsessions) = sessionn
               sesrecord(nsessions) = irecses

            end if

         else

c        --- restart = append

c            --- irecses = first free record number in the restart file

            irecses = ifree2
            nsessions = nsessions + 1
            sesname(nsessions) = sessionn
            sesrecord(nsessions) = irecses

c           --- now a new session will be added, if nsessions > 1
c               then read record  : isession_last_but_one  and
c               write it back with isession = irecses

            if ( nsessions.gt.1 ) then

               ireclbo = sesrecord(nsessions-1)
               read(iref2, rec=ireclbo, iostat=ios )
     +               (work(i),i=1,lwork)
               if ( ios.ne.0 ) then

c              --- error in reading record

                  call iseropen ( 'isinirst' )
                  call iserrint ( iref2, 1 )
                  call iserrint ( ios, 2 )
                  call iserrsub ( 20, 2, 0, 0 )
                  call iserclos ( 'isinirst' )
                  stop
               end if
               work(1) = irecses
               write(iref2, rec=ireclbo, iostat=ios )
     +               (work(i),i=1,lwork)
               if ( ios.ne.0 ) then

c              --- error in writing record

                  call iseropen ( 'isinirst' )
                  call iserrint ( iref2, 1 )
                  call iserrint ( ios, 2 )
                  call iserrsub ( 19, 2, 0, 0 )
                  call iserclos ( 'isinirst' )
                  stop
               end if

            end if

         end if

c        --- irectime = record number of the first time step of
c                    this session

         irectime   = irecses + 1
         ifree2     = irecses + 1
         irectimef  = irectime

c        --- a NEW session so session record must be written

         ndim   = iinput(57)
         ntrnsp = iinput(63)
         nturb  = iinput(70)
         write(iref2, rec=irecses, iostat=ios ) ifree2, ndim,
     +         ntrnsp, nturb, irectime,(itopol(i),i=1,lentopol)
         if ( ios.ne.0 ) then

c        --- error in writing record

            call iseropen ( 'isinirst' )
            call iserrint ( iref2, 1 )
            call iserrint ( ios, 2 )
            call iserrsub ( 19, 2, 0, 0 )
            call iserclos ( 'isinirst' )
            stop
         end if

      else

c     --- THIS IS AN EXISTING SESSION

c         irectime = record number of the first time step of
c                    this session

         irectime   = irecses + 1
         irectimef  = irectime

c        --- number of sessions to be overwritten

         noverwrt = nsessions - isession
         nsessions = isession

         if ( restart.eq.0 ) then

c        --- restart = overwrite
c            the current record will also be overwritten

            noverwrt = noverwrt + 1

c           --- an old session will be overwritten with a NEW session so
c               session record must be written

            ndim   = iinput(57)
            ntrnsp = iinput(63)
            nturb  = iinput(70)
            ifree2 = irecses + 1
            write(iref2, rec=irecses, iostat=ios ) ifree2, ndim,
     +            ntrnsp, nturb, irectime,(itopol(i),i=1,lentopol)
            if ( ios.ne.0 ) then

c           --- error in writing record

               call iseropen ( 'isinirst' )
               call iserrint ( iref2, 1 )
               call iserrint ( ios, 2 )
               call iserrsub ( 19, 2, 0, 0 )
               call iserclos ( 'isinirst' )
               stop
            end if

         end if

         if ( noverwrt.gt.0 .and. forcedel.eq.0 ) then

c        --- error

            call iseropen ( 'isinirst' )
            call iserrsub ( 707, 0, 0, 0 )
            call iserclos ( 'isinirst' )
            stop

         end if

         if ( restart.eq.1 ) then

c        --- restart = append
c            check if we can append the current session to this session

c            --- read record isession

            read(iref2, rec=irecses, iostat=ios )
     +            (work(i),i=1,lwork)
            if ( ios.ne.0 ) then

c           --- error in reading record

               call iseropen ( 'isinirst' )
               call iserrint ( iref2, 1 )
               call iserrint ( ios, 2 )
               call iserrsub ( 20, 2, 0, 0 )
               call iserclos ( 'isinirst' )
               stop
            end if

c           --- check for differences in
c               ndim, ntrnsp, nturb and array itopol

            ndim   = iinput(57)
            ntrnsp = iinput(63)
            nturb  = iinput(70)

            if ( ndim.ne.work(2) ) then

c           --- error

               call iseropen ( 'isinirst' )
               call iserrchr ( 'ndim', 1 )
               call iserrsub ( 21, 0, 0, 1 )
               call iserclos ( 'isinirst' )
               stop
            else if ( ntrnsp.ne.work(3) ) then

c           --- error

               call iseropen ( 'isinirst' )
               call iserrchr ( 'ntrnsp', 1 )
               call iserrsub ( 21, 0, 0, 1 )
               call iserclos ( 'isinirst' )
               stop
            else if ( nturb.ne.work(4) ) then

c           --- error

               call iseropen ( 'isinirst' )
               call iserrchr ( 'nturb', 1 )
               call iserrsub ( 21, 0, 0, 1 )
               call iserclos ( 'isinirst' )
               stop
            end if

            do i = 6, lentopol+5
               if( itopol(i-5).ne.work(i) ) then

c              --- error

                  call iseropen ( 'isinirst' )
                  call iserrchr ( 'itopol', 1 )
                  call iserrsub ( 21, 0, 0, 1 )
                  call iserclos ( 'isinirst' )
                  stop
               end if
            end do

c           --- get pointer to first time step to be overwritten
c               by investigating rinit
c               if the last record is reached without finding
c               (time.ge.rinit) irectime = ifree2

            iprintim = iinput(29)
            rinit = rinput(iprintim-1+9)

            notfound = .true.
100         if( notfound ) then
               read(iref2, rec=irectime, iostat=ios ) time, itimenext
               if ( time.ge.rinit ) then
                  notfound = .false.
               else if ( itimenext.eq.-1 ) then
                  irectime = ifree2
                  notfound = .false.
               else
                  irectime = itimenext
               end if
               goto 100
            end if

            ifree2 = irectime
            work(1) = ifree2

c           --- write record isession

            write(iref2, rec=irecses, iostat=ios )
     +            (work(i),i=1,lwork)
            if ( ios.ne.0 ) then

c           --- error in writing record

               call iseropen ( 'isinirst' )
               call iserrint ( iref2, 1 )
               call iserrint ( ios, 2 )
               call iserrsub ( 19, 2, 0, 0 )
               call iserclos ( 'isinirst' )
               stop
            end if

         end if

      end if

c     --- write record 1

      write(iref2, rec=1, iostat=ios ) nrec2, ifree2,
     +   maxnsess, nsessions, (sesname(i),i=1,maxnsess),
     +   (sesrecord(i),i=1,maxnsess)
      if ( ios.ne.0 ) then

c     --- error in writing record 1

         call iseropen ( 'isinirst' )
         call iserrint ( iref2, 1 )
         call iserrint ( ios, 2 )
         call iserrsub ( 19, 2, 0, 0 )
         call iserclos ( 'isinirst' )
         stop
      end if

      end
