      subroutine isout07 ( work, solut, vector, mcom, mcav, ni, nj,
     +                     rincom, nvirtual, mscal, istartup, rincav)
c ======================================================================
c
c     programmer  Hester Bijl
c     version     1.6   25-05-1999  Duncan: fixed bug in unscaled Mach
c                                           number
c     version     1.5   03-03-1999  Duncan: new call to isdstfll/isdstlss
c     version     1.4   27-07-1998  Duncan: param. istartup added
c     version     1.3   05-03-1998  Duncan: extension for cavity
c     version     1.2   12-11-1997  Hester: param. mscal added
c     version     1.1   29-04-1996  Hester: Changed lower index to 0,
c                                           upper indices to ni+1, nj+1.
c     version     1.0   11-04-1996
c
c
c   copyright (c) 1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute several quantities for output purpose for compressible comput.
c
c **********************************************************************
c
c                       KEYWORDS
c
c   write
c   compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer mcom, ni, nj, nvirtual, mscal, mcav, istartup
      double precision work(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 vector(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2),
     +                 rincom(*), rincav(*)

c     istartup  i    If 1 the cavity calculation is in the startup procedure,
c                    if 0 not
c     mcav      If 1 a cavity model is used, if 0 not
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     mscal     i    Indicates use of non-dimensionalisation for compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rincav    i    reserved
c     rincom    i    reserved
c     solut     i    reserved
c     vector    i    Input flow field
c     work      o    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp
      double precision mach2, gamcm, cons, cons1, spos2,
     +                 speed2,spos2inv

c     cons      Constant used in equation of state for compressible comput.
c     cons1     Constant used in equation of state for compressible comput.
c     gamcm     Gas constant gamma (for compressible comput.)
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     mach2     Square of the mach number (for compressible comput.)
c     speed2    Speed squared
c     spos2     Velocity of sound squared
c     spos2inv  1/(Velocity of sound) squared
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISDSTFLL   derivative of dimensional equation of state
c     ISDSTLSS   derivative of dimensionless equation of state
      double precision  isdstlss, isdstfll
c **********************************************************************
c
c                       PSEUDO CODE
c
c     if ichois =1
c        output of density, from array eddy
c     else if ichois = 2,
c        if mcom=1
c           output of scaled Mach nr: mach = h/cons1,
c           since mach = gamcm * dp/drho with h constant.
c        else if mcom=2
c            output of unscaled Mach nr: mach = h/cons
c        end if
c     end if
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Initialize work array

      do jp = 1-nvirtual, nj+1+nvirtual
         do ip = 1-nvirtual, ni+1+nvirtual
            work(ip,jp) = 0d0
         end do
      end do

      gamcm = rincom(1)
      mach2 = rincom(2)**2

c     --- compute mach number and put it into work array

      if( mcom.eq.1 ) then

         if(mcav.eq.0) then

            if (mscal.eq.0) then

              if (rincom(2).gt.0d0) then

c             --- unscaled computation

c                 --- if mach_infty (rincom(2)) is equal to 0, the mach number
c                 in
c                 the
c                    entire flow is 0.

                cons = 1d0 + ((gamcm - 1d0)*mach2)/2d0
                cons1 = mach2 / cons

                do jp = 0, nj+1
                  do ip = 0, ni+1

                     spos2 = solut(ip,jp)/cons1

                     speed2 = vector(ip,jp,1)**2 +
     +                    vector(ip,jp,2)**2

                     work(ip,jp) = sqrt(abs(speed2/spos2))

                  end do
                end do

              end if

            else if (mscal.eq.1) then

              cons  =(gamcm - 1d0)

c             --- compute unscaled Mach number

              do jp = 0, nj+1
                do ip = 0, ni+1

                  spos2 = cons*solut(ip,jp)

                  speed2 = vector(ip,jp,1)**2 +
     +                    vector(ip,jp,2)**2

                  work(ip,jp) = sqrt(abs(speed2/spos2))

                end do
              end do

            end if

         else

c        --- compute the Mach number out of the equation of state

            if (mscal.eq.0) then
              do jp = 0, nj+1
                do ip = 0, ni+1

                  spos2inv = isdstlss(solut(ip,jp), rincom,
     +                                             istartup, rincav)
                  speed2 = vector(ip,jp,1)**2 +
     +                    vector(ip,jp,2)**2

                  work(ip,jp) = sqrt(abs(speed2*spos2inv))

                end do
              end do

            else if (mscal.eq.1) then

              do jp = 0, nj+1
                do ip = 0, ni+1

                  spos2inv = isdstfll(solut(ip,jp),istartup, rincav)
                  speed2 = vector(ip,jp,1)**2 +
     +                    vector(ip,jp,2)**2

                  work(ip,jp) = sqrt(abs(speed2*spos2inv))
                end do
              end do

            end if

         end if

      end if

      end
