      subroutine isoutpres ( time, isiinput, isrinput, iscoor, issolut,
     +                       isitopol, indorien, isoutput, iseq1 )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 27-01-1999
c
c
c   copyright (c) 1999-1999 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Write the before last time level in the subdomain solution arrays
c     to the output file. This subroutine writes the pressure only
c **********************************************************************
c
c                       KEYWORDS
c
c   output
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iscoor(*), isiinput(*), isrinput(*), isitopol,
     +        issolut(*), isoutput, indorien(*), iseq1
      double precision time

c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iseq1     o    Sequence number of output quantity
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isoutput  i    Pointer to a local output array containing quantities in
c                    the vertices
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     time      i    Actual time of time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipcoor, ipiinput, ipitopol, ipoutput,
     +        itype1, ipsolut, iprinput, nblocks, ndegfd, ipiinbc,
     +        ndim, ni, nj, nk, ntimlv, nvirtual, npointsv,
     +        mcom, ipiincom, iprincom, ipwork1, idegfd,
     +        itypepost, mscal
      character * 30 name

c     iblock    Block sequence number
c     idegfd    Loop variable with respect to a given unknown
c     ipcoor    Starting address of array coor in buffer
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipoutput  Starting address of a local output array containing quantities
c               in the vertices
c     iprincom  Starting address of array RINCOM in array RINPUT
c     iprinput  Starting address of array rinput in buffer
c     ipsolut   Starting address of array solut in buffer
c     ipwork1   Starting address of array work1 in buffer
c     itype1    Indicates the type of output vector
c               1: scalar,  2: vector,  3: tensor
c     itypepost Indicates the type of postprocessing required
c               0: standard Newton    1: linear in computational domain
c     mcom      If zero the flow is incompressible else (1) compressible
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     name      Name of file
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      This solution and some related quantities are written to the file
c      isnascomp.out
c      Depending on the parameters in the file isnas.env this file is
c      formatted or unformatted
c      The records in the file isnascomp.out are filled in the following way:
c      -  ndim, nblocks
c      For each array to be written:
c         -  Name of vector
c         -  time, iseq1, itype1    (itype1=1:scalar, 2:vector)
c         For each block:
c            - iblock, nx, ny, nz
c            For all components
c               - values of vector in the sequence (1:nx+1,1:ny+1,1:nz+1)
c     The blocks are written in the natural sequence, except for the stream
c     function, which uses a special block ordering
c     The sequence of the vectors to be written is:
c        velocity
c        pressure
c        stream function
c        transport quantities
c        turbulent quantities
c        extra quantities
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISCPPER2  Compute mean values on common sides of periodical boundaries
c     ISDIMSC   Dimensionalise the unknowns for postprocessing
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISOUTV1   interpolate quantities from centroids to vertices using
c               vitual cells
c     ISOUTV13  interpolate quantities from centroids to vertices 3D using
c               vitual cells
c     ISPOSOR1  Change directions in solution array
c     ISPRESEX  Extrapolate the pressure along a fixed wall to the "tip"
c     ISPRINS2  Print one block of scalars in 2D
c     ISPRINS3  Print one block of scalars in 3D
c     ISTMSPAC  Create space for temporary array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Allocate local arrays
c     DO iblock = 1, nblocks
c        interpolate the pressure unknowns to the vertices (isout01)
c     ENDDO
c     Delete local arrays
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isoutpres')

c     --- determine nblocks

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol)

c     --- compute some general parameters

      call isactive(isiinput(1))
      ipiinput   = isingetp(isiinput(1))
      call isactive(isrinput(1))
      iprinput   = isdpgetp(isrinput(1))
      ipiinbc    = ibuffr(ipiinput +  2 - 1)
      ipiincom   = ibuffr(ipiinput +  9 - 1)
      iprincom   = ibuffr(ipiinput + 34 - 1)
      ndegfd     = ibuffr(ipiinput + 55 - 1)
      ndim       = ibuffr(ipiinput + 57 - 1)
      ntimlv     = ibuffr(ipiinput + 64 - 1)
      itypepost  = ibuffr(ipiinput + 67 - 1)
      mcom       = ibuffr(ipiinput + ipiincom - 1 )
      mscal      = ibuffr(ipiinput + ipiincom + 4 - 1 )

      nvirtual   = ibuffr(ipiinput + 54 - 1)

c     --- Write pressure

      iseq1 = 2
      itype1 = 1
      idegfd = ndim+1
      name = 'pressure'
      if ( iref74.gt.0 ) then
         write(iref74,110 ) name, time, iseq1, itype1
110      format(a/f12.5,2i3,12x,a)
      else
         write(-iref74) name
         write(-iref74) time, iseq1, itype1
      end if
      do iblock = 1, nblocks

c     --- determine subdomain parameters

         call isactive(isiinput(iblock))
         ipiinput = isingetp(isiinput(iblock))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         if ( ndim.eq.2 ) then
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
         else
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                 (nk+1+2*nvirtual)
         end if

c        --- Write first record of block

         if ( iref74.gt.0 ) then
            write(iref74,120 ) iblock, ni, nj, nk
         else
            write(-iref74) iblock, ni, nj, nk
120         format(4i8,12x,a)
         end if

c        --- activate arrays

         call isactive(iscoor(iblock))
         call isactive(isitopol)
         call isactive(isoutput)
         call isactive(issolut(iblock))
         if ( indorien(iblock).eq.-1 )
     +      call istmspac((ni+1)*(nj+1)*intlen,ipwork1)

c        --- compute pointers

         ipcoor   = isdpgetp(iscoor(iblock))
         ipsolut  = isdpgetp(issolut(iblock)) +
     +                 (ntimlv-2)*npointsv*ndegfd
         ipitopol = isingetp(isitopol)
         ipoutput = isdpgetp(isoutput)

c        --- interpolate pressure to the vertices

         if ( ndim.eq.2 ) then

c        --- pressure, 2d

            call isoutv1 ( ni, nj, nvirtual, buffer(ipcoor),
     +                     buffer(ipsolut + npointsv*(idegfd-1)),
     +                     buffer(ipoutput), itypepost )

c           --- Extrapolate pressure along wing to the tip

            call ispresex ( ibuffr(ipiinput),
     +                      ibuffr(ipiinput-1+ipiinbc),
     +                      buffer(ipoutput),  ni, nj, nvirtual,
     +                      buffer(ipcoor) )

            if ( mcom.eq.1 .and. mscal.eq.0 ) then
                call isdimsc(buffer(ipoutput),buffer(ipoutput),
     +                buffer(iprinput+iprincom-1),ni,nj,1)
            end if

c           --- Compute mean value for periodical boundaries

            call iscpper2 ( ibuffr(ipiinput),
     +                      ibuffr(ipiinput-1+ipiinbc),
     +                      buffer(ipoutput),  ni, nj, iblock )

            if ( indorien(iblock).eq.-1 ) then
               call isposor1 ( buffer(ipoutput),
     +                         buffer(ipwork1), ni, nj )
               call isprins2 ( nj, ni, buffer(ipoutput) )
            else
               call isprins2 ( ni, nj, buffer(ipoutput) )
            end if

         else

c        --- pressure, 3d

            call isoutv13 ( ni, nj, nk, nvirtual, buffer(ipcoor),
     +                      buffer(ipsolut + npointsv*(idegfd-1)),
     +                      buffer(ipoutput) )
            call isprins3 ( ni, nj, nk, buffer(ipoutput) )

         end if

      end do
      call iserclos('isoutpres')

      end
