      subroutine isoutturb ( time, isiinput, isrinput, iscoor, issolut,
     +                       isitopol, indorien, isoutput, iseq1,
     +                       iseddy, isgeom )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 27-01-1999
c
c
c   copyright (c) 1999-1999 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Write the before last time level in the subdomain solution arrays
c     to the output file. This subroutine writes special quantities related
c     to the turbulence equations
c **********************************************************************
c
c                       KEYWORDS
c
c   output
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iscoor(*), isiinput(*), isrinput(*), isitopol,
     +        issolut(*), isoutput, indorien(*), iseq1, iseddy(*),
     +        isgeom(*)
      double precision time

c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     iseq1     o    Sequence number of output quantity
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isoutput  i    Pointer to a local output array containing quantities in
c                    the vertices
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     time      i    Actual time of time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipcoor, ipiinput, ipitopol, ipoutput, i, j,
     +        itype1, ipsolut, nblocks, ndegfd, ndim, ni, nj, nk,
     +        nvirtual, npointsv,ipwork1, iupper, ipiinbc,
     +        itypepost, mturb, ipitur, nturb, ipgeom, ipeddy,
     +        ipiindsc, iprintur, ipsubf, iprinput, ipwork, iswork,
     +        lwork, leniindsc, maxnsubf, maxvpnts, nfaces, ntrnsp,
     +        ipinvjac, ipnsubf
      character * 30 name

c     i         General loop variable
c     iblock    Block sequence number
c     ipcoor    Starting address of array coor in buffer
c     ipeddy    Starting address of array eddy in buffer
c     ipgeom    Starting address of array geom in buffer
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipinvjac  Starting address of array invjac in geom
c     ipitopol  Starting address of array itopol in ibuffr
c     ipitur    Starting address of array iintur in array iinput
c     ipnsubf   Starting address of the nsubfaces array
c     ipoutput  Starting address of a local output array containing quantities
c               in the vertices
c     iprinput  Starting address of array rinput in buffer
c     iprintur  Starting address of array rintur in array RINPUT
c     ipsolut   Starting address of array solut in buffer
c     ipsubf    Starting address of the subfaces array
c     ipwork    Starting address of array work in buffer
c     ipwork1   Starting address of array work1 in buffer
c     iswork    Sequence number of array work
c     itype1    Indicates the type of output vector
c               1: scalar,  2: vector,  3: tensor
c     itypepost Indicates the type of postprocessing required
c               0: standard Newton    1: linear in computational domain
c     iupper    Last element in inparray to be read
c     j         General loop variable
c     leniindsc Length of first component of array iindsc
c     lwork     Length of array work
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     maxvpnts  Maximum number of points per block including virtual ones
c     mturb     Indicates type of turbulence modelling
c     name      Name of file
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      This solution and some related quantities are written to the file
c      isnascomp.out
c      Depending on the parameters in the file isnas.env this file is
c      formatted or unformatted
c      The records in the file isnascomp.out are filled in the following way:
c      -  ndim, nblocks
c      For each array to be written:
c         -  Name of vector
c         -  time, iseq1, itype1    (itype1=1:scalar, 2:vector)
c         For each block:
c            - iblock, nx, ny, nz
c            For all components
c               - values of vector in the sequence (1:nx+1,1:ny+1,1:nz+1)
c     The blocks are written in the natural sequence, except for the stream
c     function, which uses a special block ordering
c     The sequence of the vectors to be written is:
c        velocity
c        pressure
c        stream function
c        transport quantities
c        turbulent quantities
c        extra quantities
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISCPPER2  Compute mean values on common sides of periodical boundaries
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISOUT01   interpolate quantities from centroids to vertices
c     ISOUT013  interpolate quantities from centroids to vertices 3D
c     ISOUT261  Compute several quantities for output purpose in case of
c               turbulence models (2D)
c     ISOUT361  Compute several quantities for output purpose in case of
c               turbulence models (3D)
c     ISPOSOR1  Change directions in solution array
c     ISPRINS2  Print one block of scalars in 2D
c     ISPRINS3  Print one block of scalars in 3D
c     ISSTRES2  Compute cartesian Reynolds stresses in R2
c     ISSTRES3  Compute cartesian Reynolds stresses in R3
c     ISTMSPAC  Create space for temporary array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Allocate local arrays
c     DO iblock = 1, nblocks
c        interpolate the turbulent unknowns to the vertices (isout01)
c     ENDDO
c     Delete local arrays
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isoutturb')

c     --- determine nblocks

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol)
      nfaces   = ibuffr(ipitopol + 1)
      maxnsubf = ibuffr(ipitopol + 2)

c     --- compute some general parameters

      call isactive(isiinput(1))
      ipiinput   = isingetp(isiinput(1))
      call isactive(isrinput(1))
      ipiinbc    = ibuffr(ipiinput +  2 - 1)
      ipiindsc   = ibuffr(ipiinput +  6 - 1)
      ipitur     = ibuffr(ipiinput +  8 - 1)
      iprintur   = ibuffr(ipiinput + 31 - 1)
      ndegfd     = ibuffr(ipiinput + 55 - 1)
      ndim       = ibuffr(ipiinput + 57 - 1)
      ntrnsp     = ibuffr(ipiinput + 63 - 1)
      itypepost  = ibuffr(ipiinput + 67 - 1)
      maxvpnts   = ibuffr(ipiinput + 69 - 1)
      nturb      = ibuffr(ipiinput + 70 - 1)
      mturb      = ibuffr(ipiinput + ipitur -1 + 2 - 1)

      nvirtual   = ibuffr(ipiinput + 54 - 1)
      iswork = 0

c     --- mturb>0, compute and print specific quantities with respect to
c         turbulence

      iupper = 3
      if (mturb.eq.4) iupper = 1
      do i = 1, iupper
         iseq1 = iseq1+1
         itype1 = 1
         if ( i.eq.1 ) then
            name = 'turbulent_viscosity'
         else if ( i.eq.2 ) then
            name = 'turbulent_intensity'
         else
            name = 'length_scale'
         end if
         if ( iref74.gt.0 ) then
            write(iref74,110 ) name, time, iseq1, itype1
110         format(a/f12.5,2i3,12x,a)
         else
            write(-iref74) name
            write(-iref74) time, iseq1, itype1
         end if
         do iblock = 1, nblocks

c        --- determine subdomain parameters

            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))
            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            if ( ndim.eq.2 ) then
               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            else
               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                    (nk+1+2*nvirtual)
            end if

c           --- Write first record of block

            if ( iref74.gt.0 ) then
               write(iref74,120 ) iblock, ni, nj, nk
120            format(4i8,12x,a)
            else
               write(-iref74) iblock, ni, nj, nk
            end if

c           --- activate arrays
c               ipsolut refers to second time level and k and eps only
c               This means that we must skip a complete time level +
c               all unknowns except the last two

            call isactive(iscoor(iblock))
            call isactive(isitopol)
            call isactive(isoutput)
            call isactive(iseddy(iblock))
            call isactive(issolut(iblock))
            call isdcreat ( iswork, maxvpnts, 'work' )
            if ( indorien(iblock).eq.-1 )
     +         call istmspac((ni+1)*(nj+1)*intlen,ipwork1)

c           --- compute pointers

            ipcoor   = isdpgetp(iscoor(iblock))
            ipitopol = isingetp(isitopol)
            ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
            ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
            ipwork   = isdpgetp(iswork)
            ipeddy   = isdpgetp(iseddy(iblock))
            ipsolut  = isdpgetp(issolut(iblock)) + npointsv*ndegfd +
     +                 npointsv*(ndegfd-2)
            ipoutput = isdpgetp(isoutput)
            iprinput = isdpgetp(isrinput(1))

c           --- compute and write turbulent quantities

            if ( ndim.eq.2 ) then
               call isout261 ( buffer(ipwork), buffer(ipsolut),
     +                         mturb, ni, nj, buffer(ipeddy),
     +                         buffer(iprinput+iprintur-1),
     +                         nvirtual, i )
               call isout01 ( iblock, ni, nj, nvirtual,
     +                        nblocks, nfaces, maxnsubf,
     +                        ibuffr(ipnsubf), ibuffr(ipsubf),
     +                        buffer(ipcoor), buffer(ipwork),
     +                        buffer(ipoutput), itypepost )

c              --- Compute mean value for periodical boundaries

               call iscpper2 ( ibuffr(ipiinput),
     +                         ibuffr(ipiinput-1+ipiinbc),
     +                         buffer(ipoutput),  ni, nj, iblock )

               if ( indorien(iblock).eq.-1 ) then
                  call isposor1 ( buffer(ipoutput),
     +                            buffer(ipwork1), ni, nj )
                  call isprins2 ( nj, ni, buffer(ipoutput) )
               else
                  call isprins2 ( ni, nj, buffer(ipoutput) )
               end if
            else
               call isout361 ( buffer(ipwork), buffer(ipsolut),
     +                         mturb, ni, nj, nk, buffer(ipeddy),
     +                         buffer(iprinput+iprintur-1),
     +                         nvirtual, i )
               call isout013 ( ni, nj, nk, nvirtual, buffer(ipcoor),
     +                         buffer(ipwork), buffer(ipoutput) )
               call isprins3 ( ni, nj, nk, buffer(ipoutput) )
            end if
            call isdelarr(iswork)

         end do

      end do

      if (mturb.ne.4) then

c     --- Write Reynolds stresses

         iseq1 = iseq1+1
         itype1 = 3
         name = 'Reynolds_stresses'
         if ( iref74.gt.0 ) then
            write(iref74,110 ) name, time, iseq1, itype1
         else
            write(-iref74) name
            write(-iref74) time, iseq1, itype1
         end if
         do iblock = 1, nblocks

c        --- determine subdomain parameters

            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))
            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            if ( ndim.eq.2 ) then
               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            else
               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                    (nk+1+2*nvirtual)
            end if

c           --- Write first record of block

            if ( iref74.gt.0 ) then
               write(iref74,120 ) iblock, ni, nj, nk
            else
               write(-iref74) iblock, ni, nj, nk
            end if

c           --- activate arrays

            call isactive(iscoor(iblock))
            call isactive(isitopol)
            call isactive(isoutput)
            call isactive(iseddy(iblock))
            call isactive(isgeom(iblock))
            call isactive(issolut(iblock))
            if ( ndim.eq.2 ) then
               lwork = 1+maxvpnts*47
            else
               lwork = 1+maxvpnts*142
            end if
            call isdcreat ( iswork , lwork , 'work' )
            if ( indorien(iblock).eq.-1 )
     +         call istmspac((ni+1)*(nj+1)*intlen,ipwork1)
            ipwork   = isdpgetp(iswork)
            ipcoor   = isdpgetp(iscoor(iblock))
            ipsolut  = isdpgetp(issolut(iblock)) + npointsv*ndegfd
            ipoutput = isdpgetp(isoutput)
            iprinput = isdpgetp(isrinput(1))
            ipeddy   = isdpgetp(iseddy(iblock))
            ipitopol = isingetp(isitopol)
            ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
            ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
            ipgeom   = isdpgetp(isgeom(iblock))
            leniindsc = ibuffr(ipiinput + 106 - 1)

c           --- compute some specified quantities (Reynolds stresses etc)

            if ( ndim.eq.2 ) then

               ipinvjac = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*4
               ipgeom = ipgeom + ipinvjac
               call isstres2 ( buffer(ipwork), buffer(ipsolut),
     +                         buffer(ipcoor), buffer(ipeddy),
     +                         buffer(ipgeom), lwork, ndegfd, ni, nj,
     +                         npointsv, ibuffr(ipiinput+ipiindsc-1),
     +                         nvirtual, leniindsc, ntrnsp, nturb,
     +                         buffer(iprinput+iprintur-1) )
               do j = 1, 3

c              --- interpolate quantities to the vertices

                  call isout01 ( iblock, ni, nj, nvirtual,
     +                           nblocks, nfaces, maxnsubf,
     +                           ibuffr(ipnsubf), ibuffr(ipsubf),
     +                           buffer(ipcoor),
     +                           buffer(ipwork+npointsv*(j-1)),
     +                           buffer(ipoutput), itypepost )
                  if ( indorien(iblock).eq.-1 ) then
                     call isposor1 ( buffer(ipoutput),
     +                               buffer(ipwork1), ni, nj )
                     call isprins2 ( nj, ni, buffer(ipoutput) )
                  else
                     call isprins2 ( ni, nj, buffer(ipoutput) )
                  end if

               end do

            else

               ipinvjac = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                    (nk+1+2*nvirtual)*8
               ipgeom = ipgeom + ipinvjac
               call isstres3 ( buffer(ipwork), buffer(ipsolut),
     +                         buffer(ipcoor), buffer(ipeddy),
     +                         buffer(ipgeom), lwork, ndegfd, ni, nj,
     +                         nk, npointsv,
     +                         ibuffr(ipiinput+ipiindsc-1),
     +                         nvirtual, leniindsc, ntrnsp, nturb,
     +                         buffer(iprinput+iprintur-1) )
               do j = 1, 6

                  call isout013 ( ni, nj, nk, nvirtual,
     +                            buffer(ipcoor),
     +                            buffer(ipwork+npointsv*(j-1)),
     +                            buffer(ipoutput) )
                  call isprins3 ( ni, nj, nk, buffer(ipoutput) )

               end do

            end if

            call isdelarr(iswork)

         end do
      end if

      call iserclos('isoutturb')

      end
