      subroutine isoutv1 ( ni, nj, nvirtual, coor, scalar, output,
     +                     itypepost )
c ======================================================================
c
c     programmer   Guus Segal
c     version      1.0  20-01-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   This subroutine performs the interpolation of a scalar quantity
c   defined in the centroids of the 2D cells, to the vertices
c   Bilinear interpolation  is used.
c   The subroutine assumes that virtual cells have been filled
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, itypepost
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1:2)
      double precision output(1:ni+1,1:nj+1)
      double precision scalar(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)

c     coor      i    reserved
c     itypepost i    Indicates the type of postprocessing required
c                    0: standard Newton    1: linear in computational domain
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     output    o    Cartesian components of the flow field in the vertices
c     scalar    i    Quantities in the centroids
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j
      double precision uloc, vloc

c     i         General loop variable
c     j         General loop variable
c     uloc      Local u-coordinate for the interpolation
c     vloc      Local v-coordinate for the interpolation
c **********************************************************************
c
c                       I/O
c
c      -
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISNTON    Compute local coordinates for the interpolation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( itypepost.eq.0 ) then

c     --- Standard situation, Newton is applied
c         First, consider all cells as if they were internal

         do j = 1, nj+1
            do i = 1, ni+1
               call isnton(ni,nj,nvirtual,coor,i,j,i,j,uloc,vloc)
               output(i,j) =      uloc *     vloc  * scalar(i  ,j  )+
     +                            uloc *(1d0-vloc) * scalar(i  ,j-1)+
     +                       (1d0-uloc)*     vloc  * scalar(i-1,j  )+
     +                       (1d0-uloc)*(1d0-vloc) * scalar(i-1,j-1)
            end do
         end do

      else

c     --- Special situation, Linear interpolation in computational
c         domain is applied

         do j = 1, nj+1
            do i = 1, ni+1
               output(i,j) = 0.25d0* ( scalar(i,j) + scalar(i,j-1) +
     +                                 scalar(i-1,j) + scalar(i-1,j-1) )
            end do
         end do

      end if

      end
