      subroutine isprar02 ( array, nvirtual, ni, nj, nk, ndim )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.1    date 21-02-95 (Possibility of permutation added)
c        version 1.0    date 01-06-94
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c     Print the contents of the array array defined on a three-dimensional grid
c     Actual printing subroutine
c ********************************************************************
c
c                      KEYWORDS
c
c    3d
c    print
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer nvirtual, ni, nj, nk, ndim
      double precision array(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ndim)

c     array     i    Double precision array
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
c ********************************************************************
c
c                      LOCAL PARAMETERS
c
      integer i, j, k, l, iperm(3)
      double precision matmax
      logical null

c     i         General loop variable
c     iperm     Array of permutations of the numbers 1, 2 and 3
c               Contents: 1, 2, 3, 1, 2
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c     matmax    Maximum absolute value of matrix elements
c     null      If true the complete row of the matrix is zero
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c     For k := 1-nvirtual (1) nk+1+nvirtual do
c        For j := 1-nvirtual (1) nj+1+nvirtual do
c           For i := 1-nvirtual (1) ni+1+nvirtual do
c              write i,j,k coor(i,j,k,l) (l=1(1)ndim)
c ======================================================================
c
      matmax = 0d0
      do i = 1-nvirtual, ni+1+nvirtual
         do j = 1-nvirtual, nj+1+nvirtual
            do k = 1-nvirtual, nk+1+nvirtual
               do l = 1, ndim
                     matmax = max ( matmax, abs(array(i,j,k,l)) )
               end do
            end do
         end do
      end do
      matmax = matmax*epsmac
      if ( iprseq.eq.1 ) then

c        --- Standard sequence

         do k = 1-nvirtual, nk+1+nvirtual
            do j = 1-nvirtual, nj+1+nvirtual
               do i = 1-nvirtual, ni+1+nvirtual
                  null = .true.
                  do l = 1, ndim
                     if ( abs(array(i,j,k,l)).gt.matmax ) null = .false.
                  end do
                  if ( .not. null ) then
                     write(irefwr,100) i,j,k,
     +               (array(i,j,k,l), l=1,ndim)
100                  format(1x,ss,3i4,':',5(1pe12.3e2))
                  end if
               end do
            end do
         end do

      else if ( iprseq.eq.2 ) then

c     --- Sequence j,k,i

         if ( ndim.eq.1 ) then
            iperm(1) = 1
         else
            iperm(1) = 2
            iperm(2) = 3
            iperm(3) = 1
         end if
         do i = 1-nvirtual, ni+1+nvirtual
            do k = 1-nvirtual, nk+1+nvirtual
               do j = 1-nvirtual, nj+1+nvirtual
                  null = .true.
                  do l = 1, ndim
                     if ( abs(array(i,j,k,l)).gt.matmax ) null = .false.
                  end do
                  if ( .not. null ) then
                     write(irefwr,100) j,k,i,
     +               (array(i,j,k,iperm(l)), l=1,ndim)
                  end if
               end do
            end do
         end do

      else

c     --- Sequence k,i,j

         if ( ndim.eq.1 ) then
            iperm(1) = 1
         else
            iperm(1) = 3
            iperm(2) = 1
            iperm(3) = 2
         end if
         do j = 1-nvirtual, nj+1+nvirtual
            do i = 1-nvirtual, ni+1+nvirtual
               do k = 1-nvirtual, nk+1+nvirtual
                  null = .true.
                  do l = 1, ndim
                     if ( abs(array(i,j,k,l)).gt.matmax ) null = .false.
                  end do
                  if ( .not. null ) then
                     write(irefwr,100) k,i,j,
     +               (array(i,j,k,iperm(l)), l=1,ndim)
                  end if
               end do
            end do
         end do

      end if
      end
