      subroutine isprar04 ( matrix, ni, nj, nk, nconct, ndim, itypmat )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 18-07-1996 Correction layout
c        version 1.0    date 17-02-1995
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Print the contents of the array matrix defined on a three-dimensional grid
c     Actual printing subroutine
c **********************************************************************
c
c                       KEYWORDS
c
c    3d
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nconct, ndim, itypmat
      double precision matrix(ni+1,nj+1,nk+1,ndim,nconct)

c     itypmat        Type of matrix. Possible values
c                    1:  Momentum matrix
c                    2:  Pressure or transport matrix
c                    3:  Gradient matrix
c     matrix    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, l, idim, iworkgen(51,3), maxdx, m, jcount,
     +        mmct(3,3,-2:2,-2:2,-2:2), gmct(3,1,-2:2,-2:2,-2:2),
     +        smct(1,1,-2:2,-2:2,-2:2), i3, j3, k3, mdim, indmat,
     +        iperm(3), m1, jdim
      logical null, firstcall
      double precision work(51), c, matmax
      save firstcall, mmct, gmct, smct

c     c         General help variable
c     firstcall Indicator to find the first call of the routine
c     gmct      Matrix code table gradient matrix see array infmg
c     i         General loop variable
c     i3        General loop variable
c     idim      Loop variable used for dimension
c     indmat    Index of matrix entry
c     iperm     Array of permutations of the numbers 1, 2 and 3
c               Contents: 1, 2, 3, 1, 2
c     iworkgen  General integer work array.
c               See pseudo code for the contents
c     j         General loop variable
c     j3        General loop variable
c     jcount    General counter
c     jdim      Loop variable used for dimension
c     k         General loop variable
c     k3        General loop variable
c     l         General loop variable
c     m         General loop variable
c     m1        General loop variable
c     matmax    Maximum absolute value of matrix elements
c     maxdx     Maximum relative shift between cell indices
c     mdim      Dimension
c     mmct      Matrix code table transport equation see array infmt
c     null      If true the complete row of the matrix is zero
c     smct      Column index+row degree of freedom to connection
c     work      Work array
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISFILM03  Makes the table of matrix column numbers in R3
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   if ( layout=1 ) then
c      print correct matrix depending on sequence
c   else ( layout=2 )
c      compute mapping of shifts to matrix entries
c      print matrix correct depending on sequence
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data firstcall / .true. /
c ======================================================================
c
c     --- Compute maximum matrix element

      matmax = 0d0
      do i = 1, ni+1
         do j = 1, nj+1
            do k = 1, nk+1
               do idim = 1, ndim
                  do l = 1, nconct
                     matmax = max ( matmax, abs(matrix(i,j,k,idim,l)) )
                  end do
               end do
            end do
         end do
      end do
      matmax = matmax*epsmac
      if ( iprlayout.eq.1 ) then

c     --- Standard layout, no special precautions needed

         if ( iprseq.eq.1 ) then

c        --- Standard sequence

            do idim = 1, ndim

               if ( ndim.gt.1 ) write(irefwr,100) idim
100            format ( ' Component', i2 )
               write(irefwr,110) '   i   j   k'
110            format ( 1x, a, '    1/6/11/16   2/7/12/17  ',
     +                  ' 3/8/13/18   4/9/14/19   5/10/15/20' )

               do k = 1, nk+1
                  do j = 1, nj+1
                     do i = 1, ni+1
                        null = .true.
                        do l = 1, nconct
                           if ( abs(matrix(i,j,k,idim,l)).gt.matmax )
     +                        null = .false.
                        end do
                        if ( .not. null ) then
                           write(irefwr,120) i,j,k,
     +                       (matrix(i,j,k,idim,l), l=1,min(5,nconct))
120                        format(1x,ss,3i4,':',5(1pe12.3e2))
                           if ( nconct.gt.5 ) write(irefwr,130)
     +                        (matrix(i,j,k,idim,l),l=6,nconct)
130                        format((14x,ss, 5(1pe12.3e2)))
                        end if
                     end do
                  end do
               end do
            end do

         else if ( iprseq.eq.2 ) then

c        --- Sequence j,k,i

            if ( ndim.eq.1 ) then
               iperm(1) = 1
            else
               iperm(1) = 2
               iperm(2) = 3
               iperm(3) = 1
            end if
            do jdim = 1, ndim
               idim = iperm(jdim)
               if ( ndim.gt.1 ) write(irefwr,140) jdim
140            format ( ' Renumbered component', i2 )
               write(irefwr,110) '   j   k   i'
               do i = 1, ni+1
                  do k = 1, nk+1
                     do j = 1, nj+1
                        null = .true.
                        do l = 1, nconct
                           if ( abs(matrix(i,j,k,idim,l)).gt.matmax )
     +                        null = .false.
                        end do
                        if ( .not. null ) then
                           write(irefwr,120) j,k,i,
     +                       (matrix(i,j,k,idim,l), l=1,min(5,nconct))
                           if ( nconct.gt.5 ) write(irefwr,130)
     +                        (matrix(i,j,k,idim,l),l=6,nconct)
                        end if
                     end do
                  end do
               end do
            end do

         else

c        --- Sequence k,i,j

            if ( ndim.eq.1 ) then
               iperm(1) = 1
            else
               iperm(1) = 3
               iperm(2) = 1
               iperm(3) = 2
            end if
            do jdim = 1, ndim
               idim = iperm(jdim)
               if ( ndim.gt.1 ) write(irefwr,140) jdim
               write(irefwr,110) '   k   i   j'
               do j = 1, nj+1
                  do i = 1, ni+1
                     do k = 1, nk+1
                        null = .true.
                        do l = 1, nconct
                           if ( abs(matrix(i,j,k,idim,l)).gt.matmax )
     +                        null = .false.
                        end do
                        if ( .not. null ) then
                           write(irefwr,120) k,i,j,
     +                       (matrix(i,j,k,idim,l), l=1,min(5,nconct))
                           if ( nconct.gt.5 ) write(irefwr,130)
     +                        (matrix(i,j,k,idim,l),l=6,nconct)
                        end if
                     end do
                  end do
               end do
            end do

         end if

      else

c     --- Layout using the local shifts

         if ( firstcall ) then

c        --- first call of subroutine, initialize some arrays with shifts

            call isfilm03 ( mmct, gmct, smct, 3, 2 )
            firstcall = .false.

         end if
         if ( itypmat.eq.1 ) then
            maxdx = 2
            mdim  = 3
         else
            maxdx = 1
            mdim  = 1
         end if
         if ( iprseq.eq.1 ) then

c        --- Standard sequence

            do idim = 1, ndim

               if ( ndim.gt.1 ) write(irefwr,100) idim
               if ( mdim.gt.1 ) then
                  write(irefwr,150) '  i   j   k '
150               format ( 2x, a, ' comp           matrix with shifts' )
               else
                  write(irefwr,160) '  i   j   k '
160               format ( 2x, a, '             matrix with shifts' )
               end if
               do k = 1, nk+1
                  do j = 1, nj+1
                     do i = 1, ni+1
                        null = .true.
                        do l = 1, nconct
                           if ( abs(matrix(i,j,k,idim,l)).gt.matmax )
     +                        null = .false.
                        end do
                        if ( .not. null ) then
                           do m = 1, mdim
                              jcount = 0
                              do k3 = -maxdx, maxdx
                                 do j3 = -maxdx, maxdx
                                    do i3 = -maxdx, maxdx
                                       if ( itypmat.eq.1 ) then
                                          indmat = mmct(idim,m,i3,j3,k3)
                                       else if ( itypmat.eq.2 ) then
                                          indmat = smct(idim,m,i3,j3,k3)
                                       else
                                          indmat = gmct(idim,m,i3,j3,k3)
                                       end if
                                       if ( indmat.ge.1 .and.
     +                                      indmat.le.nconct ) then
                                          c = matrix(i,j,k,idim,indmat)
                                          if ( abs(c).gt.matmax ) then
                                             jcount = jcount+1
                                             iworkgen(jcount,1) = i3
                                             iworkgen(jcount,2) = j3
                                             iworkgen(jcount,3) = k3
                                             work(jcount) = c
                                          end if
                                       end if
                                    end do
                                 end do
                              end do
                              if ( jcount.gt.0 .and. mdim.gt.1 ) then
                                 write(irefwr,200)
     +                           i,j,k,m, (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),
     +                           l=1,min(2,jcount))
200                              format(1x,ss,3i4,i6,':',
     +                              2(3x,3i2,1pe15.7e2))
                              else if ( jcount.gt.0 ) then
                                 write(irefwr,210) i,j,k,(iworkgen(l,1),
     +                           iworkgen(l,2), iworkgen(l,3),
     +                           work(l),l=1,min(2,jcount))
210                              format(1x,ss,3i4,':',
     +                              2(3x,3i2,1pe15.7e2))
                              end if
                              if ( jcount.gt.2 .and. mdim.gt.1 ) then
                                 write(irefwr,220)
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),l=3,jcount)
220                              format((20x,ss, 2(3x,3i2,1pe15.7e2)))
                              else if ( jcount.gt.2 ) then
                                 write(irefwr,230)
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),l=3,jcount)
230                              format((14x,ss, 2(3x,3i2,1pe15.7e2)))
                              end if
                           end do
                        end if
                     end do
                  end do
               end do
            end do

         else if ( iprseq.eq.2 ) then

c        --- Sequence j,k,i

            if ( ndim.eq.1 ) then
               iperm(1) = 1
            else
               iperm(1) = 2
               iperm(2) = 3
               iperm(3) = 1
            end if
            do jdim = 1, ndim
               idim = iperm(jdim)

               if ( ndim.gt.1 ) write(irefwr,100) jdim
               if ( mdim.gt.1 ) then
                  write(irefwr,250) '  j   k   i '
250               format(2x,a, ' Ren comp           matrix with shifts')
               else
                  write(irefwr,160) '  j   k   i '
               end if
               do i = 1, ni+1
                  do k = 1, nk+1
                     do j = 1, nj+1
                        null = .true.
                        do l = 1, nconct
                           if ( abs(matrix(i,j,k,idim,l)).gt.matmax )
     +                        null = .false.
                        end do
                        if ( .not. null ) then
                           do m1 = 1, ndim
                              m = iperm(m1)
                              jcount = 0
                              do i3 = -maxdx, maxdx
                                 do k3 = -maxdx, maxdx
                                    do j3 = -maxdx, maxdx
                                       if ( itypmat.eq.1 ) then
                                          indmat = mmct(idim,m,i3,j3,k3)
                                       else if ( itypmat.eq.2 ) then
                                          indmat = smct(idim,m,i3,j3,k3)
                                       else
                                          indmat = gmct(idim,m,i3,j3,k3)
                                       end if
                                       if ( indmat.ge.1 .and.
     +                                      indmat.le.nconct ) then
                                          c = matrix(i,j,k,idim,indmat)
                                          if ( abs(c).gt.matmax ) then
                                             jcount = jcount+1
                                             iworkgen(jcount,1) = j3
                                             iworkgen(jcount,2) = k3
                                             iworkgen(jcount,3) = i3
                                             work(jcount) = c
                                          end if
                                       end if
                                    end do
                                 end do
                              end do
                              if ( jcount.gt.0 .and. mdim.gt.1 ) then
                                 write(irefwr,200) j,k,i,m1,
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),
     +                           l=1,min(2,jcount))
                              else if ( jcount.gt.0 ) then
                                 write(irefwr,210) j,k,i,
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),
     +                           l=1,min(2,jcount))
                              end if
                              if ( jcount.gt.2 .and. mdim.gt.1 ) then
                                  write(irefwr,220)
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),l=3,jcount)
                              else if ( jcount.gt.2 ) then
                                  write(irefwr,230)
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),l=3,jcount)
                              end if
                           end do
                        end if
                     end do
                  end do
               end do
            end do

         else

c        --- Sequence k,i,j

            if ( ndim.eq.1 ) then
               iperm(1) = 1
            else
               iperm(1) = 3
               iperm(2) = 1
               iperm(3) = 2
            end if
            do jdim = 1, ndim
               idim = iperm(jdim)

               if ( ndim.gt.1 ) write(irefwr,100) jdim
               if ( mdim.gt.1 ) then
                  write(irefwr,250) '  k   i   j '
               else
                  write(irefwr,160) '  k   i   j '
               end if
               do j = 1, ni+1
                  do i = 1, nk+1
                     do k = 1, nj+1
                        null = .true.
                        do l = 1, nconct
                           if ( abs(matrix(i,j,k,idim,l)).gt.matmax )
     +                        null = .false.
                        end do
                        if ( .not. null ) then
                           do m1 = 1, ndim
                              m = iperm(m1)
                              jcount = 0
                              do j3 = -maxdx, maxdx
                                 do i3 = -maxdx, maxdx
                                    do k3 = -maxdx, maxdx
                                       if ( itypmat.eq.1 ) then
                                          indmat = gmct(idim,m,i3,j3,k3)
                                       else if ( itypmat.eq.2 ) then
                                          indmat = smct(idim,m,i3,j3,k3)
                                       else
                                          indmat = gmct(idim,m,i3,j3,k3)
                                       end if
                                       if ( indmat.ge.1 .and.
     +                                      indmat.le.nconct ) then
                                          c = matrix(i,j,k,idim,indmat)
                                          if ( abs(c).gt.matmax ) then
                                             jcount = jcount+1
                                             iworkgen(jcount,1) = k3
                                             iworkgen(jcount,2) = i3
                                             iworkgen(jcount,3) = j3
                                             work(jcount) = c
                                          end if
                                       end if
                                    end do
                                 end do
                              end do
                              if ( jcount.gt.0 .and. mdim.gt.1 ) then
                                 write(irefwr,200) k,i,j,m1,
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),
     +                           l=1,min(2,jcount))
                              else if ( jcount.gt.0 ) then
                                 write(irefwr,210) k,i,j,
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),
     +                           l=1,min(2,jcount))
                              end if
                              if ( jcount.gt.2 .and. mdim.gt.1 ) then
                                  write(irefwr,220)
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),l=3,jcount)
                              else if ( jcount.gt.2 ) then
                                  write(irefwr,230)
     +                           (iworkgen(l,1),iworkgen(l,2),
     +                           iworkgen(l,3),work(l),l=3,jcount)
                              end if
                           end do
                        end if
                     end do
                  end do
               end do
            end do

         end if

      end if
      end
