      subroutine ispresex ( iinput, iinbc, output, ni, nj, nvirtual,
     +                      coor )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 28-01-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Extrapolate the pressure along a fixed wall to the tip of that wall
c    in case it ends with on a periodical or multiblock boundary (2d case)
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c     pressure
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), iinbc(*), ni, nj, nvirtual
      double precision output(ni+1,nj+1),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2)

c     coor      i    reserved
c     iinbc     i    reserved
c     iinput    i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     output   i/o   Cartesian components of the flow field in the vertices
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, i1, i2, icoupled, iface, btypenb, jpstart,
     +        ipstart, j, k, ndegfd, iprinbc, ndegfdlc, nseg, btype
      double precision pres1, pres2, dx1, dx2

c     btype     Type of boundary condition
c     btypenb   Type of boundary condition of neighbour side
c     dx1       Distance between two points
c     dx2       Distance between two points
c     i         General loop variable
c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     iface     Face number
c     iprinbc   Starting address of array rinbc in array iinput
c     ipstart   Starting address
c     j         General loop variable
c     jpstart   Starting address
c     k         General loop variable
c     ndegfd    Number of degrees of freedom
c     ndegfdlc  Local number of degrees of freedom
c     nseg      Number of segments the boundary is divided in
c     pres1     Pressure in a point
c     pres2     Pressure in a point
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    In the case of a wing ending on a multiblock boundary or a periodical
c    boundary the computed pressure may be inaccurate at the transition point
c    For that reason we have decided to perform an extrapolation along the
c    wing towards that tip
c
c    The situation is recognized as follows:
c
c    The boundary must consist of more than one segment.
c    If a segment is periodical or multiblock then
c       if it is not the first segment check if the previous segment is a wall
c       if so extrapolate
c       if it is not the last segment check if the next segment is a wall
c       if so extrapolate
c
c
c    The extrapolation is base on the following formula:
c
c        dx1      dx2
c     x--------x-------x
c     p1      p2       p
c
c     p = (dx1+dx2)/dx1 p2 - dx2/dx1 p1
c
c    hence:
c
c    For iface := 1 (1) 4 do
c       nseg := number of segments along face iface
c       if ( nseg>1 ) then
c          For k := 1 (1) nseg do
c             btype := type of boundary condition along segment k
c             if ( btype = 98 or 99 ) then
c                if ( k>1 and previous section is wall ) then
c                   extrapolate pressure along wing
c                if ( k<nseg and next section is wall ) then
c                   extrapolate pressure along wing
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Compute ndegfdlc

      iprinbc  = iinput(27)
      ndegfd   = iinput(55)
      icoupled = iinput(76)
      if ( icoupled.eq.0 ) then

c     --- Coupled approach

         ndegfdlc = ndegfd-1

      else

c     --- Decoupled approach

         ndegfdlc = ndegfd

      end if

c     --- Loop over faces

      do iface = 1, 4

         nseg = iinbc(iinbc(iface))

c        --- Loop over subfaces

         if ( nseg.gt.1 ) then

c        --- nseg > 1

            do k = 1, nseg

c           --- btype := type of boundary condition along segment k

               ipstart = iinbc(iface)+(k-1)*(2+ndegfdlc*3)
               btype = iinbc(ipstart+3)

               if ( btype.eq.98 .or. btype.eq.99 ) then

c              --- Periodical or mulitblock boundary

                  if ( k.gt.1 ) then

c                 --- Not the first segment, check previous segment

                     jpstart = iinbc(iface)+(k-2)*(2+ndegfdlc*3)
                     btypenb = iinbc(jpstart+3)
                     if ( btypenb.ge.3 .and. btypenb.le.5 .or.
     +                    btypenb.le.10 ) then

c                    --- fixed wall, interpolate pressure

                        i2 = iinbc(jpstart+2)+1
                        if ( iface.le.2 ) then

                           if ( iface.eq.1 ) then
                              i = 1
                           else
                              i = ni+1
                           end if
                           j = i2
                           pres1 = output(i,j-2)
                           pres2 = output(i,j-1)
                           dx1 = sqrt((coor(i,j-2,1)-coor(i,j-1,1))**2+
     +                                (coor(i,j-2,2)-coor(i,j-1,2))**2 )
                           dx2 = sqrt((coor(i,j-1,1)-coor(i,j,1))**2+
     +                                (coor(i,j-1,2)-coor(i,j,2))**2 )

                        else

                           if ( iface.eq.3 ) then
                              j = 1
                           else
                              j = nj+1
                           end if
                           i = i2
                           pres1 = output(i-2,j)
                           pres2 = output(i-1,j)
                           dx1 = sqrt((coor(i-2,j,1)-coor(i-1,j,1))**2+
     +                                (coor(i-2,j,2)-coor(i-1,j,2))**2 )
                           dx2 = sqrt((coor(i-1,j,1)-coor(i,j,1))**2+
     +                                (coor(i-1,j,2)-coor(i,j,2))**2 )

                        end if
                        output(i,j) = (dx1+dx2)/dx1*pres2-dx2/dx1*pres1

                     end if

                  end if

                  if ( k.lt.nseg ) then

c                 --- Not the last segment, check next segment

                     jpstart = iinbc(iface)+k*(2+ndegfdlc*3)
                     btypenb = iinbc(jpstart+3)
                     if ( btypenb.ge.3 .and. btypenb.le.5 .or.
     +                    btypenb.le.10 ) then

c                    --- fixed wall, interpolate pressure

                        i1 = iinbc(jpstart+1)
                        if ( iface.le.2 ) then

                           if ( iface.eq.1 ) then
                              i = 1
                           else
                              i = ni+1
                           end if
                           j = i1
                           pres1 = output(i,j+2)
                           pres2 = output(i,j+1)
                           dx1 = sqrt((coor(i,j+2,1)-coor(i,j+1,1))**2+
     +                                (coor(i,j+2,2)-coor(i,j+1,2))**2 )
                           dx2 = sqrt((coor(i,j+1,1)-coor(i,j,1))**2+
     +                                (coor(i,j+1,2)-coor(i,j,2))**2 )

                        else

                           if ( iface.eq.3 ) then
                              j = 1
                           else
                              j = nj+1
                           end if
                           i = i1
                           pres1 = output(i+2,j)
                           pres2 = output(i+1,j)
                           dx1 = sqrt((coor(i+2,j,1)-coor(i+1,j,1))**2+
     +                                (coor(i+2,j,2)-coor(i+1,j,2))**2 )
                           dx2 = sqrt((coor(i+1,j,1)-coor(i,j,1))**2+
     +                                (coor(i+1,j,2)-coor(i,j,2))**2 )

                        end if
                        output(i,j) = (dx1+dx2)/dx1*pres2-dx2/dx1*pres1

                     end if

                  end if

               end if

               iprinbc = iprinbc +ndegfdlc*2

            end do

         end if

      end do
      end
