      subroutine isstres3 ( work, solut, coor, eddy, invjac, lwork,
     +                      ndegfd, ni, nj, nk, npointsv, iindsc,
     +                      nvirtual, leniindsc, ntrnsp, nturb, rintur)
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.1    date   11-09-1996 Marcel: nonlinear effects
c        version 2.0    date   10-04-1996 Guus: extra parameters
c        version 1.0    date   06-03-1996
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computes cartesian Reynolds stresses in R3
c
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c     write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer lwork, ndegfd, ni, nj, nk, npointsv, nvirtual, leniindsc,
     +        iindsc(leniindsc,*), ntrnsp, nturb
      double precision coor(*), eddy(1:npointsv),
     +                 invjac(1:npointsv,1:8), work(1:lwork),
     +                 solut(1:npointsv,1:ndegfd), rintur(*)

c     coor      i    reserved
c     eddy      i    reserved
c     iindsc    i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     leniindsc i    Length of first component of array iindsc
c     lwork     i    Length of array work
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npointsv  i    Number of points in the mesh with virtual points
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rintur    i    reserved
c     solut     i    reserved
c     work      o    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, idir, ipchsym, ipcva, ipjcta, idiscret,
     +        ipjctg, ippderiv, ipveloc, iphightens, jdir,
     +        ipstress, nwork, pnttyp, vcellc, alphad, betad
      integer rspv(1:3,1:8,0:3,0:4,1:8), npf(0:3,0:4,1:8)
      double precision cv(1:8,0:3,0:4,1:8)

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     cv        reserved
c     i         General loop variable
c     idir      General loop variable for 1...ndim loop
c     idiscret  Type of discretization
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     iphightens Starting address of array HIGHTENS in array WORK
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ippderiv  Starting address of array PDERIV in array WORK
c     ipstress  Starting address of array STRESS in array WORK
c     ipveloc   Starting address of array VELOC in array WORK
c     jdir      General loop variable for 1...ndim loop
c     npf       reserved
c     nwork     Second dimension of array work
c     pnttyp    reserved
c     rspv      reserved
c     vcellc    Point type of integration cell centre for a given
c               variable type
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCHRYS3  Fill chrystoffel symbols in one integration point in R3
c     ISCOVA3D  Fill covariant base vectors in all integration points in R3
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTCF3  Compute interpolation of a coefficient in an integration point
c     ISJCTA3D  Fill Jacobian times contravariant base vectors
c               in one integration point in R3
c     ISJCTG3D  Fill Jacobian times contravariant metric tensors
c               in one integration point in R3
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISSETCON  Make array equal to a constant
c     ISSTRE3D  Compute cartesian Reynolds stresses in R3
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      17   internal error: array size too large
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isstres3')

c     --- TEMPORARY-----------------------------------------------

      call ispndf(npf,rspv,cv,3)

c     --- TEMPORARY-----------------------------------------------

c         --- allocate local arrays

      idiscret   = iindsc(7,ntrnsp+nturb)
      ipcva      = 1            + 6*npointsv
      ipjcta     = ipcva        + 72*npointsv
      ipjctg     = ipjcta       + 9*npointsv
      ipchsym    = ipjctg       + 6*npointsv
      ippderiv   = ipchsym      + 18*npointsv
      ipveloc    = ippderiv     + 9*npointsv
      iphightens = ipveloc      + 12*npointsv
      ipstress   = iphightens   + npointsv
      nwork      = ipstress     + 9*npointsv

      if ( nwork.gt.lwork ) then
        call iserrchr( 'work',1 )
        call iserrint( nwork, 1 )
        call iserrint( lwork, 2 )
        call iserrsub ( 17, 2, 0, 1 )
      end if

c     --- initialize the work array's

      do i = 1, nwork
        work( i ) = 0d0
      end do

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 72*npointsv, 0d0 )
      call iscova3d ( coor, work(ipcva), ni, nj, nk, nvirtual, npf,
     +                rspv, cv )

c     --- Determine point type corresponding to the centre of a
c         pressure cell in which the Reynolds stresses must be computed

      pnttyp = 8

c     --- Compute contravariant basis vectors, metric tensors
c         and Christoffel symbols

      call isjcta3d ( work(ipcva+9*npointsv*(pnttyp-1)),
     +                work(ipjcta), ni, nj, nk, nvirtual )

      call isjctg3d ( invjac(1,pnttyp), work(ipjcta),
     +                work(ipjctg), ni, nj, nk, nvirtual )

      call ischrys3 ( work(ipcva), work(ipjcta), work(ipchsym),
     +                invjac(1,pnttyp), ni, nj, nk, nvirtual,
     +                pnttyp )

c                                        alpha
c     --- Compute partial derivative of V      in direction beta in
c         point of type pnttyp

      do alphad = 1, 3
        do betad = 1, 3
          call isintcf3( solut(1,alphad),
     +                   work(ippderiv+npointsv*(alphad-1+3*(betad-1))),
     +                   ni, nj, nk, nvirtual, npf, rspv, cv, betad,
     +                   alphad, pnttyp )
        end do
      end do

c     --- Interpolate velocities to centre of cell faces and pnttyp

      do jdir = 1, 3
        vcellc = 8 - 2**(jdir-1)
        do idir = 1, 3
          call isintcf3( solut(1,idir),
     +                   work(ipveloc+npointsv*(idir-1+3*(jdir-1))),
     +                   ni, nj, nk, nvirtual, npf, rspv, cv, 0, idir,
     +                   vcellc )
        end do
        call isintcf3( solut(1,jdir), work(ipveloc+npointsv*(jdir+8)),
     +                 ni, nj, nk, nvirtual, npf, rspv, cv, 0, jdir,
     +                 pnttyp )
      end do

c     --- Determine the cartesian Reynolds stresses in R3

      call isstre3d(work, work(ippderiv), work(ipveloc),
     +              work(ipveloc+9*npointsv), solut, eddy,
     +              work(ipcva), work(ipcva+63*npointsv), invjac,
     +              invjac(1,pnttyp), work(iphightens), work(ipjcta),
     +              work(ipjctg), work(ipchsym), ni, nj, nk, ndegfd,
     +              nvirtual, idiscret, work(ipstress), pnttyp, rintur)

      call iserclos('isstres3')

      end
