      subroutine isexchvc( ieq, isiinput, issolut, isitopol, iadmin )
c ======================================================================
c
c     programmer   Jason Frank
c     version      2.0   date    18-01-1999  Jason:  Complete revision
c     version      1.0   date    14-12-1998
c
c   copyright (c) 1998-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Exchange virtual cells in parallel
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ieq, iadmin(*), isiinput(*), isitopol, issolut(*)

c     iadmin    i   reserved
c     ieq       i   Equation number
c     isiinput  i   Array of sequence numbers (nblocks) of array iinput
c     isitopol  i   Sequence number of array itopol
c     issolut   i   Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, icoupled, idiscret, ilev, ipiindsc, ipiinput,
     +        ipitopol, ipnsubf, ipsmsg, ipsubf, iseqpres, issmsg,
     +        itrans, lensmsg, maxnsubf, nblocks, ndegfd, ndim,
     +        nfaces, ni, nj, nk, ntimlv, nvirtual

c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idiscret  Type of discretization
c     ilev      Time level of the multi-block transport of information
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsmsg    Starting address of array smsg in buffer
c     ipsubf    Starting address of the subfaces array
c     iseqpres  Sequence number of pressure equation
c     issmsg    Sequence number of array smsg
c     itrans    reserved
c     lensmsg   Length of array smsg
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISEXCH01  Performs the actual data exchange
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
cc
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isexchvc')

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ndim     = ibuffr(ipiinput + 57 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if

      ntimlv   = ibuffr(ipiinput + 64 -1)
      ilev     = ntimlv

      if ( ieq.lt.iseqpres ) then
         itrans = ieq
      else
         itrans = ieq + ( ndim - 1 ) * ( 1 - icoupled )
      end if

c     --- compute nblocks, nfaces and maxnsubfaces

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol + 1 - 1)
      nfaces   = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)

c     --- compute an upper bound for the length of a multi-block submessage

      lensmsg = 0
      do iblock = 1, nblocks
         call isactive(isiinput(iblock))
         ipiinput = isingetp(isiinput(iblock))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         if ( nk.eq.0 ) then
            lensmsg = max(lensmsg,3*(max(ni,nj)+1))
         else
            lensmsg = max(lensmsg,
     +                     3*(max(ni,nj,nk)+1)*(max(ni,nj,nk)+1))
         end if
      end do
      issmsg = 0
      call isdcreat(issmsg, lensmsg, 'smsg' )
      nvirtual = ibuffr(ipiinput + 54 - 1)

      if ( itrans.lt.0 ) then

c     --- coefs array: ntraq1 = ncoefs, ntraq2 = 0

         ndegfd = ibuffr(ipiinput + 66 - 1)
         ntimlv = 1

      else

         ndegfd = ibuffr(ipiinput + 55 - 1)

      end if

c     --- compute pointers

      ipiindsc = ibuffr(ipiinput-1+6)
      idiscret = ibuffr(ipiinput-1+ipiindsc+1)
      ipsmsg   = isdpgetp(issmsg)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

      call isexch01 ( itrans, ilev, ndegfd, ntimlv, nvirtual,
     +                nblocks, nfaces, maxnsubf, idiscret,
     +                lensmsg, isiinput, issolut, iadmin,
     +                ibuffr(ipnsubf), ibuffr(ipsubf),
     +                buffer(ipsmsg) )

      call isdelarr(issmsg)
      call isdcrpri

      call iserclos('isexchvc')

      end
