      subroutine iserrbnd ( iinbc, ntrnsp, nturb )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date   10-04-1996 Kees K b.c. for pressure
c        version 1.0    date   13-02-1996
c
c   copyright (c) 1996-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Check boundary conditions in adjacent sides in 2D
c     Give error message in case SnnUt is present in adjacent sides
c **********************************************************************
c
c                       KEYWORDS
c
c     boundary_conditions
c     error_manipulation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinbc(*), ntrnsp, nturb

c     iinbc     i    reserved
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer itable(4,2), ipstart, iside, nseg, snnut
      parameter ( snnut = 6 )

c     ipstart   Starting address
c     iside     Sequence number of side
c     itable    Internal table containing the type of boundary conditions at
c               the vertices of the sides in the following way
c               itable(i,1) lower part of side i
c               itable(i,2) upper part of side i
c     nseg      Number of segments the boundary is divided in
c     snnut     Sigma_nn and u_t prescribed
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     711   Sigma_nn and _t given at adjacent sides
c **********************************************************************
c
c                       PSEUDO CODE
c
c      Fill array itable with the values of typbc at the lower and upper part
c      of the boundaries
c      If ( at two adjacent sides we have SnnUt given ) then
c         give error message
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'iserrbnd' )

c     --- Fill array itable with the values of typbc at the lower and upper part
c         of the boundaries

      do iside = 1, 4
         ipstart = iinbc(iside)
         nseg = iinbc(ipstart)
         itable(iside,1) = iinbc(ipstart+3)
         ipstart = ipstart+(nseg-1)*(2+3*(ntrnsp+nturb+2))
         itable(iside,2) = iinbc(ipstart+3)
      end do

c     --- Check adjacent sides

      if ( itable(1,1).eq.snnut .and. itable(3,1).eq.snnut ) then

c     --- Boundary condition at adjacent sides 1 and 3

         call iserrint ( 1, 1 )
         call iserrint ( 3, 2 )
         call iserrsub ( 711, 2, 0, 0 )

      end if

      if ( itable(1,2).eq.snnut .and. itable(4,1).eq.snnut ) then

c     --- Boundary condition at adjacent sides 1 and 4

         call iserrint ( 1, 1 )
         call iserrint ( 4, 2 )
         call iserrsub ( 711, 2, 0, 0 )

      end if

      if ( itable(3,2).eq.snnut .and. itable(2,1).eq.snnut ) then

c     --- Boundary condition at adjacent sides 2 and 3

         call iserrint ( 2, 1 )
         call iserrint ( 3, 2 )
         call iserrsub ( 711, 2, 0, 0 )

      end if

      if ( itable(2,2).eq.snnut .and. itable(4,2).eq.snnut ) then

c     --- Boundary condition at adjacent sides 2 and 4

         call iserrint ( 2, 1 )
         call iserrint ( 4, 2 )
         call iserrsub ( 711, 2, 0, 0 )

      end if

      call iserclos ( 'iserrbnd' )

      end
