      subroutine isfillstr ( iglobstr, rglobstr, iintim, rintim,
     +                       icluster, iclustern, idimens, lncluster,
     +                       ilocstr, rlocstr, maxclust, ichoice, ndim )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 20-04-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Fill the global and local structure arrays from the information read
c    by the preprocessor extended with default values
c **********************************************************************
c
c                       KEYWORDS
c
c   structure
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iglobstr(*), iintim(*), icluster(*), iclustern(*),
     +        idimens(*), maxclust, lncluster, ilocstr(maxclust,*),
     +        ichoice(*), ndim
      double precision rglobstr(*), rintim(*), rlocstr(maxclust,*)

c     ichoice   i    Integer choice parameters for the intermediate file
c     icluster  i    Array in which information about the clusters is stored
c                    icluster(i), i=1(1)nclusters contains the starting
c                    position of information about cluster i
c                    In this starting position the number of equations is
c                    stored, followed by the equation sequence numbers
c     iclustern o    Rearranged array icluster
c     idimens   i    Integer dimension parameters for the intermediate file
c     iglobstr i/o   Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     iintim    i    Contains information of the time integration
c                    See the programmers Guide for a description
c     ilocstr   i    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c     lncluster i    Maximum length of array icluster
c     maxclust  i    Maximum number of clusters
c     ndim      i    Dimension of space (2 or 3)
c     rglobstr i/o   Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     rintim    i    Contains information of the time integration
c                    See the programmers Guide for a description
c     rlocstr   i    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iclustmax, ih, ipcluster, iseq, j, neqclust, nclusters,
     +        ndegfd, nextrcls, nturb, neq

c     i         General loop variable
c     iclustmax Maximum number of clusters
c     ih        Help variable to store a constant temporarily
c     ipcluster Starting address of cluster in array icluster
c     iseq      Sequence number read after transport or turbulence equation
c     j         General loop variable
c     nclusters Number of clusters
c     ndegfd    Number of degrees of freedom
c     neq       Number of equations
c     neqclust  Number of equations in cluster
c     nextrcls  Number of extra clusters
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      17   internal error: array size too large
c     618   The first cluster does not consist of velocity and pressure
c     619   The first cluster does not consist of velocity and pressure
c     620   The last cluster does not contain all turbulence equations
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Fill iglobstr(1) if necessary
c    If ( stationary solution required ) then
c       Fill default values if necessary
c    Fill array iclustern from icluster extended with defaults
c    Fill local structure arrays with defaults if necessary
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isfilstr' )
      if ( iglobstr(1).eq.0 ) then

c     --- Fill iglobstr(1) if necessary

         if ( iglobstr(4).gt.0 .or. iglobstr(5).gt.0 ) then

c        --- miniter or maxiter stationary found

            iglobstr(1) = 1

         else if ( iintim(10).ne.0 ) then

c        --- stationary solution according to iintim

            iglobstr(1) = 1

         end if

      end if
      iintim(10) = iglobstr(1)

      if ( iglobstr(1).gt.0 ) then

c     --- Stationary solution, fill default values if necessary

         iglobstr(4) = max(1,iglobstr(4))
         if ( iglobstr(5).eq.0 ) iglobstr(5) = 10000
         if ( rintim(29).le.0d0 .and. rintim(30).le.0d0 )
     +      rintim(30) = 1d-2

      end if
      iglobstr(4) = max(1,iglobstr(4))
      iglobstr(5) = max(1,iglobstr(5))
      iglobstr(7) = max(1,iglobstr(7))
      iglobstr(8) = max(1,iglobstr(8))
      rglobstr(1) = rintim(29)
      rglobstr(2) = rintim(30)

c     --- Fill array iclustern from icluster extended with defaults

      nclusters = idimens(8)
      ndegfd    = idimens(2)
      nturb     = idimens(6)

      if ( ichoice(4).eq.0 ) then
         neq = ndegfd-ndim+1
      else
         neq = ndegfd
      end if

      do i = 1, neq
         iclustern(i) = 0
      end do

c     --- First store number of equations per cluster in iclustern

      ipcluster = 0
      iclustmax = 0
      do i = 1, nclusters
         iseq = icluster(ipcluster+1)
         neqclust = icluster(ipcluster+2)
         iclustmax = max(iclustmax,iseq+neqclust-1)
         iclustern(iseq) = neqclust
         ipcluster = ipcluster+neqclust+2
      end do
      if ( iintim(17).eq.1 .and. nclusters.eq.0 ) then

c     --- iintim(17) = 1, i.e. pressure-correction iteration
c         and nclusters = 0, no clusters yet
c         In this case the first cluster must consist of velocity and pressure

         nclusters = 1
         iclustern(1) = 2
         iclustmax = 2

      end if

      if ( iintim(16).gt.1 .and. neq.gt.iclustmax ) then

c     --- iintim(16) > 1, i.e. substepping is required for the turbulence
c         equations

         nextrcls = neq-iclustmax-nturb
         do i = 1, nextrcls
            iclustern(i+nclusters) = 1
         end do
         nextrcls = nextrcls+1
         iclustern(nextrcls+nclusters) = nturb

      else

c     --- Standard situation

         nextrcls = neq-iclustmax
         do i = 1, nextrcls
            iclustern(i+nclusters) = 1
         end do

      end if

c     --- Next fill the pointers in iclustern

      ipcluster = nclusters+nextrcls+1
      do i = 1, nclusters+nextrcls
         neqclust = iclustern(i)
         iclustern(i) = ipcluster
         ipcluster = ipcluster+neqclust+1
      end do
      if ( ipcluster.gt.lncluster ) then

c     --- Array icluster too small

         call iserrchr ( 'iclustern', 1 )
         call iserrint ( ipcluster, 1 )
         call iserrint ( lncluster, 2 )
         call iserrsub ( 17, 2, 0, 1 )
         go to 1000

      end if

c     --- Copy information from icluster into iclustern

      nclusters = idimens(8)
      ipcluster = 0
      do i = 1, nclusters
         iseq = icluster(ipcluster+1)
         neqclust = icluster(ipcluster+2)
         ih = iclustern(iseq)
         iclustern(ih) = neqclust
         do j = 1, neqclust
            iclustern(ih+j) = icluster(ipcluster+2+j)
         end do
         ipcluster = ipcluster+neqclust+2
      end do
      if ( iintim(17).eq.1 .and. nclusters.eq.0 ) then

c     --- iintim(17) = 1, i.e. pressure-correction iteration
c         and nclusters = 0, no clusters yet
c         In this case the first cluster must consist of velocity and pressure

         nclusters = 1
         ih = iclustern(1)
         iclustern(ih) = 2
         iclustern(ih+1) = 1
         iclustern(ih+2) = 2

      end if

      if ( iintim(16).gt.1 .and. neq.gt.iclustmax ) nextrcls=nextrcls-1
      do i = 1, nextrcls
         ih = iclustern(i+nclusters)
         iclustern(ih) = 1
         iclustern(ih+1) = iclustmax+i
      end do

      if ( iintim(16).gt.1 ) then

c     --- iintim(16) > 1, i.e. substeps for turbulence equations

         ih = iclustern(nextrcls+1+nclusters)
         iclustern(ih) = nturb
         do i = 1, nturb
            iclustern(ih+i) = neq-nturb+i
         end do
         idimens(8) = nclusters+nextrcls+1
         idimens(9) = iclustern(nclusters+nextrcls+1)+
     +                iclustern(iclustern(nclusters+nextrcls+1))

      else

c     --- Extend with equations that not have been clustered yet

         idimens(8) = nclusters+nextrcls
         idimens(9) = iclustern(nclusters+nextrcls)+
     +                iclustern(iclustern(nclusters+nextrcls))

      end if

c     --- Fill local structure arrays with defaults if necessary

      do i = 1, nclusters+nextrcls
         ilocstr(i,2) = max(1,ilocstr(i,2))
         ilocstr(i,3) = max(1,ilocstr(i,3))
         ilocstr(i,5) = max(1,ilocstr(i,5))
         if ( rlocstr(i,1).le.0d0 .and. rlocstr(i,2).le.0d0 )
     +      rlocstr(i,1) = 1d-2
      end do

      if ( iintim(17).eq.1 ) then

c     --- iintim(17) = 1, i.e. pressure-correction iteration
c         In this case the first cluster must consist of velocity and pressure

         ih = iclustern(1)
         if ( iclustern(ih).ne.2 ) then

c        --- The first cluster does not consist of velocity and pressure

            call iserrint ( iclustern(ih), 1 )
            call iserrsub ( 618, 1, 0, 0 )

         end if
         if ( iclustern(ih+1).ne.1 .or. iclustern(ih+2).ne.2 ) then

c        --- The first cluster does not consist of velocity and pressure

            call iserrint ( iclustern(ih+1), 1 )
            call iserrint ( iclustern(ih+2), 2 )
            call iserrsub ( 619, 2, 0, 0 )

         end if
         ilocstr(1,1) = 1
         ilocstr(1,2) = max(1,ilocstr(i,2))
         if ( ilocstr(1,3).le.1 ) ilocstr(1,3) = 1000
         ilocstr(1,4) = 1
         rlocstr(1,1) = rintim(19)
         rlocstr(1,2) = rintim(20)

      end if

      if ( iintim(16).gt.1 ) then

c     --- iintim(16) > 1, i.e. substeps for turbulence equations
c         In this case the last cluster must consist of turbulence quantities

         nclusters = idimens(8)
         ih = iclustern(nclusters)
         if ( iclustern(ih).ne.nturb ) then

c        --- The last cluster does not consist of all turbulence quantities

            call iserrint ( iclustern(ih), 1 )
            call iserrint ( nturb, 2 )
            call iserrsub ( 620, 2, 0, 0 )

         end if
         ilocstr(nclusters,5) = iintim(16)

      end if

1000  call iserclos ( 'isfilstr' )
      end
