      subroutine isrdcoef ( nmain, codemn, nmcode, iretur, iincof,
     +                      rincof, maxtrans, maxcoefs, ndim, icheckeq )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.5    date 29-04-1998  Kees K: minor correction c_
c        version 1.4    date 08-01-1998  Guus: minor correction c_
c        version 1.3    date 24-12-1997  Guus: extra coefficients with respect
c                                        to the momentum equations
c        version 1.2    date 10-06-1997  Guus: extra coefficients
c        version 1.1    date 01-12-1994  Hester Bijl: The test in line 38,
c                                        mu and rho non-positive, changed into
c                                        mu and rho negative to enable
c                                        inviscid computations.
c        version 1.0    date 28-08-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read coefficient part of input files and store result in
c    arrays iincof and rincof
c **********************************************************************
c
c                       KEYWORDS
c
c       coefficients
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, maxtrans, maxcoefs,
     +        iincof(maxcoefs,maxtrans+1), ndim, icheckeq(maxtrans+1)
      double precision rincof(maxcoefs,maxtrans+1)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     icheckeq       Work array to indicate if an equation has been considered
c                    (icheckeq(i)=1 or not icheckeq(i)=0)
c     iincof    o    reserved
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     maxcoefs  i    Maximum number of coefficients allowed
c     maxtrans  i    Maximum number of transport equations allowed
c     ndim      i    Dimension of space (2 or 3)
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rincof    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, newmain, lenchar, nncode
      parameter ( nmainnw=30, newmain=2, nncode=20, lenchar=14 )
      integer nmcode1(newmain), nmcoda(nmainnw), itype(nncode),
     +        ncodmx(nncode), nout, nstrng, nmain1, numsub,
     +        ncode(nncode), i, iequa, ih, iseq, iseqprev, keyprev
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nncode)
      character *14 codema(nmainnw), code1(newmain)
      character *16 code(nncode)
      character *1 strout(1)
      logical ready

c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     code1     Table containing the  main keywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     iequa     General equation sequence number
c     ih        Help variable to store a constant temporarily
c     iseq      Sequence number read after transport or turbulence equation
c     iseqprev  Previous value of iseq
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     keyprev   Sequence number of previous keyword found with respect to code
c     lenchar   Number of characters in main keywords
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     newmain   Number of main keywords stored in array code1
c     nmain1    Total number of main keywords (newmain+nmain)
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode1   Array containing the number of significant characters for
c               each keyword in code1
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     numsub    Number of subkeywords that may be detected by isintx02
c               In the first step numsub=0 for any other step numsub=nncode
c     ready     Indicates if a while loop is finished or not
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding to
c               the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the erroneous part
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword COEFFICIENTS
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    MOMentum_equations
c    TRANSport_equation i
c
c    The keyword MOMENTUM may be followed by the following subkeywords:
c
c    MU = g
c    RHO = g
c    FORCE1 = g
c    FORCE2 = g
c    FORCE3 = g
c    MASS_FRACTION = g
c    INTERFACE_DRAG = g
c    C_1 = g
c    C_2 = g
c    C_3 = g
c
c    g may take one of the forms number, FUNC=i or TIME_FUNC=i as described for
c    the velocity boundary conditions
c
c    The keyword TRANSPORT a may be followed by the following subkeywords:
c
c    CAPacity = g
c    DIFFusion = g
c    DIFXX = g
c    DIFXY = g
c    DIFYY = g
c    DIFXZ = g
c    DIFYZ = g
c    DIFZZ = g
c    SOURCE = g
c    CONStant = g
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   No integer number has been found after the keyword %
c     530   integer number read out of range
c     531   Internal error in subroutine; character array too small
c     561   Keyword corresponds to MOMENTUM but has been found after TRANSPORT
c     562   The subkeyword FORCE 3 = has been found, whereas the problem is
c           only two-dimensional.
c     563   Keyword corresponds to TRANSPORT but has been found after MOMENTUM
c     564   Keyword DIFFUSION and DIFF.. has been found for the same equation
c     565   The value of the parameter is non-positive
c     566   The keyword % has been found for ndim=2
c     567   Equation has been found more than once
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Make table of new main keywords consisting of the local main keywords
c    and the global main keywords
c    ready := false
c    number_of_subkeywords := 0
c    while ( not ready ) do
c       iseq := 0
c       Read next subkeywords until a new main keyword is found
c       if ( main keyword = transport or turbulence ) then
c          Read sequence number iseq
c       if ( number_of_subkeywords=0 ) then
c          number_of_subkeywords := actual number
c       else
c          interpret subkeywords as function of keyword_previous
c       if ( keyword found = main keyword ) then
c          ready := true
c       keyword_previous := keyword found
c       iseq_previous := iseq
c
c    Remark:
c
c      The first array iincof/rincof(.,1) corresponds to the momentum equations
c      The arrays iincof/rincof(.,2 to maxtrans+1) correspond to the transport
c      equations 1 to maxtrans
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code1 / 'MOMENTUM_EQ', 'TRANSPORT_EQ' /
      data nmcode1 / 3, 4 /
      data code  / 'RHO', 'MU', 'FORCE', 'FORCE', 'FORCE',
     +             'MASS_FRACTION', 'INTERFACE_DRAG', 'C_', 'C_', 'C_',
     +             'CAPACITY', 'CONSTANT', 'SOURCE', 'DIFFUSION',
     +             'DIFXX', 'DIFXY', 'DIFYY', 'DIFXZ', 'DIFYZ',
     +             'DIFZZ'  /
      data ncode  / 3, 2, 5, 5, 5,   7, 7, 2, 2, 2,
     +              3, 4, 6, 4, 5,   5, 5, 5, 5, 5 /
      data ncodmx / 3, 2, 5, 5, 5,  13,14, 2, 2, 2,
     +              8, 8, 6, 9, 5,   5, 5, 5, 5, 5 /
      data itype  / 16, 16, 25, 25, 25,   16, 16, 25, 25, 25,
     +              16, 16, 16, 16, 16,   16, 16, 16, 16, 16 /
      data tabmin / 0d0, 0d0, 1d0, 0d0, 0d0, 0d0, 0d0, 1d0, 12*0d0 /
      data tabmax / 0d0, 0d0, 3d0, 0d0, 0d0, 0d0, 0d0, 3d0, 12*0d0 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  RHO
c               2:  MU
c               3:  FORCE
c               4:  FORCE
c               5:  FORCE
c               6:  MASS_FRACTION
c               7:  INTERFACE_DRAG
c               8:  C_
c               9:  C_
c              10:  C_
c              11:  CAPACITY
c              12:  CONSTANT
c              13:  SOURCE
c              14:  DIFFUSION
c              15:  DIFXX
c              16:  DIFXY
c              17:  DIFYY
c              18:  DIFXZ
c              19:  DIFYZ
c              20:  DIFZZ
c     code1     Table containing the  main keywords corresponding to this
c               subroutine. Contents:
c               1:  MOMENTUM_EQUATION
c               2:  TRANSPORT_EQUATION
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdcoef' )

c     --- Make table of new main keywords consisting of the local main keywords
c         and the global main keywords and the subsubkeywords

      nmain1 = newmain+nmain
      if ( nmain1.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( newmain, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, newmain
         codema(i)(1:lenchar) = code1(i)(1:lenchar)
         nmcoda(i) = nmcode1(i)
      end do
      do i = 1, nmain
         codema(i+newmain)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i+newmain) = nmcode(i)
      end do

c     --- Clear array icheckeq

      do i = 1, maxtrans+1
         icheckeq(i) = 0
      end do

      ready = .false.
      numsub = 0

200   if ( .not. ready ) then

c     --- while loop ready

         iseq = 0

c        --- Read next subkeywords until a new main keyword is found

         do i = 1, numsub
            table(i) = 0d0
         end do
         nout = numsub
         nstrng = 1
         call isintx02 ( code, ncode, numsub, itype, table, codema,
     +                   nmcoda, nmain1, iretur, tabmin, tabmax,
     +                   tabout, nout, strout, nstrng, ncodmx )
         if ( iretur.eq.2 ) then

c        --- transport found
c            Read sequence number iseq

            call isasknxt
            if ( ityprd.ne.2 ) then

c           --- Error not an integer number found

               call iserrchr ( codema(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            else

               iseq = nint(realrd)
               if ( iseq.lt.1 .or. iseq.gt.maxtrans ) then

c              --- iseq out of range

                  call iserrchr ( codema(iretur), 1 )
                  call iserrint ( iseq, 1 )
                  call iserrint ( 1, 2 )
                  call iserrint ( maxtrans, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  call iserrcrd
                  iseq = 1

               end if

            end if

         end if

         if ( numsub.eq.0 ) then

c        --- number_of_subkeywords = 0

            numsub = nncode

         else

c        --- interpret subkeywords as function of keyword_previous

            if ( keyprev.eq.1 ) then
               iequa = 1
            else
               iequa = 1+iseqprev
            end if
            if ( icheckeq(iequa).ne.0 ) then
               call iserrint ( iseq, 1 )
               call iswarsub ( 567, 1, 0, 0 )
               call iserrcrd
            end if
            icheckeq(iequa) = 1

c           --- subkeywords corresponding to momentum equation

            do i = 1, 10
               ih = nint(table(i))
               if ( iequa.ne.1 .and. ih.ne.0 ) then

c              --- keyword found that corresponds to momentum equation,
c                  whereas we are dealing with a transport equation

                  call iserrint ( iequa-1, 1 )
                  call iserrchr ( code(i), 1 )
                  call iserrsub ( 561, 1, 0, 1 )

               end if
               if ( ih.gt.0 .or. ih.lt.-100 ) then
                  iincof(i,iequa) = ih
               else if ( ih.lt.0 ) then
                  iincof(i,iequa) = -1
                  rincof(i,iequa) = tabout(-ih)
                  if ( i.le.2 .and. tabout(-ih).lt.0d0 ) then

c                 --- mu or rho negative

                     call iserreal ( tabout(-ih), 1 )
                     call iserrchr ( code(i), 1 )
                     call iserrsub ( 565, 0, 1, 1 )
                  end if
               end if
            end do
            if ( ndim.eq.2 .and. ih.ne.0 )
     +         call iswarsub ( 562, 0, 0, 0 )

c           --- subkeywords corresponding to transport equation
c               subkeywords capacity, constant and source

            do i = 11, 14
               ih = nint(table(i))
               if ( iequa.eq.1 .and. ih.ne.0 ) then

c              --- keyword found that corresponds to transport equation,
c                  whereas we are dealing with the momentum equation

                  call iserrchr ( code(i), 1 )
                  call iserrsub ( 563, 0, 0, 1 )

               end if
               if ( ih.gt.0 .or. ih.lt.-100 ) then
                  iincof(i-10,iequa) = ih
               else if ( ih.lt.0 ) then
                  iincof(i-10,iequa) = -1
                  rincof(i-10,iequa) = tabout(-ih)
                  if ( i.eq.6 .and. tabout(-ih).le.0d0 ) then

c                 --- capacity non-positive

                     call iserreal ( tabout(-ih), 1 )
                     call iserrchr ( code(i), 1 )
                     call iserrsub ( 565, 0, 1, 1 )
                  end if
               end if
            end do

c           --- subkeywords diffusion, diffxx ...

            ih = nint(table(14))
            if ( ih.ne.0 ) then

c           --- subkeyword diffusion

               if ( ih.gt.0 .or. ih.lt.-100 ) then
                  iincof(4,iequa) = ih
                  iincof(6,iequa) = ih
                  if ( ndim.eq.3 ) iincof(9,iequa) = ih
               else
                  iincof(4,iequa) = -1
                  rincof(4,iequa) = tabout(-ih)
                  iincof(6,iequa) = -1
                  rincof(6,iequa) = tabout(-ih)
                  if ( ndim.eq.3 ) then
                     iincof(9,iequa) = -1
                     rincof(9,iequa) = tabout(-ih)
                  end if
                  if ( tabout(-ih).lt.0d0 ) then

c                 --- diffusion negative

                     call iserreal ( tabout(-ih), 1 )
                     call iserrchr ( code(9), 1 )
                     call iserrsub ( 565, 0, 1, 1 )
                  end if
               end if
               do i = 15, 20
                  ih = nint(table(i))
                  if ( ih.ne.0 ) then
                     call iserrchr ( code(i), 1 )
                     call iswarsub ( 564, 0, 0, 1 )
                  end if
               end do
            else

c           --- not subkeyword diffusion, check on diffxx and so on

               do i = 15, 20
                  ih = nint(table(i))
                  if ( ih.gt.0 .or. ih.lt.-100 ) then
                     iincof(i-11,iequa) = ih
                  else if ( ih.lt.0 ) then
                     iincof(i-11,iequa) = -1
                     rincof(i-11,iequa) = tabout(-ih)
                     if ( (i.eq.15 .or. i.eq.17.or.  i.eq.20) .and.
     +                     tabout(-ih).lt.0d0 ) then

c                    --- difxx, diffyy or diffzz non-positive

                        call iserreal ( tabout(-ih), 1 )
                        call iserrchr ( code(i), 1 )
                        call iserrsub ( 565, 0, 1, 1 )
                     end if
                  end if
                  if ( ndim.eq.2 .and. i.ge.18 .and. ih.ne.0 ) then
                     call iserrchr ( code(i), 1 )
                     call iswarsub ( 566, 0, 0, 1 )
                  end if
               end do
            end if

         end if

         if ( iretur.gt.newmain ) then

c        --- main keyword found

            ready = .true.
            iretur = iretur-newmain

         else if ( iretur.le.-1 ) then

c        --- end of file found

            ready = .true.

         end if
         keyprev = iretur
         iseqprev = iseq
         go to 200

      end if

      call iserclos ( 'isrdcoef' )
      end
