      subroutine isrdinit ( nmain, codemn, nmcode, iretur, iincnd,
     +                      rincnd, maxtrans, maxturb, ndim, icheckeq )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.6    date 20-04-1998 set iretur=-2 in case of end of file
c        version 1.5    date 02-04-1998 lamturb added
c        version 1.4    date 08-05-1996 refine added
c        version 1.3    date 09-05-1995 restart changed
c        version 1.2    date 15-03-1995 (Correction if end of file has been foun
c        version 1.1    date 01-12-1994 (New sequence of equations)
c        version 1.0    date 28-07-1994
c
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read initial conditions part of input files and store result in
c    arrays iincnd and rincnd
c **********************************************************************
c
c                       KEYWORDS
c
c       initial_conditions
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, maxtrans, maxturb, ndim,
     +        iincnd(maxtrans+4+maxturb),
     +        icheckeq(maxtrans+4+maxturb)
      double precision rincnd(maxtrans+4+maxturb)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     icheckeq       Work array to indicate if an equation has been considered
c                    (icheckeq(i)=1 or not icheckeq(i)=0)
c     iincnd    o    reserved
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     ndim      i    Dimension of space (2 or 3)
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rincnd    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, newmain, lenchar
      parameter ( nmainnw=40, newmain=12, lenchar=14 )
      integer nmcode1(newmain), nmcoda(nmainnw), nmcodamx(nmainnw),
     +        nmcodmx(newmain), i, iseq, nmain1, ifunc
      double precision value
      character *16 codema(nmainnw), code1(newmain)
      logical ready, readnx

c     code1     Table containing the  main keywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     ifunc     Parameter k after FUNC =
c     iseq      Sequence number read after transport or turbulence equation
c     lenchar   Number of characters in main keywords
c     newmain   Number of main keywords stored in array code1
c     nmain1    Total number of main keywords (newmain+nmain)
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcodamx  Table containing the maximum number of characters in
c               the keyword table codema. If the keyword contains more
c               characters a warning is given.
c     nmcode1   Array containing the number of significant characters for
c               each keyword in code1
c     nmcodmx   Table containing the maximum number of characters in
c               the keyword table code. If the keyword contains more
c               characters a warning is given.
c     readnx    If true the next item must be read
c     ready     Indicates if a while loop is finished or not
c     value     Real number read by isreadfn
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isintx00

c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the erroneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX00  Compare text in string with table (variable length)
c     ISREADFN  Read items of the shape number, VALUE=number or FUNC=i
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword INITIAL_CONDITION
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    RESTart = value
c    INSESsion = value
c    U_MOMentum = g
c    V_MOMentum = g
c    W_MOMentum = g
c    TRANsport a = g
c    PRESsure = g
c    K_TURBulence = g
c    EPS_TURBulence = g
c    OMEGA_TURBulence = g
c    REfIne = value
c    LAM2TURB = value
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   No integer number has been found after the keyword %
c     531   Internal error in subroutine; character array too small
c     567   Equation has been found more than once
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Read subkeywords until a main keyword is found
c    interpret subkeywords
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code1 / 'RESTART', 'U_MOMENTUM', 'V_MOMENTUM', 'W_MOMENTUM',
     +             'TRANSPORT', 'PRESSURE', 'K_TURBULENCE',
     +             'EPS_TURBULENCE', 'OMEGA_TURBULENCE', 'INSESSION',
     +             'REFINE', 'LAMTURB' /
      data nmcode1  /  4,  5,  5,  5,  4,    4,  6,  8, 10, 5, 4, 3/
      data nmcodmx  /  7, 10, 10, 10,  9,    8, 12, 14, 16, 9, 6, 7 /

c     code1     Table containing the  main keywords corresponding to this
c               subroutine. Contents:
c               1:  RESTart
c               2:  U_MOMentum
c               3:  V_MOMentum
c               4:  W_MOMentum
c               5:  TRANsport
c               6:  PRESsure
c               7:  K_TURBulence
c               8:  EPS_TURBulence
c               9:  OMEGA_TURBulence
c              10:  INSESsion
c              11:  REFIne
c              12:  LAM2turb
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdinit' )

c     --- Make table of new main keywords consisting of the local main keywords
c         and the global main keywords

      nmain1 = newmain+nmain
      if ( nmain1.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( newmain, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do 100 i = 1, newmain
         codema(i)(1:lenchar) = code1(i)(1:lenchar)
         nmcoda(i) = nmcode1(i)
         nmcodamx(i) = nmcodmx(i)
100   continue
      do 110 i = 1, nmain
         codema(i+newmain)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i+newmain) = nmcode(i)
         nmcodamx(i+newmain) = 100
110   continue

c     --- Clear array icheckeq

      do 130 i = 1, maxtrans+4+maxturb
         icheckeq(i) = 0
130   continue

      ready = .false.
      readnx = .true.

200   if ( .not. ready ) then

c     --- while loop ready

c         --- Read next main keyword

         if ( readnx ) call isasknxt
         if ( ityprd.eq.0 ) then

c        --- end of file found, stop reading

            ready = .true.
            iretur = -2
            go to 200
         end if
         iretur = isintx00 ( codema, nmcoda, nmain1, 1, nmcodamx )
         readnx = .true.
         if ( iretur.eq.1 ) then

             iincnd(1) = 2
             iincnd(4) = 0

c            --- restart found, read one  number for restart time

            call isasknxt
            if ( ityprd.ne.3 ) then

c           --- Not an real number found after the keyword

               call iserrchr ( code1(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            end if

            if ( ityprd.eq.3 ) rincnd(1) = realrd

         else if ( iretur.eq.11 ) then

             iincnd(1) = 2
             iincnd(4) = 1

c            --- refine found, read one  number for restart time

            call isasknxt
            if ( ityprd.ne.3 ) then

c           --- Not an real number found after the keyword

               call iserrchr ( code1(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            end if

            if ( ityprd.eq.3 ) rincnd(1) = realrd

         else if ( iretur.eq.12 ) then

             iincnd(1) = 2
             iincnd(4) = 2

c            --- lam2turb found, read one  number for restart time

            call isasknxt
            if ( ityprd.ne.3 ) then

c           --- Not an real number found after the keyword

               call iserrchr ( code1(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            end if

            if ( ityprd.eq.3 ) rincnd(1) = realrd

         else if ( iretur.eq.10 ) then

c        --- insession found, read one  number for sesion number

            call isasknxt
            if ( ityprd.ne.2 ) then

c           --- Not an integer number found after the keyword

               call iserrchr ( code1(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            end if

            if ( ityprd.eq.2 ) iincnd(2) = nint(realrd)

         else if ( iretur.le.newmain ) then

c        --- Other possibility found, i.e. g must be read

            if ( iretur.eq.5 ) then

c           --- transport found, read sequence number

               call isasknxt
               if ( ityprd.ne.2 ) then

c              --- Not an integer number found after the keyword

                  call iserrchr ( code1(iretur), 1 )
                  call iserrsub ( 528, 0, 0, 1 )
                  call iserrcrd
                  iseq = 5

               else

                  iseq = nint(realrd)+4

               end if

            else

               if ( iretur.le.4 ) then

c              --- U, V or W_momentum

                  iseq = iretur-1
                  if ( iseq.gt.ndim ) then

c                 --- w_momentum found in 2D

                  end if

               else if ( iretur.le.6 ) then

c              --- pressure

                  iseq = 4

               else

c              --- turbulence

                  if ( iretur.eq.9 ) then
                    iseq = 4+maxtrans+2
                  else
                    iseq = 4+maxtrans+iretur-6
                  end if

               end if

            end if

            if ( icheckeq(iseq).ne.0 ) then
               call iserrint ( iseq, 1 )
               call iswarsub ( 567, 1, 0, 0 )
               call iserrcrd
            end if
            icheckeq(iseq) = 1

c           --- read g

            call isreadfn ( ifunc, value )
            if ( ifunc.gt.0 .or. ifunc.lt.-100 ) then

c           --- function found

               iincnd(iseq) = -100-ifunc

            else

c           --- value found

               iincnd(iseq) = -1
               rincnd(iseq) = value

            end if

         end if

         if ( iretur.gt.newmain ) then

c        --- main keyword found

            ready = .true.
            iretur = iretur-newmain

         else if ( iretur.le.-1 .or. ityprd.eq.0 ) then

c        --- end of file found

            ready = .true.

         end if
         go to 200

      end if

      call iserclos ( 'isrdinit' )
      end
