      subroutine isreadfn ( ifunc, value )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date   01-04-1996 Guus: extra warning
c        version 1.1    date   28-03-1996 Kees K. timefn added
c        version 1.0    date   24-08-1994 Subroutine extracted from the SEPRAN
c                                         subroutine READFN version 1.1
c
c
c
c   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                         DESCRIPTION
c
c   read rest of record of the type: number
c   or value = number, time_func = i   or  func = i
c **********************************************************************
c
c                      KEYWORDS
c
c  interpret
c  keyword
c  read_cards
c **********************************************************************
c
c                      INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ifunc
      double precision value

c     ifunc   o    Parameter k after FUNC =
c     value   o    Real number read by isreadfn
c **********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cseprd'
      include 'IScommon/csepch'
c **********************************************************************
c
c                      LOCAL PARAMETERS
c
      logical funcfn, timefn

c     funcfn    Indicates if func has been found (true) or not (false)
c     timefn    Indicates if time_func has been found (true) or not (false)
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the errorneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISWARSUB  Warnings
c **********************************************************************
c
c                    I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c     528   Not an integer number found after the keyword
c     535   Parameter in FUNC = k non-positive
c     551   Text not recognised
c     552   No number found after keyword
c     588   Keyword contains too much characters
c **********************************************************************
c
c                    PSEUDO CODE
c
c    read next item
c    func_found := false
c    time_func_found := false
c    if ( next item = text ) then
c       if ( text = 'FUNC' or 'TIME_FUNC' ) then
c          func_found := true
c       if ( text =  'TIME_FUNC' ) then
c          time_func_found := true
c       else if ( text # 'VALUE' ) then
c          error
c       read next item
c    if ( func_found ) then
c       if ( next item = integer ) then
c          ifunc = next item
c       if ( time_func_found ) then
c          ifunc =  -100-ifunc
c       else
c          error
c    else if ( next item = real ) then
c       value = next item
c    else
c       error
c ======================================================================
c
      call iseropen ( 'isreadfn' )
      call isasknxt
      funcfn = .false.
      timefn = .false.
      if ( ityprd.eq.4 ) then

c     --- text found, analyse

         if ( string(1:3).eq.'FUN' .or. string(1:6).eq.'TIME_F' ) then

c        --- Func found

            funcfn = .true.

            if (  string(1:6).eq.'TIME_F' ) then
                timefn = .true.
            else if ( nchar.gt.4 ) then
               call iserrint ( nchar, 1 )
               call iserrint ( 4, 2 )
               call iserrchr ( 'FUNC', 1 )
               call iswarsub ( 588, 2, 0, 1 )
            end if

         else if ( string(1:3).ne.'VAL' ) then

c        --- Wrong text found

            call iserrsub ( 551, 0, 0, 0 )
            call iserrcrd

          end if
          call isasknxt

       end if
       if ( ityprd.lt.2 .or. ityprd.gt.3 ) then

c      --- Not a number found

          call iserrchr ( string(1:nchar), 1 )
          call iserrsub ( 552, 0, 0, 1 )
          call iserrcrd

       end if
       if ( funcfn ) then

c      --- FUNC = i, found

          if ( ityprd.ne.2 ) then

c         --- Not an integer found

             call iserrchr ( string(1:nchar), 1 )
             call iserrsub ( 528, 0, 0, 1 )
             call iserrcrd

          else

             ifunc = nint(realrd)
             if ( ifunc.le.0 .or. ifunc.gt.20000 ) then

c            --- IFUNC out of range

                call iserrint ( ifunc, 1 )
                call iserrint ( 20000, 2 )
                call iserrsub ( 535, 2, 0, 0 )
                call iserrcrd

             end if
             if ( timefn ) ifunc = -100-ifunc

          end if

       else

c      --- value found

          value = realrd
          ifunc = 0

       end if

      call iserclos ( 'isreadfn' )
      end
