      subroutine iswrtif1 ( idimens, maxcoefs, maxtrans, iincof,
     +                      rincof, ndim, icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date 06-01-1998 Guus: Extension of coefficients
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.1    date 11-06-1997 Extension iincof
c        version 1.0    date 27-09-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write second part of the intermediate file
c    It concerns the blocks 5 to 5
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idimens(100), maxcoefs, maxtrans,
     +        iincof(maxcoefs,maxtrans+1), ndim, icoupled
      double precision rincof(maxcoefs,maxtrans+1)

c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     idimens   i    Integer dimension parameters for the intermediate file
c     iincof    i    reserved
c     maxcoefs  i    Maximum number of coefficients allowed
c     maxtrans  i    Maximum number of transport equations allowed
c     ndim      i    Dimension of space (2 or 3)
c     rincof    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, itrans, ncoefs, ntrnsp

c     i         General loop variable
c     itrans    reserved
c     ncoefs    Number of coefficients
c     ntrnsp    Number of transport equations
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c
c     Block 5: coefficients
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Get ntrnsp and ncoefs
c     write coefficients for momentum equation
c     write coefficients for transport equations
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Get ntrnsp and ncoefs

      ntrnsp = idimens(3)
      ncoefs = idimens(5)

c     --- write coefficients for momentum equation

      if ( icoupled.eq.0 ) then

c     --- Coupled equations

         write(irefwr,100) (iincof(i,1),i=1,ncoefs)
         write(irefwr,110) (rincof(i,1),i=1,ncoefs)

      else

c     --- Decoupled equations
c         First u-momentum

         write(irefwr,120) (iincof(i,1),i=1,ncoefs)
         write(irefwr,110) (rincof(i,1),i=1,ncoefs)

c        --- Next v-momentum

         write(irefwr,130) (iincof(i,1),i=1,ncoefs)
         write(irefwr,110) (rincof(i,1),i=1,ncoefs)

         if ( ndim.eq.3 ) then

c        --- Finally w-momentum

            write(irefwr,140) (iincof(i,1),i=1,ncoefs)
            write(irefwr,110) (rincof(i,1),i=1,ncoefs)

         end if

      end if

100   format ( /'**** BLOCK 5: Parameters with respect to the',
     +          ' coefficients'//
     +          '** Subblock 5.1 momentum equations'//
     +          '* iincof :'//(10i6) )
110   format ( /'* rincof :'//
     +          ' rho    = ',d15.7/ ' mu     = ',d15.7/
     +          ' fone   = ',d15.7/ ' ftwo   = ',d15.7/
     +          ' fthree = ',d15.7/ ' mass_frac= ',d15.7/
     +          ' interface_drag = ',d15.7/
     +          ' cone = ',d15.7/
     +          ' ctwo = ',d15.7/
     +          ' cthree = ',d15.7///
     +          ' dummies:'/(5d15.7) )
120   format ( /'**** Subblock 5.1a u-momentum equations'//
     +          '* iincof :'//(10i6) )
130   format ( /'**** Subblock 5.1b v-momentum equations'//
     +          '* iincof :'//(10i6) )
140   format ( /'**** Subblock 5.1c w-momentum equations'//
     +          '* iincof :'//(10i6) )

c     --- write coefficients for transport equations

      do itrans = 1, ntrnsp
         write(irefwr,200) itrans+1,itrans,
     +     (iincof(i,1+itrans),i=1,ncoefs)
200      format ( /'** Subblock 5.',i2,' transport equation',i2//
     +             '* iincof :'/(10i6) )
         write(irefwr,210) (rincof(i,1+itrans),i=1,ncoefs)
210      format ( /'* rincof :'/
     +          ' capacity = ',d15.7/' constant = ',d15.7/
     +          ' source   = ',d15.7/' difxx    = ',d15.7/
     +          ' difxy    = ',d15.7/' difyy    = ',d15.7/
     +          ' difxz    = ',d15.7/' difyz    = ',d15.7/
     +          ' difzz    = ',d15.7//'dummies:'/
     +           (5d15.7) )
      end do

      end
