      subroutine iswrtif3 ( idimens, iintim, rintim, maxtrans, maxturb,
     +                      iincnd, rincnd, ndim )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.6    date 27-05-1998 changes for iintim
c        version 2.5    date 26-03-1998 changes for iintim
c        version 2.4    date 10-02-1997 changes for iintim
c        version 2.3    date 22-04-1996 changes for restart
c        version 2.2    date 10-05-1995 Extension with restart
c        version 2.1    date 11-01-1995 Extension of time arrays
c        version 2.0    date 01-12-1994 Extra parameters maxtrans and maxturb
c        version 1.0    date 27-09-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write fourth part of the intermediate file
c    It concerns the blocks 7 to 8
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idimens(100),  maxtrans, maxturb, ndim, iintim(100),
     +        iincnd(4+maxtrans+maxturb)
      double precision rintim(100), rincnd(4+maxtrans+maxturb)

c     idimens   i    Integer dimension parameters for the intermediate file
c     iincnd    i    reserved
c     iintim    i    reserved
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     ndim      i    Dimension of space (2 or 3)
c     rincnd    i    reserved
c     rintim    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ntrnsp, nturb

c     i         General loop variable
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c     Block 7: Parameters with respect to the time integration
c     Block 8: Parameters with respect to the initial condition
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Get ndegfd
c    write array iintim
c    write array rintim
c    write array iincnd
c    write array rincnd
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Get ndegfd

      ntrnsp = idimens(3)
      nturb  = idimens(6)

c     --- write array iintim

      write(irefwr,100) (iintim(i),i=1,17)
100   format ( /'**** BLOCK 7: Parameters with respect to the',
     +          ' time integration'//
     +          '* iintim:'//
     +          'method =  ', i5/
     +          4i2,12x, '  # Positions 2-5 not yet defined'/
     +          'nfrac  = ', i5/
     +          'tvar   = ', i5/
     +          'ndt    = ', i5/
     +           i2,18x,  '  # Positions 9 not yet defined'/
     +          'istationary =  ', i5/
     +          'restart        =  ', i5/
     +          'ouput          =  ', i5/
     +          'force_deletion =  ', i5/
     +          'outsession     =  ', i5/
     +          'norm           =  ', i5/
     +          'nsubsteps      =  ', i5/
     +          'press_corr_time_iteration =  ', i5 )
      write(irefwr,105) (iintim(i),i=18,100)
105   format(    3i2, '  # Positions 18-20 not yet defined'/
     +          10i2, '  # Positions 21-30 not yet defined'/
     +          10i2, '  # Positions 31-40 not yet defined'/
     +          10i2, '  # Positions 41-50 not yet defined'/
     +          10i2, '  # Positions 51-60 not yet defined'/
     +          10i2, '  # Positions 61-70 not yet defined'/
     +          10i2, '  # Positions 71-80 not yet defined'/
     +          10i2, '  # Positions 81-90 not yet defined'/
     +          10i2, '  # Positions 91-100 not yet defined' )

c     --- write array rintim

      write(irefwr,110) (rintim(i),i=1,20)
110   format ( /'* rintim :'//
     +           d15.7,'                   # Pos.1 is dummy'/
     +          'tstart     =', d15.7/
     +          'tend       =', d15.7/
     +          'dt         =', d15.7/
     +          'theta      =', d15.7/
     +          'toutinit   =', d15.7/
     +          'toutend    =', d15.7/
     +          'toutstep   =', d15.7/
     +          'rinit      =', d15.7/
     +          'rend       =', d15.7/
     +          'rstep      =', d15.7/
     +          6f4.0, '  # Positions 12-17 not yet defined'/,
     +          'relaxation_press_corr  =', d15.7/
     +          'absaccuracy_press_corr =', d15.7/
     +          'relaccuracy_press_corr =', d15.7 )
      write(irefwr,120) (rintim(i),i=21,30)
120   format ( 'absaccuracy_mom =', d15.7/
     +          'relaccuracy_mom =', d15.7/
     +          'absaccuracy_prs =', d15.7/
     +          'relaccuracy_prs =', d15.7/
     +          'absaccuracy_trs =', d15.7/
     +          'relaccuracy_trs =', d15.7/
     +          'absaccuracy_tur =', d15.7/
     +          'relaccuracy_tur =', d15.7/
     +          'absaccuracy_stat =', d15.7/
     +          'relaccuracy_stat =', d15.7 )
      write(irefwr,140) (rintim(i),i=31,40)
140   format (  3d15.7, '  # theta1 ... theta3'/
     +          3d15.7, '  # theta4 ... theta6'/
     +          3d15.7, '  # theta7 ... theta9'/
     +          d15.7,30x, '  # theta10')
      write(irefwr,150) (rintim(i),i=41,60)
150   format (  3d15.7, '  # dt1 ... dt3'/
     +          3d15.7, '  # dt4 ... dt6'/
     +          3d15.7, '  # dt7 ... dt9'/
     +          d15.7,30x, '  # dt10'/
     +          3d15.7, '  # tend1 ... tend3'/
     +          3d15.7, '  # tend4 ... tend6'/
     +          3d15.7, '  # tend7 ... tend9'/
     +          d15.7,30x, '  # tend10' )
      write(irefwr,160) (rintim(i),i=61,100)
160   format (  10f4.0, '  # Positions 61-70 not yet defined'/
     +          10f4.0, '  # Positions 71-80 not yet defined'/
     +          10f4.0, '  # Positions 81-90 not yet defined'/
     +          10f4.0, '  # Positions 91-100 not yet defined'/ )

c     --- write array iincnd

      write(irefwr,200) (iincnd(i),i=1,ndim), iincnd(4),
     +                  (iincnd(i+4),i=1,ntrnsp),
     +                  (iincnd(i+4+maxtrans),i=1,nturb)
200   format ( /'**** BLOCK 8: Parameters with respect to the',
     +          ' initial conditions'//
     +          '* iincnd:'//(10i6) )

c     --- write array rincnd

      write(irefwr,210) (rincnd(i),i=1,ndim), rincnd(4),
     +                  (rincnd(i+4),i=1,ntrnsp),
     +                  (rincnd(i+4+maxtrans),i=1,nturb)

210   format ( //'* rincnd:'//(5d15.7) )

      end
