      subroutine iswrtif9 ( idimens, maxtrans, maxturb, iseqeq, ndim,
     +                      icoupled )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.2    date 05-01-2000 Guus: correction iseqeq
c        version 2.1    date 21-10-1999 Guus: skip pressure separately
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.1    date 08-09-1997 New block number
c        version 1.0    date 08-03-1995
c
c
c   copyright (c) 1995-2000  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write  sequence of equations to  the intermediate file
c    It concerns the blocks 17 to 17
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idimens(100), maxturb, maxtrans,
     +        iseqeq(maxtrans+2+maxturb), ndim, icoupled

c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     idimens   i    Integer dimension parameters for the intermediate file
c     iseqeq    i    Array with sequence numbers of equations
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     ndim      i    Dimension of space (2 or 3)
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  itrans, ntrnsp, nturb, lshift

c     itrans    reserved
c     lshift    Local shift
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c
c     Block 15: Parameters with respect sequence numbers of the equations
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Get ntrnsp and nturb
c     write integer information
c        write sequence number for momentum equation
c        write sequence number for transport equations
c        write sequence number for turbulence equations
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Get ntrnsp and nturb

      ntrnsp = idimens(3)
      nturb  = idimens(6)

c     --- write integer information
c         write information for momentum equation

      write(irefwr,100)
100   format ( /'**** BLOCK 17: Sequence of the equations'//)

      if ( icoupled.eq.0 ) then

c     --- Coupled equations

         write(irefwr,110) iseqeq(1)
110      format ( /'* iseqeq:'/
     +             i5,3x,' # momentum equation '/ )
         lshift = 0

      else

c     --- Decoupled equations
c         First u-momentum

         write(irefwr,120) iseqeq(1)
120      format ( /'* iseqeq:'/
     +             i5,3x,' # u-momentum equation '/ )

c        --- Next v-momentum

         write(irefwr,130) iseqeq(1)+1
130      format ( i5,3x,' # v-momentum equation '/ )

         if ( ndim.eq.3 ) then

c        --- Finally w-momentum

            write(irefwr,140) iseqeq(1)+2
140         format ( i5,3x,' # w-momentum equation '/ )

         end if
         lshift = ndim-1

      end if

      write(irefwr,150) iseqeq(1+1+lshift)
150   format ( i5,3x,' # pressure equation'/ )

c            --- write information about the transport equations

      lshift = lshift+1
      do itrans = 1, ntrnsp
         write(irefwr,160) iseqeq(lshift+1+itrans), itrans
160      format ( i5,3x,' # transport equation',i2,/ )
      end do

c     --- write information about the turbulence equations

      do itrans = 1, nturb
         write(irefwr,170) iseqeq(1+lshift+ntrnsp+itrans), itrans
170      format ( i5,3x,' # turbulence equation',i2,/ )
      end do

      end
