      subroutine isactive ( index )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date   08-03-1996 Put name of array in message
c        version 1.0    date   21-06-1994 Extracted from the SEPRAN subroutine
c                                         ini070 version 1.3
c
c
c   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c     Check if an already existing array is positioned in array IBUFFR or not
c
c     If the array is available in IBUFFR, then the priority is set equal
c     to 10, and the subroutine returns
c
c     When the array is only available on backing storage, and not in IBUFFR
c     then the array is read from file 3 and space is reserved in array IBUFFR
c
c     if no space is available in array IBUFFR, ISACTIVE tries to
c     create space in the following ways:
c
c       -  by writing low priority arrays to backing storage
c
c       -  if this is not sufficient by compressing
c          array ibuffr after writing of all arrays with priority
c          smaller than 10 to backing storage
c
c     The name of the calling subroutine is extracted from common blocks
c     CISMESSC and CISMESSG, so it is supposed that the subroutine ISEROPEN has
c     been used
c
c **********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer index

c     index   i  Integer variable containing the sequence number of the
c                array to be processed
c **********************************************************************
c
c                    COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/cisnamtab'
c **********************************************************************
c
c                    LOCAL PARAMETERS

      integer indfor, length, number, ipstar, iextra, nbuffr, iinfor,
     +        ibfree, nbufef

c     ibfree  Next free position in array IBUFFR
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     iinfor  Number of arrays that has been referenced
c     indfor  Help parameter to store the record number from which the
c             array has been stored in file 3
c     ipstar  Computed starting position in array IBUFFR
c     length  Length of the array in words
c     nbufef  Last position in IBUFFR before array inback
c     nbuffr  Length of buffer array ibuffr
c     number  Absolute value of index
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISINI049  Create space in IBUFFR by writing low priority arrays to
c               file 3, or to compress IBUFFR
c     ISINICHK  Check IBUFFR
c     ISPRINBF  Print and check length of array IBUFFR
c     ISREADFL  Read from direct access file
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                              I/O
c
c     If necessary arrays are written to and/or read from file 3 (iref3)
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c    1400 the array from which the starting address must be found
c         does not exist
c
c **********************************************************************
c
c                    PSEUDO CODE
c
c     If ( idebug > 0 ) then
c        Set iextra = extra space needed
c        check all control integers
c     If ( INDEX = 0 or INDEX > IINFOR ) then
c
c         Error: Array does not exist;  STOP
c
c     Set priority
c
c     If ( array on file 3 ) then
c
c         If ( enough space in IBUFFR after IBFREE ) then
c
c             IPSTAR = IBFREE
c             Raise IBFREE
c
c         else
c
c             Create space by writing low priority arrays to file 3
c             or compressing IBUFFR
c
c         Read array from file 3
c
c     Set INBACK ( |INDEX| ) negative if necessary
c ======================================================================
c
c     --- extract constants from ibuffr

      nbuffr = ibuffr(7)
      iinfor = ibuffr(3)
      ibfree = ibuffr(4)
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call isinichk ( ibuffr, ibuffr(11) )
      end if
      number = abs (index)
      if ( index.eq.0 .or. number.gt.iinfor ) then

c     --- index < 0: error

         call iseropen ( 'isactive' )
         call iserrint ( index,  1 )
         call iserrint ( number, 2 )
         call iserrint ( iinfor+1, 3 )
         call iserrsub ( 1400, 3, 0, 0 )
         call isstop

      end if

c     --- Set priority 10 in INFOR

      ibuffr(13+3*(number-1)) = max(ibuffr(13+3*(number-1)),10)
      length = ibuffr(12+3*(number-1))
      ipstar = ibuffr(11+3*(number-1))

      if ( ipstar.lt.0 ) then

c     --- ipstar < 0  array in file 3

         indfor = -ipstar
         if ( ibfree+length+iextra*2-1.le.3*nbuffr/4 ) then

c        --- ibfree+length-1 <= nbuffr:  Read array in IBUFFR
c                                        compute ipstar and raise ibfree
c            For a 32-bits computer, ibfree must always be odd

            ibuffr(11+3*(number-1)) = ibfree
            ipstar = ibfree
            ibfree = ibfree+length+iextra*2
            if ( intlen.eq.2 .and. mod(ibfree,2).eq.0 )
     +           ibfree = ibfree+1

         else

c        --- try to create free space by writing low priority arrays
c            to backing storage   ( file 3 ) or compressing ibuffr

            call isini049 ( length, ipstar, ibuffr )
            ibuffr(11+3*(number-1)) = ipstar

         end if

         call isprinbf ( ibuffr, ibfree-1 )
         call isreadfl ( indfor, ipstar-iextra, ipstar+length+iextra-1,
     +                   3, ibuffr )

      end if

      nbufef = ibuffr(1)-6*ibuffr(2)
      if ( index.gt.0 .and. ibuffr(nbufef+number).gt.0 )
     +      ibuffr(nbufef+number ) = -ibuffr(nbufef+number )
      if ( ioutpt.gt.2 ) then
         write(irefwr,100) namarrays(abs(index)), abs(index), ipstar,
     +                     namsubs(levelsub)//':isactive'
100      format ( ' Activate array ', a, ' with sequence number', i4,
     +            ' and starting address', i9/' in subroutine ', a )
      end if
      ibuffr(4) = ibfree
      end
