      function isdpleng ( index01 )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        inilen version 1.0)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     Return with length of array corresponding to index01 in double precisions
c     It is checked if pointer is available
c ********************************************************************
c
c                      KEYWORDS
c
c    memory_management
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer isdpleng, index01

c     index01  i   The value of index01 is the reference number of the
c                  array of which the starting position must be computed with
c                  respect to array INFOR
c                  This means that the positions (i,index01) (i=1,2,3) in INFOR
c                  contain all the information about the storage of the array
c     isdpleng o   Gives length of array corresponding to index01
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer length, nvectr

c     length    Length of the array in words
c     nvectr    Maximum number of arrays that may be stored in IBUFFR
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c    1412   INDEX out of range
c ********************************************************************
c
c                    PSEUDO CODE
c
c   Check if index01 between 1 and 1500
c   Set lengthr
c ======================================================================
c
      nvectr = ibuffr(2)
      length = 0
      if ( index01.lt.1 .or. index01.gt.nvectr ) then

c     --- index01 is not a valid index01

         call iseropen ( 'isdpleng' )
         call iserrint ( index01, 1 )
         call iserrint ( abs(index01), 2 )
         call iserrint ( nvectr, 3 )
         call iserrsub ( 1412, 3, 0, 0 )
         call iserclos ( 'isdpleng' )

      else

c     --- index01 in the correct range,

         length = ibuffr(12+3*(index01-1))

      end if

      isdpleng = length/intlen

      end
