      subroutine isermess ( mesnmseq, nints, nreal, nchar, type )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.3    date   15-09-1999 Guus: print traceback through
c                                         subroutine
c        version 1.2    date   06-08-1997 Guus: return in case of error
c        version 1.1    date   30-04-1995 Improved layout
c        version 1.0    date   21-06-1994 Extracted from the SEPRAN subroutine
c                                         ermess1 version 1.8
c
c
c     copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
c     permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "Ingenieursbureau SEPRA".
c     all rights reserved. no part of this publication may be reproduced,
c     stored in a retrieval system ( e.g., in memory, disk, or core)
c     or be transmitted by any means, electronic, mechanical, photocopy,
c     recording, or otherwise, without written permission from the
c     publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISERMESS prints a (warning or error) message.
c
c **********************************************************************
c
c                       KEYWORDS
c
c    error_manipulation
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  mesnmseq, nchar, nints, nreal
      character *(*) type

c     mesnmseq  i  Message sequence number.
c     nchar     i  Number of strings in a message
c     nints     i  Number of integers in a message
c     nreal     i  Number of reals in a message
c     type      i   Indicates if it concerns an error or a warning
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cismessc'
      include 'IScommon/cismessg'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ifrom, ipttrg, iref, istarmes(21), ito, lenspe,
     +        linenr, mxerlc, numcha, numint, numlin, numrea, numspe
      logical nxtlin, exist
      parameter ( lenspe = 2 )
      character*80 errlin, messpe(lenspe)
      character*160 line

c     errlin    One line of the error message
c     exist     Flag: true= error number exists in error text file.
c     i         General loop variable
c     ifrom     Information in array istarmes is read from position ifrom
c     ipttrg    Pointer to first position not filled in LINE
c     iref      Reference to file
c     istarmes  Integer array containing a part of the starting records
c               of the messages.
c     ito       Information in array istarmes is read to position ito.
c     lenspe    Length of special message array.
c     line      Text line
c     linenr    Record number of message line
c     messpe    Special error message
c               'Failure on open of error text file'.
c     mxerlc    1 + maximum number of messages that are stored
c               in the message file.
c     numcha    Number of character strings in error message.
c     numint    Number of integers in error message
c     numlin    Number of lines of a message
c     numrea    Number of reals in error message
c     numspe    The number of the special error message.
c     nxtlin    True: scan next line, false :don't scan next line
c **********************************************************************
c
c                       I/O
c
c     ISERMESS writes to the default output device.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    Message number not yet added to the Error Text File.
c    Number of integers, reals or strings in message not correct.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERERRL       scans and prints one line of a message
c     ISREADFL       Read from direct access file
c     ISTRACEBACK    print traceback of calling subroutines
c **********************************************************************
c
c                       PSEUDO CODE
c
c     read maximum message number from error text file ;
c     if (message number correct) then
c        read starting record number ;
c        calculate number of lines ;
c        if ( number of lines > 0 ) then
c           for each line do
c              read line ;
c              process and print line ;
c        else
c           print warning : message does not exist ;
c     else
c        print warning : message number out of range ;
c     if (number of integers, reals and/or strings not correct) then
c        print warning ;
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- initialize some parameters

      data messpe /
     + ' Failure on open of error message text file ',
     + ' unit= $, status= $'/
      numspe = -1
      iref   = 4
      exist  = .true.

      write (irefwr,'(//1x,78(''*'')/)')

c     --- check if there is enough space to print the module name
c         on the same line as the message number.
c         (Criterion: at most 79 characters on one line)
c         Otherwise, print it on the next line.
c         48 = number of free positions at the end of the
c              message heading line.

      if ( levelsub.eq.0 ) then
         write (irefwr,1000) type, mesnmseq, 'main'
      else
         write (irefwr,1000) type, mesnmseq, namsubs(levelsub)
      end if

c     --- read largest message number permitted

      if (mesnmseq.eq.numspe) then

c     --- special error message: failure in open of text file.

         mxerlc = numspe + 1
      else
         ifrom = 1
         ito   = 1
         linenr= 1
         call isreadfl ( linenr, ifrom, ito, iref, istarmes )
         mxerlc = istarmes(1)
      end if

      if ( (mesnmseq.gt.0 .and. mesnmseq.le.mxerlc)
     +     .or.   mesnmseq.eq.numspe  ) then

c     --- Read starting record of message

         if (mesnmseq.eq.numspe) then

c        --- special error message: open failure of error text file

            numlin = lenspe
            linenr = 1
         else
            linenr = 2 + (mesnmseq-1)/nrec4
            ito    = mod(mesnmseq,nrec4)+1
            if (ito.eq.1) ito = nrec4+1
            call isreadfl (linenr, ifrom, ito, iref, istarmes)
            linenr = istarmes(ito-1)
            numlin = istarmes(ito) - linenr
         end if

         if ( numlin.gt.0 ) then

c        --- Read and print message

            i=linenr
            numint=0
            numrea=0
            numcha=0
            nxtlin=.true.
            ipttrg=0
            line=' '

  100       if (nxtlin) then

c           --- read one message line

               if (mesnmseq.eq.numspe) then

c              --- special message: failure in open of text file

                  errlin = messpe(i)
               else
                  read ( irefer, rec=i ) errlin
               end if

c              --- scan message line for integer fields ($),
c                  real fields (&) and character fields (%), and
c                  write line to output device.

               call isererrl ( errlin,ints,nints,reals,nreal,chars,
     +                         nchar,numint,numrea,numcha,ipttrg,line )

               i = i + 1
               nxtlin = i.le.(linenr+numlin-1)
               goto 100
            end if
            if ( icarcn.eq.0 ) then
               if ( ipttrg.gt.0 ) write (irefwr,'(1x,a)') line(1:ipttrg)
            else
               if ( ipttrg.gt.0 ) write (irefwr,'(2x,a)') line(1:ipttrg)
            end if

         else

c        --- Message not present in the file

            write (irefwr,1100) type, mesnmseq
            write (irefwr,1200) (ints(i),i=1,nints)
            write (irefwr,1300) (reals(i),i=1,nreal)
            write (irefwr,1600) (chars(i),i=1,nchar)
            exist = .false.

         end if

      else

c     --- Message number out of range

         write (irefwr,1100) type, mesnmseq
         write (irefwr,1200) (ints(i),i=1,nints)
         write (irefwr,1300) (reals(i),i=1,nreal)
         write (irefwr,1600) (chars(i),i=1,nchar)
         exist = .false.

      end if

c     --- warning if number of integers, reals, or strings not correct
c         (if error number exists)

      if (exist) then
         if(numint.ne.nints.or.numrea.ne.nreal.or.numcha.ne.nchar)then
            write (irefwr,'(//1x,78(''*'')/)')
            write (irefwr,1400) type,mesnmseq,numint,nints,numrea,nreal,
     +                          numcha,nchar
            write (irefwr,1200) (ints(i),i=1,nints)
            write (irefwr,1300) (reals(i),i=1,nreal)
            write (irefwr,1600) (chars(i),i=1,nchar)
            write (irefwr,'(/1x,78(''*'')//)')
         end if
      end if

c     --- Write extra information if available

      if ( t.gt.0 ) write(irefwr,200) t
200   format(/' Actual time is ',ss,1pe12.3e2/)
      if ( iblockrf.gt.0 .and. (mesnmseq.lt.1600 .or. mesnmseq.gt.1699))
     +   write(irefwr,210) iblockrf
210   format(/' Actual block number is ',i6/)
      if ( inodenr.gt.0 ) write(irefwr,220) inodenr
220   format(/' Actual node number is ',i6/)

c     --- write trace back of subroutines

      call istraceback
      write (irefwr,'(/1x,78(''*'')//)')

 1000 format (' *** ',a,' number ',i5,' , in module ',a/)
 1100 format (' *** WARNING : '/
     +        ' ',a,' number ',i6,' has not yet been added',
     +        ' to the error message file')
 1200 format (' *** Ints  : ',5i13)
 1300 format (' *** Reals : ',5e13.7)
 1400 format (
     + ' *** INTERNAL ERROR : number of integers, reals,',
     + ' or strings in message '/
     + ' not equal to number of integers, reals, or strings',
     + ' in program.'/
     + ' ',a,' number : ',i5/
     + ' number of integers in message :',i4,
     + ', in program :',i4/
     + ' number of reals in message    :',i4,
     + ', in program :',i4/
     + ' number of strings in message  :',i4,
     + ', in program :',i4)
 1600 format (' *** Strings : '/(1x,a78))

      end
