      subroutine iserrsub ( mesnmseq, nints, nreal, nchar )
c ======================================================================
c
c        Programmer     Guus Segal
c        version 1.4    date 23-11.1998 Jason: call to isersend
c        version 1.3    date 06-08-1997 Guus: return in case of error
c        version 1.2    date 08-02-1996 Set ierror
c        version 1.1    date 01-02-1996 Stop at node as soon as error has
c                            		been detected
c        version 1.0    date 21-06-1994 Extracted from the SEPRAN subroutine
c                            		errsub version 2.2
c
c
c     copyright (c) 1994-1997  "Ingenieursbureau SEPRA"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "Ingenieursbureau SEPRA"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISERRSUB prints an error message.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     error_manipulation
c     parallel
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer mesnmseq, nints, nreal, nchar

c     mesnmseq i    Message sequence number.
c     nchar    i    Number of strings in a message
c     nints    i    Number of integers in a message
c     nreal    i    Number of reals in a message
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/ciscom'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c     None.
c **********************************************************************
c
c                       I/O
c
c     ISERRSUB writes to the default output device if it is called at the host
c     If ISERRSUB is called at the node information is sent to the host
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERMESS  Prints a message, stored in the Error Text File.
c     ISERSEND  Send error message to host
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     Number of errors = maxerr , STOP.
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If ( mesnum<0 or nodenr=0 ) then
c        Increment NERROR (number of errors);
c        Call ISERMESS;
c        if (number of errors = maxerr) then
c           Print message;
c           STOP;
c     else ( nodenr>0 )
c        Call isersend to send error message to host
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( mesnmseq.lt.0 .or. inodenr.eq.ISHOST ) then

c     --- Program runs at host

         nerror = nerror + 1

c        --- Print the message, stored in the Error Text File.

         call isermess ( abs(mesnmseq), nints, nreal, nchar, 'ERROR' )

         ierror = abs(mesnmseq)

c        --- Stop if number of errors = maxerr

         if ( nerror.eq.maxerr ) then
            write (irefwr,'(//1x,78(''*'')/)')
            write (irefwr,1500)
            write (irefwr,'(/1x,78(''*'')//)')
            call isstop
         end if

 1500    format (' STOP : number of errors = 10 ')

      else

c     --- Program runs at node
c         Send error description to host

         call isersend ( mesnmseq, nints, nreal, nchar, ICERROR )
         ierror = mesnmseq

      end if

      end
