      subroutine isincopy ( iarr1, iarr2, ist1, ist2, length, idirec )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        ini000 version 3.3)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     copy information from integer array iarr1 into integer array iarr2
c     Both arrays may be part of the same integer array
c
c ********************************************************************
c
c                      KEYWORDS
c
c     copy
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer iarr1(*), iarr2(*), ist1, ist2, length, idirec

c     iarr1   i    Input array to be copied
c     iarr2   o    Output array (result of copying)
c     idirec  i    Indication whether the information must be copied
c                  from the first to the last position (idirec=1)
c                  or vice versa (idirec=-2)
c                  when idirec=-1 the contents of iarr1 are stored in
c                  reversed order in iarr2
c                     iarr1 from the back
c                     iarr2 forward
c                  when idirec=0 the subroutine returns when ist1=ist2
c                  when ist1>ist2 the subroutine reacts as if idirec=1
c                  when ist1<ist2 the subroutine reacts as if idirec=-2
c     ist1    i    First position in iarr1
c     ist2    i    First position in iarr2
c     length  i    Length of the array in words
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
       integer jdirec, i, ih, ih1, ih2

c     i       General loop variable
c     ih      Help variable to store a constant temporarily
c     ih1     Help variable to store a constant temporarily
c     ih2     Help variable to store a constant temporarily
c     jdirec  Defines the type of copying to be performed.
c             Possible values:
c             -2  backward copying
c             -1  copying in reversed order
c              1  forward copying
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                    I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c ======================================================================
c
      if ( ist1.ne.ist2 .or. idirec.ne.0 ) then
         if ( idirec.eq.0 ) then
            if ( ist1.gt.ist2 ) then
               jdirec = 1
            else
               jdirec = -2
            end if
         else
            jdirec = idirec
         end if
         if ( jdirec+1.gt.0 ) then

c        --- jdirec = 1 forward copying

            ih2 = ist2-1
            ih1 = ist1-1

c$dir no_parallel

            do 110 i = 1,length
               iarr2(i+ih2) = iarr1(i+ih1)
110         continue

         else if ( jdirec+1.lt.0 ) then

c        --- jdirec = -2  backward copying

            ih = length

c$dir no_parallel

            do 210 i = 1,length
               iarr2(ih+ist2-i) = iarr1(ih+ist1-i)
210         continue

         else

c        --- jdirec = -1   copying in reversed order

            ih = length+1
            ih2 = ist2-1
            ih1 = ist1-1

c$dir no_parallel

            do 310 i = 1,length
               iarr2(i+ih2) = iarr1(ih+ih1-i)
310         continue

         end if

      end if

      end
