      subroutine isreadcd
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Subroutine extracted from the SEPRAN
c                                        subroutine readcd version 1.7)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     read a record from the standard input file
c
c     ISREADCD has the following characteristics:
c
c     Blank records are skipped
c     Records starting with a * in column 1 are skipped
c     Only information stored before the comment sign # is stored in the
c     string cards
c     The input is stored in cards, with trailing blanks excluded
c     The input is moved to the end of cards, hence cards(80) contains the
c     last non-blank character before the comments
c
c     The following special commands are recognised and executed immediately
c
c       set warn on/off
c       set max warn =
c       set output on/off/out  or level=-1/0/1/2/3
c       set time on/off
c
c       All letters are translated to upper case, except those between
c       begin and end quotes
c       However, if ispecl=3 at input, no translation takes place
c       Furthermore with respect to a blank card the subroutine reacts as
c       in the case ispecl=1
c
c       If 0 < ispecl < 4, the following special actions are carried out:
c
c       If ispecl=1 at input, the special situation for the
c       program a111 is present
c       In that case the subroutine reacts on a blank card by
c       setting ispecl = 2 and returning to the calling subroutine
c       Furthermore the subroutine checks the presence of the
c       command stop.
c       If stop is found, subroutine isstop is called (stop!)
c ********************************************************************
c
c                      KEYWORDS
c
c    read
c    read_cards
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none

c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'
      include 'IScommon/cisnas'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      logical ready
      integer next, j, jquost, ios, nextpr
      character *1 tab

c     ios       Iostatus variable
c     j         General loop variable
c     jquost    Position of start quote in text
c     next      Next column number with an interesting phenomenon
c     nextpr    Contains previous value of next
c     ready     Indicates if a while loop is finished or not
c     tab       Horizontal tab character
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISASKUPC  Translate all lower case letters into upper case
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISREADNM  Read number
c     ISSTOP    Stop ISNAS
c ********************************************************************
c
c                    I/O
c
c    Records are read form the input file
c ********************************************************************
c
c                    ERROR MESSAGES
c
c    1411   End of file found
c    1423   Error in reading record
c ********************************************************************
c
c                    PSEUDO CODE
c
c   ready := false
c   while ( not ready ) do
c
c      cards := ' '; cardim := ' '
c      irecnr := irecnr+1
c      continue := true
c      Read record into cardim
c      If (end of record reached) then
c         ityprd := 0
c         ready := true
c         if ( istop = 0 ) stop
c      else
c         lastp := last non-blank in cardim
c         last  := last non-blank before comment character #
c         icolum := first non-blank character
c         if ( first character = * ) then
c         else if ( record is blank ) then
c            if ( ispecl=1 or ispecl=3 or ispecl = 7 ) then
c               ispecl := 2
c               ready := true
c         else
c            if ( ispecl = 3 or ispecl = 7 ) then
c               Copy cardim into cards
c            else
c               Copy Uppercase(cardim) into cards
c               Translate tab into space
c            if ( ispecl = 1 and first word = stop ) stop
c            if ( first word = set ) then
c               if ( next word = warn ) then
c                  if ( next word = on or off ) then
c                     set nwarn; continue := false
c               else if ( next word = time ) then
c                  if ( next word = on or off ) then
c                     set itime; continue := false
c               else if ( next word = max ) then
c                  read next number
c                  set maxwar; continue := false
c               else if ( next word = output) then
c                  if ( next word = on or off or out ) then
c                     set ioutpt; continue := false
c                  else if ( next word = level ) then
c                     read level; set ioutpt; continue := false
c            If ( continue ) then
c               Reset all texts between quotes or after FILE =
c               move information to end of cards
c               print cards if necessary
c               ready := true
c ======================================================================
c
      call iseropen ( 'isreadcd' )

c     --- Set tab character

      tab = char(9)

      ready = .false.

100   if ( .not. ready ) then

c     --- Initialize cardim and cards

         cardim = ' '
         cards  = ' '
         irecnr = irecnr+1
         if ( irefre.eq.0 ) then
            read( *, 110, iostat = ios ) cardim
         else
            read( irefre, 110, iostat = ios ) cardim
         end if
110      format(a)

         if ( ios.lt.0 ) then

c        --- ios < 0,  input exhausted

            ityprd = 0
            ready  = .true.
            if ( ispecl.eq.6 .or. ispecl.eq.7 ) then

c           --- ispecl = 6 or 7, reading from the environment file

               write(*,111)
               write(*,112)
               write(*,113)
111            format(//' The sepran.env file is incorrect')
112            format(  ' end of file has been reached whereas more',
     +                  ' input is expected')
113            format(  ' Please check your environment file'//
     +                  ' Execution is terminated' )
               call isstop

            else if ( istop.eq.0 ) then

c           --- istop = 0, stop the process

               call iserrint ( irecnr, 1 )
               call iserrsub ( 1411, 1, 0, 0 )
               call isstop

            end if

         else if ( ios.gt.0 ) then

c        --- ios > 0,  error in reading record

            if ( ispecl.eq.6 .or. ispecl.eq.7 ) then

c           --- ispecl = 6 or 7, reading from the environment file

               write(*,111)
               write(*,114) ios
               write(*,113)
114            format(  ' A read error has occured',
     +                  ' The iostat value is',i5 )
               call isstop

            else

               call iserrint ( irecnr, 1 )
               call iserrint ( ios, 2 )
               call iserrsub ( 1423, 2, 0, 0 )
               call isstop

            end if

         else

c        --- Find last non-blank
c            First all tabs are translated in blanks

            do 115 j = 1, 80
               if ( cardim(j:j).eq.tab ) cardim(j:j) = ' '
115         continue
            last = 80
            do 120 j = last,1,-1
               lastp = j
               if ( cardim(j:j).ne.' ' ) goto 130
120         continue

c           --- Check for presence of comment character

130         last = lastp
            next = index ( cardim, '#' )
            if ( next.gt.0 ) then
               last = next - 1
               icolum = 1
               if ( last.eq.0 ) go to 180
            end if

            do 150 j = last,1,-1
               next = j
               if ( cardim(j:j).ne.' ' ) goto 160
150         continue

160         last = next

c           --- Find first non-blank

            do 170 j = 1,last
               icolum = j
               if ( cardim(j:j).ne.' ' ) goto 180
170         continue
            icolum = last + 1

180         if ( icolum.gt.last ) then

c           --- empty card found

               if ( ispecl.eq.1 .or. ispecl.eq.3 .or. ispecl.eq.7 ) then

c              --- Special situation:  ispecl = -1 at input
c                  Set ispecl = 2 and return

                  ispecl = 2
                  ready = .true.

               end if

            else if ( cardim(1:1).ne.'*' ) then

c           --- Not a comment record, nor a blank record
c               Translate letters from lower case to upper case
c               and put into cards
c               copy cardim into cards

               cards(icolum:last) = cardim(icolum:last)

               if ( ispecl.ne.3 .and. ispecl.ne.7 ) call
     +              isaskupc ( cards(icolum:last) )

c              --- Check on presence of stop in the case of ispecl = 1 at input

               if ( ispecl.eq.1 ) then

c              --- ispecl = 1 at input

                  if ( index(cards(icolum:last),'STOP').ne.0 )
     +                 call isstop

               end if

c              --- Check on presence of set records

               if ( ( ispecl.eq.0 .or. ispecl.ge.4 ) .and.
     +                index(cards(icolum:icolum+2),'SET').ne.0 ) then

c              --- Set record found, check remaining text

                  nextpr = icolum + 3
                  next = index(cards(nextpr:last),'WARN')
                  if ( next.gt.0 ) then

c                 --- WARN found, check rest of text

                     nextpr = next + nextpr + 3
                     next = index(cards(nextpr:last),'O')
                     if ( next.gt.0 ) then

c                     --- Letter O found, check rest of text

                         next = next + nextpr
                         if ( cards(next:next+1).eq.'FF' ) then

c                        --- Set warn off

                            nwarn = 11
                            go to 100

                         else if ( cards(next:next).eq.'N' ) then

c                        --- Set warn on

                            nwarn = 0
                            go to 100

                         end if

                     end if

                     nextpr = last

                  else

                     nextpr = icolum + 3

                  end if

                  next = index(cards(nextpr:last),'TIME')
                  if ( next.gt.0 ) then

c                 --- TIME found, check rest of text

                     nextpr = next + nextpr + 3
                     next = index(cards(nextpr:last),'O')
                     if ( next.gt.0 ) then

c                    --- Letter O found, check rest of text

                         next = next + nextpr
                         if ( cards(next:next+1).eq.'FF' ) then

c                        --Set time off

                            itime = 0
                            go to 100

                         else if ( cards(next:next).eq.'N' ) then

c                        --- Set time on

                            itime = 1
                            go to 100

                         end if

                     end if

                     nextpr = last

                  else

                     nextpr = icolum + 3

                  end if

                  next = index(cards(nextpr:last),'MAX')
                  if ( next.gt.0 ) then

c                 --- MAX found, check rest of text

                     nextpr = next + nextpr + 2
                     next = index(cards(nextpr:last),'WARN')
                     if ( next.gt.0 ) then

c                    --- Letters WARN found, check rest of text

                         next = next + nextpr + 2

c                        --- Find first digit after position next

200                      next = next + 1
                         if ( next.lt.last ) then
                            if ( cards(next:next).ge.'0' .and.
     +                           cards(next:next).le.'9') then

c                            --- number found, read with isreadnm

                               icolum = next
                               call isreadnm
                               maxwar = realrd
                               go to 100

                            end if
                            go to 200

                         end if

                     end if

                     nextpr = last

                  else

                     nextpr = icolum + 3

                  end if

                  next = index(cards(nextpr:last),'OUTPUT')
                  if ( next.gt.0 ) then

c                 --- OUTPUT found, check rest of text

                     nextpr = next + nextpr + 5
                     next = index(cards(nextpr:last),'LEV')
                     if ( next.gt.0 ) then

c                    --- LEVEL found, check rest of text

                        next = next + nextpr + 4

c                       --- Find first digit after position next

250                      next = next + 1
                         if ( next.le.last ) then
                            if ( cards(next:next).ge.'0' .and.
     +                           cards(next:next).le.'9') then

c                           --- number found, read with isreadnm

                               icolum = next
                               call isreadnm
                               ioutpt = realrd
                               go to 100

                            end if
                            go to 250

                         end if

                     else

                        next = index(cards(nextpr:last),'O')
                        if ( next.gt.0 ) then

c                       --- Letter O found, check rest of text

                            next = next + nextpr
                            if ( cards(next:next+1).eq.'FF' ) then

c                           --- Set output off

                               ioutpt = 0
                               go to 100

                            else if ( cards(next:next).eq.'N' ) then

c                           --- Set output on

                               ioutpt = 1
                               go to 100

                            else if ( cards(next:next+1).eq.'UT' ) then

c                           --- Set output out

                               ioutpt = -1
                               go to 100

                            end if

                        end if

                     end if

                  end if

               end if

c              --- Check on presence of texts
c                  Texts are all characters between quotes
c
c                All texts are put into cards in the original form

               ready = .true.
               next = icolum
400            jquost = index(cards(next:last),'''')
               if ( jquost.gt.0 ) then

c              --- ' found, check for presence of next quote

                  nextpr = jquost+next
                  jquost = nextpr
                  next = index(cards(nextpr:last),'''')
                  if ( next.gt.0 ) then

c                 --- end ' found

                     next = next + nextpr - 1
                     cards(jquost:next) = cardim(jquost:next)
                     next = next + 1
                     go to 400

                  end if

               end if

            end if

c           --- Move cards to end in order to decrease the checking of cards

            if ( last.lt.80 ) then

c           --- last < 80, move over 80-last positions

               do 500 j = last, icolum, -1
                  cards(80-last+j:80-last+j) = cards(j:j)
500            continue
               icolum = icolum + 80 - last

            end if
            if ( ioutpt.ge.0 ) write(irefwr,510) cardim(1:lastp)
510         format(1x,a)

         end if

         go to 100

      end if

      kcolum = icolum
      call iserclos ( 'isreadcd' )
      end
