      subroutine isreadnm
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Subroutine extracted from the SEPRAN
c                                        subroutine READNM version 1.3)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     read one number from the input file
c
c       This subroutine supposes that the parameter ICOLUM indicates the
c       first position of the number. This first position may be a
c       digit or one of the following signs:  . + -
c       The number is read free format
c       It is indicated if the number is real or integer
c
c        a number is defined by:
c         start:  + - . or digit
c         end:    non digit (except e or d after real), or end of record
c
c         in between digits . e d + and - are permitted
c
c       After reading the number, icolum points to the next position to be read
c       If the number is incorrect, realrd gets the value 0
c
c ********************************************************************
c
c                      KEYWORDS
c
c   read
c   read_cards
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
c
c     none
c
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the erroneous part
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      logical ready, lreal, lexp, lsign
      integer icount, mantis, ios
      character * 7 format

c    format   Character string to store the format necessary for reading of
c             the number with an internal read
c    icount   Counts the number of positions necessary for the number
c    ios      Iostatus variable
c    lexp     Indicates if the number contains an exponent (true) or not (false)
c    lreal    Indicates if the number is real (true) or integer (false)
c    lsign    Indicates if a sign has been read in the exponent (true) or not
c             (false)
c    mantis   Number of characters of the mantisse plus one
c    ready    Indicates if a while loop is finished or not
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c    1424   Error in reading of number
c ********************************************************************
c
c                    PSEUDO CODE
c
c   lreal := false
c   lexp  := false
c   ready := false
c   icount:= 0
c   kcolum:= icolum
c   if ( cards(icolum) = + or - ) then
c      icount := 1
c      string(1) := cards(icolum)
c      icolum := icolum+1
c
c   while ( not ready ) do
c      icount := icount+1
c      string(icount) := cards(icolum)
c      icolum := icolum+1
c      if ( icolum > 80 ) then
c         ready := true
c         icount := icount+1
c      else if ( lreal and string(icount) = '.' ) then
c         ready := true
c      else if ( string(icount) = '.' ) then
c         lreal := true
c      else if ( string(icount) = 'D', 'd', 'E' or 'e' ) then
c         ready := true
c         lexp := true
c      else if ( string(icount) not a digit ) then
c         ready := true
c         icolum := icolum - 1
c   mantis = icount
c
c   if ( lexp ) then
c      lsign := false
c      ready := false
c      while ( not ready ) do
c         if ( icolum > 80 ) then
c            ready := true
c            icount := icount+1
c         else
c            icount := icount+1
c            string(icount) := cards(icolum)
c            icolum := icolum+1
c            else if ( lsign and string(icount) = '+' or '-' ) then
c                  ready := true
c            else if ( string(icount) = '+', '-' or digit ) then
c               lsign := true
c            else (other character found) then
c               ready := true
c               icolum := icolum - 1
c      if ( string(icount-1) # digit ) then
c         icolum = icolum - icount + mantis
c         icount = mantis
c         lexp = false
c
c      if ( lexp or lreal ) then
c         ityprd := 3
c      else
c         ityprd := 2
c      translate number with internal read and store in realrd
c ======================================================================
c
      call iseropen ( 'isreadnm' )

c     --- Initialize some parameters

      lreal  = .false.
      lexp   = .false.
      ready  = .false.
      icount = 0
      kcolum = icolum
      if ( index('+-',cards(icolum:icolum)).gt.0 ) then

c     --- sign found in first position

         icount = icount + 1
         string(icount:icount) = cards(icolum:icolum)
         icolum = icolum + 1

      end if

100   if ( .not. ready ) then

c     --- First while loop for mantissa

         if ( icolum.gt.80 ) then

c        --- End of record reached
c            Raise icount, since only the positions 1 until icount-1 contain
c            the number to be interpreted

            ready = .true.
            icount = icount + 1

         else

            icount = icount + 1
            string(icount:icount) = cards(icolum:icolum)
            icolum = icolum + 1
            if ( lreal .and. string(icount:icount).eq.'.' ) then

c           --- Second decimal point found
c               This decimal point can not be part of the number, so reading is
c               stopped

               ready = .true.

            else if ( string(icount:icount).eq.'.' ) then

c           --- Decimal point found, so number is real

               lreal = .true.

            else if ( index('DEde',string(icount:icount)).gt.0 ) then

c           --- Letter E, D, e or d found, possibly floating point number

               lexp  = .true.
               ready = .true.

            else if ( string(icount:icount).lt.'0' .or.
     +                string(icount:icount).gt.'9' ) then

c           --- Next character is not a digit, end of number

               ready = .true.
               icolum = icolum - 1

            end if
            go to 100

         end if

      end if

c     --- End of first while loop

      mantis = icount
      if ( lexp ) then

c     --- Exponent sign found, read rest of number and check if number
c         contains an exponent

         lsign = .false.
         ready = .false.

200      if ( .not. ready ) then

c        --- Second while loop for exponent

            if ( icolum.gt.80 ) then

c           --- End of record reached
c               Raise icount, since only the positions 1 until icount-1 contain
c               the number to be interpreted

               ready = .true.
               icount = icount + 1

            else
               icount = icount + 1
               string(icount:icount) = cards(icolum:icolum)
               icolum = icolum + 1
               if ( lsign .and. index('+-',string(icount:icount))
     +              .gt.0 ) then

c              --- + or -, found not immediately after the exponent sign
c                  This sign can not be part of the number

                  ready = .true.

               else if ( string(icount:icount).ge.'0' .and.
     +                   string(icount:icount).le.'9' .or.
     +                   index('+-',string(icount:icount)).gt.0 ) then

c              --- Digit or + or - found, set lreal

                  lreal = .true.

               else

c              --- Other character found, i.e. end of number

                  ready = .true.
                  icolum = icolum - 1

               end if
               go to 200

            end if

         end if

c        --- End of second while loop
c            Check if exponent really exists

         if ( string(icount-1:icount-1).lt.'0' .or.
     +        string(icount-1:icount-1).gt.'9' ) then

c        --- Last position of number is not a digit, so exponent does not
c            exist
c            Reset parameters

            icolum = icolum - icount + mantis
            icount = mantis
            lexp = .false.

         end if

      end if

      if ( lexp .or. lreal ) then

c     --- Number is real

         ityprd = 3

      else

c     --- Number is integer

         ityprd = 2

      end if

c     --- Translate the number using internal write

      format(1:2) = '(g'
      if ( icount.le.10 ) then
         write ( format(3:3), fmt='(i1)' ) icount - 1
         format(4:7) = '.0 )'
      else
         write ( format(3:4), fmt='(i2)' ) icount - 1
         format(5:7) = '.0)'
      end if
      read ( string(1:icount-1), fmt=format, iostat = ios ) realrd

      if ( ios.ne.0 ) then

c     --- ios # 0, error during reading

         call iserrint ( ios, 1 )
         call iserrchr ( string(1:icount-1), 1 )
         call iserrsub ( 1424, 1, 0, 1 )
         call iserrcrd
         realrd = 0

      end if
      call iserclos ( 'isreadnm' )
      end
