      subroutine htim(n,amat,rhsd,usol,solutv,r,eps,imax,work,icontr,
     .                 infmat , prec, nprec,ndim,nconct, iinmtvc)
c ==================================================================
c
c
c     programmer       Kees Vuik
c     version 1.2      date 21-11-97 Kees K. iinmtvc added
c     version 1.1      date 03-10-95 Kees K. solutv added
c     version 1.0      date 07-08-1992
c     developed at     HP700
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     The subroutine htim is used as an innerloop of GMRESR.
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c     amat    i   Matrix from the equations to be solved.
c
c     eps     i   Determine the accuracy of the final approximation.
c
c     icontr  i   Integer array in which information about the solution
c                 process must be given by the user. Icontr(1) controls
c                 the preconditioning, and icontr(9) controls the method
c                 choosen in the innerloop. In this version only
c                 icontr(9) = 2 works. In this case the innerloop method
c                 is the preconditioned GMRES method.
c
c     iinmtvc i   reserved
c
c     imax    i   The maximum number of iterations to be performed.
c
c     infmat  i   Integer array with information of the matrix structure,`
c                 to be used in matrix-vector multiplication subroutine
c
c     n       i   The number of rows in the matrix A.
c
c     nconct  i   Maximal number of connections in one row of the
c                 matrix.
c
c     ndim    i   Integer indicating the amount of unknowns in every
c                 gridpoint. In the momentum equations ndim = 2 or 3,
c                 whereas in the pressure and transport equations
c                 ndim = 1.
c
c     nprec   i   Number of diagonals which are used in the precond-
c                 itioning.
c
c     prec    i   Array which contains the preconditioning matrix
c
c     r       o   After the iterations r = A*x
c
c     rhsd    i   Vector containing the right-hand side vector of the
c                 system of equations.
c
c     solutv  i   Work array for solut with virtual points
c
c     usol    o   Solution vector of length n. At output the array con-
c                 tains the last iterate, which is an approximation to the
c                 solution of the system.
c
c     work    o   Work array of size n * (imax+4).
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     method  Variable used in the choice of the innerloop.
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c     polgmr  Subroutine, which calculates an approximate solution
c             using the preconditioned GMRES method.
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine htim1 is used to solve an unsymmetric system of
c     linear equations of the shape A.x = f. Starting from an initial
c     vector, in each iteration a vector is sought which minimizes the
c     residual vector A.x-f in a certain Krylov-subspace. The process
c     is used as a polynomial preconditioner in combination with the
c     GMRESR method. This is a driver routine, so it only calls meth-
c     ods for the innerloop.
c
c ************************************************************************
c
c     Input:
c
c     The elements ICONTR(1),ICONTR(9), EPS,  ITMAX must have got
c     a value, and the arrays AMAT and RHSD  must have been filled by
c     the calling program.
c
c     Output:
c
c     The calculated solution vector is stored in USOL, and the
c     corresponding vector A*USOL in R.
c
c ************************************************************************
c
c ==================================================================
c
c     Parameters:
c
      integer          n,imax,icontr(9), infmat(*),nprec,ndim,nconct,
     +                 iinmtvc(*)
      double precision amat(n,*),rhsd(n),usol(n),r(n),eps,
     1                 work(n,*),prec(n,*),solutv(*)
c
c     Local parameters:
c
      integer method
      method = icontr(9)
      if(method.eq.2) then
         call polgmr(n,amat,rhsd,usol,solutv,r,eps,imax,work,icontr,
     .                 infmat , prec, nprec,ndim,nconct, iinmtvc)
      endif
      return
      end
