      subroutine isdmlu3 ( amat, nusol, ndimso, nconct, prec, nprec,
     +                     initer, reiter)
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.3    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.2    date 02-04-1997 Kees K. literal constants in
c                                     double precision for f90
c        version 1.1    date 23-02-1996 Kees V. default for IBLU changed
c        version 1.0    date 09-08-1995 extracted from old ISNaS
c                       subroutine dmlu3.
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Calculates an upper triangular matrix U and a lower triangular
c     matrix L, which form an incomplete decomposition of A.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*), nusol, ndimso, nconct, nprec
      double precision amat(1:nusol,1:ndimso,1:nconct),
     +                 prec(1:nusol,1:ndimso,1:nprec), reiter(*)

c     amat      i    The coefficients matrix used in the linear system
c     initer    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      o    The preconditioning matrix
c     reiter    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          i, nx
      double precision amod

c     amod      Parameter used in the average of ILU and MILU
c     i         General loop variable
c     nx        Number of grid x points on finest grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     This subroutine calculates an upper triangular matrix U and a lower
c     triangular matrix L, These matrices form an incomplete decomposition
c     for the matrix A using the following rules:
c
c            A = L U - R,
c
c     (a) diag(L) = I,
c
c     (b) The nonzero pattern of L and U are equal to the nonzero pattern
c         of A,
c
c     (c) If a(i,j) # 0 then L*U(i,j) = a(i,j)
c
c     The off-diagonal elements of U are stored in prec(1:n,1:ndimso,6:9),
c     whereas inverse(diag(U)) is stored in prec(1:n,1:ndimso,1).
c     The off-diagonal elements of L are stored in prec(1:n,1:ndimso,2:5),
c
c     Remarks
c
c     There are two difficulties to obtain the incomplete decomposition
c
c     - firstly, since the index of the matrix element can be less than
c       1 we discriminate 9 different situations:(nx = ni+1)
c              i = 1,
c              i = 2,...,nx-1
c              i = nx,
c              i = nx+1,
c              i = nx+2,...,n-nx-2,
c              i = n-nx-1
c              i = n-nx
c              i = n-nx+1,...,n-1
c              i = n
c
c     - secondly, it is known that in the virtual cells the corresponding
c       row and collumn in the matrix consists of zero elements. So it is
c       possible that d(i) = 0. To circumvent this possibility we imple-
c       ment the following check:
c          if(a(i,1).eq.0d0) then prec(i,1) = 1 else ...
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Filling of the parameter amod

      amod = 0d0
      if ( reiter(7).eq.-1d0 ) then
         if ( initer(11).eq.3 ) then
            if ( initer(21).eq.0 ) then
               amod = 1d0
            else
               amod = 0.975d0
            end if
         end if
         if ( initer(11).ge.4 ) amod = 1d0
         if ( initer(9).eq.6 )  amod = 0.9d0
         reiter(7) = amod
      else
         amod = reiter(7)
      end if

      nx = initer(12)+1

c     --- Compute the incomplete decomposition such that the part
c         prec(i,1,*) is filled
c         i = 1

      i = 1

c     --- The matrix U

      if ( amat(i,1,1).eq.0d0 ) then
         prec(i,1,1) = 1d0
      else
         prec(i,1,1) = amat(i,1,1)
         prec(i,1,1) = 1d0/prec(i,1,1)
      end if

      prec(i,1,6) = amat(i,1,6)
      prec(i,1,7) = amat(i,1,7)
      prec(i,1,8) = amat(i,1,8)
      prec(i,1,9) = amat(i,1,9)

c     --- The matrix L

      prec(i+1   ,1,5) = prec(i,1,1)*amat(i+1   ,1,5)
      prec(i+nx-1,1,4) = prec(i,1,1)*amat(i+nx-1,1,4)
      prec(i+nx  ,1,3) = prec(i,1,1)*amat(i+nx  ,1,3)
      prec(i+nx+1,1,2) = prec(i,1,1)*amat(i+nx+1,1,2)

c     --- i = 2, nx-1

      do i = 2, nx-1

c     --- The matrix U

         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -prec(i,1,5)*(prec(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,7))
            prec(i,1,1) = 1d0/prec(i,1,1)
         end if

         prec(i,1,6) = amat(i,1,6)
         prec(i,1,7) = amat(i,1,7)-prec(i,1,5)*prec(i-1,1,8)
         prec(i,1,8) = amat(i,1,8)-prec(i,1,5)*prec(i-1,1,9)
         prec(i,1,9) = amat(i,1,9)

c        --- The matrix L

         prec(i+1   ,1,5) = prec(i,1,1)*amat(i+1   ,1,5)
         prec(i+nx-1,1,4) = prec(i,1,1)*(amat(i+nx-1,1,4)
     +                      -prec(i+nx-1,1,3)*prec(i-1,1,6))
         prec(i+nx  ,1,3) = prec(i,1,1)*(amat(i+nx  ,1,3)
     +                      -prec(i+nx  ,1,2)*prec(i-1,1,6))
         prec(i+nx+1,1,2) = prec(i,1,1)*amat(i+nx+1,1,2)
      end do

c     --- i = nx

      i = nx

c     --- The matrix U

      if ( amat(i,1,1).eq.0d0 ) then
         prec(i,1,1) = 1d0
      else
         prec(i,1,1) = amat(i,1,1)
     +                 -prec(i,1,4)*(prec(i-nx+1,1,7)
     +                 +amod*prec(i-nx+1,1,6)+amod*prec(i-nx+1,1,9))
     +                 -prec(i,1,5)*(prec(i-1   ,1,6)
     +                 +amod*prec(i-1   ,1,7))
         prec(i,1,1) = 1d0/prec(i,1,1)
      end if

      prec(i,1,6) = amat(i,1,6)
     +              -prec(i,1,4)*prec(i-nx+1,1,8)
      prec(i,1,7) = amat(i,1,7)-prec(i,1,5)*prec(i-1,1,8)
      prec(i,1,8) = amat(i,1,8)-prec(i,1,5)*prec(i-1,1,9)
      prec(i,1,9) = amat(i,1,9)

c     --- The matrix L

      prec(i+1   ,1,5) = prec(i,1,1)*(amat(i+1   ,1,5)
     +                   -prec(i+1,1,3)*prec(i-nx+1,1,7))
      prec(i+nx-1,1,4) = prec(i,1,1)*(amat(i+nx-1,1,4)
     +                   -prec(i+nx-1,1,3)*prec(i-1,1,6))
      prec(i+nx  ,1,3) = prec(i,1,1)*(amat(i+nx  ,1,3)
     +                   -prec(i+nx  ,1,2)*prec(i-1,1,6))
      prec(i+nx+1,1,2) = prec(i,1,1)*amat(i+nx+1,1,2)

c     --- i = nx+1

      i = nx+1

c     --- The matrix U

      if ( amat(i,1,1).eq.0d0 ) then
         prec(i,1,1) = 1d0
      else
         prec(i,1,1) = amat(i,1,1)
     +                 -prec(i,1,3)*prec(i-nx  ,1,8)
     +                 -prec(i,1,4)*(prec(i-nx+1,1,7)
     +                 +amod*prec(i-nx+1,1,6)+amod*prec(i-nx+1,1,9))
     +                 -prec(i,1,5)*(prec(i-1   ,1,6)
     +                 +amod*prec(i-1   ,1,7))
         prec(i,1,1) = 1d0/prec(i,1,1)
      end if

      prec(i,1,6) = amat(i,1,6)
     +              -prec(i,1,3)*prec(i-nx  ,1,9)
     +              -prec(i,1,4)*prec(i-nx+1,1,8)
      prec(i,1,7) = amat(i,1,7)-prec(i,1,5)*prec(i-1,1,8)
      prec(i,1,8) = amat(i,1,8)-prec(i,1,5)*prec(i-1,1,9)
      prec(i,1,9) = amat(i,1,9)

c     --- The matrix L

      prec(i+1   ,1,5) = prec(i,1,1)*(amat(i+1   ,1,5)
     +                   -prec(i+1,1,2)*prec(i-nx  ,1,8)
     +                   -prec(i+1,1,3)*prec(i-nx+1,1,7))
      prec(i+nx-1,1,4) = prec(i,1,1)*(amat(i+nx-1,1,4)
     +                   -prec(i+nx-1,1,3)*prec(i-1,1,6))
      prec(i+nx  ,1,3) = prec(i,1,1)*(amat(i+nx  ,1,3)
     +                   -prec(i+nx  ,1,2)*prec(i-1,1,6))
      prec(i+nx+1,1,2) = prec(i,1,1)*amat(i+nx+1,1,2)

c     --- i = nx+2, n-nx-1

      do i = nx+2, nusol-nx-1

c     --- The matrix U

         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -prec(i,1,2)*(prec(i-nx-1,1,9)
     +                    +amod*prec(i-nx-1,1,7))
     +                    -prec(i,1,3)*prec(i-nx  ,1,8)
     +                    -prec(i,1,4)*(prec(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,6)+amod*prec(i-nx+1,1,9))
     +                    -prec(i,1,5)*(prec(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,7))
            prec(i,1,1) = 1d0/prec(i,1,1)
         end if

         prec(i,1,6) = amat(i,1,6)
     +                 -prec(i,1,3)*prec(i-nx  ,1,9)
     +                 -prec(i,1,4)*prec(i-nx+1,1,8)
         prec(i,1,7) = amat(i,1,7)-prec(i,1,5)*prec(i-1,1,8)
         prec(i,1,8) = amat(i,1,8)-prec(i,1,5)*prec(i-1,1,9)
         prec(i,1,9) = amat(i,1,9)

c        --- The matrix L

         prec(i+1   ,1,5) = prec(i,1,1)*(amat(i+1   ,1,5)
     +                      -prec(i+1,1,2)*prec(i-nx  ,1,8)
     +                      -prec(i+1,1,3)*prec(i-nx+1,1,7))
         prec(i+nx-1,1,4) = prec(i,1,1)*(amat(i+nx-1,1,4)
     +                      -prec(i+nx-1,1,3)*prec(i-1,1,6))
         prec(i+nx  ,1,3) = prec(i,1,1)*(amat(i+nx  ,1,3)
     +                      -prec(i+nx  ,1,2)*prec(i-1,1,6))
         prec(i+nx+1,1,2) = prec(i,1,1)*amat(i+nx+1,1,2)
      end do

c     --- i = n-nx

      i = nusol-nx

c     --- The matrix U

      if ( amat(i,1,1).eq.0d0 ) then
         prec(i,1,1) = 1d0
      else
         prec(i,1,1) = amat(i,1,1)
     +                 -prec(i,1,2)*(prec(i-nx-1,1,9)
     +                 +amod*prec(i-nx-1,1,7))
     +                 -prec(i,1,3)*prec(i-nx  ,1,8)
     +                 -prec(i,1,4)*(prec(i-nx+1,1,7)
     +                 +amod*prec(i-nx+1,1,6)+amod*prec(i-nx+1,1,9))
     +                 -prec(i,1,5)*(prec(i-1   ,1,6)
     +                 +amod*prec(i-1   ,1,7))
         prec(i,1,1) = 1d0/prec(i,1,1)
      end if

      prec(i,1,6) = amat(i,1,6)
     +              -prec(i,1,3)*prec(i-nx  ,1,9)
     +              -prec(i,1,4)*prec(i-nx+1,1,8)
      prec(i,1,7) = amat(i,1,7)-prec(i,1,5)*prec(i-1,1,8)
      prec(i,1,8) = amat(i,1,8)-prec(i,1,5)*prec(i-1,1,9)

c     --- The matrix L

      prec(i+1   ,1,5) = prec(i,1,1)*(amat(i+1   ,1,5)
     +                   -prec(i+1,1,2)*prec(i-nx  ,1,8)
     +                   -prec(i+1,1,3)*prec(i-nx+1,1,7))
      prec(i+nx-1,1,4) = prec(i,1,1)*(amat(i+nx-1,1,4)
     +                   -prec(i+nx-1,1,3)*prec(i-1,1,6))
      prec(i+nx  ,1,3) = prec(i,1,1)*(amat(i+nx  ,1,3)
     +                   -prec(i+nx  ,1,2)*prec(i-1,1,6))

c     --- i = n-nx+1

      i = nusol-nx+1

c     --- The matrix U

      if ( amat(i,1,1).eq.0d0 ) then
         prec(i,1,1) = 1d0
      else
         prec(i,1,1) = amat(i,1,1)
     +                 -prec(i,1,2)*(prec(i-nx-1,1,9)
     +                 +amod*prec(i-nx-1,1,7))
     +                 -prec(i,1,3)*prec(i-nx  ,1,8)
     +                 -prec(i,1,4)*(prec(i-nx+1,1,7)
     +                 +amod*prec(i-nx+1,1,6)+amod*prec(i-nx+1,1,9))
     +                 -prec(i,1,5)*(prec(i-1   ,1,6)
     +                 +amod*prec(i-1   ,1,7))
         prec(i,1,1) = 1d0/prec(i,1,1)
      end if

      prec(i,1,6) = amat(i,1,6)
     +              -prec(i,1,3)*prec(i-nx  ,1,9)
     +              -prec(i,1,4)*prec(i-nx+1,1,8)
      prec(i,1,7) = amat(i,1,7)-prec(i,1,5)*prec(i-1,1,8)

c     --- The matrix L

      prec(i+1   ,1,5) = prec(i,1,1)*(amat(i+1   ,1,5)
     +                   -prec(i+1,1,2)*prec(i-nx  ,1,8)
     +                   -prec(i+1,1,3)*prec(i-nx+1,1,7))
      prec(i+nx-1,1,4) = prec(i,1,1)*(amat(i+nx-1,1,4)
     +                   -prec(i+nx-1,1,3)*prec(i-1,1,6))

c     --- i = n-nx+2, n-1

      do i = nusol-nx+2, nusol-1

c     --- The matrix U

         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -prec(i,1,2)*(prec(i-nx-1,1,9)
     +                    +amod*prec(i-nx-1,1,7))
     +                    -prec(i,1,3)*prec(i-nx  ,1,8)
     +                    -prec(i,1,4)*(prec(i-nx+1,1,7)
     +                    +amod*prec(i-nx+1,1,6)+amod*prec(i-nx+1,1,9))
     +                    -prec(i,1,5)*(prec(i-1   ,1,6)
     +                    +amod*prec(i-1   ,1,7))
            prec(i,1,1) = 1d0/prec(i,1,1)
         end if

         prec(i,1,6) = amat(i,1,6)
     +                 -prec(i,1,3)*prec(i-nx  ,1,9)
     +                 -prec(i,1,4)*prec(i-nx+1,1,8)

c        --- The matrix L

         prec(i+1   ,1,5) = prec(i,1,1)*(amat(i+1   ,1,5)
     +                      -prec(i+1,1,2)*prec(i-nx  ,1,8)
     +                      -prec(i+1,1,3)*prec(i-nx+1,1,7))
      end do

c     --- i = n

      i = nusol

c     --- The matrix U

      if ( amat(i,1,1).eq.0d0 ) then
         prec(i,1,1) = 1d0
      else
         prec(i,1,1) = amat(i,1,1)
     +                 -prec(i,1,2)*(prec(i-nx-1,1,9)
     +                 +amod*prec(i-nx-1,1,7))
     +                 -prec(i,1,3)*prec(i-nx  ,1,8)
     +                 -prec(i,1,4)*(prec(i-nx+1,1,7)
     +                 +amod*prec(i-nx+1,1,6)+amod*prec(i-nx+1,1,9))
     +                 -prec(i,1,5)*(prec(i-1   ,1,6)
     +                 +amod*prec(i-1   ,1,7))
         prec(i,1,1) = 1d0/prec(i,1,1)
      end if

      end
