      subroutine isgsup(matrix, rhs, iinput, rinput,
     +       jac, pold, dpold, dpiter, infmat, ni, nj,
     +       nvirtual, ichoiceg, iter, densup, rhowork, drhodp, matdp,
     +       rhoint1, rhoint2, pinterm1 ,pinterm2, dpiterv, iinmtvc,
     +       nsubfaces, subfaces)
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      2.4    date 02-12-1999 Duncan: adaptation for periodic
c                                                 boundary conditions
c     version      2.3    date 13-06-1999 Duncan: extended for low Mach
c                                                 number
c     version      2.2    date 06-05-1999 Duncan: extended for periodical
c                                                 boundary conditions
c     version      2.1    date 23-03-1999 Duncan: inclusion boundary points
c     version      2.0    date 16-11-1998 Duncan: total revision
c     version      1.3    date 22-07-1998 Duncan:extension for density
c                                                   upwind j direction
c                                                 extension for density
c                                                   upwind i direction
c     version      1.2    date 08-07-1998 Kees V: nusol, nsolut cleaned
c     version      1.1    date 12-04-1998 Duncan:fixed bug division by
c                                         zero in matrix update
c     version      1.0    date 01-04-1998
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    subroutine isgsup updates the matrix and the right hand side of the
c    system for the GMRES step when ichoiceg = 1 else the earlier update
c    is removed again form the matrix .
c    For estimating the drhodp derivative, a secant type method is used.
c    WARNING: the mixed non-linear Gauss-Seidel method with GMRES
c    acceleration is only implemented for discretisation without density
c    bias
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iinput(*), ichoiceg, infmat(*), iter,
     +    iinmtvc(*), nsubfaces(*), subfaces(*)
      double precision jac(*), matrix((ni+1)*(nj+1),*), rinput(*),
     +                 pold(*), dpold(*), dpiter(*), rhs(*),
     +                 densup((ni+1)*(nj+1),3,*), rhowork(*),
     +                 drhodp((ni+1)*(nj+1)), matdp(*),
     +                 rhoint1(*), rhoint2(*),
     +                 pinterm1(*), pinterm2(*), dpiterv(*)

c     densup    i    coefficients of density upwind in pressure matrix
c     dpiter   i/o   work array for non linear Gauss-Seidel algorithm
c     dpiterv   i    work array for non linear Gauss-Seidel algorithm with
c                    virtual points
c     dpold     i    pressure correction at previous time step
c     ichoiceg  i    Choice parameter update matrix and rhside
c     iinmtvc   i    reserved
c     iinput    i    reserved
c     infmat    i    Integer information for the matrix
c     iter      i    The current iteration number.
c     jac       i    sqrt(g) in pnttyp
c     matrix   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c     nvirtual  i    Number of rows of virtual cells
c     pinterm1  i    Work array for pressure
c     pinterm2  i    Work array for pressure
c     pold      i    Pressure at the old time level
c     rhoint1   i    Work array for density and density increment
c     rhoint2   i    Work array for density
c     rhowork   i    Work array for density
c     rhs      i/o   Right hand side  (any level)
c     rinput    i    reserved
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iprincom, mcomupw, mscal, j, nusol,
     +         nk, k, istartup, iprincav, idummy(29), totshift(9),
     +         periodic, ipinfmat, nconct, iconct
      double precision epssc, dt, deltatin, mnm, updat, resto,
     +         delpold, drhol, drhor, drhob,drhot, pres1tr, pres2tr,
     +         dircc
      logical boundaryp

c     boundaryp Point is on the boundary
c     delpold   Most recent estimate for the total pressure correction
c     deltatin  Inverse of deltat, i.e. 1/deltat
c     dircc     Direction of increment for secant approximation
c     drhob     Increment in density at position 3 in pressure stencil
c     drhodp    drho/dp
c     drhol     Increment in density at position 5 in pressure stencil
c     drhor     Increment in density at position 6 in pressure stencil
c     drhot     Increment in density at position 8 in pressure stencil
c     dt        Time step
c     epssc     Parameter in the secant type estimate for the drhodp derivative
c     iconct    Loop variable for loop over all connections in the matrix
c     idummy    Integer dummy
c     ipinfmat  Starting address in array infmat
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     istartup  If 1 the cavity calculation is in the startup procedure,
c               if 0 not
c     j         General loop variable
c     k         General loop variable
c     matdp     Contribution to the rhs due to -DGdp
c     mcomupw   If zero the rho equation for compressible flow is solved by
c               a central scheme, otherwise by a upwind scheme
c     mnm       General help variable
c               a central scheme, otherwise by a upwind scheme
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     nconct    Number of connections in a row of the matrix
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     periodic  Periodic boundary conditions
c     pres1tr   Lower transit pressure in equation of state
c     pres2tr   Upper transit pressure in equation of state
c     resto     General help paremeter
c     totshift  total shift in the matrix
c     updat     General help parameter
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isstdls, isstdfl, isdstfll, isdstlss

c     DAXPY      Vector a*X + Y
c     DCOPY      Copy double precision array into other one (BLAS subroutine)
c     ISDSTFLL   Derivative of dimensional equation of state
c     ISDSTLSS   Derivative of dimensionless equation of state
c     ISGSPER    Correct matrix shifts for periodic boundary conditions
c     ISMATVEC   Computes the product of the sparse matrix, stored in amat,
c                and a vector.
c     ISSETCON   Make array equal to a constant
c     ISSTDFL    dimensionfull equation of state
c     ISSTDFL2   dimensionfull equation of state vector form
c     ISSTDLS    dimensionless equation of state
c     ISSTDLS2   dimensionless equation of state vector form
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c   In case of update do
c   for all non-virtual points do
c                                  __    drhodp
c   matrix(i,j,1)=matrix(i,j,1) + V g *  ------
c                                      dt
c   end do
c   for all non-virtual points do
c                                                    rho(p  )-rho(p )
c                                                         old      gauss
c   rhs(i,j) =   rhs(i,j)+ dt*DG ( p      - p   ) + ----------------------
c                                    gauss   old             dt
c   end do
c   in case of restore do
c   for all non-virtual points do
c                                  __    drhodp
c   matrix(i,j,1)=matrix(i,j,1) - V g *  ------
c                                      dt
c   end do
c   for all non-virtual points do
c                                                    rho(p  )-rho(p )
c                                                         old      gauss
c   rhs(i,j) =   rhs(i,j)- dt*DG ( p      - p   ) - ----------------------
c                                    gauss   old             dt
c   end do
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      updat    =  1d0
      resto    = -1d0

      if (ichoiceg.eq.1) then

          mnm  = updat

      else if (ichoiceg.eq.-1) then

          mnm  = resto

      end if

      dt         = rinput(iinput(29)-1 +4)
      deltatin   = 1d0/dt
      iprincom   = iinput(34)
      iprincav   = iinput(35)
      ipinfmat   = infmat(13)
      mcomupw    = iinput(iinput(9)+1)
      mscal      = iinput(iinput(9)+4)
      epssc      = 1d-6
      nk         = iinput(53)
      nusol      = (ni+1)*(nj+1)*(nk+1)
      istartup   = iinput(iinput(11)-1 +2)
      idummy(29) = 0
      pres1tr    = rinput(iprincav + 3 -1)
      pres2tr    = rinput(iprincav + 4 -1)
      periodic   = iinput(56)
      nconct     = infmat(6)

c     --- calculate intermediate pressure pold + dpiter = pinterm1 and
c                   intermediate pressure pold + dpold  = pinterm2

      call issetcon(drhodp,nusol,0d0)
      call dcopy(nusol,pold, 1, pinterm1, 1)
      call daxpy(nusol,1d0, dpiter, 1, pinterm1, 1)

      call dcopy(nusol,pold, 1, pinterm2, 1)
      call daxpy(nusol,1d0, dpold, 1, pinterm2, 1)

c     --- calculate the intermediate density rhoint1=rho(pold + dpiter)

      if( mscal.eq.0) then
         call isstdls2(pold, rinput(iprincom), istartup,
     +                       rhowork, nusol, rinput(iprincav))
         call isstdls2(pinterm1, rinput(iprincom), istartup,
     +                       rhoint1, nusol, rinput(iprincav))
         call isstdls2(pinterm2, rinput(iprincom), istartup,
     +                       rhoint2, nusol, rinput(iprincav))
      else if( mscal.eq.1) then
         call isstdfl2(pold, istartup, rhowork, nusol,
     +                                       rinput(iprincav))
         call isstdfl2(pinterm1, istartup, rhoint1, nusol,
     +                                       rinput(iprincav))
         call isstdfl2(pinterm2, istartup, rhoint2, nusol,
     +                                       rinput(iprincav))
      end if

      do k=1,2

        if((ichoiceg.eq.1.and.k.eq.1).or.
     +                  (ichoiceg.eq.-1.and.k.eq.2)) then

c       --- update the right hand side

c           --- calculate the increments in density and
c                             store in rhoint1 and rhoint2

          call daxpy(nusol,-1d0, rhowork, 1, rhoint1,1)
          call daxpy(nusol,-1d0, rhowork, 1, rhoint2,1)

c         --- in the linear part or low Mach number part change
c             delta rho = rho+ - rho- by delta rho = 1/c^2*delta p

         if( mscal.eq.0) then

            do j=1, nusol

               if((pold(j).lt.pres1tr.and.
     +                    pold(j)+dpiter(j).lt.pres1tr).or.
     +         (pold(j).gt.pres2tr.and.
     +                    pold(j)+dpiter(j).gt.pres2tr)) then
                  rhoint1(j)=
     +            isdstlss(pold(j),  rinput(iprincom),
     +                                  istartup, rinput(iprincav))*
     +                            dpiter(j)
               end if

            end do

         else if(mscal.eq.1) then

            do j=1, nusol

               if((pold(j).lt.pres1tr.and.
     +                   pold(j)+dpiter(j).lt.pres1tr).or.
     +          (pold(j).gt.pres2tr.and.
     +                   pold(j)+dpiter(j).gt.pres2tr)) then
                 rhoint1(j)=
     +           isdstfll(pold(j), istartup, rinput(iprincav))*
     +                           dpiter(j)
               end if

            end do

         end if

          call issetcon(dpiterv,
     +               (ni+1+2*nvirtual)*(nj+1+2*nvirtual),0d0)
          call ismatvec ( matrix, dpiter, dpiterv, matdp,
     +                      infmat, idummy, iinmtvc )

          do j = 1, (ni+1)*nj
            if( mod(j,ni+1).ne.0 ) then

            rhs(j) = rhs(j) + mnm *
     +                          ( - matdp(j)-rhoint1(j)
     +                                     *jac(j)*deltatin)
            end if
          end do

          if( mcomupw.eq.1) then

             do j=1, (ni+1)*nj
              if( mod(j,ni+1).ne.0 ) then

                do iconct = 2, nconct

                   totshift(iconct)=infmat(ipinfmat-1+iconct)

                end do

                boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                      (mod(j,ni+1).eq. 1) .or.
     +                      (         j .le.ni) .or.
     +                      (         j .gt.((ni+1)*(nj-1))))
                if( periodic.eq.1 .and. boundaryp) then

                   call isgsper(iinmtvc(1), iinmtvc(4),
     +                 iinmtvc(5), iinmtvc(10), iinmtvc(11),
     +                 iinmtvc(12), nsubfaces, subfaces,
     +                 totshift, j)
                end if

                drhol=rhoint1(j+totshift(5))
                drhor=rhoint1(j+totshift(6))
                drhob=rhoint1(j+totshift(3))
                drhot=rhoint1(j+totshift(8))
                rhs(j) = rhs(j) + mnm * (
     +            -rhoint1(j)*
     +                   (densup(j,2,1)+densup(j,2,2))
     +            - drhob*densup(j,1,2)
     +            - drhot*densup(j,3,2)
     +            - drhol*densup(j,1,1)
     +            - drhor*densup(j,3,1))

              end if
             end do
          end if

c         --- recalculate the intermediate density and
c                             store in rhoint1 and rhoint2

          call daxpy(nusol,1d0, rhowork, 1, rhoint1,1)
          call daxpy(nusol,1d0, rhowork, 1, rhoint2,1)

        end if

        if((ichoiceg.eq.1.and.k.eq.2).or.
     +                  (ichoiceg.eq.-1.and.k.eq.1)) then

          if (iter.gt.1) then

c         --- nondimensional computation

            if( mscal.eq.0) then

               do j=1, (ni+1)*nj
                if( mod(j,ni+1).ne.0 ) then

                    delpold=dpold(j)

                  if(abs(dpiter(j)-delpold).gt.
     +                      epssc*abs(pold(j)+delpold)) then

                    drhodp(j)=
     +                      (rhoint1(j)-rhoint2(j))/
     +                        (dpiter(j)-delpold)

                  else

                    drhodp(j)=
     +                      (isstdls((pold(j)+delpold)*(1+epssc),
     +                          rinput(iprincom), istartup,
     +                                           rinput(iprincav))-
     +                          rhoint2(j))/
     +                        ((pold(j)+delpold)*epssc)

                  end if
                end if
               end do

            else if( mscal.eq.1) then

               do j=1, (ni+1)*nj
                if( mod(j,ni+1).ne.0 ) then

                       delpold=dpold(j)

                     if(abs(dpiter(j)-delpold).gt.
     +                         epssc) then

                       drhodp(j)=
     +                         (rhoint1(j)-rhoint2(j))/
     +                           (dpiter(j)-delpold)
                     else

                       dircc    =sign(1d-6,dpiter(j)-delpold)
                       drhodp(j)=
     +                         (isstdfl((pold(j)+delpold)*(1+dircc),
     +                             istartup, rinput(iprincav))-
     +                             rhoint2(j))/
     +                           ((pold(j)+delpold)*dircc)

                     end if
                end if
               end do

c              --- linearty check : if pold, pold+ dpold and pold+dpiter all
c                   on same side of the phase transition in the linear part,
c                   then replace the secant approximation is replaced by
c                   the real derivative

               do j=1, (ni+1)*nj
                if( mod(j,ni+1).ne.0 ) then
                   if( ( (pold(j).gt.pres2tr) .and.
     +                    (pold(j)+dpold(j).gt.pres2tr) .and.
     +                     (pold(j)+dpiter(j).gt.pres2tr) ) ) then
                      drhodp(j)=
     +                            isdstfll(pold(j), istartup,
     +                                             rinput(iprincav))

                   end if

                end if
               end do

            end if

          else if (iter.eq.1) then

            if( mscal.eq.0) then

               do j = 1, nusol

                   if(abs(dpiter(j)).gt.
     +                 epssc*abs(pold(j))) then

                      drhodp(j)=
     +                          (rhoint1(j)-rhowork(j))/
     +                          (dpiter(j))

                   else
                      if( abs(pold(j)*epssc).gt.1d-12) then
                         drhodp(j)=
     +                          (isstdfl(pold(j)*(1+epssc),
     +                               istartup, rinput(iprincav))-
     +                           rhowork(j))/(pold(j)*epssc)
                      else
                         drhodp(j)=
     +                          (isdstfll(pold(j),
     +                               istartup, rinput(iprincav)))
                      end if
                   end if

               end do

             else if( mscal.eq.1) then

               do j = 1, nusol
                   if(abs(dpiter(j)).gt.
     +                 epssc*abs(pold(j))) then

                      drhodp(j)=
     +                          (rhoint1(j)-rhowork(j))/
     +                          (dpiter(j))

                   else

                      if( abs(pold(j)*epssc).gt.1d-12) then
                         drhodp(j)=
     +                          (isstdfl(pold(j)*(1+epssc),
     +                               istartup, rinput(iprincav))-
     +                           rhowork(j))/(pold(j)*epssc)
                      else
                         drhodp(j)=
     +                          (isdstfll(pold(j),
     +                               istartup, rinput(iprincav)))
                      end if

                   end if

               end do

c              --- linearty check : if pold, pold+ dpold and pold+dpiter all
c                   on same side of the phase transition in the linear part,
c                   then replace the secant approximation is replaced by
c                   the real derivative

               do j = 1, nusol
                   if( ( (pold(j).gt.pres2tr) .and.
     +                    (pold(j)+dpold(j).gt.pres2tr) .and.
     +                     (pold(j)+dpiter(j).gt.pres2tr) ) .or.
     +                 ( (pold(j).lt.pres1tr) .and.
     +                    (pold(j)+dpold(j).lt.pres1tr) .and.
     +                     (pold(j)+dpiter(j).lt.pres1tr) )) then
                      drhodp(j)=
     +                            isdstfll(pold(j), istartup,
     +                                             rinput(iprincav))

                   end if

               end do

             end if

          end if

c         --- Loop over all connections in the matrix

          do j=1,(ni+1)*nj
           if( mod(j,ni+1).ne.0) then
             matrix(j,1) = matrix(j,1)
     +                               +   mnm*
     +       drhodp(j)*jac(j)*deltatin
           end if
          end do

          if( mcomupw.eq.1) then

            do j=1,(ni+1)*nj
             if( mod(j,ni+1).ne.0) then

               matrix(j,1) =matrix(j,1)+mnm*
     +                   drhodp(j)*densup(j,2,1)
             end if
            end do

            do j=1,(ni+1)*nj
             if( mod(j,ni+1).ne.0) then

               matrix(j,1) =matrix(j,1)+mnm*
     +                   drhodp(j)*densup(j,2,2)

             end if
            end do

            do j=1,(ni+1)*nj
             if( mod(j,ni+1).ne.0) then

               do iconct = 2, nconct

                  totshift(iconct)=infmat(ipinfmat-1+iconct)

               end do

               boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                     (mod(j,ni+1).eq. 1) .or.
     +                     (         j .le.ni) .or.
     +                     (         j .gt.((ni+1)*(nj-1))))
               if( periodic.eq.1 .and. boundaryp) then

                  call isgsper(iinmtvc(1), iinmtvc(4),
     +                iinmtvc(5), iinmtvc(10), iinmtvc(11),
     +                iinmtvc(12), nsubfaces, subfaces,
     +                totshift, j)
               end if

               matrix(j,3) =matrix(j,3)+ mnm*
     +                   drhodp(j+totshift(3))*densup(j,1,2)
             end if
            end do

            do j=1,(ni+1)*nj
             if( mod(j,ni+1).ne.0) then

               do iconct = 2, nconct

                  totshift(iconct)=infmat(ipinfmat-1+iconct)

               end do

               boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                     (mod(j,ni+1).eq. 1) .or.
     +                     (         j .le.ni) .or.
     +                     (         j .gt.((ni+1)*(nj-1))))
               if( periodic.eq.1 .and. boundaryp) then

                  call isgsper(iinmtvc(1), iinmtvc(4),
     +                iinmtvc(5), iinmtvc(10), iinmtvc(11),
     +                iinmtvc(12), nsubfaces, subfaces,
     +                totshift, j)
               end if

               matrix(j,5) =matrix(j,5)+ mnm*
     +                   drhodp(j+totshift(5))*densup(j,1,1)
             end if
            end do

            do j=1,(ni+1)*nj
             if( mod(j,ni+1).ne.0) then

               do iconct = 2, nconct

                  totshift(iconct)=infmat(ipinfmat-1+iconct)

               end do

               boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                     (mod(j,ni+1).eq. 1) .or.
     +                     (         j .le.ni) .or.
     +                     (         j .gt.((ni+1)*(nj-1))))
               if( periodic.eq.1 .and. boundaryp) then

                  call isgsper(iinmtvc(1), iinmtvc(4),
     +                iinmtvc(5), iinmtvc(10), iinmtvc(11),
     +                iinmtvc(12), nsubfaces, subfaces,
     +                totshift, j)
               end if

               matrix(j,6) =matrix(j,6)+ mnm*
     +                   drhodp(j+totshift(6))*densup(j,3,1)
             end if
            end do

            do j=1,(ni+1)*nj
             if( mod(j,ni+1).ne.0) then

               do iconct = 2, nconct

                  totshift(iconct)=infmat(ipinfmat-1+iconct)

               end do

               boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                     (mod(j,ni+1).eq. 1) .or.
     +                     (         j .le.ni) .or.
     +                     (         j .gt.((ni+1)*(nj-1))))
               if( periodic.eq.1 .and. boundaryp) then

                  call isgsper(iinmtvc(1), iinmtvc(4),
     +                iinmtvc(5), iinmtvc(10), iinmtvc(11),
     +                iinmtvc(12), nsubfaces, subfaces,
     +                totshift, j)
               end if

               matrix(j,8) =matrix(j,8)+ mnm*
     +                   drhodp(j+totshift(8))*densup(j,3,2)
             end if
            end do

          end if

        end if

      end do

      end
