      subroutine ismtvc33 ( gmat, pressure, rhside, ni, nj, nk,
     +                      nvirtual, gmctinv )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   17-07-1996
c
c
c   copyright (c) 1996-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the of the matrix vector multiplication
c     f := f - G p in 3D
c     where p is the pressure including virtual points,
c           g is the gradient matrix (no virtual points) and
c           f is the right-hand side (no virtual points) to be updated
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, gmctinv(0:3,3,10)
      double precision gmat(ni+1,nj+1,nk+1,3,10),
     +                 rhside(ni+1,nj+1,nk+1,3),
     +                 pressure(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual,
     +                          1-nvirtual:nk+1+nvirtual)

c     gmat      i    Gradient matrix
c     gmctinv   i    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pressure  i    Array containing the pressure
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ishift, j, jshift, k, kshift, l

c     i         General loop variable
c     ishift    Shift in i-direction
c     j         General loop variable
c     jshift    Shift in j-direction
c     k         General loop variable
c     kshift    Shift in k-direction
c     l         General loop variable
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Since the matrix has a different structure for both degrees of freedom
c     we have to perform different loops
c
c     Algorithm:
c
c     For l := 1, 10 do
c        ishift := gmctinv(1,1,l)
c        jshift := gmctinv(2,1,l)
c        kshift := gmctinv(3,1,l)
c        For i := 1 (1) ni+1 do
c           For j := 1 (1) nj do
c              For k := 1 (1) nk do
c                 f(i,j,k,1) := f(i,j,k,1)+a(i,j,k,1,l)*
c                             x(i+ishift,j+jshift,k+kshift)
c
c     For l := 1, 10 do
c        ishift := gmctinv(1,2,l)
c        jshift := gmctinv(2,2,l)
c        kshift := gmctinv(3,2,l)
c        For i := 1 (1) ni do
c           For j := 1 (1) nj+1 do
c              For k := 1 (1) nk do
c                 f(i,j,k,2) := f(i,j,k,2)+a(i,j,k,2,l)*
c                             x(i+ishift,j+jshift,k+kshift)
c
c     For l := 1, 10 do
c        ishift := gmctinv(1,3,l)
c        jshift := gmctinv(2,3,l)
c        kshift := gmctinv(3,3,l)
c        For i := 1 (1) ni do
c           For j := 1 (1) nj do
c              For k := 1 (1) nk+1 do
c                 f(i,j,k,3) := f(i,j,k,3)+a(i,j,k,3,l)*
c                             x(i+ishift,j+jshift,k+kshift)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Loop over the connections

      do l = 1, 10

         ishift = gmctinv(1,1,l)
         jshift = gmctinv(2,1,l)
         kshift = gmctinv(3,1,l)

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni+1
                  rhside(i,j,k,1) = rhside(i,j,k,1) - gmat(i,j,k,1,l) *
     +                              pressure(i+ishift,j+jshift,k+kshift)
               end do
            end do
         end do

         ishift = gmctinv(1,2,l)
         jshift = gmctinv(2,2,l)
         kshift = gmctinv(3,2,l)

         do k = 1, nk
            do j = 1, nj+1
               do i = 1, ni
                  rhside(i,j,k,2) = rhside(i,j,k,2) - gmat(i,j,k,2,l) *
     +                              pressure(i+ishift,j+jshift,k+kshift)
               end do
            end do
         end do

         ishift = gmctinv(1,3,l)
         jshift = gmctinv(2,3,l)
         kshift = gmctinv(3,3,l)

         do k = 1, nk+1
            do j = 1, nj
               do i = 1, ni
                  rhside(i,j,k,3) = rhside(i,j,k,3) - gmat(i,j,k,3,l) *
     +                              pressure(i+ishift,j+jshift,k+kshift)
               end do
            end do
         end do

      end do

      end
