      subroutine isprechk ( initer, irrind, amat, nusol2 )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 2.0    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.9    date 22-09-1997 Kees V. changes for uncoupled
c        version 1.8    date 25-08-1997 Kees V. Multi-grid prec 3D added
c        version 1.7    date 17-04-1997 Kees V. Multi-grid prec added
c        version 1.6    date 23-01-1997 Kees V. CGS is allowed
c        version 1.5    date 16-10-1996 Kees V. Warning for some
c                                               molecules
c        version 1.4    date 27-06-1996 Kees V. Bi-CGSTAB is allowed
c        version 1.3    date 23-05-1996 Kees V. test if diagonal >= 0
c        version 1.2    date 18-03-1996 Kees V. adding ILU in 3D
c        version 1.1    date 05-07-1995 Kees V. adding ILU_Fill
c        version 1.0    date 05-07-1995
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   This subroutine checks if the requested preconditioner is implemented.
c   Furthermore the diagonal of the matrix should be non-negative.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*), irrind, nusol2
      double precision amat(*)

c     amat      i    The coefficients matrix used in the linear system
c     initer    i    reserved
c     irrind    i    Variable indicating possible failures.
c     nusol2    i    Number of scalar unknowns for the solvers
c                    nusol2 = nusol * ndimso
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          iprinlin, istop, method, mkind, ndim, j,
     +                 nconct, iperiod
      double precision diagmin
      logical          coupled

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     diagmin   Minimal value of the main diagonal of a matrix
c     iperiod   Parameter indicating if periodical boundary conditions
c               are present per boundary
c               iperiod is a 3 digit number with the sequence 321
c               Each digit refers to 1 direction
c               The digits have the following meaning
c               0:  no periodical boundary conditions
c               1:  symmetrical periodical boundary conditions
c               2:  anti-symmetrical periodical boundary conditions
c     iprinlin  Control parameter indicating the amount of output.
c     istop     Indicates which actions should be done in case of termination
c     j         General loop variable
c     method    Control parameter indicating the iterative method.
c     mkind     Control parameter indicating the preconditioner.
c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1003   this preconditioning is not allowed in this subroutine
c    1009   The matrix contains negative main diagonal elements
c    1011   ILUD preconditioner is replaced by ILUD postconditioner.
c **********************************************************************
c
c                       PSEUDO CODE
c
c   This subroutine checks if the requested preconditioner is
c   implemented.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isprechk' )

c     --- Initialization of parameters

      mkind    = initer(1)
      iprinlin = initer(2)
      istop    = initer(5)
      method   = initer(9)
      ndim     = initer(15)
      nconct   = initer(18)
      iperiod  = initer(23)
      coupled  = initer(29).eq.0

c     --- If periodic boundary conditions are used, or for the 2
c         dimensional coupled momentum equations the number of connections
c         is not equal to 13, the choice preconditioner = ILUD leads
c         to wrong results. Therefore a warning is given and the ILUD
c         postconditioner is used.

      if ( iperiod.gt.0 .and. mkind.eq.2 .or.
     +     ndim.eq.2 .and. initer(11).eq.1 .and. mkind.eq.2 .and.
     +     nconct.ne.13 .and. coupled ) then

         if ( iprinlin.ge.0 ) then
            call iswarsub ( 1011, 0, 0, 0 )
         end if
         initer(1) = -2
      end if

c     --- Preconditioning is allowed at this moment, abs(mkind)
c         should be <= 2 for the coupled momentum equations,
c                   <= 5 for the other equations.
c         Moreover, in 3D case abs(mkind) should be
c                   <= 2 for the coupled momentum equations, or
c                   = 3, 5 for uncoupled momentum equations,
c                          pressure or transport equations
c         Furthermore at this moment not all iteration methods
c         can be combined with all preconditioners.
c         Also the GMRESR method can not use every method in the
c         inner loop.
c         Finally for the transition from ISNaS_92 to ISNaS_94
c         only method = 2, 4, or 6 can be used.

      if ( initer(11).eq.1 .and. abs(mkind).gt.2 .and. coupled
     +    .or. abs(mkind).gt.5
     +    .or. ndim  .eq.3 .and. abs(mkind).eq.4
     +    .or. initer(8).ne.0 .and. method.ne.-1
     +    .or. method.ne.2 .and. method.ne.4 .and. method.ne.6
     +    .and. method.ne.1 ) then

         if ( iprinlin.ge.0 ) then
            if ( ndim.eq.2 ) call iserrint ( 2, 1 )
            if ( ndim.eq.3 ) call iserrint ( 3, 1 )
            if ( initer(11).eq.1 ) then
               if ( coupled ) then
                  call iserrchr ( 'coupled momentum', 1 )
               else
                  call iserrchr ( 'uncoupled momentum', 1 )
               end if
            end if
            if ( initer(11).eq.3 )
     +         call iserrchr ( 'pressure', 1 )
            if ( initer(11).eq.4 )
     +         call iserrchr ( 'transport', 1 )
            if ( initer(8).eq.0 ) then
               if ( method.eq.0 )
     +            call iserrchr ( 'LSQR', 2 )
               if ( method.eq.1 )
     +            call iserrchr ( 'CGS', 2 )
               if ( method.eq.2 )
     +            call iserrchr ( 'GMRES', 2 )
               if ( method.eq.3 )
     +            call iserrchr ( 'GCR', 2 )
               if ( method.eq.4 )
     +            call iserrchr ( 'Bi-CGSTAB', 2 )
               if ( method.eq.6 )
     +            call iserrchr ( 'one multiplication', 2 )
            else
               if ( method.eq.0 )
     +            call iserrchr (
     +              'GMRESR (with inner loop LSQR)', 2 )
               if ( method.eq.1 )
     +            call iserrchr (
     +              'GMRESR (with inner loop CGS)', 2 )
               if ( method.eq.2 )
     +            call iserrchr (
     +              'GMRESR (with inner loop GMRES)', 2 )
               if ( method.eq.3 )
     +            call iserrchr (
     +              'GMRESR (with inner loop GCR)', 2 )
               if ( method.eq.4 )
     +            call iserrchr (
     +              'GMRESR (with inner loop Bi-CGSTAB)', 2 )
               if ( method.eq.5 )
     +            call iserrchr (
     +              'GMRESR (with inner loop multigrid)', 2 )
               if ( method.eq.6 )
     +            call iserrchr (
     +              'GMRESR (with inner loop one multiplication)', 2 )
            end if
            if ( abs(mkind).eq.0 )
     +         call iserrchr ( 'no', 3 )
            if ( abs(mkind).eq.1 )
     +         call iserrchr ( 'a diagonal', 3 )
            if ( abs(mkind).eq.2 )
     +         call iserrchr ( 'an ILUD', 3 )
            if ( abs(mkind).eq.3 )
     +         call iserrchr ( 'an ILU', 3 )
            if ( abs(mkind).eq.4 )
     +         call iserrchr ( 'an ILU_Fill', 3 )
            if ( abs(mkind).eq.5 )
     +         call iserrchr ( 'a Multi-grid', 3 )
            if ( abs(mkind).eq.6 )
     +         call iserrchr ( 'a test', 3 )
            call iserrsub ( 1003, 1, 0, 3 )
         end if

         irrind = 1
         initer(3) = 1
         if ( istop.eq.0 ) then
            call isstop
         end if
      end if

c     --- The main diagonal of the matrix is checked

      diagmin = amat(1)

      do j = 2, nusol2
         if ( amat(j).lt.diagmin ) diagmin = amat(j)
      end do

      if ( diagmin.lt.0d0 ) then
         if ( iprinlin.ge.0 ) then
            if ( initer(11).eq.1 ) call iserrchr ( 'momentum', 1 )
            if ( initer(11).eq.3 ) call iserrchr ( 'pressure', 1 )
            if ( initer(11).eq.4 ) call iserrchr ( 'transport', 1 )
            call iswarsub ( 1009, 0, 0, 1 )
         end if
      end if

      call iserclos ( 'isprechk' )

      end
