      subroutine istinvl ( bresul, amat, nusol, ndimso, nconct,
     +                     infmat, initer )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.2    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.1    date 22-09-1997 Kees V. changes for uncoupled
c        version 1.0    date 20-04-1995 extracted from old ISNaS
c                       subroutine tinvl.
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by L preconditioning matrix given
c     in amat.
c     In this case we obtain:
c                 -1
c            b = L  x.
c     The array amat should be filled by subroutine istmlu.f This subroutine
c     contains compiler directives to run in vector speed on the
c     Convex.
c     Warning
c     -------
c     In this subroutine it is possible that shifts correctly point
c     into another degree of freedom, when the momentum equations
c     are solved.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          infmat(*),initer(*),nusol,ndimso,nconct
      double precision amat(1:nusol,1:ndimso,1:nconct),
     +                 bresul(nusol,ndimso)

c     amat      i    The coefficients matrix used in the linear system
c     bresul   i/o   The result vector
c     infmat    i    Integer information for the matrix
c     initer    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, icshift, icstart, j, k, m, ncube, nx, ny, npxy, nz
      logical coupled

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     i         General loop variable
c     icshift   Shift to compute address of column in the matrix
c     icstart   Starting address in array infmat for a specific matrix
c     j         General loop variable
c     k         General loop variable
c     m         General loop variable
c     ncube     Product of nx, ny, and nz
c     npxy      Product of nx and ny
c     nx        Number of grid x points on finest grid
c     ny        Number of grid y points on finest grid
c     nz        Number of points in the z-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      nx = initer(12)+1
      ny = initer(13)+1
      nz = initer(14)+1
      npxy = nx*ny
      coupled  = initer(29).eq.0

c     --- The multiplication b = L^(-1) x

      do i = 2, nx-1
         bresul(i,1) = bresul(i,1)
     +                 -amat(i,1,5)*bresul(i-1   ,1)
      end do

      i = nx

      bresul(i,1) = bresul(i,1)
     +              -amat(i,1,4)*bresul(i-nx+1,1)
     +              -amat(i,1,5)*bresul(i-1   ,1)

      i = nx+1

      bresul(i,1) = bresul(i,1)
     +              -amat(i,1,3)*bresul(i-nx  ,1)
     +              -amat(i,1,4)*bresul(i-nx+1,1)
     +              -amat(i,1,5)*bresul(i-1   ,1)

      do i = nx+2, npxy-nx

         bresul(i,1) = bresul(i,1)-amat(i,1,2)*bresul(i-nx-1,1)
     +                 -amat(i,1,3)*bresul(i-nx  ,1)
     +                 -amat(i,1,4)*bresul(i-nx+1,1)
     +                 -amat(i,1,5)*bresul(i-1   ,1)
      end do

      do i = npxy-nx+1, npxy-1

         bresul(i,1) = bresul(i,1)-amat(i,1,2)*bresul(i-nx-1,1)
     +                 -amat(i,1,3)*bresul(i-nx  ,1)
     +                 -amat(i,1,4)*bresul(i-nx+1,1)
     +                 -amat(i,1,5)*bresul(i-1   ,1)
     +                 -amat(i,1,14)*bresul(i-npxy+nx,1)
      end do

      i = npxy

      bresul(i,1) = bresul(i,1)-amat(i,1,2)*bresul(i-nx-1,1)
     +              -amat(i,1,3)*bresul(i-nx  ,1)
     +              -amat(i,1,4)*bresul(i-nx+1,1)
     +              -amat(i,1,5)*bresul(i-1   ,1)
     +              -amat(i,1,13)*bresul(i-npxy+1 ,1)
     +              -amat(i,1,14)*bresul(i-npxy+nx,1)

      i = npxy+1

      bresul(i,1) = bresul(i,1)-amat(i,1,2)*bresul(i-nx-1,1)
     +              -amat(i,1,3)*bresul(i-nx  ,1)
     +              -amat(i,1,4)*bresul(i-nx+1,1)
     +              -amat(i,1,5)*bresul(i-1   ,1)
     +              -amat(i,1,12)*bresul(i-npxy   ,1)
     +              -amat(i,1,13)*bresul(i-npxy+1 ,1)
     +              -amat(i,1,14)*bresul(i-npxy+nx,1)

      do i = npxy+2, npxy+nx

         bresul(i,1) = bresul(i,1)-amat(i,1,2)*bresul(i-nx-1,1)
     +                 -amat(i,1,3)*bresul(i-nx  ,1)
     +                 -amat(i,1,4)*bresul(i-nx+1,1)
     +                 -amat(i,1,5)*bresul(i-1   ,1)
     +                 -amat(i,1,11)*bresul(i-npxy-1 ,1)
     +                 -amat(i,1,12)*bresul(i-npxy   ,1)
     +                 -amat(i,1,13)*bresul(i-npxy+1 ,1)
     +                 -amat(i,1,14)*bresul(i-npxy+nx,1)
      end do

      do i = npxy+nx+1, 2*npxy

         bresul(i,1) = bresul(i,1)-amat(i,1,2)*bresul(i-nx-1,1)
     +                 -amat(i,1,3)*bresul(i-nx  ,1)
     +                 -amat(i,1,4)*bresul(i-nx+1,1)
     +                 -amat(i,1,5)*bresul(i-1   ,1)
     +                 -amat(i,1,10)*bresul(i-npxy-nx,1)
     +                 -amat(i,1,11)*bresul(i-npxy-1 ,1)
     +                 -amat(i,1,12)*bresul(i-npxy   ,1)
     +                 -amat(i,1,13)*bresul(i-npxy+1 ,1)
     +                 -amat(i,1,14)*bresul(i-npxy+nx,1)
      end do

      do k = 3, nz

c$dir no_recurrence

         do j = 1, npxy

            i = (k-1)*npxy+j

            bresul(i,1) = bresul(i,1)
     +                    -amat(i,1,10)*bresul(i-npxy-nx,1)
     +                    -amat(i,1,11)*bresul(i-npxy-1 ,1)
     +                    -amat(i,1,12)*bresul(i-npxy   ,1)
     +                    -amat(i,1,13)*bresul(i-npxy+1 ,1)
     +                    -amat(i,1,14)*bresul(i-npxy+nx,1)
         end do

         do j = 1, ny

c$dir no_recurrence

            do m = 1, nx

               i = (k-1)*npxy+(j-1)*nx+m
               bresul(i,1) = bresul(i,1)-amat(i,1,2)*bresul(i-nx-1,1)
     +                       -amat(i,1,3)*bresul(i-nx  ,1)
     +                       -amat(i,1,4)*bresul(i-nx+1,1)
            end do

            do m = 1, nx

               i = (k-1)*npxy+(j-1)*nx+m
               bresul(i,1) = bresul(i,1)-amat(i,1,5)*bresul(i-1,1)
            end do
         end do
      end do

      if ( initer(11).eq.1 .and. coupled ) then

c     --- If initer(11).eq.1 we solve the coupled momentum equations.
c         The lower triangular system is divided in the following blocks:
c         | L   : O   : O   | | b  |   | x  |
c         |  11 :     :     | |  1 |   |   1|
c         | ................| |....|   |....|
c         | L   : L   : O   | | b  |   | x  |
c         |  21 :  22 :     | |  2 | = |  2 |
c         | ................| |....|   |....|
c         | L   : L   : L   | | b  |   | x  |
c         |   31:  32 :  33 | |  3 |   |  3 |
c                                  -1
c         The multiplication b  = L   x  is already done in the part given
c                             1    11  1
c         above.
c         The computation b  = x  - L   b
c                          2    2    21  1
c         We divide the computation into two parts
c         firstly the parts where the shift may be less than 1,
c         secondly the part where the shift is always larger than 1.
c         The first part

      ncube   = npxy*nz
      icstart = infmat(13)+50

      do j = 36, 51

         icshift = infmat(icstart+j)+ncube

         do i = max(1,1-icshift), npxy+nx
            bresul(i,2) = bresul(i,2)-amat(i,2,j)*bresul(icshift+i,1)
         end do
      end do

c     --- The second part

      do i = npxy+nx+1, nusol
         bresul(i,2) = bresul(i,2)-amat(i,2,36)*bresul(i-npxy-nx  ,1)
     +                 -amat(i,2,37)*bresul(i-npxy-nx+1,1)
     +                 -amat(i,2,38)*bresul(i-npxy     ,1)
     +                 -amat(i,2,39)*bresul(i-npxy+1   ,1)
     +                 -amat(i,2,40)*bresul(i-nx-1     ,1)
     +                 -amat(i,2,41)*bresul(i-nx       ,1)
     +                 -amat(i,2,42)*bresul(i-nx+1     ,1)
     +                 -amat(i,2,43)*bresul(i-nx+2     ,1)
     +                 -amat(i,2,44)*bresul(i-1        ,1)
     +                 -amat(i,2,45)*bresul(i          ,1)
     +                 -amat(i,2,46)*bresul(i+1        ,1)
     +                 -amat(i,2,47)*bresul(i+2        ,1)
     +                 -amat(i,2,48)*bresul(i+npxy-nx  ,1)
     +                 -amat(i,2,49)*bresul(i+npxy-nx+1,1)
     +                 -amat(i,2,50)*bresul(i+npxy     ,1)
     +                 -amat(i,2,51)*bresul(i+npxy+1   ,1)
      end do

c     --- The computation b  = L  ^(-1) (x  - L   b )
c                          2    22        2    21  1

      do k = 1, nz

c$dir no_recurrence

         do j = 1, npxy

            i = (k-1)*npxy+j
            bresul(i,2) = bresul(i,2)
     +                    -amat(i,2,14)*bresul(i-npxy+nx,2)
     +                    -amat(i,2,13)*bresul(i-npxy+1 ,2)
     +                    -amat(i,2,12)*bresul(i-npxy   ,2)
     +                    -amat(i,2,11)*bresul(i-npxy-1 ,2)
     +                    -amat(i,2,10)*bresul(i-npxy-nx,2)
         end do

         do j = 1, ny

c$dir no_recurrence

            do m = 1, nx

               i = (k-1)*npxy+(j-1)*nx+m
               bresul(i,2) = bresul(i,2)
     +                       -amat(i,2,4)*bresul(i-nx+1,2)
     +                       -amat(i,2,3)*bresul(i-nx  ,2)
     +                       -amat(i,2,2)*bresul(i-nx-1,2)
            end do

            do m = 1, nx

               i = (k-1)*npxy+(j-1)*nx+m
               bresul(i,2) = bresul(i,2)-amat(i,2,5)*bresul(i-1,2)
            end do
         end do
      end do

c     --- The computation b  = x  - L   b
c                          3    3    31  1
c         We divide the computation into two parts
c         firstly the parts where the shift may be less than 1,
c         secondly the part where the shift is always larger than 1.
c         The first part

      icstart = icstart+51

      do j = 20, 35

         icshift = infmat(icstart+j)+2*ncube
         do i = max(1,1-icshift), npxy+nx
            bresul(i,3) = bresul(i,3)-amat(i,3,j)*bresul(icshift+i,1)
         end do
      end do

c     --- The second part

      do i = npxy+nx+1, nusol
         bresul(i,3) = bresul(i,3)-amat(i,3,20)*bresul(i-npxy-nx  ,1)
     +                 -amat(i,3,21)*bresul(i-npxy-nx+1,1)
     +                 -amat(i,3,22)*bresul(i-npxy-1   ,1)
     +                 -amat(i,3,23)*bresul(i-npxy     ,1)
     +                 -amat(i,3,24)*bresul(i-npxy+1   ,1)
     +                 -amat(i,3,25)*bresul(i-npxy+2   ,1)
     +                 -amat(i,3,26)*bresul(i-npxy+nx  ,1)
     +                 -amat(i,3,27)*bresul(i-npxy+nx+1,1)
     +                 -amat(i,3,28)*bresul(i-nx       ,1)
     +                 -amat(i,3,29)*bresul(i-nx+1     ,1)
     +                 -amat(i,3,30)*bresul(i-1        ,1)
     +                 -amat(i,3,31)*bresul(i          ,1)
     +                 -amat(i,3,32)*bresul(i+1        ,1)
     +                 -amat(i,3,33)*bresul(i+2        ,1)
     +                 -amat(i,3,34)*bresul(i+nx       ,1)
     +                 -amat(i,3,35)*bresul(i+nx+1     ,1)
      end do

c     --- The computation b  = x  - L   b - L   b
c                          3    3    31  1   32  2

      do i = 1, nusol
         bresul(i,3) = bresul(i,3)-amat(i,3,36)*bresul(i-npxy-nx  ,2)
     +                 -amat(i,3,37)*bresul(i-npxy-1   ,2)
     +                 -amat(i,3,38)*bresul(i-npxy     ,2)
     +                 -amat(i,3,39)*bresul(i-npxy+1   ,2)
     +                 -amat(i,3,40)*bresul(i-npxy+nx-1,2)
     +                 -amat(i,3,41)*bresul(i-npxy+nx  ,2)
     +                 -amat(i,3,42)*bresul(i-npxy+nx+1,2)
     +                 -amat(i,3,43)*bresul(i-npxy+2*nx,2)
     +                 -amat(i,3,44)*bresul(i-nx       ,2)
     +                 -amat(i,3,45)*bresul(i-1        ,2)
     +                 -amat(i,3,46)*bresul(i          ,2)
     +                 -amat(i,3,47)*bresul(i+1        ,2)
     +                 -amat(i,3,48)*bresul(i+nx-1     ,2)
     +                 -amat(i,3,49)*bresul(i+nx       ,2)
     +                 -amat(i,3,50)*bresul(i+nx+1     ,2)
     +                 -amat(i,3,51)*bresul(i+2*nx     ,2)
      end do

c     --- The computation b  = L  ^(-1) (x  - L   b - L   b )
c                          3    33        3    31  1   32  2

      do k = 1, nz

c$dir no_recurrence

         do j = 1, npxy

            i = (k-1)*npxy+j
            bresul(i,3) = bresul(i,3)
     +                    -amat(i,3,14)*bresul(i-npxy+nx,3)
     +                    -amat(i,3,13)*bresul(i-npxy+1 ,3)
     +                    -amat(i,3,12)*bresul(i-npxy   ,3)
     +                    -amat(i,3,11)*bresul(i-npxy-1 ,3)
     +                    -amat(i,3,10)*bresul(i-npxy-nx,3)
         end do

         do j = 1, ny

c$dir no_recurrence

            do m = 1, nx

               i = (k-1)*npxy+(j-1)*nx+m
               bresul(i,3) = bresul(i,3)
     +                       -amat(i,3,4)*bresul(i-nx+1,3)
     +                       -amat(i,3,3)*bresul(i-nx  ,3)
     +                       -amat(i,3,2)*bresul(i-nx-1,3)
            end do

            do m = 1, nx

               i = (k-1)*npxy+(j-1)*nx+m
               bresul(i,3) =  bresul(i,3)-amat(i,3,5)*bresul(i-1   ,3)
            end do
         end do
      end do
      end if
      end
