      subroutine tinvl(x,b,amat,n,ndimso,nconct,
     1                  prec,nprec,infmat)
c ==================================================================
c
c
c     programmer       Kees Vuik
c     version 1.2      date 15-03-1993 C. Vuik Vectorized on the
c                                      Convex
c     version 1.1      date 22-02-1993 C. Vuik Preconditioning of the
c                                      momentum equations.
c     version 1.0      date 20-11-1992
c     developed at     HP700
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by L preconditioning matrix given
c     in amat and prec.
c
c     In this case we obtain:
c                 -1
c            b = L  x.
c
c     The arrays prec and amat should be filled by subroutine tmlu.f
c
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     ilud
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c     amat    i  the coefficient matrix for the momentum or
c                an equation similar to the pressure equation.
c
c     b       o  the result vector, which contains:
c                     -1
c                b = L  x.
c
c     infmat  i  if infmat(8) is 1 we use the momentum equations,
c                whereas if infmat(8) is larger than or equal to 3
c                we use equations, with a structure similar to the
c                pressure equations.
c
c     n       i  number of unknowns in the solution vector.
c
c     nconct  i  number of connections in one row of the matrix.
c
c     ndimso  i  integer indicating the dimension of the space in
c                which the problem must be solved (ndimso = 1 or ndim).
c
c     nprec   i  number of diagonals, which are used in the pre-
c                conditioning. In this subroutine nprec = 1.
c
c     prec    i  the preconditioning matrix.
c
c     x       i  the original vector.
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     i       loop counter.
c
c     ishift  shift to compute address of column in the matrix.
c
c     istart  starting address in array infmat for a specific matrix.
c
c     j       loop counter.
c
c     kz      loop counter.
c
c     m       loop counter.
c
c     ncube   product of nx, ny, and nz.
c
c     npxy    product of nx and ny.
c
c     nx      number of points in the x-direction.
c
c     ny      number of points in the y-direction.
c
c     nz      number of points in the z-direction.
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     Multiplication of x by L preconditioning matrix given
c     in amat and prec.
c
c     In this case we obtain:
c                 -1
c            b = L  x.
c
c     The arrays prec and amat should be filled by subroutine tmlu.f
c
c ==================================================================
c
c     Parameters
c
      integer          infmat(*),n,ndimso,nconct,nprec
      double precision amat(1:n,1:ndimso,1:nconct),
     1                 prec(1:n,1:ndimso,1:nprec),
     2                 x(n,ndimso),b(n,ndimso)
c
c     local parameters
c
      integer i,ishift,istart,j,kz,m,ncube,nx,ny,npxy,nz
      nx = infmat(2)+1
      ny = infmat(3)+1
      nz = infmat(4)+1
      npxy = nx*ny
c                             -1
c     The multiplication b = L  x
c
      b(1,1) = x(1,1)
      do i = 2, nx-1
         b(i,1) = x(i,1)
     .                   -amat(i,1,5)*b(i-1   ,1)
      enddo
      i = nx
      b(i,1) = x(i,1)
     .                -amat(i,1,4)*b(i-nx+1,1)
     .                -amat(i,1,5)*b(i-1   ,1)
      i = nx+1
      b(i,1) = x(i,1)
     .                -amat(i,1,3)*b(i-nx  ,1)
     .                -amat(i,1,4)*b(i-nx+1,1)
     .                -amat(i,1,5)*b(i-1   ,1)
      do i = nx+2, npxy-nx
         b(i,1) =  x(i,1)-amat(i,1,2)*b(i-nx-1,1)
     .                   -amat(i,1,3)*b(i-nx  ,1)
     .                   -amat(i,1,4)*b(i-nx+1,1)
     .                   -amat(i,1,5)*b(i-1   ,1)
      enddo
      do i = npxy-nx+1, npxy-1
         b(i,1) =  x(i,1)-amat(i,1,2)*b(i-nx-1,1)
     .                   -amat(i,1,3)*b(i-nx  ,1)
     .                   -amat(i,1,4)*b(i-nx+1,1)
     .                   -amat(i,1,5)*b(i-1   ,1)
     .                   -amat(i,1,14)*b(i-npxy+nx,1)
      enddo
      i = npxy
      b(i,1) =  x(i,1)-amat(i,1,2)*b(i-nx-1,1)
     .                -amat(i,1,3)*b(i-nx  ,1)
     .                -amat(i,1,4)*b(i-nx+1,1)
     .                -amat(i,1,5)*b(i-1   ,1)
     .                -amat(i,1,13)*b(i-npxy+1 ,1)
     .                -amat(i,1,14)*b(i-npxy+nx,1)
      i = npxy+1
      b(i,1) =  x(i,1)-amat(i,1,2)*b(i-nx-1,1)
     .                -amat(i,1,3)*b(i-nx  ,1)
     .                -amat(i,1,4)*b(i-nx+1,1)
     .                -amat(i,1,5)*b(i-1   ,1)
     .                -amat(i,1,12)*b(i-npxy   ,1)
     .                -amat(i,1,13)*b(i-npxy+1 ,1)
     .                -amat(i,1,14)*b(i-npxy+nx,1)
      do i = npxy+2, npxy+nx
         b(i,1) =  x(i,1)-amat(i,1,2)*b(i-nx-1,1)
     .                   -amat(i,1,3)*b(i-nx  ,1)
     .                   -amat(i,1,4)*b(i-nx+1,1)
     .                   -amat(i,1,5)*b(i-1   ,1)
     .                   -amat(i,1,11)*b(i-npxy-1 ,1)
     .                   -amat(i,1,12)*b(i-npxy   ,1)
     .                   -amat(i,1,13)*b(i-npxy+1 ,1)
     .                   -amat(i,1,14)*b(i-npxy+nx,1)
      enddo
      do i = npxy+nx+1, 2*npxy
         b(i,1) =  x(i,1)-amat(i,1,2)*b(i-nx-1,1)
     .                   -amat(i,1,3)*b(i-nx  ,1)
     .                   -amat(i,1,4)*b(i-nx+1,1)
     .                   -amat(i,1,5)*b(i-1   ,1)
     .                   -amat(i,1,10)*b(i-npxy-nx,1)
     .                   -amat(i,1,11)*b(i-npxy-1 ,1)
     .                   -amat(i,1,12)*b(i-npxy   ,1)
     .                   -amat(i,1,13)*b(i-npxy+1 ,1)
     .                   -amat(i,1,14)*b(i-npxy+nx,1)
      enddo
      do kz = 3, nz
c$dir no_recurrence
	 do j = 1, npxy
	 i = (kz-1)*npxy+j
	 b(i,1) = x(i,1)
     .                   -amat(i,1,10)*b(i-npxy-nx,1)
     .                   -amat(i,1,11)*b(i-npxy-1 ,1)
     .                   -amat(i,1,12)*b(i-npxy   ,1)
     .                   -amat(i,1,13)*b(i-npxy+1 ,1)
     .                   -amat(i,1,14)*b(i-npxy+nx,1)
	 enddo
	 do j = 1, ny
c$dir no_recurrence
	 do m = 1, nx
	 i = (kz-1)*npxy+(j-1)*nx+m
         b(i,1) =  b(i,1)-amat(i,1,2)*b(i-nx-1,1)
     .                   -amat(i,1,3)*b(i-nx  ,1)
     .                   -amat(i,1,4)*b(i-nx+1,1)
	 enddo
	 do m = 1, nx
	 i = (kz-1)*npxy+(j-1)*nx+m
         b(i,1) =  b(i,1)-amat(i,1,5)*b(i-1,1)
	 enddo
	 enddo
      enddo
      if(infmat(8).eq.1) then
c
c     If infmat(8).eq.1 we solve the momentum equations. The lower
c     triangular system is divided in the following blocks:
c
c        | L   : O   : O   | | b  |   | x  |
c        |  11 :     :     | |  1 |   |   1|
c        | ................| |....|   |....|
c        | L   : L   : O   | | b  |   | x  |
c        |  21 :  22 :     | |  2 | = |  2 |
c        | ................| |....|   |....|
c        | L   : L   : L   | | b  |   | x  |
c        |   31:  32 :  33 | |  3 |   |  3 |
c
c
c                              -1
c     The multiplication b  = L   x  is already done in the part given
c                         1    11  1
c     above.
c
c     The computation b  = x  - L   b
c                      2    2    21  1
c
c     We divide the computation into two parts
c
c     firstly the parts where the shift may be less than 1,
c
c     secondly the part where the shift is always larger than 1.
c
c
c     The first part
c
      ncube  = npxy*nz
      istart = infmat(13)+50
      do i = 1, npxy+nx
         b(i,2) = x(i,2)
      enddo
      do j = 36, 51
         ishift = infmat(istart+j)+ncube
         do i = max(1,1-ishift), npxy+nx
            b(i,2) = b(i,2)-amat(i,2,j)*b(ishift+i,1)
         enddo
      enddo
c
c     The second part
c
      do i = npxy+nx+1, n
         b(i,2) = x(i,2)-amat(i,2,36)*b(i-npxy-nx  ,1)
     .                  -amat(i,2,37)*b(i-npxy-nx+1,1)
     .                  -amat(i,2,38)*b(i-npxy     ,1)
     .                  -amat(i,2,39)*b(i-npxy+1   ,1)
     .                  -amat(i,2,40)*b(i-nx-1     ,1)
     .                  -amat(i,2,41)*b(i-nx       ,1)
     .                  -amat(i,2,42)*b(i-nx+1     ,1)
     .                  -amat(i,2,43)*b(i-nx+2     ,1)
     .                  -amat(i,2,44)*b(i-1        ,1)
     .                  -amat(i,2,45)*b(i          ,1)
     .                  -amat(i,2,46)*b(i+1        ,1)
     .                  -amat(i,2,47)*b(i+2        ,1)
     .                  -amat(i,2,48)*b(i+npxy-nx  ,1)
     .                  -amat(i,2,49)*b(i+npxy-nx+1,1)
     .                  -amat(i,2,50)*b(i+npxy     ,1)
     .                  -amat(i,2,51)*b(i+npxy+1   ,1)
      enddo
c                           -1
c     The computation b  = L  (x  - L   b )
c                      2    22  2    21  1
c
      do kz = 1, nz
c$dir no_recurrence
	 do j = 1, npxy
	 i = (kz-1)*npxy+j
         b(i,2) =  b(i,2)
     .                   -amat(i,2,14)*b(i-npxy+nx,2)
     .                   -amat(i,2,13)*b(i-npxy+1 ,2)
     .                   -amat(i,2,12)*b(i-npxy   ,2)
     .                   -amat(i,2,11)*b(i-npxy-1 ,2)
     .                   -amat(i,2,10)*b(i-npxy-nx,2)
	 enddo
	 do j = 1, ny
c$dir no_recurrence
	 do m = 1, nx
	 i = (kz-1)*npxy+(j-1)*nx+m
         b(i,2) =  b(i,2)
     .                   -amat(i,2,4) *b(i-nx+1,2)
     .                   -amat(i,2,3) *b(i-nx  ,2)
     .                   -amat(i,2,2) *b(i-nx-1,2)
	 enddo
	 do m = 1, nx
	 i = (kz-1)*npxy+(j-1)*nx+m
         b(i,2) =  b(i,2)-amat(i,2,5)*b(i-1,2)
	 enddo
	 enddo
      enddo
c
c     The computation b  = x  - L   b
c                      3    3    31  1
c
c     We divide the computation into two parts
c
c     firstly the parts where the shift may be less than 1,
c
c     secondly the part where the shift is always larger than 1.
c
c
c     The first part
c
      istart = istart+51
      do i = 1, npxy+nx
         b(i,3) = x(i,3)
      enddo
      do j = 20, 35
         ishift = infmat(istart+j)+2*ncube
         do i = max(1,1-ishift), npxy+nx
            b(i,3) = b(i,3)-amat(i,3,j)*b(ishift+i,1)
         enddo
      enddo
c
c     The second part
c
      do i = npxy+nx+1, n
         b(i,3) = x(i,3)-amat(i,3,20)*b(i-npxy-nx  ,1)
     .                  -amat(i,3,21)*b(i-npxy-nx+1,1)
     .                  -amat(i,3,22)*b(i-npxy-1   ,1)
     .                  -amat(i,3,23)*b(i-npxy     ,1)
     .                  -amat(i,3,24)*b(i-npxy+1   ,1)
     .                  -amat(i,3,25)*b(i-npxy+2   ,1)
     .                  -amat(i,3,26)*b(i-npxy+nx  ,1)
     .                  -amat(i,3,27)*b(i-npxy+nx+1,1)
     .                  -amat(i,3,28)*b(i-nx       ,1)
     .                  -amat(i,3,29)*b(i-nx+1     ,1)
     .                  -amat(i,3,30)*b(i-1        ,1)
     .                  -amat(i,3,31)*b(i          ,1)
     .                  -amat(i,3,32)*b(i+1        ,1)
     .                  -amat(i,3,33)*b(i+2        ,1)
     .                  -amat(i,3,34)*b(i+nx       ,1)
     .                  -amat(i,3,35)*b(i+nx+1     ,1)
      enddo
c
c     The computation b  = x  - L   b - L   b
c                      3    3    31  1   32  2
c
      do i = 1, n
         b(i,3) = b(i,3)-amat(i,3,36)*b(i-npxy-nx  ,2)
     .                  -amat(i,3,37)*b(i-npxy-1   ,2)
     .                  -amat(i,3,38)*b(i-npxy     ,2)
     .                  -amat(i,3,39)*b(i-npxy+1   ,2)
     .                  -amat(i,3,40)*b(i-npxy+nx-1,2)
     .                  -amat(i,3,41)*b(i-npxy+nx  ,2)
     .                  -amat(i,3,42)*b(i-npxy+nx+1,2)
     .                  -amat(i,3,43)*b(i-npxy+2*nx,2)
     .                  -amat(i,3,44)*b(i-nx       ,2)
     .                  -amat(i,3,45)*b(i-1        ,2)
     .                  -amat(i,3,46)*b(i          ,2)
     .                  -amat(i,3,47)*b(i+1        ,2)
     .                  -amat(i,3,48)*b(i+nx-1     ,2)
     .                  -amat(i,3,49)*b(i+nx       ,2)
     .                  -amat(i,3,50)*b(i+nx+1     ,2)
     .                  -amat(i,3,51)*b(i+2*nx     ,2)
      enddo
c                           -1
c     The computation b  = L  (x  - L   b - L   b )
c                      3    33  3    31  1   32  2
c
      do kz = 1, nz
c$dir no_recurrence
	 do j = 1, npxy
	 i = (kz-1)*npxy+j
         b(i,3) =  b(i,3)
     .                   -amat(i,3,14)*b(i-npxy+nx,3)
     .                   -amat(i,3,13)*b(i-npxy+1 ,3)
     .                   -amat(i,3,12)*b(i-npxy   ,3)
     .                   -amat(i,3,11)*b(i-npxy-1 ,3)
     .                   -amat(i,3,10)*b(i-npxy-nx,3)
	 enddo
	 do j = 1, ny
c$dir no_recurrence
         do m = 1, nx
	 i = (kz-1)*npxy+(j-1)*nx+m
         b(i,3) =  b(i,3)
     .                   -amat(i,3,4) *b(i-nx+1,3)
     .                   -amat(i,3,3) *b(i-nx  ,3)
     .                   -amat(i,3,2) *b(i-nx-1,3)
	 enddo
         do m = 1, nx
	 i = (kz-1)*npxy+(j-1)*nx+m
         b(i,3) =  b(i,3)-amat(i,3,5) *b(i-1   ,3)
	 enddo
	 enddo
      enddo
      endif
      end
