      function dnrm2(n, dx, incx)
      implicit double precision (a-h,o-z)
      double precision dx(*)
      data zero, one /0d0,1d0/
c
c        version 1.1    date   22-08-91 (Zdenek: no dimension, cdc, ce)
c
c     euclidean norm of the n-vector stored in dx() with storage
c     increment incx .
c     if    n .le. 0 return with result = 0.
c     if n .ge. 1 then incx must be .ge. 1
c
c           c.l.lawson, 1978 jan 08
c
c     four phase method     using two built-in constants that are
c     hopefully applicable to all machines.
c         cutlo = maximum of  sqrt(u/eps)  over all known machines.
c         cuthi = minimum of  sqrt(v)      over all known machines.
c     where
c         eps = smallest no. such that eps + 1. .gt. 1.
c         u   = smallest positive no.   (underflow limit)
c         v   = largest  no.            (overflow  limit)
c
c     brief outline of algorithm..
c
c     phase 1    scans zero components.
c     move to phase 2 when a component is nonzero and .le. cutlo
c     move to phase 3 when a component is .gt. cutlo
c     move to phase 4 when a component is .ge. cuthi/m
c     where m = n for x() real and m = 2*n for complex.
c
c     values for cutlo and cuthi..
c     from the environmental parameters listed in the imsl converter
c     document the limiting values are as follows..
c     cutlo, s.p.   u/eps = 2**(-102) for  honeywell.  close seconds are
c                   univac and dec at 2**(-103)
c                   thus cutlo = 2**(-51) = 4.44089e-16
c     cuthi, s.p.   v = 2**127 for univac, honeywell, and dec.
c                   thus cuthi = 2**(63.5) = 1.30438e19
c     cutlo, d.p.   u/eps = 2**(-67) for honeywell and dec.
c                   thus cutlo = 2**(-33.5) = 8.23181d-11
c     cuthi, d.p.   same as s.p.  cuthi = 1.30438d19
c     data cutlo, cuthi / 8.232d-11,  1.304d19 /
c     data cutlo, cuthi / 4.441e-16,  1.304e19 /
      data cutlo, cuthi /8.232d-11,1.304d19/
chpu      on real underflow ignore
c
      if (n.gt.0) go to 10
      dnrm2 = zero
      go to 140
c
   10 assign 30 to next
      sum = zero
      nn = n*incx
c                                                 begin main loop
      i = 1
   20 go to next, (30, 40, 70, 80)
   30 if (abs(dx(i)).gt.cutlo) go to 110
      assign 40 to next
      xmax = zero
c
c                        phase 1.  sum is zero
c
   40 if (dx(i).eq.zero) go to 130
      if (abs(dx(i)).gt.cutlo) go to 110
c
c                                prepare for phase 2.
      assign 70 to next
      go to 60
c
c                                prepare for phase 4.
c
   50 i = j
      assign 80 to next
      sum = (sum/dx(i))/dx(i)
   60 xmax = abs(dx(i))
      go to 90
c
c                   phase 2.  sum is small.
c                             scale to avoid destructive underflow.
c
   70 if (abs(dx(i)).gt.cutlo) go to 100
c
c                     common code for phases 2 and 4.
c                     in phase 4 sum is large.  scale to avoid overflow.
c
   80 if (abs(dx(i)).le.xmax) go to 90
      sum = one + sum*(xmax/dx(i))**2
      xmax = abs(dx(i))
      go to 130
c
   90 sum = sum + (dx(i)/xmax)**2
      go to 130
c
c
c                  prepare for phase 3.
c
  100 sum = (sum*xmax)*xmax
c
c
c     for real or d.p. set hitest = cuthi/n
c     for complex      set hitest = cuthi/(2*n)
c
  110 hitest = cuthi/float(n)
c
c                   phase 3.  sum is mid-range.  no scaling.
c
      do 120 j=i,nn,incx
         if (abs(dx(j)).ge.hitest) go to 50
         sum = sum + dx(j)**2
  120 continue
      dnrm2 = sqrt(sum)
      go to 140
c
  130 continue
      i = i + incx
      if (i.le.nn) go to 20
c
c              end of main loop.
c
c              compute square root and adjust for scaling.
c
      dnrm2 = xmax*sqrt(sum)
  140 continue
chpu      on real underflow abort
      return
      end
