      subroutine uscompphi(eddy,phi,ni,nj,nk,nvirtual,ndim)
c ======================================================================
c
c        programmer     Philipe Reynier
c        version 1.0    date 06-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Sample user subroutine to calculate phi
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni,nj,nk,ndim,nvirtual,ndegfd
      double precision phi(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual),
     +                 eddy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:7),
     +                 a, b, c, rk2, R, rn

c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     phi            o    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision ak(7), Tc

c     ak array of the series coefficients
c     Tc temperature at the critical point

      integer i, j, k

c     i              General loop variable
c     j              General loop variable
c     k              General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c         usccoefhydr
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
      call uscoefhydr(a,b,c,rk2,R,rn)

      ak(1) = 3.308373371068d0
      ak(2) = 9.577904496647d0
      ak(3) = -42.7878181830733d0
      ak(4) = 143.5204743454d0
      ak(5) = -208.024055939d0
      ak(6) = 139.5022086855d0
      ak(7) = -35.8884206558d0

      Tc = 653.15d0

c     --- calculation of phi
c         eddy(i,j,7) is the temperature

      do i=1-nvirtual,1+ni+nvirtual
         do j=1-nvirtual,1+nj+nvirtual
            phi(i,j) = 1d0 - ak(1)
            do k=2,7
               phi(i,j) = phi(i,j)
     +                  + ak(k)*((eddy(i,j,7)/Tc)**(k-1))
            end do
            phi(i,j) = R*phi(i,j)
         end do
      end do
      end
