      subroutine usdrhodh(eddy,solut,ni,nj,nk,nvirtual,ndim,ndegfd,
     +                      drhodh)
c ======================================================================
c
c        programmer     Philipe Reynier
c        version 1.0    date 06-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Sample user subroutine to calculate drho/dh at constant pressure
c     Here drho/de (and not drho/dh) is calculated from the Van der Waals
c     equation with e = Cv * T
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni,nj,nk,ndim,nvirtual,ndegfd
      double precision solut(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual,1:ndegfd),
     +                 eddy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:7),
     +                 drhodh(1-nvirtual:1+ni+nvirtual,
     +                        1-nvirtual:1+nj+nvirtual),
     +                 phi(1-nvirtual:1+ni+nvirtual,
     +                     1-nvirtual:1+nj+nvirtual),
     +                 a, b, c, rk2, R, rn

c     drhodh         o    Contains drho/dh in all centres of the cells
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     solut          i    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision alp1, alp2, c1, den, temprn,
     +                 temprn1, term1, term2, term3, term4, v

      integer i, j


c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c         usccoefhydr
c         uscompphi
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call uscoefhydr(a,b,c,rk2,R,rn)
      call uscompphi(eddy,phi,ni,nj,nk,nvirtual,ndim)

      c1 = c*(rn + 1d0)

c     --- Calculation of drho/de at constant pressure
c         eddy(i,j,2) is the density
c         eddy(i,j,7) is the temperature

      do i=1,ni
         do j=1,nj
            v = 1d0/eddy(i,j,2)
            alp1 = v - a
            alp2 = v + b
            temprn = eddy(i,j,7)**rn
            temprn1 = eddy(i,j,7)**(1d0 + rn)
            term1 = R/alp1+ c*rn/(temprn1*(alp2**2))
            term2 = R*eddy(i,j,7)/(alp1**2d0)
     +                     - 2d0*c/(temprn*(alp2**3d0))
            term3 = phi(i,j) + rn*c1/(temprn1*alp2)
            term4 = c1/(temprn*(alp2**2d0))
            den = term3*term2/term1 + term4
            drhodh(i,j) = - (eddy(i,j,2)**2d0)/den
         end do
      end do
      end
