#include <sys/types.h>   /* /usr/include/sys/types.h */
#include <sys/times.h>   /* /usr/include/sys/times.h */

/* invoer: pointer naar een long integer;
 * uitvoer: via de long int function name de verblijftijd van het huidige pro-
 *          gram in aantal honderdsten van seconden sinds de laatste aanroep van
 *          settim();
 * resultaat: vulling van de door de ingevoerde pointer aangewezen long integer
 *            met de sinds de laatste aanroep van settim() verbruikte cpu-tijd
 *            van het huidige program in honderdsten van seconden;
 * externals: unsigned notsettim, hier geinitialiseerd, en getest op ~0 ter con-
 *               trole of settim() minstens 1-maal eerder dan deze function is
 *               aangeroepen; wordt in settim() op 0 gezet;
 *            long tickstart, gezet door settim(), zie aldaar;
 * aangeroepen door: gebruikers-programma;
 * roept aan: times() (zie TIMES(2)).
 * om met succes deze function aan te roepen, dient eerst settim() aangeroepen,
 * zoniet dan melding en doorgaan van het aanroepende program.
 * installatie en gebruik:
 *   compileren met cc -c filenaam.c, geeft filenaam.o (zie CC(1)),
 *   in bibliotheek zetten met ar r /usr/lib/libthd.a filenaam.o (zie AR(1)),
 *   link-en met de ld option -l, kan via compiler doorgegeven worden aan ld
 *   door -lthd achteraan het complete compile-command te plaatsen:
 *      compile-command [options] filenaam ... -lthd (zie LD(1) en betreffende
 *   COMPILE-COMMAND(1) ).
 */

unsigned notsettim = ~0;
extern clock_t tickstart;
extern long hz;

#ifdef UNDERSCORE
   float isasktim_(itime) float *itime;
#else
#ifdef CRAY
   float ISASKTIM(itime) float *itime;
#else
   float isasktim(itime) float *itime;
#endif
#endif
{ clock_t times(), ticks;
  struct tms cput;
  if ( notsettim ) { printf("asktim: settim not called\n"); return 0; }
  ticks = times(&cput);
  *itime = (float) cput.tms_utime / hz ;
  return ( (float) ( ticks - tickstart ) / hz ) ;
} /* end of asktim */
