      subroutine iserrecv ( icontext )
c ======================================================================
c
c        Programmer     Jason Frank
c        version 1.0    date 13-11-1998 Receive error information on host
c
c     copyright (c) 1994-1997  "Ingenieursbureau SEPRA"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "Ingenieursbureau SEPRA"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Receives error description from node
c
c **********************************************************************
c
c                       KEYWORDS
c
c     error_manipulation
c     parallel
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer icontext

c     icontext i     MPI message tag ( in {ISERCTXT, ISWRCTXT} )
c **********************************************************************
c
c                       COMMON BLOCKS
c
cpar      include 'IScommon/cisnas'
cpar      include 'IScommon/cismessg'
cpar      include 'IScommon/cismessc'
cpar      include 'IScommon/ciscom'
cpar      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
cpar      include 'mpif.h'
cpar      character * 8 namsubsv(20)
cpar      integer ierr, iblocksv, levelsv, i, nints, nreal, 
cpar     +   nchar, msgnum, isource, istatus(MPI_STATUS_SIZE)

c     i        general loop variable
c     iblocksv temporary storage of iblockrf
c     ierr     MPI error status
c     isource  node number of source of error
c     istatus  MPI receive status
c     levelsv  temporary storage for levelsub
c     mesnum   Message sequence number.
c     namsubsv temporary storage for array namsubs
c     nchar    Number of strings in a message
c     nints    Number of integers in a message
c     nreal    Number of reals in a message
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c      ISERRSUB  Print error message and quit, if limit is exceeded
c      ISSTOP    ISNaS stop execution routine
c      ISWARSUB  Print warning message
c      MPI_RECV  MPI receive function
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
cpar      if (inodenr.ne.ISHOST) then
c     --- node receives error message:  stop signal

cpar      call isstop

cpar      else 
c     --- host receives error message from node: receive description

cpar	 levelsv = levelsub
cpar	 iblocksv = iblockrf
cpar	 do i = 1, levelsv
cpar	    namsubsv(i) = namsubs(i)
cpar	 end do

cpar	 call MPI_RECV ( info, 10, ISINT, MPI_ANY_SOURCE,
cpar     +      icontext, MPI_COMM_WORLD, istatus, ierr )
cpar	 isource = istatus(MPI_SOURCE)

cpar	 msgnum = info(2)
cpar	 nints = info(3)
cpar	 nreal = info(4)
cpar	 nchar = info(5)
cpar	 levelsub = info(6)
cpar	 iblockrf = info(8)

cpar	 call MPI_RECV ( namsubs, info(6)*8, ISCHAR, isource,
cpar     +      icontext, MPI_COMM_WORLD, istatus, ierr )

cpar	 if ( ierr.ne.MPI_SUCCESS ) then
cpar	    print *, 'Fatal error in iserrecv: namsubs array'
cpar	    call isstop
cpar	 end if

cpar	 call MPI_RECV ( ints, nints, ISINT, isource,
cpar     +      icontext, MPI_COMM_WORLD, istatus, ierr )

cpar	 if ( ierr.ne.MPI_SUCCESS ) then
cpar	    print *, 'Fatal error in iserrecv: ints array'
cpar	    call isstop
cpar	 end if

cpar	 call MPI_RECV ( reals, nreal, ISDOUBLE, isource,
cpar     +      icontext, MPI_COMM_WORLD, istatus, ierr )

cpar	 if ( ierr.ne.MPI_SUCCESS ) then
cpar	    print *, 'Fatal error in iserrecv: reals array'
cpar	    call isstop
cpar	 end if

cpar	 call MPI_RECV ( chars, nchar, ISCHAR, isource,
cpar     +      icontext, MPI_COMM_WORLD, istatus, ierr )

cpar	 if ( ierr.ne.MPI_SUCCESS ) then
cpar	    print *, 'Fatal error in iserrecv: chars array'
cpar	    call isstop
cpar	 end if

cpar	 if ( icontext.eq.ICERROR ) then
      
cpar	    call iserrsub ( msgnum, nints, nreal, nchar )
         
cpar	 elseif ( icontext.eq.ICWARN ) then
      
cpar	    call iswarsub ( msgnum, nints, nreal, nchar )
         
cpar	 end if

cpar	 iblockrf = iblocksv
cpar	 levelsub = levelsv
cpar	 do i = 1, levelsub
cpar	    namsubs(i) = namsubsv(i)
cpar	 end do

cpar      end if

      end

