      subroutine isprrecv 
c ======================================================================
c
c        Programmer     Jason Frank
c        version 1.0    date 13-11-1998 Receive print information on host
c
c     copyright (c) 1994-1997  "Ingenieursbureau SEPRA"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "Ingenieursbureau SEPRA"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Receives message from nodew to be printed
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none

c **********************************************************************
c
c                       COMMON BLOCKS
c
cpar      include 'IScommon/cisnas'
cpar      include 'IScommon/cismessg'
cpar      include 'IScommon/cismessc'
cpar      include 'IScommon/ciscom'
cpar      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
cpar      include 'mpif.h'
cpar      integer, dimension(:), allocatable :: loc_ints, loc_reals
cpar      integer ierr, iblocksv, i, icount, nints, nreal, nchar,
cpar     +   msgnum, isource, istatus(MPI_STATUS_SIZE)

c     i        general loop variable
c     iblocksv temporary storage of iblockrf
c     ierr     MPI error status
c     isource  node number of source of error
c     istatus  MPI receive status
c     mesnum   Message sequence number.
c     nchar    Number of strings in a message
c     nints    Number of integers in a message
c     nreal    Number of reals in a message
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISERRSUB  Print error message and quit, if limit is exceeded
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISPRINT   ISNaS print function
c     MPI_RECV  MPI blocking receive function
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

cpar      call iseropen ('isprrecv')

cpar      isource = IS_ANY
cpar      icount = 10
cpar      call MPI_RECV ( info, icount, ISINT, isource, ICPRINT,
cpar     +   MPI_COMM_WORLD, istatus, ierr)

cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if
      
cpar      isource = istatus (MPI_SOURCE)

cpar      msgnum = info(2)
cpar      nints = info(3)
cpar      nreal = info(4)
cpar      nchar = info(5)
cpar      iblocksv = iblockrf
cpar      iblockrf = info(8)

cpar      if ( nints.gt.1000 ) then
c        --- Special situation, use FORTRAN 90 extension allocate

cpar	 allocate (loc_ints(1:nints))
cpar	 call MPI_RECV ( loc_ints, nints, ISINT, isource, 
cpar     +      ICPRINT, MPI_COMM_WORLD, istatus, ierr)

cpar      else 
c        --- Standard situation, use cismessg

cpar	 call MPI_RECV ( ints, nints, ISINT, isource, ICPRINT,
cpar     +      MPI_COMM_WORLD, istatus, ierr)

cpar      end if
cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if
      
cpar      if ( nreal.gt.1000 ) then
c        --- Special situation, use FORTRAN 90 extension allocate
 
cpar	 allocate (loc_reals(nreal))
cpar	 call MPI_RECV ( loc_reals, nreal, ISDOUBLE, 
cpar     +      isource, ICPRINT, MPI_COMM_WORLD, istatus, ierr)
cpar      else
c        --- Standard situation, use cismessg

cpar	 call MPI_RECV ( reals, nreal, ISDOUBLE, isource, 
cpar     +      ICPRINT, MPI_COMM_WORLD, istatus, ierr)

cpar      end if
cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if
      
cpar      call MPI_RECV ( chars, nchar*80, ISCHAR, isource, ICPRINT,
cpar     +   MPI_COMM_WORLD, istatus, ierr)
cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if
      
cpar      if ( nints.le.1000 .and. nreal.le.1000 ) then
cpar	 call isprint ( info(2), nints, ints, nreal, reals,
cpar     +                  nchar, chars )
cpar      else if ( nints.gt.1000 .and. nreal.le.1000 ) then
cpar	 call isprint ( info(2), nints, loc_ints, nreal,
cpar     +                  reals, nchar, chars )
cpar         deallocate (loc_ints)
cpar      else if ( nints.le.1000 .and. nreal.gt.1000 ) then
cpar	 call isprint ( info(2), nints, ints, nreal,
cpar     +                  loc_reals, nchar, chars )
cpar	 deallocate (loc_reals)
cpar      else
cpar	 call isprint ( info(2), nints, loc_ints, nreal,
cpar     +                  loc_reals, nchar, chars )
cpar	 deallocate (loc_ints, loc_reals)
cpar      end if

cpar      iblockrf = iblocksv

cpar      call iserclos ('isprrecv')

      end
