      subroutine issend ( ifirst, itypec, lengsd, rarray, iarray,
     +                    carray, nodenr )
c ======================================================================
c
c        Programmer     Guus Segal
c        version 1.4    date 17-08-1998 Jason: modified for MPI
c        version 1.3    date 08-02-1996 Adaptation for multiple errors
c        version 1.2    date 31-01-1996 Corrections for sending prints
c        version 1.1    date 07-12-1994 Use the pvm include file
c        version 1.0    date 04-07-1994
c
c
c     copyright (c) 1994-1996  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Send data to node or host.
c     This implementation is completely based on MPI
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     send
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ifirst, itypec, lengsd, iarray(*), nodenr
      double precision rarray(*)
      character *(*) carray(*)

c     carray  i    Character array for issend and isreceiv
c     iarray  i    Integer array for issend and isreceiv
c     ifirst  i    reserved
c     itypec  i    reserved
c     lengsd  i    Number of data to be sent, i.e. number of integers,
c                  strings or reals.
c                  lengsd = 0, is allowed.
c     nodenr  i    Sequence number of node to which message must be sent.
c                  If the message must be sent to the host, nodenr should be 0.
c     rarray  i    Real array to be used for sending and receiving of data
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/ciscom'
      include 'IScommon/cisparal'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical clear, send, clearbuf, specsend, firstsnd
      integer msgtype, ierr
      include 'mpif.h'
      save clearbuf, specsend, firstsnd
      data clearbuf /.true./, specsend /.false./, firstsnd /.true./

c     clear     Indicates if the buffer must be cleared (true) or not
c     clearbuf  If true the buffer must be cleared in the next step
c     firstsnd  If true the last standard data has been sent
c     ierr      Return code of a subroutine (<0 is error)
c     msgtype   Message type for PVM, may be used for identification
c     send      If true the message must be sent
c     specsend  If true a warning or output message has been put in the buffer
c **********************************************************************
c
c                       I/O
c
c     Information is sent to host or node depending on the parameter
c     ifirst
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSPR  Special error message subroutine in case of parallel computing
c     ISPACK    "Safe" wrapper for MPI_PACK
c     MPI_SEND  Immediately sends the data in the active MPI message buffer
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1703   The length of the array to be send is 0
c    1704   This is not the first array to be sent, whereas ifirst = 0 or 1
c    1705   clear_buffer = false and special_send = false, whereas ifirst =
c           0 or 1
c    1706   clear_buffer = true, whereas ifirst = 2 or 3
c    1707   first_send = true, whereas ifirst = 2 or 3
c    1750   PVM error
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The following logicals are used:
c     clear_buffer:  if true the buffer must be cleared
c     special_send:  if true a warning or output message has been put in
c                    the buffer
c     first_send:    if true the last standard data has been sent
c     send:          if true the message must be sent
c     pack:          if true the message must be packed
c     clear:         if true the buffer must be cleared
c
c     Code:
c
c     At initialization:
c        clear_buffer := true
c        special_send := false
c        first_send := true
c     if ( itypec<0 ) then
c        special_send := true
c        clear := clear_buffer
c        send := itypec=-1
c     else ( itypec>=0 )
c        if ( ifirst=0 ) then
c           if ( lengsd<=0 ) error
c           if ( not first_send ) error
c           if ( not clear_buffer and not special_send ) error
c           clear := not special_send
c           send := true
c        else if ( ifirst=1 ) then
c           if ( not clear_buffer and not special_send ) error
c           if ( not first_send ) error
c           clear := not special_send
c           send := false
c           first_send := false
c        else if ( ifirst=2 ) then
c           if ( clear_buffer ) error
c           if ( first_send ) error
c           send  := false
c           clear := false
c        else ( ifirst=3 )
c           if ( clear_buffer ) error
c           if ( first_send ) error
c           send := true
c           clear := false
c     if ( clear ) then
c        Clear and prepare send buffer
c        clear_buffer := false
c     Fill positions 1, 8, 9 and 10 of array info in common ciscom
c     if ( itypec>0 ) then
c        Fill position 2 of array info
c     Pack array info in send buffer
c     if ( lengsd>0 ) Pack actual array in send buffer
c     if ( send ) then
c        Perform actual send
c        special_send := false
c        first_send   := true
c        clear_buffer := true
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'issend' )

      if ( itypec.lt.0 ) then

c     --- itypec < 0

         specsend = .true.
         clear = clearbuf
         send = itypec.eq.-1 .or. itypec.eq.-4

      else

c     --- itypec >= 0

         if ( ifirst.eq.0 ) then

c        --- ifirst=0, only one array to be sent

            if ( lengsd.le.0 ) then

c           --- Error: pack = false

               call iserrspr ( 1703, 0, 0, 0 )

            end if
            if ( .not. firstsnd ) then

c           --- Error: first_send = false

               call iserrint ( ifirst, 1 )
               call iserrspr ( 1704, 1, 0, 0 )

            end if

            if ( .not. clearbuf .and. .not. specsend ) then

c           --- Error: clear_buffer = false and special_send = false

               call iserrint ( ifirst, 1 )
               call iserrspr ( 1705, 1, 0, 0 )

            end if
            clear = .not. specsend
            send = .true.

         else if ( ifirst.eq.1 ) then

c        --- ifirst=1, first array to be sent, but not the last one

            if ( .not. clearbuf .and. .not. specsend ) then

c           --- Error: clear_buffer = false and special_send = false

               call iserrint ( ifirst, 1 )
               call iserrspr ( 1705, 1, 0, 0 )

            end if
            clear = .not. specsend
            send = .false.
            firstsnd = .false.

         else if ( ifirst.eq.2 ) then

c        --- ifirst=1, neither the first array to be sent, nor the last one

            if ( clearbuf ) then

c           --- Error: clear_buffer = true

               call iserrint ( ifirst, 1 )
               call iserrspr ( 1706, 1, 0, 0 )

            end if

            if ( firstsnd ) then

c           --- Error: first_send = true

               call iserrint ( ifirst, 1 )
               call iserrspr ( 1707, 1, 0, 0 )

            end if

            send  = .false.
            clear = .false.

         else if ( ifirst.eq.3 ) then

c        --- ifirst=3, the last array to be sent, but not the first one

            if ( clearbuf ) then

c           --- Error: clear_buffer = true

               call iserrint ( ifirst, 1 )
               call iserrspr ( 1706, 1, 0, 0 )

            end if

            if ( firstsnd ) then

c           --- Error: first_send = true

               call iserrint ( ifirst, 1 )
               call iserrspr ( 1707, 1, 0, 0 )

            end if

            send = .true.
            clear = .false.

         end if

      end if

      if ( clear ) then

c     --- clear = true, Clear and prepare send buffer

         isndpos = 1

         clearbuf = .false.

      end if

c     --- Fill positions 1, 8, 9 and 10 of array info in common ciscom

      if ( itypec.ne.-4 ) info(1) = itypec
      info(8) = iblockrf
      info(9) = inodenr
      if ( itypec.eq.-1 .or. itypec.ge.0 .and.
     +    (ifirst.eq.0 .or. ifirst.eq.3) ) then
         info(10) = 1
      else
         info(10) = 0
      end if

      if ( itypec.ge.0 ) info(2) = lengsd

c     --- Pack array info in send buffer
c         Pack actual array in send buffer

      if ( itypec.lt.0 .and. itypec.gt.-3 ) info(6) = levelsub
      call ispack ( info, 10, MPI_INTEGER, isndbuf, mxmsglen,
     +   isndpos, MPI_COMM_WORLD, ierr)
      if ( ierr.ne.MPI_SUCCESS ) then

c     --- packing in buffer produces an error, this is a severe error

         call iserrint ( ierr, 1 )
         call iserrspr ( 1750, 0, 0, 0 )

      end if
      if ( itypec.lt.0 .and. itypec.gt.-3 ) then

c     --- error or warning, send information for traceback

         if ( levelsub.gt.0 ) call ispack ( namsubs, levelsub*8,
     +           MPI_CHARACTER, isndbuf, mxmsglen, isndpos,
     +           MPI_COMM_WORLD, ierr)
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- packing in buffer produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if
         if ( info(3).gt.0 ) call ispack ( ints, info(3),
     +        MPI_INTEGER, isndbuf, mxmsglen, isndpos,
     +        MPI_COMM_WORLD, ierr )
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- packing in buffer produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if
         if ( info(4).gt.0 ) call ispack ( reals, info(4),
     +        MPI_DOUBLE_PRECISION, isndbuf, mxmsglen, isndpos,
     +        MPI_COMM_WORLD, ierr )
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- packing in buffer produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if
         if ( info(5).gt.0 ) call ispack ( chars, info(5)*80,
     +        MPI_CHARACTER, isndbuf, mxmsglen, isndpos,
     +        MPI_COMM_WORLD, ierr )
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- packing in buffer produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if

      else if ( itypec.eq.-3 ) then

c     --- itypec = -3, prints

         if ( info(3).gt.0 ) call ispack ( iarray, info(3),
     +        MPI_INTEGER, isndbuf, mxmsglen, isndpos,
     +        MPI_COMM_WORLD, ierr )
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- packing in buffer produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if
         if ( info(4).gt.0 ) call ispack ( rarray, info(4),
     +        MPI_DOUBLE_PRECISION, isndbuf, mxmsglen, isndpos,
     +        MPI_COMM_WORLD, ierr )
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- packing in buffer produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if
         if ( info(5).gt.0 ) call ispack ( carray, info(5)*80,
     +        MPI_CHARACTER, isndbuf, mxmsglen, isndpos,
     +        MPI_COMM_WORLD, ierr )
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- packing in buffer produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if

      else if ( itypec.eq.0 .and. lengsd.gt.0 ) then

c     --- itypec = 0, characters

         call ispack ( carray, lengsd*len(carray(1)),
     +       MPI_CHARACTER, isndbuf, mxmsglen, isndpos,
     +       MPI_COMM_WORLD, ierr )

      else if ( itypec.eq.3 .and. lengsd.gt.0 ) then

c     --- itypec = 3, integers

         call ispack ( iarray, lengsd, MPI_INTEGER, isndbuf,
     +       mxmsglen, isndpos, MPI_COMM_WORLD, ierr )

      else if ( itypec.eq.6 .and. lengsd.gt.0 ) then

c     --- itypec = 6, reals

         call ispack ( rarray, lengsd, MPI_DOUBLE_PRECISION, isndbuf,
     +       mxmsglen, isndpos, MPI_COMM_WORLD, ierr )

      end if
      if ( itypec.gt.0 .and. ierr.ne.MPI_SUCCESS ) then

c     --- packing in buffer produces an error, this is a severe error

         call iserrint ( ierr, 1 )
         call iserrspr ( 1750, 0, 0, 0 )

      end if

      if ( send ) then

c     --- Perform actual send

         msgtype = 1
         call MPI_SEND ( isndbuf, isndpos, MPI_PACKED, nodenr,
     +       msgtype, MPI_COMM_WORLD, ierr )
         if ( ierr.ne.MPI_SUCCESS ) then

c        --- sending produces an error, this is a severe error

            call iserrint ( ierr, 1 )
            call iserrspr ( 1750, 0, 0, 0 )

         end if
         specsend = .false.
         firstsnd = .true.
         clearbuf = .true.

      end if

      call iserclos ( 'issend' )
      end
