      program isexpand

c***********************************************************************
c
c  purpose: read direct access   reserved.unf  ... local or ISNAS(Unix or DOS)
c           read short source    short.tmp     ... local
c           write long source    long.tmp      ... local
c           messages (if any)    report.fex    ... local
c
c  usage:   isexpand
c
c  files:   short.tmp      i   Fortran source to be expanded
c           long.tmp       o   Fortran source, expanded
c
c           reserved.unf   i   direct access sorted list of all explanations
c
c           report.fex     o   error messages (if any)
c
c***********************************************************************
c
c  programmer   Zdenek Vlasek
c
c  version      2.1  01-09-92  advise at error message
c  version      2.0  31-08-92  complete revision
c  version      1.2  26-08-92  filenames, filenumbers, column 2, safe opening
c  version      1.1  24-08-92  another layout, closer to sepcheck
c  version      1.0  06-08-92  in Fortran
c  version      0.0  09-07-92  (in Turbo-Pascal)
c
c***********************************************************************
c
c  Syntax of direct access file reserved.unf --- see buildres.for
c
c***********************************************************************
c
c  Syntax of input file short.tmp is free,
c  some special lines provoke its expansion, they look like these
c
C     X  reserved
*     xYZZyx12  i/o  reserved
c           xyzzyz24         i            Reserved. See reserved.dat
c
c  Syntax of special lines:
c            column 1 is 'c' or 'C' or '*'
c            column 2 is ' '   (not '|')
c            keyword starts in columns 3..mxkeypos=13
c            keyword consists of letters, digits, underscores and '@'
c            keyword length <= lenkey=8
c            keyword delimiter is ' '
c            i/o information (if present) is one of 'i' 'o' 'io' 'i/o'
c            explanation starts by word 'reserved' or 'Reserved' and
c              the next character is  ' '  '.'  ','  ':'  or none
c
c***********************************************************************
c
c  Syntax of output file long.tmp ( compared to short.tmp ):
c
c            only the special lines may cause its replacement by one or
c                 more new lines
c            new lines look like these
c
c|    x  .................................
c|       .................................
c|       .................................
c|       .................................
c|    xyzzyx12  i/o  ::::::::::::::::::::::::::::::::::::::::
c|                   ::::::::::::::::::::::::::::::::::::::::
c|                   ::::::::::::::::::::::::::::::::::::::::
c|                   ::::::::::::::::::::::::::::::::::::::::
c|                   ::::::::::::::::::::::::::::::::::::::::
c|          xyzzyx24         i            .....................................
c|                                        .....................................
c|                                        .....................................
c|                                        .....................................
c|                                        .....................................
c  Syntax of new lines:
c            columns 1..2 are 'c|'
c            keyword is in lowercase
c            i/o information (if present) is preserved
c            the explanation is adjusted to the column where 'reserved' was
c               (here we fill the space by ......... or :::::::::::)
c
c***********************************************************************

cimp      implicit none

c     ---------------------- constants -----------

      integer lenkey, mxkeypos, mxkeys, posexpl
      character * 50 pathnam(3), warn
      parameter (
     .  lenkey = 8,
     .  mxkeypos = 13,
     .  mxkeys = 2000,
     .  posexpl = 20,
     .  warn = '!!!!!! do not edit this direct access file !!!!!! ')

c     ---------------------- local variables -----

      logical ok
      integer ik, ik1, ik2, ike, ios, ipath, iunf, j,
     .        k1, k2, k3, k4, k5, k6, nkeys, nreplac, nsub
      character      w
      character *  3 io
      character *  8 kwrd, prevkey
      character * 50 nam
      character * 80 fixline
c     *  'reserved' may start at column 73 and explanation may be 61 chars
      character *133 newline

c     ---------------------- commons to keep huge arrays on HP -------

      character *  8 keys(mxkeys)
      common /ckeys/ keys

      integer bpos(mxkeys), epos(mxkeys)
      common /cpos/ bpos, epos

c     ---------------------- commons for subroutine errre -------------

      integer iline, nerr
      character * 80 line
      common /cint/ nerr, iline
     .       /cchar/ line

      integer funf, fold, fnew, fout
      common /crefs/ funf, fold, fnew, fout

c     ---------------------- local functions -----

      character c
      logical keychar

      keychar ( c ) = c.ge.'a' .and. c.le.'z' .or.
     .                c.ge.'A' .and. c.le.'Z' .or.
     .                c.ge.'0' .and. c.le.'9' .or.
     .                c.eq.'_'  .or.
     .                c.eq.'@'
c     *  Character '@' is used in FTN77-specific routines on PC

c***********************************************************************
c
c   bpos      where keys(*) begin in file reserved.unf
c   epos      where keys(*) end   in file reserved.unf
c   fixline   one fix-syntax line
c   fnew      long.tmp
c   fold      short.tmp
c   fout      report.fex, error messages
c   funf      reserved.unf
c   ik        position in keys(*)
c   ik1       position in keys(*)
c   ik2       position in keys(*)
c   ike       position in keys(*) as read from reserved.unf
c   iline     linenumber in a current file
c   io        string like 'i/o'
c   ios       iostat in open/read
c   ipath     position in pathnam(*)
c   iunf      position in file reserved.unf
c   j         position in a string
c   k1        column where the first word begins
c   k2        column where the first word ends
c   k3        column where the second word begins
c   k4        column where the second word ends
c   k5        column where the output line ends
c   k6        column where 'reserved' begins
c   keys      sorted keywords from reserved.unf
c   kwrd      keyword extracted from input line
c   lenkey    max length of keyword in keys(*)
c   line      current line
c   mxkeypos  max column where a keyword starts in reserved.upd
c   mxkeys    max number of keywords in reserved.upd
c   nam       the current pathname
c   nerr      counter of errors
c   newline   line to be sent to long.tmp
c   nkeys     number of keywords in keys(*)
c   nreplac   number of performed expansions
c   nsub      number of subroutines
c   ok        true = successfull read
c   pathnam   all pathnames for reserved.unf
c   posexpl   column where the explanation starts in fix syntax
c   prevkey   previous key
c   w         character to be inspected
c   warn      constant string, headline in reserved.unf
c
c***********************************************************************

c     *  Initialize

      data pathnam / 'reserved.unf',
     .               '/mnt/deft/isnas/instal/reserved.unf',
     .               'c:\isnas\instal\reserved.unf' /

      nerr = 0

      fout = 21
      funf = 22
      fold = 23
      fnew = 24

      open ( fout, file = 'report.fex' )
      open ( fold, file = 'short.tmp' )
      open ( fnew, file = 'long.tmp' )

c     *  Try to open reserved.unf on different pathes


      do ipath = 1, 3
        nam = pathnam(ipath)
        open ( funf, file = nam, iostat = ios,
     .         access = 'DIRECT', recl = 80, status='OLD' )
        if ( ios .eq. 0 ) goto 10
      enddo

      iline = -1
      call errre ( 1, 'unable to open reserved.unf')

   10 continue

c     *  Read and check line 1

      iline = 1
      read ( funf, rec=iline, iostat=ios ) line
      if ( ios .ne. 0 ) then
        call errre ( 1, 'unreadable line 1 of '//nam)
      endif
      if ( line(1:50) .ne. warn ) then
        call errre ( 1, 'line 1 is bad in '//nam)
      endif
      read ( line, 9995, iostat=ios ) fixline, nkeys
      if ( ios .ne. 0 ) then
        call errre ( 1, 'line 1, unradable nkeys in '//nam)
      endif

*     Fill keys(*), bpos(*), epos(*)

      if ( nkeys .gt. mxkeys ) then
        call errre ( 1, 'mxkeys overflow, part 1 of '//nam)
      endif

      prevkey = ' '
      do ik = 1, nkeys
        iline = iline + 1
        read ( funf, rec=iline, iostat=ios ) line
        if ( ios .ne. 0 ) then
          call errre ( 1, 'unreadable line of '//nam)
        endif

        read ( line, 9996, iostat=ios ) ike,
     .       keys(ik), bpos(ik), epos(ik)
        if ( ios .ne. 0 ) then
          call errre ( 1, 'keyword or positions not readable in *.unf')
        endif

c       *  bpos(*), epos(*) point into the file reserved.unf

        if ( prevkey .ge. keys(ik) ) then
          call errre ( 1, 'keyword order in '//nam)
        else
          prevkey = keys(ik)
        endif

      enddo

c----------------------------------------------------------------------------
c----------------------------------------------------------------------------
c     *  Now, we have a sorted array keys(1:nkeys) of keywords
c     *  and their positions in the direct access file reserved.unf
c----------------------------------------------------------------------------
c----------------------------------------------------------------------------
      nsub = 0
      nreplac = 0

  100 continue

c     *  Read next subroutine from short.tmp
c     *  Expand the comments, write into the file long.tmp

      nsub = nsub + 1

      do 700 iline = 1, 1000000

        line = ' '
        ok = .false.
        read ( fold, 9999, end=720, err=110 ) line
        ok = .true.
 110    if ( .not. ok ) then
          call errre ( 1, 'the input line of the source is unreadable')
        endif

        if ( nkeys .eq. 0 ) goto 500

c       *  Check column 1 and column 2

        w = line(1:1)
        if ( w .ne. 'c' .and. w .ne. 'C' .and. w .ne. '*'
     .      .or. line(2:2) .ne. ' ' ) goto 500

c       *  Analyse the line: keyword  (if any) :   line(k1:k2)
c       *                    2nd word (if any) :   line(k3:k4)

c       *  Fill k1 by begin position 1st word, if any, otherwise k1=81

        do k1 = 3, 80
          if ( line(k1:k1) .ne. ' ' ) goto 130
        enddo
        k1 = 81
  130   continue

        if ( k1 .gt. mxkeypos ) goto 500

c       *  Something starts at k1, k1<=mxkeypos

        if ( .not. keychar(line(k1:k1)) ) goto 500

c       *  A keyword may start at k1

c       *  Fill k2 by end position of keyword

        do k2 = k1, 79
          if ( .not. keychar(line(k2+1:k2+1)) ) goto 150
        enddo
        k2 = 80
  150   continue

c       *  Check the length

        if ( k2-k1+1 .gt. lenkey ) goto 500

c       *  Check keyword delimiter

        if ( line(k2+1:k2+1) .ne. ' ' ) goto 500

c       *  Initialize newline(*), convert newline(k1:k2) to lowercase, ASCII

        newline = line
        newline(1:2) = 'c|'

        do j = k1, k2
          if ( newline(j:j) .ge. 'A' .and. newline(j:j) .le. 'Z' )
     .       newline(j:j) = char ( ichar(newline(j:j)) + 32 )
        enddo

c       *  Get the keyword

        kwrd = newline(k1:k2)

c       *  Fill k3 by begin position of 2nd word, if any

        do k3 = k2+2, 80
          if ( line(k3:k3) .ne. ' ' ) goto 170
        enddo
        k3 = 81
  170   continue

c       *  Is there space for 'reserved' ?
        if ( k3 .gt. 73 ) goto 500

c       *  Fill k4 by end position of 2nd word

        do k4 = k3, 79
          if ( line(k4+1:k4+1) .eq. ' ' ) goto 210
        enddo
        k4 = 80
  210   continue

c       *  Is 2nd word an i/o string ?

        if ( k4.le.78 .and. k4-k3 .ge. 0 .and. k4-k3 .le. 2 ) then
c         *  It may be an i/o string

          io = line(k3:k4)
          if ( io .eq. 'i  ' .or.
     .         io .eq. 'o  ' .or.
     .         io .eq. 'io ' .or.
     .         io .eq. 'i/o' ) then
c           *  Keep it there

c           *  Fill k3 by begin position of 3rd word, if any

            do k3 = k4+2, 80
              if ( line(k3:k3) .ne. ' ' ) goto 230
            enddo
            k3 = 81
  230       continue
          endif
        endif

c       *  When no i/o   :  k1..k2          k3..k4  ... ( k3<=81, k4<=80 )
c       *       i/o found:  k1..k2  ??..k4  k3..        ( k3<=81 )

c       *  Search for the word 'reserved' or 'Reserved' in line(k3:)

c       *  Is there space for 'reserved' ?
        if ( k3 .gt. 73 ) goto 500

        k6 = (k3-1) + index ( line(k3:), 'eserved' ) - 1

        if ( k6 .ne. k3 ) goto 500

c       *  Check the first character
        w = line(k6:k6)
        if ( w .ne. 'r' .and. w.ne.'R' ) goto 500

        if ( k6 .le. 72 ) then
c         *  Check the delimiter
          w = line(k6+8:k6+8)
          if ( w.ne.' ' .and. w.ne.'.' .and.
     .         w.ne.',' .and. w.ne.':' ) goto 500
        endif

c       *  Found 'reserved' or 'Reserved'

c       *  Is kwrd in sorted keys(*) ?
c       *  Perform the binary search, fill ik>0 if found, ik=0 if not found

        ik1 = 0
        ik2 = nkeys + 1
c       *  Notice, nkeys>=1 here, so ik2>=2
  300   continue
c         *  It holds  ik2-ik1>=2,  keys(ik1) < kwrd < keys(ik2)
          ik = ( ik1 + ik2 ) /2
          if ( kwrd .eq. keys(ik) ) then
c           * Found in the list
            goto 310
          elseif ( kwrd .lt. keys(ik) ) then
            ik2 = ik
          else
            ik1 = ik
          endif
c         *  It holds  ik2-ik1>=1,  keys(ik1) < kwrd < keys(ik2)
        if ( ik2-ik1 .ge. 2 ) goto 300
c       *  The kwrd shoud be between the keys(ik1) and keys(ik1+1=ik2)
c       *  Not found, tell it by ik=0
        ik = 0
  310   continue

        if ( ik .eq. 0 ) then
          call errre ( -1, kwrd//' is unknown, no expansion' )
          goto 500
        endif

c       *  There is kwrd=keys(ik)

        if ( epos(ik) .lt. 0 ) then
          call errre ( -1, kwrd// ' is duplicate, no expansion' )
          goto 500
        else
c         *  The first expansion of this keyword, remember it by '-' sign
          nreplac = nreplac + 1
          epos(ik) = -epos(ik)
        endif

c       *  Read explanation lines from reserved.unf, modify newline, write

        do 480 iunf = bpos(ik), abs(epos(ik))

          read ( funf, rec=iunf, iostat=ios ) fixline

          if ( ios .ne. 0 ) then
            line = ' '
            write ( fout, '(a,i6)' ) 'recordnumber = ', iunf
            call errre ( 1, 'unreadable line in '//nam )
          endif

          if ( iunf .eq. bpos(ik)+1 ) then
c           *  The first continuation of the explanation, initialize newline
            newline(1:k3-1) = 'c|'
          endif

c         *  Adjust the explanation into newline

          newline(k3:) = fixline(posexpl:)

c       *  Let k5 = length(newline)

          do k5 = 133, 1, -1
            if ( newline(k5:k5) .ne. ' ' ) goto 460
          enddo
          k5 = 0
  460     continue

          write ( fnew, 9999) newline(1:k5)

          if (  k5 .gt. 80 ) then
            call errre ( -1, kwrd// ' new line is too long, '//
     .        'shift "reserved" to column 21' )
          endif

  480   continue

        goto 700

c       ------------------------  Unchanged line(*)
  500   continue

c       *  Let k5 = length(line)

        do k5 = 80, 1, -1
          if ( line(k5:k5) .ne. ' ' ) goto 510
        enddo
        k5 = 0
  510   continue

        if ( k5.eq.0 ) then
*         Here we need to write an empty line, yet FTN77 writes ' '
          write ( fnew, 9999)
        else
          write ( fnew, 9999) line(1:k5)
        endif

c       *  Detect the end of routine (if present) to avoid duplicate keyword

        if ( line(1:9) .eq. '      end' ) goto 710
        if ( line(1:9) .eq. '      END' ) goto 710

  700 continue

c---------------------- One subroutine finished, reset epos(*)
  710 continue

      do ik = 1, nkeys
        epos(ik) = abs(epos(ik))
      enddo

      goto 100
c----------------------------------------------------- Loop back

c---------------------- File short.tmp is exhausted
  720 continue
      iline = iline - 1

      close ( funf )
      close ( fold )
      close ( fnew )
      if ( nerr .eq. 0 ) then
        close ( fout, status='DELETE')
      else
        close ( fout )
      endif

      if ( .false. ) then
         write ( fout, '(/(a,i5))' )
     .     'number of subroutines = ', nsub,
     .     'number of input lines = ', iline,
     .     'number of expansions  = ', nreplac,
     .     'number of errors      = ', nerr
      endif

 9995 format ( a50, i5 )
 9996 format ( i5, 2x, a8, 1x, 2i6, 52x )
 9999 format ( a )

      end
c ======================================================================
      subroutine errre ( stopit, message )

cimp      implicit none

c     ---------------------- i/o parameters ------

      integer stopit
      character * (*) message

c     message   i  an error message
c     stopit    i  -1 = one line warning, do not stop
c     stopit    i   0 = error,            do not stop
c     stopit    i   1 = error,            stop

c     --------------------- commons ------------------

      integer iline, nerr
      character * 80 line
      common /cint/ nerr, iline
     .       /cchar/ line

      integer funf, fold, fnew, fout
      common /crefs/ funf, fold, fnew, fout

c     --------------------------------------------

      nerr = nerr + 1

      if ( stopit .eq. -1 ) then

c       * Warning

        write ( fout, '( a, i5, 2a )' )
     .         'line', iline, ', warning: ', message

      else

c       *  Error

        if ( iline .ge. 0 ) then
          write ( fout, '( 80(''_'') / a / a, i5, 2a / 80(''^'') )' )
     .      line, '  !!! error, line ', iline, ': ', message
        else
          write ( fout, '( 80(''_'') / 2a / 80(''^'') )' )
     .      '  !!! error: ', message
        endif

        if ( stopit .eq. 0 ) then
          write ( fout, '(a)' ) '------- fatal error ------ stop ------'
          close ( fout )
          stop
        endif

      endif

      end
