      program isreduce

c***********************************************************************
c
c  purpose: read  long source    long.tmp      ... local
c           write short source   short.tmp     ... local
c           messages (if any)    report.fex    ... local
c
c  usage:   isreduce
c
c  files:   long.tmp       i   Fortran source to be reduced
c           short.tmp      o   Fortran source, reduced
c           report.fex     o   error messages (if any)
c
c***********************************************************************
c
c  programmer   Zdenek Vlasek
c
c  version      1.0  31-08-92  complete revision
c  version      0.0  09-07-92  (in Turbo-Pascal)
c
c***********************************************************************
c
c  Syntax of input file long.tmp  :  see the program isexpand
c  Syntax of input file short.tmp :  see the program isexpand
c
c***********************************************************************

cimp      implicit none

c     ---------------------- constants -----------

      integer lenkey, mxkeypos, posexpl
      parameter (
     .  lenkey = 8,
     .  mxkeypos = 13,
     .  posexpl = 20 )

c     ---------------------- local variables -----

      logical ok, iofound
      integer j, k1, k2, k3, k4, k5, k6, nreplac, nsub
      character      w
      character *  3 io
      character * 80 newline

c     ---------------------- commons for subroutine errr -------------

      integer iline, nerr
      character * 80 line
      common /cint/ nerr, iline
     .       /cchar/ line

      integer funf, fold, fnew, fout
      common /crefs/ funf, fold, fnew, fout

c     ---------------------- local functions -----

      character c
      logical keychar

      keychar ( c ) = c.ge.'a' .and. c.le.'z' .or.
     .                c.ge.'A' .and. c.le.'Z' .or.
     .                c.ge.'0' .and. c.le.'9' .or.
     .                c.eq.'_'  .or.
     .                c.eq.'@'
c     *  Character '@' is used in FTN77-specific routines on PC

c***********************************************************************
c
c   fnew      short.tmp
c   fold      long.tmp
c   fout      report.fex, error messages
c   iline     linenumber in a current file
c   io        string like 'i/o'
c   iofound   true = i/o string found
c   ipath     position in pathnam(*)
c   iunf      position in file reserved.unf
c   j         position in a string
c   k1        column where the first word begins
c   k2        column where the first word ends
c   k3        column where the second word begins
c   k4        column where the second word ends
c   k5        column where the output line ends
c   k6        column where 'reserved' begins
c   keys      sorted keywords from reserved.unf
c   lenkey    max length of keyword
c   line      current line
c   mxkeypos  max column where a keyword starts in reserved.upd
c   nerr      counter of errors
c   newline   line to be sent to short.tmp
c   nreplac   number of performed reductions
c   nsub      number of subroutines
c   ok        true = successfull read
c   pathnam   all pathnames for reserved.unf
c   posexpl   column where the explanation starts in fix syntax
c   w         character to be inspected
c   warn      constant string, headline in reserved.unf
c
c***********************************************************************

c     *  Initialize

      nerr = 0

      fout = 21
      fold = 23
      fnew = 24

      open ( fout, file = 'report.fex' )
      open ( fold, file = 'long.tmp' )
      open ( fnew, file = 'short.tmp' )

      nsub = 0
      nreplac = 0

  100 continue

c     *  Read next subroutine from long.tmp
c     *  Reduce the comments, write into the file short.tmp

      nsub = nsub + 1

      do 700 iline = 1, 1000000

        line = ' '
        ok = .false.
        read ( fold, 9999, end=720, err=110 ) line
        ok = .true.
 110    if ( .not. ok ) then
          call errr ( 1, 'the input line of the source is unreadable')
        endif

c       *  Check column 1 and column 2

        w = line(1:1)
        if ( w .ne. 'c' .and. w .ne. 'C' .and. w .ne. '*'
     .      .or. line(2:2) .ne. '|' ) goto 500

c       *  This line must be skipped (no keyword)
c       *                or replaced (keyword)

c       *  Analyse the line: keyword  (if any) :   line(k1:k2)
c       *                    2nd word (if any) :   line(k3:k4)

c       *  Fill k1 by begin position 1st word, if any, otherwise k1=81

        do k1 = 3, 80
          if ( line(k1:k1) .ne. ' ' ) goto 130
        enddo
        k1 = 81
  130   continue

        if ( k1 .gt. mxkeypos ) goto 700

c       *  Something starts at k1, k1<=mxkeypos, it must be a keyword

        if ( .not. keychar(line(k1:k1)) ) then
          call errr ( -1, 'bad character in keyword field')
          goto 500
        endif

c       *  A keyword starts at k1

c       *  Fill k2 by end position of keyword

        do k2 = k1, 79
          if ( .not. keychar(line(k2+1:k2+1)) ) goto 150
        enddo
        k2 = 80
  150   continue

c       *  Check the length

        if ( k2-k1+1 .gt. lenkey ) then
          call errr ( -1, 'too long keyword' )
          goto 500
        endif

c       *  Check keyword delimiter

        if ( line(k2+1:k2+1) .ne. ' ' ) then
          call errr ( -1, 'bad delimiter of keyword' )
          goto 500
        endif

c       *  Initialize newline(*), convert newline(k1:k2) to lowercase, ASCII

        newline = line
        newline(1:2) = 'c '

        do j = k1, k2
          if ( newline(j:j) .ge. 'A' .and. newline(j:j) .le. 'Z' )
     .       newline(j:j) = char ( ichar(newline(j:j)) + 32 )
        enddo

c       *  Fill k3 by begin position of 2nd word, if any

        do k3 = k2+2, 80
          if ( line(k3:k3) .ne. ' ' ) goto 170
        enddo
        k3 = 81
  170   continue

c       *  Fill k4 by end position of 2nd word

        do k4 = k3, 79
          if ( line(k4+1:k4+1) .eq. ' ' ) goto 210
        enddo
        k4 = 80
  210   continue

c       *  Is 2nd word an i/o string ?

        iofound = .false.

        if ( k4.le.78 .and. k4-k3 .ge. 0 .and. k4-k3 .le. 2 ) then
c         *  It may be an i/o string

          io = line(k3:k4)
          if ( io .eq. 'i  ' .or.
     .         io .eq. 'o  ' .or.
     .         io .eq. 'io ' .or.
     .         io .eq. 'i/o' ) then
c           *  Keep it there

            iofound = .true.

c           *  Fill k3 by begin position of 3rd word, if any

            do k3 = k4+2, 80
              if ( line(k3:k3) .ne. ' ' ) goto 230
            enddo
            k3 = 81
  230       continue
          endif
        endif

c       *  When no i/o   :  k1..k2          k3..k4  ... ( k3<=81, k4<=80 )
c       *       i/o found:  k1..k2  ??..k4  k3..        ( k3<=81 )

        if ( k3 .le. 72 ) then

c         *  Preserve the adjusting
          k6 = k3

        else
c         *  Default adjusting of 'reserved'

           if ( iofound ) then
             k6 = k2 + 2
           else
             k6 = k4 + 2
             if ( k6 .gt. 72 ) then
               call errr ( -1, 'i/o string at endline, deleted')
               k6 = k2 + 2
             endif

           endif

c          *  Not sooner then posexpl
           if ( k6 .lt. posexpl ) k6 = posexpl

        endif

        newline(k6:) = 'reserved'

c       *  Let k5 = length(newline)
        k5 = k6 + 7

        write ( fnew, 9999) newline(1:k5)

        goto 700

c       ------------------------  Unchanged line(*)
  500   continue

c       *  Let k5 = length(line)

        do k5 = 80, 1, -1
          if ( line(k5:k5) .ne. ' ' ) goto 510
        enddo
        k5 = 0
  510   continue

        if ( k5.eq.0 ) then
*         Here we need to write an empty line, yet FTN77 writes ' '
          write ( fnew, 9999)
        else
          write ( fnew, 9999) line(1:k5)
        endif

c       *  Detect the end of routine (if present) to avoid duplicate keyword

        if ( line(1:9) .eq. '      end' ) goto 710
        if ( line(1:9) .eq. '      END' ) goto 710

  700 continue

c---------------------- One subroutine finished
  710 continue

      goto 100
c----------------------------------------------------- Loop back

c---------------------- File long.tmp is exhausted
  720 continue
      iline = iline - 1

      close ( fold )
      close ( fnew )
      if ( nerr .eq. 0 ) then
        close ( fout, status='DELETE')
      else
        close ( fout )
      endif

      if ( .false. ) then
         write ( fout, '(/(a,i5))' )
     .     'number of subroutines = ', nsub,
     .     'number of input lines = ', iline,
     .     'number of reductions  = ', nreplac,
     .     'number of errors      = ', nerr
      endif

 9999 format ( a )

      end
c ======================================================================
      subroutine errr ( stopit, message )

cimp      implicit none

c     ---------------------- i/o parameters ------

      integer stopit
      character * (*) message

c     message   i  an error message
c     stopit    i  -1 = one line warning, do not stop
c     stopit    i   0 = error,            do not stop
c     stopit    i   1 = error,            stop

c     --------------------- commons ------------------

      integer iline, nerr
      character * 80 line
      common /cint/ nerr, iline
     .       /cchar/ line

      integer funf, fold, fnew, fout
      common /crefs/ funf, fold, fnew, fout

c     --------------------------------------------

      nerr = nerr + 1

      if ( stopit .eq. -1 ) then

c       * Warning

        write ( fout, '( a, i5, 2a )' )
     .         'line', iline, ', warning: ', message

      else

c       *  Error

        if ( iline .ge. 0 ) then
          write ( fout, '( 80(''_'') / a / a, i5, 2a / 80(''^'') )' )
     .      line, '  !!! error, line ', iline, ': ', message
        else
          write ( fout, '( 80(''_'') / 2a / 80(''^'') )' )
     .      '  !!! error: ', message
        endif

        if ( stopit .eq. 0 ) then
          write ( fout, '(a)' ) '------- fatal error ------ stop ------'
          close ( fout )
          stop
        endif

      endif

      end
